require "lua/Constants"
require "lua/UnitFunctions"
require "lua/Functions"
require "lua/Functions_2"
require "lua/Text"
require "lua/Units"
require "math"

--spawn locations for the convoy
convoySpawnLocationsEasy = 
{
	--bottom spawn locations
	{7,1},
	{4,1},

	--top locations
	{6,29},
	{11,29},
	{13,24},
}

convoySpawnLocations = 
{
	--bottom spawn locations
	{7,1},
	{4,1},
	-- east spawn locations
	{1,11},
	{1,19},
	--top locations
	{6,29},
	{11,29},
	{13,24},
}

--Initiate TruckSpawner Function
function TruckSpawner()
	for i = 1, #convoySpawnLocations do					--reset spawnPos indexes
		ScriptManager:storeValue(string.format("spawnPos_%i",i),0)
		print("RESETTING SPAWNPOS")
	end
	
	local spawnAmount = nil
	local spawnPos = -1
	local counter = math.random(1,2)			
	local turnCount = ScriptManager:getCurrentRound()+counter			-- Next trucks spawning in
	
	if turnCount <= ScriptManager:getCurrentRound()+2 then				-- amount of trucks
		spawnAmount = math.random(1,3)
	elseif turnCount > ScriptManager:getCurrentRound()+2 then
		spawnAmount = math.random(3,5)
	end
	
	for i = 1, spawnAmount do
		ConvoySpawn()
	end
	
	print(string.format("More trucks in %i truns",counter))
	
	ScriptManager:addOnTurnEndTrigger(1,-1,turnCount,"TruckSpawner","TruckSpawner",false)		--Add trigger for the next wave
	ScriptManager:runForPlayer(0,"TruckShowDialogue")	
end

function TruckShowDialogue()
	table.insert(cutsceneQueue,_TruckShowDialogue)
	CutsceneQueue()
end

--Hint for trucks being spawned
function _TruckShowDialogue()
	ScriptManager:blockUI(true)
	ScriptManager:displaySubtitleAutoEx(Text.T_MAP_SU_2_2_MISSION_PRIMARY_01_FLAVOR,false)
	ScriptManager:wait(2)
	ScriptManager:blockUI(false)
end

-- Target tiles around the factory
targetTile = 
{
	{11,17},
	{12,18},
	{12,16},
	{13,16},
	{13,18},
}

function ConvoySpawn()
	local spawnPosIndex = nil
	local spawnPosStoreIndex = nil
	local spawnPosUsed = 0
	local spawnPos = nil
	local targetIndex = ScriptManager:getValue("targetIndex")
	local target = targetTile[targetIndex]
	local squadId = ScriptManager:getValue("convoySquadIndex")+1
	
	repeat																						
		spawnPosIndex = math.random(1,#convoySpawnLocations)									-- choose waypoint
		spawnPosStoreIndex = ScriptManager:getValue(string.format("spawnPos_%i",spawnPosIndex))	
	until spawnPosStoreIndex == 0 																-- use waypoint if not used before
	
	spawnPos = convoySpawnLocations[spawnPosIndex]									

	ScriptManager:spawnUnit(1,Units.UNIT_DE_CONVOY,spawnPos[1],spawnPos[2],squadId,AiState.MOVE)
	ScriptManager:setTargetTile(1,squadId,target[1],target[2])	
	ScriptManager:addOnMovedToTileTrigger(12,17,-1,squadId,1,false,"TruckReachedFactory",string.format("TruckReachedFactory_%i",squadId),true)		--Create a unique trigger for the spawned truck

	print(string.format("spawn truck @ %i,%i and move to %i,%i",spawnPos[1],spawnPos[2],targetTile[targetIndex][1],targetTile[targetIndex][2]))
	
	if targetIndex == #targetTile then
		targetIndex = 0
	end
	
	ScriptManager:storeValue("convoySquadIndex",squadId)
	ScriptManager:storeValue("targetIndex",targetIndex+1)
	ScriptManager:storeValue(string.format("spawnPos_%i",spawnPosIndex),spawnPosIndex) 	--stores unused index
end

--Table to store trucks which reached the factory
trucksReachedFactory = {}

--function to store the truck, which reached the factory into a table and remove the trigger
function TruckReachedFactory(x,y,squadId,unitId)
	print("remove TruckReachedFactory_"..squadId)
	
	ScriptManager:removeTrigger(string.format("TruckReachedFactory_%i",squadId))
	table.insert(trucksReachedFactory,#trucksReachedFactory+1,unitId)
	
	print(string.format("store Truck: %i into Table",unitId))
end

moveToLastUnit = 
{
	{"moveCam","unitLastSelected",-1,0.8,1,true},
}

function TrucksCheckIfReachedTrigger()
	table.insert(cutsceneQueue,_TrucksCheckIfReachedTrigger)
	CutsceneQueue()
end

--Cutscene to show trucks reaching the factory and showing a progress counter if progress counter reaches 5, spawn a trigger and reset the counter
function _TrucksCheckIfReachedTrigger()
local counter = ScriptManager:getValue("truckCounter")
local index = #trucksReachedFactory
	if counter == -1 then
		counter = 0
	end
	
	--Start cutscene if at least 1 truck reached the factory
	if index > 0 then														
		truckReachedFactory = 1
		ScriptManager:startCutscene()
		local camera = ScriptManager:moveCameraEx(12,17,0.85,2,false)
		ScriptManager:wait(1.5)
		ScriptManager:revealFogOfWar(12,17,2,true)
		ScriptManager:waitForCompletion(camera)
		ScriptManager:wait(1)
			
			repeat
				print(string.format("removed truck from table %i",trucksReachedFactory[index]))
				ScriptManager:removeUnitEx(trucksReachedFactory[index],true)										--Remove truck from game & table
				table.remove(trucksReachedFactory,index)

			
				index = index-1
				counter = counter+1
				ScriptManager:setMissionProgress(4,ScriptManager:getMissionProgress(4)+1)							--Set progress for counter
			
				PrintFactoryProgress()																				--Show progress
			
				if counter >= 5 then																				--Spawn Super tiger
					ScriptManager:playSound("sfx_siren_2")
					ScriptManager:spawnUnit(1,Units.UNIT_DE_TIGER_EXTREME,11,17,1337,AiState.ATTACK_AGGRESSIVE)		
					ScriptManager:displaySubtitleAutoEx(Text.T_MAP_SU_2_2_MISSION_SECONDARY_01_FLAVOR,true)
					if ScriptManager:getMissionStatus(4) ~= MissionStatus.FAILED then								--Secondary mission failed
						ScriptManager:setMissionStatus(4,MissionStatus.FAILED)
					end
					counter = counter-5 																			--Reset counter
				end
			
				ScriptManager:wait(0.5)
			until index == 0
		
			ScriptManager:revealFogOfWar(12,17,2,false)
			CutsceneFunction(moveToLastUnit)
			ScriptManager:stopCutscene()
		
		end
	
	print(string.format("Truck reached Tankfactory %i/5",counter))
	ScriptManager:storeValue("truckCounter",counter)
end

--print the progress of trucks, reaching the factory
function PrintFactoryProgress()
	if ScriptManager:getMissionProgress(4) <= 5 then
	elseif ScriptManager:getMissionProgress(4) > 5 then
		ScriptManager:setMissionProgress(4,1)
	end
		ScriptManager:preparePopupText()
		ScriptManager:addPopupTextParameter("x",ScriptManager:getMissionProgress(4))
		ScriptManager:displaySubtitleAutoEx(Text.T_MAP_SU_2_2_MISSION_SECONDARY_01_COUNTER,false)
		ScriptManager:wait(1.5)
end