require "lua/Constants"
require "lua/UnitFunctions"
require "lua/Functions"
require "lua/Functions_2"
require "lua/Text"
require "lua/Units"
require "math"

kv1SpawnPositions = -- Spawn positions for the kv1 tanks (secondary mission)
{
	{1,15},
	{8,1},
	{17,1},
	{25,1},
	{27,9},
	{30,15},
	{13,1},
	{25,14},
	{8,8},
	{10,14},
	{1,5},
}

function MissionSpawnKv1()
	print(tostring(kv1SpawnPositions))
--	print(string.format("%s",kv1SpawnPositions)) -- breaks the script on WP8!?
	print(kv1SpawnPositions)
	print(_G["kv1SpawnPositions"])
--	print(_G[string.format("%s",kv1SpawnPositions)]) -- breaks the script on WP8!?
	local tableName = _G[kv1SpawnPositions]
	print(tableName)
	local unit = nil
	local spawnIndex = nil
	local data = #kv1SpawnPositions
	local Index = 0
	ScriptManager:storeValue("repairedIndex",3) --store value for repair "x" more kv mission
	repeat
		spawnIndex = math.random(1,#kv1SpawnPositions)																		--generate index
		data = kv1SpawnPositions[spawnIndex]																				--choose random index from kv1SpawnPositions
		table.remove(kv1SpawnPositions,spawnIndex)																			--remove index from table to prevent using it again
		unit = ScriptManager:spawnUnit(1,Units.UNIT_SU_TANK_KV1,data[1],data[2],31337,AiState.PASSIVE)						--Spawn kv1 for enemies
		Index = Index+1																										--Increase index by 1
	until Index == 5																									--Until Index is 3  spawn kv's
	ScriptManager:addOnUnitSightedTrigger(1,Units.UNIT_SU_TANK_KV1,-1,true,"MissionKv1Sighted","MissionKv1Sighted",false)						--Add on sighted triggers
	print("spawnKv1")
end

--cutscene for spotting a kv1 for the first time
missionKv1Cutscene = 
{
	{"moveCam","newKv",-1,1,1.5,true},
	{"showMarker","newKv",-1,2},
	{"showDialogue",Text.T_MAP_SU_2_3_MISSION_SECONDARY_02_FLAVOR,true},
	{"moveCam","unitLastSelected",-1,0.8,2,true},
}

function MissionKv1SightedStart()
	ScriptManager:blockUI(true)
	ScriptManager:setMissionStatus(7,MissionStatus.ACCOMPLISHED)	--Find a kv1 done
	ScriptManager:wait(2)
	ScriptManager:startCutscene()
	CutsceneFunction(missionKv1Cutscene)
	ScriptManager:stopCutscene()
	ScriptManager:setMissionStatus(8,MissionStatus.IN_PROGRESS)		--repair it to do
	ScriptManager:blockUI(false)
end


function MissionKv1Sighted(activeUnit, passiveUnit)
	ScriptManager:removeTrigger("MissionKv1Sighted")
	print("unitSigted"..passiveUnit:getId())
	local x = passiveUnit:getX()
	local y = passiveUnit:getY()
	ScriptManager:storeValue("xPosition",x)
	ScriptManager:storeValue("yPosition",y)
	ScriptManager:removeUnitEx(passiveUnit:getId(),true)							--remove old kv1
	local unitKv = ScriptManager:spawnUnit(0,Units.UNIT_SU_TANK_KV1,x,y,0,0)		--create new one for the player
	ScriptManager:storeValue("newKv",unitKv:getId())								--store id for cutscene
	ScriptManager:addOnUnitSelectedTrigger(-1,-1,unitKv:getId(),"DisableKvInteraction",string.format("DisableKvInteraction_%i",unitKv:getId()),false)	--add trigger to disable the interaction buttons
	ScriptManager:addOnUnitSuppliedTrigger(-1,-1,unitKv:getId(),"KvRepair","KvRepair",false)							--add trigger for resupplying
	ScriptManager:addOnUnitDestroyedTrigger(-1,-1,unitKv:getId(),"KvDestroyed",string.format("KvDestroyed_%i",unitKv:getId()),false)						--add trigger if a kv1 gets destroyed before mission accomplished
	ScriptManager:storeValue(string.format("storeKvIndex_%i",unitKv:getId()),3)											--add index for resupply count index
	unitKv:setHealth(300)			--set parameters after spawn
	unitKv:setFuel(0)
	unitKv:setAmmo(0)
	table.insert(cutsceneQueue,_MissionKv1Sighted)
	CutsceneQueue()
	ScriptManager:addOnUnitSightedTrigger(1,Units.UNIT_SU_TANK_KV1,-1,true,"MissionKv1Sighted","MissionKv1Sighted",false)
end

function _MissionKv1Sighted()
	local x = ScriptManager:getValue("xPosition")
	local y = ScriptManager:getValue("yPosition")
	if ScriptManager:getMissionStatus(7) == MissionStatus.NOT_STARTED or ScriptManager:getMissionStatus(7) == MissionStatus.IN_PROGRESS then		--if mission 4 not started yet show mission cutscene
		MissionKv1SightedStart()
	end
	if ScriptManager:getMissionStatus(9) == MissionStatus.IN_PROGRESS then
		KVsightedAgain()
	end
end

kv1SightedAgain = 
{
	{"moveCam","newKv",-1,1,1,true},
	{"showMarker","newKv",-1,2},
	{"showDialogue",Text.T_MAP_SU_2_3_MISSION_SECONDARY_03_FLAVOR,true},
	{"moveCam","unitLastSelected",-1,1,1,true},
}

function KVsightedAgain()
	ScriptManager:blockUI(true)
	CutsceneFunction(kv1SightedAgain)
	ScriptManager:blockUI(false)
end

function KvDestroyed()
	ScriptManager:runForPlayer(0,"_KvDestroyed")
end

function _KvDestroyed()	--if kv gets destroyed before player resupplies it
	ScriptManager:setMissionProgress(9,ScriptManager:getMissionProgress(9)+1)
	if ScriptManager:getMissionStatus(8) ~= MissionStatus.ACCOMPLISHED then
		ScriptManager:setMissionStatus(8,MissionStatus.FAILED)
		ScriptManager:setMissionStatus(9,MissionStatus.IN_PROGRESS)
	elseif ScriptManager:getMissionProgress(9) == 3 then
		ScriptManager:setMissionStatus(9,MissionStatus.FAILED)
	end
end

function DisableKvInteraction(unit)	--if kv1 selected disable all interaction buttons
	ScriptManager:disableAllInteractionButtons()
	ScriptManager:addOnUnitDeselectedTrigger(-1,-1,unit:getId(),"EnableKvInteraction","EnableKvInteraction",false)
end

function EnableKvInteraction()	--if kv1 diselected, enable all interaction buttons
	ScriptManager:removeTrigger("EnableKvInteraction")
	ScriptManager:enableAllInteractionButtons()
end

function KvRepair(activeUnit, passiveUnit)	--kv1 repair function
local supplyIndex = ScriptManager:getValue(string.format("storeKvIndex_%i",passiveUnit:getId()))
		print("Index:"..supplyIndex)
		supplyIndex = supplyIndex-1
		print("Index:"..supplyIndex)
	if supplyIndex > 0 then
		passiveUnit:setFuel(0)
		passiveUnit:setAmmo(0)
		passiveUnit:setHealth(passiveUnit:getHealth()+200)
		kvRepairProgress(supplyIndex)
	elseif supplyIndex == 0 then	--if kv1 resupplied 3 times set fuel and ammo to max and start next function
		passiveUnit:setFuel(100)
		passiveUnit:setAmmo(100)
		passiveUnit:setHealth(2000)
		ScriptManager:wait(1)
		kvRepaired(passiveUnit)
	end
	ScriptManager:storeValue(string.format("storeKvIndex_%i",passiveUnit:getId()),supplyIndex)
end

function kvRepairProgress(supplyIndex)
	ScriptManager:dismissSubtitle()
	ScriptManager:preparePopupText()
	ScriptManager:addPopupTextParameter("x",supplyIndex)
	ScriptManager:displaySubtitleAutoEx(Text.T_MAP_SU_2_3_MISSION_SECONDARY_03_COUNTER,false) --TODO:new text makro
end

function kvRepaired(passiveUnit)	--function to manage the repaired kv1
	ScriptManager:removeTrigger("DisableAbilities")
	if ScriptManager:getMissionStatus(8) ~= MissionStatus.ACCOMPLISHED then
		ScriptManager:blockUI(true)
		ScriptManager:setMissionStatus(8,MissionStatus.ACCOMPLISHED)
		ScriptManager:wait(2)
		ScriptManager:startCutscene()
		ScriptManager:moveCameraEx(passiveUnit:getX(),passiveUnit:getY(),1,5,false)--move camera to the repaired kv1
		ScriptManager:displaySubtitleAutoEx(Text.T_MAP_SU_2_3_MISSION_SECONDARY_02_FLAVOR_2,true) --TODO:add textmakro
		ScriptManager:displaySubtitleAutoEx(Text.T_MAP_SU_2_3_MISSION_SECONDARY_02_FLAVOR_3,true)
		ScriptManager:stopCutscene()
		ScriptManager:setMissionStatus(9,MissionStatus.IN_PROGRESS)
		ScriptManager:blockUI(false)
	elseif ScriptManager:getMissionStatus(8) == MissionStatus.ACCOMPLISHED then
		kvRepairMoreProgress()
	end
	ScriptManager:removeTrigger(string.format("KvDestroyed_%i",passiveUnit:getId()))	--remove trigger for kv getting destroyed
	ScriptManager:removeTrigger(string.format("DisableKvInteraction_%i",passiveUnit:getId())) --remove trigger Id
end

function kvRepairMoreProgress()
	local repairedIndex = ScriptManager:getValue("repairedIndex")
	print(repairedIndex)
	repairedIndex = repairedIndex-1													--
	print(repairedIndex)
	ScriptManager:dismissSubtitle()
	ScriptManager:preparePopupText()
	ScriptManager:addPopupTextParameter("x",repairedIndex)
	if repairedIndex > 0 then
		ScriptManager:displaySubtitleAutoEx(Text.T_MAP_SU_2_3_MISSION_SECONDARY_03_COUNTER_2,true) --TODO:new text makro
	elseif repairedIndex == 0 then														--if 3 kv1 repaired mission accomplished
		ScriptManager:setMissionStatus(9,MissionStatus.ACCOMPLISHED)
	end
	ScriptManager:storeValue("repairedIndex",repairedIndex)
end