require "lua/Constants"
require "lua/UnitFunctions"
require "lua/Functions"
require "lua/Functions_2"
require "lua/Text"
require "lua/Units"
require "math"

tigerMissionParameters =
{
	{"moveCam","superTiger",-1,0.9,2,true},
	{"revealFow","superTiger",-1,2,true},
	{"showMarker","superTiger",-1,2},
	{"wait",1},
	{"showDialogue",Text.T_MAP_SU_2_3_MISSION_SECONDARY_04_FLAVOR,true},
	{"revealFow","superTiger",-1,2,false},
	{"moveCam","unitLastSelected",-1,0.9,2,true},
}

function TigerStartMission()
	ScriptManager:removeTrigger("TigerStartMission")
	ScriptManager:runForPlayer(0,"_TigerStartMission")
end

function _TigerStartMission()
	table.insert(cutsceneQueue,__TigerStartMission)
	CutsceneQueue()
end

function __TigerStartMission()
	ScriptManager:startCutscene()
	ScriptManager:playSound("sfx_siren_2")
	CutsceneFunction(tigerMissionParameters)
	ScriptManager:stopCutscene()
	ScriptManager:setMissionStatus(10,MissionStatus.IN_PROGRESS)
	ScriptManager:storeValue("CutsceneIndex",0)
end

tigerPatrolSpots = 
{
	{31,9},
	{10,8},
	{20,13},
	{21,5},
}

--spawns Tiger and sets first move location
function TigerSpawnAndPatrol()		
	local superTiger = ScriptManager:spawnUnit(1,Units.UNIT_DE_TIGER_EXTREME,27,5,1337,AiState.MOVE_AGGRESSIVE)
	ScriptManager:storeValue("superTigerPosX",superTiger:getX())
	ScriptManager:storeValue("superTigerPosY",superTiger:getY())
	ScriptManager:storeValue("superTiger",superTiger:getId())
	ScriptManager:addOnUnitHasAttackedTrigger(0,-1,superTiger:getId(),"TiggerCheckHP","TiggerCheckHP",false)
--	ScriptManager:addOnUnitDestroyedTrigger(0,-1,superTiger:getId(),"_TigerSpawnPlayerTiger","_TigerSpawnPlayerTiger",true)
	ScriptManager:addOnUnitSightedTrigger(1,-1,superTiger:getId(),true,"TigerStartMission","TigerStartMission",false)
	ScriptManager:addOnUnitHasAttackedTrigger(1,-1,superTiger:getId(),"TigerStartMission","TigerStartMission",false)
	_TigerChooseWayPoint()
end

function _TigerSpawnPlayerTiger()
	ScriptManager:runForPlayer(0,"TigerSpawnPlayerTiger")
end

function _TigerChooseWayPoint()
	ScriptManager:removeTrigger("_TigerChooseWayPoint")
	ScriptManager:runForPlayer(0,"TigerChooseWayPoint")
end

function TigerChooseWayPoint()
	local unitId = ScriptManager:getValue("superTiger")
	local waypointStoreIndex = 0
	local wayPointIndex = 0	--define local variables
	local wayPoint = 0
	local wayPointsUsed = 0
	local squadId = 1337
	--checks how many storages are used
	for i = 1, #tigerPatrolSpots do	
		if ScriptManager:getValue(string.format("waypoint_%i_%i",squadId,i)) ~= 0 then
			wayPointsUsed = wayPointsUsed + 1 --increase used index by 1 of theres a stored value ~= 0
		end
	end
	print("waypointsUsed:"..wayPointsUsed)											--TODO:remove prints for release
	--if #tigerPatrolWaypoints are used = resets the storages (0)
	if wayPointsUsed == #tigerPatrolSpots then			
		for i = 1, #tigerPatrolSpots do
			ScriptManager:storeValue(string.format("waypoint_%i_%i",squadId,i),0)
			print("RESETTING WAYPOINTS")
		end
	end
	--repeats until if finds an index storage which is empty
	repeat	
		wayPointIndex = math.random(1,#tigerPatrolSpots)												--randomly chooses a index from #tigerPatrolWaypoints
		waypointStoreIndex = ScriptManager:getValue(string.format("waypoint_%i_%i",squadId,wayPointIndex))	-- get index from storage
	until waypointStoreIndex == 0 -- if finds an empty storage us this index
	wayPoint = tigerPatrolSpots[wayPointIndex]
	ScriptManager:addOnMovedToTileTrigger(wayPoint[1],wayPoint[2],1,squadId,2,true,"_TigerChooseWayPoint","_TigerChooseWayPoint",false) -- creat new trigger for the target location
	ScriptManager:setTargetTile(1,squadId,wayPoint[1],wayPoint[2])																				-- set new move order
	print("New waypoint = "..wayPoint[1],wayPoint[2])												--TODO:remove prints for release
	ScriptManager:storeValue(string.format("waypoint_%i_%i",squadId,wayPointIndex),wayPointIndex) 	--stores unused index
end


--function TigerChooseWayPoint() 
--	print("Tiger sets new movement destination...")
--	ScriptManager:removeTrigger("_TigerChooseWayPoint")
--	local wayPointIndex = nil	--define local variables
--	local wayPoint = nil
--	local currentWayPoint = ScriptManager:getValue("tigerCurrentWayPoint")
--	local oldWayPoint = ScriptManager:getValue("tigerOldWayPoint")
--	print(currentWayPoint,oldWayPoint)
--	repeat	--repeats until if finds an index which is ~= current/old position
--		wayPointIndex = math.random(1,#tigerPatrolSpots)
--	until wayPointIndex ~= currentWayPoint and wayPointIndex ~= oldWayPoint
--	wayPoint = tigerPatrolSpots[wayPointIndex]
--	ScriptManager:addOnMovedToTileTrigger(wayPoint[1],wayPoint[2],1,1337,1,true,"_TigerChooseWayPoint","_TigerChooseWayPoint",false) -- creat new trigger for the target location
--	print("wayPointIndex = "..wayPointIndex)							--TODO:remove prints for release
--	ScriptManager:setTargetTile(1,1337,wayPoint[1],wayPoint[2])			--set new movement destination
--	ScriptManager:storeValue("tigerCurrentWayPoint",wayPointIndex)		--store new index
--	ScriptManager:storeValue("tigerOldWayPoint",currentWayPoint)
--	print(string.format("TigerMovesTo = %i, %i",wayPoint[1],wayPoint[2]))
--end

function TiggerCheckHP()
	local unitId = ScriptManager:getValue("superTiger")
	local superTigerUnit = Unit:getUnitById(ScriptManager:getValue("superTiger"))
	if superTigerUnit ~= nil then
		ScriptManager:storeValue("superTigerPosX",superTigerUnit:getX())
		ScriptManager:storeValue("superTigerPosY",superTigerUnit:getY())
	end
	if superTigerUnit ~= nil then
		if superTigerUnit:getHealth() < 600 then
			print("superTigerUnit isnot nil")
			ScriptManager:runForPlayer(0,"TigerBelow600")
		end
	elseif superTigerUnit == nil then
		print("superTigerUnit is nil")
		ScriptManager:storeValue("superTigerHp",300)
		ScriptManager:runForPlayer(0,"TigerSpawnPlayerTiger")
	end
end

function TigerBelow600()
	local unitId = ScriptManager:getValue("superTiger")
	local superTigerUnit = Unit:getUnitById(ScriptManager:getValue("superTiger"))
	print("KillTiger = "..unitId)
	ScriptManager:storeValue("superTigerHp",superTigerUnit:getHealth())
	ScriptManager:removeUnit(unitId,"TigerSpawnPlayerTiger")
end

playerTigerCutscene = 
{
	{"moveCam","tigerPlayer",-1,1,2,true},
	{"showDialogue",Text.T_MAP_SU_2_3_MISSION_SECONDARY_04_FLAVOR_1,true}, --TODO:text makros
	{"showDialogue",Text.T_MAP_SU_2_3_MISSION_SECONDARY_04_FLAVOR_2,true},
	{"moveCam","tigerPlayer",-1,0.8,5,false},
	{"showDialogue",Text.T_MAP_SU_2_3_MISSION_SECONDARY_05,true},
	{"wait","waitForCam"},	
}

function TigerSpawnPlayerTiger()
	local posY = nil
	local posX = nil
	local unit = nil
	local deadTigerX = ScriptManager:getValue("superTigerPosX")
	local deadTigerY = ScriptManager:getValue("superTigerPosY")
	local tigerPlayer = ScriptManager:spawnUnit(0,Units.UNIT_DE_TIGER_EXTREME,deadTigerX,deadTigerY,0,0)
	tigerPlayer:setHealth(ScriptManager:getValue("superTigerHp"))
	tigerPlayer:setFuel(0)
	tigerPlayer:setAmmo(0)
	ScriptManager:storeValue("tigerPlayer",tigerPlayer:getId())
	local tigerInfantry = ScriptManager:spawnUnit(1,Units.UNIT_DE_MP40,tigerPlayer:getX(),tigerPlayer:getY(),123456,AiState.ATTACK_AGGRESSIVE)
	ScriptManager:moveUnitEx(tigerInfantry:getId(),tigerInfantry:getX()+(math.random(1,4)),tigerInfantry:getY()+(math.random(1,4)),true)
	ScriptManager:removeUnitEx(tigerInfantry:getId(),true)
	print("SpawnUnit = "..tigerPlayer:getId())
	ScriptManager:storeValue("tigerSupplyIndex",0) --Add index for resupply progress
	ScriptManager:addOnUnitSelectedTrigger(-1,-1,tigerPlayer:getId(),"DisableAbilities","DisableAbilities",false)
	ScriptManager:addOnUnitSuppliedTrigger(0,-1,tigerPlayer:getId(),"TigerRepair","TigerRepair",false)
	ScriptManager:addOnUnitDestroyedTrigger(0,0,tigerPlayer:getId(),"_TigerDeadBeforeResupply","_TigerDeadBeforeResupply",false)
	__StartTigerMission()
end

function __StartTigerMission()
	ScriptManager:runForPlayer(0,"StartTigerMission")
end

function StartTigerMission()
	table.insert(cutsceneQueue,_StartTigerMission)
	CutsceneQueue()
end

function _StartTigerMission()
	ScriptManager:blockUI(true)
	ScriptManager:setMissionStatus(10,MissionStatus.ACCOMPLISHED)
	ScriptManager:wait(2)
	ScriptManager:startCutscene()
	CutsceneFunction(playerTigerCutscene)
	ScriptManager:stopCutscene()
	ScriptManager:setMissionStatus(11,MissionStatus.IN_PROGRESS)
	ScriptManager:blockUI(false)
end

function _TigerDeadBeforeResupply()
	ScriptManager:runForPlayer(0,"TigerDeadBeforeResupply")
end

function TigerDeadBeforeResupply()
	if ScriptManager:getMissionStatus(11) ~= MissionStatus.ACCOMPLISHED then
		ScriptManager:setMissionStatus(11,MissionStatus.FAILED)
	end
end

function DisableAbilities()
	ScriptManager:disableAllInteractionButtons()
	ScriptManager:addOnUnitDeselectedTrigger(-1,-1,-1,"EnableAbilities","EnableAbilities",false)
end

function EnableAbilities()
	ScriptManager:removeTrigger("EnableAbilities")
	ScriptManager:enableAllInteractionButtons()
end

function TigerRepair()
	local tigerPlayer = ScriptManager:getValue("tigerPlayer")
	local tigerPlayerUnit = Unit:getUnitById(tigerPlayer)
	local supplyIndex = ScriptManager:getValue("tigerSupplyIndex")
	supplyIndex = supplyIndex+1
	if supplyIndex < 5 then
		tigerPlayerUnit:setFuel(0)
		tigerPlayerUnit:setAmmo(0)
		tigerPlayerUnit:setHealth(tigerPlayerUnit:getHealth()+1000)
		TigerRepairProgress(supplyIndex)
	elseif supplyIndex == 5 then
		tigerPlayerUnit:setFuel(100)
		tigerPlayerUnit:setAmmo(100)
		tigerPlayerUnit:setHealth(5000)
		TigerRepaired()
		ScriptManager:removeTrigger("TigerRepair")
	end
	ScriptManager:storeValue("tigerSupplyIndex",supplyIndex)
end

function TigerRepairProgress(supplyIndex)
	ScriptManager:dismissSubtitle()
	ScriptManager:preparePopupText()
	ScriptManager:addPopupTextParameter("x",supplyIndex)
	ScriptManager:displaySubtitleAutoEx(Text.T_MAP_SU_2_3_MISSION_SECONDARY_05_COUNTER,false) --TODO:new text makro
end

function TigerRepaired()
	ScriptManager:removeTrigger("DisableAbilities")
	ScriptManager:blockUI(true)
	ScriptManager:displaySubtitleAutoEx(Text.T_MAP_SU_2_3_MISSION_SECONDARY_05_FLAVOR,true) --TODO:add textmakro
	ScriptManager:setMissionStatus(11,MissionStatus.ACCOMPLISHED)
	ScriptManager:blockUI(false)
	EnableAbilities()
end