require "lua/Constants"
require "lua/Functions"
require "lua/Text"
require "lua/Units"
require "math"

Missions = {
	MISSION_MAIN = 1,
	MISSION_BUNKER = 2,
}

UnitIDs = {
	UNIT_ID_KV1_01 = 15,
	UNIT_ID_KV1_02 = 16,
	UNIT_ID_KV1_03 = 37,
	UNIT_ID_KV1_04 = 28,
	UNIT_ID_BUNKER_01 = 2,
	UNIT_ID_BUNKER_02 = 4,
	UNIT_ID_BUNKER_03 = 3,
}
Units_Templates = {
	UNIT_TEMP_TIGER = 4,
	UNIT_TEMP_ARTILLERY = 6,
}

function onGameStart()

	ScriptManager:gainGold(0,1500)
	
	ScriptManager:preparePopupText()
	--print(string.format("[SCRIPT] onGameStart()..."));
	ScriptManager:createMission(Missions.MISSION_MAIN, true, Text.T_MISSION_MAP_1_3_INTRO_MISSION_MAIN);
	ScriptManager:createMission(Missions.MISSION_BUNKER, false, Text.T_MISSION_MAP_1_3_INTRO_MISSION_SECONDARY);
	
	
	ScriptManager:setMissionXp(Missions.MISSION_MAIN,200)
	ScriptManager:setMissionXp(Missions.MISSION_BUNKER,100)
	
	--ScriptManager:addOnEnterTileTrigger(5, 9, 0, -1, -1, true, "onEnteredCrossRoad");
	ScriptManager:addOnUnitDestroyedTrigger(-1, -1, UnitIDs.UNIT_ID_KV1_01, "updateKV1mission");
	ScriptManager:addOnUnitDestroyedTrigger(-1, -1, UnitIDs.UNIT_ID_KV1_02, "updateKV1mission");
	ScriptManager:addOnUnitDestroyedTrigger(-1, -1, UnitIDs.UNIT_ID_KV1_03, "updateKV1mission");
	ScriptManager:addOnUnitDestroyedTrigger(-1, -1, UnitIDs.UNIT_ID_KV1_04, "updateKV1mission");
	
	ScriptManager:addOnUnitDestroyedTrigger(-1, -1, UnitIDs.UNIT_ID_BUNKER_01, "updateBunkermission");
	ScriptManager:addOnUnitDestroyedTrigger(-1, -1, UnitIDs.UNIT_ID_BUNKER_02, "updateBunkermission");
	ScriptManager:addOnUnitDestroyedTrigger(-1, -1, UnitIDs.UNIT_ID_BUNKER_03, "updateBunkermission");
	
	ScriptManager:addOnUnitHasAttackedTrigger(-1,Units.UNIT_SU_BUNKER,-1,"onTargetAttacked","onTargetAttacked")
	ScriptManager:addOnUnitHasAttackedTrigger(-1,Units.UNIT_DE_HQ,-1,"ProtectTheSupplyDepotHint","ProtectTheSupplyDepotHint")
		
	ScriptManager:sleep(0.0, "StartIntro");
	ScriptManager:blockUI(true)

	ScriptManager:addOnUnitDestroyedTrigger(0,-1,-1,"checkForEnd","checkForEnd")
	ScriptManager:addOnTurnStartTrigger(0,-1,-1,"CheckForGameEnd")	
	return 1
end

function checkForEnd()
	if ScriptManager:noUnitsLeft(0) or (Unit:getUnitById(0) == nil and Unit:getUnitById(1) == nil) then
		ScriptManager:runForPlayer(0, "WaitforFailed")
	end
end


--Game End Check
function CheckForGameEnd()
	MainGoalAccomplished(Missions.MISSION_MAIN,onMissionAccomplished,MissionFailed)		--Main Mission ID, Script to start if Accomplished, Script to start if Failed
end


function endGameWait()
	ScriptManager:continueAfterCutscene("MissionFailed")
end

--Create Secondary Mission

function onTargetAttacked(attacker, defender)		
	targetID = -1
	if attacker:getTemplateId() == Units.UNIT_SU_BUNKER then				
		targetID = attacker:getId()
	end
	if defender:getTemplateId() == Units.UNIT_SU_BUNKER then
		targetID = defender:getId()
	end
	ScriptManager:removeTrigger("onTargetAttacked")
	ScriptManager:runForPlayer(0, "MoveCamToTarget")	
end

function MoveCamToTarget()
	ScriptManager:disableEndTurnButton()
	ScriptManager:enableSkipCutsceneButton()
	ScriptManager:moveCamera(Unit:getUnitById(targetID):getX(),Unit:getUnitById(targetID):getY(),0.9,2,"CamOnTarget")
end

function CamOnTarget()
	ScriptManager:revealFogOfWar(Unit:getUnitById(targetID):getX(),Unit:getUnitById(targetID):getY(),1,true)
	ScriptManager:showMarkerAt(Unit:getUnitById(targetID):getX(),Unit:getUnitById(targetID):getY())
	ScriptManager:displaySubtitle(Text.T_MISSION_MAP_1_3_INTRO_DIALOGUE_04)
	ScriptManager:sleep(5,"HideDialogue")
	ScriptManager:moveCamera(Unit:getUnitById(targetID):getX(),Unit:getUnitById(targetID):getY(),0.9,3,"ZoomCamOutAndEndScript")
end

function ZoomCamOutAndEndScript()
	ScriptManager:moveCamera(Unit:getUnitById(targetID):getX(),Unit:getUnitById(targetID):getY(),0.8,2,"EndMetaCutScene")
	ScriptManager:hideMarkerAt(Unit:getUnitById(targetID):getX(),Unit:getUnitById(targetID):getY())
	ScriptManager:revealFogOfWar(Unit:getUnitById(targetID):getX(),Unit:getUnitById(targetID):getY(),1,false)
end

function EndMetaCutScene()
	ScriptManager:setMissionStatus(Missions.MISSION_BUNKER,MissionStatus.IN_PROGRESS)
	ScriptManager:disableSkipCutsceneButton()
	ScriptManager:enableEndTurnButton()
end

--END


function HideDialogue()
	ScriptManager:dismissSubtitle()
end

function EnableEndTurn()
	ScriptManager:enableEndTurnButton()
end

function StartIntro()
	ScriptManager:startCutscene();
	ScriptManager:sleep(0,"MoveCam1")
	ScriptManager:sleep(1.5,"ShowDialogue1")
end

function ShowDialogue1()
	ScriptManager:displaySubtitle(Text.T_MISSION_MAP_1_3_INTRO_DIALOGUE_02)
	ScriptManager:sleep(4 ,"HideDialogue")
end

function MoveCam1()
	ScriptManager:moveCamera(3,11,1.2,6,"RevealFOW_1")
end

function RevealFOW_1()
	ScriptManager:revealFogOfWar(3,11,1,true);
	ScriptManager:sleep(2,"Dialogue3")
	ScriptManager:sleep(1,"wait_2")
end

function wait_2()
	ScriptManager:moveUnit(64 ,4,12, "tank_1")
end

function tank_1()
	ScriptManager:moveUnit(63 ,2,11, "tank_2")
end

function tank_2()
	ScriptManager:moveUnit(65 ,4,10, "tank_3")
end

function tank_3()
	ScriptManager:fortifyUnit(15)
	ScriptManager:sleep(1,"wait_1")
end

function Dialogue3()
	ScriptManager:displaySubtitle(Text.T_MISSION_MAP_1_3_INTRO_DIALOGUE_03)
end

function wait_1()
	ScriptManager:sleep(1,"MoveCam_1")
	ScriptManager:sleep(2 ,"HideDialogue3")
end

function HideDialogue3()
	ScriptManager:dismissSubtitle()
end

function MoveCam_1()
	ScriptManager:revealFogOfWar(3,11,1,false);
	ScriptManager:sleep(1.5,"WaitMoveCam1")
end

function WaitMoveCam1()
	ScriptManager:moveUnit(66 ,3,10, "")
	ScriptManager:moveUnit(67 ,3,12, "")
	ScriptManager:moveUnit(68 ,4,11, "")
	ScriptManager:moveCamera(10,2,1.1,3,"move_units")
end

function move_units()
	ScriptManager:displaySubtitle(Text.T_MISSION_MAP_1_3_INTRO_DIALOGUE_05)
	local unit = ScriptManager:spawnUnit(0, Units_Templates.UNIT_TEMP_ARTILLERY, 9, 1, 0, 0);
	spawnArtillery = unit:getId()
	ScriptManager:moveUnit(spawnArtillery,10,3, "ArtilleryinPosition")
end

function ArtilleryinPosition()
	local unit = ScriptManager:spawnUnit(0, Units_Templates.UNIT_TEMP_TIGER, 9, 1, 0, 0);
	spawnTiger = unit:getId()
	ScriptManager:moveUnit(spawnTiger,8,2, "wait_tiger")

end

function wait_tiger()
	ScriptManager:dismissSubtitle()
	ScriptManager:sleep(0.5,"wait_tiger_2")
end

function wait_tiger_2()
	ScriptManager:moveCamera(5,5,0.65,2.0,"IntroEnd")
end

function IntroEnd()
	--TODO:SCREENSHOT
	--TODO DELETE GOD!
	--ScriptManager:spawnUnit(0,30,3,24,0,0)
--	ScriptManager:moveCamera(0,0,0.77,0,"")

	ScriptManager:stopCutscene();
	if ScriptManager:isControllerConnected() then
		ScriptManager:setControllerCursorPosition( 5, 5 );
	end
	ScriptManager:continueAfterCutscene("CreateMainMission")
end

function CreateMainMission()
	ScriptManager:setMissionStatus(Missions.MISSION_MAIN, MissionStatus.IN_PROGRESS);
	ScriptManager:sleep(2,"blockend")
end


function blockend()

	ScriptManager:blockUI(false)
end

function updateKV1mission()

	ScriptManager:runForPlayer(0,"onKV1Destroyed")

end

function onKV1Destroyed(unit)
	ScriptManager:setMissionProgress(Missions.MISSION_MAIN, ScriptManager:getMissionProgress(1) + 1);
	if ScriptManager:getMissionProgress(Missions.MISSION_MAIN) == 4 then
		ScriptManager:setMissionStatus(Missions.MISSION_MAIN, MissionStatus.ACCOMPLISHED);
		if ScriptManager:getMissionStatus(Missions.MISSION_BUNKER) ==  MissionStatus.IN_PROGRESS then
			ScriptManager:setMissionStatus(Missions.MISSION_BUNKER, MissionStatus.FAILED);
		end
		ScriptManager:runForPlayer(0, "onMissionAccomplished")
		ScriptManager:blockUI(true)
	end
	tankDestroyed()
end

function tankDestroyed()																	--Tank destroyed! x/4
	ScriptManager:preparePopupText();
	ScriptManager:addPopupTextParameter("x",ScriptManager:getMissionProgress(1))
	ScriptManager:displaySubtitle(Text.T_MISSION_MAP_1_3_INTRO_MISSION_MAIN_COUNT)
	ScriptManager:sleep(3,"HideDialogue")
end

function updateBunkermission()

	ScriptManager:runForPlayer(0,"onBunkerDestroyed")

end

function onBunkerDestroyed(unit)
	if ScriptManager:getMissionStatus(Missions.MISSION_BUNKER) ~= MissionStatus.IN_PROGRESS and
	ScriptManager:getMissionStatus(Missions.MISSION_BUNKER) ~= MissionStatus.ACCOMPLISHED then
		ScriptManager:setMissionStatus(Missions.MISSION_BUNKER,MissionStatus.IN_PROGRESS)
		ScriptManager:removeTrigger("onTargetAttacked")
	end
	ScriptManager:setMissionProgress(Missions.MISSION_BUNKER, ScriptManager:getMissionProgress(Missions.MISSION_BUNKER) + 1);
	if ScriptManager:getMissionProgress(Missions.MISSION_BUNKER) == 3 then
		ScriptManager:setMissionStatus(Missions.MISSION_BUNKER, MissionStatus.ACCOMPLISHED);
	end
	bunkerDestroyed()
end

function bunkerDestroyed()																	--Bunker destroyed! x/3
	ScriptManager:preparePopupText();
	ScriptManager:addPopupTextParameter("x",ScriptManager:getMissionProgress(2))
	ScriptManager:displaySubtitle(Text.T_MISSION_MAP_1_3_INTRO_DIALOGUE_04_COUNT)
	ScriptManager:sleep(3,"HideDialogue")
end

function onMissionAccomplished()
	ScriptManager:disableEndTurnButton()
	ScriptManager:blockUI(true)	
	ScriptManager:sleep(5,"outroText")
end

function Victory()
	--print(string.format("[Script]Mission Accomplished"))										--Outro
	ScriptManager:displaySubtitleAutoWithCallback(Text.T_MISSION_VICTORY,"")
	--ScriptManager:sleep(3,"HideDialogue")
	ScriptManager:sleep(5,"EndGame")
end

function WaitforFailed()
	ScriptManager:disableEndTurnButton()
	ScriptManager:blockUI(true)
	ScriptManager:sleep(3,"endGameWait")
end

function MissionFailed()
	ScriptManager:disableEndTurnButton()
	ScriptManager:blockUI(true)
	ScriptManager:displaySubtitleAutoWithCallback(Text.T_MAP_3_2_MISSION_FAILED,"")
	ScriptManager:sleep(3,"HideDialogue")
	ScriptManager:sleep(4,"EndGame")
end

function outroText()
	--print(string.format("[Script]Outro"))	
	ScriptManager:displaySubtitleAutoWithCallback(Text.T_MISSION_MAP_1_3_INTRO_MISSION_OUTRO,"")							
	--ScriptManager:sleep(5,"HideDialogue")
	ScriptManager:blockUI(true)
	ScriptManager:disableEndTurnButton()
	ScriptManager:sleep(4,"Victory")
end

function EndGame()
	ScriptManager:endGame()
end