require "lua/Constants"
require "lua/Functions"
require "lua/Text"
require "lua/Units"
require "math"


Missions = {
	MISSION_MAIN = 1,
	MISSION_AA = 2,
	MISSION_BASE_1 = 3,
	MISSION_BASE_2 = 4,
}

UnitId = {
	HQ_1 = 40, --Southeast
	HQ_2 = 0,	--Northeast
}
Units_Templates = {
	UNIT_TEMP_FIGHTER = 8,
	UNIT_TEMP_BOMBER = 9,
}

function onGameStart()

	ScriptManager:gainGold(0,1000)
	ScriptManager:preparePopupText()
	--print(string.format("[SCRIPT] onGameStart()..."));
	ScriptManager:createMission(Missions.MISSION_MAIN, true, "");
	ScriptManager:setMissionStatus(Missions.MISSION_MAIN, MissionStatus.IN_PROGRESS);
	
	ScriptManager:createMission(Missions.MISSION_BASE_1, true, Text.T_MISSION_MAP_1_1_INTRO_DIALOGUE_02); --south eastern
	
	ScriptManager:createMission(Missions.MISSION_BASE_2, true, Text.T_MISSION_MAP_1_2_INTRO_MISSION_MAIN_2); --north-east
	
	ScriptManager:createMission(Missions.MISSION_AA, false, Text.T_MISSION_MAP_2_1_INTRO_MISSION_SECONDARY);
	
	ScriptManager:setMissionXp(Missions.MISSION_BASE_1,100)
	ScriptManager:setMissionXp(Missions.MISSION_BASE_2,100)
	ScriptManager:setMissionXp(Missions.MISSION_MAIN,150)
	ScriptManager:setMissionXp(Missions.MISSION_AA,100)
	
	--Don't attack depot trigger
	ScriptManager:addOnUnitHasAttackedTrigger(-1,Units.UNIT_SU_HQ,-1,"ProtectTheSupplyDepotHint","ProtectTheSupplyDepotHint")
	ScriptManager:addOnUnitHasAttackedTrigger(-1,Units.UNIT_DE_HQ,-1,"ProtectTheSupplyDepotHint","ProtectTheSupplyDepotHint")
		
	ScriptManager:addOnUnitDestroyedTrigger(-1, -1, UnitId.HQ_1, "DepotDestroyed");
	ScriptManager:addOnUnitCapturedTrigger(-1, -1, UnitId.HQ_1, "onHq1Captured");
	
	ScriptManager:addOnUnitDestroyedTrigger(-1, -1, UnitId.HQ_2, "DepotDestroyed");
	ScriptManager:addOnUnitCapturedTrigger(-1, -1, UnitId.HQ_2, "onHq2Captured");
	
	ScriptManager:addOnUnitDestroyedTrigger(-1, Units.UNIT_SU_AA, -1, "onAADestroyed");
	
	--iNTRO ON UNIT SELECTED SHOE DIALOGUE

	ScriptManager:addOnUnitHasAttackedTrigger(-1,Units.UNIT_SU_AA,-1,"onTargetAttacked","onTargetAttacked")
	
	
	ScriptManager:sleep(0.0,"Sleep_Intro_1_End")
	
	ScriptManager:addOnUnitDestroyedTrigger(0,-1,-1,"checkForEnd","checkForEnd")
	ScriptManager:addOnTurnStartTrigger(0,-1,-1,"CheckForGameEnd")
	return 1
end

--Script to move cam to the supply depot and warn the player if it gets attacked

function HideMarker()
	ScriptManager:removeAllMarkers()
end

--Show warning
function DontAttackDepotWarning()
	ScriptManager:displaySubtitleAutoWithCallback(Text.T_MISSION_MAP_2_2_WARNING,"")
	ScriptManager:sleep(3,"HideDialogue")
		
end

--Game End Check
function CheckForGameEnd()
	MainGoalAccomplished(Missions.MISSION_MAIN,onMissionAccomplished,MissionFailed)		--Main Mission ID, Script to start if Accomplished, Script to start if Failed
end

--END

function checkForEnd()
	if ScriptManager:noUnitsLeft(0) or Unit:getUnitById(3) == nil then
		ScriptManager:runForPlayer(0, "endGameWait")
	end
end

function endGameWait()
	ScriptManager:continueAfterCutscene("MissionFailed")
end

function HideDialogue()
	ScriptManager:dismissSubtitle()
end

function EnableEndTurn()
	ScriptManager:enableEndTurnButton()
end


--Create AA Mission

function onTargetAttacked(attacker, defender)		
	targetID = -1
	if attacker:getTemplateId() == Units.UNIT_SU_AA then				
		targetID = attacker:getId()
	end
	if defender:getTemplateId() == Units.UNIT_SU_AA then
		targetID = defender:getId()
	end
	ScriptManager:removeTrigger("onTargetAttacked")
	if ScriptManager:getCurrentTurn() % 2 == 1 then
		ScriptManager:addOnTurnStartTrigger(0, -1,ScriptManager:getCurrentRound()+1,"MoveCamToTarget", "MoveCamToTarget")
	else
		MoveCamToTarget()
	end
end

function MoveCamToTarget()
	ScriptManager:disableEndTurnButton()
	ScriptManager:enableSkipCutsceneButton()
	ScriptManager:removeTrigger("MoveCamToTarget")
	ScriptManager:moveCamera(Unit:getUnitById(targetID):getX(),Unit:getUnitById(targetID):getY(),0.9,2,"CamOnTarget")
end

function CamOnTarget()
	ScriptManager:revealFogOfWar(Unit:getUnitById(targetID):getX(),Unit:getUnitById(targetID):getY(),1,true)
	ScriptManager:showMarkerAt(Unit:getUnitById(targetID):getX(),Unit:getUnitById(targetID):getY())
	ScriptManager:displaySubtitle(Text.T_MISSION_MAP_2_1_INTRO_MISSION_SECONDARY_DIALOGUE)
	ScriptManager:sleep(5,"HideDialogue")
	ScriptManager:moveCamera(Unit:getUnitById(targetID):getX(),Unit:getUnitById(targetID):getY(),0.9,3,"ZoomCamOutAndEndScript")
end

function ZoomCamOutAndEndScript()
	ScriptManager:moveCamera(Unit:getUnitById(targetID):getX(),Unit:getUnitById(targetID):getY(),0.8,2,"EndMetaCutScene")
	ScriptManager:hideMarkerAt(Unit:getUnitById(targetID):getX(),Unit:getUnitById(targetID):getY())
	ScriptManager:revealFogOfWar(Unit:getUnitById(targetID):getX(),Unit:getUnitById(targetID):getY(),1,false)
end

function EndMetaCutScene()
	ScriptManager:setMissionStatus(Missions.MISSION_AA,MissionStatus.IN_PROGRESS)
	ScriptManager:disableSkipCutsceneButton()
	ScriptManager:enableEndTurnButton()
end


--END

function Sleep_Intro_1_End()
	ScriptManager:startCutscene();
	ScriptManager:sleep(0,"MoveCam1")
end

function showDialogue1()
	ScriptManager:displaySubtitle(Text.T_MISSION_MAP_2_1_INTRO_MISSION_MAIN)
	ScriptManager:sleep(5,"HideDialogue")
end

function MoveCam1()
	ScriptManager:moveCamera(3,9,0.8,3,"Airplanes")
end

function Airplanes()
	local unit = ScriptManager:spawnUnit(0, Units_Templates.UNIT_TEMP_FIGHTER, 1, 12, 0, 0);
	spawnFighter = unit:getId()
	ScriptManager:moveUnit(spawnFighter,2 ,9 , "FighterinPosition")
end

function FighterinPosition()
	local unit = ScriptManager:spawnUnit(0, Units_Templates.UNIT_TEMP_BOMBER, 1, 5, 0, 0);
	spawnBomber = unit:getId()
	ScriptManager:moveUnit(spawnBomber,3 ,9 , "BomberinPosition")
end

function BomberinPosition()
	ScriptManager:displaySubtitle(Text.T_MISSION_MAP_2_1_INTRO_DIALOGUE_05)
	ScriptManager:sleep(3,"ToStart_Intro")
end

function ToStart_Intro()
	ScriptManager:dismissSubtitle();
	ScriptManager:sleep(1,"ToStart_Intro1")
end

function ToStart_Intro1()
	ScriptManager:moveCamera(3,11,0.7,2,"endCutscene")
end

function endCutscene()
	ScriptManager:sleep(0.5,"endCutscene2")
end

function endCutscene2()

	ScriptManager:stopCutscene();
	if ScriptManager:isControllerConnected() then
		ScriptManager:setControllerCursorPosition( 4, 11 );
	end
	ScriptManager:disableEndTurnButton()
	ScriptManager:blockUI(true)
	ScriptManager:continueAfterCutscene("secondMainMission")
	
--ScriptManager:spawnUnit(0,30,3,7,0,0)			--TODO:DELETE GOD

--ScriptManager:moveCamera(0,0,0.77,0,"")		--TODO:SCREENSHOT

end

function secondMainMission()
	ScriptManager:setMissionStatus(3,MissionStatus.IN_PROGRESS)
	ScriptManager:sleep(3,"secondMission2")
	ScriptManager:blockUI(true)
end

function secondMission2()
	ScriptManager:setMissionStatus(4,MissionStatus.IN_PROGRESS)
	ScriptManager:sleep(3,"waittoEndblock")
end

function waittoEndblock()
	showDialogue1()
	ScriptManager:enableEndTurnButton()
	ScriptManager:blockUI(false)	--unblock 2x because of two blocks => incremental
	ScriptManager:blockUI(false)
end


function BaseCaptured()
	if (ScriptManager:getMissionStatus(Missions.MISSION_BASE_1) == MissionStatus.ACCOMPLISHED and
		ScriptManager:getMissionStatus(Missions.MISSION_BASE_2) == MissionStatus.ACCOMPLISHED) then
		
		ScriptManager:setMissionStatus(Missions.MISSION_MAIN, MissionStatus.ACCOMPLISHED);
		if ScriptManager:getMissionStatus(Missions.MISSION_AA) ==  MissionStatus.IN_PROGRESS then
			ScriptManager:setMissionStatus(Missions.MISSION_AA, MissionStatus.FAILED);
		end
		ScriptManager:disableEndTurnButton()
		ScriptManager:displaySubtitleAutoWithCallback(Text.T_MISSION_TUTORIAL_1_28,"onMissionAccomplished")
		--ScriptManager:sleep(3,"onMissionAccomplished")
	end
end


function onAADestroyed(unit)
	if ScriptManager:getMissionStatus(Missions.MISSION_AA) ~= MissionStatus.IN_PROGRESS and
	ScriptManager:getMissionStatus(Missions.MISSION_AA) ~= MissionStatus.ACCOMPLISHED then
		ScriptManager:setMissionStatus(Missions.MISSION_AA,MissionStatus.IN_PROGRESS)
		ScriptManager:removeTrigger("onTargetAttacked")
	end
	ScriptManager:setMissionProgress(Missions.MISSION_AA, ScriptManager:getMissionProgress(Missions.MISSION_AA) + 1);
	if ScriptManager:getMissionProgress(Missions.MISSION_AA) == 5 then
			ScriptManager:runForPlayer(0,"SetAAmissionaccomplishedNextturn")
	end
	aadestroyedCounter()
end

function SetAAmissionaccomplishedNextturn()

	ScriptManager:setMissionStatus(Missions.MISSION_AA, MissionStatus.ACCOMPLISHED);
end

function aadestroyedCounter()																	--AA destroyed! x/5
	ScriptManager:preparePopupText();
	ScriptManager:addPopupTextParameter("x",ScriptManager:getMissionProgress(2))
	ScriptManager:displaySubtitle(Text.T_MISSION_MAP_2_1_INTRO_MISSION_SECONDARY_COUNT)
	ScriptManager:sleep(3,"HideDialogue")
end

function onHq1Captured(unit)
	if ScriptManager:getMissionStatus(Missions.MISSION_BASE_1) == MissionStatus.IN_PROGRESS then
		ScriptManager:setMissionStatus(Missions.MISSION_BASE_1, MissionStatus.ACCOMPLISHED);
		BaseCaptured()
	end
end

function onHq2Captured(unit)
	if ScriptManager:getMissionStatus(Missions.MISSION_BASE_2) == MissionStatus.IN_PROGRESS then
		ScriptManager:setMissionStatus(Missions.MISSION_BASE_2, MissionStatus.ACCOMPLISHED);
		BaseCaptured()
	end
end

--MISSION FAIL TRIGGER END MISSION

function DepotDestroyed()

	ScriptManager:runForPlayer(0,"DepotDestroyed_1")
end

function DepotDestroyed_1()

	if ScriptManager:getMissionStatus( Missions.MISSION_BASE_1 ) ~= MissionStatus.ACCOMPLISHED then
		ScriptManager:setMissionStatus(Missions.MISSION_BASE_1,MissionStatus.FAILED)
		print( "base 1 failed" );
	end
	if ScriptManager:getMissionStatus( Missions.MISSION_BASE_2 ) ~= MissionStatus.ACCOMPLISHED then
		ScriptManager:setMissionStatus(Missions.MISSION_BASE_2,MissionStatus.FAILED)
		print( "base 2 failed" );
	end
		ScriptManager:disableEndTurnButton()
		ScriptManager:disableSkipCutsceneButton()
		ScriptManager:blockUI(true)	
		ScriptManager:sleep(5,"MissionFailed")
		
end

function MissionFailed()
	HideDialogue()
	ScriptManager:blockUI(true)
	ScriptManager:disableEndTurnButton()
	ScriptManager:displaySubtitleAutoWithCallback(Text.T_MAP_3_2_MISSION_FAILED,"")
	ScriptManager:sleep(5,"EndGame")
end

--END GAME

function onMissionAccomplished()
	ScriptManager:disableEndTurnButton()
	ScriptManager:blockUI(true)
	HideDialogue()
	ScriptManager:displaySubtitleAutoWithCallback(Text.T_MISSION_VICTORY,"")
	--ScriptManager:sleep(2.5,"DismissDialogue5")
	ScriptManager:sleep(4,"EndGame")
end

function DismissDialogue5()
	ScriptManager:dismissSubtitle()
end

function EndGame()
	ScriptManager:blockUI(true)
	ScriptManager:endGame()
end
