require "lua/Constants"
require "lua/Functions"
require "lua/Text"
require "lua/Units"
require "math"


Missions = {
	MISSION_MAIN = 1,
	MISSION_HANGAR = 2,
	MISSION_BASE_1 = 3,
	MISSION_BASE_2 = 4,
	MISSION_BASE_3 = 5,
	MISSION_KILL_PLANES = 7,
	MISSION_MAIN_PROTECT = 8,
}

UnitId = {
	HQ_01 = 4,
	HQ_02 = 5,
	HQ_03 = 6,
}

function onGameStart()

	ScriptManager:gainGold(0,1500)
	--print(string.format("[SCRIPT] onGameStart()..."));
	ScriptManager:preparePopupText()
	
	ScriptManager:createMission(Missions.MISSION_MAIN, true,Text.T_MISSION_MAP_2_2_MAIN_2);
	
	ScriptManager:createMission(Missions.MISSION_KILL_PLANES,false,Text.T_MISSION_MAP_2_2_SECONDARY_2)
	
	ScriptManager:createMission(Missions.MISSION_HANGAR, false,Text.T_MISSION_MAP_3_1_SECONDARY_OBJ_01_LOG);
	
	ScriptManager:createMission(Missions.MISSION_BASE_1, false, "");
	ScriptManager:setMissionStatus(Missions.MISSION_BASE_1, MissionStatus.IN_PROGRESS);
	
	ScriptManager:createMission(Missions.MISSION_BASE_2, false, "");
	ScriptManager:setMissionStatus(Missions.MISSION_BASE_2, MissionStatus.IN_PROGRESS);
	
	ScriptManager:createMission(Missions.MISSION_BASE_3, false, "");
	ScriptManager:setMissionStatus(Missions.MISSION_BASE_3, MissionStatus.IN_PROGRESS);
	
	ScriptManager:createMission(Missions.MISSION_MAIN_PROTECT, true,Text.T_MISSION_MAP_2_2_WARNING);
	
	ScriptManager:removeReinforcementsFromBase(0,2)
	ScriptManager:removeReinforcementsFromBase(0,3)
	ScriptManager:removeReinforcementsFromBase(0,4)
	ScriptManager:removeReinforcementsFromBase(0,5)
	
	ScriptManager:setMissionXp(Missions.MISSION_MAIN,200)
	ScriptManager:setMissionXp(Missions.MISSION_HANGAR,150)
	ScriptManager:setMissionXp(Missions.MISSION_MAIN_PROTECT,100)
	
	
	ScriptManager:addOnUnitDestroyedTrigger(-1, -1, UnitId.HQ_01, "DepotDestroyed", "HQ1DC");
	ScriptManager:addOnUnitCapturedTrigger( -1, -1, UnitId.HQ_01, "onHq1Captured", "HQ1DC");
	
	ScriptManager:addOnUnitDestroyedTrigger(-1, -1, UnitId.HQ_02, "DepotDestroyed", "HQ2DC");
	ScriptManager:addOnUnitCapturedTrigger( -1, -1, UnitId.HQ_02, "onHq2Captured", "HQ2DC");
	
	ScriptManager:addOnUnitDestroyedTrigger(-1, -1, UnitId.HQ_03, "DepotDestroyed", "HQ3DC");
	ScriptManager:addOnUnitCapturedTrigger( -1, -1, UnitId.HQ_03, "onHq3Captured", "HQ3DC");
	
	ScriptManager:addOnUnitDestroyedTrigger(-1, -1, 3, "onHangarDestroyedOrCaptured","hangardestroyed");
	ScriptManager:addOnUnitCapturedTrigger(-1, -1, 3, "onHangarDestroyedOrCaptured","hangardestroyed");

	ScriptManager:addOnTurnStartTrigger(1,-1,2, "SpawnEnemyReinforcements_1", "SpawnEnemyReinforcements_1");
	
	--Spawn Planes Pre respawner
	
	local StartPlane1 = ScriptManager:spawnUnit(1, Units.UNIT_SU_FIGHTER, 5, 4, 30, AiState.AIRPLANE);
	local StartPlane2 = ScriptManager:spawnUnit(1, Units.UNIT_SU_FIGHTER, 4, 3, 30, AiState.AIRPLANE);
	ScriptManager:storeValue("StartPlane1",StartPlane1:getId())
	ScriptManager:storeValue("StartPlane2",StartPlane2:getId())
	
	ScriptManager:addOnUnitDestroyedTrigger(1,-1,StartPlane1:getId(),"StartPlane1Destroyed","StartPlane1Destroyed")
	ScriptManager:addOnUnitDestroyedTrigger(1,-1,StartPlane2:getId(),"StartPlane1Destroyed","StartPlane1Destroyed")
	
	ScriptManager:addOnUnitHasAttackedTrigger(-1,Units.UNIT_SU_FIGHTER,-1,"onTargetAttacked","onTargetAttacked")
		
	ScriptManager:addOnUnitHasAttackedTrigger(-1,Units.UNIT_DE_HQ,-1,"ProtectTheSupplyDepotHint","ProtectTheSupplyDepotHint")
	ScriptManager:addOnUnitHasAttackedTrigger(-1,Units.UNIT_SU_HQ,-1,"ProtectTheSupplyDepotHint","ProtectTheSupplyDepotHint")
	
	ScriptManager:sleep(0,"IntroStart")
	ScriptManager:addOnTurnStartTrigger(0,-1,-1,"CheckForGameEnd")
	ScriptManager:addOnUnitDestroyedTrigger(0,-1,-1,"checkForEnd","checkForEnd")
	end

--TODO:Screenshots

--TODO: DELETE GOD
--ScriptManager:spawnUnit(0,30,1,8,0,0) 
--ScriptManager:spawnUnit(0,30,2,8,0,0) 
--ScriptManager:moveCamera(0,0,0.77,0,"")

function checkForEnd()
	if ScriptManager:noUnitsLeft(0) or Unit:getUnitById(0) == nil then
		ScriptManager:runForPlayer(0, "endGameWait")
	end
end

function endGameWait()
	ScriptManager:continueAfterCutscene("MissionFailed")
end

--Game End Check
function CheckForGameEnd()
	MainGoalAccomplished(Missions.MISSION_MAIN,onMissionAccomplished,MissionFailed)		--Main Mission ID, Script to start if Accomplished, Script to start if Failed
 	end

--Start cutscene

function IntroStart()
	ScriptManager:startCutscene();
	ice_3()
end

function HideDialogue()
	ScriptManager:dismissSubtitle()
end

function EnableEndTurn()
	ScriptManager:enableEndTurnButton()
end

--IntroCutscene

function ice_3()
	ScriptManager:moveCamera(19,18,1,4,"Transport_1")
	ScriptManager:displaySubtitleAutoWithCallback(Text.T_MISSION_MAP_2_2_INTRO_TEXT,"")
	ScriptManager:sleep(3,"dialogueTransporter")
	ScriptManager:sleep(3,"SpawnTransporter")
	ScriptManager:sleep(9,"ToStart_Intro")
end

function dialogueTransporter()
	
	ScriptManager:sleep(5,"HideDialogue")
end

function Transport_1()
	ScriptManager:sleep(1.5,"Transport_2")
end

function Transport_2()
	ScriptManager:moveCamera(20,16,1,1.0,"")
	ScriptManager:sleep(2,"Transport_2_1")
end

function Transport_2_1()
	ScriptManager:moveCamera(19,17,0.7,2.0,"")
	
end

function ToStart_Intro()
	ScriptManager:dismissSubtitle()
	ScriptManager:sleep(1,"mainMissionCutscene")
end

--END

--Start Main Mission

function mainMissionCutscene()

	ScriptManager:dismissSubtitle()
	ScriptManager:moveCamera(14,2,0.9,2,"RevealFOW_Intro")
	ScriptManager:displaySubtitleAutoWithCallback(Text.T_MISSION_MAP_2_2_INTRO_TEXT_3,"")

end

function RevealFOW_Intro()

	ScriptManager:revealFogOfWar(14,2,1,true);
	ScriptManager:sleep(2,"MoveCam_1")

end

function MoveCam_1()

	ScriptManager:revealFogOfWar(14,2,1,false);
	ScriptManager:sleep(1,"MoveCam_1_1")

end

function MoveCam_1_1()

	ScriptManager:moveCamera(11,18,0.7,2,"Base_2")
	
end

function Base_2()

	ScriptManager:revealFogOfWar(11,18,1,true);
	ScriptManager:sleep(2.5,"Base_2_unit_2")
	
end

function Base_2_unit_2()

	ScriptManager:revealFogOfWar(11,18,1,false);
	ScriptManager:sleep(3,"ShowDialogueBase3")
	ScriptManager:moveCamera(4,8,0.7,3.5,"Base_3")
	
end

function ShowDialogueBase3()

	ScriptManager:dismissSubtitle()
	ScriptManager:displaySubtitleAutoWithCallback(Text.T_MISSION_MAP_2_2_INTRO_DIALOGUE_CUTSCENE_POST,"")

end

function Base_3()

	ScriptManager:revealFogOfWar(4,8,1,true);
	ScriptManager:sleep(3,"Base_3_unit_2")
end

function ShowDialogueHangar()

	ScriptManager:dismissSubtitle()
	ScriptManager:displaySubtitleAutoWithCallback(Text.T_MISSION_MAP_2_2_INTRO_DIALOGUE_07,"")
	ScriptManager:sleep(7,"HideDialogue")
	
end

function Base_3_unit_2()

	ScriptManager:revealFogOfWar(4,8,1,false);
	ScriptManager:moveCamera(19,17,0.9,3,"EndMainMissionCutscene")
	
end

function EndMainMissionCutscene()

	ScriptManager:stopCutscene()
	if ScriptManager:isControllerConnected() then
		ScriptManager:setControllerCursorPosition( 18, 17 );
	end
	ScriptManager:continueAfterCutscene("AddPostCutsceneTrigger")

end

function AddPostCutsceneTrigger()

	ScriptManager:sleep(2,"ShowPostCutsceneDialogueShow")
	ScriptManager:continueAfterCutscene("StartmainMission1")
	
end

function ShowPostCutsceneDialogueShow()

	ScriptManager:displaySubtitleAutoWithCallback(Text.T_MISSION_MAP_2_2_INTRO_TEXT_2,"")
	ScriptManager:sleep(5,"HideDialogue")
	ScriptManager:sleep(5,"EnableEndTurn")
	
end

function StartmainMission1()
	ScriptManager:setMissionStatus(Missions.MISSION_MAIN,MissionStatus.IN_PROGRESS)
end

--END



--KILL PLANES MISSION 
--TODO: FIX PLANES MISSION


function onTargetAttacked(attacker, defender)		
	targetID = -1
	if attacker:getTemplateId() == Units.UNIT_SU_FIGHTER then				
		targetID = attacker:getId()
	end
	if defender:getTemplateId() == Units.UNIT_SU_FIGHTER then
		targetID = defender:getId()
	end
	ScriptManager:removeTrigger("onTargetAttacked")
	
	if targetID ~= nil then
		ScriptManager:runForPlayer(0, "MoveCamToTarget")
	elseif targetID == nil then
		ScriptManager:runForPlayer(0, "startPlaneMissionAfterCutscene")
	end
end

function MoveCamToTarget()
	
	if ScriptManager:getMissionStatus(Missions.MISSION_KILL_PLANES)~=MissionStatus.IN_PROGRESS and targetID ~= nil then
		ScriptManager:blockUI(true)
		ScriptManager:enableSkipCutsceneButton()
		ScriptManager:moveCamera(Unit:getUnitById(targetID):getX(),Unit:getUnitById(targetID):getY(),0.9,2,"CamOnTarget")
	else
	ScriptManager:setMissionStatus(Missions.MISSION_KILL_PLANES,MissionStatus.IN_PROGRESS) 
	
	end
end

function CamOnTarget()
	ScriptManager:revealFogOfWar(Unit:getUnitById(targetID):getX(),Unit:getUnitById(targetID):getY(),1,true)
	ScriptManager:showMarkerAt(Unit:getUnitById(targetID):getX(),Unit:getUnitById(targetID):getY())
	ScriptManager:displaySubtitle(Text.T_MISSION_MAP_2_2_SECONDARY_2)
	ScriptManager:sleep(5,"HideDialogue")
	ScriptManager:moveCamera(Unit:getUnitById(targetID):getX(),Unit:getUnitById(targetID):getY(),0.9,3,"ZoomCamOutAndEndScript")
end

function ZoomCamOutAndEndScript()
	ScriptManager:moveCamera(Unit:getUnitById(targetID):getX(),Unit:getUnitById(targetID):getY(),0.8,2,"EndMetaCutScene")
	ScriptManager:hideMarkerAt(Unit:getUnitById(targetID):getX(),Unit:getUnitById(targetID):getY())
	ScriptManager:revealFogOfWar(Unit:getUnitById(targetID):getX(),Unit:getUnitById(targetID):getY(),1,false)
end

function EndMetaCutScene()
	ScriptManager:setMissionStatus(Missions.MISSION_KILL_PLANES,MissionStatus.IN_PROGRESS)
	ScriptManager:disableSkipCutsceneButton()
	ScriptManager:blockUI(false)
end


function StartPlane1Destroyed(unit)		--Check if both planes are death
if ScriptManager:getMissionStatus(Missions.MISSION_KILL_PLANES)~=MissionStatus.IN_PROGRESS 
	
then
	ScriptManager:setMissionStatus(Missions.MISSION_KILL_PLANES,MissionStatus.IN_PROGRESS)
	ScriptManager:removeTrigger("onTargetAttacked")
	
	ScriptManager:blockUI(false)
end
	local StartPlane1 = Unit:getUnitById(ScriptManager:getValue("StartPlane1"));
	local StartPlane2 = Unit:getUnitById(ScriptManager:getValue("StartPlane2"));
	--print(string.format("[SCRIPT] PlaneDestroyed() - StartPlane1: %s - StartPlane2: %s...", tostring(StartPlane1 ~= nil), tostring(StartPlane2 ~= nil)));
	if StartPlane1 == nil and StartPlane2 == nil then
	
		ScriptManager:removeTrigger("StartPlane1Destroyed");
		ScriptManager:runForPlayer(0,"startPlaneMissionAfterCutscene")
	end
end

function startPlaneMissionAfterCutscene()

	ScriptManager:setMissionStatus(Missions.MISSION_KILL_PLANES,MissionStatus.ACCOMPLISHED)
	if ScriptManager:getMissionStatus(Missions.MISSION_HANGAR) ~= MissionStatus.ACCOMPLISHED then
		ScriptManager:disableEndTurnButton()
		ScriptManager:blockUI(true)
		ScriptManager:sleep(2,"StartPlaneMission")
	end
end

function StartPlaneMission()

	ScriptManager:startCutscene()
	ScriptManager:moveCamera(4,4,0.9,2.0,"SpawnPlanes")
end

function SpawnPlanes()	--Spawn planes and start respawn mechanic
	ScriptManager:revealFogOfWar(4,4,2,true);
	spawnWave(EnemySpawn_4, "respawnEnemyReinforcements_4");
	ScriptManager:sleep(2,"ShowPlaneMarker")
	ScriptManager:sleep(1,"ShowDialogueHangar")
	ScriptManager:sleep(5,"MoveCam_3")
end

function ShowPlaneMarker()
	ScriptManager:showMarkerAt(4,4)
end

function MoveCam_3()
	ScriptManager:moveCamera(4,4,0.77,2.0,"StopHangarMission")
	ScriptManager:hideMarkerAt(4,4)
end

function StopHangarMission()
	ScriptManager:revealFogOfWar(4,4,2,false);
	ScriptManager:stopCutscene()
	ScriptManager:continueAfterCutscene("startHangarMissionWait")

end

function startHangarMissionWait()
	
	ScriptManager:sleep(1,"StartHangarMission")
end

function StartHangarMission()
	if  ScriptManager:getMissionStatus(Missions.MISSION_HANGAR) ~= MissionStatus.IN_PROGRESS then 
		ScriptManager:setMissionStatus(Missions.MISSION_HANGAR,MissionStatus.IN_PROGRESS)
	end
	ScriptManager:blockUI(false)
	ScriptManager:enableEndTurnButton()
end

--END

function displayWarning()
	
	ScriptManager:sleep(5,"HideDialogue")
	--ScriptManager:removeTrigger(displayWarning)
	if 	ScriptManager:getMissionStatus(Missions.MISSION_MAIN_PROTECT) ~= MissionStatus.IN_PROGRESS and 
		ScriptManager:getMissionStatus(Missions.MISSION_MAIN_PROTECT) ~= MissionStatus.ACCOMPLISHED	then
		ScriptManager:setMissionStatus(Missions.MISSION_MAIN_PROTECT, MissionStatus.IN_PROGRESS);
	end
end

function depot_damaged()

	ScriptManager:displaySubtitle(Text.T_MISSION_MAP_2_2_WARNING)
	ScriptManager:sleep(5,"HideDialogue")
end

--Supply bases behaviour
--TODO: FIX THE SUPPLY DEPOT BEHAVIOUR

function onHq1Captured(unit)
		DisableEndTurnButton()
	if ScriptManager:getMissionStatus(Missions.MISSION_BASE_1) == MissionStatus.IN_PROGRESS then
		ScriptManager:setMissionStatus(Missions.MISSION_BASE_1, MissionStatus.ACCOMPLISHED)
		ScriptManager:addOnUnitDestroyedTrigger(-1, -1, UnitId.HQ_01, "DepotDestroyed", "HQdestroyed1");
		displayWarning()
	end
	supplydepot_mission_counter()		--start counting and update mission
	if ScriptManager:getMissionProgress(Missions.MISSION_MAIN) ~= 3 then
		ScriptManager:sleep(1,"SpawnReinforcements_1")
		ScriptManager:removeTrigger("HQ1DC");
		ScriptManager:removeTrigger("SpawnEnemyReinforcements_1")
		ScriptManager:addOnTurnStartTrigger(1,-1,ScriptManager:getCurrentRound()+1, "SpawnEnemyReinforcements_2", "SpawnEnemyReinforcements_2");
		ScriptManager:addOnTurnStartTrigger(1,-1,ScriptManager:getCurrentRound()+6, "SpawnEnemyReinforcements_3", "SpawnEnemyReinforcements_3");
	end	
end

function onHq2Captured(unit)
	DisableEndTurnButton()
	if ScriptManager:getMissionStatus(Missions.MISSION_BASE_2) == MissionStatus.IN_PROGRESS then
		ScriptManager:setMissionStatus(Missions.MISSION_BASE_2, MissionStatus.ACCOMPLISHED);
		ScriptManager:addOnUnitDestroyedTrigger(-1, -1, UnitId.HQ_02, "DepotDestroyed", "HQdestroyed2");
		--print(string.format("[SCRIPT] Base 2 captured"));
		displayWarning()	
	end
	

	supplydepot_mission_counter()		--start counting and update mission
	
	ScriptManager:removeTrigger("HQ2DC");
	ScriptManager:removeTrigger("SpawnEnemyReinforcements_2")
	
	if ScriptManager:getMissionProgress(Missions.MISSION_MAIN) ~= 3 then
		ScriptManager:sleep(1,"SpawnReinforcements_2")
	end
	
		
end

function onHq3Captured(unit)
	DisableEndTurnButton()
	if ScriptManager:getMissionStatus(Missions.MISSION_BASE_3) == MissionStatus.IN_PROGRESS then
		ScriptManager:setMissionStatus(Missions.MISSION_BASE_3, MissionStatus.ACCOMPLISHED);
		ScriptManager:addOnUnitDestroyedTrigger(-1, -1, UnitId.HQ_03, "DepotDestroyed", "HQdestroyed3");
		--print(string.format("[SCRIPT] Base 3 captured"));
		displayWarning()	
	end
	
		supplydepot_mission_counter()				--start counting and update mission
		
	if ScriptManager:getMissionProgress(Missions.MISSION_MAIN) ~= 3 then
		ScriptManager:sleep(1,"SpawnReinforcements_3")
		--print(string.format("[SCRIPT] spawn reinforcements started"));
	end
		ScriptManager:removeTrigger("HQ3DC");
		ScriptManager:removeTrigger("SpawnEnemyReinforcements_3")
		ScriptManager:removeTrigger("SpawnEnemyReinforcements_4")
end

--TODO: COUNTER

function supplydepot_mission_counter()
	ScriptManager:setMissionProgress(Missions.MISSION_MAIN,ScriptManager:getMissionProgress(Missions.MISSION_MAIN)+1)				--Update on mission progress when one AA is destroyed
	showprogresstext()
	ScriptManager:enableEndTurnButton()
	if ScriptManager:getMissionProgress(Missions.MISSION_MAIN) == 3 then
		ScriptManager:setMissionStatus(Missions.MISSION_MAIN,MissionStatus.ACCOMPLISHED)
		ScriptManager:runForPlayer(0,"onMissionAccomplished")
		
	end
	
end

function showprogresstext()

	ScriptManager:preparePopupText();
	ScriptManager:addPopupTextParameter("x",ScriptManager:getMissionProgress(Missions.MISSION_MAIN))
	--print(string.format("[SCRIPT] Count mission objective()..."));
	ScriptManager:displaySubtitleAutoWithCallback(Text.T_MISSION_MAP_2_2_MAIN_2_COUNTER_NEW,"")							--supply depots captured! x/3		
	ScriptManager:sleep(3,"showWarning")	
end


function showWarning()
	ScriptManager:dismissSubtitle()
	ScriptManager:sleep(3,"displayWarning")

end

--MISSION FAIL TRIGGER END MISSION

function DepotDestroyed()

	ScriptManager:runForPlayer(0,"DepotDestroyed_1")
	
end

function DepotDestroyed_1()
	DisableEndTurnButton()
	ScriptManager:blockUI(true)
	if ScriptManager:getMissionStatus(Missions.MISSION_BASE_1) == MissionStatus.IN_PROGRESS then
		ScriptManager:setMissionStatus(Missions.MISSION_MAIN,MissionStatus.FAILED)
		ScriptManager:displaySubtitleAutoWithCallback(Text.T_MISSION_MAP_2_2_FAILED,"")
		ScriptManager:sleep(5,"before_MissionFailed")
	elseif ScriptManager:getMissionStatus(Missions.MISSION_BASE_2) == MissionStatus.IN_PROGRESS then
		ScriptManager:setMissionStatus(Missions.MISSION_MAIN,MissionStatus.FAILED)
		ScriptManager:sleep(5,"before_MissionFailed")
		ScriptManager:displaySubtitleAutoWithCallback(Text.T_MISSION_MAP_2_2_FAILED,"")
	elseif ScriptManager:getMissionStatus(Missions.MISSION_BASE_3) == MissionStatus.IN_PROGRESS then
		ScriptManager:setMissionStatus(Missions.MISSION_MAIN,MissionStatus.FAILED)
		ScriptManager:displaySubtitleAutoWithCallback(Text.T_MISSION_MAP_2_2_FAILED,"")
		ScriptManager:sleep(5,"before_MissionFailed")
	end
end

function before_MissionFailed()

	ScriptManager:runForPlayer(0,"MissionFailed")

end

function MissionFailed()
	HideDialogue()
	ScriptManager:blockUI(true)
	ScriptManager:disableEndTurnButton()
	ScriptManager:displaySubtitleAutoWithCallback(Text.T_MAP_3_2_MISSION_FAILED,"")
	ScriptManager:sleep(3,"HideDialogue")
	ScriptManager:sleep(5,"EndGame")
end





--Main mission updated and mission end

function onMissionAccomplished()
	DisableEndTurnButton()
	ScriptManager:blockUI(true)
	local progress = ScriptManager:getMissionStatus(Missions.MISSION_MAIN)
	--[[
	if ScriptManager:getMissionStatus(Missions.MISSION_MAIN) == MissionStatus.ACCOMPLISHED then
	ScriptManager:setMissionStatus(Missions.MISSION_MAIN_PROTECT, MissionStatus.ACCOMPLISHED)
	end
	]]--
	ScriptManager:displaySubtitleAutoWithCallback(Text.T_MISSION_MAP_2_2_OUTRO,"OutroDialogue")
	--ScriptManager:sleep(3,"OutroDialogue")
	--print(string.format("MISSION_MAIN = %i", progress))
end

function OutroDialogue()
	ScriptManager:blockUI(true)
	if ScriptManager:getMissionStatus(Missions.MISSION_MAIN) == MissionStatus.ACCOMPLISHED then
	   ScriptManager:setMissionStatus(Missions.MISSION_MAIN_PROTECT, MissionStatus.ACCOMPLISHED)
	end
	
	ScriptManager:displaySubtitleAutoWithCallback(Text.T_MISSION_VICTORY,"")
	ScriptManager:sleep(4,"EndGame")
end

function EndGame()
	ScriptManager:blockUI(true)
	ScriptManager:endGame()
end

function onHangarDestroyedOrCaptured(unit)
	if ScriptManager:getMissionStatus(Missions.MISSION_HANGAR) == MissionStatus.IN_PROGRESS or ScriptManager:getMissionStatus(Missions.MISSION_HANGAR) == MissionStatus.NOT_STARTED then
		ScriptManager:setMissionStatus(Missions.MISSION_HANGAR, MissionStatus.ACCOMPLISHED);
		ScriptManager:blockUI(true)
		ScriptManager:removeTrigger("hangardestroyed")
	end
	ScriptManager:sleep(4,"unblock")
end

function unblock()

	ScriptManager:blockUI(false)
	
end

-- Spawning behaviour


function SpawnReinforcements_1(playerId, turnId, roundId)
	ScriptManager:startCutscene();
	ScriptManager:moveCamera(13,2,1,2.0,"wait_sleep1")
	
end

function wait_sleep1()
	ScriptManager:sleep(0.5,"spawnRe1")
end

function spawnRe1()
	ScriptManager:displaySubtitleAutoWithCallback(Text.T_MISSION_REINFORCEMENTS,"")
	local unit1 = ScriptManager:spawnUnit(0, 4, 13, 2, 0, 0);
	spawnReinforcement = unit1:getId()
	--ScriptManager:moveUnit(spawnReinforcement,13,2, "")
	local unit2 = ScriptManager:spawnUnit(0, 0, 12, 1, 0, 0);
	spawnReinforcement = unit2:getId()
	--ScriptManager:moveUnit(spawnReinforcement,12,2, "")
	local unit3 = ScriptManager:spawnUnit(0, 0, 13, 1, 0, 0);
	spawnReinforcement = unit3:getId()
	--ScriptManager:moveUnit(spawnReinforcement,13,1, "")
	ScriptManager:sleep(1,"SpawnSleep1")
end

function SpawnSleep1()
	ScriptManager:moveCamera(14,2,0.7,2,"end_reinforcement_cutscene")
	
		
end

function SpawnReinforcements_2(playerId, turnId, roundId)
	ScriptManager:startCutscene();
	ScriptManager:moveCamera(10,20,1,2,"wait_Resleep2")
	
end

function wait_Resleep2()
	ScriptManager:sleep(0.5,"spawnRe2")
end

function spawnRe2()
	ScriptManager:displaySubtitleAutoWithCallback(Text.T_MISSION_REINFORCEMENTS,"")
	local unit1 = ScriptManager:spawnUnit(0, 4, 10, 20, 0, 0);
	spawnReinforcement = unit1:getId()
	--ScriptManager:moveUnit(spawnReinforcement,13,2, "")
	local unit2 = ScriptManager:spawnUnit(0, 0, 11, 20, 0, 0);
	spawnReinforcement = unit2:getId()
	--ScriptManager:moveUnit(spawnReinforcement,12,2, "")
	local unit3 = ScriptManager:spawnUnit(0, 0, 10, 19, 0, 0);
	spawnReinforcement = unit3:getId()
	--ScriptManager:moveUnit(spawnReinforcement,13,1, "")
	ScriptManager:sleep(1,"SpawnReSleep2")
end

function SpawnReSleep2()
	ScriptManager:moveCamera(8,20,0.7,2.0,"end_reinforcement_cutscene")
	
	
	
end

function SpawnReinforcements_3(playerId, turnId, roundId)
	ScriptManager:startCutscene()
	ScriptManager:moveCamera(1,5,1,2.0,"wait_Resleep3")
end

function wait_Resleep3()
	ScriptManager:sleep(0.5,"spawnRe3")
end

function spawnRe3()
	ScriptManager:displaySubtitleAutoWithCallback(Text.T_MISSION_REINFORCEMENTS,"")
	ScriptManager:spawnUnit(0, 4, 1, 5, 0, 0);
	ScriptManager:spawnUnit(0, 0, 1, 6, 0, 0);
	ScriptManager:spawnUnit(0, 0, 1, 4, 0, 0);
	ScriptManager:sleep(1,"SpawnReSleep3")
end

function SpawnReSleep3()
	ScriptManager:moveCamera(4,8,0.7,2.0,"end_reinforcement_cutscene")	
end

function end_reinforcement_cutscene()
	ScriptManager:sleep(0.5,"sleep_and_stop")
end

function sleep_and_stop()
	ScriptManager:stopCutscene()
	EnableEndTurnButton()
end

--	Enemy Depot 1 Spawner
--	Respawns enemies after all killed and x turns has past

--	Enemy wave 1
EnemySpawn_1 = --unit,x,y,squadID
{
	{Units.UNIT_SU_RIFLE,14,2,18,AiState.ATTACK_AGGRESSIVE},
	{Units.UNIT_SU_RIFLE,14,2,18,AiState.ATTACK_AGGRESSIVE},
	{Units.UNIT_SU_MORTAR,14,2,18,AiState.ATTACK_AGGRESSIVE}
}

--	Enemy wave 2
EnemySpawn_2 = --unit,x,y,squadID
{
	{Units.UNIT_SU_RIFLE,11,18,19,AiState.ATTACK_AGGRESSIVE},
	{Units.UNIT_SU_MORTAR,11,18,19,AiState.ATTACK_AGGRESSIVE},
	{Units.UNIT_SU_TANK_T34,11,18,19,AiState.ATTACK_AGGRESSIVE}
}

--	Enemy wave 3
EnemySpawn_3 = --unit,x,y,squadID
{
	{Units.UNIT_SU_TANK_T34,4,8,20,AiState.ATTACK_AGGRESSIVE},
	{Units.UNIT_SU_TANK_T34,4,8,20,AiState.ATTACK_AGGRESSIVE},
	{Units.UNIT_SU_AA,4,8,20,AiState.ATTACK_AGGRESSIVE}
}

EnemySpawn_4 = --unit,x,y,squadID
{
	{Units.UNIT_SU_FIGHTER,5,4,30,AiState.AIRPLANE},
	{Units.UNIT_SU_FIGHTER,4,3,30,AiState.AIRPLANE},
}

-- Spawns #wave enemies and creates 3 trigger
function spawnWave(wave, respawnCallback) 
	ScriptManager:removeTrigger(respawnCallback);
	for i = 1, #wave do
		local data = wave[i];
		local spawnedUnit = ScriptManager:spawnUnit(1, data[1], data[2], data[3], data[4], data[5]);
		ScriptManager:addOnUnitDestroyedTrigger(1, -1, spawnedUnit:getId(), respawnCallback, respawnCallback);
	end
end

--check if base is still alive
function SpawnEnemyReinforcements_1(playerId, turnId, roundId)
	--print(string.format("[SCRIPT]spawnEnemyWaveDepot_1"));
	if ScriptManager:getMissionStatus(Missions.MISSION_BASE_1) == MissionStatus.IN_PROGRESS then	
		spawnWave(EnemySpawn_1, "respawnEnemyReinforcements_1");
	end
end

--respawns units if all spawned units are dead
function respawnEnemyReinforcements_1()
	local enemyCounter = ScriptManager:getValue("r1_counter") + 1;
	if enemyCounter == 3 then
		--print(string.format("[SCRIPT]resetSpawnEnemy1"));
		ScriptManager:addOnTurnStartTrigger(1,-1,ScriptManager:getCurrentRound()+2, "SpawnEnemyReinforcements_1", "SpawnEnemyReinforcements_1");
		ScriptManager:storeValue("r1_counter", 0);
	else
		ScriptManager:storeValue("r1_counter", enemyCounter);
	end
end

--check if base is still alive
function SpawnEnemyReinforcements_2(playerId, turnId, roundId)
	--print(string.format("[SCRIPT]spawnEnemyWaveDepot_1"));
	if ScriptManager:getMissionStatus(Missions.MISSION_BASE_2) == MissionStatus.IN_PROGRESS and
	ScriptManager:getCurrentRound() < 20 then	
			spawnWave(EnemySpawn_2, "respawnEnemyReinforcements_2");
	end
end

--respawns units if all spawned units are dead
function respawnEnemyReinforcements_2()
	local enemyCounter = ScriptManager:getValue("r2_counter") + 1;
	if enemyCounter == 3 then
		--print(string.format("[SCRIPT]resetSpawnEnemy2"));
		ScriptManager:addOnTurnStartTrigger(1,-1,ScriptManager:getCurrentRound()+3, "SpawnEnemyReinforcements_2", "SpawnEnemyReinforcements_2");
		ScriptManager:storeValue("r2_counter", 0);
	else
		ScriptManager:storeValue("r2_counter", enemyCounter);
	end
end

--check if base is still alive
function SpawnEnemyReinforcements_3(playerId, turnId, roundId)
	--print(string.format("[SCRIPT]spawnEnemyWaveDepot_3"));
	if ScriptManager:getMissionStatus(Missions.MISSION_BASE_3) == MissionStatus.IN_PROGRESS and
	ScriptManager:getCurrentRound() < 30 then	
			spawnWave(EnemySpawn_3, "respawnEnemyReinforcements_3");
	end
end

--respawns units if all spawned units are dead
function respawnEnemyReinforcements_3()
	local enemyCounter = ScriptManager:getValue("r3_counter") + 1;
	if enemyCounter == 3 then
		--print(string.format("[SCRIPT]resetSpawnEnemy3"));
		ScriptManager:addOnTurnStartTrigger(1,-1,ScriptManager:getCurrentRound()+5, "SpawnEnemyReinforcements_3", "SpawnEnemyReinforcements_3");
		ScriptManager:storeValue("r3_counter", 0);
	else
		ScriptManager:storeValue("r3_counter", enemyCounter);
	end
end

--check if base is still alive
function SpawnEnemyReinforcements_4(playerId, turnId, roundId)
	--print(string.format("[SCRIPT]spawnEnemyWaveDepot_4"));
	if ScriptManager:getMissionStatus(Missions.MISSION_HANGAR) == MissionStatus.IN_PROGRESS and
	ScriptManager:getCurrentRound() < 20 then	
			spawnWave(EnemySpawn_4, "respawnEnemyReinforcements_4");
	end
end

--respawns units if all spawned units are dead
function respawnEnemyReinforcements_4()
	local enemyCounter = ScriptManager:getValue("r4_counter") + 1;
	if  ScriptManager:getMissionProgress(Missions.MISSION_HANGAR) == MissionStatus.IN_PROGRESS and enemyCounter == 2 then
		--print(string.format("[SCRIPT]resetSpawnEnemy4"));
		ScriptManager:addOnTurnStartTrigger(1,-1,ScriptManager:getCurrentRound()+2, "SpawnEnemyReinforcements_4", "SpawnEnemyReinforcements_4");
		ScriptManager:storeValue("r4_counter", 0);
	else
		ScriptManager:storeValue("r4_counter", enemyCounter);
	end
end

--END RESPAWN FUNCTIONS

function SpawnTransporter()
		SpawnTransporter2()
		local transporter = ScriptManager:spawnUnit(
		0 ,12 , 	--PLAYER, UNIT
		18, 21 ,	--X, Y
		30, 5); --SQUADID, STATE
		ScriptManager:storeValue("transporterId",transporter:getId());
		
		local unit1 = ScriptManager:spawnUnit(
		0 ,1 , 	--PLAYER, UNIT
		18, 21 ,	--X, Y
		30, 5); --SQUADID, STATE
		ScriptManager:storeValue("unit1",unit1:getId());
		
		local unit2 = ScriptManager:spawnUnit(
		0 ,1 , 	--PLAYER, UNIT
		18, 21 ,	--X, Y
		30, 5); --SQUADID, STATE
		ScriptManager:storeValue("unit2",unit2:getId());
		
		local unit3 = ScriptManager:spawnUnit(
		0 ,1 , 	--PLAYER, UNIT
		18, 21 ,	--X, Y
		30, 5); --SQUADID, STATE
		ScriptManager:storeValue("unit3",unit3:getId());
		ScriptManager:sleep(0,"MoveTransporter")
end

function MoveTransporter()
	local transporter = ScriptManager:getValue("transporterId");
	ScriptManager:moveUnit(transporter,19,18,"UnloadUnit1")
end

function UnloadUnit1()
	local transporter = ScriptManager:getValue("transporterId");
	local unit1 = ScriptManager:getValue("unit1");
	ScriptManager:unloadUnitFromTransporter(transporter,unit1,18,17,"UnloadUnit2")
end

function UnloadUnit2()
	local transporter = ScriptManager:getValue("transporterId");
	local unit2 = ScriptManager:getValue("unit2");
	ScriptManager:unloadUnitFromTransporter(transporter,unit2,18,18,"UnloadUnit3")
end

function UnloadUnit3()
	local transporter = ScriptManager:getValue("transporterId");
	local unit3 = ScriptManager:getValue("unit3");
	ScriptManager:unloadUnitFromTransporter(transporter,unit3,18,19,"MoveTransporter2")
end

function SpawnTransporter2()
		local transporter = ScriptManager:spawnUnit(
		0 ,12 , 	--PLAYER, UNIT
		21, 17 ,	--X, Y
		30, 5); --SQUADID, STATE
		ScriptManager:storeValue("transporterId2",transporter:getId());
		
		local unit1 = ScriptManager:spawnUnit(
		0 ,0 , 	--PLAYER, UNIT
		21, 17 ,	--X, Y
		30, 5); --SQUADID, STATE
		ScriptManager:storeValue("unit1_2",unit1:getId());
		
		local unit2 = ScriptManager:spawnUnit(
		0 ,0 , 	--PLAYER, UNIT
		21, 17 ,	--X, Y
		30, 5); --SQUADID, STATE
		ScriptManager:storeValue("unit2_2",unit2:getId());
		
		local unit3 = ScriptManager:spawnUnit(
		0 ,0 , 	--PLAYER, UNIT
		21, 17 ,	--X, Y
		30, 5); --SQUADID, STATE
		ScriptManager:storeValue("unit3_2",unit3:getId());
end

function MoveTransporter2()
	local transporter = ScriptManager:getValue("transporterId2");
	ScriptManager:moveUnit(transporter,20,16,"UnloadUnit1_2")
end

function UnloadUnit1_2()
	local transporter = ScriptManager:getValue("transporterId2");
	local unit1 = ScriptManager:getValue("unit1_2");
	ScriptManager:unloadUnitFromTransporter(transporter,unit1,20,15,"UnloadUnit2_2")
end

function UnloadUnit2_2()
	local transporter = ScriptManager:getValue("transporterId2");
	local unit2 = ScriptManager:getValue("unit2_2");
	ScriptManager:unloadUnitFromTransporter(transporter,unit2,19,15,"UnloadUnit3_2")
end

function UnloadUnit3_2()
	local transporter = ScriptManager:getValue("transporterId2");
	local unit3 = ScriptManager:getValue("unit3_2");
	ScriptManager:unloadUnitFromTransporter(transporter,unit3,19,16,"")
end