require "lua/Constants"
require "lua/Functions"
require "lua/Text"
require "lua/Units"
require "math"

--	MISSIONS

Missions = {
	MISSION_MAIN_DEFEAT = 1,
	MISSION_MAIN_SURVIVE = 2,
	MISSION_SECONDARY_PROTECT = 3,
	MISSION_SECONDARY_DESTROY = 4,
}

-- CONTENT TABLES

-- ENEMY WAVES 

waves1 = {
	{Units.UNIT_DE_TANK_4, Units.UNIT_DE_TANK_4},																	--1st wave
	{Units.UNIT_DE_PANZERFAUST, Units.UNIT_DE_MP40, Units.UNIT_DE_PANZERFAUST},										--2nd wave
	{Units.UNIT_DE_TANK_3, Units.UNIT_DE_TANK_3, Units.UNIT_DE_TANK_3, Units.UNIT_DE_TANK_3, Units.UNIT_DE_TANK_3},	--3rd wave
	{Units.UNIT_DE_TIGER},																							--4th wave
	{Units.UNIT_DE_TANK_3, Units.UNIT_DE_TANK_3, Units.UNIT_DE_TANK_3, Units.UNIT_DE_TANK_3},						--5th wave
	{Units.UNIT_DE_TANK_3, Units.UNIT_DE_TANK_3, Units.UNIT_DE_TANK_3, Units.UNIT_DE_TANK_3}						--6th wave
}

waves2 = {
	{Units.UNIT_DE_PANZERFAUST, Units.UNIT_DE_MP40, Units.UNIT_DE_MP40},				--1st wave
	{Units.UNIT_DE_MP40, Units.UNIT_DE_MP40, Units.UNIT_DE_MP40, Units.UNIT_DE_MP40}	--2nd wave
}

waves3 = {
	{Units.UNIT_DE_FIGHTER, Units.UNIT_DE_FIGHTER},							--1st wave
	{Units.UNIT_DE_FIGHTER,Units.UNIT_DE_FIGHTER, Units.UNIT_DE_BOMBER},	--2nd wave
	{Units.UNIT_DE_BOMBER, Units.UNIT_DE_FIGHTER, Units.UNIT_DE_FIGHTER},	--3rd wave
	{Units.UNIT_DE_FIGHTER, Units.UNIT_DE_FIGHTER, Units.UNIT_DE_FIGHTER}	--4th wave
}

wavesRandom = {							--1st wave
	{Units.UNIT_DE_TIGER},	
	{Units.UNIT_DE_TIGER},	
	{Units.UNIT_DE_TIGER},	
	{Units.UNIT_DE_TIGER, Units.UNIT_DE_TIGER},	
}

waves4 = {
	{Units.UNIT_DE_TANK_4, Units.UNIT_DE_TANK_4, Units.UNIT_DE_TANK_4, Units.UNIT_DE_TANK_3, Units.UNIT_DE_TANK_3},		--1st wave
	{Units.UNIT_DE_TANK_4, Units.UNIT_DE_TANK_4, Units.UNIT_DE_TIGER},													--2nd wave
}

-- ENEMY SPAWN CONTROLLER


--TIMERS in ROUNDS

spawnTimer1 = {		--north spawner waves1 (mixedunits)
	1, 3, 4, 6, 8
}

spawnTimer2 = {		-- middle spawner waves2 (mixed)
	1, 4, 7, 9, 10, 11, 12, 15, 20  
}

spawnTimer3 = { 	--south spawner waves3 (Planes)
	2, 3, 5, 7, 8, 10, 12, 15, 19, 22 
}

spawnTimer4 = {		--Tiger Spawner wavesRandom (Random)
	2, 5, 8, 10, 12, 14, 15, 16, 17, 18
}

spawnTimer5 = {		--south Spawner wave 5 (mixed)
	5, 10, 14, 17, 20
}

--POSITIONS

spawnPos = {		--spawn positions for Tiger Tank
	{2, 1, 4}, --(x,y, spawn position ID)
	{1, 14, 3},
	{1, 21, 2},
	{3, 27, 1}
}

basePos = {   --Player base positions
	{16,19},
	{18,15},
	{14,13}
}

--	START GAME

function onGameStart()
	--print(string.format("[SCRIPT] onGameStart()..."));
	ScriptManager:preparePopupText()
--	CREATE MISSIONS

--	Starting gold
	ScriptManager:gainGold(0,500)
	
--	MAIN MISSION 1: Defeat 70 germans
	ScriptManager:createMission(Missions.MISSION_MAIN_DEFEAT,true, Text.T_MAP_3_3_MISSION_MAIN)
	ScriptManager:setMissionXp(Missions.MISSION_MAIN_DEFEAT,350)
	
--	MAIN MISSION 2: Hold at least 1 base
	ScriptManager:createMission(Missions.MISSION_MAIN_SURVIVE, true, Text.T_MAP_3_3_MISSION_MAIN_2);
	ScriptManager:setMissionProgress(Missions.MISSION_MAIN_SURVIVE,3)
	ScriptManager:setMissionXp(Missions.MISSION_MAIN_SURVIVE,150)
	
--	SECONDARY MISSION 1: Don't loose any base
	ScriptManager:createMission(Missions.MISSION_SECONDARY_PROTECT, false, Text.T_MAP_3_3_MISSION_SECONDARY);
	ScriptManager:setMissionXp(Missions.MISSION_SECONDARY_PROTECT,250)

--	MISSION TRIGGER
	
	--PLAYER BASE DESTROYED TRIGGER
	ScriptManager:addOnUnitDestroyedTrigger(-1,Units.UNIT_SU_HQ,-1,"BaseDestroyed","BaseDestroyed")
	
	--Main 2: Kill X Germans
	ScriptManager:addOnUnitDestroyedTrigger(1,-1,-1,"enemyUnitDestroyed","enemyUnitDestroyed")
	
	--Secondary: Don't loose any base!
	ScriptManager:addOnUnitHasAttackedTrigger(-1,Units.UNIT_SU_HQ,-1,"onTargetAttacked","onTargetAttacked")
	
	
	ScriptManager:addOnTurnStartTrigger(0,-1,-1,"CheckForGameEnd")
	
--	START GAMEFUNCTIONS
	--Create spawn triggers
	for i = 1, #spawnTimer1 do
		local round = spawnTimer1[i]
		ScriptManager:addOnTurnStartTrigger(1,-1,round,"spawnEnemyBase1",string.format("spawn1 %i", round))
	end
	for i = 1, #spawnTimer2 do
		local round = spawnTimer2[i]
		ScriptManager:addOnTurnStartTrigger(1,-1,round,"spawnEnemyBase2",string.format("spawn2 %i", round))
	end
	for i = 1, #spawnTimer3 do
		local round = spawnTimer3[i]
		ScriptManager:addOnTurnStartTrigger(1,-1,round,"spawnEnemyBase3",string.format("spawn3 %i", round))
	end
	for i = 1, #spawnTimer4 do
		local round = spawnTimer4[i]
		ScriptManager:addOnTurnStartTrigger(1,-1,round,"spawnEnemyBase4",string.format("spawn4 %i", round))
	end
	for i = 1, #spawnTimer5 do
		local round = spawnTimer5[i]
		ScriptManager:addOnTurnStartTrigger(1,-1,round,"spawnEnemyBase5",string.format("spawn5 %i", round))
	end
	
	--Add basic wave progression variables
	ScriptManager:storeValue("base1Wave",1)
	ScriptManager:storeValue("base2Wave",1)
	ScriptManager:storeValue("base3Wave",1)
	ScriptManager:storeValue("base4Wave",1)
	ScriptManager:storeValue("base5Wave",1)
	
	--Initiate IntroCutScene()
	ScriptManager:sleep(0,"IntroCutScene")
	
	--Set camera starting position
	ScriptManager:moveCamera(16,17,0.9,0,"")
	
	return 1
end

--Game End Check	
function CheckForGameEnd()
	MainGoalAccomplished(2,missionAccomplishedEndAfterCutscene,MissionFailedEndWait)		--Main Mission ID, Script to start if Accomplished, Script to start if Failed
end

IntroDialogues = {	--Dialogue, wait until next dialogue (includes hide sleeper), hide after x seconds
	{Text.T_MAP_3_3_DIALOGUE_1, 1, 5},
	{Text.T_MAP_3_3_DIALOGUE_2, 2, 3},
	{Text.T_MAP_3_3_DIALOGUE_3, 1, 8},
	{Text.T_MAP_3_3_DIALOGUE_4, 6, 5}
}

_introDialogueIndex = 1

function showDialogues()
	local textDialogue = IntroDialogues[_introDialogueIndex][1]
	local timeDialouge = IntroDialogues[_introDialogueIndex][2]
	local hideDialouge = IntroDialogues[_introDialogueIndex][3]
	ScriptManager:displaySubtitle(textDialogue)
	ScriptManager:sleep(hideDialouge, "HideDialogue")
	_introDialogueIndex = _introDialogueIndex + 1
	
	if _introDialogueIndex <= #IntroDialogues then 
		ScriptManager:sleep(timeDialouge+hideDialouge, "showDialogues")
	end
end

--	END

--END

--	INTRO CUTSCENE

function IntroCutScene() 					--Zoom camera on starting position
	ScriptManager:startCutscene()
	ScriptManager:moveCamera(16,17,0.7,8,"CamInPositionMainMission")
	ScriptManager:sleep(1.5,"showDialogues")
	spawnAndMoveUnits()
end

function CamInPositionMainMission()			--Show markers at player base
	ScriptManager:showMarkerAt(14,13)	
	ScriptManager:showMarkerAt(16,19)
	ScriptManager:showMarkerAt(18,15)
	ScriptManager:sleep(3.5,"HideAllMarkers")
	ScriptManager:sleep(4,"TeaserBunker")
end

--CUTSCENE: units spawning & moving

--Tanks spawning and moving into position

--Tank1
function spawnAndMoveUnits()
	ScriptManager:moveUnit(17,16,18,"moveTank2")
end

--Tank2
function moveTank2()
	local unit = ScriptManager:spawnUnit(0, Units.UNIT_SU_TANK_T34, 21, 18, 0, 0);
	ScriptManager:moveUnit(unit:getId(), 17, 19, "SpawnTransporter")
end

--Tank3
function moveTank3()
	local unit = ScriptManager:spawnUnit(0, Units.UNIT_SU_TANK_DESTROYER, 21, 16, 0, 0);
	ScriptManager:moveUnit(unit:getId(), 17, 18, "")
end

--Transporter and infantry
function SpawnTransporter() -- Spawn transporter and load with units
		local x = 21
		local y = 14
		local transporter = ScriptManager:spawnUnit(
		0 ,Units.UNIT_SU_TRANSPORT_TRUCK , 	--PLAYER, UNIT
		x, y ,	--X, Y
		30, 5); --SQUADID, STATE
		ScriptManager:storeValue("transporterId",transporter:getId());
		
		local unit1 = ScriptManager:spawnUnit(
		0 ,Units.UNIT_SU_RIFLE , 	--PLAYER, UNIT
		x, y ,	--X, Y
		30, 5); --SQUADID, STATE
		ScriptManager:storeValue("unit1",unit1:getId());
		
		local unit2 = ScriptManager:spawnUnit(
		0 ,Units.UNIT_SU_RIFLE , 	--PLAYER, UNIT
		x, y ,	--X, Y
		30, 5); --SQUADID, STATE
		ScriptManager:storeValue("unit2",unit2:getId());
		
		local unit3 = ScriptManager:spawnUnit(
		0 ,Units.UNIT_SU_MORTAR , 	--PLAYER, UNIT
		x, y ,	--X, Y
		30, 5); --SQUADID, STATE
		ScriptManager:storeValue("unit3",unit3:getId());
		ScriptManager:sleep(0,"MoveTransporter")
end

function MoveTransporter()
	local transporter = ScriptManager:getValue("transporterId");
	ScriptManager:moveUnit(transporter,16,14,"UnloadUnit1")
end

--Unload transporter units
function UnloadUnit1()
	local transporter = ScriptManager:getValue("transporterId");
	local unit1 = ScriptManager:getValue("unit1");
	ScriptManager:unloadUnitFromTransporter(transporter,unit1,15,13,"UnloadUnit2")
end

function UnloadUnit2()
	local transporter = ScriptManager:getValue("transporterId");
	local unit2 = ScriptManager:getValue("unit2");
	ScriptManager:unloadUnitFromTransporter(transporter,unit2,15,15,"UnloadUnit3")
end

function UnloadUnit3()
	local transporter = ScriptManager:getValue("transporterId");
	local unit3 = ScriptManager:getValue("unit3");
	ScriptManager:unloadUnitFromTransporter(transporter,unit3,16,15,"moveTank3")
end

--Move Camera to bunkers (INTRO CUTSCENE HINT)

_bunkerPos = {  	 -- bunker positions x,y
	{14,21},
	{13,17},
	{11,13},
	{12,7}
}

bunkerIndex = 1 	-- Index of table content

function TeaserBunker()
	local x = _bunkerPos[bunkerIndex][1]
	local y = _bunkerPos[bunkerIndex][2]
	ScriptManager:moveCamera(x, y,0.8,2,"camAtBunker")
end

function camAtBunker()
	local x = _bunkerPos[bunkerIndex][1]
	local y = _bunkerPos[bunkerIndex][2]
	ScriptManager:revealFogOfWar(x,y,1,true)
	ScriptManager:showMarkerAt(x,y)
	ScriptManager:sleep(1, "bunkerFinished")
end

function bunkerFinished()
	local x = _bunkerPos[bunkerIndex][1]
	local y = _bunkerPos[bunkerIndex][2]
	ScriptManager:revealFogOfWar(x,y,1,false)
	ScriptManager:hideMarkerAt(x,y)
	
	bunkerIndex = bunkerIndex + 1
	if bunkerIndex > #_bunkerPos then 
		cameraMoveToBunkers() 			-- if index > of index in table continue cutscene (Back to base)
	else
		TeaserBunker()					-- repeat function with increased index
	end
end

function cameraMoveToBunkers()
	ScriptManager:moveCamera(16,17,0.77,3,"endCutScene")
end

--End Intro Cutscene
function endCutScene()
	ScriptManager:stopCutscene()
	if ScriptManager:isControllerConnected() then
		ScriptManager:setControllerCursorPosition( 16, 17 );
	end
	ScriptManager:continueAfterCutscene("setMainInProgress")
end

--	Short mission briefing POST CUTSCENE
function setMainInProgress()
	ScriptManager:setMissionStatus(Missions.MISSION_MAIN_SURVIVE,MissionStatus.IN_PROGRESS)
	ScriptManager:setMissionStatus(Missions.MISSION_MAIN_DEFEAT,MissionStatus.IN_PROGRESS)
	ScriptManager:wait( 3 );
	ScriptManager:displaySubtitleAutoEx(Text.T_MAP_3_3_MISSION_MAIN_SUB, false ); -- TODO: CHECK IF REMOVED CORRECT!!!!!!!!!!!
end

--	MISSION MAIN SURIVIVE

--If base is destroyed set secondary mission to failed and count remaining bases
function BaseDestroyed()
	targetID = nil
	if ScriptManager:getMissionStatus(Missions.MISSION_SECONDARY_PROTECT) == MissionStatus.IN_PROGRESS or ScriptManager:getMissionStatus(Missions.MISSION_SECONDARY_PROTECT) == MissionStatus.NOT_STARTED then
		ScriptManager:setMissionStatus(Missions.MISSION_SECONDARY_PROTECT,MissionStatus.FAILED)
	end
	ScriptManager:setMissionProgress(Missions.MISSION_MAIN_SURVIVE, ScriptManager:getMissionProgress(Missions.MISSION_MAIN_SURVIVE) - 1);
	if ScriptManager:getMissionProgress(Missions.MISSION_MAIN_SURVIVE) == 0 then
		ScriptManager:removeTrigger("BaseDestroyed");
		ScriptManager:runForPlayer(0,"MissionFailedEndWait")
		return
	end
	baseDestroyedCounter()
end

--	Mission failed

function MissionFailedEndWait()
		ScriptManager:blockUI(true)
		ScriptManager:setMissionStatus(Missions.MISSION_MAIN_DEFEAT, MissionStatus.FAILED);
		ScriptManager:setMissionStatus(Missions.MISSION_MAIN_SURVIVE, MissionStatus.FAILED);
		ScriptManager:sleep(3,"missionFailedEnd")
end

--	SHOW MAIN MISSION 1 PROGRESSION

--Count remaining bases
function baseDestroyedCounter()														
	--print(string.format("[SCRIPT]showMainMissionCounter"))
	ScriptManager:preparePopupText();
	ScriptManager:addPopupTextParameter("x",ScriptManager:getMissionProgress(Missions.MISSION_MAIN_SURVIVE))
	ScriptManager:displaySubtitle(Text.T_MAP_3_3_MISSION_MAIN_2_COUNTER)
	ScriptManager:sleep(3,"HideDialogue")
end

--	END MISSION MAIN SURVIVE

--	MISSION MAIN 2 DEFEAT

--Count enemies killed till 70, then set main mission to accomplished
function enemyUnitDestroyed()
	ScriptManager:runForPlayer(0,"_enemyUnitDestroyed")
end

function _enemyUnitDestroyed()
	ScriptManager:setMissionProgress(Missions.MISSION_MAIN_DEFEAT, ScriptManager:getMissionProgress(Missions.MISSION_MAIN_DEFEAT) + 1);
	if ScriptManager:getMissionProgress(Missions.MISSION_MAIN_DEFEAT) == 70 then
		ScriptManager:setMissionStatus(Missions.MISSION_MAIN_DEFEAT, MissionStatus.ACCOMPLISHED);
		ScriptManager:removeTrigger("enemyUnitDestroyed");
		ScriptManager:runForPlayer(0,"missionAccomplishedEndAfterCutscene")
		if ScriptManager:getMissionStatus(Missions.MISSION_SECONDARY_PROTECT) == MissionStatus.IN_PROGRESS or ScriptManager:getMissionStatus(Missions.MISSION_SECONDARY_PROTECT) == MissionStatus.NOT_STARTED then
			print("SECONDARY MISSION ACCOMPLISHED")
			ScriptManager:setMissionStatus(Missions.MISSION_SECONDARY_PROTECT,MissionStatus.ACCOMPLISHED)
		end
		if ScriptManager:getMissionStatus(Missions.MISSION_MAIN_SURVIVE) == MissionStatus.IN_PROGRESS then
			ScriptManager:setMissionStatus(Missions.MISSION_MAIN_SURVIVE,MissionStatus.ACCOMPLISHED)
		end
		return
	end
	if ScriptManager:getMissionStatus(Missions.MISSION_MAIN_DEFEAT) ~= MissionStatus.ACCOMPLISHED then -- Don't show counter when mission progress >= 50
		if ScriptManager:getMissionProgress(Missions.MISSION_MAIN_DEFEAT) <=60 and
			ScriptManager:getMissionProgress(Missions.MISSION_MAIN_DEFEAT) % 5 == 0 then
			ScriptManager:runForPlayer(0,"enemyUnitDestroyedCounter")
		elseif ScriptManager:getMissionProgress(Missions.MISSION_MAIN_DEFEAT) >=60 then
		ScriptManager:runForPlayer(0,"enemyUnitDestroyedCounter")
		end
	end
end

function missionAccomplishedEndAfterCutscene()
	ScriptManager:blockUI(true)
	ScriptManager:sleep(3,"waitforplayerturn") 
end

function waitforplayerturn()
	ScriptManager:runForPlayer(0,"missionAccomplishedEnd")
end

--	SHOW MAIN MISSION 1 PROGRESSION

--Display Counter for 3 seconds
function enemyUnitDestroyedCounter()
	HideDialogue()											
	--print(string.format("[SCRIPT]showMain2MissionCounter"))
	ScriptManager:preparePopupText();
	ScriptManager:addPopupTextParameter("x",ScriptManager:getMissionProgress(Missions.MISSION_MAIN_DEFEAT))
	ScriptManager:displaySubtitle(Text.T_MAP_3_3_MISSION_MAIN_COUNTER)
	ScriptManager:sleep(3,"HideDialogue")
end

--	MISSION SECONDARY Don't loose a single depot

--TODO:Secondary mission

--
--Create Secondary Mission

function onTargetAttacked(attacker, defender)
	targetID = -1
	if attacker:getTemplateId() == Units.UNIT_SU_HQ then				
		targetID = attacker:getId()
	end
	if defender:getTemplateId() == Units.UNIT_SU_HQ then
		targetID = defender:getId()
	end
	ScriptManager:removeTrigger("onTargetAttacked")
	ScriptManager:runForPlayer(0, "MoveCamToTarget")
end

function MoveCamToTarget()
	if targetID ~= nil then
		print("target ID nil?!")
		print(string.format("%i",targetID))
		ScriptManager:blockUI(true)
		ScriptManager:moveCamera(Unit:getUnitById(targetID):getX(),Unit:getUnitById(targetID):getY(),0.9,2,"CamOnTarget")
	end
end

function CamOnTarget()
	ScriptManager:revealFogOfWar(Unit:getUnitById(targetID):getX(),Unit:getUnitById(targetID):getY(),1,true)
	ScriptManager:showMarkerAt(Unit:getUnitById(targetID):getX(),Unit:getUnitById(targetID):getY())
	ScriptManager:displaySubtitle(Text.T_MAP_3_3_MISSION_SECONDARY)
	ScriptManager:sleep(5,"HideDialogue")
	ScriptManager:moveCamera(Unit:getUnitById(targetID):getX(),Unit:getUnitById(targetID):getY(),0.9,3,"ZoomCamOutAndEndScript")
end

function ZoomCamOutAndEndScript()
	ScriptManager:moveCamera(Unit:getUnitById(targetID):getX(),Unit:getUnitById(targetID):getY(),0.8,2,"EndPseudoCutScene")
	ScriptManager:hideMarkerAt(Unit:getUnitById(targetID):getX(),Unit:getUnitById(targetID):getY())
	ScriptManager:revealFogOfWar(Unit:getUnitById(targetID):getX(),Unit:getUnitById(targetID):getY(),1,false)
end

function EndPseudoCutScene()
	ScriptManager:setMissionStatus(Missions.MISSION_SECONDARY_PROTECT,MissionStatus.IN_PROGRESS)
	ScriptManager:blockUI(false)
end

--END

--	ENEMY SPAWN TARGET PRIORITY MANAGER

function getTargetPos(id)
	if id == 1 then
		if Unit:getUnitById(0) ~= nil then
			return basePos[1]
		end
		if Unit:getUnitById(3) ~= nil then
			return basePos[2]
		end
		if Unit:getUnitById(2) ~= nil then
			return basePos[3]
		end
	end
	if id == 2 then
		if Unit:getUnitById(3) ~= nil then
			return basePos[2]
		end
		if Unit:getUnitById(0) ~= nil then
			return basePos[1]
		end
		if Unit:getUnitById(2) ~= nil then
			return basePos[3]
		end
	end
	if id == 3 then
		if Unit:getUnitById(3) ~= nil then
			return basePos[2]
		end
		if Unit:getUnitById(2) ~= nil then
			return basePos[3]
		end
		if Unit:getUnitById(0) ~= nil then
			return basePos[1]
		end
	end
	if id == 4 then
		if Unit:getUnitById(2) ~= nil then
			return basePos[3]
		end
		if Unit:getUnitById(3) ~= nil then
			return basePos[2]
		end
		if Unit:getUnitById(0) ~= nil then
			return basePos[1]
		end
	end
end

--	END

--	SPAWN ENEMYS spawnTimer1
function spawnEnemyBase1()
	ScriptManager:removeTrigger(string.format("spawn1 %i", ScriptManager:getCurrentRound()))
	local wave = ScriptManager:getValue("base1Wave")
	waveSpawner(3, 27, wave, wave, waves1, getTargetPos(1))		--(x, y, squadId, index, mainWave, targetX, targetY)
	wave = wave + 1 --	Increase wave progression
	--Store new progress
	ScriptManager:storeValue("base1Wave",wave)
end

--	SPAWN ENEMYS spawnTimer2
function spawnEnemyBase2()
	ScriptManager:removeTrigger(string.format("spawn2 %i", ScriptManager:getCurrentRound()))
	local wave = ScriptManager:getValue("base2Wave")
	waveSpawner(1, 21, 50+wave, wave, waves2, getTargetPos(2))		--(x, y, squadId, index, mainWave, targetX, targetY)
	wave = wave + 1 --	Increase wave progression
	--Store new progress
	ScriptManager:storeValue("base2Wave",wave)
end

--	SPAWN ENEMYS spawnTimer3
function spawnEnemyBase3()
	ScriptManager:removeTrigger(string.format("spawn3 %i", ScriptManager:getCurrentRound()))
	local wave = ScriptManager:getValue("base3Wave")
	waveSpawner(1, 14, 100+wave, wave, waves3, getTargetPos(3))		--(x, y, squadId, index, mainWave, targetX, targetY)
	wave = wave + 1 --	Increase wave progression
	--Store new wave progress
	ScriptManager:storeValue("base3Wave",wave)
end

--	SPAWN ENEMYS spawnTimer4
function spawnEnemyBase4()
	ScriptManager:removeTrigger(string.format("spawn4 %i", ScriptManager:getCurrentRound()))
	local wave = ScriptManager:getValue("base4Wave")
	local randomPos = spawnPos[math.random(1, #spawnPos)]
	waveSpawner(randomPos[1], randomPos[2], 150+wave, wave, wavesRandom, getTargetPos(randomPos[3]))		--(x, y, squadId, index, mainWave, targetX, targetY)
	wave = wave + 1 --	Increase wave progression
	--Store new progress
	ScriptManager:storeValue("base4Wave",wave)
end

--	SPAWN ENEMYS spawnTimer5
function spawnEnemyBase5()
	ScriptManager:removeTrigger(string.format("spawn5 %i", ScriptManager:getCurrentRound()))
	local wave = ScriptManager:getValue("base5Wave")
	waveSpawner(8, 1, 200+wave, wave, waves4, getTargetPos(4))		--(x, y, squadId, index, mainWave, targetX, targetY)
	wave = wave + 1 --	Increase wave progression
	--Store new progress
	ScriptManager:storeValue("base5Wave",wave)
end

--	WAVE SPAWNER
function waveSpawner(x, y, squadId, index, mainWave, targetPos)
	--check if index is a wholenumber
	local wholeNumber = math.floor(index)
	if wholeNumber ~= index then
		return
	end
	
	--If index reach end of array, use last index again
	if index > #mainWave then
		index = #mainWave
	end

	--Spawn units, AI state squad ID and set target tile
	local wave = mainWave[index]
	for i = 1, #wave do
		--print Debugmessage: Unit with ID x, at x,y, with SuqadId x and Target x,y
		--print(string.format("[SCRIPT SPAWN HERE]unit With ID %i, at %i/%i, with SquadId %i and Target %i/%i", wave[i],x,y,squadId,targetPos[1],targetPos[2]))
		ScriptManager:spawnUnit(1,wave[i],x,y,squadId,AiState.ATTACK_AGGRESSIVE)
		ScriptManager:setTargetTile(1,squadId,targetPos[1],targetPos[2])
	end
end


--	ENDING DIALOGUES

function missionFailedEnd()
	ScriptManager:blockUI(true)
	ScriptManager:disableEndTurnButton()
	ScriptManager:displaySubtitleAutoWithCallback(Text.T_MAP_3_3_DIALOGUE_MISSION_FAILED,"showEndGameScreen") 
end



function missionAccomplishedEnd()
	ScriptManager:blockUI(true)
	ScriptManager:disableEndTurnButton()
	ScriptManager:displaySubtitleAutoWithCallback(Text.T_MAP_3_3_DIALOGUE_MISSION_ACCOMPLISHED,"showEndGameScreen")
end

function showEndGameScreen()
	ScriptManager:sleep(1.5,"EndGameFunction")
end
