require "lua/Constants"
require "lua/Functions"
require "lua/Functions_2"
require "lua/Text"
require "lua/Units"
require "math"


-- mission stuff
missionIds = {
	capOutposts 	= 1,
	capLastOutpost  = 2,
	destroyKV		= 3,
	getAirfield		= 4,
	catchGeneral	= 5,
};

-- missionId, primary( boolean ), textkey, xpReward
missions = {
	{ missionIds.capOutposts, 		true,	Text.T_MAP_DE_3_1_MISSION_PRIMARY_01,	250 },
	{ missionIds.capLastOutpost,	true,	Text.T_MAP_DE_3_1_MISSION_PRIMARY_02,	175 },
	{ missionIds.destroyKV,			false,	Text.T_MAP_DE_3_1_MISSION_SECONDARY_01, 150 },
	{ missionIds.getAirfield,		false,	Text.T_MAP_SU_4_3_MISSION_SECONDARY_01,	100 },
};

enemySquadIds = {
	southeastDepot	=  1,
	southwestDepot	=  2,
	northeastDepot	=  3,
	lastDepot		=  4,
	centerVillage	=  5,
	northForest		=  6,
	centerForest	=  7,
	attackers		=  8,
	airplanes		=  9,
	reinforcements	= 10,
};

southeastDepotIds	= { 14, 15, 16, 17, 18, 19, 20, 21, 22 };
southwestDepotIds	= { 23, 24, 25, 26, 27, 28, 58, 61 };
northeastDepotIds	= { 29, 30, 31, 32, 33, 34, 35, 60 };
centerVillageIds	= { 36, 37, 38, 39 };
northForestIds		= { 40, 41, 42, 43, 44, 45 };
centerForestIds		= { 46, 47, 48 };
attackerIds			= { 49, 50, 51, 52, 53, 54, 55, 59 };
airplaneIds			= { 56, 57 };
outpostIds			= {  0,  1,  2 };

kvsToDestroy	= 3;
lastDepotPos	= { 4, 17 };

lastDepotUnits = {
	--playerId,	templateId,				tileX, tileY,		squadId,			AiState
	{ 1,	Units.UNIT_SU_HQ,				4, 17, enemySquadIds.lastDepot,	AiState.DEFEND },
	{ 1,	Units.UNIT_SU_RIFLE,			3, 17, enemySquadIds.lastDepot,	AiState.DEFEND },
	{ 1,	Units.UNIT_SU_RIFLE,			6, 16, enemySquadIds.lastDepot,	AiState.DEFEND },
	{ 1,	Units.UNIT_SU_RIFLE,			6, 19, enemySquadIds.lastDepot,	AiState.DEFEND },
	{ 1,	Units.UNIT_SU_MORTAR,			5, 19, enemySquadIds.lastDepot,	AiState.DEFEND },
	{ 1,	Units.UNIT_SU_AA,				3, 18, enemySquadIds.lastDepot,	AiState.DEFEND },
	{ 1,	Units.UNIT_SU_AA,				4, 19, enemySquadIds.lastDepot,	AiState.DEFEND },
	{ 1,	Units.UNIT_SU_TANK_T34,			4, 16, enemySquadIds.lastDepot,	AiState.DEFEND },
	{ 1,	Units.UNIT_SU_TANK_KV1,			5, 16, enemySquadIds.lastDepot,	AiState.DEFEND },
	{ 1,	Units.UNIT_SU_TANK_DESTROYER,	5, 18, enemySquadIds.lastDepot,	AiState.DEFEND },
	{ 1,	Units.UNIT_SU_KATYUSHA,			5, 18, enemySquadIds.lastDepot,	AiState.DEFEND },
};

-- BEGIN Init


function onGameStart()
	CreateMissions( missions );
	setTriggers();
	storeValues();
	
	ScriptManager:gainGold( 0, 500 );
	setGuardAreaForSquads();
	
	
	-- start intro cutscene
	ScriptManager:sleep( 0, "startIntroCutscene" ); -- without sleep player turn popup shows
end

function storeValues()
	
end

function setTriggers()
	-- load all needed values after loading
	ScriptManager:addOnGameResumeTrigger( "loadValues", "loadValues", false );
	
	-- check if the player game over
	ScriptManager:addOnUnitHasAttackedTrigger(	1, Units.UNIT_DE_HQ, -1, "ProtectTheSupplyDepotHintSingle",			"ProtectTheSupplyDepotHintSingle",			false );
	ScriptManager:addOnUnitHasAttackedTrigger(	1, Units.UNIT_SU_HQ, -1, "ProtectTheSupplyDepotHintSingleNoDeath",	"ProtectTheSupplyDepotHintSingleNoDeath",	false );
	ScriptManager:addOnUnitDestroyedTrigger(	0, Units.UNIT_DE_HQ, -1, "missionFailedTransition",					"missionFailedTransition",					false );
	
	-- trigger for secondary intro
	ScriptManager:addOnTurnStartTrigger( 0, -1, 1, "prepareSecondaryIntroCutscene", "prepareSecondaryIntroCutscene", false );
	
	-- trigger for depots
	ScriptManager:addOnUnitCapturedTrigger(		0, Units.UNIT_SU_HQ, -1, "onDepotCaptured", "onDepotCaptured",	false );
	ScriptManager:addOnUnitHasAttackedTrigger(	0, Units.UNIT_SU_HQ, -1, "onDepotAttacked", "onDepotAttacked",	false );
	ScriptManager:addOnUnitDestroyedTrigger(	1, Units.UNIT_SU_HQ, -1, "missionFailed",	"missionFailed",	false );
	
	-- trigger for airfield
	ScriptManager:addOnUnitCapturedTrigger(	 0, Units.UNIT_SU_AIRFIELD, -1, "onAirfieldUnderControl", "onAirfieldUnderControl", false );
	ScriptManager:addOnUnitDestroyedTrigger( 1, Units.UNIT_SU_AIRFIELD, -1, "onAirfieldUnderControl", "onAirfieldUnderControl", false );
	
	-- trigger for tiger
	ScriptManager:addOnUnitSightedTrigger(	 1, Units.UNIT_SU_TANK_KV1, -1, true, "onKvSeen", "onKvSeen", false );
	ScriptManager:addOnUnitDestroyedTrigger( 1, Units.UNIT_SU_TANK_KV1, -1, "onKvDestroyed", "onKvDestroyed", false );
	
	-- set guard area again and again and again... cause reasons
	ScriptManager:addOnTurnEndTrigger( 0, -1, -1, -1, "setGuardAreaForSquads", "setGuardAreaForSquads" );
	
	-- trigger for airfield message, that no planes can be bought on this map
	ScriptManager:addOnUnitSelectedTrigger( 0, Units.UNIT_SU_AIRFIELD, -1, "onAirfieldSelected", "onAirfieldSelected", false );
end

function setGuardAreaForSquads()
	ScriptManager:setGuardArea( 1, enemySquadIds.northForest, 10, 17, 2 );
end

function loadValues()
	
end


-- END Init
-- BEGIN Intro Cutscene


function startIntroCutscene()
	StartCutscene();
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE_3_1_MISSION_INTRO_FLAVOR_01 );
	ScriptManager:wait( 4 );
	HideDialogue();
	ScriptManager:wait( 1 );
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE_3_1_MISSION_INTRO_FLAVOR_02 );
	ScriptManager:wait( 1 );
	ScriptManager:moveCameraEx( 10, 11, 0.4, 3, true );
	ScriptManager:wait( 2 );
	HideDialogue();
	
	ScriptManager:moveCameraEx( 3, 8, 0.7, 3, false );
	ScriptManager:wait( 2 );
	ScriptManager:displaySubtitle( Text.T_MAP_DE_3_1_MISSION_INTRO_FLAVOR_03 );
	ScriptManager:wait( 1 );
	
	ScriptManager:revealFogOfWar( 3, 8, 1, true );
	ScriptManager:showMarkerAt( 3, 8 );
	ScriptManager:wait( 2 );
	ScriptManager:removeAllMarkers();
	ScriptManager:revealFogOfWar( 3, 8, 1, false );
	
	ScriptManager:moveCameraEx( 16, 3, 0.7, 4, true );
	HideDialogue();
	
	ScriptManager:revealFogOfWar( 16, 3, 1, true );
	ScriptManager:showMarkerAt( 16, 3 );
	ScriptManager:wait( 2 );
	ScriptManager:removeAllMarkers();
	ScriptManager:revealFogOfWar( 16, 3, 1, false );
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE_3_1_MISSION_INTRO_FLAVOR_04 );
	ScriptManager:moveCameraEx( 16, 17, 0.7, 4, true );
	
	ScriptManager:revealFogOfWar( 16, 17, 1, true );
	ScriptManager:showMarkerAt( 16, 17 );
	ScriptManager:wait( 2 );
	ScriptManager:removeAllMarkers();
	ScriptManager:revealFogOfWar( 16, 17, 1, false );
	HideDialogue();
	
	ScriptManager:moveCameraEx( 9, 2, 0.7, 5, true );
	
	EndCutscene();
	
	startTheMission();
end

function startTheMission()
	ScriptManager:setMissionStatus(		missionIds.capOutposts, MissionStatus.IN_PROGRESS );
	ScriptManager:setMissionProgress(	missionIds.capOutposts, 0 );
	
	--TODO: remove this function!
	--testing();
	--removeSquads();
	--ScriptManager:addOnUnitSelectedTrigger( 1, -1, -1, "showParameters", "showParameters",false)
end

function testing()
	ScriptManager:spawnUnit( 0, Units.GOTT,  1,  1, 42, AiState.WAIT );
	ScriptManager:spawnUnit( 0, Units.GOTT, 15,  1, 42, AiState.WAIT );
	ScriptManager:spawnUnit( 0, Units.GOTT, 19, 21, 42, AiState.WAIT );
	ScriptManager:spawnUnit( 0, Units.GOTT,  6, 17, 42, AiState.WAIT );
end

function removeSquads()
	for i = 1, #southeastDepotIds do
		ScriptManager:removeUnitEx( southeastDepotIds[i], false );
	end	
	for i = 1, #southwestDepotIds do
		ScriptManager:removeUnitEx( southwestDepotIds[i], false );
	end	
	for i = 1, #northeastDepotIds do
		ScriptManager:removeUnitEx( northeastDepotIds[i], false );
	end	
	for i = 1, #centerVillageIds do
		ScriptManager:removeUnitEx( centerVillageIds[i], false );
	end	
	for i = 1, #northForestIds do
		ScriptManager:removeUnitEx( northForestIds[i], false );
	end	
	for i = 1, #centerForestIds do
		ScriptManager:removeUnitEx( centerForestIds[i], false );
	end	
	for i = 1, #attackerIds do
		ScriptManager:removeUnitEx( attackerIds[i], false );
	end	
	for i = 1, #airplaneIds do
		ScriptManager:removeUnitEx( airplaneIds[i], false );
	end		
end

function showParameters( selectedUnit )
	--print( string.format( "[] %s", selectedUnit:get() ) );
end

function prepareSecondaryIntroCutscene()
	table.insert( cutsceneQueue, startSecondaryIntroCutscene );
	CutsceneQueue();
end

function startSecondaryIntroCutscene()
	StartCutscene();
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE_3_1_MISSION_SECONDARY_02_FLAVOR_01 );
	ScriptManager:moveCameraEx( 18, 6, 0.7, 4, true );
	ScriptManager:wait( 2 );
	HideDialogue();
	ScriptManager:wait( 1 );
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE_3_1_MISSION_SECONDARY_02_FLAVOR_02 );
	ScriptManager:revealFogOfWar( 18, 6, 1, true );
	ScriptManager:showMarkerAt( 18, 6 );
	ScriptManager:wait( 4 );
	ScriptManager:removeAllMarkers();
	ScriptManager:revealFogOfWar( 18, 6, 1, false );
	ScriptManager:wait( 1 );
	HideDialogue();
	
	EndCutscene();
	DisablePlayerInteraction();
	
	ScriptManager:setMissionStatus( missionIds.getAirfield, MissionStatus.IN_PROGRESS );
	ScriptManager:removeTrigger( "startSecondaryIntroCutscene" );
	Unit:changeSquadState( 1, enemySquadIds.airplanes, AiState.AIRPLANE ); -- let the airplanes attack now
	ScriptManager:addOnTurnStartTrigger( 1, -1, ScriptManager:getCurrentRound() +  6, "spawnAdditionalPlanes", "spawnAdditionalPlanes", false );
	ScriptManager:addOnTurnStartTrigger( 1, -1, ScriptManager:getCurrentRound() + 11, "spawnAdditionalPlanes", "spawnAdditionalPlanes", false );
	ScriptManager:wait( 2 );
	
	EnablePlayerInteraction();
end


-- END Intro Cutscene
-- BEGIN Building Stuff


function onDepotCaptured()
	if ScriptManager:getMissionStatus( missionIds.capOutposts ) == MissionStatus.IN_PROGRESS then
		ScriptManager:setMissionProgress( missionIds.capOutposts, ScriptManager:getMissionProgress( missionIds.capOutposts ) + 1 );
		
		if ScriptManager:getMissionProgress( missionIds.capOutposts ) >= #outpostIds then
			DisablePlayerInteraction();
		
			ScriptManager:setMissionStatus( missionIds.capOutposts, MissionStatus.ACCOMPLISHED );
			ScriptManager:wait( 2 );
			
			EnablePlayerInteraction();
			startDepotsCapturedCutscene();
		
		else
			ShowMissionProgressPopup( ScriptManager:getMissionProgress( missionIds.capOutposts ), Text.T_MAP_DE_3_1_MISSION_PRIMARY_COUNTER );
		end
	
	elseif ScriptManager:getMissionStatus( missionIds.capLastOutpost ) == MissionStatus.IN_PROGRESS then
		DisablePlayerInteraction();
		
		ScriptManager:setMissionStatus( missionIds.capLastOutpost, MissionStatus.ACCOMPLISHED );
		ScriptManager:wait( 2 );
		
		startVictoryCutscene();
	end
end

function startDepotsCapturedCutscene()
	StartCutscene();
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE_3_1_MISSION_PRIMARY_FLAVOR_01 );	
	local depot = spawnUnitFromTable( lastDepotUnits[1] ); -- depot
	local depotX = depot:getX();
	local depotY = depot:getY();
	ScriptManager:wait( 5 );
	HideDialogue();
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE_3_1_MISSION_PRIMARY_FLAVOR_02 );
	ScriptManager:wait( 4 );
	HideDialogue();
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE_3_1_MISSION_PRIMARY_FLAVOR_03 );
	ScriptManager:wait( 1 );
	ScriptManager:moveCameraEx( depotX, depotY, 0.7, 5, true );
	
	ScriptManager:revealFogOfWar( depotX, depotY, 1, true );
	ScriptManager:showMarkerAt( depotX, depotY );
	ScriptManager:wait( 2 );
	ScriptManager:removeAllMarkers();
	ScriptManager:revealFogOfWar( depotX, depotY, 1, false );
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE_3_1_MISSION_PRIMARY_FLAVOR_04 );
	ScriptManager:wait( 4 );
	HideDialogue();
	
	EndCutscene();
	DisablePlayerInteraction();
	
	ScriptManager:setMissionStatus( missionIds.capLastOutpost, MissionStatus.IN_PROGRESS );
	for i = 2, #lastDepotUnits do
		spawnUnitFromTable( lastDepotUnits[i] );
	end
	--ScriptManager:addOnTurnStartTrigger( 1, -1, -1, "spawnDefenders", "spawnDefenders", false );
	ScriptManager:wait( 2 );
	
	EnablePlayerInteraction();
end

function spawnDefenders()
	for i = 2, #lastDepotUnits do
		spawnUnitFromTable( lastDepotUnits[i] );
	end
	ScriptManager:removeTrigger( "spawnDefenders" );
end

function spawnUnitFromTable( spawnData )
	local unit = ScriptManager:spawnUnit( spawnData[1], spawnData[2], spawnData[3],spawnData[4], spawnData[5], spawnData[6] );
	return unit;
end

function onDepotAttacked( activeUnit, passiveUnit )
	DisablePlayerInteraction();
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE_3_1_MISSION_PRIMARY_WARNING );
	ScriptManager:moveCameraEx( passiveUnit:getX(), passiveUnit:getY(), 1.0, 1, true );
	ScriptManager:showMarkerAt( passiveUnit:getX(), passiveUnit:getY() );
	ScriptManager:wait( 2 );
	ScriptManager:removeAllMarkers();
	
	EnablePlayerInteraction();
end

function onAirfieldUnderControl()
	DisablePlayerInteraction();
	
	ScriptManager:setMissionStatus( missionIds.getAirfield, MissionStatus.ACCOMPLISHED );
	ScriptManager:removeTrigger( "onAirfieldUnderControl" );
	ScriptManager:removeTrigger( "spawnAdditionalPlanes" );
	ScriptManager:wait( 2 );
	
	EnablePlayerInteraction();
end

function onAirfieldSelected()
	ScriptManager:displaySubtitleAutoEx( Text.T_MAP_DE_3_1_MISSION_SECONDARY_02_FLAVOR_03, false );
end


-- END Building Stuff
-- BEGIN Enemy Stuff

function spawnAdditionalPlanes()
	-- spawn two more fighter for the enemy ( no more after that )
	ScriptManager:spawnUnit( 1, Units.UNIT_SU_FIGHTER, 18, 5, enemySquadIds.airplanes, AiState.AIRPLANE );
	ScriptManager:spawnUnit( 1, Units.UNIT_SU_FIGHTER, 19, 6, enemySquadIds.airplanes, AiState.AIRPLANE );
end


function onKvSeen( seeingUnit, seenUnit )
	ScriptManager:storeValue( "kvId", seenUnit:getId() );

	ScriptManager:runForPlayer( 0, "prepareKvCutscene" );
	ScriptManager:removeTrigger( "onKvSeen" );
end

function prepareKvCutscene()
	table.insert( cutsceneQueue, startKvCutscene );
	CutsceneQueue();
end

function startKvCutscene()
	StartCutscene();
	
	local kvX, kvY;
	kvX = Unit:getUnitById( ScriptManager:getValue( "kvId" ) ):getX();
	kvY = Unit:getUnitById( ScriptManager:getValue( "kvId" ) ):getY();
	
	ScriptManager:moveCameraEx( kvX, kvY, 0.7, 2, true );
	ScriptManager:displaySubtitle( Text.T_MAP_DE_3_1_MISSION_SECONDARY_01_FLAVOR_01 );
	ScriptManager:revealFogOfWar( kvX, kvY, 1, true );
	ScriptManager:showMarkerAt( kvX, kvY );
	ScriptManager:wait( 3 );
	ScriptManager:removeAllMarkers();
	ScriptManager:revealFogOfWar( kvX, kvY, 1, false );
	ScriptManager:wait( 2 );
	HideDialogue();
	
	EndCutscene();
	DisablePlayerInteraction();
	
	ScriptManager:setMissionStatus(		missionIds.destroyKV, MissionStatus.IN_PROGRESS );
	ScriptManager:setMissionProgress(	missionIds.destroyKV, 0 );
	ScriptManager:deleteValue( "kvId" );
	
	EnablePlayerInteraction();
end

function onKvDestroyed()
	ScriptManager:setMissionProgress( missionIds.destroyKV, ScriptManager:getMissionProgress( missionIds.destroyKV ) + 1 );
	
	if ScriptManager:getMissionProgress( missionIds.destroyKV ) >= kvsToDestroy then
		DisablePlayerInteraction();
		
		ScriptManager:setMissionStatus( missionIds.destroyKV, MissionStatus.ACCOMPLISHED );
		ScriptManager:removeTrigger( "onKvDestroyed" );
		ScriptManager:wait( 2 );
		
		EnablePlayerInteraction();
	
	else
		ShowMissionProgressPopup( ScriptManager:getMissionProgress( missionIds.destroyKV ), Text.T_MAP_DE_3_1_MISSION_SECONDARY_01_COUNTER );
	end
end


-- END Enemy Stuff
-- BEGIN Victory


function startVictoryCutscene()
	StartCutscene();
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE_3_1_MISSION_VICTORY_FLAVOR_01 );
	ScriptManager:wait( 4 );
	HideDialogue();
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE_3_1_MISSION_VICTORY_FLAVOR_02 );
	ScriptManager:wait( 4 );
	HideDialogue();
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE_3_1_MISSION_VICTORY_FLAVOR_03 );
	ScriptManager:wait( 4 );
	HideDialogue();
	
	EndCutscene();
	
	victoryEndGame();
end

function victoryEndGame()
	DisablePlayerInteraction();
	HideDialogue(); -- remove possible texts on the screens

	-- accomplish secondary missions that are accomplished when the game is finished with certain condition
	
	
	ScriptManager:displaySubtitle( Text.T_MISSION_VICTORY );
	ScriptManager:wait( 4 );
	HideDialogue();
	endGame();
end


-- END Victory
-- BEGIN Defeated


function missionFailedTransition()
	-- fail mission in player turn
	ScriptManager:runForPlayer( 0, "missionFailed" );
end

function missionFailed()
	DisablePlayerInteraction();
	HideDialogue(); -- remove possible texts on the screens
	
	ScriptManager:displaySubtitle( Text.T_MISSION_FAILED );
	ScriptManager:wait( 4 );
	HideDialogue();
	endGame();
end


-- END Defeated


function endGame()
	ScriptManager:endGame()
end