require "lua/Constants"
require "lua/Functions"
require "lua/Functions_2"
require "lua/Text"
require "lua/Units"
require "math"


-- mission stuff
missionIds = {
	chaseHim 	= 1,
	reachEnd	= 2,
	barricades  = 3,
	capDepots	= 4,
};

-- missionId, primary( boolean ), textkey, xpReward
missions = {
	{ missionIds.chaseHim,		true,	Text.T_MAP_DE_3_2_MISSION_PRIMARY1,			150 },
	{ missionIds.reachEnd,		true,	Text.T_MAP_DE_3_2_MISSION_PRIMARY2,			250 },
	{ missionIds.barricades,	false,	Text.T_MAP_DE_3_2_MISSION_SECONDARY1,		125 },
	{ missionIds.capDepots,		false,	Text.T_MISSION_MAP_2_1_INTRO_MISSION_MAIN,	125 },
};

enemySquadIds = {
	southDepot			=  1,
	eastDepot			=  2,
	centerBarricade		=  3,
	eastBarricade		=  4,
	firstAttackers		=  5,
	northAttackers		=  6,
	bridgeAttackers		=  7,
	lakeAttackers		=  8,
	eastForestGuard		=  9,
	centerBarricade2	= 10, -- needed for separate guard area
};

southDepotIds		= { 27, 28, 29, 30, 31, 32 };
eastDepotIds		= { 40, 41, 42, 43, 44, 45 };
centerBarricadeIds	= {  3,  4,  5,  6,  7, 20, 21, 61, 65 };
eastBarricadeIds	= {  8,  9, 10, 11, 12, 13, 14, 66, 67 };
firstAttackerIds	= { 15, 22, 23, 24, 25, 26, 63, 64 };
northAttackerIds	= { 16, 33, 34, 35 };
bridgeAttackerIds	= { 17, 46, 47, 48, 49 };
lakeAttackerIds		= { 19, 36, 37, 38, 39 };
eastForestGuard		= { 18, 50, 51, 52 };
introTransporterId	= 62;

playerUnits			= {};
playerUnitIdsStart	=  53; -- start of the player unitIds in editor
playerUnitIdsEnd	=  60; -- the end of them
maxId				= 150; -- last id searched when loading the playerunits table ( search range: playerUnitIdsStart - maxId )

unitsInGoal			= {};
unitsNeededToWin	= 5;

centerBarricadeUnitsDestroyed	= 0;
eastBarricadeUnitsDestroyed		= 0;
barricadesInBlock				= 2;

depotsToCap			= 2;
barricadesToShatter	= 2;


-- BEGIN Init


function onGameStart()
	CreateMissions( missions );
	addPlayerUnits();
	camouflageSquads();	
	setTriggers();
	storeValues();
	setGuardAreaForSquads()
	
	ScriptManager:gainGold( 0, 1500 );
	
	-- start intro cutscene
	ScriptManager:sleep( 0, "startIntroCutscene" ); -- without sleep player turn popup shows
end

function addPlayerUnits()
	for i = playerUnitIdsStart, playerUnitIdsEnd do
		table.insert( playerUnits, i );
		--print( string.format( "playerUnit %i added to table", i ) )
	end
end

function camouflageSquads()
	-- only some have stealth @ beginning
	for i = 1, #firstAttackerIds do
		ScriptManager:camouflageUnit( firstAttackerIds[i] );
	end
end

function storeValues()
	for i = 1, #playerUnits do
		ScriptManager:storeValue( string.format( "playerUnit %i", playerUnits[i] ) , playerUnits[i] );
	end
	
	ScriptManager:storeValue( "eastBarricadeUnitsDestroyed",	eastBarricadeUnitsDestroyed );
	ScriptManager:storeValue( "centerBarricadeUnitsDestroyed",	centerBarricadeUnitsDestroyed );
end

function setTriggers()
	local x, y;
	
	-- load all needed values after loading
	ScriptManager:addOnGameResumeTrigger( "loadValues", "loadValues", false );
	
	-- set guard area again and again and again... cause reasons
	ScriptManager:addOnTurnEndTrigger( 0, -1, -1, -1, "setGuardAreaForSquads", "setGuardAreaForSquads" );
	
	-- check if the player has units left
	ScriptManager:addOnUnitDestroyedTrigger( 0, -1, -1, "checkForEnd", "checkForEnd" , false );
	
	-- main mission trigger
	ScriptManager:addOnUnitPurchasedTrigger( 0, "onUnitBought", "onUnitBought", false );
	ScriptManager:addOnUnitDestroyedTrigger( 0, -1, -1, "onPlayerUnitDestroyed", "onPlayerUnitDestroyed", false );
	
	x = 29;
	for y = 13, 19 do
		for i = x, 31 do
			ScriptManager:addOnEnterTileTrigger( i, y, 0, -1, -1, false, "onGoalReached", "onGoalReached", false );
			--print( string.format( "%i / %i", i, y ) );
		end
		
		if y % 2 == 0 then
			x = x - 1;
		end
	end
	
	
	-- trigger for end chase mission
	for y = 13, 19 do
		ScriptManager:addOnEnterTileTrigger( 13, y, 0, -1, -1, true, "chaseFinished", "chaseFinished", false );
	end
	
	-- barricade trigger
	ScriptManager:addOnUnitHasAttackedTrigger(	1, -1, -1, "onEnemyHasAttacked", "onEnemyHasAttacked",	false );
	ScriptManager:addOnUnitDestroyedTrigger(	1, -1, -1, "onEnemyDestroyed",	 "onEnemyDestroyed",	false );
	
	-- depot trigger	
	x = 7;
	for y = 1, 11 do
		ScriptManager:addOnEnterTileTrigger( x, y, 0, -1, -1, true, "nearDepot", "nearDepot", false );
	end
	y = 11;
	for x = 8, 12 do
		ScriptManager:addOnEnterTileTrigger( x, y, 0, -1, -1, true, "nearDepot", "nearDepot", false );
	end
	
	ScriptManager:addOnUnitCapturedTrigger( 0, Units.UNIT_SU_HQ, -1, "onDepotCaptured", "onDepotCaptured", false );
end

function loadValues()
	for i = playerUnitIdsStart, maxId do
		local value = ScriptManager:getValue( string.format( "playerUnit %i", i ) );
		
		-- if unit was found put it back into the table
		if value ~= -1 then
			table.insert( playerUnits, i );
		end
	end
	
	for i = 1, unitsNeededToWin do
		local value = ScriptManager:getValue( string.format( "goalUnits %i" , i ) );
		--print( value );
		-- if unit was found put it back into the table
		if value ~= -1 then
			--print( "loaded" );
			table.insert( unitsInGoal, value );
		end
	end
	
	eastBarricadeUnitsDestroyed		= ScriptManager:getValue( "eastBarricadeUnitsDestroyed" );
	centerBarricadeUnitsDestroyed	= ScriptManager:getValue( "centerBarricadeUnitsDestroyed" );
end

function setGuardAreaForSquads()
	-- set guard areas for some squads
	ScriptManager:setGuardArea( 1, enemySquadIds.centerBarricade,	 9, 13, 2 );
	ScriptManager:setGuardArea( 1, enemySquadIds.centerBarricade2,	12, 12, 2 );
	ScriptManager:setGuardArea( 1, enemySquadIds.bridgeAttackers,	23,  3, 2 );
	ScriptManager:setGuardArea( 1, enemySquadIds.eastBarricade,		27, 11, 2 );
	ScriptManager:setGuardArea( 1, enemySquadIds.eastForestGuard,	30,  7, 2 );
end


-- END Init
-- BEGIN Intro Cutscene


function startIntroCutscene()
	StartCutscene();
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE_3_2_MISSION_INTRO_FLAVOR_01 );
	ScriptManager:wait( 4 );
	HideDialogue();
	ScriptManager:wait( 1 );
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE_3_2_MISSION_INTRO_FLAVOR_02 );
	ScriptManager:moveCameraEx( 14, 16, 0.7, 4, true );
	HideDialogue();
	ScriptManager:wait( 1 );
	
	ScriptManager:revealFogOfWar( 14, 16, 2, true );
	ScriptManager:displaySubtitle( Text.T_MAP_DE_3_2_MISSION_INTRO_FLAVOR_03 );
	ScriptManager:moveUnitEx( introTransporterId, 16, 15, true );
	ScriptManager:wait( 2 );
	ScriptManager:removeUnitEx( introTransporterId, true );
	HideDialogue();
	ScriptManager:revealFogOfWar( 14, 16, 2, false );
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE_3_2_MISSION_INTRO_FLAVOR_04 );
	ScriptManager:moveCameraEx( 3, 16, 0.7, 5, true );
	ScriptManager:wait( 1 );
	HideDialogue();
	
	EndCutscene();
	
	startTheMission();
end

function startTheMission()
	ScriptManager:setMissionStatus( missionIds.chaseHim,	MissionStatus.IN_PROGRESS );
	ScriptManager:setMissionStatus( missionIds.barricades,	MissionStatus.IN_PROGRESS );
	
	ScriptManager:setMissionProgress( missionIds.barricades, 0 );
	
	--TODO: remove this function!
	--testing();
	--removeSquads();
	--ScriptManager:addOnUnitSelectedTrigger( 1, -1, -1, "showParameters", "showParameters",false)
end

function testing()
	ScriptManager:spawnUnit( 0, Units.GOTT, 23, 17, 42, AiState.WAIT );
	ScriptManager:spawnUnit( 0, Units.GOTT, 24, 19, 42, AiState.WAIT );
end

function removeSquads()
	for i = 1, #southDepotIds do
		ScriptManager:removeUnitEx( southDepotIds[i], false );
	end
	for i = 1, #eastDepotIds do
		ScriptManager:removeUnitEx( eastDepotIds[i], false );
	end
	for i = 1, #centerBarricadeIds do
		ScriptManager:removeUnitEx( centerBarricadeIds[i], false );
	end
	for i = 1, #eastBarricadeIds do
		ScriptManager:removeUnitEx( eastBarricadeIds[i], false );
	end
	for i = 1, #firstAttackerIds do
		ScriptManager:removeUnitEx( firstAttackerIds[i], false );
	end
	for i = 1, #northAttackerIds do
		ScriptManager:removeUnitEx( northAttackerIds[i], false );
	end
	for i = 1, #bridgeAttackerIds do
		ScriptManager:removeUnitEx( bridgeAttackerIds[i], false );
	end
	for i = 1, #lakeAttackerIds do
		ScriptManager:removeUnitEx( lakeAttackerIds[i], false );
	end
	for i = 1, #eastForestGuard do
		ScriptManager:removeUnitEx( eastForestGuard[i], false );
	end
	for i = 1, #introTransporterId do
		ScriptManager:removeUnitEx( introTransporterId[i], false );
	end
end

function showParameters( selectedUnit )
	--print( string.format( "[] %s", selectedUnit:get() ) );
end


-- END Intro Cutscene
-- BEGIN Main Mission Stuff


function chaseFinished( tileX, tileY )
	DisablePlayerInteraction();
	
	ScriptManager:setMissionStatus( missionIds.chaseHim, MissionStatus.ACCOMPLISHED );
	ScriptManager:wait( 2 );
	
	EnablePlayerInteraction();
	StartCutscene()
	
	ScriptManager:moveCameraEx( 29, 16, 0.7, 5, true );
	ScriptManager:revealFogOfWar( 29, 16, 2, true );
	local unit = ScriptManager:spawnUnit( 1, Units.UNIT_SU_TRANSPORT_TRUCK, 26, 13, 10, AiState.RETREAT );
	ScriptManager:moveUnitEx( unit:getId(), 31, 17, false );
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE_3_2_MISSION_PRIMARY1_FLAVOR_01 );
	ScriptManager:wait( 4 );
	ScriptManager:revealFogOfWar( 29, 16, 2, false );
	HideDialogue();
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE_3_2_MISSION_PRIMARY1_FLAVOR_02 );
	ScriptManager:moveCameraEx( tileX, tileY, 0.7, 5, true );
	ScriptManager:removeUnitEx( unit:getId(), false );
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE_3_2_MISSION_PRIMARY1_FLAVOR_03 );
	ScriptManager:wait( 5 );
	HideDialogue();
	
	EndCutscene();
	DisablePlayerInteraction();
	
	ScriptManager:setMissionStatus( missionIds.reachEnd, MissionStatus.IN_PROGRESS );
	ScriptManager:removeTrigger( "chaseFinished" );
	ScriptManager:wait( 2 );
	
	EnablePlayerInteraction();
end

function onUnitBought( unit )
	local unitId = unit:getId();
	
	-- add the bought unit to the table with the player units and save it
	table.insert( playerUnits, unitId );
	ScriptManager:storeValue( string.format( "playerUnit %i", unitId ) , unitId );
	--print( string.format( "playerUnit %i added to table", unitId ) )
end

function onPlayerUnitDestroyed( unit )
	local unitId = unit:getId();
	
	-- remove destroyed units from the table with the player units and delete it from the save file
	for i = 1, #playerUnits do
		if unitId == playerUnits[i] then
			table.remove( playerUnits, i );
			ScriptManager:deleteValue( string.format( "playerUnit %i", unitId ) );
			--print( string.format( "playerUnit %i removed from table", unitId ) )
			break;
		end	
	end
	
	-- check for victory
	if #playerUnits < 5 and #playerUnits <= #unitsInGoal then
		startVictoryCutscene();
	end
end

function onGoalReached( tileX, tileY, playerId, unitTemplateId, unitId )
	local alreadyIn = false;
	--print( string.format( "unit %i reached goal", unitId ) );
	
	-- check if unit
	if #unitsInGoal >= 1 then
		for i = 1, #unitsInGoal do
			if unitsInGoal[i] == unitId then
				alreadyIn = true;
				--print( "unit was already in" );
				break;
			end
		end
	end
	
	if alreadyIn == false then
		table.insert( unitsInGoal, unitId );
		--print( #unitsInGoal );
		ScriptManager:storeValue( string.format( "goalUnits %i", #unitsInGoal ), unitId );
		--print( "unit added to goalUnits" );
		checkForVictory();
	end
end

function checkForVictory()
	-- check how many player units are alive and ( if necessary ) adapt the goal
	local neededToWin = unitsNeededToWin;
	
	if #playerUnits < 5 then
		--print( "< 5 units needed to win" );
		neededToWin = #playerUnits;
	end
	
	-- check for victory
	if #unitsInGoal >= neededToWin then
		startVictoryCutscene();
		
	else
		--print( "popup incoming!" );
		ScriptManager:preparePopupText();
		ScriptManager:addPopupTextParameter( "y", neededToWin );
		ScriptManager:addPopupTextParameter( "x", #unitsInGoal );
		ScriptManager:displaySubtitleAutoEx( Text.T_MAP_DE_3_2_MISSION_PRIMARY_COUNTER, false );
	end
end


-- END Main Mission Stuff
-- BEGIN Enemy Stuff


function nearDepot( tileX, tileY )
	StartCutscene();
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE_3_2_MISSION_SECONDARY2_FLAVOR_01 );
	ScriptManager:moveCameraEx( 9, 7, 0.7, 2, true );
	ScriptManager:revealFogOfWar( 9, 7, 1, true );
	ScriptManager:showMarkerAt( 9, 7 );
	ScriptManager:wait( 2 );
	ScriptManager:removeAllMarkers();
	ScriptManager:revealFogOfWar( 9, 7, 1, false );
	HideDialogue();
	ScriptManager:wait( 1 );
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE_3_2_MISSION_SECONDARY2_FLAVOR_02 );
	ScriptManager:wait( 5 );
	HideDialogue();
	
	EndCutscene();
	DisablePlayerInteraction();
	
	ScriptManager:setMissionStatus(		missionIds.capDepots, MissionStatus.IN_PROGRESS );
	ScriptManager:setMissionProgress(	missionIds.capDepots, 0 );
	ScriptManager:removeTrigger( "nearDepot" );
	ScriptManager:wait( 2 );
	
	EnablePlayerInteraction();
end

function onDepotCaptured()
	ScriptManager:setMissionProgress( missionIds.capDepots, ScriptManager:getMissionProgress( missionIds.capDepots ) + 1 );
	
	if ScriptManager:getMissionProgress( missionIds.capDepots ) >= depotsToCap then
		DisablePlayerInteraction();
		
		ScriptManager:setMissionStatus( missionIds.capDepots, MissionStatus.ACCOMPLISHED );
		ScriptManager:removeTrigger( "onDepotCaptured" );
		ScriptManager:wait( 2 );
		
		EnablePlayerInteraction();
		ScriptManager:displaySubtitleAutoEx( Text.T_MISSION_MAP_1_3_INTRO_MISSION_OUTRO, false );
	
	else
		ShowMissionProgressPopup( ScriptManager:getMissionProgress( missionIds.capDepots ), Text.T_MAP_DE_3_2_MISSION_SECONDARY2_COUNTER );
	end
end

function onEnemyHasAttacked( activeUnit, passiveUnit )
	local squadId;
	
	-- find enemy unit
	if activeUnit:getPlayerId() == 1 then
		squadId = activeUnit:getSquadId();
		
	else
		squadId = passiveUnit:getSquadId();
	end
	
	-- if one part of the squad is attacked, make the rest aggressive to prevent easy fight for the player
	if squadId == enemySquadIds.centerBarricade or squadId == enemySquadIds.centerBarricade2 then
		Unit:changeSquadState( 1, enemySquadIds.centerBarricade,	AiState.ATTACK_AGGRESSIVE );
		Unit:changeSquadState( 1, enemySquadIds.centerBarricade2,	AiState.ATTACK_AGGRESSIVE );
		
	elseif squadId == enemySquadIds.eastBarricade or squadId == enemySquadIds.eastForestGuard then
		Unit:changeSquadState( 1, enemySquadIds.eastBarricade,		AiState.ATTACK_AGGRESSIVE );
		Unit:changeSquadState( 1, enemySquadIds.eastForestGuard,	AiState.ATTACK_AGGRESSIVE );
	end
end

function onEnemyDestroyed( unit )
	local squadId		= unit:getSquadId();
	local templateId	= unit:getTemplateId();
	
	if ( squadId == enemySquadIds.centerBarricade or squadId == enemySquadIds.centerBarricade2 ) and
		templateId == Units.UNIT_SU_BARRICADE then
		centerBarricadeUnitsDestroyed = barricadeMissionUpdate( centerBarricadeUnitsDestroyed );
		ScriptManager:storeValue( "centerBarricadeUnitsDestroyed", centerBarricadeUnitsDestroyed );
	
	elseif squadId == enemySquadIds.eastBarricade and templateId == Units.UNIT_SU_BARRICADE then
		eastBarricadeUnitsDestroyed = barricadeMissionUpdate( eastBarricadeUnitsDestroyed );
		ScriptManager:storeValue( "eastBarricadeUnitsDestroyed", eastBarricadeUnitsDestroyed );
	end
end

function barricadeMissionUpdate( barricadeUnitsDestroyed )
	barricadeUnitsDestroyed = barricadeUnitsDestroyed + 1;
	
	if barricadeUnitsDestroyed >= barricadesInBlock then
		ScriptManager:setMissionProgress( missionIds.barricades, ScriptManager:getMissionProgress( missionIds.barricades ) + 1 );
		
		if ScriptManager:getMissionProgress( missionIds.barricades ) >= barricadesToShatter then
			DisablePlayerInteraction()
			
			ScriptManager:setMissionStatus( missionIds.barricades, MissionStatus.ACCOMPLISHED );
			ScriptManager:wait( 2 );
			
			EnablePlayerInteraction();
		
		else
			ShowMissionProgressPopup( ScriptManager:getMissionProgress( missionIds.barricades ), Text.T_MAP_DE_3_2_MISSION_SECONDARY1_COUNTER );
		end
	end
	
	return barricadeUnitsDestroyed;
end


-- END Enemy Stuff
-- BEGIN Victory


function startVictoryCutscene()
	DisablePlayerInteraction();
	
	ScriptManager:setMissionStatus( missionIds.reachEnd, MissionStatus.ACCOMPLISHED );
	ScriptManager:wait( 2 );
	
	EnablePlayerInteraction();
	StartCutscene();
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE_3_2_MISSION_VICTORY_FLAVOR_01 );
	ScriptManager:wait( 4 );
	HideDialogue();
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE_3_2_MISSION_VICTORY_FLAVOR_02 );
	ScriptManager:wait( 4 );
	HideDialogue();
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE_3_2_MISSION_VICTORY_FLAVOR_03 );
	ScriptManager:wait( 4 );
	HideDialogue();
	
	EndCutscene();
	
	victoryEndGame();
end

function victoryEndGame()
	DisablePlayerInteraction();
	HideDialogue(); -- remove possible texts on the screens

	-- accomplish secondary missions that are accomplished when the game is finished with certain condition
	
	
	ScriptManager:displaySubtitle( Text.T_MISSION_VICTORY );
	ScriptManager:wait( 4 );
	HideDialogue();
	endGame();
end


-- END Victory
-- BEGIN Defeated


function checkForEnd()
	if ScriptManager:noUnitsLeft( 0 ) then
		ScriptManager:runForPlayer( 0, "missionFailed" )
	end
end

function missionFailed()
	DisablePlayerInteraction();
	HideDialogue(); -- remove possible texts on the screens
	
	ScriptManager:displaySubtitle( Text.T_MISSION_FAILED );
	ScriptManager:wait( 4 );
	HideDialogue();
	endGame();
end


-- END Defeated


function endGame()
	ScriptManager:endGame()
end