require "lua/Constants"
require "lua/Functions"
require "lua/Functions_2"
require "lua/Text"
require "lua/Units"
require "math"


-- mission stuff
missionIds = {
	capManor	= 1,
	destroyArty	= 2,
	getBase		= 3,
};

-- missionId, primary( boolean ), textkey, xpReward
missions = {
	{ missionIds.capManor,		true,	Text.T_MAP_DE_3_3_MISSION_PRIMARY,					300 },
	{ missionIds.destroyArty,	false,	Text.T_MISSION_MAP_2_3_INTRO_MISSION_SECONDARY_2,	200 },
	{ missionIds.getBase,		false,	Text.T_MAP_DE_3_3_MISSION_SECONDARY_02,				150 }
};

enemySquadIds = {
	baseDefense		=  1,
	bridgeDefense	=  2,
	valleyDefense	=  3,
	villageDefense	=  4,
	manorDefense	=  5,
	arty			=  6,
	attackers		=  7,
	baseAirplanes	=  8,
	manorAirplanes	=  9,
	attackWave		= 10,
	forestDefense	= 11,
	forestDefense2	= 12,
};

baseDefenseIds		= {  5,  6,  7,  8,  9, 10, 11, 12, 13 };
bridgeDefenseIds	= { 14, 15, 16, 17, 49 };
valleyDefenseIds	= { 22, 23, 24 };
villageDefenseIds	= { 25, 26, 27, 28, 60 };
manorDefenseIds		= { 29, 30, 31, 32, 33, 34, 35, 36, 64, 65, 66, 67 };
artyIds				= { 37, 38, 61, 62, 39, 40, 41, 42 }; -- first 2 are the artys, next two are the AA
attackerIds			= { 18, 19, 20, 21, 50 };
baseAirplanesIds	= { 43, 44 };
manorAirplaneIds	= { 45, 46 };
forestDefenseIds	= { 47, 48 };
forestDefense2Ids	= { 63, 68, 69, 70 };

manorAirfieldId		= 2
baseAirfieldId		= 3;
artysToDestroy		= 2;
baseBuildings		= 2;
manorSpawnTiles		= { { 19, 21 }, { 20, 20 }, { 21, 20 } }
airfieldTiles		= { { 17, 23 }, { 18, 23 }, { 19, 22 } };
baseAirfieldTiles	= { {  5, 23 }, {  6, 23 } }

-- they respawn in a certain interval ( every x turns )
attackWaveRespawnInterval		=  5; -- defines how often waves will spawn
lastAttackWaveSpawnedInRound	= -6; -- indicates the round in which the last wave spawned
attackWaveUnits1 = {
--	playerId, templateID, 				tileX,					 tileY,							 squadId,				AiState
	{ 1, Units.UNIT_SU_RIFLE,			manorSpawnTiles[1][1], manorSpawnTiles[1][2], enemySquadIds.attackWave, AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_SU_RIFLE,			manorSpawnTiles[2][1], manorSpawnTiles[2][2], enemySquadIds.attackWave, AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_SU_MORTAR,			manorSpawnTiles[3][1], manorSpawnTiles[3][2], enemySquadIds.attackWave, AiState.ATTACK_AGGRESSIVE },
};

attackWaveUnits2 = {
--	playerId, templateID, 				tileX,					 tileY,							 squadId,				AiState
	{ 1, Units.UNIT_SU_RIFLE,			manorSpawnTiles[1][1], manorSpawnTiles[1][2], enemySquadIds.attackWave, AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_SU_TANK_T34,		manorSpawnTiles[2][1], manorSpawnTiles[2][2], enemySquadIds.attackWave, AiState.ATTACK_AGGRESSIVE },
};

attackWaveUnits3 = {
--	playerId, templateID, 				tileX,					 tileY,							 squadId,				AiState
	{ 1, Units.UNIT_SU_MORTAR,			manorSpawnTiles[1][1], manorSpawnTiles[1][2], enemySquadIds.attackWave, AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_SU_TANK_DESTROYER,	manorSpawnTiles[2][1], manorSpawnTiles[2][2], enemySquadIds.attackWave, AiState.ATTACK_AGGRESSIVE },
};

attackWaveUnits4 = {
--	playerId, templateID, 				tileX,					 tileY,							 squadId,				AiState
	{ 1, Units.UNIT_SU_MORTAR,			manorSpawnTiles[1][1], manorSpawnTiles[1][2], enemySquadIds.attackWave, AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_SU_KATYUSHA,		manorSpawnTiles[2][1], manorSpawnTiles[2][2], enemySquadIds.attackWave, AiState.ATTACK_AGGRESSIVE },
};

attackWaveUnits5 = {
--	playerId, templateID, 				tileX,					 tileY,							 squadId,				AiState
	{ 1, Units.UNIT_SU_TANK_T34,		manorSpawnTiles[1][1], manorSpawnTiles[1][2], enemySquadIds.attackWave, AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_SU_TANK_T34,		manorSpawnTiles[2][1], manorSpawnTiles[2][2], enemySquadIds.attackWave, AiState.ATTACK_AGGRESSIVE },
};

-- if player takes the base, give the ai more airplanes which respawn in a certain interval ( every x turns )
planeWaveRespawnInterval	= 4; -- defines how often waves will spawn
lastPlaneWaveSpawnedInRound	= 0; -- indicates the round in which the last wave spawned
--		playerId, templateID, 			tileX,				 tileY,					 squadId,				AiState
fighter = {  1, Units.UNIT_SU_FIGHTER, airfieldTiles[1][1], airfieldTiles[1][2], enemySquadIds.attackWave, AiState.AIRPLANE };


-- BEGIN Init


function onGameStart()
	CreateMissions( missions );
	setTriggers();
	storeValues();
	
	ScriptManager:gainGold( 0, 1500 );
	
	-- start intro cutscene
	ScriptManager:sleep( 0, "startIntroCutscene" ); -- without sleep player turn popup shows
end

function storeValues()
	ScriptManager:storeValue( "attackWaveRespawnInterval",		attackWaveRespawnInterval );
	ScriptManager:storeValue( "lastAttackWaveSpawnedInRound",	lastAttackWaveSpawnedInRound );
	ScriptManager:storeValue( "lastPlaneWaveSpawnedInRound", 	lastPlaneWaveSpawnedInRound );
end

function setTriggers()
	-- load all needed values after loading
	ScriptManager:addOnGameResumeTrigger( "loadValues", "loadValues", false );
	
	-- check if the player has units left
	ScriptManager:addOnUnitDestroyedTrigger( 0, -1, -1, "checkForEnd", "checkForEnd" , false );
	
	-- building trigger
	ScriptManager:addOnUnitCapturedTrigger(	 0, Units.UNIT_SU_HQ,		-1, "onDepotControlled",		"onDepotControlled",		false );
	ScriptManager:addOnUnitDestroyedTrigger( 1, Units.UNIT_SU_HQ,		-1, "onDepotControlled",		"onDepotControlled",		false );
	
	ScriptManager:addOnUnitCapturedTrigger(	 0, Units.UNIT_SU_AIRFIELD, -1, "onAirfieldCaptured",		"onAirfieldCaptured",		false );
	ScriptManager:addOnUnitDestroyedTrigger( 1, Units.UNIT_SU_AIRFIELD, -1, "onAirfieldDestroyed",		"onAirfieldDestroyed",		false );
	
	ScriptManager:addOnUnitCapturedTrigger(	 0, Units.UNIT_SU_MANOR,	-1,	"startVictoryCutscene",		"startVictoryCutscene",		false );
	ScriptManager:addOnUnitDestroyedTrigger( 1, Units.UNIT_SU_MANOR,	-1, "missionFailed",			"missionFailed",			false );
	
	ScriptManager:addOnUnitDestroyedTrigger( 0, Units.UNIT_DE_HQ,		-1, "missionFailedTransition",	"missionFailedTransition",	false );
	
	ScriptManager:addOnUnitHasAttackedTrigger( 0, Units.UNIT_SU_MANOR,	-1, "manorAttacked", 					"manorAttacked",					false );
	ScriptManager:addOnUnitHasAttackedTrigger( 1, Units.UNIT_DE_HQ,		-1, "ProtectTheSupplyDepotHintSingle",	"ProtectTheSupplyDepotHintSingle",	false );
	
	-- trigger for arty
	ScriptManager:addOnUnitHasAttackedTrigger(	0, Units.UNIT_DE_BOMBER,	-1, "onPlaneAttacked", 		"onPlaneAttacked",		false );
	ScriptManager:addOnUnitHasAttackedTrigger(	0, Units.UNIT_DE_FIGHTER,	-1, "onPlaneAttacked",		"onPlaneAttacked",		false );
	ScriptManager:addOnUnitHasAttackedTrigger(	1, Units.UNIT_SU_HOWITZER,	-1, "onArtyHasAttacked",	"onArtyHasAttacked",	false );
	ScriptManager:addOnUnitHasAttackedTrigger(	0, Units.UNIT_SU_HOWITZER,	-1, "onArtyHasAttacked",	"onArtyHasAttacked",	false );
	ScriptManager:addOnUnitDestroyedTrigger(	1, Units.UNIT_SU_HOWITZER,	-1, "onArtyDestroyed",		"onArtyDestroyed", 		false );
	
	-- attack wave trigger
	ScriptManager:addOnTurnStartTrigger( 1, -1, -1, "onEnemyTurnStart", "onEnemyTurnStart", false );
	
	-- manor planes aggressive trigger
	ScriptManager:addOnEnterTileTrigger( 8, 7, 0, -1, -1, false, "manorPlaneAttack", "manorPlaneAttack", false );
end

function loadValues()
	attackWaveRespawnInterval		= ScriptManager:storeValue( "attackWaveRespawnInterval" );
	lastAttackWaveSpawnedInRound	= ScriptManager:storeValue( "lastAttackWaveSpawnedInRound" );
	lastPlaneWaveSpawnedInRound		= ScriptManager:storeValue( "lastPlaneWaveSpawnedInRound" );
end


-- END Init
-- BEGIN Intro Cutscene


function startIntroCutscene()
	StartCutscene();
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE_3_3_MISSION_INTRO_FLAVOR_01 );
	ScriptManager:wait( 4 );
	HideDialogue();
	ScriptManager:wait( 1 );
	
	ScriptManager:moveCameraEx( 20, 21, 0.7, 6, false );
	ScriptManager:wait( 2 );
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE_3_3_MISSION_INTRO_FLAVOR_02 );
	ScriptManager:wait( 4 );
	
	ScriptManager:revealFogOfWar( 20, 21, 1, true );
	ScriptManager:showMarkerAt( 20, 21 );
	ScriptManager:wait( 2 );
	ScriptManager:removeAllMarkers();
	ScriptManager:revealFogOfWar( 20, 21, 1, false );
	HideDialogue();
	ScriptManager:wait( 1 );
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE_3_3_MISSION_INTRO_FLAVOR_03 );
	ScriptManager:moveCameraEx( 2, 6, 0.7, 6, true );
	ScriptManager:wait( 1 );
	HideDialogue();
	ScriptManager:wait( 1 );
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE_3_3_MISSION_INTRO_FLAVOR_04 );
	ScriptManager:moveCameraEx( 5, 22, 0.7, 4, true );
	ScriptManager:wait( 1 );
	HideDialogue();
	ScriptManager:wait( 1 );
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_4_3_MISSION_INTRO_FLAVOR_07 );
	ScriptManager:moveCameraEx( 2, 6, 0.7, 4, true );
	HideDialogue();
	
	EndCutscene();
	
	startTheMission();
end

function startTheMission()
	ScriptManager:setMissionStatus( missionIds.capManor, MissionStatus.IN_PROGRESS );
	ScriptManager:setMissionStatus( missionIds.getBase,	 MissionStatus.IN_PROGRESS );
	
	ScriptManager:setMissionProgress( missionIds.getBase, 0 );
	
	--TODO: remove this function!
	--testing();
	--removeSquads();
	--ScriptManager:addOnUnitSelectedTrigger( 1, -1, -1, "showParameters", "showParameters",false )
end

function testing()
	ScriptManager:spawnUnit( 0, Units.GOTT, 13, 1, 42, AiState.WAIT );
	ScriptManager:spawnUnit( 0, Units.GOTT, 14, 1, 42, AiState.WAIT );
	
	ScriptManager:spawnUnit( 0, Units.UNIT_DE_BOMBER, 20, 12, 42, AiState.WAIT );
	ScriptManager:spawnUnit( 0, Units.UNIT_DE_PANZERFAUST, 23, 12, 42, AiState.WAIT );
	ScriptManager:spawnUnit( 0, Units.UNIT_DE_FIGHTER, 21, 12, 42, AiState.WAIT );
end

function removeSquads()
	--for i = 1, # do
	--	ScriptManager:removeUnitEx( [i], false );
	--end
end

function showParameters( selectedUnit )
	--print( string.format( "[] %s", selectedUnit:get() ) );
end


-- END Intro Cutscene
-- BEGIN Unit Stuff


function onEnemyTurnStart()
	-- remove action points from the arty in order to make them unmoveable but they are not shown in fow
	if ScriptManager:getMissionStatus( missionIds.destroyArty ) ~= MissionStatus.ACCOMPLISHED then
		for i = 1, ( artysToDestroy * 2 ) do -- 1x arty + 1aa for each arty
			local unit = Unit:getUnitById( artyIds[i] )
			if unit ~= nil then
				unit:setActionPoints( 0 );
			end
		end
	end

	-- check if a wave spawns this round
	if lastAttackWaveSpawnedInRound + attackWaveRespawnInterval <= ScriptManager:getCurrentRound() then
		-- find a wave to spawn by random number
		local rnd = math.random( 0, 4 );
		local spawnTable = 0;
		
		if rnd == 0 then
			spawnTable = attackWaveUnits1
		
		elseif rnd == 1 then
			spawnTable = attackWaveUnits2
		
		elseif rnd == 2 then
			spawnTable = attackWaveUnits3
		
		elseif rnd == 3 then
			spawnTable = attackWaveUnits4
		
		else
			spawnTable = attackWaveUnits5
		end
		
		-- spawn the wave
		for i = 1, #spawnTable do
			spawnWaveUnit( spawnTable[i] );
		end
		
		lastAttackWaveSpawnedInRound = ScriptManager:getCurrentRound(); -- reset round count
		ScriptManager:storeValue( "lastAttackWaveSpawnedInRound",	lastAttackWaveSpawnedInRound );
	end

	-- only spawn planes when the player has an airfield as well!
	if ScriptManager:getMissionStatus( missionIds.getBase ) == MissionStatus.ACCOMPLISHED and
		Unit:getUnitById( manorAirfieldId ):getPlayerId() == 1 then
		-- check if a plane spawns this round
		if lastPlaneWaveSpawnedInRound + planeWaveRespawnInterval <= ScriptManager:getCurrentRound() then
			spawnWaveUnit( fighter );
			
			lastPlaneWaveSpawnedInRound = ScriptManager:getCurrentRound(); -- reset round count
			ScriptManager:storeValue( "lastPlaneWaveSpawnedInRound", lastPlaneWaveSpawnedInRound );
		end
	end
end

function spawnWaveUnit( spawnData )
	ScriptManager:spawnUnit( spawnData[1], spawnData[2], spawnData[3], spawnData[4], spawnData[5], spawnData[6] );
end

function manorPlaneAttack()
	Unit:changeSquadState( 1, enemySquadIds.manorAirplanes, AiState.AIRPLANE );
	ScriptManager:removeTrigger( "manorPlaneAttack" );
end

function onPlaneAttacked( attacker, defender)
	if defender:getTemplateId() == Units.UNIT_SU_HOWITZER then
		StartCutscene();
		
		ScriptManager:displaySubtitle( Text.T_MAP_DE_3_3_MISSION_SECONDARY_01_FLAVOR_04 );
		ScriptManager:wait( 4 );
		HideDialogue();
		
		EndCutscene();
		DisablePlayerInteraction()
		
		ScriptManager:setMissionStatus( missionIds.destroyArty, MissionStatus.IN_PROGRESS );
		ScriptManager:removeTrigger( "onArtyHasAttacked" );
		ScriptManager:removeTrigger( "onPlaneAttacked" );
		ScriptManager:wait( 2 );
		
		EnablePlayerInteraction();
	end
end

function onArtyHasAttacked( attacker, defender )
	local templateId = defender:getTemplateId();
	-- find the unit thats NOT the artillery!
	if templateId ~= Units.UNIT_SU_HOWITZER then
		ScriptManager:storeValue( "unitX", defender:getX() );
		ScriptManager:storeValue( "unitY", defender:getX() );
	
	else
		ScriptManager:storeValue( "unitX", attacker:getX() );
		ScriptManager:storeValue( "unitY", attacker:getX() );
		
	end
	
	ScriptManager:runForPlayer( 0, "startArtyCutscene" );
	ScriptManager:removeTrigger( "onArtyHasAttacked" );
	ScriptManager:removeTrigger( "onPlaneAttacked" );
end

function startArtyCutscene()
	StartCutscene();
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE_3_3_MISSION_SECONDARY_01_FLAVOR_01 );
	ScriptManager:wait( 4 );
	HideDialogue();
	ScriptManager:wait( 1 );
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE_3_3_MISSION_SECONDARY_01_FLAVOR_02 );
	ScriptManager:moveCameraEx( 19, 12, 0.7, 3, true );
	ScriptManager:wait( 1 );
	
	ScriptManager:revealFogOfWar( 19, 12, 1, true );
	ScriptManager:showMarkerAt( 19, 12 );
	ScriptManager:showMarkerAt( 19, 13 );
	ScriptManager:wait( 2 );
	ScriptManager:removeAllMarkers();
	ScriptManager:revealFogOfWar( 19, 12, 1, false );
	HideDialogue();
	ScriptManager:wait( 1 );
	
	ScriptManager:moveCameraEx( ScriptManager:getValue( "unitX"), ScriptManager:getValue( "unitY"), 0.7, 3, true );
	
	EndCutscene();
	DisablePlayerInteraction()
	
	ScriptManager:setMissionStatus( missionIds.destroyArty, MissionStatus.IN_PROGRESS );
	ScriptManager:deleteValue( "unitX" );
	ScriptManager:deleteValue( "unitY" );
	ScriptManager:wait( 2 );
	
	EnablePlayerInteraction();
end

function onArtyDestroyed()
	ScriptManager:setMissionProgress( missionIds.destroyArty, ScriptManager:getMissionProgress( missionIds.destroyArty ) + 1 );
	ScriptManager:runForPlayer( 0, "showArtyMissionProgress" );
end

function showArtyMissionProgress()
	if ScriptManager:getMissionProgress( missionIds.destroyArty ) >= artysToDestroy then
		DisablePlayerInteraction();
		
		ScriptManager:setMissionStatus( missionIds.destroyArty, MissionStatus.ACCOMPLISHED );
		ScriptManager:removeTrigger( "onArtyDestroyed" );
		ScriptManager:wait( 2 );
		
		EnablePlayerInteraction();
		
		ScriptManager:displaySubtitle( Text.T_MAP_DE_3_3_MISSION_SECONDARY_01_FLAVOR_03 );
		ScriptManager:sleep( 4, "HideDialogue" );
	
	else
		ShowMissionProgressPopup( ScriptManager:getMissionProgress( missionIds.destroyArty ), Text.T_MAP_DE_3_3_MISSION_SECONDARY_01_COUNTER );
	end
end


-- END Unit Stuff
-- BEGIN Building Stuff


function manorAttacked( activeUnit, passiveUnit )
	DisablePlayerInteraction();
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE_3_1_MISSION_PRIMARY_WARNING );
	ScriptManager:moveCameraEx( passiveUnit:getX(), passiveUnit:getY(), 1.0, 1, true );
	ScriptManager:showMarkerAt( passiveUnit:getX(), passiveUnit:getY() );
	ScriptManager:wait( 2 );
	ScriptManager:removeAllMarkers();
	
	EnablePlayerInteraction();
end

function onDepotControlled()
	checkBaseMissionProgress();
	ScriptManager:removeTrigger( "onDepotControlled" );
end

function onAirfieldCaptured( capturer, airfield )
	onAirfieldDestroyed( airfield );
end

function onAirfieldDestroyed( unit )
	if unit:getId() == baseAirfieldId then
		checkBaseMissionProgress();
		ScriptManager:removeTrigger( "onAirfieldControlled" );
	end
end

function checkBaseMissionProgress()
	ScriptManager:setMissionProgress( missionIds.getBase, ScriptManager:getMissionProgress( missionIds.getBase ) + 1 );
	
	if ScriptManager:getMissionProgress( missionIds.getBase ) >= baseBuildings then
		startBaseMissionCutscene();
	end
end

function startBaseMissionCutscene()
	DisablePlayerInteraction()
	
	ScriptManager:setMissionStatus( missionIds.getBase, MissionStatus.ACCOMPLISHED );
	ScriptManager:wait( 2 );
	
	EnablePlayerInteraction();
	StartCutscene()
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE_3_3_MISSION_SECONDARY_02_FLAVOR_01 );
	ScriptManager:wait( 4 );
	HideDialogue();
	
	EndCutscene();
end

-- END Building Stuff
-- BEGIN Victory


function startVictoryCutscene()
	DisablePlayerInteraction();
	
	ScriptManager:setMissionStatus( missionIds.capManor, MissionStatus.ACCOMPLISHED );
	ScriptManager:wait( 2 );
	
	EnablePlayerInteraction();
	StartCutscene();
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE_3_3_MISSION_VICTORY_FLAVOR_01 );
	ScriptManager:wait( 4 );
	HideDialogue();
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE_3_3_MISSION_VICTORY_FLAVOR_02 );
	ScriptManager:wait( 4 );
	HideDialogue();
	
	EndCutscene();
	
	victoryEndGame();
end

function victoryEndGame()
	DisablePlayerInteraction();
	HideDialogue(); -- remove possible texts on the screens

	-- accomplish secondary missions that are accomplished when the game is finished with certain condition
	
	
	ScriptManager:displaySubtitle( Text.T_MISSION_VICTORY );
	ScriptManager:wait( 4 );
	HideDialogue();
	endGame();
end


-- END Victory
-- BEGIN Defeated


function missionFailedTransition()
	-- fail mission in player turn
	ScriptManager:runForPlayer( 0, "missionFailed" );
end

function missionFailed()
	DisablePlayerInteraction();
	HideDialogue(); -- remove possible texts on the screens
	
	ScriptManager:displaySubtitle( Text.T_MISSION_FAILED );
	ScriptManager:wait( 4 );
	HideDialogue();
	endGame();
end


-- END Defeated


function endGame()
	ScriptManager:endGame()
end