require "lua/Constants"
require "lua/Functions"
require "lua/Functions_2"
require "lua/Text"
require "lua/Units"
require "math"


-- mission stuff


missionIds = {
	captureTankFactory		= 1,
	captureNorthernDepot	= 2,
	captureSouthernDepot	= 3,
	RepelAttackers 			= 4,
	captureOutpost			= 5,


};

-- missionId, primary( boolean ), textkey, xpReward
missions = {
	{ missionIds.captureTankFactory, 			true, Text.T_MAP_DE_4_2_MISSION_PRIMARY_01,	  125 },
	{ missionIds.captureNorthernDepot, 			true, Text.T_MAP_DE_4_2_MISSION_PRIMARY_02,   125 },
	{ missionIds.captureSouthernDepot, 			true, Text.T_MAP_DE_4_2_MISSION_PRIMARY_03,	  125 },
	{ missionIds.RepelAttackers,			 	true, Text.T_MAP_DE_4_2_MISSION_PRIMARY_04,    25 },
	{ missionIds.captureOutpost,			 	false,Text.T_MAP_4_2_MISSION_SECONDARY_01,     25 },
	
};

enemySquadIds = {

tankFactory = 1,
middleLeft = 2,
southLeft = 3,
depotNorth = 4,
middleRight = 5,
depotSouth = 6,
bunkerDepotNorthRight = 7,
bunkerDepotNorthLeft = 8,
mgNestTankFactoryNorth = 9,
mgNestTankFactorySouth = 10,
mgNestDepotSouth = 11,
bunkerdepotSouth = 12,
lakeMiddle = 13,


firstAttackers		= 54,

attackersFactory	= 55,
attackersSouth		= 56,
attackersNorth		= 57,


--attackersLeftPath 	= 55,
--attackersRightPath	= 56,
--attackersUpPath		= 57,

};


tankFactoryIds	= { 7, 9, 26, 27, 28, 31, 32,  };
middleLeftIds 	= { 12, 13, 14, 43, 44,  };
southLeftIds 	= { 15, 16, 17, 25, 33, 35, 36, 38, 39, };
depotNorthIds 	= { 11, 18, 19, 34, 45, 46, 47, 56, };
middleRightIds	= { 40, 41, 42, };
depotSouthIds	= { 10, 20, 21, 22, 23, 24, 30, };
bunkerDepotNorthRightIds = { 6 };
bunkerDepotNorthLeftIds = { 29 };
mgNestTankFactoryNorthIds = { 8 };
mgNestTankFactorySouthIds = { 3 };
mgNestDepotSouthIds = { 5 };
bunkerdepotSouth = { 4 };


firstAttackers = {
-- playerId, templateID, tileX, tileY, squadId, AiState
{ 1, Units.UNIT_SU_MORTAR,			7, 13, enemySquadIds.firstAttackers, AiState.ATTACK_AGGRESSIVE };
{ 1, Units.UNIT_SU_RIFLE,			8, 13, enemySquadIds.firstAttackers, AiState.ATTACK_AGGRESSIVE };
{ 1, Units.UNIT_SU_RIFLE,			8, 12, enemySquadIds.firstAttackers, AiState.ATTACK_AGGRESSIVE };			

};


attackersLeftPath = {
-- playerId, templateID, tileX, tileY, squadId, AiState
	{ 1, Units.UNIT_SU_MORTAR, 			8, 34, enemySquadIds.attackersFactory, AiState.ATTACK_AGGRESSIVE };
	{ 1, Units.UNIT_SU_RIFLE,  			9, 34, enemySquadIds.attackersFactory, AiState.ATTACK_AGGRESSIVE };	
	{ 1, Units.UNIT_SU_RIFLE, 			10, 34, enemySquadIds.attackersFactory, AiState.ATTACK_AGGRESSIVE };
	{ 1, Units.UNIT_SU_ANTI_INFANTRY, 	7, 34, enemySquadIds.attackersFactory, AiState.ATTACK_AGGRESSIVE };
};

attackersRightPath = {
-- playerId, templateID, tileX, tileY, squadId, AiState
	{ 1, Units.UNIT_SU_ANTI_INFANTRY, 	16, 34, enemySquadIds.attackersNorth, AiState.ATTACK_AGGRESSIVE };
	{ 1, Units.UNIT_SU_RIFLE, 			17, 34, enemySquadIds.attackersNorth, AiState.ATTACK_AGGRESSIVE };
	{ 1, Units.UNIT_SU_MORTAR, 			16, 33, enemySquadIds.attackersNorth, AiState.ATTACK_AGGRESSIVE };
};

attackersPathUp = {
-- playerId, templateID, tileX, tileY, squadId, AiState
	{ 1, Units.UNIT_SU_RIFLE, 			20, 1, enemySquadIds.attackersSouth, AiState.ATTACK_AGGRESSIVE };
	{ 1, Units.UNIT_SU_MORTAR, 			22, 1, enemySquadIds.attackersSouth, AiState.ATTACK_AGGRESSIVE };
	{ 1, Units.UNIT_SU_TANK_KV1, 		21, 1, enemySquadIds.attackersSouth, AiState.ATTACK_AGGRESSIVE };
};


enemyBuildingIds = {

tankFactory = 0,
depotNorth = 1,
depotSouth = 2,
outpost	= 57,
};

attackersToRepel = 4;
attackersRepelled = 0;


-- BEGIN Init


function onGameStart()
	CreateMissions( missions );
	setTriggers();
	storeValues();
	
	ScriptManager:gainGold( 0, 1800 );
	
	-- start intro cutscene
	ScriptManager:sleep( 0, "startIntroCutscene" ); -- without sleep player turn popup shows
end

function storeValues()
	ScriptManager:storeValue( "attackersRepelled", attackersRepelled );

end



function setTriggers()
	
	-- load all needed values after loading
	ScriptManager:addOnGameResumeTrigger( "loadValues", "loadValues", false );
	
	ScriptManager:addOnUnitCapturedTrigger( 0, Units.UNIT_DE_TANKFACTORY_CAPTURABLE, enemyBuildingIds.tankFactory, "onTankFactoryCaptured", "onTankFactoryCaptured", false );
	ScriptManager:addOnUnitDestroyedTrigger( 1, Units.UNIT_DE_TANKFACTORY_CAPTURABLE, enemyBuildingIds.tankFactory, "onTankFactoryCaptured","onTankFactoryCaptured", false );
	
	ScriptManager:addOnUnitCapturedTrigger( 0, Units.UNIT_SU_HQ, enemyBuildingIds.depotNorth, "onDepotNorthCaptured", "onDepotNorthCaptured", false );
	ScriptManager:addOnUnitDestroyedTrigger( 1, Units.UNIT_SU_HQ, enemyBuildingIds.depotNorth, "onDepotNorthCaptured", "onDepotNorthCaptured", false );
	
	ScriptManager:addOnUnitCapturedTrigger( 0, Units.UNIT_SU_HQ, enemyBuildingIds.depotSouth, "onDepotSouthCaptured", "onDepotSouthCaptured", false );
	ScriptManager:addOnUnitDestroyedTrigger( 1, Units.UNIT_SU_HQ, enemyBuildingIds.depotSouth, "onDepotSouthCaptured", "onDepotSouthCaptured", false );
	
	local x
	for x = 1, 12 do
	ScriptManager:addOnEnterTileTrigger( x, 8, 0, -1, -1, false, "triggerFirstAttack", "triggerFirstAttack", false );
	end
				
	-- check if the player has units left
	ScriptManager:addOnUnitDestroyedTrigger( 0, -1, -1, "checkForEnd", "checkForEnd" , false );
	
	-- Triggerline for the secondary outpost objective

	local x
	for x = 1, 12 do
	ScriptManager:addOnEnterTileTrigger( x, 11, 0, -1, -1, true, "playOutpostCutscene", "playOutpostCutscene", false );
	end 

	-- when the outpost is captured	
	ScriptManager:addOnUnitCapturedTrigger( 0, Units.UNIT_SU_HQ, enemyBuildingIds.outpost, "onOutpostCaptured", "onOutpostCaptured", false );
	
	
	-- when the outpost gets destroyed
	ScriptManager:addOnUnitDestroyedTrigger( 1, Units.UNIT_SU_HQ, enemyBuildingIds.outpost, "onOutpostDestroyed", "onOutpostDestroyed", false );
	
		
		
end

function loadValues()
	ScriptManager:getValue( "attackersRepelled" );
end


-- END Init
-- BEGIN Intro Cutscene


function startIntroCutscene()
	StartCutscene();
	
	-- TODO insert content here
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE_4_2_MISSION_INTRO_FLAVOR_01 );
	ScriptManager:wait( 5 );
	HideDialogue();
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE_4_2_MISSION_INTRO_FLAVOR_02 );
	
	ScriptManager:moveCameraEx( 19, 7, 0.7, 3, true );
	ScriptManager:wait( 2 );
	
	ScriptManager:moveCameraEx( 20, 29, 0.7, 3, true );
	ScriptManager:wait( 2 );
	
	ScriptManager:moveCameraEx( 2, 31, 0.7, 3, true );
	ScriptManager:wait( 2 );
	
	ScriptManager:moveCameraEx( 8, 2, 0.7, 3, true );
	ScriptManager:wait( 1 );
	HideDialogue();
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE_4_2_MISSION_INTRO_FLAVOR_03 );
	ScriptManager:wait( 4 );
	HideDialogue();
		
	ScriptManager:displaySubtitle( Text.T_MAP_DE_4_2_MISSION_INTRO_FLAVOR_04 );
	ScriptManager:wait( 4 );
	HideDialogue();
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE_4_2_MISSION_INTRO_FLAVOR_05 );
	ScriptManager:wait( 4 );
	HideDialogue();
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE_4_2_MISSION_INTRO_FLAVOR_06 );
	ScriptManager:wait( 3 );
	HideDialogue();
	
	EndCutscene();
	
	startTheMission();
end

function startTheMission()

	ScriptManager:removeReinforcementsFromBase( enemyBuildingIds.tankFactory, Units.UNIT_DE_MP40 );
	ScriptManager:removeReinforcementsFromBase( enemyBuildingIds.tankFactory, Units.UNIT_DE_PANZERFAUST );
	ScriptManager:removeReinforcementsFromBase( enemyBuildingIds.tankFactory, Units.UNIT_DE_ANTI_INFANTRY );


	ScriptManager:setMissionStatus( missionIds.captureTankFactory, 	 MissionStatus.IN_PROGRESS );
	ScriptManager:setMissionStatus( missionIds.captureNorthernDepot, MissionStatus.IN_PROGRESS );
	ScriptManager:setMissionStatus( missionIds.captureSouthernDepot, MissionStatus.IN_PROGRESS );

	
	ScriptManager:fortifyUnit( 13 );
	ScriptManager:fortifyUnit( 14 );	
	
	--TODO: remove this function!
	--testing();
	--removeSquads();
	--ScriptManager:addOnUnitSelectedTrigger( 1, -1, -1, "showParameters", "showParameters",false)
end

function testing()
	ScriptManager:spawnUnit( 0, Units.GOTT, 1, 2, 42, AiState.WAIT );
	ScriptManager:spawnUnit( 0, Units.GOTT, 2, 2, 42, AiState.WAIT );
end



function removeSquads()

	for i = 1, #tankFactoryIds do
	ScriptManager:removeUnitEx(tankFactoryIds[i], false)
	end

	for i = 1, #middleLeftIds do
	ScriptManager:removeUnitEx(middleLeftIds[i], false );
	end

	for i = 1, #southLeftIds do
	ScriptManager:removeUnitEx(southLeftIds[i], false );
	end
	
	for i = 1, #depotNorthIds do
	ScriptManager:removeUnitEx(depotNorthIds[i], false );
	end
	
	for i = 1, #middleRightIds do
	ScriptManager:removeUnitEx( middleRightIds[i], false );
	end
	
	for i = 1, # depotSouthIds do
	ScriptManager:removeUnitEx(depotSouthIds[i], false );
	end
	
	for i = 1, # bunkerDepotNorthRightIds do
	ScriptManager:removeUnitEx(bunkerDepotNorthRightIds[i], false );
	end
	
	for i = 1, #bunkerDepotNorthLeftIds do
	ScriptManager:removeUnitEx(bunkerDepotNorthLeftIds[i], false );
	end
	
	for i = 1, #mgNestTankFactoryNorthIds do
	ScriptManager:removeUnitEx(mgNestTankFactoryNorthIds[i], false );
	end
	
	for i = 1, #mgNestTankFactorySouthIds do
	ScriptManager:removeUnitEx(mgNestTankFactorySouthIds[i], false );
	end
	
	for i = 1, # mgNestDepotSouthIds do
	ScriptManager:removeUnitEx(mgNestDepotSouthIds[i], false );
	end
	
	for i = 1, # bunkerdepotSouth do
	ScriptManager:removeUnitEx(bunkerdepotSouth[i], false );
	end		
end


function showParameters( selectedUnit )
	--print( string.format( "[] %s", selectedUnit:get() ) );
end


-- END Intro Cutscene
-- BEGIN


-- insert missionstuff here


function playOutpostCutscene()
	
	ScriptManager:removeTrigger( "playOutpostCutscene" );
	
	ScriptManager:startCutscene();
	
	ScriptManager:moveCameraEx( 9, 21, 0.7, 2, true );
	ScriptManager:wait( 1 );
	
	ScriptManager:revealFogOfWar( 9, 21, 1, true );
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE_3_1_MISSION_PRIMARY_FLAVOR_03 );
	ScriptManager:showMarkerAt( 9, 21 );
	ScriptManager:wait( 5 );
	ScriptManager:removeAllMarkers();
	ScriptManager:wait( 1 )
	ScriptManager:revealFogOfWar( 9, 21, 1, false );
	ScriptManager:wait( 1 );
	
	ScriptManager:dismissSubtitle();
	ScriptManager:stopCutscene();
	
	
	local unit = ScriptManager:spawnUnit( 1, Units.UNIT_SU_TANK_T34, 9, 23, 2, AiState.DEFEND ):getId();
	
	ScriptManager:moveUnitEx( unit, 9, 22, false );
	ScriptManager:fortifyUnit( unit );
	
	ScriptManager:setMissionStatus( missionIds.captureOutpost, MissionStatus.IN_PROGRESS );	
end


function onOutpostCaptured()
	ScriptManager:removeTrigger( "onOutpostCaptured" );
	ScriptManager:removeTrigger( "onOutpostDestroyed" );
	ScriptManager:setMissionStatus( missionIds.captureOutpost, MissionStatus.ACCOMPLISHED );
end


function onOutpostDestroyed()
	ScriptManager:removeTrigger( "onOutpostCaptured" );
	ScriptManager:removeTrigger( "onOutpostDestroyed" );
	ScriptManager:setMissionStatus( missionIds.captureOutpost, MissionStatus.FAILED );
end



function triggerFirstAttack()
	ScriptManager:removeTrigger( "triggerFirstAttack" );
	ScriptManager:wait( 1 );
	for i = 1, #firstAttackers do
	local data = firstAttackers[i]
	local unit = ScriptManager:spawnUnit( data[1], data[2], data[3], data[4], data[5], data[6] );
	end
	ScriptManager:setTargetTile( 1, enemySquadIds.firstAttackers, 7, 5 );
end


function onTankFactoryCaptured()
	ScriptManager:removeTrigger( "onTankFactoryCaptured" );
	ScriptManager:setMissionStatus( missionIds.captureTankFactory, MissionStatus.ACCOMPLISHED );
	
	checkForVictory()
	
	ScriptManager:wait( 2 );
	
	spawnAttackersLeftPath()
	spawnAttackersRightPath()
	spawnAttackersPathUp()
	
	-- when a squadmember of the tankfactory attackers gets destroyed
	ScriptManager:addOnUnitDestroyedTrigger( 1, -1, -1, "onAttackerDestroyed", "onAttackerDestroyed", false );
	
	ScriptManager:startCutscene()
	
	ScriptManager:wait( 1 );
	
	ScriptManager:moveCameraEx( 8, 34, 0.7, 2, true );
	ScriptManager:revealFogOfWar( 8, 34, 2, true );
	
	ScriptManager:wait( 1 );
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE_4_2_SECONDARY_01_FLAVOR_01 );
		
	ScriptManager:wait( 2 );
	
	ScriptManager:moveCameraEx( 16, 34, 0.7, 2, true );
	
	ScriptManager:revealFogOfWar( 16, 34, 2, true );
	
	ScriptManager:wait( 2 );
	
	ScriptManager:moveCameraEx( 21, 1, 0.7, 3, true );
	
	ScriptManager:revealFogOfWar( 21, 1, 2, true );
	
	ScriptManager:wait( 2 );
	
	ScriptManager:moveCameraEx( 2, 31, 0.7, 3, true );
	ScriptManager:dismissSubtitle();
	
	ScriptManager:revealFogOfWar( 8, 34, 2, false );
	ScriptManager:revealFogOfWar( 16, 34, 2, false );
	ScriptManager:revealFogOfWar( 21, 1, 2, false );
	
	ScriptManager:wait( 1 );
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE_4_2_MISSION_SECONDARY_01_FLAVOR_02 );
	
	ScriptManager:wait( 5 );
	
	ScriptManager:dismissSubtitle();
	
	ScriptManager:stopCutscene();
	
	ScriptManager:wait( 1 );
	
	ScriptManager:setMissionStatus( missionIds.RepelAttackers, MissionStatus.IN_PROGRESS );
	
end


function onDepotNorthCaptured()
	ScriptManager:removeTrigger( "onDepotNorthCaptured" );
	ScriptManager:setMissionStatus( missionIds.captureNorthernDepot, MissionStatus.ACCOMPLISHED );
	
	ScriptManager:wait( 1 );
	checkForVictory()
end


function onDepotSouthCaptured()
	ScriptManager:removeTrigger( "onDepotSouthCaptured" );
	ScriptManager:setMissionStatus(missionIds.captureSouthernDepot, MissionStatus.ACCOMPLISHED );
	
	ScriptManager:wait( 1 );
	checkForVictory()
end


function spawnAttackersLeftPath()
	local unit
	for i = 1, #attackersLeftPath do
	local data = attackersLeftPath[i]
	unit = ScriptManager:spawnUnit( data[1],data[2],data[3],data[4],data[5],data[6] );
	end	
	ScriptManager:setTargetTile( 1, enemySquadIds.attackersFactory, 2, 31 );	
end


function spawnAttackersRightPath()
	for i = 1, #attackersRightPath do
	local data = attackersRightPath[i]
	local unit = ScriptManager:spawnUnit( data[1], data[2], data[3], data[4], data[5], data[6] );
	end
	ScriptManager:setTargetTile( 1, enemySquadIds.attackersNorth, 20, 29 );
end


function spawnAttackersPathUp()
	for i = 1, #attackersPathUp do
	local data = attackersPathUp[i]
	local unit = ScriptManager:spawnUnit( data[1], data[2], data[3], data[4], data[5], data[6] );
	end	
	ScriptManager:setTargetTile( 1, enemySquadIds.attackersSouth, 19, 7 );
end


function onAttackerDestroyed( unit )
	
	if unit:getSquadId() == enemySquadIds.attackersFactory then
		ScriptManager:setMissionProgress( missionIds.RepelAttackers, ScriptManager:getMissionProgress( missionIds.RepelAttackers ) +1 );
		ScriptManager:storeValue( "attackersRepelled", attackersRepelled );
		ShowMissionProgressPopup( ScriptManager:getMissionProgress( missionIds.RepelAttackers ), Text.T_MAP_DE_4_2_MISSION_SECONDARY_01_COUNTER );
		
--		elseif unit:getSquadId() == enemySquadIds.attackersNorth then
--			ScriptManager:setMissionProgress( missionIds.RepelAttackers, ScriptManager:getMissionProgress( missionIds.RepelAttackers ) +1 );
--			ScriptManager:storeValue( "attackersRepelled", attackersRepelled );
--			ShowMissionProgressPopup( ScriptManager:getMissionProgress( missionIds.RepelAttackers ), Text.T_MAP_DE_4_2_MISSION_SECONDARY_01_COUNTER );
--		
--		elseif unit:getSquadId() == enemySquadIds.attackersSouth then	
--			ScriptManager:setMissionProgress( missionIds.RepelAttackers, ScriptManager:getMissionProgress( missionIds.RepelAttackers ) +1 );
--			ScriptManager:storeValue( "attackersRepelled", attackersRepelled );
--			ShowMissionProgressPopup( ScriptManager:getMissionProgress( missionIds.RepelAttackers ), Text.T_MAP_DE_4_2_MISSION_SECONDARY_01_COUNTER );
	end	
		
		if 	ScriptManager:getMissionProgress( missionIds.RepelAttackers ) >= attackersToRepel then
			
			ScriptManager:removeTrigger( "onAttackerDestroyed" );
			
			ScriptManager:setMissionStatus( missionIds.RepelAttackers, MissionStatus.ACCOMPLISHED );
				
			ScriptManager:displaySubtitle( Text.T_MAP_DE_4_2_MISSION_SECONDARY_01_FLAVOR_03 );
			ScriptManager:wait( 3 );
			ScriptManager:dismissSubtitle();
			
			checkForVictory()
		end
end
		
-- END
-- BEGIN Victory

function checkForVictory()
	if 	ScriptManager:getMissionStatus( missionIds.captureTankFactory )		== MissionStatus.ACCOMPLISHED and
		ScriptManager:getMissionStatus( missionIds.captureNorthernDepot )	== MissionStatus.ACCOMPLISHED and
		ScriptManager:getMissionStatus( missionIds.captureSouthernDepot )	== MissionStatus.ACCOMPLISHED and
		ScriptManager:getMissionStatus( missionIds.RepelAttackers )			== MissionStatus.ACCOMPLISHED then 	
		startVictoryCutscene()
	end
end


function startVictoryCutscene()
	StartCutscene();
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE_4_2_MISSION_OUTRO_FLAVOR_01 );
	ScriptManager:wait( 3 );
	HideDialogue();
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE_4_2_MISSION_OUTRO_FLAVOR_02 );
	ScriptManager:wait( 4 );
	HideDialogue()
	
	EndCutscene();
	
	victoryEndGame();
end

function victoryEndGame()
	DisablePlayerInteraction();
	HideDialogue(); -- remove possible texts on the screens

	-- accomplish secondary missions that are accomplished when the game is finished with certain condition
	
	
	ScriptManager:displaySubtitle( Text.T_MISSION_VICTORY );
	ScriptManager:wait( 4 );
	HideDialogue();
	endGame();
end


-- END Victory
-- BEGIN Defeated


function checkForEnd()
	if ScriptManager:noUnitsLeft( 0 ) then
		ScriptManager:runForPlayer( 0, "missionFailed" )
	end
end

function missionFailed()
	DisablePlayerInteraction();
	HideDialogue(); -- remove possible texts on the screens
	
	ScriptManager:displaySubtitle( Text.T_MISSION_FAILED );
	ScriptManager:wait( 4 );
	HideDialogue();
	endGame();
end


-- END Defeated


function endGame()
	ScriptManager:endGame()
end