require "lua/Constants"
require "lua/Functions"
require "lua/Functions_2"
require "lua/Text"
require "lua/Units"
require "math"


-- mission stuff
missionIds = {
		captureRefinery	= 1,
		destroyBunkers	= 2,
		captureDepot	= 3,
		captureAirfield	= 4,
};

-- missionId, primary( boolean ), textkey, xpReward
missions = {	
	{ missionIds.captureRefinery, 	true,	Text.T_MAP_DE_4_3_MISSION_PRIMARY_01,		175 },
	{ missionIds.destroyBunkers, 	false,	Text.T_MAP_DE_4_3_MISSION_SECONDARY_01,		50 },
	{ missionIds.captureDepot, 		false,	Text.T_MAP_DE_4_3_MISSION_SECONDARY_02,		50 },
	{ missionIds.captureAirfield,	false,	Text.T_MAP_DE_4_3_MISSION_SECONDARY_03,		50 },
};

enemySquadIds = {

refinery		= 1,
airfield		= 2,
northeast		= 3,
southMiddle		= 4,
enemyDepot		= 5,
southLeft		= 6,
woodsSouth		= 7, 
woodsNorth		= 8,
nearDepotSouth	= 9,
nearDepotNorth	= 10,
planes			= 11,
bunkers			= 12,
mgNests			= 13,
depotGuardTank	= 14,

};


refineryIds 		= { 10, 11, 12, 14, 18, 23, 25, 26, };
airfieldIds 		= { 24, 61, };
northeastIds		= { 13, 41, 42, 55, };
southMiddleIds		= { 16, 17, 34, 40, };
enemyDepotIds		= { 22, 35, 36, 37, 38,}
southLeftIds		= { 31, 32, 33, 39,  };
woodsSouthIds		= { 29, 30, 58, };
woodsNorthIds		= { 27, 28, 44, 45, 56, };
nearDepotSouthIds	= { 43, 57, };
nearDepotNorthIds	= { 62, 63, };
planeIds 			= { 60 };
bunkerIds 			= { 1, 3, 4, 7, 59,  };
mgNestIds 			= { 2, 5, 9, 19, 20, };

depotGuardTank		= 21;

bunkersToDestroy = 5;


depotCutsceneTriggerTiles = {
	{ 14, 6 };
	{ 14, 5 };
	{ 14, 4 };
	{ 13, 3 };
	{ 13, 2 };
	{ 12, 1 };
};




-- waves that spawn

-- I spawn either one destroyer or one Bomber depending on the number of spawns and the round count
-- ( every x turns I spawn a plane wave (Destroyer) and every y turns the bomber spawns instead of fighters )
spawnPlaneWaves				=  0; -- 0 == false; 1 == true; can't save booleans so I have to use numbers
planeWaveRespawnInterval	=  5; -- defines how often waves will spawn
bomberAppearance			=  2; -- every bomberAppearance-spawn a bomber appears instead of a Destroyer
numberOfPlaneWavesSpawned	=  0; -- the number of plane waves that have already spawned in this game
lastPlaneWaveSpawnedInRound	= -3; -- indicates the round in which the last wave spawned 
planeWaveUnits = {
--	playerId,		 templateID, 			tileX, 		tileY,		squadID			AiState
	{ 1,			Units.UNIT_SU_FIGHTER,		29,			3,		  	11,			AiState.ATTACK_AGGRESSIVE },
	{ 1,			Units.UNIT_SU_BOMBER,		29,			3,			11,			AiState.ATTACK_AGGRESSIVE },		
}


-- BEGIN Init


function onGameStart()

	CreateMissions( missions );
	setTriggers();
	storeValues();
	
	ScriptManager:gainGold( 0, 2100 );
	
	-- start intro cutscene
	ScriptManager:sleep( 0, "startIntroCutscene" ); -- without sleep player turn popup shows
end


function storeValues()
	ScriptManager:storeValue( "bunkersToDestroy", bunkersToDestroy);
	ScriptManager:storeValue( "spawnPlaneWaves", spawnPlaneWaves );
	ScriptManager:storeValue( "numberOfPlaneWavesSpawned",		numberOfPlaneWavesSpawned );
	ScriptManager:storeValue( "lastPlaneWaveSpawnedInRound",	lastPlaneWaveSpawnedInRound );
end


function setTriggers()
	-- load all needed values after loading
	ScriptManager:addOnGameResumeTrigger( "loadValues", "loadValues", false );
	
	-- check if the player has units left
	ScriptManager:addOnUnitDestroyedTrigger( 0, -1, -1, "checkForEnd", "checkForEnd" , false );
	

	ScriptManager:addOnUnitDestroyedTrigger( 1, Units.UNIT_SU_REFINERY, -1, "onRefineryDestroyed", "onRefineryDestroyed", false );
	
	ScriptManager:addOnUnitCapturedTrigger( 0, Units.UNIT_SU_HQ, -1, "onDepotCaptured", "onDepotCaptured", false );
	ScriptManager:addOnUnitDestroyedTrigger( 1, Units.UNIT_SU_HQ, -1, "onDepotDestroyed", "onDepotDestroyed", false );
	
	ScriptManager:addOnUnitCapturedTrigger( 0, Units.UNIT_SU_AIRFIELD, -1, "onAirfieldCaptured", "onAirfieldCaptured", false );
	ScriptManager:addOnUnitDestroyedTrigger( 1, Units.UNIT_SU_AIRFIELD, -1, "onAirfieldDestroyed","onAirfieldDestroyed", false );
	
	ScriptManager:addOnUnitSightedTrigger( 1, Units.UNIT_SU_BUNKER, -1, true, "onBunkerSighted", "onBunkerSighted", false );
	
	ScriptManager:addOnUnitDestroyedTrigger( 1, Units.UNIT_SU_BUNKER, -1, "onBunkerDestroyed", "onBunkerDestroyed", false );
	
	-- when the enemy plane has attacked for the first time
	ScriptManager:addOnUnitHasAttackedTrigger( -1, Units.UNIT_SU_FIGHTER, -1, "onFighterHasAttacked", "onFighterHasAttacked", false );
	
	-- triggers the depot cutscene
	for i = 1, #depotCutsceneTriggerTiles do
	ScriptManager:addOnEnterTileTrigger( depotCutsceneTriggerTiles[i][1], depotCutsceneTriggerTiles[i][2], 0, -1, -1, true, "startDepotCutscene", "startDepotCutscene", false );
	end	
			
end

function loadValues()
	bunkersToDestroy = ScriptManager:getValue( "bunkersToDestroy" );
	spawnPlaneWaves	 = ScriptManager:getValue( "spawnPlaneWaves" );
	numberOfPlaneWavesSpawned		= ScriptManager:getValue( "numberOfPlaneWavesSpawned" );
	lastPlaneWaveSpawnedInRound		= ScriptManager:getValue( "lastPlaneWaveSpawnedInRound" );
	
end


-- END Init
-- BEGIN Intro Cutscene


function startIntroCutscene()
	
	StartCutscene();
	
	ScriptManager:moveCameraEx( 2, 10, 0.5, 2, true );
	ScriptManager:wait( 1 );
	
	local Truck = ScriptManager:spawnUnit( 0, Units.UNIT_DE_TRANSPORT_TRUCK, 1, 5, -1, AiState.WAIT );
	
	ScriptManager:moveUnitEx( Truck:getId(), 1, 12, true );
	ScriptManager:wait( 1 );
	
	ScriptManager:spawnUnit( 0, Units.UNIT_DE_MP40, 1, 13, -1, AiState.WAIT );
	ScriptManager:spawnUnit( 0, Units.UNIT_DE_PANZERFAUST, 1, 11, -1, AiState.WAIT );
	ScriptManager:wait( 2 );
	ScriptManager:displaySubtitle( Text.T_MAP_DE_4_3_MISSION_INTRO_FLAVOR_01 );
	ScriptManager:wait( 5 );
	ScriptManager:dismissSubtitle();
	
	ScriptManager:moveCameraEx( 30, 12, 1, 4, true)
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE_4_3_MISSION_INTRO_FLAVOR_02 );
	ScriptManager:revealFogOfWar( 30, 12, 1, true );
	ScriptManager:showMarkerAt( 30, 12 );
	
	ScriptManager:wait( 5 );
	ScriptManager:dismissSubtitle();
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE_4_3_MISSION_INTRO_FLAVOR_03 );

	local truckOne = ScriptManager:spawnUnit( 1, Units.UNIT_SU_SUPPLY_TRUCK, 29, 13, -1, AiState.WAIT );
	ScriptManager:moveUnitEx( truckOne:getId(), 29, 11, true );
	ScriptManager:wait( 2 );
	ScriptManager:moveUnitEx( truckOne:getId(), 31, 12, true );
	ScriptManager:removeUnitEx( truckOne:getId(), true );
	
	ScriptManager:wait( 2 );
	
	local truckTwo = ScriptManager:spawnUnit( 1, Units.UNIT_SU_SUPPLY_TRUCK, 31, 12, -1, AiState.WAIT );
	ScriptManager:moveUnitEx( truckTwo:getId(), 29, 11, true );
	ScriptManager:wait( 2 );
	ScriptManager:moveUnitEx( truckTwo:getId(), 29, 14, true );
	ScriptManager:removeUnitEx( truckTwo:getId(), true );
	
	ScriptManager:dismissSubtitle();
	
	ScriptManager:moveCameraEx( 28, 12, 0.5, 2, true );
	ScriptManager:displaySubtitle( Text.T_MAP_DE_4_3_MISSION_INTRO_FLAVOR_04 );
	ScriptManager:wait( 6 );
	ScriptManager:dismissSubtitle();
	
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE_4_3_MISSION_INTRO_FLAVOR_05 );
	ScriptManager:moveCameraEx( 23, 18, 0.7, 2, true );
	ScriptManager:wait( 1 );
	
	ScriptManager:moveCameraEx( 20, 4, 0.7, 2, true );
	ScriptManager:wait( 1 );
	
	ScriptManager:moveCameraEx( 18, 12, 0.7, 1, true );
	ScriptManager:wait( 1 )

	ScriptManager:moveCameraEx( 2, 12, 0.7, 3, true );
	ScriptManager:dismissSubtitle();

	ScriptManager:displaySubtitle( Text.T_MAP_DE_4_3_MISSION_INTRO_FLAVOR_06 );
	ScriptManager:wait( 5 );
	ScriptManager:dismissSubtitle();
	
	ScriptManager:removeAllMarkers();
	ScriptManager:revealFogOfWar( 30, 12, 1, false );
	
	EndCutscene();

	startTheMission();
end

function startTheMission()
	ScriptManager:setMissionStatus( missionIds.captureRefinery, MissionStatus.IN_PROGRESS );
	
	--TODO: remove this function!
	--testing();
	--removeSquads();
	--ScriptManager:addOnUnitSelectedTrigger( 1, -1, -1, "showParameters", "showParameters",false)
	
	
	
	ScriptManager:setTargetTile( 1, enemySquadIds.planes, 2, 12 ); 
	
	ScriptManager:fortifyUnit( 21 );
end

-- END Intro Cutscene
-- BEGIN

function testing()
	ScriptManager:spawnUnit( 0, Units.GOTT, 1, 2, 42, AiState.WAIT );
	ScriptManager:spawnUnit( 0, Units.GOTT, 2, 2, 42, AiState.WAIT );
	
end

function removeSquads()

	for i = 1, #refineryIds do
	ScriptManager:removeUnitEx( refineryIds[i], false );
	end

	for i = 1, #airfieldIds do
	ScriptManager:removeUnitEx( airfieldIds[i], false );
	end
	
	for i = 1, # northeastIds do
	ScriptManager:removeUnitEx( northeastIds[i], false );
	end
	
	for i = 1, #southMiddleIds do
	ScriptManager:removeUnitEx( southMiddleIds[i], false );
	end
	
	for i = 1, #enemyDepotIds do
	ScriptManager:removeUnitEx( enemyDepotIds[i], false );
	end
	
	for i = 1, #southLeftIds do
	ScriptManager:removeUnitEx( southLeftIds[i], false );
	end
	
	for i = 1, #woodsSouthIds do
	ScriptManager:removeUnitEx( woodsSouthIds[i], false );
	end
	
	for i = 1, #woodsNorthIds do
	ScriptManager:removeUnitEx( woodsNorthIds[i], false );	
	end
	
	for i = 1, #nearDepotSouthIds do
	ScriptManager:removeUnitEx( nearDepotSouthIds[i], false );
	end
	
	for i = 1, #nearDepotNorthIds do
	ScriptManager:removeUnitEx( nearDepotNorthIds[i], false );
	end
	
	for i = 1, #planeIds do
	ScriptManager:removeUnitEx( planeIds[i], false );
	end
	
	for i = 1, #bunkerIds do
	ScriptManager:removeUnitEx( bunkerIds[i], false );
	end
	
	for i = 1,# mgNestIds do
	ScriptManager:removeUnitEx( mgNestIds[i], false );
	end
end



function showParameters( selectedUnit )
	--print( string.format( "[] %s", selectedUnit:get() ) );
end


-- insert missionstuff here

function onBunkerSighted()

	ScriptManager:removeTrigger( "onBunkerSighted" );
	
	StartCutscene();
	ScriptManager:displaySubtitle( Text.T_MAP_DE_4_3_MISSION_SECONDARY_01_FLAVOR_01 );
	ScriptManager:wait( 5 );
	ScriptManager:dismissSubtitle();
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE_4_3_MISSION_SECONDARY_01_FLAVOR_02 );
	ScriptManager:wait( 6 );
	ScriptManager:dismissSubtitle();
	
	EndCutscene();
	
	ScriptManager:setMissionStatus( missionIds.destroyBunkers, MissionStatus.IN_PROGRESS );
end

function onFighterHasAttacked()
	ScriptManager:removeTrigger( "onFighterHasAttacked" );
	ScriptManager:runForPlayer( 0, "playAirfieldCutscene")
end


function playAirfieldCutscene()
	
	ScriptManager:wait( 1 );
	
	ScriptManager:startCutscene()
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE_4_3_MISSION_SECONDARY_03_FLAVOR_01 );
	ScriptManager:wait( 6 );
	ScriptManager:dismissSubtitle();
	
	ScriptManager:moveCameraEx( 30, 2, 0.7, 4, true );
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE_4_3_MISSION_SECONDARY_03_FLAVOR_02 );
	ScriptManager:wait( 7 );
	ScriptManager:dismissSubtitle()
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE_4_3_MISSION_SECONDARY_03_FLAVOR_03 );
	ScriptManager:wait( 5 );
	ScriptManager:dismissSubtitle();
	
	ScriptManager:moveCameraEx( 2, 12, 0.7, 4, true ); 
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE_4_3_MISSION_SECONDARY_03_FLAVOR_04 );
	ScriptManager:wait( 5 );
	ScriptManager:dismissSubtitle();
	
	ScriptManager:stopCutscene();
	
	ScriptManager:wait( 1 );
	
	ScriptManager:setMissionStatus( missionIds.captureAirfield, MissionStatus.IN_PROGRESS );

end


function onBunkerDestroyed()

	ScriptManager:setMissionProgress( missionIds.destroyBunkers, ScriptManager:getMissionProgress(missionIds.destroyBunkers) + 1 );
	ShowMissionProgressPopup (ScriptManager:getMissionProgress( missionIds.destroyBunkers ), Text.T_MAP_DE_4_3_MISSION_SECONDARY_01_COUNTER );

	if ScriptManager:getMissionProgress( missionIds.destroyBunkers ) >= bunkersToDestroy then
		DisablePlayerInteraction();

		ScriptManager:setMissionStatus( missionIds.destroyBunkers, MissionStatus.ACCOMPLISHED );
		ScriptManager:removeTrigger( "onBunkerDestroyed" );
		
		ScriptManager:displaySubtitle( Text.T_MAP_DE_4_3_MISSION_SECONDARY_01_FLAVOR_03 );
		ScriptManager:wait( 5 );
		ScriptManager:dismissSubtitle();
		
		EnablePlayerInteraction();		
	end
end


function startDepotCutscene()

	ScriptManager:removeTrigger( "startDepotCutscene" );

	ScriptManager:startCutscene();
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE_4_3_MISSION_SECONDARY_02_FLAVOR_01 );
	ScriptManager:wait( 7 );
	ScriptManager:dismissSubtitle();
	
	ScriptManager:moveCameraEx( 18, 19, 1, 3, true );	
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE_4_3_MISSION_SECONDARY_02_FLAVOR_02 );
	ScriptManager:wait( 5 );
	ScriptManager:dismissSubtitle();
	
	ScriptManager:moveCameraEx( 13, 3, 0.7, 3, true );
	ScriptManager:displaySubtitle( Text.T_MAP_DE_4_3_MISSION_SECONDARY_02_FLAVOR_03 );
	ScriptManager:wait( 2 );
	ScriptManager:dismissSubtitle();
	
	ScriptManager:stopCutscene();
	
	ScriptManager:setMissionStatus( missionIds.captureDepot, MissionStatus.IN_PROGRESS );

end



function onDepotCaptured()

	ScriptManager:removeTrigger( "onDepotCaptured" );
	ScriptManager:removeTrigger( "onDepotDestroyed" );
	
	ScriptManager:startCutscene();
	
	ScriptManager:moveCameraEx( 18, 19, 0.7, 2, true );
	ScriptManager:displaySubtitle( Text.T_MAP_DE_4_3_MISSION_SECONDARY_02_FLAVOR_04 );
	ScriptManager:wait( 5 );
	ScriptManager:dismissSubtitle();
	 
	ScriptManager:stopCutscene();
	
	ScriptManager:wait( 1 );
	
	ScriptManager:setMissionStatus( missionIds.captureDepot, MissionStatus.ACCOMPLISHED );
	
	spawnPlaneWaves	= 1
	
	ScriptManager:addOnTurnStartTrigger( 1, -1, -1, "startPlaneWavespawn", "startPlaneWavespawn", false );
end


function onDepotDestroyed()
	ScriptManager:removeTrigger( "onDepotDestroyed" );
	ScriptManager:removeTrigger( "onDepotDestroyed" );
	ScriptManager:setMissionStatus( missionIds.captureDepot, MissionStatus.FAILED );
end


function onAirfieldCaptured()

	ScriptManager:removeTrigger( "startPlaneWavespawn" );
	
	ScriptManager:removeTrigger( "onAirfieldCaptured" );
	ScriptManager:removeTrigger( "onAirfieldDestroyed" );
	ScriptManager:setMissionStatus(missionIds.captureAirfield, MissionStatus.ACCOMPLISHED );

	spawnPlaneWaves = 0;
	ScriptManager:storeValue( "spawnPlaneWaves", spawnPlaneWaves );	
end


function onAirfieldDestroyed()

	ScriptManager:removeTrigger( "startPlaneWavespawn" );

	ScriptManager:removeTrigger( "onAirfieldCaptured" );
	ScriptManager:removeTrigger( "onAirfieldDestroyed" );
	ScriptManager:setMissionStatus(missionIds.captureAirfield, MissionStatus.ACCOMPLISHED );
	
	spawnPlaneWaves = 0;
	ScriptManager:storeValue( "spawnPlaneWaves", spawnPlaneWaves );	
end


function onRefineryDestroyed()
	ScriptManager:removeTrigger( "onRefineryDestroyed" );
	
	ScriptManager:setMissionStatus( missionIds.captureRefinery,MissionStatus.ACCOMPLISHED );
	
	ScriptManager:wait( 1 );
	
	startVictoryCutscene();
end



--BEGIN WaveLogic


function startPlaneWavespawn()

	print("beep")

	-- spawn logic for plane waves
	if spawnPlaneWaves == 1 then -- should waves spawn? 1 == yes, 0 == no		
		-- check if a wave spawns this round
		if lastPlaneWaveSpawnedInRound + planeWaveRespawnInterval <= ScriptManager:getCurrentRound() then
			numberOfPlaneWavesSpawned = numberOfPlaneWavesSpawned + 1;
			ScriptManager:storeValue( "numberOfPlaneWavesSpawned", numberOfPlaneWavesSpawned );
			
			-- check if current spawn is a bomber spawn
			if numberOfPlaneWavesSpawned % bomberAppearance == 0 then -- spawn Bomber
				spawnWaveUnit( planeWaveUnits[2] );
				ScriptManager:setTargetTile( 1, enemySquadIds.planes, 2, 12 );
				
			else -- spawn a destroyer
				for i = 1, #planeWaveUnits -1 do --	-1 zieht in der spawning-Tabelle die letzte Einheit ab (d.h., der Bomber wird nicht mitgespawnt)
					spawnWaveUnit( planeWaveUnits[1] );
					ScriptManager:setTargetTile( 1, enemySquadIds.planes, 18, 19 );
				end
			end
			
			lastPlaneWaveSpawnedInRound = ScriptManager:getCurrentRound(); -- reset round count
			ScriptManager:storeValue( "lastPlaneWaveSpawnedInRound",	lastPlaneWaveSpawnedInRound );
		end
	end
end



function spawnWaveUnit( spawnData )
	local unit = ScriptManager:spawnUnit( spawnData[1], spawnData[2], spawnData[3], spawnData[4], spawnData[5], spawnData[6] );
end


-- END
-- BEGIN Victory


function startVictoryCutscene()
	ScriptManager:startCutscene();
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE_4_3_MISSION_OUTRO_FLAVOR_01 );
	ScriptManager:wait( 3 );
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE_4_3_MISSION_OUTRO_FLAVOR_02 );
	ScriptManager:wait( 4 );
	ScriptManager:dismissSubtitle();
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE_4_3_MISSION_OUTRO_FLAVOR_03 );
	ScriptManager:wait( 4 );
	HideDialogue();
	
	ScriptManager:stopCutscene();
	
	victoryEndGame();
end

function victoryEndGame()
	DisablePlayerInteraction();
	HideDialogue(); -- remove possible texts on the screens

	-- accomplish secondary missions that are accomplished when the game is finished with certain condition
	
	
	ScriptManager:displaySubtitle( Text.T_MISSION_VICTORY );
	ScriptManager:wait( 4 );
	HideDialogue();
	endGame();
end


-- END Victory
-- BEGIN Defeated


function checkForEnd()
	if ScriptManager:noUnitsLeft( 0 ) then
		ScriptManager:runForPlayer( 0, "missionFailed" )
	end
end

function missionFailed()
	DisablePlayerInteraction();
	HideDialogue(); -- remove possible texts on the screens
	
	ScriptManager:displaySubtitle( Text.T_MISSION_FAILED );
	ScriptManager:wait( 4 );
	HideDialogue();
	endGame();
end


-- END Defeated


function endGame()
	ScriptManager:endGame()
end