require "lua/Constants"
require "lua/Functions"
require "lua/Functions_2"
require "lua/Text"
require "lua/Units"
require "math"


-- mission stuff
missionIds = {
	defendFactory		= 1,
	defendCity			= 2,
	defendVillage		= 3,
	recaptureAirfield	= 4,
};

-- missionId, primary( boolean ), textkey, xpReward
missions = {
	{ missionIds.defendFactory,		true,	Text.T_MAP_DE5_1_MISSION_PRIMARY_01,	150 },
	{ missionIds.defendCity, 		false,	Text.T_MAP_DE5_1_MISSION_SECONDARY_01,	 50 },
	{ missionIds.defendVillage, 	false,	Text.T_MAP_DE5_1_MISSION_SECONDARY_02,	 75 },
	{ missionIds.recaptureAirfield, false,	Text.T_MAP_DE5_1_MISSION_SECONDARY_03,	100 },
};

lastSpawnRound	= 6;
airfieldId		= 2;

enemySquadIds = {
	firstAttackers		= 1,
	airfieldAttackers	= 2,
	wave				= 3,
	planeWave			= 4,
};

wave2 = {
--  playerId, templateID, tileX, tileY, squadId AiState
	{ 1, Units.UNIT_SU_RIFLE,	1, 21, enemySquadIds.wave, AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_SU_MORTAR,	1, 19, enemySquadIds.wave, AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_SU_AA,		1, 20, enemySquadIds.wave, AiState.ATTACK_AGGRESSIVE },
	
	{ 1, Units.UNIT_SU_RIFLE,	28, 26, enemySquadIds.wave, AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_SU_MORTAR,	27, 27, enemySquadIds.wave, AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_SU_AA,		28, 27, enemySquadIds.wave, AiState.ATTACK_AGGRESSIVE },
	
	{ 1, Units.UNIT_SU_RIFLE,	25, 1, enemySquadIds.wave, AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_SU_MORTAR,	27, 1, enemySquadIds.wave, AiState.ATTACK_AGGRESSIVE },
	
	{ 1, Units.UNIT_SU_RIFLE,	3, 2, enemySquadIds.wave, AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_SU_AA,		3, 1, enemySquadIds.wave, AiState.ATTACK_AGGRESSIVE },
};

wave3 = {
--  playerId, templateID, tileX, tileY, squadId AiState
	{ 1, Units.UNIT_SU_RIFLE,	 1, 21, enemySquadIds.wave, AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_SU_TANK_KV1, 1, 20, enemySquadIds.wave, AiState.ATTACK_AGGRESSIVE },
	
	{ 1, Units.UNIT_SU_RIFLE,	 28, 26, enemySquadIds.wave, AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_SU_TANK_KV1, 28, 27, enemySquadIds.wave, AiState.ATTACK_AGGRESSIVE },
	
	{ 1, Units.UNIT_SU_RIFLE,	 26, 2, enemySquadIds.wave, AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_SU_TANK_KV1, 26, 1, enemySquadIds.wave, AiState.ATTACK_AGGRESSIVE },
	
	{ 1, Units.UNIT_SU_RIFLE,	 3, 2, enemySquadIds.wave, AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_SU_HOWITZER, 3, 1, enemySquadIds.wave, AiState.ATTACK_AGGRESSIVE },
};

wave4 = {
--  playerId, templateID, tileX, tileY, squadId AiState
	{ 1, Units.UNIT_SU_RIFLE,	 1, 21, enemySquadIds.wave, AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_SU_HOWITZER, 1, 20, enemySquadIds.wave, AiState.ATTACK_AGGRESSIVE },
	
	{ 1, Units.UNIT_SU_RIFLE,	 28, 26, enemySquadIds.wave, AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_SU_HOWITZER, 28, 27, enemySquadIds.wave, AiState.ATTACK_AGGRESSIVE },
	
	{ 1, Units.UNIT_SU_RIFLE,	 26, 2, enemySquadIds.wave, AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_SU_HOWITZER, 26, 1, enemySquadIds.wave, AiState.ATTACK_AGGRESSIVE },
	
	{ 1, Units.UNIT_SU_RIFLE,	 3, 2, enemySquadIds.wave, AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_SU_TANK_KV1, 3, 1, enemySquadIds.wave, AiState.ATTACK_AGGRESSIVE },
};

wave5 = {
--  playerId, templateID, tileX, tileY, squadId AiState
	{ 1, Units.UNIT_SU_RIFLE,			1, 21, enemySquadIds.wave, AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_SU_MOBILE_AA,		1, 20, enemySquadIds.wave, AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_SU_TANK_DESTROYER,	2, 20, enemySquadIds.wave, AiState.ATTACK_AGGRESSIVE },
	
	{ 1, Units.UNIT_SU_RIFLE,			27, 27, enemySquadIds.wave, AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_SU_MOBILE_AA,		28, 27, enemySquadIds.wave, AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_SU_TANK_DESTROYER,	28, 26, enemySquadIds.wave, AiState.ATTACK_AGGRESSIVE },
	
	{ 1, Units.UNIT_SU_RIFLE,			25, 1, enemySquadIds.wave, AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_SU_MOBILE_AA,		26, 1, enemySquadIds.wave, AiState.ATTACK_AGGRESSIVE },
	
	{ 1, Units.UNIT_SU_RIFLE,			2, 1, enemySquadIds.wave, AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_SU_TANK_DESTROYER,	3, 2, enemySquadIds.wave, AiState.ATTACK_AGGRESSIVE },
};

wave6 = {
--  playerId, templateID, tileX, tileY, squadId AiState
	{ 1, Units.UNIT_SU_RIFLE,	 1, 21, enemySquadIds.wave, AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_SU_TANK_T34, 1, 20, enemySquadIds.wave, AiState.ATTACK_AGGRESSIVE },
	
	{ 1, Units.UNIT_SU_RIFLE,	 28, 26, enemySquadIds.wave, AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_SU_TANK_T34, 28, 27, enemySquadIds.wave, AiState.ATTACK_AGGRESSIVE },
	
	{ 1, Units.UNIT_SU_RIFLE,	 26, 2, enemySquadIds.wave, AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_SU_TANK_T34, 26, 1, enemySquadIds.wave, AiState.ATTACK_AGGRESSIVE },
	
	{ 1, Units.UNIT_SU_RIFLE,	 3, 2, enemySquadIds.wave, AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_SU_HOWITZER, 3, 1, enemySquadIds.wave, AiState.ATTACK_AGGRESSIVE },
};

wave7 = {
--  playerId, templateID, tileX, tileY, squadId AiState
	{ 1, Units.UNIT_SU_RIFLE,			1, 21, enemySquadIds.wave, AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_SU_KATYUSHA,		1, 20, enemySquadIds.wave, AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_SU_ANTI_INFANTRY,	2, 20, enemySquadIds.wave, AiState.ATTACK_AGGRESSIVE },
	
	{ 1, Units.UNIT_SU_RIFLE,			27, 27, enemySquadIds.wave, AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_SU_KATYUSHA,		28, 27, enemySquadIds.wave, AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_SU_ANTI_INFANTRY,	28, 26, enemySquadIds.wave, AiState.ATTACK_AGGRESSIVE },
	
	{ 1, Units.UNIT_SU_RIFLE,			25, 1, enemySquadIds.wave, AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_SU_ANTI_INFANTRY,	26, 2, enemySquadIds.wave, AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_SU_AA,				27, 1, enemySquadIds.wave, AiState.ATTACK_AGGRESSIVE },
	
	{ 1, Units.UNIT_SU_RIFLE,	 3, 2, enemySquadIds.wave, AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_SU_KATYUSHA, 3, 1, enemySquadIds.wave, AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_SU_AA,		 2, 1, enemySquadIds.wave, AiState.ATTACK_AGGRESSIVE },
};


-- BEGIN Init


function onGameStart()
	CreateMissions( missions );
	setTriggers();
	
	--Target tile for enemies
	ScriptManager:setTargetTile( 1, enemySquadIds.wave, 33, 21 );
	
	ScriptManager:gainGold( 0, 3000 );
	ScriptManager:removeReinforcementsFromBase( airfieldId,Units.UNIT_DE_BOMBER );
	ScriptManager:removeReinforcementsFromBase( airfieldId,Units.UNIT_DE_FIGHTER );
	ScriptManager:removeReinforcementsFromBase( airfieldId,Units.UNIT_DE_DESTROYER );
	
	-- start intro cutscene
	ScriptManager:sleep( 0, "startIntroCutscene" ); -- without sleep player turn popup shows
end

function setTriggers()
	-- check if the player has units left
	ScriptManager:addOnUnitDestroyedTrigger( 0, -1, -1, "checkForEnd", "checkForEnd" , false );
	
	-- Start attackedAirfieldCutscene
	ScriptManager:addOnTurnStartTrigger( 0, -1, 1, "attackedAirfieldCutscene", "attackedAirfieldCutscene", false );
	
	-- Countdown counter
	ScriptManager:addOnUnitDestroyedTrigger( 1, -1, -1, "onEnemyDestroyed", "onEnemyDestroyed", false );
	
	-- defendFactory failed
	ScriptManager:addOnEnterTileTrigger( 10, 22, 1, -1, -1, true, "lostFactory", "lostFactory", false );
	ScriptManager:addOnEnterTileTrigger(  9, 23, 1, -1, -1, true, "lostFactory", "lostFactory", false );
	ScriptManager:addOnEnterTileTrigger( 10, 24, 1, -1, -1, true, "lostFactory", "lostFactory", false );
	ScriptManager:addOnEnterTileTrigger( 11, 24, 1, -1, -1, true, "lostFactory", "lostFactory", false );
	ScriptManager:addOnEnterTileTrigger( 11, 23, 1, -1, -1, true, "lostFactory", "lostFactory", false );
	ScriptManager:addOnEnterTileTrigger( 11, 22, 1, -1, -1, true, "lostFactory", "lostFactory", false );
	
	-- defendCity failed
	ScriptManager:addOnUnitDestroyedTrigger( 0, Units.UNIT_DE_HQ, 1, "lostCity", "lostCity", false );
	
	-- defendVillage failed
	ScriptManager:addOnUnitDestroyedTrigger( 0, Units.UNIT_DE_HQ, 0, "lostVillage", "lostVillage", false );
	
	-- Airfield recaptured
	ScriptManager:addOnUnitCapturedTrigger( 0, Units.UNIT_DE_AIRFIELD, -1, "airfieldRecaptured", "airfieldRecaptured", false );
	
	-- Spawn enemy troops
	ScriptManager:addOnTurnStartTrigger( 1, -1, -1, "onEnemyTurnStart", "onEnemyTurnStart", false );
end


-- END Init
-- BEGIN Intro Cutscene


function startIntroCutscene()
	StartCutscene();
	
	-- Primary Objective defendFactory 
	ScriptManager:displaySubtitle( Text.T_MAP_DE5_1_MISSION_PRIMARY_01_FLAVOR_01 );
	ScriptManager:wait( 10 );
	HideDialogue();
	ScriptManager:wait( 1 );
	
	ScriptManager:moveCameraEx( 19, 15, 0.7, 3, true );
	ScriptManager:wait( 1 );
	
	-- Secondary Objective defendCity
	ScriptManager:displaySubtitle( Text.T_MAP_DE5_1_MISSION_SECONDARY_01_FLAVOR_01 );
	ScriptManager:showMarkerAt( 19, 15 );
	ScriptManager:wait( 5 );
	ScriptManager:hideMarkerAt( 19, 15 );
	HideDialogue();
	ScriptManager:wait( 1 );
	
	ScriptManager:moveCameraEx( 11, 10, 0.7, 5, true );
	ScriptManager:wait( 1 );
	
	-- Secondary Objective defendVillage
	ScriptManager:displaySubtitle( Text.T_MAP_DE5_1_MISSION_SECONDARY_02_FLAVOR_01 );
	ScriptManager:showMarkerAt( 11, 10 );
	ScriptManager:wait( 5 );
	ScriptManager:hideMarkerAt( 11, 10 );
	HideDialogue();
	ScriptManager:wait( 1 );
	
	-- Enemies approach
	ScriptManager:moveCameraEx( 24, 4, 1, 5, true );
	ScriptManager:displaySubtitle( Text.T_MAP_DE5_1_MISSION_PRIMARY_01_FLAVOR_02 );
	
	local unit2 = ScriptManager:spawnUnit( 1, Units.UNIT_SU_TANK_T34, 26, 1, 1, AiState.ATTACK_AGGRESSIVE );
	local unit2Id = unit2:getId();
	ScriptManager:moveUnitEx(	unit2Id, 24, 3, true );
	ScriptManager:attackUnitEx(	unit2Id, 29,	true );
	
	local unit3 = ScriptManager:spawnUnit( 1, Units.UNIT_SU_FIGHTER, 28, 1, 1, AiState.AIRPLANE );
	local unit3Id = unit3:getId();
	ScriptManager:moveUnitEx(	unit3Id, 25, 3, true );
	ScriptManager:attackUnitEx(	unit3Id, 29,	true );
	
	local unit1 = ScriptManager:spawnUnit( 1, Units.UNIT_SU_RIFLE, 26, 1, 1, AiState.ATTACK_AGGRESSIVE );
	local unit1Id = unit1:getId();
	ScriptManager:moveUnitEx(	unit1Id, 23, 3, true );
	ScriptManager:attackUnitEx(	unit1Id, 29,	true );
	
	HideDialogue();
	ScriptManager:wait( 1 );
	
	ScriptManager:setMissionProgress( missionIds.defendFactory, ScriptManager:getMissionProgress( missionIds.defendFactory ) + 3 );
	
	-- Warplane hint
	ScriptManager:moveCameraEx( 13, 4, 0.7, 5, true );
	ScriptManager:displaySubtitle( Text.T_MAP_DE5_1_MISSION_PRIMARY_01_FLAVOR_03 );
	
	ScriptManager:showMarkerAt( 12, 4 );
	ScriptManager:showMarkerAt( 12, 3 );
	ScriptManager:showMarkerAt( 13, 3 );
	ScriptManager:wait( 5 );
	ScriptManager:hideMarkerAt( 13, 3 );
	ScriptManager:hideMarkerAt( 12, 3 );
	ScriptManager:hideMarkerAt( 12, 4 );
	HideDialogue();
	ScriptManager:wait( 1 );
	
	EndCutscene();
	
	startTheMission();
end

function startTheMission()
	ScriptManager:setMissionStatus( missionIds.defendFactory,	MissionStatus.IN_PROGRESS );
	ScriptManager:setMissionStatus( missionIds.defendCity,		MissionStatus.IN_PROGRESS );
	ScriptManager:setMissionStatus( missionIds.defendVillage,	MissionStatus.IN_PROGRESS );
	
	ScriptManager:setMissionProgress( missionIds.defendFactory, 9 );
	
	--TODO: remove this function!
	--removeSquads();
	--ScriptManager:addOnUnitSelectedTrigger( 1, -1, -1, "showParameters", "showParameters",false)
end

function removeSquads()
	--for i = 1, # do
	--	ScriptManager:removeUnitEx( [i], false );
	--end
end

function showParameters( selectedUnit )
	--print( string.format( "[] %s", selectedUnit:get() ) );
end


-- END Intro Cutscene
-- BEGIN Game Stuff


function recaptureAirfield()
	ScriptManager:setMissionStatus( missionIds.recaptureAirfield, MissionStatus.IN_PROGRESS );
end

function attackedAirfieldCutscene()
	ScriptManager:removeTrigger( "attackedAirfieldCutscene" );
	
	StartCutscene();
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE5_1_MISSION_PRIMARY_01_FLAVOR_04 );
	ScriptManager:moveCameraEx( 6, 17, 1,5, true );
	
	local unit, unitId;
	
	--Kill all enemies in area
	for x = 9, 15 do 
		for y = 1, 5 do
			unit = Unit:getUnitAt( x, y );
			
			if unit ~= nil and unit:getTemplateId()~=Units.UNIT_DE_AIRFIELD then 
				ScriptManager:removeUnitEx( unit:getId(), true );
			end
		end
	end
	
	HideDialogue();
	ScriptManager:displaySubtitle( Text.T_MAP_DE5_1_MISSION_SECONDARY_03_FLAVOR_01 );
	
	unit = ScriptManager:spawnUnit( 1, Units.UNIT_SU_MORTAR, 14, 1, 2, AiState.ATTACK_AGGRESSIVE );
	unitId = unit:getId();
	ScriptManager:moveUnitEx( unitId, 13, 3, false );
	
	unit = ScriptManager:spawnUnit( 1, Units.UNIT_SU_MORTAR, 14, 1, 2, AiState.ATTACK_AGGRESSIVE );
	unitId = unit:getId();
	ScriptManager:moveUnitEx( unitId, 12, 3, false );
	
	unit = ScriptManager:spawnUnit( 1, Units.UNIT_SU_TANK_KV1, 11, 1, 2, AiState.ATTACK_AGGRESSIVE );
	unitId = unit:getId();
	ScriptManager:moveUnitEx( unitId, 12, 4, false );
	
	unit = ScriptManager:spawnUnit( 1, Units.UNIT_SU_AA, 11, 1, 2, AiState.ATTACK_AGGRESSIVE );
	unitId = unit:getId();
	ScriptManager:moveUnitEx( unitId, 13, 2, false );
	
	unit = ScriptManager:spawnUnit( 1, Units.UNIT_SU_FIGHTER, 9, 1, 2, AiState.AIRPLANE );
	unitId = unit:getId();
	ScriptManager:moveUnitEx( unitId, 12, 5, false );
	
	unit = ScriptManager:spawnUnit( 1, Units.UNIT_SU_RIFLE, 14, 4, 2, AiState.ATTACK_AGGRESSIVE );
	unitId = unit:getId();
	ScriptManager:captureUnitEx( unitId, airfieldId, true );
	
	ScriptManager:moveCameraEx( 13, 4, 0.7, 5, true );
	HideDialogue();
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE5_1_MISSION_SECONDARY_03_FLAVOR_02 );
	ScriptManager:wait( 3 );
	HideDialogue();
	
	ScriptManager:setMissionProgress( missionIds.defendFactory, ScriptManager:getMissionProgress( missionIds.defendFactory ) + 6 );
	
	EndCutscene();
	
	recaptureAirfield();
end

function onEnemyTurnStart()
	local currentRound = ScriptManager:getCurrentRound();
	if currentRound == 1 then
		for i = 1, #wave2 do
			spawnWave( wave2[i] );
		end
	
	elseif currentRound == 2 then
		for i = 1, #wave3 do
			spawnWave( wave3[i] );
		end
	
	elseif currentRound == 3 then
		for i = 1, #wave4 do
			spawnWave( wave4[i] );
		end
	
	elseif currentRound == 4 then
		for i = 1, #wave5 do
			spawnWave( wave5[i] );
		end
	
	elseif currentRound == 5 then
		for i = 1, #wave6 do
			spawnWave( wave6[i] );
		end
	
	elseif currentRound == lastSpawnRound then
		for i = 1, #wave7 do
			spawnWave( wave7[i] );
		end
		ScriptManager:runForPlayer( 0, "lastWavePopup" );
	end
	
	local airfield = Unit:getUnitById( airfieldId )
	if airfield ~= nil then
		local playerIdAirfield = airfield:getPlayerId();
		--print( string.format( "currentRound %i , lastSpawnRound: %i", currentRound, lastSpawnRound ) );
		--print( string.format( "airfield player %i", playerIdAirfield ) );
		--print( Unit:getUnitById( airfieldId ) );
		if currentRound <= lastSpawnRound and playerIdAirfield == 1 then
			ScriptManager:spawnUnit( 1, Units.UNIT_SU_FIGHTER, 12, 3, enemySquadIds.planeWave, AiState.AIRPLANE );
			ScriptManager:setMissionProgress( missionIds.defendFactory, ScriptManager:getMissionProgress( missionIds.defendFactory ) + 1 );
		end
	end
end

function lastWavePopup() 
	ScriptManager:wait( 1 );
	StartCutscene();
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE5_1_MISSION_PRIMARY_01_FLAVOR_05 );
	ScriptManager:wait( 5 );
	HideDialogue();
	
	EndCutscene();
end

function spawnWave( spawnData )
	ScriptManager:spawnUnit( spawnData[1], spawnData[2], spawnData[3], spawnData[4],spawnData[5], spawnData[6] );
	ScriptManager:setMissionProgress( missionIds.defendFactory, ScriptManager:getMissionProgress( missionIds.defendFactory ) + 1 );
	--print( ScriptManager:getMissionProgress( missionIds.defendFactory ) );
end

function onEnemyDestroyed( destroyedUnit )
	local 	squadId = destroyedUnit:getSquadId();
	if  squadId == enemySquadIds.wave or
		squadId == enemySquadIds.airfieldAttackers or
		squadId == enemySquadIds.firstAttackers or
		squadId == enemySquadIds.planeWave then
		ScriptManager:setMissionProgress( missionIds.defendFactory, ScriptManager:getMissionProgress( missionIds.defendFactory ) - 1 );
		--print( ScriptManager:getMissionProgress( missionIds.defendFactory ) );
		--print( ScriptManager:getCurrentRound() );
		if ScriptManager:getCurrentRound() >= lastSpawnRound+1 then
			ScriptManager:runForPlayer( 0 , "remainingEnemiesPopup" );
		end
	end
end

--AA destroyed! x/5
function remainingEnemiesPopup()
	if ScriptManager:getMissionProgress(missionIds.defendFactory) > 0 then
		ScriptManager:preparePopupText();
		ScriptManager:addPopupTextParameter( "x", ScriptManager:getMissionProgress( missionIds.defendFactory ) );
		ScriptManager:displaySubtitle( Text.T_MAP_DE5_1_MISSION_PRIMARY_COUNTER );
		ScriptManager:wait( 3 );
		HideDialogue();
	
	else
		startVictoryCutscene();
	end
end

--Airfield is recaptured
function airfieldRecaptured()
	DisablePlayerInteraction();
	
	ScriptManager:setMissionStatus( missionIds.recaptureAirfield, MissionStatus.ACCOMPLISHED );
	ScriptManager:wait( 2 );
	ScriptManager:addReinforcementsToBase( airfieldId, Units.UNIT_DE_BOMBER );
	ScriptManager:addReinforcementsToBase( airfieldId, Units.UNIT_DE_FIGHTER );
	ScriptManager:addReinforcementsToBase( airfieldId, Units.UNIT_DE_DESTROYER );
	
	EnablePlayerInteraction();
end

--Factory is lost
function lostFactory( tileX, tileY, playerId, templateId )
	if templateId ~= Units.UNIT_SU_FIGHTER then
		ScriptManager:runForPlayer( 0, "lostFactory2" );
	end
end

function lostFactory2()
	DisablePlayerInteraction();
	StartCutscene();
	
	ScriptManager:moveCameraEx( 10, 23, 0.7, 1, true );
	ScriptManager:displaySubtitle( Text.T_MAP_DE5_1_MISSION_PRIMARY_01_FLAVOR_06 );
	ScriptManager:wait( 5 );
	ScriptManager:setMissionStatus( missionIds.defendFactory, MissionStatus.FAILED );
	HideDialogue();
	
	EndCutscene();
	EnablePlayerInteraction();
	
	missionFailed();
end

--City is lost
function lostCity()
	ScriptManager:runForPlayer( 0, "lostCity2" );
end

function lostCity2()
	DisablePlayerInteraction();
	
	ScriptManager:setMissionStatus( missionIds.defendCity, MissionStatus.FAILED );
	ScriptManager:wait( 2 );
	
	EnablePlayerInteraction();
	StartCutscene();
	
	ScriptManager:moveCameraEx( 19, 15, 0.7, 3, true );
	ScriptManager:displaySubtitle( Text.T_MAP_DE5_1_MISSION_SECONDARY_01_FLAVOR_02 );
	ScriptManager:wait( 5 );
	HideDialogue();
	
	EndCutscene();
end

--Village is lost 
function lostVillage()
	ScriptManager:runForPlayer( 0, "lostVillage2" );
end
	
function lostVillage2()
	DisablePlayerInteraction();
	
	ScriptManager:setMissionStatus( missionIds.defendVillage, MissionStatus.FAILED );
	ScriptManager:wait( 2 );
	
	EnablePlayerInteraction();
	StartCutscene();
	
	ScriptManager:moveCameraEx( 11, 10, 0.7, 3, true );
	ScriptManager:displaySubtitle( Text.T_MAP_DE5_1_MISSION_SECONDARY_02_FLAVOR_02 );
	ScriptManager:wait( 5 );
	HideDialogue();
	
	EndCutscene();
end


-- END Game Stuff
-- BEGIN Victory


function startVictoryCutscene()
	DisablePlayerInteraction();
	
	ScriptManager:setMissionStatus( missionIds.defendFactory, MissionStatus.ACCOMPLISHED );
	ScriptManager:wait( 2 );
	
	EnablePlayerInteraction();
	StartCutscene();
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE5_1_MISSION_PRIMARY_01_FLAVOR_07 );
	ScriptManager:wait( 13 );
	HideDialogue();
	
	EndCutscene();
	
	if	ScriptManager:getMissionStatus( missionIds.defendVillage ) ~= MissionStatus.FAILED then 
		ScriptManager:setMissionStatus( missionIds.defendVillage, MissionStatus.ACCOMPLISHED );
	end
	
	if	ScriptManager:getMissionStatus( missionIds.defendCity ) ~= MissionStatus.FAILED then 
		ScriptManager:setMissionStatus( missionIds.defendCity, MissionStatus.ACCOMPLISHED );
	end
	
	victoryEndGame();
end

function victoryEndGame()
	DisablePlayerInteraction();
	HideDialogue(); -- remove possible texts on the screens
	
	-- accomplish secondary missions that are accomplished when the game is finished with certain condition
	
	ScriptManager:displaySubtitle( Text.T_MISSION_VICTORY );
	ScriptManager:wait( 4 );
	HideDialogue();
	
	endGame();
end


-- END Victory
-- BEGIN Defeated


function checkForEnd()
	if ScriptManager:noUnitsLeft( 0 ) then
		ScriptManager:runForPlayer( 0, "missionFailed" );
	end
end

function missionFailed()
	DisablePlayerInteraction();
	
	HideDialogue(); -- removes possible texts on the screens
	
	ScriptManager:displaySubtitle( Text.T_MISSION_FAILED );
	ScriptManager:wait( 4 );
	HideDialogue();
	
	endGame();
end


-- END Defeated


function endGame()
	ScriptManager:endGame();
end