require "lua/Constants"
require "lua/Functions"
require "lua/Functions_2"
require "lua/Text"
require "lua/Units"
require "math"

-- mission stuff
missionIds = {
	destroyTransporters	= 1,
	defendHq	= 2
};

-- missionId, primary( boolean ), textkey, xpReward
missions = {
	{ missionIds.destroyTransporters,	true,	Text.T_MAP_DE5_2_MISSION_PRIMARY_01,	150 },
	{ missionIds.defendHq, 				false,	Text.T_MAP_DE5_2_MISSION_SECONDARY_02,	 50 },
};

enemySquadIds = {
	wave1		=  1,
	wave2		=  2,
	wave3		=  3,
	wave4		=  4,
	wave5		=  5,
	wave6		=  6,
	wave7		=  7,
	wave8		=  8,
	air			=  9,
	ambushTroop	= 10,
};

firstPoint	= {	10,	15	};
secondPoint = {	16,	 8 };
thirdPoint	= { { 23, 15 }, { 24,  4 } };
fourthPoint	= { 29,  8 };
finalPoint	= {	33,	21 };

enemyTransporters			= 10;
pointIndicator				= { 0, 0, 0, 0, 0, 0, 0, 0 };
advanceGuardSpawnRound		= 1;
firstTransporterSpawnRound 	= 2;
lastTransporterSpawnRound 	= 7;

wave1 = {
--  initial air attackers
	{ 1, Units.UNIT_SU_FIGHTER,	1, 	23, enemySquadIds.wave1, AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_SU_FIGHTER,	31, 23, enemySquadIds.wave1, AiState.ATTACK_AGGRESSIVE },
};

wave2 = {
--  air attackers
	{ 1, Units.UNIT_SU_FIGHTER,	 1, 23, enemySquadIds.air, AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_SU_BOMBER,	31,  1, enemySquadIds.air, AiState.ATTACK_AGGRESSIVE },
};

wave3 = {
--  escort
	{ 1, Units.UNIT_SU_MORTAR,	 1, 8, enemySquadIds.wave3, AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_SU_RIFLE,	 1, 5, enemySquadIds.wave3, AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_SU_RIFLE,	 2, 6, enemySquadIds.wave3, AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_SU_TANK_T34, 2, 8, enemySquadIds.wave3, AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_SU_FIGHTER,	 1, 1, enemySquadIds.wave3, AiState.ATTACK_AGGRESSIVE },
--	air attackers
	{ 1, Units.UNIT_SU_FIGHTER,	18, 23, enemySquadIds.air, AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_SU_BOMBER,	31, 23, enemySquadIds.air, AiState.ATTACK_AGGRESSIVE },
};

wave4 = {
--transporters
	{ 1, Units.UNIT_SU_TANKYTRANSPORTER, 1, 6, enemySquadIds.wave4, AiState.RETREAT },
	{ 1, Units.UNIT_SU_TANKYTRANSPORTER, 1, 7, enemySquadIds.wave4, AiState.RETREAT },
--  escort
	{ 1, Units.UNIT_SU_MORTAR,			1, 8, enemySquadIds.wave4, AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_SU_RIFLE,			1, 5, enemySquadIds.wave4, AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_SU_RIFLE,			2, 6, enemySquadIds.wave4, AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_SU_TANK_DESTROYER,	2, 8, enemySquadIds.wave4, AiState.ATTACK_AGGRESSIVE },
--	air attackers
	{ 1, Units.UNIT_SU_FIGHTER,			23, 23, enemySquadIds.air, AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_SU_BOMBER,			12,  1, enemySquadIds.air, AiState.ATTACK_AGGRESSIVE },
};

wave5 = {
--transporters
	{ 1, Units.UNIT_SU_TANKYTRANSPORTER, 1, 6, enemySquadIds.wave5, AiState.RETREAT },
	{ 1, Units.UNIT_SU_TANKYTRANSPORTER, 1, 7, enemySquadIds.wave5, AiState.RETREAT },
--  escort
	{ 1, Units.UNIT_SU_MORTAR,	 1, 8, enemySquadIds.wave5, AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_SU_RIFLE,	 1, 5, enemySquadIds.wave5, AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_SU_RIFLE,	 2, 6, enemySquadIds.wave5, AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_SU_KATYUSHA, 2,	8, enemySquadIds.wave5, AiState.ATTACK_AGGRESSIVE },
--	air attackers
	{ 1, Units.UNIT_SU_FIGHTER, 23, 23, enemySquadIds.air, AiState.ATTACK_AGGRESSIVE },
};

wave6 = {
--transporters
	{ 1, Units.UNIT_SU_TANKYTRANSPORTER, 1, 6, enemySquadIds.wave6, AiState.RETREAT },
	{ 1, Units.UNIT_SU_TANKYTRANSPORTER, 1, 7, enemySquadIds.wave6, AiState.RETREAT },
--  escort
	{ 1, Units.UNIT_SU_MORTAR,	 1, 8, enemySquadIds.wave6, AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_SU_RIFLE,	 1, 5, enemySquadIds.wave6, AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_SU_RIFLE,	 2, 6, enemySquadIds.wave6, AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_SU_TANK_KV1, 2,	8, enemySquadIds.wave6, AiState.ATTACK_AGGRESSIVE },
--	air attackers
	{ 1, Units.UNIT_SU_BOMBER, 1, 1, enemySquadIds.air, AiState.ATTACK_AGGRESSIVE },
};

wave7 = {
--	transporters
	{ 1, Units.UNIT_SU_TANKYTRANSPORTER, 1, 6, enemySquadIds.wave7, AiState.RETREAT },
	{ 1, Units.UNIT_SU_TANKYTRANSPORTER, 1, 7, enemySquadIds.wave7, AiState.RETREAT },
--  escort
	{ 1, Units.UNIT_SU_MORTAR,			1, 	8,	enemySquadIds.wave7, AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_SU_RIFLE,			1, 	5,	enemySquadIds.wave7, AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_SU_MORTAR,			2, 	6, 	enemySquadIds.wave7, AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_SU_ANTI_INFANTRY,	2,	8,	enemySquadIds.wave7, AiState.ATTACK_AGGRESSIVE },
--	air attackers
	{ 1, Units.UNIT_SU_FIGHTER, 1, 23, enemySquadIds.air, AiState.ATTACK_AGGRESSIVE },
};

wave8 = {
	--	ambush troop
	{ 1, Units.UNIT_SU_MORTAR,			9, 4, enemySquadIds.ambushTroop, AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_SU_RIFLE,			9, 5, enemySquadIds.ambushTroop, AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_SU_MORTAR,			8, 5, enemySquadIds.ambushTroop, AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_SU_ANTI_INFANTRY,	8, 4, enemySquadIds.ambushTroop, AiState.ATTACK_AGGRESSIVE },

	--  air attackers
	{ 1, Units.UNIT_SU_FIGHTER,	 1, 23, enemySquadIds.air, AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_SU_BOMBER,	 1,  1, enemySquadIds.air, AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_SU_FIGHTER,	12,  1, enemySquadIds.air, AiState.ATTACK_AGGRESSIVE },
};

wave9 = {
	--  air attackers
	{ 1, Units.UNIT_SU_BOMBER,	1, 	23, enemySquadIds.air, AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_SU_FIGHTER,	1, 	1, 	enemySquadIds.air, AiState.ATTACK_AGGRESSIVE },
};


-- BEGIN Init


function onGameStart()
	CreateMissions( missions );
	setTriggers();
	storeValues();
	
	ScriptManager:gainGold( 0, 3000 );
	
	-- start intro cutscene
	ScriptManager:sleep( 0, "startIntroCutscene" ); -- without sleep player turn popup shows
end

function storeValues()
	ScriptManager:storeValue( "enemyTransporters", enemyTransporters );
	
	for i = 1, #pointIndicator do
		ScriptManager:storeValue( string.format( "pointIndicator %i", pointIndicator[i] ), pointIndicator[i] );
	end
end

function setTriggers()
	-- load all needed values after loading
	ScriptManager:addOnGameResumeTrigger( "loadValues", "loadValues", false );
	
	-- check if the player has units left
	ScriptManager:addOnUnitDestroyedTrigger( 0, -1, -1, "checkForEnd", "checkForEnd" , false );
	
	-- enemy transporter escaped
	ScriptManager:addOnEnterTileTrigger( finalPoint[1], finalPoint[2], 1, -1, -1, true, "transporterEscaped", "transporterEscaped", false );
	
	-- enemy transporter got destroyed
	ScriptManager:addOnUnitDestroyedTrigger( -1, Units.UNIT_SU_TANKYTRANSPORTER, -1, "transporterDestroyed", "transporterDestroyed", false );
	
	-- HQ got destroyed
	ScriptManager:addOnUnitDestroyedTrigger( 0, Units.UNIT_DE_HQ, 0, "hqDestroyed", "hqDestroyed", false );
	
	-- start second intro cutscene
	ScriptManager:addOnTurnStartTrigger( 0, -1, advanceGuardSpawnRound, "secondIntroCutscene", "secondIntroCutscene", false );
	
	-- start third intro cutscene
	ScriptManager:addOnTurnStartTrigger( 0, -1, firstTransporterSpawnRound, "thirdIntroCutscene", "thirdIntroCutscene", false );
	
	-- start last Transporters cutscene
	ScriptManager:addOnTurnStartTrigger( 0, -1, lastTransporterSpawnRound, "lastTransporterCutscene", "lastTransporterCutscene", false );
	
	-- Spawn enemy troops
	ScriptManager:addOnTurnStartTrigger( 1, -1, -1, "spawnEnemies", "spawnEnemies", false );
	
	-- Evacuation route
	addOnEnterAreaTrigger2( firstPoint[1], firstPoint[2], 1, -1, -1, false, "firstPointReached", "firstPointReached", false );
	
	addOnEnterAreaTrigger2( secondPoint[1], secondPoint[2], 1, -1, -1, false, "secondPointReached", "secondPointReached", false );
	
	addOnEnterAreaTrigger2( thirdPoint[1][1], thirdPoint[1][2], 1, -1, -1, false, "thirdPointReached", "thirdPointReached", false );
	addOnEnterAreaTrigger2( thirdPoint[2][1], thirdPoint[2][2], 1, -1, -1, false, "thirdPointReached", "thirdPointReached", false );
	
	addOnEnterAreaTrigger2( fourthPoint[1], fourthPoint[2], 1, -1, -1, false, "fourthPointReached", "thirdPointReached", false );
end

function loadValues()
	enemyTransporters = ScriptManager:getValue( "enemyTransporters" );
	
	for i = 1, #pointIndicator do
		pointIndicator[i] = ScriptManager:getValue( string.format( "pointIndicator %i", pointIndicator[i] ) );
	end
end


-- END Init
-- BEGIN Intro Cutscene


function startIntroCutscene()
	StartCutscene();
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE5_2_MISSION_PRIMARY_01_FLAVOR_01 );
	ScriptManager:wait( 7 );
	HideDialogue();
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE5_2_MISSION_PRIMARY_01_FLAVOR_02 );
	ScriptManager:wait( 7 );
	HideDialogue();
	
	ScriptManager:moveCameraEx( 24, 9, 0.7, 2, true );
	ScriptManager:showMarkerAt( 24, 9 );
	ScriptManager:displaySubtitle( Text.T_MAP_DE5_2_MISSION_SECONDARY_02_FLAVOR_01 );
	ScriptManager:wait( 7 );
	
	ScriptManager:hideMarkerAt( 24, 9 );
	HideDialogue();
	
	EndCutscene();
	
	startTheMission();
end

function secondIntroCutscene()
	StartCutscene()
	
	ScriptManager:moveCameraEx( 2, 7, 0.7, 3, true );
	ScriptManager:displaySubtitle( Text.T_MAP_DE5_2_MISSION_PRIMARY_01_FLAVOR_03 );
	ScriptManager:revealFogOfWar( 1, 8, 2, true );
	
	local unit = ScriptManager:spawnUnit( 1, Units.UNIT_SU_BOMBER, 5, 1, enemySquadIds.air, AiState.ATTACK_AGGRESSIVE );
	local unitId = unit:getId();
	ScriptManager:moveUnitEx(	unitId,  1, 9,	true );
	ScriptManager:attackUnitEx(	unitId, 16,		true );
	
	local unit = ScriptManager:spawnUnit( 1, Units.UNIT_SU_TANK_KV1, 1, 6, enemySquadIds.wave2, AiState.ATTACK_AGGRESSIVE );
	local unitId = unit:getId();
	ScriptManager:moveUnitEx(	unitId,  2, 8,	true );
	ScriptManager:attackUnitEx(	unitId, 16,		true );
	
	local unit = ScriptManager:spawnUnit( 1, Units.UNIT_SU_RIFLE, 1, 6, enemySquadIds.wave2, AiState.ATTACK_AGGRESSIVE );
	local unitId = unit:getId();
	ScriptManager:moveUnitEx( unitId, 2, 9, true );

	local unit = ScriptManager:spawnUnit( 1, Units.UNIT_SU_RIFLE, 1, 6, enemySquadIds.wave2, AiState.ATTACK_AGGRESSIVE );
	local unitId = unit:getId();
	ScriptManager:moveUnitEx( unitId, 2, 7, true );
	ScriptManager:setTargetTile( 1, enemySquadIds.wave2, firstPoint[1], firstPoint[2] );
	
	ScriptManager:revealFogOfWar( 1, 8, 2, false );
	HideDialogue();
	
	EndCutscene();
end

function thirdIntroCutscene()
	StartCutscene();
	
	ScriptManager:spawnUnit( 1, Units.UNIT_SU_TANKYTRANSPORTER, 1, 7, enemySquadIds.wave3, AiState.RETREAT );
	ScriptManager:spawnUnit( 1, Units.UNIT_SU_TANKYTRANSPORTER, 1, 6, enemySquadIds.wave3, AiState.RETREAT );
	ScriptManager:setTargetTile( 1, enemySquadIds.wave3, firstPoint[1], firstPoint[2] );
	ScriptManager:moveCameraEx( 1, 7, 0.7, 3, true )
	ScriptManager:revealFogOfWar( 1, 7, 1, true );
	ScriptManager:displaySubtitle( Text.T_MAP_DE5_2_MISSION_PRIMARY_01_FLAVOR_04 );
	ScriptManager:wait( 7 );
	ScriptManager:revealFogOfWar( 1, 7, 1, false );
	HideDialogue();
	
	EndCutscene();
end

function lastTransporterCutscene()
	StartCutscene();
	
	ScriptManager:moveCameraEx( 1, 6, 0.7, 3, true );
	ScriptManager:revealFogOfWar( 1, 7, 2, true );
	ScriptManager:displaySubtitle( Text.T_MAP_DE5_2_MISSION_PRIMARY_01_FLAVOR_05 );
	ScriptManager:wait( 7 );
	ScriptManager:revealFogOfWar( 1, 7, 2, false );
	HideDialogue();
	
	EndCutscene();
end

function startTheMission()
	ScriptManager:setMissionStatus( missionIds.defendHq,			MissionStatus.IN_PROGRESS );
	ScriptManager:setMissionStatus( missionIds.destroyTransporters,	MissionStatus.IN_PROGRESS );
	
	--TODO: remove this function!
	--testing();
	--removeSquads();
	--ScriptManager:addOnUnitSelectedTrigger( 1, -1, -1, "showParameters", "showParameters",false)
end

function removeSquads()
	--for i = 1, # do
	--	ScriptManager:removeUnitEx( [i], false );
	--end
end

function showParameters( selectedUnit )
	--print( string.format( "[] %s", selectedUnit:get() ) );
end


-- END Intro Cutscene
-- BEGIN Game Stuff


function transporterEscaped( tilex, tileY, playerId, templateId, unitId )
	if templateId ~= Units.UNIT_SU_TANKYTRANSPORTER then 
	else 
		ScriptManager:runForPlayer( 0, "transporterEscaped2" );
	end
end

function transporterEscaped2()
	DisablePlayerInteraction()
	
	ScriptManager:setMissionStatus( missionIds.destroyTransporters, MissionStatus.FAILED );
	ScriptManager:wait( 2 );
	
	EnablePlayerInteraction();
	StartCutscene();
	
	ScriptManager:moveCameraEx( 33, 21, 0.7, 3, true );
	ScriptManager:revealFogOfWar( 33, 21, 1, true );
	ScriptManager:displaySubtitle( Text.T_MAP_DE5_2_MISSION_PRIMARY_01_FLAVOR_06 );
	ScriptManager:showMarkerAt( 33, 21 );
	ScriptManager:wait( 5 );
	ScriptManager:hideMarkerAt( 33, 21 );
	ScriptManager:revealFogOfWar( 33, 21, 1, false );
	HideDialogue();
	
	EndCutscene();
	missionFailed();
end

function transporterDestroyed()
	enemyTransporters = enemyTransporters -1;
	ScriptManager:storeValue( "enemyTransporters", enemyTransporters );
	if ScriptManager:getCurrentRound() > lastTransporterSpawnRound and enemyTransporters > 0 then
		remainingTransportersPopup();
	
	elseif enemyTransporters <= 0 then
		ScriptManager:setMissionStatus( missionIds.destroyTransporters, MissionStatus.ACCOMPLISHED );
		startVictoryCutscene();
	end
end

function remainingTransportersPopup()
	ScriptManager:preparePopupText();
	ScriptManager:addPopupTextParameter( "enemyTransporters", enemyTransporters );
	ScriptManager:displaySubtitle( Text.T_MAP_DE5_2_MISSION_PRIMARY_COUNTER );
	ScriptManager:wait( 3 );
	HideDialogue()
end

function hqDestroyed() 
	ScriptManager:runForPlayer( 0, "hqDestroyed2" );
end

function hqDestroyed2()
	DisablePlayerInteraction()
	
	ScriptManager:setMissionStatus( missionIds.defendHq, MissionStatus.FAILED );
	ScriptManager:wait( 2 );
	
	EnablePlayerInteraction();
	StartCutscene();
	
	ScriptManager:moveCameraEx( 24, 9, 0.7, 3, true );
	ScriptManager:displaySubtitle( Text.T_MAP_DE5_2_MISSION_SECONDARY_02_FLAVOR_02 );
	ScriptManager:wait( 5 );
	HideDialogue();
	
	EndCutscene();
end

function spawnEnemies()
	local currentRound = ScriptManager:getCurrentRound();
	if currentRound == 1 then
		for i = 1, #wave1 do
			spawnWave( wave1[i] );
		end
		ScriptManager:setTargetTile( 1, enemySquadIds.wave1, firstPoint[1], firstPoint[2] );
	
	elseif currentRound == 2 then
		for i = 1, #wave2 do
			spawnWave( wave2[i] );
		end
		ScriptManager:setTargetTile( 1, enemySquadIds.wave2, firstPoint[1], firstPoint[2] );
	
	elseif currentRound == 3 then
		for i = 1, #wave3 do
			spawnWave( wave3[i] );
		end
		ScriptManager:setTargetTile( 1, enemySquadIds.wave3, firstPoint[1], firstPoint[2] );
	
	elseif currentRound == 4 then
		for i = 1, #wave4 do
			spawnWave( wave4[i] );
		end
		ScriptManager:setTargetTile( 1, enemySquadIds.wave4, firstPoint[1], firstPoint[2] );
	
	elseif currentRound == 5 then
		for i = 1, #wave5 do
			spawnWave( wave5[i] );
		end
		ScriptManager:setTargetTile( 1, enemySquadIds.wave5, firstPoint[1], firstPoint[2] );
	
	elseif currentRound == 6 then
		for i = 1, #wave6 do
			spawnWave( wave6[i] );
		end
		ScriptManager:setTargetTile( 1, enemySquadIds.wave6, firstPoint[1], firstPoint[2] );
	
	elseif currentRound == 7 then
		for i = 1, #wave7 do
			spawnWave( wave7[i] );
		end
		ScriptManager:setTargetTile( 1, enemySquadIds.wave7, firstPoint[1], firstPoint[2] );
	
	elseif currentRound == 8 then
		for i = 1, #wave8 do
			spawnWave( wave8[i] );
		end
		ScriptManager:setTargetTile( 1, enemySquadIds.ambushTroop, 24, 9 );
	
	elseif currentRound == 9 then
		for i = 1, #wave9 do
			spawnWave( wave9[i] );
		end
		ScriptManager:setTargetTile( 1, enemySquadIds.wave8, firstPoint[1], firstPoint[2] );
	end
end

function spawnWave( spawnData )
	ScriptManager:spawnUnit( spawnData[1], spawnData[2], spawnData[3], spawnData[4],spawnData[5], spawnData[6] );
end

function firstPointReached( tileX, tileY, playerId, templateId, unitId )
	pointReached( tileX, tileY, playerId, templateId, unitId, 1, secondPoint );
end

function secondPointReached(tileX, tileY, playerId, templateId, unitId)
	local rnd = math.random( 1, 2 );
	pointReached( tileX, tileY, playerId, templateId, unitId, 2, thirdPoint[rnd] );
end

function thirdPointReached(tileX, tileY, playerId, templateId, unitId)
	pointReached( tileX, tileY, playerId, templateId, unitId, 3, fourthPoint );
end

function fourthPointReached(tileX, tileY, playerId, templateId, unitId)
	pointReached( tileX, tileY, playerId, templateId, unitId, 4, finalPoint );
end

function pointReached(tileX, tileY, playerId, templateId, unitId, indicator, point )
	local unit=Unit:getUnitById( unitId );
	local squadId=unit:getSquadId();
	if squadId <= 8 then
		if pointIndicator[squadId] == indicator-1 then
			pointIndicator[squadId] = indicator;
			ScriptManager:storeValue( string.format( "pointIndicator %i", pointIndicator[squadId] ), pointIndicator[squadId] );
			
			ScriptManager:setTargetTile( 1, squadId, point[1], point[2] );
		end
	end
end


-- END Game Stuff
-- BEGIN Victory


function startVictoryCutscene()
	StartCutscene();
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE5_2_MISSION_PRIMARY_01_FLAVOR_07 );
	ScriptManager:wait( 10 );
	HideDialogue();
	
	EndCutscene();
	
	if 	ScriptManager:getMissionStatus( missionIds.defendHq ) ~= MissionStatus.FAILED then 
		ScriptManager:setMissionStatus( missionIds.defendHq, MissionStatus.ACCOMPLISHED );
	end
	
	victoryEndGame();
end

function victoryEndGame()
	DisablePlayerInteraction();
	HideDialogue(); -- remove possible texts on the screens
	
	-- accomplish secondary missions that are accomplished when the game is finished with certain condition
	
	ScriptManager:displaySubtitle( Text.T_MISSION_VICTORY );
	ScriptManager:wait( 4 );
	HideDialogue();
	endGame();
end


-- END Victory
-- BEGIN Defeated


function checkForEnd()
	if ScriptManager:noUnitsLeft( 0 ) then
		ScriptManager:runForPlayer( 0, "missionFailed" );
	end
end

function missionFailed()
	DisablePlayerInteraction();
	HideDialogue(); -- remove possible texts on the screens
	
	ScriptManager:displaySubtitle( Text.T_MISSION_FAILED );
	ScriptManager:wait( 4 );
	HideDialogue();
	endGame();
end


-- END Defeated


function endGame()
	ScriptManager:endGame();
end