require "lua/Constants"
require "lua/Functions"
require "lua/Functions_2"
require "lua/Text"
require "lua/Units"
require "math"


-- mission stuff
missionIds = {
	destroyFactory			= 1,
	destroyAirfield			= 2,
	captureCityBase			= 3,
	captureNorthernOutpost	= 4,
	destroyArtillery		= 5,
};

-- missionId, primary( boolean ), textkey, xpReward
missions = {
	{ missionIds.destroyFactory, 		true,	Text.T_MAP_DE5_3_MISSION_PRIMARY_01,	100 },
	{ missionIds.destroyAirfield, 		false,	Text.T_MAP_DE5_3_MISSION_SECONDARY_01,	100 },
	{ missionIds.captureCityBase,		false,	Text.T_MAP_DE5_3_MISSION_SECONDARY_02,	 50 },
	{ missionIds.captureNorthernOutpost,false,	Text.T_MAP_DE5_3_MISSION_SECONDARY_03,	100 },
	{ missionIds.destroyArtillery,		false,	Text.T_MAP_DE5_3_MISSION_SECONDARY_04,	 50 }
};

enemySquadIds = {
	warplanes			= 2,
	airfieldDefenders	= 3,
	southernTroops		= 4,
	airDefense			= 5,
	cityDefenders		= 6,
	northernTroops		= 7,
	outpostDefenders	= 8,
	reinforcements		= 9,
};

camoflagedUnits = { 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 28, 32, 33, 43, 46, 47, 48, 49 }	

bombFactory		= 29;
airfield		=  9;
cityBase		=  8;
northernOutpost	=  6;


-- BEGIN Init


function onGameStart()
	CreateMissions( missions );
	setTriggers();
	
	ScriptManager:gainGold( 0, 1000 );
	
	ScriptManager:storeValue( "heavyArtillery", 2 );
	
	prepareUnits();
	
	-- start intro cutscene
	ScriptManager:sleep( 0, "startIntroCutscene" ); -- without sleep player turn popup shows
end

function setTriggers()
	-- check if the player has units left
	ScriptManager:addOnUnitDestroyedTrigger( 0, -1, -1, "checkForEnd", "checkForEnd" , false );
	
	-- bomb factory destroyed
	ScriptManager:addOnUnitDestroyedTrigger( 1, Units.UNIT_SU_BOMB_FACTORY, bombFactory, "factoryDestroyed", "factoryDestroyed", false );
	
	-- airfield destroyed
	ScriptManager:addOnUnitDestroyedTrigger( 1, Units.UNIT_SU_AIRFIELD, airfield, "airfieldDestroyed", "airfieldDestroyed", false );
	
	-- capturing the airfield fails missionIds.destroyAirfield
	ScriptManager:addOnUnitCapturedTrigger( 1, Units.UNIT_SU_AIRFIELD, airfield, "airfieldCaptured", "airfieldCaptured",false );
	
	-- capturing the airfield fails missionIds.destroyAirfield
	ScriptManager:addOnUnitCapturedTrigger( -1, Units.UNIT_SU_AIRFIELD, -1, "airfieldCaptured", "airfieldCaptured", false );
	
	-- city base captured. Enemy reinforcements get spawned at northern outpost 
	ScriptManager:addOnUnitCapturedTrigger( -1, Units.UNIT_SU_HQ, cityBase, "cityBaseCaptured", "cityBaseCaptured", false );
	
	-- city base destroyed fails missionIds.captureCityBase
	ScriptManager:addOnUnitDestroyedTrigger( 1, Units.UNIT_SU_HQ, cityBase, "cityBaseDestroyed", "cityBaseDestroyed", false );
	
	-- northern outpost captured
	ScriptManager:addOnUnitCapturedTrigger( -1, Units.UNIT_SU_HQ, northernOutpost, "northernOutpostCaptured", "northernOutpostCaptured", false );
	
	-- northern outpost destroyed fails missionIds.captureNortehrnOutpost
	ScriptManager:addOnUnitDestroyedTrigger( 1, Units.UNIT_SU_HQ, northernOutpost, "northernOutpostDestroyed", "northernOutpostDestroyed", false );
	
	-- heavy artillery attacked and confirms concerns about the explosives formula
	ScriptManager:addOnUnitHasAttackedTrigger( 1, Units.UNIT_SU_HEAVY_ARTILLERY, -1, "heavyArtilleryAttacked", "heavyArtilleryAttacked", false );
	
	-- airfield sighted. This activates a secondary Mission
	ScriptManager:addOnUnitSightedTrigger( -1, Units.UNIT_SU_AIRFIELD, airfield, true, "airfieldSighted", "airfieldSighted", false );
	
	-- city base sighted. This activates a secondary Mission
	ScriptManager:addOnUnitSightedTrigger( 1, Units.UNIT_SU_HQ, cityBase, true, "cityBaseSighted", "cityBaseSighted", false );
	
	-- northern outpost sighted. This activates a secondary Mission
	ScriptManager:addOnUnitSightedTrigger( 1, Units.UNIT_SU_HQ, northernOutpost, true, "northernOutpostSighted", "northernOutpostSighted", false );
	
	-- heavy artillery destroyed
	ScriptManager:addOnUnitDestroyedTrigger( 1, Units.UNIT_SU_HEAVY_ARTILLERY, -1, "artilleryDestroyed", "artilleryDestroyed", false );
	
	-- makes Soviet units hold their ground
	ScriptManager:addOnTurnStartTrigger( 1, -1, -1, "onEnemyTurnStart", "onEnemyTurnStart", false );
	
	-- first attack on bomb factory spawns reinforcements at northern outpost
	ScriptManager:addOnUnitHasAttackedTrigger( -1, Units.UNIT_SU_BOMB_FACTORY, -1, "factoryAttacked", "factoryAttacked", false );
end

function factoryDestroyed()
	ScriptManager:setMissionStatus( missionIds.destroyFactory, MissionStatus.ACCOMPLISHED );
	startVictoryCutscene();
end

function airfieldDestroyed()
	ScriptManager:removeTrigger( "airfieldCaptured" );
	ScriptManager:removeTrigger( "airfieldDestroyed" );
	ScriptManager:setMissionStatus( missionIds.destroyAirfield, MissionStatus.ACCOMPLISHED );
end

function airfieldCaptured()
	ScriptManager:removeTrigger( "airfieldCaptured" );
	ScriptManager:removeTrigger( "airfieldDestroyed" );
	ScriptManager:setMissionStatus( missionIds.destroyAirfield, MissionStatus.FAILED );
end

function cityBaseCaptured()
	ScriptManager:removeTrigger( "cityBaseDestroyed" );
	ScriptManager:setMissionStatus( missionIds.captureCityBase,MissionStatus.ACCOMPLISHED );
	enemeyReinforcements();
end

function northernOutpostCaptured()
	ScriptManager:removeTrigger( "northernOutpostDestroyed" );
	ScriptManager:setMissionStatus( missionIds.captureNorthernOutpost, MissionStatus.ACCOMPLISHED );
end

function artilleryDestroyed()
	ScriptManager:storeValue( "heavyArtillery", ScriptManager:getValue( "heavyArtillery" ) - 1 );
	if ScriptManager:getValue( "heavyArtillery" ) <= 0 then
		ScriptManager:setMissionStatus( missionIds.destroyArtillery, MissionStatus.ACCOMPLISHED )
		
	else
		DisablePlayerInteraction()
		
		local missionStatus = ScriptManager:getMissionStatus( missionIds.destroyArtillery );
		if missionStatus == 0 then
			ScriptManager:setMissionStatus( missionIds.destroyArtillery, MissionStatus.IN_PROGRESS );
		end
		ScriptManager:displaySubtitle( Text.T_MAP_DE5_3_MISSION_SECONDARY_04_FLAVOR_04 );
		ScriptManager:wait( 3 );
		HideDialogue();
		EnablePlayerInteraction();
	end	
end

function enemeyReinforcements()
	local unit = Unit:getUnitAt( 15, 29 );
	if unit ~= nil then
		local playerId = unit:getPlayerId()
		if playerId == 1 then
			DisablePlayerInteraction()
			ScriptManager:spawnUnit( 1, Units.UNIT_SU_TANK_KV1, 14, 29, enemySquadIds.reinforcements, AiState.ATTACK_AGGRESSIVE );
			ScriptManager:wait( 0.1 );
			ScriptManager:spawnUnit( 1, Units.UNIT_SU_TANK_T34, 15, 30, enemySquadIds.reinforcements, AiState.ATTACK_AGGRESSIVE );
			ScriptManager:spawnUnit( 1, Units.UNIT_SU_TANK_T34, 15, 28, enemySquadIds.reinforcements, AiState.ATTACK_AGGRESSIVE );
			ScriptManager:setTargetTile( 1, enemySquadIds.reinforcements, 3, 15 );
			EnablePlayerInteraction();
		end	
	end
end


-- END Init
-- BEGIN Intro Cutscene


function prepareUnits()
	for i = 1, #camoflagedUnits do
		ScriptManager:camouflageUnit( camoflagedUnits[i] )
	end
	
	ScriptManager:fortifyUnit( 7 )
	ScriptManager:setGuardArea( 1, enemySquadIds.cityDefenders, 3, 15, 1 );
end

function startIntroCutscene()
	StartCutscene();
	
	ScriptManager:moveCameraEx(		12, 31, 1,5, false );
	ScriptManager:revealFogOfWar(	12, 31, 1, true );
	ScriptManager:showMarkerAt(		12, 31 );
	ScriptManager:displaySubtitle( Text.T_MAP_DE5_3_MISSION_PRIMARY_01_FLAVOR_01 );
	ScriptManager:wait( 5 );
	HideDialogue();
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE5_3_MISSION_PRIMARY_01_FLAVOR_02 );
	ScriptManager:wait( 5 );
	ScriptManager:hideMarkerAt(		12, 31 );
	ScriptManager:revealFogOfWar(	12, 31, 1, false );
	ScriptManager:moveCameraEx(		11, 29, 0.5, 2, true );
	HideDialogue();
	
	ScriptManager:moveCameraEx( 6, 6, 0.5, 7, false );
	ScriptManager:displaySubtitle( Text.T_MAP_DE5_3_MISSION_PRIMARY_01_FLAVOR_03 );
	ScriptManager:wait( 5 );
	
	local unit = ScriptManager:spawnUnit( 0, Units.UNIT_DE_TANK_3, 1, 1, -1, AiState.ATTACK );
	local unitId = unit:getId();
	ScriptManager:moveUnitEx( unitId, 2, 2, true );
	ScriptManager:moveUnitEx( unitId, 2, 5, true );
	
	local unit = ScriptManager:spawnUnit( 0, Units.UNIT_DE_TANK_4, 1, 1, -1, AiState.ATTACK );
	local unitId = unit:getId();
	ScriptManager:moveUnitEx( unitId, 2, 2, true );
	ScriptManager:moveUnitEx( unitId, 2, 4, true );
	
	local unit = ScriptManager:spawnUnit( 	0, Units.UNIT_DE_TRANSPORT_TRUCK,	1, 1, -1, AiState.ATTACK );
	ScriptManager:spawnUnit(				0, Units.UNIT_DE_MP40,				1, 1, -1, AiState.ATTACK );
	ScriptManager:spawnUnit(				0, Units.UNIT_DE_PANZERFAUST,		1, 1, -1, AiState.ATTACK );
	ScriptManager:spawnUnit(				0, Units.UNIT_DE_PANZERFAUST,		1, 1, -1, AiState.ATTACK );
	
	local unitId = unit:getId();
	ScriptManager:moveUnitEx( unitId, 2, 2, true );
	ScriptManager:moveUnitEx( unitId, 1, 3, true );
	
	local unit = ScriptManager:spawnUnit(	0, Units.UNIT_DE_TRANSPORT_TRUCK,	1, 1, -1, AiState.ATTACK );
	ScriptManager:spawnUnit(				0, Units.UNIT_DE_MP40,				1, 1, -1, AiState.ATTACK );
	ScriptManager:spawnUnit(				0, Units.UNIT_DE_MP40,				1, 1, -1, AiState.ATTACK );
	ScriptManager:spawnUnit(				0, Units.UNIT_DE_AA,				1, 1, -1, AiState.ATTACK );
	
	local unitId = unit:getId();
	ScriptManager:moveUnitEx( unitId, 2, 2, true );
	ScriptManager:spawnUnit( 0, Units.UNIT_DE_SUPPLY_TRUCK, 1, 1, -1, AiState.AIRPLANE );
	
	local unit = ScriptManager:spawnUnit( 0, Units.UNIT_DE_FIGHTER, 1, 6, -1, AiState.AIRPLANE );
	local unitId = unit:getId()
	ScriptManager:moveUnitEx( unitId, 2, 6, true );
	ScriptManager:spawnUnit( 0, Units.UNIT_DE_FIGHTER, 1, 5, -1, AiState.AIRPLANE );
	ScriptManager:spawnUnit( 0, Units.UNIT_DE_FIGHTER, 1, 4, -1, AiState.AIRPLANE );
	HideDialogue();
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE5_3_MISSION_PRIMARY_01_FLAVOR_04 );
	ScriptManager:moveCameraEx( 4, 5, 0.7, 1, true );
	ScriptManager:wait( 1 );
	HideDialogue();
	EndCutscene();
	
	startTheMission();
end

function startTheMission()
	ScriptManager:setMissionStatus( missionIds.destroyFactory,	 MissionStatus.IN_PROGRESS );
	ScriptManager:setMissionStatus( missionIds.destroyArtillery, MissionStatus.NOT_STARTED );
	
	--TODO: remove this function!
	--testing();
	--removeSquads();
	--ScriptManager:addOnUnitSelectedTrigger( 1, -1, -1, "showParameters", "showParameters",false)
end


-- END Intro Cutscene
-- BEGIN Game Stuff


function heavyArtilleryAttacked( attacker, defender )
	ScriptManager:storeValue( "artilleryX",	attacker:getX() );
	ScriptManager:storeValue( "artilleryY",	attacker:getY() );
	ScriptManager:storeValue( "defenderX",	defender:getX() );
	ScriptManager:storeValue( "defenderY",	defender:getY() );
	
	ScriptManager:removeTrigger( "heavyArtilleryAttacked" );
	ScriptManager:runForPlayer( 0, "artilleryCutscene" );
end

function artilleryCutscene( attacker, defender )
	StartCutscene();
	
	ScriptManager:moveCameraEx(		ScriptManager:getValue( "artilleryX" ), ScriptManager:getValue( "artilleryY" ), 1, 3,	true )
	ScriptManager:revealFogOfWar(	ScriptManager:getValue( "artilleryX" ), ScriptManager:getValue( "artilleryY" ), 1,		true )
	ScriptManager:showMarkerAt(		ScriptManager:getValue( "artilleryX" ), ScriptManager:getValue( "artilleryY" ) );
	ScriptManager:displaySubtitle( Text.T_MAP_DE5_3_MISSION_SECONDARY_04_FLAVOR_01 );
	ScriptManager:wait( 3 );
	HideDialogue();
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE5_3_MISSION_SECONDARY_04_FLAVOR_02 );
	ScriptManager:wait( 3 );
	ScriptManager:hideMarkerAt(		ScriptManager:getValue( "artilleryX" ), ScriptManager:getValue( "artilleryY" ) );
	ScriptManager:revealFogOfWar(	ScriptManager:getValue( "artilleryX" ), ScriptManager:getValue( "artilleryY" ), 1, false );
	HideDialogue();
	
	ScriptManager:moveCameraEx( ScriptManager:getValue( "defenderX" ), ScriptManager:getValue( "defenderY" ), 1, 5, false );
	ScriptManager:displaySubtitle( Text.T_MAP_DE5_3_MISSION_SECONDARY_04_FLAVOR_03 );
	ScriptManager:wait( 5 );
	HideDialogue();
	
	EndCutscene();
	
	ScriptManager:setMissionStatus( missionIds.destroyArtillery, MissionStatus.IN_PROGRESS );
end

function airfieldSighted()
	ScriptManager:removeTrigger( "airfieldSighted" )
	
	StartCutscene();
	
	ScriptManager:moveCameraEx( 15, 4, 1, 2, true );
	ScriptManager:showMarkerAt( 15, 4 );
	ScriptManager:displaySubtitle( Text.T_MAP_DE5_3_MISSION_SECONDARY_01_FLAVOR_01 );
	ScriptManager:wait( 5 );
	HideDialogue();
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE5_3_MISSION_SECONDARY_01_FLAVOR_02 );
	ScriptManager:wait( 3 );
	ScriptManager:hideMarkerAt( 15, 4 );
	HideDialogue();
	
	EndCutscene();
	
	ScriptManager:setMissionStatus( missionIds.destroyAirfield, MissionStatus.IN_PROGRESS );
end

function cityBaseSighted()
	ScriptManager:removeTrigger( "cityBaseSighted" );
	
	StartCutscene();
	
	ScriptManager:moveCameraEx( 3, 15, 1, 2, true );
	ScriptManager:showMarkerAt( 3, 15 );
	ScriptManager:displaySubtitle( Text.T_MAP_DE5_3_MISSION_SECONDARY_02_FLAVOR_01 );
	ScriptManager:wait( 3 );
	ScriptManager:hideMarkerAt( 3, 15 );
	HideDialogue();
	
	EndCutscene();
	
	ScriptManager:setMissionStatus( missionIds.captureCityBase, MissionStatus.IN_PROGRESS );
end

function cityBaseDestroyed()
	ScriptManager:removeTrigger( "cityBaseDestroyed" );
	ScriptManager:setMissionStatus(missionIds.captureCityBase, MissionStatus.FAILED );
end

function northernOutpostSighted()
	ScriptManager:removeTrigger( "northernOutpostSighted" )

	StartCutscene()
	
	ScriptManager:moveCameraEx( 15, 29, 1, 2, true );
	ScriptManager:showMarkerAt( 15, 29 );
	ScriptManager:displaySubtitle( Text.T_MAP_DE5_3_MISSION_SECONDARY_03_FLAVOR_01 );
	ScriptManager:wait( 3 );
	ScriptManager:hideMarkerAt( 15, 29 );
	HideDialogue();
	
	EndCutscene();
	
	ScriptManager:setMissionStatus( missionIds.captureNorthernOutpost, MissionStatus.IN_PROGRESS );
end

function onEnemyTurnStart()
	for i = 1, #camoflagedUnits do
		local unit = Unit:getUnitById( camoflagedUnits[i] );
		if unit ~= nil then
			unit:setActionPoints( 0 );
		end
	end
	
	local tank = Unit:getUnitById( 28 );
	if tank ~= nil then
		tank:setActionPoints( 0 );
	end
	
	local unit = Unit:getUnitById( 21 );
	if unit ~= nil then
		unit:setActionPoints( 0 );
	end
	
	local unit = Unit:getUnitById( 31 );
	if unit ~= nil then
		unit:setActionPoints( 0 );
	end
	
	local unit = Unit:getUnitById( 27 );
	if unit ~= nil then
		unit:setActionPoints( 0 );
	end
	
	local unit = Unit:getUnitById( 30 );
	if unit ~= nil then
		unit:setActionPoints( 0 );
	end
end

function factoryAttacked()
	ScriptManager:removeTrigger( "factoryAttacked" );
	local northernOutpost = Unit:getUnitById( northernOutpost );
	if northernOutpost ~= nil then
		local northernOutpostPlayerId = northernOutpost:getPlayerId();
		if northernOutpostPlayerId == 1 then 
			ScriptManager:spawnUnit( 1,Units.UNIT_SU_TANK_T34,	14, 29, -1, AiState.ATTACK_AGGRESSIVE );
			ScriptManager:spawnUnit( 1,Units.UNIT_SU_RIFLE,		15, 30, -1, AiState.ATTACK_AGGRESSIVE );
			ScriptManager:spawnUnit( 1,Units.UNIT_SU_RIFLE,		15, 28, -1, AiState.ATTACK_AGGRESSIVE );
		end
	end
end

function northernOutpostDestroyed()
	ScriptManager:removeTrigger( "northernOutpostDestroyed" );
	ScriptManager:setMissionStatus( missionIds.captureNorthernOutpost, MissionStatus.FAILED );
end


-- END Game Stuff
-- BEGIN Victory


function startVictoryCutscene()
	StartCutscene();
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE5_3_MISSION_PRIMARY_01_FLAVOR_05 );
	ScriptManager:wait( 3 );
	HideDialogue();
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE5_3_MISSION_PRIMARY_01_FLAVOR_06 );
	ScriptManager:wait( 3 );
	HideDialogue();
	
	EndCutscene();
	
	victoryEndGame();
end

function victoryEndGame()
	DisablePlayerInteraction();
	HideDialogue(); -- remove possible texts on the screens
	
	-- accomplish secondary missions that are accomplished when the game is finished with certain condition
	
	ScriptManager:displaySubtitle( Text.T_MISSION_VICTORY );
	ScriptManager:wait( 4 );
	HideDialogue();
	endGame();
end


-- END Victory
-- BEGIN Defeated


function checkForEnd()
	if ScriptManager:noUnitsLeft( 0 ) then
		ScriptManager:runForPlayer( 0, "missionFailed" );
	end
end

function missionFailed()
	DisablePlayerInteraction();
	HideDialogue(); -- remove possible texts on the screens
	
	ScriptManager:displaySubtitle( Text.T_MISSION_FAILED );
	ScriptManager:wait( 4 );
	HideDialogue();
	endGame();
end


-- END Defeated


function endGame()
	ScriptManager:endGame();
end