require "lua/Constants"
require "lua/Functions"
require "lua/Functions_2"
require "lua/Text"
require "lua/Units"
require "math"


-- mission stuff
missionIds = {
	destroyFactories	= 1,
	destroyRefinery		= 2,
	destroyTrucks		= 3,
	breakBarrier		= 4,
};

-- missionId, primary( boolean ), textkey, xpReward
missions = {
	{ missionIds.destroyFactories,		true,	Text.T_MAP_DE6_1_MISSION_PRIMARY_01,	175 },
	{ missionIds.destroyRefinery,		true,	Text.T_MAP_DE6_1_MISSION_PRIMARY_02,	175 },
	{ missionIds.destroyTrucks,			true,	Text.T_MAP_DE6_1_MISSION_PRIMARY_03,	175 },
	{ missionIds.breakBarrier,			false,	Text.T_MAP_DE6_1_MISSION_SECONDARY_01,	 50 },
};

enemySquadIds = {
	factory1		=  1, -- defending the lower 2 factories
	factory2		=  2, -- defending the upper factory
	refinery		=  3,
	outpost			=  4, -- defending the truck depot in the south
	roadBlock		=  5,
	mgNests			=  6,
	firstAttackers	=  7,
	waitingSquad	=  8, 
	southVillage	=  9,
	behindRoadBlock	= 10,
	frozenRiverN	= 11,
	frozenRiverMid	= 12,
	outInTheWild	= 13,
};

factoryIds			= { 22, 23, 24, 25, 26, 27, 28, 30, 31, 32, 33, 34, };
refineryIds			= { 35, 36, 37, 38, 39, 40, };
outpostIds			= { 42, 43, 44, 45, 46, 47, 48, 49, };
roadBlockIds		= { 50, 51, 52, 53, 54, 55, 56, 57, };
firstAttackerIds	= { 58, 59, 60, 62, 63, 64, 65, }
waitingIds			= { 66, 67, 68, 69, 70, };
southVillageIds		= { 71, 72, 73, };
behindRoadBlockIds	= { 74, 75, 76, }; 
frozenRiverIds		= { 77, 78, 79, 80, 81, };
outInTheWildIds		= { 82, 83, 84, 85, };
mgNestIds			= { 41, 86, 87, 88, }; -- bunker in roadBlockIds!

nearRoadBlock = {
	{  6,  9 }, {  7,  8 }, {  9,  8 }, { 10,  8 }, { 11,  8 }, { 12,  8 }, { 13,  8 }, { 12,  9 }, { 13, 10 },
	{ 12, 11 }, { 13, 12 }, { 12, 13 }, { 12, 14 }, { 11, 15 }, { 10, 15 }, {  9, 15 }, {  7, 14 }, { 17,  8 },
};

factoriesToDestroy	= 3;
trucksToDestroy		= 5;
roadBlocksToKill	= #roadBlockIds

-- BEGIN Init


function onGameStart()
	CreateMissions( missions );
	setTriggers();
	
	ScriptManager:gainGold( 0, 1000 );
	
	for i = 1, 2 do
		ScriptManager:camouflageUnit( frozenRiverIds[i] );
	end
	
	for i = 1, #firstAttackerIds do
		if i % 2 == 1 then
			ScriptManager:fortifyUnit( firstAttackerIds[i] );
		end
	end
	
	-- start intro cutscene
	ScriptManager:sleep( 0, "startIntroCutscene" ); -- without sleep player turn popup shows
end

function setTriggers()
	-- check if the player has units left
	ScriptManager:addOnUnitDestroyedTrigger( 0, -1, -1, "checkForEnd", "checkForEnd" , false );
	
	-- trigger for secondary intro cutscene
	ScriptManager:addOnTurnStartTrigger( 0, -1, 1, "prepareSecondaryIntroCutscene", "startSecondaryIntroCutscene", false );
	
	-- trigger for guard area
	ScriptManager:addOnTurnStartTrigger( 1, -1, -1, "refreshGuardArea", "refreshGuardArea", false );	

	-- trigger for enemy buildings
	ScriptManager:addOnUnitDestroyedTrigger( 1, Units.UNIT_SU_REFINERY_LOWHP,	 -1, "onRefineryDestroyed",	"onRefineryDestroyed",	false );
	ScriptManager:addOnUnitDestroyedTrigger( 1, Units.UNIT_SU_TANKFACTORY_LOWHP, -1, "onFactoryDestroyed",	"onFactoryDestroyed",	false );
	
	-- trigger for enemy trucks
	ScriptManager:addOnUnitDestroyedTrigger( 1, Units.UNIT_SU_SUPPLY_TRUCK,		-1, "onTruckDestroyed", "onTruckDestroyed", false );
	ScriptManager:addOnUnitDestroyedTrigger( 1, Units.UNIT_SU_TRANSPORT_TRUCK,	-1, "onTruckDestroyed", "onTruckDestroyed", false );
	
	-- trigger for enemy infantry
	ScriptManager:addOnUnitDestroyedTrigger( 1, -1, -1, "onUnitDestroyed", "onUnitDestroyed", false );
	
	-- trigger for frozen river s units to help either refinery or road block
	ScriptManager:addOnUnitHasAttackedTrigger( -1, Units.UNIT_DE_MG_NEST,	mgNestIds[1],	"backupRefinery",	"backupRefinery",	false );
	ScriptManager:addOnUnitHasAttackedTrigger( -1, Units.UNIT_SU_BUNKER,	-1,				"backupRoadBlock",	"backupRoadBlock",	false );
	ScriptManager:addOnUnitSightedTrigger(		1, Units.UNIT_SU_RIFLE,		-1, true, 		"riverMidAttack",	"riverMidAttack",	false );
	
	-- trigger for road block
	for i = 1, #nearRoadBlock do
		ScriptManager:addOnEnterTileTrigger( nearRoadBlock[i][1], nearRoadBlock[i][2], 0, -1, -1, 
											 true, "startRoadBlockCutscene", "startRoadBlockCutscene", false );
	end
	ScriptManager:addOnUnitHasAttackedTrigger( -1, -1, -1, "roadblockMissionStarter", "startRoadBlockCutscene", false );
end


-- END Init
-- BEGIN Intro Cutscene


function startIntroCutscene()
	StartCutscene();
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE6_1_MISSION_INTRO_FLAVOR_01 );
	ScriptManager:moveCameraEx( 3, 19, 0.7, 5, true );
	ScriptManager:wait( 1 );
	
	MarkUnit( 3, 19, 1, 2 );
	HideDialogue();
	ScriptManager:wait( 1 );
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE6_1_MISSION_INTRO_FLAVOR_02 );
	ScriptManager:moveCameraEx( 16, 17, 0.6, 5, true );
	ScriptManager:wait( 1 );
	factoryPos = { { 17, 20 }, { 15, 17 }, { 15, 15 }, };
	
	for i = 1, #factoryPos do
		local x = factoryPos[i][1];
		local y = factoryPos[i][2];
		
		ScriptManager:revealFogOfWar(	x, y, 1, true );
		ScriptManager:showMarkerAt(		x, y );
	end
	ScriptManager:wait( 2 );
	ScriptManager:revealFogOfWar( 16, 18, 4, false );
	ScriptManager:revealFogOfWar( 16, 16, 1, false );-- cause it overlaps...............
	ScriptManager:removeAllMarkers();
	HideDialogue();
	ScriptManager:wait( 1 );
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE6_1_MISSION_INTRO_FLAVOR_03 );
	ScriptManager:moveCameraEx( 3, 7, 0.7, 7, true );
	HideDialogue();
	
	EndCutscene();
	
	startTheMission();
end

function startTheMission()
	ScriptManager:setMissionStatus( missionIds.destroyFactories, MissionStatus.IN_PROGRESS );
	ScriptManager:setMissionStatus( missionIds.destroyRefinery,	 MissionStatus.IN_PROGRESS );
	
	ScriptManager:setMissionProgress( missionIds.destroyFactories,	0 );
	ScriptManager:setMissionProgress( missionIds.destroyTrucks,		0 );
	ScriptManager:setMissionProgress( missionIds.breakBarrier,		roadBlocksToKill );
	
	--TODO: remove this function!
	--testing();
	--removeSquads();
	--ScriptManager:addOnUnitSelectedTrigger( 1, -1, -1, "showParameters", "showParameters",false)
end

function testing()
	ScriptManager:spawnUnit( 0, Units.GOTT, 1, 1, 42, AiState.WAIT );
	ScriptManager:spawnUnit( 0, Units.GOTT, 1, 2, 42, AiState.WAIT );
end

function removeSquads()
	for i = 1, #factoryIds do
		ScriptManager:removeUnitEx( factoryIds[i], false );
	end	
	for i = 1, #refineryIds do
		ScriptManager:removeUnitEx( refineryIds[i], false );
	end
	for i = 1, #outpostIds do
		ScriptManager:removeUnitEx( outpostIds[i], false );
	end
	for i = 1, #roadBlockIds do
		ScriptManager:removeUnitEx( roadBlockIds[i], false );
	end
	for i = 1, #firstAttackerIds do
		ScriptManager:removeUnitEx( firstAttackerIds[i], false );
	end
	for i = 1, #waitingIds do
		ScriptManager:removeUnitEx( waitingIds[i], false );
	end
	for i = 1, #southVillageIds do
		ScriptManager:removeUnitEx( southVillageIds[i], false );
	end
	for i = 1, #behindRoadBlockIds do
		ScriptManager:removeUnitEx( behindRoadBlockIds[i], false );
	end
	for i = 1, #frozenRiverIds do
		ScriptManager:removeUnitEx( frozenRiverIds[i], false );
	end
	for i = 1, #outInTheWildIds do
		ScriptManager:removeUnitEx( outInTheWildIds[i], false );
	end
	for i = 1, #mgNestIds do
		ScriptManager:removeUnitEx( mgNestIds[i], false );
	end
end

function showParameters( selectedUnit )
	--print( string.format( "[] %s", selectedUnit:get() ) );
end

function prepareSecondaryIntroCutscene()
	table.insert( cutsceneQueue, startSecondaryIntroCutscene );
	CutsceneQueue();
end

function startSecondaryIntroCutscene()
	ScriptManager:removeTrigger( "startSecondaryIntroCutscene" );
	
	StartCutscene();
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE6_1_MISSION_INTRO_FLAVOR_04 );
	ScriptManager:moveCameraEx( 16, 2, 0.7, 6, true );
	ScriptManager:wait( 1 );
	HideDialogue();
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE6_1_MISSION_INTRO_FLAVOR_05 );
	MarkUnit( 16, 2, 1, 2 );
	ScriptManager:wait( 1 );
	ScriptManager:moveCameraEx( 3, 7, 0.7, 4, true );
	HideDialogue();
	
	EndCutscene();
	DisablePlayerInteraction()
	
	ScriptManager:setMissionStatus( missionIds.destroyTrucks, MissionStatus.IN_PROGRESS );
	ScriptManager:wait( 2 );
	
	EnablePlayerInteraction();
end


-- END Intro Cutscene
-- BEGIN Unit Stuff


function refreshGuardArea()
	-- set guard areas every turn to make sure they stay
	ScriptManager:setGuardArea( 1, enemySquadIds.roadBlock,		 9, 11, 2 );
	ScriptManager:setGuardArea( 1, enemySquadIds.southVillage,	11,  4, 2 );
end

function backupRefinery()
	ScriptManager:removeTrigger( "backupRefinery" );
	
	prepareBackup( 3, 16 );
end
	
function backupRoadBlock()
	ScriptManager:removeTrigger( "backupRoadBlock" );
	
	prepareBackup( 8, 10 );
end

function prepareBackup( posX, posY )
	-- depending on where the player attacks the frozenRiverMid squad attacks there
	ScriptManager:setTargetTile( 1, enemySquadIds.frozenRiverMid, posX, posY );
	ScriptManager:addOnTurnStartTrigger( 1, -1, -1, "backup", "backup", false );
end

function backup()
	ScriptManager:removeTrigger( "backup" );
	
	Unit:changeSquadState( 1, enemySquadIds.frozenRiverMid, AiState.ATTACK_AGGRESSIVE );
end
	
function riverMidAttack( seeingUnit, seenUnit )
	-- if the frozenRiverMid squad gets discovered early, just attack the player
	local squadId = seenUnit:getSquadId();
	
	if squadId == enemySquadIds.frozenRiverMid then
		Unit:changeSquadState( 1, squadId, AiState.ATTACK_AGGRESSIVE );
	end
end

function onUnitDestroyed( unit )
	local squadId = unit:getSquadId();
	
	if squadId == enemySquadIds.roadBlock then
		ScriptManager:setMissionProgress( missionIds.breakBarrier, ScriptManager:getMissionProgress( missionIds.breakBarrier ) - 1 );
		ScriptManager:runForPlayer( 0, "showRoadBlockMissionProgress" );
	end
end

function showRoadBlockMissionProgress()
	local progress = ScriptManager:getMissionProgress( missionIds.breakBarrier )
	
	if progress <= 0 then
		ScriptManager:removeTrigger( "onUnitDestroyed" );
		DisablePlayerInteraction()
		
		ScriptManager:setMissionStatus( missionIds.breakBarrier, MissionStatus.ACCOMPLISHED );
		ScriptManager:wait( 2 );
		
		EnablePlayerInteraction();
	
	else
		ShowMissionProgressPopup( progress, Text.T_MAP_DE6_1_MISSION_SECONDARY_01_COUNTER );
	end
end

function onTruckDestroyed()
	-- TODO: mb the south village squad should attack when trucks are getting destroyed but it conflicts with the guard area, so not sure...
	progressCheck( missionIds.destroyTrucks, trucksToDestroy, Text.T_MAP_DE6_1_MISSION_PRIMARY_03_COUNTER );
end


-- END Unit Stuff
-- BEGIN Objectives


function roadblockMissionStarter( attacker, defender )
	local squadId;
	if attacker:getPlayerId() == 1 then
		squadId = attacker:getSquadId()
	
	else
		squadId = defender:getSquadId()
	end
	
	if squadId == enemySquadIds.roadBlock then
		ScriptManager:runForPlayer( 0, "prepareRoadBlockCutscene" );
	end
end

function prepareRoadBlockCutscene()
	table.insert( cutsceneQueue, startRoadBlockCutscene );
	CutsceneQueue();
end

function startRoadBlockCutscene()
	ScriptManager:removeTrigger( "startRoadBlockCutscene" );
	StartCutscene();
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE6_1_MISSION_SECONDARY_01_FLAVOR_01 )
	ScriptManager:moveCameraEx( 9, 11, 0.7, 3, true );
	ScriptManager:wait( 1 );
	HideDialogue();
	
	EndCutscene();
	DisablePlayerInteraction();
	
	ScriptManager:setMissionStatus( missionIds.breakBarrier, MissionStatus.IN_PROGRESS );
	ScriptManager:wait( 2 );
	
	EnablePlayerInteraction();
end

function onRefineryDestroyed()
	ScriptManager:removeTrigger( "onRefineryDestroyed" );
	accomplishToVictoryCheck( missionIds.destroyRefinery );
end

function onFactoryDestroyed()
	-- after one factory is destroyed the frozenRiverN squad will push  towards it and aid the defenders
	Unit:changeSquadState( 1, enemySquadIds.frozenRiverN, AiState.ATTACK_AGGRESSIVE );
	progressCheck( missionIds.destroyFactories, factoriesToDestroy, Text.T_MAP_DE6_1_MISSION_PRIMARY_01_COUNTER );
end

function progressCheck( missionId, toDestroy, counterText )
	ScriptManager:setMissionProgress( missionId, ScriptManager:getMissionProgress( missionId ) + 1 );
	
	local progress = ScriptManager:getMissionProgress( missionId );
	
	if progress >= toDestroy then
		accomplishToVictoryCheck( missionId );
		
	else
		ShowMissionProgressPopup( progress, counterText );
	end
end

function accomplishToVictoryCheck( missionId )
	DisablePlayerInteraction();
	
	ScriptManager:setMissionStatus( missionId, MissionStatus.ACCOMPLISHED );
	ScriptManager:wait( 2 );
	
	EnablePlayerInteraction();
	checkForVictory( missionId );
end


-- END Objectives
-- BEGIN Victory


function checkForVictory( accomplishedMissionId )
	if	ScriptManager:getMissionStatus( missionIds.destroyFactories )	== MissionStatus.ACCOMPLISHED and
		ScriptManager:getMissionStatus( missionIds.destroyRefinery )	== MissionStatus.ACCOMPLISHED and
		ScriptManager:getMissionStatus( missionIds.destroyTrucks )		== MissionStatus.ACCOMPLISHED then
		startVictoryCutscene();
		
	else
		if accomplishedMissionId ~= missionIds.destroyTrucks then
			StartCutscene();
			
			if accomplishedMissionId == missionIds.destroyFactories then
				ScriptManager:displaySubtitle( Text.T_MAP_DE6_1_MISSION_PRIMARY_01_FLAVOR_01 );
			
			elseif accomplishedMissionId == missionIds.destroyRefinery then
				ScriptManager:displaySubtitle( Text.T_MAP_DE6_1_MISSION_PRIMARY_02_FLAVOR_01 );
			end
			ScriptManager:wait( 3 );
			HideDialogue();
			
			EndCutscene();
		end
	end
end

function startVictoryCutscene()
	StartCutscene();
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE6_1_MISSION_VICTORY_FLAVOR_01 );
	ScriptManager:wait( 5 );
	HideDialogue();
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE6_1_MISSION_VICTORY_FLAVOR_02 );
	ScriptManager:wait( 4 );
	HideDialogue();
	
	EndCutscene();
	
	victoryEndGame();
end

function victoryEndGame()
	DisablePlayerInteraction();
	HideDialogue(); -- remove possible texts on the screens

	ScriptManager:displaySubtitle( Text.T_MISSION_VICTORY );
	ScriptManager:wait( 4 );
	HideDialogue();
	endGame();
end


-- END Victory
-- BEGIN Defeated


function checkForEnd()
	if ScriptManager:noUnitsLeft( 0 ) then
		ScriptManager:runForPlayer( 0, "missionFailed" )
	end
end

function missionFailed()
	DisablePlayerInteraction();
	HideDialogue(); -- remove possible texts on the screens
	
	ScriptManager:displaySubtitle( Text.T_MISSION_FAILED );
	ScriptManager:wait( 4 );
	HideDialogue();
	endGame();
end


-- END Defeated


function endGame()
	ScriptManager:endGame()
end