require "lua/Constants"
require "lua/Functions"
require "lua/Functions_2"
require "lua/Text"
require "lua/Units"
require "math"


-- mission stuff
missionIds = {
	reachEnd		= 1,
	barricades		= 2,
	destroyFactory	= 3,
};

-- missionId, primary( boolean ), textkey, xpReward
missions = {
	{ missionIds.reachEnd,			true,  Text.T_MAP_DE6_2_MISSION_PRIMARY_01,		125 },
	{ missionIds.barricades,		true,  Text.T_MAP_DE6_2_MISSION_PRIMARY_02,		150 },
	{ missionIds.destroyFactory,	false, Text.T_MAP_DE6_2_MISSION_SECONDARY_01,	 50 },
};

enemySquadIds = {
	firstBlock	= 1,
	secondBlock	= 2, -- TODO: maybe split into two separate squad with arty defending depot and rest guarding an area; make arty immovable?
	thirdBlock	= 3,
	factory		= 4,
	firstAmbush	= 5,
	topAmbush	= 6, -- defend @ top village
	endAmbush	= 7, -- defend @ end
	topAmbush2	= 8, -- camouflaged; attack the player when @ top village
};

firstBlockIds	= {  3,  4,  5,  6,  7, 48 };
secondBlockIds	= {  8,  9, 10, 11, 12, 13, 14, 50, 51 };
thirdBlockIds	= { 15, 16, 17, 18, 19, 20, 21 };
factoryIds		= { 23, 24, 25, 26, };
firstAmbushIds	= { 27, 28, 29, 34, 48, 49, };
topAmbushIds	= { 31, 52, 53, 30, 32, 33, }; -- camouflage the first three
endAmbushIds	= { 35, 36, 37, 38, };

firstBlockToDestroy		= #firstBlockIds;
secondBlockToDestroy	= #secondBlockIds;
thirdBlockToDestroy		= #thirdBlockIds;

barricadesTotal		= 3;
firstCutsceneSeen	= 0;
secondCutsceneSeen	= 0;
thirdCutsceneSeen	= 0;

unitsInGoal			= {};
unitsNeededToWin	= 5;
playerUnits			= 9; -- 9 = number of start units

centerShaft = { {  8,  8 }, {  8, 11 } }
upperShaft	= { { 13, 13 }, { 14, 18 } };


-- BEGIN Init


function onGameStart()
	CreateMissions( missions );
	setTriggers();
	storeValues();
	
	for i = 1, #firstAmbushIds do
		ScriptManager:camouflageUnit( firstAmbushIds[i] );
	end
	
	for i = 1, 3 do
		ScriptManager:camouflageUnit( topAmbushIds[i] );
	end
	
	ScriptManager:gainGold( 0, 500 );
	
	-- start intro cutscene
	ScriptManager:sleep( 0, "startIntroCutscene" ); -- without sleep player turn popup shows
end

function storeValues()
	ScriptManager:storeValue( "firstBlockToDestroy",	firstBlockToDestroy );
	ScriptManager:storeValue( "secondBlockToDestroy",	secondBlockToDestroy );
	ScriptManager:storeValue( "thirdBlockToDestroy",	thirdBlockToDestroy );
	ScriptManager:storeValue( "firstCutsceneSeen",		firstCutsceneSeen );
	ScriptManager:storeValue( "secondCutsceneSeen",		secondCutsceneSeen );
	ScriptManager:storeValue( "thirdCutsceneSeen",		thirdCutsceneSeen );
	ScriptManager:storeValue( "playerUnits",			playerUnits );
end

function setTriggers()
	-- load all needed values after loading
	ScriptManager:addOnGameResumeTrigger( "loadValues", "loadValues", false );
	
	-- guard area
	ScriptManager:addOnTurnStartTrigger( 1, -1, -1, "onEnemyTurnStart", "onEnemyTurnStart", false );
	
	-- check if the player has units left
	ScriptManager:addOnUnitDestroyedTrigger( 0, -1, -1, "checkForEnd", "checkForEnd" , false );
	
	-- trigger for mine shafts
	for i = 1, #centerShaft do
		ScriptManager:addOnEnterTileTrigger( centerShaft[i][1], centerShaft[i][2], 0, -1, -1, true, 
											 "onCenterShaftEntered", "onCenterShaftEntered", false );
		-- tables have the same length, so use all in this loop
		ScriptManager:addOnEnterTileTrigger( upperShaft[i][1], upperShaft[i][2], 0, -1, -1, true,
											 "onUpperShaftEntered", "onUpperShaftEntered", false );
	end
	
	-- trigger for reaching / leaving the end zone
	for x = 14, 18 do
		ScriptManager:addOnEnterTileTrigger( x, 3, 0, -1, -1, false, "onEndReached", "onEndReached", false );
		ScriptManager:addOnEnterTileTrigger( x, 2, 0, -1, -1, false, "onEndReached", "onEndReached", false );
		ScriptManager:addOnEnterTileTrigger( x, 1, 0, -1, -1, false, "onEndReached", "onEndReached", false );
	end
	for x = 12, 18 do
		ScriptManager:addOnEnterTileTrigger( x, 5, 0, -1, -1, false, "onEndLeft",	 "onEndLeft",	 false );
		ScriptManager:addOnEnterTileTrigger( x, 4, 0, -1, -1, false, "onEndLeft",	 "onEndLeft",	 false );
	end
	
	ScriptManager:addOnUnitPurchasedTrigger( 0,			"onUnitBought",			 "onUnitBought" );
	ScriptManager:addOnUnitDestroyedTrigger( 0, -1, -1, "onPlayerUnitDestroyed", "onPlayerUnitDestroyed" );
	
	for y = 8, 20 do
		ScriptManager:addOnEnterTileTrigger( 11, y, 0, -1, -1, true, "startFactoryCutscene", "startFactoryCutscene", false );
	end
	
	-- trigger to change top ambush state from passive to attack
	ScriptManager:addOnUnitSightedTrigger(		 0, -1, -1, false,	"onBeingSeen",			"onBeingSeen",			false );
	ScriptManager:addOnUnitHasAttackedTrigger(	-1, -1, -1,			"onPlayerHasAttacked",	"onPlayerHasAttacked",	false );
	
	-- trigger for destroying roadblocks
	ScriptManager:addOnUnitDestroyedTrigger( 1, -1, -1, "onEnemyDestroyed", "onEnemyDestroyed", false );
	
	-- trigger for factory
	ScriptManager:addOnUnitDestroyedTrigger( 1, Units.UNIT_SU_TANKFACTORY_LOWHP, -1, "onFactoryDestroyed", "onFactoryDestroyed", false );
end

function loadValues()
	firstBlockToDestroy		= ScriptManager:getValue( "firstBlockToDestroy" );
	secondBlockToDestroy	= ScriptManager:getValue( "secondBlockToDestroy" );
	thirdBlockToDestroy		= ScriptManager:getValue( "thirdBlockToDestroy" );
	firstCutsceneSeen		= ScriptManager:getValue( "firstCutsceneSeen" );
	secondCutsceneSeen		= ScriptManager:getValue( "secondCutsceneSeen" );
	thirdCutsceneSeen		= ScriptManager:getValue( "thirdCutsceneSeen" );
	playerUnits				= ScriptManager:getValue( "playerUnits" );
	
	local value;
	for i = 1, 120 do
		value = ScriptManager:getValue( string.format( "goalUnits %i", i ) );
		if value ~= -1 then
			table.insert( unitsInGoal, value );
			--print( "legit value found" );
		end
	end
end


-- END Init
-- BEGIN Intro Cutscene


function startIntroCutscene()
	StartCutscene();
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE6_2_MISSION_INTRO_FLAVOR_01 );
	ScriptManager:wait( 5 );
	HideDialogue();
	ScriptManager:wait( 1 );
	
	ScriptManager:moveCameraEx( 16, 4, 0.7, 7, false );
	ScriptManager:wait( 2 );
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE6_2_MISSION_INTRO_FLAVOR_02 );
	ScriptManager:wait( 4 );
	HideDialogue();
	ScriptManager:wait( 1 );
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE6_2_MISSION_INTRO_FLAVOR_03 );
	ScriptManager:wait( 2 );
	ScriptManager:moveCameraEx( 3, 4, 0.7, 7, true );
	HideDialogue();
	
	EndCutscene();
	
	startTheMission();
end

function startTheMission()
	ScriptManager:setMissionStatus(		missionIds.reachEnd, MissionStatus.IN_PROGRESS );
	ScriptManager:setMissionProgress(	missionIds.barricades, 0 );
	
	--TODO: remove this function!
	--testing();
	--removeSquads();
	--ScriptManager:addOnUnitSelectedTrigger( 1, -1, -1, "showParameters", "showParameters",false)
end

function testing()
	ScriptManager:spawnUnit( 0, Units.GOTT, 1, 2, 42, AiState.WAIT );
	ScriptManager:spawnUnit( 0, Units.GOTT, 2, 2, 42, AiState.WAIT );
	--ScriptManager:spawnUnit( 0, Units.UNIT_DE_MP40, 7, 7, 42, AiState.WAIT );
	--ScriptManager:spawnUnit( 0, Units.UNIT_DE_MP40, 6, 7, 42, AiState.WAIT );
	ScriptManager:setMissionStatus( missionIds.barricades, MissionStatus.ACCOMPLISHED );
	
	for x = 13, 17 do
		ScriptManager:spawnUnit( 0, Units.UNIT_DE_MP40, x, 5, 0, 0 );
	end
end

function removeSquads()
	for i = 1, #firstBlockIds do
		ScriptManager:removeUnitEx( firstBlockIds[i], false );
	end
	
	for i = 1, #secondBlockIds do
		ScriptManager:removeUnitEx( secondBlockIds[i], false );
	end
	
	for i = 1, #thirdBlockIds do
		ScriptManager:removeUnitEx( thirdBlockIds[i], false );
	end
	
	for i = 1, #factoryIds do
		ScriptManager:removeUnitEx( factoryIds[i], false );
	end
	
	for i = 1, #firstAmbushIds do
		ScriptManager:removeUnitEx( firstAmbushIds[i], false );
	end
	
	for i = 1, #topAmbushIds do
		ScriptManager:removeUnitEx( topAmbushIds[i], false );
	end
end

function showParameters( selectedUnit )
	--print( string.format( "[] %s", selectedUnit:get() ) );
end


-- END Intro Cutscene
-- BEGIN Mine Shafts


function onCenterShaftEntered( tileX, tileY, playerId, unitTemplateId, unitId )
	if secondCutsceneSeen == 0 then
		StartCutscene();
		
		ScriptManager:displaySubtitle( Text.T_MAP_DE6_2_MISSION_PRIMARY_02_FLAVOR_06 );
		ScriptManager:moveCameraEx( tileX, tileY, 0.7, 2, true );
		ScriptManager:wait( 1 );
		HideDialogue();
		ScriptManager:wait( 1 );
		
		ScriptManager:displaySubtitle( Text.T_MAP_DE6_2_MISSION_PRIMARY_02_FLAVOR_03 );
		ScriptManager:wait( 6 );
		HideDialogue();
		ScriptManager:wait( 1 );
		
		ScriptManager:displaySubtitle( Text.T_MAP_DE6_2_MISSION_PRIMARY_02_FLAVOR_04 );
		ScriptManager:wait( 4 );
		HideDialogue();
		
		EndCutscene();
		
		secondCutsceneSeen = 1;
		ScriptManager:storeValue( "secondCutsceneSeen", secondCutsceneSeen );
	end
	
	OnShaftEntered( tileY, unitId, centerShaft )
end

function onUpperShaftEntered( tileX, tileY, playerId, unitTemplateId, unitId )
	OnShaftEntered( tileY, unitId, upperShaft );
end


-- END Mine Shafts
-- BEGIN Unit Stuff


function onEnemyTurnStart()
	ScriptManager:setGuardArea( 1, enemySquadIds.firstBlock,	 2, 13, 2 );
	ScriptManager:setGuardArea( 1, enemySquadIds.thirdBlock,	14,  7, 2 );
	ScriptManager:setGuardArea( 1, enemySquadIds.topAmbush,		 4, 17, 2 );
	ScriptManager:setGuardArea( 1, enemySquadIds.endAmbush,		16,  3, 2 );
end

function onEnemyDestroyed( unit )
	local squadId = unit:getSquadId();
	local templateId = unit:getTemplateId();
	
	if templateId ~= Units.UNIT_SU_HQ then
		if squadId == enemySquadIds.firstBlock then
			firstBlockToDestroy = barricadeProgress( firstBlockToDestroy );
			ScriptManager:storeValue( "firstBlockToDestroy", firstBlockToDestroy );
		
		elseif squadId == enemySquadIds.secondBlock then
			secondBlockToDestroy = barricadeProgress( secondBlockToDestroy );
			ScriptManager:storeValue( "secondBlockToDestroy", secondBlockToDestroy );
		
		elseif squadId == enemySquadIds.thirdBlock then
			thirdBlockToDestroy = barricadeProgress( thirdBlockToDestroy );
			ScriptManager:storeValue( "thirdBlockToDestroy", thirdBlockToDestroy );
		end
	end
end

function barricadeProgress( toDestroy )
	toDestroy = toDestroy - 1;
	
	if toDestroy == 0 then
		ScriptManager:setMissionProgress( missionIds.barricades, ScriptManager:getMissionProgress( missionIds.barricades ) + 1 );
		
		ScriptManager:runForPlayer( 0, "showBarricadeProgress" );
		
		--TODO: show progress for units in the barricade?
	end
	return toDestroy;
end

function showBarricadeProgress()
	if ScriptManager:getMissionProgress( missionIds.barricades ) >= barricadesTotal then
		DisablePlayerInteraction();
		
		ScriptManager:setMissionStatus( missionIds.barricades, MissionStatus.ACCOMPLISHED );
		--checkEndZoneObjective();
		ScriptManager:wait( 2 );
		
		EnablePlayerInteraction();
		checkForVictory();
		
	else
		ShowMissionProgressPopup( ScriptManager:getMissionProgress( missionIds.barricades ), Text.T_MAP_DE6_2_MISSION_PRIMARY_02_COUNTER );
	end
end

function onPlayerHasAttacked( attacker, defender )
	if attacker:getPlayerId() == 0 then -- player attacked
		onBeingSeen( defender, attacker );
	
	else
		onBeingSeen( attacker, defender );
	end
end

function onBeingSeen( seeingUnit, seenUnit )
	local squadId = seeingUnit:getSquadId();
	if squadId == enemySquadIds.topAmbush1 or squadId == enemySquadIds.topAmbush2 then
		Unit:changeSquadState( 1, enemySquadIds.topAmbush,	AiState.ATTACK_AGGRESSIVE );
		Unit:changeSquadState( 1, enemySquadIds.topAmbush2,	AiState.ATTACK_AGGRESSIVE );
		
	elseif squadId == enemySquadIds.firstBlock then
		if firstCutsceneSeen == 0 then
			ScriptManager:runForPlayer( 0, "showFirstBlockCutscene")
		end
	
	elseif squadId == enemySquadIds.secondBlock then
		if secondCutsceneSeen == 0 then
			ScriptManager:runForPlayer( 0, "showSecondBlockCutscene" );
		end
	
	elseif squadId == enemySquadIds.thirdBlock then
		if thirdCutsceneSeen == 0 then
			ScriptManager:runForPlayer( 0, "showThirdBlockCutscene" );
		end
	end
end

function showFirstBlockCutscene()
	StartCutscene();
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE6_2_MISSION_PRIMARY_02_FLAVOR_01 );
	ScriptManager:moveCameraEx( 3, 11, 0.7, 2, true );
	ScriptManager:wait( 2 );
	HideDialogue();
	
	EndCutscene();
	DisablePlayerInteraction();
	
	firstCutsceneSeen = 1;
	ScriptManager:storeValue( "firstCutsceneSeen", firstCutsceneSeen );
	
	if ScriptManager:getMissionStatus( missionIds.barricades ) ~= MissionStatus.IN_PROGRESS then
		ScriptManager:setMissionStatus( missionIds.barricades, MissionStatus.IN_PROGRESS );
		ScriptManager:wait( 2 );
	end
	
	EnablePlayerInteraction();
end

function showSecondBlockCutscene()
	StartCutscene();
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE6_2_MISSION_PRIMARY_02_FLAVOR_02 );
	ScriptManager:moveCameraEx( 9, 15, 0.7, 4, true );
	HideDialogue();
	ScriptManager:wait( 1 );
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE6_2_MISSION_PRIMARY_02_FLAVOR_03 );
	ScriptManager:moveCameraEx( 8, 9, 0.8, 4, true );
	ScriptManager:wait( 4 );
	HideDialogue();
	ScriptManager:wait( 1 );
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE6_2_MISSION_PRIMARY_02_FLAVOR_04 );
	ScriptManager:moveCameraEx( 5, 17, 0.7, 4, true );
	HideDialogue();
	
	EndCutscene();
	DisablePlayerInteraction();
	
	secondCutsceneSeen = 1;
	ScriptManager:storeValue( "secondCutsceneSeen", secondCutsceneSeen );
	
	if ScriptManager:getMissionStatus( missionIds.barricades ) ~= MissionStatus.IN_PROGRESS then
		ScriptManager:setMissionStatus( missionIds.barricades, MissionStatus.IN_PROGRESS );
		ScriptManager:wait( 2 );
	end
	
	EnablePlayerInteraction();
end

function showThirdBlockCutscene()
	StartCutscene();
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE6_2_MISSION_PRIMARY_02_FLAVOR_05 );
	ScriptManager:wait( 4 );
	HideDialogue();
	
	EndCutscene();
	DisablePlayerInteraction();
	
	thirdCutsceneSeen = 1;
	ScriptManager:storeValue( "thirdCutsceneSeen", thirdCutsceneSeen );
	
	if ScriptManager:getMissionStatus( missionIds.barricades ) ~= MissionStatus.IN_PROGRESS then
		ScriptManager:setMissionStatus( missionIds.barricades, MissionStatus.IN_PROGRESS );
		ScriptManager:wait( 2 );
	end
	
	EnablePlayerInteraction();
end

function startFactoryCutscene()
	ScriptManager:removeTrigger( "startFactoryCutscene" );
	
	StartCutscene();
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE6_2_MISSION_SECONDARY_01_FLAVOR_01 );
	ScriptManager:moveCameraEx( 17, 18, 0.7, 4, true );
	MarkUnit( 17, 18, 1, 2 );
	HideDialogue();
	ScriptManager:wait( 1 );
		
	ScriptManager:displaySubtitle( Text.T_MAP_DE6_2_MISSION_SECONDARY_01_FLAVOR_02 );
	ScriptManager:moveCameraEx( 13, 13, 0.7, 3, true );
	HideDialogue();
	
	EndCutscene();
	DisablePlayerInteraction();
	
	ScriptManager:setMissionStatus( missionIds.destroyFactory, MissionStatus.IN_PROGRESS );
	ScriptManager:wait( 2 );
	
	EnablePlayerInteraction();
end

function onFactoryDestroyed()
	ScriptManager:removeTrigger( "onFactoryDestroyed" );
	DisablePlayerInteraction();
	
	ScriptManager:setMissionStatus( missionIds.destroyFactory, MissionStatus.ACCOMPLISHED );
	ScriptManager:wait( 2 );
	
	EnablePlayerInteraction();
end


-- END Unit Stuff
-- BEGIN End Zone Stuff


function onUnitBought( unit )
	playerUnits = playerUnits + 1;
	ScriptManager:storeValue( "playerUnits", playerUnits );
end

function onPlayerUnitDestroyed( unit )
	local checkObjective = false;
	
	playerUnits = playerUnits - 1;
	print( playerUnits );
	ScriptManager:storeValue( "playerUnits", playerUnits );
	
	-- check if unit was in end zone and remove it if it is;
	checkObjective = leftCheck( unit:getId() );
	
	-- check if the need units for the objective have changed by this destruction
	if checkObjective == false and playerUnits < 5 and playerUnits <= #unitsInGoal then
		checkObjective = true;
	end
	
	-- if one of the two checks above happen, check objective / victory
	if checkObjective then
		-- only show if it is relevant ( = the barricades are gone and the player should leave now )
		if ScriptManager:getMissionStatus( missionIds.barricades ) == MissionStatus.ACCOMPLISHED then
			if checkEndZoneObjective() then
				checkForVictory();
			end
		end
	end
end

function onEndReached( tileX, tileY, playerId, unitTemplateId, unitId )
	local alreadyIn = false;
	print( string.format( "unit %i reached goal", unitId ) );
	
	-- check if unit is already in goal
	if #unitsInGoal >= 1 then
		for i = 1, #unitsInGoal do
			if unitsInGoal[i] == unitId then
				alreadyIn = true;
				print( "unit was already in" );
				break;
			end
		end
	end
	
	if alreadyIn == false then
		table.insert( unitsInGoal, unitId );
		print( #unitsInGoal );
		ScriptManager:storeValue( string.format( "goalUnits %i", unitId ), unitId );
		print( "unit added to goalUnits" );
		
		if ScriptManager:getMissionStatus( missionIds.barricades ) == MissionStatus.ACCOMPLISHED then
			if checkEndZoneObjective() then
				checkForVictory();
			end
			
		else
			ScriptManager:displaySubtitleAutoEx( Text.T_MAP_DE6_2_MISSION_PRIMARY_01_FLAVOR_01, false );
		end
	end
end

function checkEndZoneObjective()
	-- check how many player units are alive and ( if necessary ) adapt the goal
	local neededToWin = unitsNeededToWin;
	print( playerUnits );
	print( neededToWin );
	
	if playerUnits < 5 then
		print( "< 5 units needed to win" );
		neededToWin = playerUnits;
	end
	
	-- check for victory
	if #unitsInGoal >= neededToWin then
		ScriptManager:setMissionStatus( missionIds.reachEnd, MissionStatus.ACCOMPLISHED );
		
		return true;
		
	else
		print( "popup incoming!" );
		ScriptManager:preparePopupText();
		ScriptManager:addPopupTextParameter( "y", neededToWin );
		ScriptManager:addPopupTextParameter( "x", #unitsInGoal );
		ScriptManager:displaySubtitleAutoEx( Text.T_MAP_DE_3_2_MISSION_PRIMARY_COUNTER, false ); -- no changerino, text is the same 
		
		return false;
	end
end

function onEndLeft( tileX, tileY, playerId, unitTemplateId, unitId )
	if leftCheck( unitId ) == true then
	
		-- only show if it is relevant ( = the barricades are gone and the player should leave now )
		if ScriptManager:getMissionStatus( missionIds.barricades ) == MissionStatus.ACCOMPLISHED then
			if checkEndZoneObjective() then
				checkForVictory();
			end
		end
	end
end

function leftCheck( unitId )
	local returnValue = false;
	if #unitsInGoal >= 1 then
		for i = 1, #unitsInGoal do
			if unitsInGoal[i] == unitId then
				print( "found" .. unitId );
				table.remove( unitsInGoal, i );
				ScriptManager:deleteValue( string.format( "goalUnits %i", unitId ) );
				returnValue = true;
				break;
			end
		end
	end
	
	return returnValue;
end


-- END End Zone Stuff
-- BEGIN Victory


function checkForVictory()
	if  ScriptManager:getMissionStatus( missionIds.barricades )	== MissionStatus.ACCOMPLISHED and
		ScriptManager:getMissionStatus( missionIds.reachEnd )	== MissionStatus.ACCOMPLISHED then
		
		ScriptManager:runForPlayer( 0, "startVictoryCutscene" );
		
	elseif ScriptManager:getMissionStatus( missionIds.barricades ) == MissionStatus.ACCOMPLISHED then
		StartCutscene();
		
		ScriptManager:displaySubtitle( Text.T_MAP_DE6_2_MISSION_PRIMARY_02_FLAVOR_07 );
		
		ScriptManager:wait( 3 );
		HideDialogue();
		
		EndCutscene();
		checkEndZoneObjective();
	end
end

function startVictoryCutscene()
	StartCutscene();
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE6_2_MISSION_VICTORY_FLAVOR_01 );
	ScriptManager:wait( 4 );
	HideDialogue();
	
	EndCutscene();
	
	victoryEndGame();
end

function victoryEndGame()
	DisablePlayerInteraction();
	HideDialogue(); -- remove possible texts on the screens

	-- accomplish secondary missions that are accomplished when the game is finished with certain condition
	
	
	ScriptManager:displaySubtitle( Text.T_MISSION_VICTORY );
	ScriptManager:wait( 4 );
	HideDialogue();
	endGame();
end


-- END Victory
-- BEGIN Defeated


function checkForEnd()
	if ScriptManager:noUnitsLeft( 0 ) then
		ScriptManager:runForPlayer( 0, "missionFailed" )
	end
end

function missionFailed()
	DisablePlayerInteraction();
	HideDialogue(); -- remove possible texts on the screens
	
	ScriptManager:displaySubtitle( Text.T_MISSION_FAILED );
	ScriptManager:wait( 4 );
	HideDialogue();
	endGame();
end


-- END Defeated


function endGame()
	ScriptManager:endGame()
end