require "lua/Constants"
require "lua/Functions"
require "lua/Functions_2"
require "lua/Text"
require "lua/Units"
require "math"


-- mission stuff
missionIds = {
	destroyMine		= 1,
	destroyArty		= 2,
	breakBarrier	= 3,
	getAirfields	= 4,
};

-- missionId, primary( boolean ), textkey, xpReward
missions = {
	{ missionIds.destroyMine,	true,	Text.T_MAP_DE6_3_MISSION_PRIMARY,		200 },
	{ missionIds.destroyArty,	false,	Text.T_MAP_DE6_3_MISSION_SECONDARY_01,	 75 },
	{ missionIds.breakBarrier,	false,	Text.T_MAP_DE6_3_MISSION_SECONDARY_02,	 75 },
	{ missionIds.getAirfields,	false,	Text.T_MAP_DE6_3_MISSION_SECONDARY_03,	 50 },
};

enemySquadIds = {
	mine			=  1,
	airfieldW		=  2,
	airfieldE		=  3,
	barrier1		=  4,
	barrier2		=  5,
	barrier3		=  6,
	arty1			=  7, -- attack from the very beginning
	arty2			=  8,
	arty3			=  9, -- attack from the very beginning
	artyAA1			= 10,
	artyAA2			= 11,
	artyAA3			= 12,
	firstAttack		= 13,
	firstPlane		= 14,
	southShaftDef	= 15,
	riverForestDef	= 16,
	planeWave1		= 17,
	planeWave2		= 18,
};

mineIds				= { 22, 23, 24, 25, 26, 27, 28, };
airfieldWIds		= { 17, 18, 19, 20, 21, };
airfieldEIds		= { 29, 30, 31, 32, 33, };
barrier1Ids			= { 34, 35, 36, 37, 38, 39, };
barrier2Ids			= { 44, 45, 46, 47, 48, 49, 52, };
barrier3Ids			= { 53, 54, 55, 56, 57, 58, 59, };
arty1Ids			= { 40, 41, 42, };
arty2Ids			= { 50, 51, };
arty3Ids			= { 60, 61, 62, };
firstAttackerIds	= { 43, 63, 64, 65, 66, 74 };
southShaftDefIds	= { 67, 68, 69, 70, };
riverForestDef		= { 71, 72, 73, };

barricadesTotal			= 3;
firstBlockToDestroy		= #barrier1Ids;
secondBlockToDestroy	= #barrier2Ids;
thirdBlockToDestroy		= #barrier3Ids;
artysTotal				= 3;
arty1ToDestroy			= #arty1Ids;
arty2ToDestroy			= #arty2Ids;
arty3ToDestroy			= #arty3Ids;
airfieldsToDestroy		= 2;

firstShaft	= { {  6, 10 }, {  8, 12 } };
secondShaft	= { { 13,  2 }, { 15,  3 } };
thirdShaft	= { { 22,  9 }, { 23, 12 } };

waveSpawnAirfield			= 2;
spawnPlanes					= 1;
planeWaveRespawnInterval	= 4; -- defines how often waves will spawn
bomberWave					= 2; -- count starts @ 0 so its +1 on the number
lastPlaneWaveSpawnedInRound	= 0; -- indicates the round in which the last wave spawned
numberOfPlaneWavesSpawned	= 0;


-- BEGIN Init


function onGameStart()
	CreateMissions( missions );
	setTriggers();
	storeValues();
	
	ScriptManager:gainGold( 0, 750 );
	
	-- start intro cutscene
	ScriptManager:sleep( 0, "startIntroCutscene" ); -- without sleep player turn popup shows
end

function storeValues()
	ScriptManager:storeValue( "firstBlockToDestroy",	firstBlockToDestroy );
	ScriptManager:storeValue( "secondBlockToDestroy",	secondBlockToDestroy );
	ScriptManager:storeValue( "thirdBlockToDestroy",	thirdBlockToDestroy );
	
	ScriptManager:storeValue( "arty1ToDestroy", arty1ToDestroy );
	ScriptManager:storeValue( "arty2ToDestroy", arty2ToDestroy );
	ScriptManager:storeValue( "arty3ToDestroy", arty3ToDestroy );
	
	ScriptManager:storeValue( "spawnPlanes",					spawnPlanes );
	ScriptManager:storeValue( "numberOfPlaneWavesSpawned",		numberOfPlaneWavesSpawned );
	ScriptManager:storeValue( "lastPlaneWaveSpawnedInRound",	lastPlaneWaveSpawnedInRound );
end

function setTriggers()
	-- load all needed values after loading
	ScriptManager:addOnGameResumeTrigger( "loadValues", "loadValues", false );
	
	-- trigger foor guard area and wave spawn
	ScriptManager:addOnTurnStartTrigger( 1, -1, -1, "onEnemyTurnStart", "onEnemyTurnStart", false );
	
	-- check if the player has units left
	ScriptManager:addOnUnitDestroyedTrigger( 0, -1, -1, "checkForEnd", "checkForEnd" , false );
	
	-- trigger for mining facility
	ScriptManager:addOnUnitDestroyedTrigger( 1, Units.UNIT_SU_MINING_FACILITY, -1, "onMineDestroyed", "onMineDestroyed", false );
	
	-- trigger for airfields
	ScriptManager:addOnUnitHasAttackedTrigger(	1, Units.UNIT_SU_FIGHTER,	-1, "onAirplaneHasAttacked", "onAirplaneHasAttacked",	false );
	ScriptManager:addOnUnitHasAttackedTrigger(	1, Units.UNIT_SU_BOMBER,	-1, "onAirplaneHasAttacked", "onAirplaneHasAttacked",	false );
	ScriptManager:addOnUnitCapturedTrigger(		0, Units.UNIT_SU_AIRFIELD,	-1, "onAirfieldCaptured",	 "onAirfieldGotten",		false );
	ScriptManager:addOnUnitDestroyedTrigger(	1, Units.UNIT_SU_AIRFIELD,	-1, "onAirfieldDestroyed",	 "onAirfieldGotten",		false );
	
	-- trigger for barricades
	ScriptManager:addOnUnitDestroyedTrigger( 1, -1, -1, "onUnitDestroyed", "onUnitDestroyed", false );
	
	-- trigger for mine shafts
	for i = 1, #firstShaft do
		ScriptManager:addOnEnterTileTrigger( firstShaft[i][1],	firstShaft[i][2],	-1, -1, -1, true, 
											 "onFirstShaftEntered", "onFirstShaftEntered", false );
		ScriptManager:addOnEnterTileTrigger( secondShaft[i][1],	secondShaft[i][2],	-1, -1, -1, true,
											 "onSecondShaftEntered", "onSecondShaftEntered", false );
		ScriptManager:addOnEnterTileTrigger( thirdShaft[i][1],	thirdShaft[i][2],	-1, -1, -1, true,
											 "onThirdShaftEntered", "onThirdShaftEntered", false );
	end
	
	-- trigger for making artillery aggressive
	local x = 12;
	for y = 10, 14 do
		if y % 2 == 0 then
			x = x + 1;
		end
		ScriptManager:addOnEnterTileTrigger( x, y, 0, -1, -1, false, "arty2Aggressive", "arty2Aggressive", false );
	end
	ScriptManager:addOnUnitHasAttackedTrigger( -1, Units.UNIT_SU_HOWITZER, -1, "onArtilleryHasAttacked", "onArtilleryHasAttacked", false );
end

function loadValues()
	firstBlockToDestroy	 = ScriptManager:getValue( "firstBlockToDestroy" );
	secondBlockToDestroy = ScriptManager:getValue( "secondBlockToDestroy" );
	thirdBlockToDestroy	 = ScriptManager:getValue( "thirdBlockToDestroy" );
	
	arty1ToDestroy = ScriptManager:getValue( "arty1ToDestroy" );
	arty2ToDestroy = ScriptManager:getValue( "arty2ToDestroy" );
	arty3ToDestroy = ScriptManager:getValue( "arty3ToDestroy" );
	
	spawnPlanes					= ScriptManager:getValue( "spawnPlanes" );
	numberOfPlaneWavesSpawned	= ScriptManager:getValue( "numberOfPlaneWavesSpawned" );
	lastPlaneWaveSpawnedInRound	= ScriptManager:getValue( "lastPlaneWaveSpawnedInRound" );
end


-- END Init
-- BEGIN Intro Cutscene


function startIntroCutscene()
	StartCutscene();
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE6_3_INTRO_FLAVOR_01 );
	ScriptManager:moveCameraEx( 11, 7, 0.7, 4, true );
	ScriptManager:dismissSubtitle();
	ScriptManager:wait( 1 );
	
	ScriptManager:moveCameraEx( 15, 14, 0.7, 3, true );
	ScriptManager:wait( 1 );
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE6_3_INTRO_FLAVOR_02 );
	ScriptManager:moveCameraEx( 22, 2, 0.7, 5, true );
	ScriptManager:dismissSubtitle();
	ScriptManager:wait( 1 );
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE6_3_INTRO_FLAVOR_03 );
	ScriptManager:moveCameraEx( 30,  2, 0.7, 3, true );
	ScriptManager:wait( 1 );
	ScriptManager:moveCameraEx( 31, 10, 0.7, 3, true );
	ScriptManager:dismissSubtitle();
	ScriptManager:wait( 1 );
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE6_3_INTRO_FLAVOR_04 );
	ScriptManager:moveCameraEx( 27, 15, 0.7, 2, true );
	MarkUnit( 27, 15, 1, 3 );
	ScriptManager:dismissSubtitle();
	ScriptManager:wait( 1 );
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE6_3_INTRO_FLAVOR_05 );
	ScriptManager:moveCameraEx( 3, 3, 0.7, 8, true );
	ScriptManager:dismissSubtitle();
	ScriptManager:wait( 1 );
	
	EndCutscene();
	
	startTheMission();
end

function startTheMission()
	ScriptManager:setMissionStatus( missionIds.destroyMine,	 MissionStatus.IN_PROGRESS );
	ScriptManager:setMissionStatus( missionIds.breakBarrier, MissionStatus.IN_PROGRESS );
	
	ScriptManager:setMissionProgress( missionIds.getAirfields,	0 );
	ScriptManager:setMissionProgress( missionIds.breakBarrier,	0 );
	ScriptManager:setMissionProgress( missionIds.destroyArty,	0 );
	
	--TODO: remove this function!
	--testing();
	--removeSquads();
	--ScriptManager:addOnUnitSelectedTrigger( 1, -1, -1, "showParameters", "showParameters",false)
end

function testing()
	ScriptManager:spawnUnit( 0, Units.GOTT, 15,  1, 42, AiState.WAIT );
	ScriptManager:spawnUnit( 0, Units.GOTT, 21, 16, 42, AiState.WAIT );
end

function removeSquads()
	--for i = 1, # do
	--	ScriptManager:removeUnitEx( [i], false );
	--end
end

function showParameters( selectedUnit )
	--print( string.format( "[] %s", selectedUnit:get() ) );
end


-- END Intro Cutscene
-- BEGIN Mine Shafts


function onFirstShaftEntered( tileX, tileY, playerId, unitTemplateId, unitId )
	mineShaftStuff( tileY, unitId, firstShaft, 10, 14 )
end

function onSecondShaftEntered( tileX, tileY, playerId, unitTemplateId, unitId )
	mineShaftStuff( tileY, unitId, secondShaft, 16, 5 );
	Unit:changeSquadState( 1, enemySquadIds.arty2, AiState.ATTACK );
end

function onThirdShaftEntered( tileX, tileY, playerId, unitTemplateId, unitId )
	mineShaftStuff( tileY, unitId, thirdShaft, 25, 14 );
end

function mineShaftStuff( tileY, unitId, shaftTable, moveX, moveY )
	OnShaftEntered( tileY, unitId, shaftTable );
	
	if ScriptManager:getMissionStatus( missionIds.destroyArty ) == MissionStatus.NOT_STARTED then
		ScriptManager:removeTrigger( "onArtilleryHasAttacked" );
		
		ScriptManager:storeValue( "artyX",	moveX );
		ScriptManager:storeValue( "artyY",	moveY );
		ScriptManager:storeValue( "defX",	shaftTable[2][1] );
		ScriptManager:storeValue( "defY",	shaftTable[2][2] );
		
		startArtyCutscene();
	end
end


-- END Mine Shafts
-- BEGIN Unit Stuff


function onAirplaneHasAttacked( attacker, defender )
	ScriptManager:removeTrigger( "onAirplaneHasAttacked" );
	
	ScriptManager:storeValue( "defenderX", defender:getX() );
	ScriptManager:storeValue( "defenderY", defender:getY() );
	
	ScriptManager:runForPlayer( 0, "startAirplaneCutscene" );
end

function startAirplaneCutscene()
	StartCutscene();
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE6_3_MISSION_SECONDARY_03_FLAVOR_01 );
	ScriptManager:wait( 4 );
	ScriptManager:dismissSubtitle();
	ScriptManager:wait( 1 );
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE6_3_MISSION_SECONDARY_03_FLAVOR_02 );
	ScriptManager:moveCameraEx( 2, 14, 0.7, 4, true );
	ScriptManager:dismissSubtitle();
	ScriptManager:wait( 1 );
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE6_3_MISSION_SECONDARY_03_FLAVOR_03 );
	MarkUnit( 2, 14, 1, 2 );
	ScriptManager:wait( 1 );
	ScriptManager:moveCameraEx( ScriptManager:getValue( "defenderX" ), ScriptManager:getValue( "defenderY"), 0.7, 3, true );	
	ScriptManager:dismissSubtitle();
	
	EndCutscene();
	DisablePlayerInteraction();
	
	ScriptManager:deleteValue( "defenderX" );
	ScriptManager:deleteValue( "defenderY" );
	
	ScriptManager:setMissionStatus( missionIds.getAirfields, MissionStatus.IN_PROGRESS );
	ScriptManager:wait( 2 );
	
	EnablePlayerInteraction();
end

function onUnitDestroyed( unit )
	local squadId = unit:getSquadId();
	
	if squadId == enemySquadIds.barrier1 then
		firstBlockToDestroy = missionProgress( firstBlockToDestroy, true );
		ScriptManager:storeValue( "firstBlockToDestroy", firstBlockToDestroy );
	
	elseif squadId == enemySquadIds.barrier2 then
		secondBlockToDestroy = missionProgress( secondBlockToDestroy, true );
		ScriptManager:storeValue( "secondBlockToDestroy", secondBlockToDestroy );
	
	elseif squadId == enemySquadIds.barrier3 then
		thirdBlockToDestroy = missionProgress( thirdBlockToDestroy, true );
		ScriptManager:storeValue( "thirdBlockToDestroy", thirdBlockToDestroy );
	
	elseif squadId == enemySquadIds.arty1 or squadId == enemySquadIds.artyAA1 then
		arty1ToDestroy = missionProgress( arty1ToDestroy, false );
		ScriptManager:storeValue( "arty1ToDestroy", arty1ToDestroy );
	
	elseif squadId == enemySquadIds.arty2 or squadId == enemySquadIds.artyAA2 then
		arty2ToDestroy = missionProgress( arty2ToDestroy, false );
		ScriptManager:storeValue( "arty2ToDestroy", arty2ToDestroy );
	
	elseif squadId == enemySquadIds.arty3 or squadId == enemySquadIds.artyAA3 then
		arty3ToDestroy = missionProgress( arty3ToDestroy, false );
		ScriptManager:storeValue( "arty3ToDestroy", arty3ToDestroy );
	end
end

function missionProgress( toDestroy, isBarricadeMission )
	toDestroy = toDestroy - 1;
	print( toDestroy );
	local missionId, progressFuncName;
	if isBarricadeMission then
		missionId = missionIds.breakBarrier;
		progressFuncName = "showBarricadeProgress"
		
	else -- is arty mission
		missionId = missionIds.destroyArty;
		progressFuncName = "showArtyProgress"
	end
	
	if toDestroy == 0 then
		ScriptManager:setMissionProgress( missionId, ScriptManager:getMissionProgress( missionId ) + 1 );
		ScriptManager:runForPlayer( 0, progressFuncName );
	end
	return toDestroy;
end

function showBarricadeProgress()
	showProgress( missionIds.breakBarrier, barricadesTotal, Text.T_MAP_DE6_3_MISSION_SECONDARY_02_COUNTER );
end

function showArtyProgress()
	showProgress( missionIds.destroyArty, artysTotal, Text.T_MAP_DE6_3_MISSION_SECONDARY_01_COUNTER );
end

function showProgress( missionId, totalToDestroy, counterText )
	if ScriptManager:getMissionStatus( missionId ) ~= MissionStatus.ACCOMPLISHED then
		if ScriptManager:getMissionProgress( missionId ) >= totalToDestroy then
			DisablePlayerInteraction();
			
			ScriptManager:setMissionStatus( missionId, MissionStatus.ACCOMPLISHED );
			ScriptManager:wait( 2 );
			
			EnablePlayerInteraction();
		
		else
			ShowMissionProgressPopup( ScriptManager:getMissionProgress( missionId ), counterText );
		end
	end
end

function onArtilleryHasAttacked( attacker, defender )
	ScriptManager:removeTrigger( "onArtilleryHasAttacked" );
	
	if attacker:getPlayerId() == 1 then
		ScriptManager:storeValue( "artyX",	attacker:getX() );
		ScriptManager:storeValue( "artyY",	attacker:getY() );
		ScriptManager:storeValue( "defX",	defender:getX() );
		ScriptManager:storeValue( "defY",	defender:getY() );
	
	else
		ScriptManager:storeValue( "defX",	attacker:getX() );
		ScriptManager:storeValue( "defY",	attacker:getY() );
		ScriptManager:storeValue( "artyX",	defender:getX() );
		ScriptManager:storeValue( "artyY",	defender:getY() );
	end
	
	ScriptManager:runForPlayer( 0, "startArtyCutscene" );
end

function startArtyCutscene()
	StartCutscene();
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE6_3_MISSION_SECONDARY_01_FLAVOR_01 );
	ScriptManager:moveCameraEx( ScriptManager:getValue( "artyX"), ScriptManager:getValue( "artyY"), 0.7, 3, true );
	HideDialogue();
	ScriptManager:wait( 1 );
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE6_3_MISSION_SECONDARY_01_FLAVOR_02 );
	ScriptManager:moveCameraEx( ScriptManager:getValue( "defX"), ScriptManager:getValue( "defY" ),  0.7,  3,  true );
	HideDialogue();
	
	EndCutscene();
	DisablePlayerInteraction();
	
	ScriptManager:deleteValue( "artyX" );
	ScriptManager:deleteValue( "artyY" );
	ScriptManager:deleteValue( "defX" );
	ScriptManager:deleteValue( "defY" );
	
	ScriptManager:setMissionStatus( missionIds.destroyArty, MissionStatus.IN_PROGRESS );
	ScriptManager:wait( 2 );
	
	EnablePlayerInteraction();
end

function arty2Aggressive()
	ScriptManager:removeTrigger( "arty2Aggressive" );
	Unit:changeSquadState( 1, enemySquadIds.arty2, AiState.ATTACK );
end


-- END Unit Stuff
-- BEGIN Wave Spawn


function onEnemyTurnStart()
	-- set guard areas
	ScriptManager:setGuardArea( 0, enemySquadIds.barrier2,		 20,  6, 2 );
	ScriptManager:setGuardArea( 0, enemySquadIds.artyAA1,		  8, 15, 1 );
	ScriptManager:setGuardArea( 0, enemySquadIds.artyAA2,		 17,  4, 1 );
	ScriptManager:setGuardArea( 0, enemySquadIds.artyAA3, 		 24, 14, 1 );
	ScriptManager:setGuardArea( 0, enemySquadIds.southShaftDef,	 11,  1, 2 );
	ScriptManager:setGuardArea( 0, enemySquadIds.riverForestDef, 29,  8, 2 );
	
	-- artillery should not move, so remove ap
	removeAp( arty1Ids );
	removeAp( arty2Ids );
	removeAp( arty3Ids );
	
	-- wave spawn
	if spawnPlanes == 1 then
		-- check if a plane spawns this round
		if lastPlaneWaveSpawnedInRound + planeWaveRespawnInterval <= ScriptManager:getCurrentRound() then
			local bomberIndicator =  numberOfPlaneWavesSpawned % 3;
			
			if bomberIndicator == bomberWave then
				ScriptManager:spawnUnit( 1, Units.UNIT_SU_BOMBER, 31, 11, enemySquadIds.planeWave1, AiState.AIRPLANE );
			else
				ScriptManager:spawnUnit( 1, Units.UNIT_SU_FIGHTER, 31, 11, enemySquadIds.planeWave1, AiState.AIRPLANE );
				ScriptManager:spawnUnit( 1, Units.UNIT_SU_FIGHTER, 32, 11, enemySquadIds.planeWave1, AiState.AIRPLANE );
			end
			
			numberOfPlaneWavesSpawned = numberOfPlaneWavesSpawned + 1;
			ScriptManager:storeValue( "numberOfPlaneWavesSpawned", numberOfPlaneWavesSpawned );
			
			lastPlaneWaveSpawnedInRound = ScriptManager:getCurrentRound(); -- reset round count
			ScriptManager:storeValue( "lastPlaneWaveSpawnedInRound", lastPlaneWaveSpawnedInRound );
		end
	end
end

function removeAp( idTable )
	for i = 1, #idTable do
		local unit = Unit:getUnitById( idTable[i] );
		if unit ~= nil then
			unit:setActionPoints( 0 );
		end
	end
end


-- END Wave Spawn
-- BEGIN Building Stuff


function onAirfieldDestroyed( airfield )
	onAirfieldCaptured( -1, airfield );
end

function onAirfieldCaptured( playerUnit, airfield )
	ScriptManager:setMissionProgress( missionIds.getAirfields, ScriptManager:getMissionProgress( missionIds.getAirfields ) + 1 );
	
	if airfield:getId() == waveSpawnAirfield then
		spawnPlanes = 0;
		ScriptManager:storeValue( "spawnPlanes", spawnPlanes );
	end
	
	local progress = ScriptManager:getMissionProgress( missionIds.getAirfields )
	if progress >= airfieldsToDestroy then
		ScriptManager:removeTrigger( "onAirfieldGotten" );
		DisablePlayerInteraction();
		
		ScriptManager:setMissionStatus( missionIds.getAirfields, MissionStatus.ACCOMPLISHED );
		ScriptManager:wait( 2 );
		
		EnablePlayerInteraction();
		
	else
		ShowMissionProgressPopup( progress, Text.T_MAP_DE6_3_MISSION_SECONDARY_03_COUNTER );
	end
end

function onMineDestroyed()
	DisablePlayerInteraction();
	
	ScriptManager:setMissionStatus( missionIds.destroyMine, MissionStatus.ACCOMPLISHED );
	ScriptManager:wait( 2 );
	
	EnablePlayerInteraction();
	startVictoryCutscene();
end


-- END Building Stuff
-- BEGIN Victory


function startVictoryCutscene()
	StartCutscene();
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE6_3_VICTORY_FLAVOR_01 );
	ScriptManager:wait( 4 );
	HideDialogue();
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE6_3_VICTORY_FLAVOR_02 );
	ScriptManager:wait( 4 );
	HideDialogue();
	
	EndCutscene();
	
	victoryEndGame();
end

function victoryEndGame()
	DisablePlayerInteraction();
	HideDialogue(); -- remove possible texts on the screens
	
	ScriptManager:displaySubtitle( Text.T_MISSION_VICTORY );
	ScriptManager:wait( 4 );
	HideDialogue();
	endGame();
end


-- END Victory
-- BEGIN Defeated


function checkForEnd()
	if ScriptManager:noUnitsLeft( 0 ) then
		ScriptManager:runForPlayer( 0, "missionFailed" )
	end
end

function missionFailed()
	DisablePlayerInteraction();
	HideDialogue(); -- remove possible texts on the screens
	
	ScriptManager:displaySubtitle( Text.T_MISSION_FAILED );
	ScriptManager:wait( 4 );
	HideDialogue();
	endGame();
end


-- END Defeated


function endGame()
	ScriptManager:endGame()
end