require "lua/Constants"
require "lua/Functions"
require "lua/Functions_2"
require "lua/Text"
require "lua/Units"
require "math"
--[[atm the player can hide his units in the forest around the starting position and te enemy will stop attacking when
	reinforcements arrive and storm off to fight those... what to do? target tile is not feeling right cause then they cause go
	in head first
--]]

-- mission stuff
missionIds = {
	keepUnits			= 1, -- at least one has to survive in order to win
	destroyAttackers	= 3, -- big enemy groups spawn, destroy more to win
	dontLoseUnits		= 4, -- lose less than x in order to get this
};

-- missionId, primary( boolean ), textkey, xpReward
missions = {
	{ missionIds.keepUnits,			true,	Text.T_MAP_DE7_1_MISSION_PRIMARY_01,	150 },
	{ missionIds.destroyAttackers,	true,	Text.T_MAP_DE7_1_MISSION_PRIMARY_02,	100 },
	{ missionIds.dontLoseUnits,		false,	Text.T_MAP_DE7_1_MISSION_SECONDARY_01,	 75 },
};

enemySquadIds = {
	introEnemies		=  0,
	firstAttacker		=  1,
	northCrossroadNS	=  2,
	northVillage		=  5,
	bridge				=  6, -- guard area!
	eastDepot			=  7,
	southVillage		=  8, -- guard area; attack when crossing the bridge
	industrialArea		=  9, -- guard area
	southwestDepot		= 10,
	southAttacker		= 11, -- guard area; attack when south depot is captured
	reinforcements		= 12,
	attackWave			= 13,
	maxId				= 14, -- always last!
};

--ids of the units i use to move / attack / etc in the intro cutscene
introEnemies = {
	fortifyMg	= 17,
	firstMg		= 14,
	killerMg	= 13,
	at			= 15,
	tank		= 16,
};

introPlayer = {
	mg		= 11,
	tank	= 12,
};

playerReinforcements = {
	{ Units.UNIT_DE_MP40,				2, 6 },
	{ Units.UNIT_DE_MP40,				3, 5 },
	{ Units.UNIT_DE_MP40,				5, 3 },
	{ Units.UNIT_DE_PANZERFAUST,		2, 5 },
	{ Units.UNIT_DE_PANZERFAUST,		4, 2 },
	{ Units.UNIT_DE_TANK_3,				4, 4 },
	{ Units.UNIT_DE_TANK_4,				3, 4 },
	{ Units.UNIT_DE_TIGER,				3, 3 },
	{ Units.UNIT_DE_ANTI_INFANTRY,		4, 3 },
	{ Units.UNIT_DE_SUPPLY_TRUCK,		1, 3 },
	{ Units.UNIT_DE_TRANSPORT_TRUCK,	2, 3 },
	{ Units.UNIT_DE_ARTILLERY,			2, 2 },
};

-- keep the kv1 in last position!
enemyReinforcements = {
	{ Units.UNIT_SU_RIFLE,			20, 27 },
	{ Units.UNIT_SU_RIFLE,			21, 28 },
	{ Units.UNIT_SU_RIFLE,			20, 30 },
	{ Units.UNIT_SU_TANK_T34,		19, 30 },
	{ Units.UNIT_SU_ANTI_INFANTRY,	20, 29 },
	{ Units.UNIT_SU_TANK_DESTROYER,	19, 29 },
	{ Units.UNIT_SU_KATYUSHA,		21, 29 },
	{ Units.UNIT_SU_TANK_KV1,		20, 28 },
};

enemyAttackWave1 = {
	{ Units.UNIT_SU_RIFLE,		21, 30 },
	{ Units.UNIT_SU_TANK_T34,	21, 29 },
	{ Units.UNIT_SU_RIFLE,		21, 28 },
};

enemyAttackWave2 = {
	{ Units.UNIT_SU_RIFLE,	21, 30 },
	{ Units.UNIT_SU_MORTAR,	21, 29 },
	{ Units.UNIT_SU_MORTAR,	21, 28 },
};

enemyAttackWave3 = {
	{ Units.UNIT_SU_MORTAR,			21, 28 },
	{ Units.UNIT_SU_ANTI_INFANTRY,	21, 29 },
};

southDepotId		= 46;
playerDefSquad		= 25;
playerDefUnitsTotal	= 10;
enemyUnitsTotal		= 38;
noOfWavesSpawned	=  0;
maxWaves			= 10;


-- BEGIN Init


function onGameStart()
	CreateMissions( missions );
	setTriggers();
	setGuardArea();
	storeValues();
	
	ScriptManager:setTargetTile( 1, enemySquadIds.firstAttacker, 3, 25 );
	
	ScriptManager:fortifyUnit( introEnemies.fortifyMg );
	
	ScriptManager:gainGold( 0, 1000 );
	
	-- start intro cutscene
	ScriptManager:sleep( 0, "startIntroCutscene" ); -- without sleep player turn popup shows
end

function storeValues()
	ScriptManager:storeValue( "noOfWavesSpawned",	noOfWavesSpawned );
end

function setTriggers()
	-- load all needed values after loading
	ScriptManager:addOnGameResumeTrigger( "loadValues", "loadValues", false );
	
	-- check for player units
	ScriptManager:addOnUnitDestroyedTrigger( 0, -1, -1, "onPlayerUnitDestroyed", "onPlayerUnitDestroyed" , false );
	
	-- spawn reinforcements for the player after a few turns
	ScriptManager:addOnTurnStartTrigger( 0, -1, 3, "preparePlayerReinforcementCutscene", "preparePlayerReinforcementCutscene", false );
	
	-- check for enemy units
	ScriptManager:addOnUnitDestroyedTrigger( 1, -1, -1, "onEnemyUnitDestroyed", "onEnemyUnitDestroyed", false );
	
	-- enemy guard areas / ai behaviour
	--ScriptManager:addOnTurnStartTrigger( 1, -1, -1, "setGuardArea",		"setGuardArea",		false ); -- with updated guard area no longer needed
	ScriptManager:addOnTurnStartTrigger( 1, -1,  1, "crossroadAttack",	"crossroadAttack",	false );
	ScriptManager:addOnUnitCapturedTrigger( 0, Units.UNIT_DE_HQ, southDepotId, "southDepotCaptured", "southDepotCaptured", false );
	
	-- enemy wave spawn
	ScriptManager:addOnTurnStartTrigger( 1, -1, -1, "onEnemyTurnStart", "onEnemyTurnStart",  false );
	
	-- enemy reinforcement spawn
	ScriptManager:addOnEnterTileTrigger( 15, 18, 0, -1, -1, false, "enemyReinforcementTrigger", "enemyReinforcementTrigger", false );
	ScriptManager:addOnEnterTileTrigger( 17, 19, 0, -1, -1, false, "enemyReinforcementTrigger", "enemyReinforcementTrigger", false );
	ScriptManager:addOnEnterTileTrigger( 20, 20, 0, -1, -1, false, "enemyReinforcementTrigger", "enemyReinforcementTrigger", false );
	ScriptManager:addOnEnterTileTrigger( 20, 21, 0, -1, -1, false, "enemyReinforcementTrigger", "enemyReinforcementTrigger", false );
end

function loadValues()
	noOfWavesSpawned = ScriptManager:getValue( "noOfWavesSpawned" );
end

function setGuardArea()
	ScriptManager:setGuardArea( 1, enemySquadIds.bridge,			17, 19, 1 );
	ScriptManager:setGuardArea( 1, enemySquadIds.southVillage,		19,  3, 2 );
	ScriptManager:setGuardArea( 1, enemySquadIds.industrialArea,	12,  6, 1 );
	ScriptManager:setGuardArea( 1, enemySquadIds.southAttacker,		10,  7, 1 );
end

function southDepotCaptured()
	Unit:changeSquadState( 1, enemySquadIds.southAttacker, AiState.ATTACK_AGGRESSIVE );
end


-- END Init
-- BEGIN Intro Cutscene



function startIntroCutscene()
	StartCutscene();
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE7_1_MISSION_INTRO_FLAVOR_01 );
	
	ScriptManager:moveUnitEx(	introEnemies.firstMg, 10, 24, true );
	ScriptManager:attackUnitEx( introEnemies.firstMg, introPlayer.mg, true );
	ScriptManager:wait( 1 );
	
	ScriptManager:moveUnitEx(	introEnemies.killerMg, 8, 25, true );
	ScriptManager:attackUnitEx(	introEnemies.killerMg, introPlayer.mg, true );
	ScriptManager:fortifyUnit(	introEnemies.killerMg );
	ScriptManager:wait( 1 );
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE7_1_MISSION_INTRO_FLAVOR_02 );
	
	ScriptManager:attackUnitEx( introEnemies.tank, introPlayer.tank, true );
	ScriptManager:wait( 1 );
	
	ScriptManager:attackUnitEx( introEnemies.at, introPlayer.tank, true );
	ScriptManager:dismissSubtitle();
	ScriptManager:wait( 1 );
	
	ScriptManager:moveCameraEx( 5, 23, 0.7, 2, false );
	ScriptManager:displaySubtitle( Text.T_MAP_DE7_1_MISSION_INTRO_FLAVOR_03 );
	ScriptManager:wait( 4 );
	ScriptManager:dismissSubtitle();
	ScriptManager:wait( 1 );
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE7_1_MISSION_INTRO_FLAVOR_04 );
	ScriptManager:wait( 6 );
	ScriptManager:dismissSubtitle();
	ScriptManager:wait( 1 );
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE7_1_MISSION_INTRO_FLAVOR_05 );
	ScriptManager:wait( 5 );
	ScriptManager:dismissSubtitle();
	ScriptManager:wait( 1 );
	
	EndCutscene();
	
	startTheMission();
end

function startTheMission()
	ScriptManager:setMissionStatus( missionIds.keepUnits,		MissionStatus.IN_PROGRESS );
	ScriptManager:setMissionStatus( missionIds.dontLoseUnits,	MissionStatus.IN_PROGRESS );
	
	ScriptManager:setMissionProgress( missionIds.keepUnits,			playerDefUnitsTotal );
	ScriptManager:setMissionProgress( missionIds.destroyAttackers,	enemyUnitsTotal );
	
	--TODO: remove this function!
	--testing();
	--removeSquads();
	--ScriptManager:addOnUnitSelectedTrigger( 1, -1, -1, "showParameters", "showParameters",false)
end

function testing()
	ScriptManager:spawnUnit( 0, Units.GOTT, 7,  1, 42, AiState.WAIT );
	--ScriptManager:spawnUnit( 0, Units.GOTT, 5, 29, 42, AiState.WAIT );
	
	ScriptManager:addOnEnterTileTrigger( 8, 1, 0, Units.GOTT, -1, true, "atheism", "atheism", false );
end

function atheism( tileX, tileY, playerId, unitTemplateId, unitId )
	ScriptManager:removeUnitEx( unitId, true );
end

function removeSquads()
	--for i = 1, # do
	--	ScriptManager:removeUnitEx( [i], false );
	--end
end

function showParameters( selectedUnit )
	--print( string.format( "[] %s", selectedUnit:get() ) );
end


-- END Intro Cutscene
-- BEGIN Player Defense


function onPlayerUnitDestroyed( unit )
	local squadId = unit:getSquadId();
	
	if unit:getSquadId() == playerDefSquad then
		local progress = ScriptManager:getMissionProgress( missionIds.keepUnits );
		ScriptManager:setMissionProgress( missionIds.keepUnits, progress - 1 );
		print( "player units left " .. progress );
		
		-- need to check this here because if two units die in an enemy turn it might get skipped
		if progress - 1 < playerDefUnitsTotal / 2 and ScriptManager:getMissionStatus( missionIds.dontLoseUnits ) == MissionStatus.IN_PROGRESS then
			ScriptManager:runForPlayer( 0, "prepareDontLoseUnitsFailed" );
		
		else -- no need to show this when the secondary fails
			ScriptManager:runForPlayer( 0, "prepareKeepUnitsProgress" );
		end
	end
end

function prepareDontLoseUnitsFailed()
	table.insert( cutsceneQueue, showDontLoseUnitsFailed );
	SubtitleQueue();
end

function showDontLoseUnitsFailed()
	DisablePlayerInteraction();
	
	ScriptManager:setMissionStatus( missionIds.dontLoseUnits, MissionStatus.FAILED );
	ScriptManager:wait( 2 );
	
	EnablePlayerInteraction();
end

function prepareKeepUnitsProgress()
	table.insert( cutsceneQueue, showKeepUnitsProgress );
	SubtitleQueue();
end

function showKeepUnitsProgress()
	if ScriptManager:getMissionProgress( missionIds.keepUnits ) <= 0 then
		DisablePlayerInteraction();
		
		ScriptManager:setMissionStatus( missionIds.keepUnits, MissionStatus.FAILED );
		ScriptManager:wait( 2 );
		
		EnablePlayerInteraction();
		missionFailed();
	
	else  -- TODO: if too many units, only show each 3rd / 5th unit until < 5
		ShowMissionProgressPopup( ScriptManager:getMissionProgress( missionIds.keepUnits ), Text.T_MAP_DE7_1_MISSION_PRIMARY_01_COUNTER );
	end
end

function preparePlayerReinforcementCutscene()
	ScriptManager:removeTrigger( "preparePlayerReinforcementCutscene" );
	table.insert( cutsceneQueue, startPlayerReinforcementCutscene );
	SubtitleQueue();
end

function startPlayerReinforcementCutscene()
	DisablePlayerInteraction();
	
	for i = 1, #playerReinforcements do
		ScriptManager:spawnUnit( 0, playerReinforcements[i][1], playerReinforcements[i][2], playerReinforcements[i][3], 0, 0 );
	end
	
	EnablePlayerInteraction();
	StartCutscene();
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE7_1_MISSION_PRIMARY_02_FLAVOR_01 );
	ScriptManager:moveCameraEx( 3, 4, 0.7, 6, true );
	ScriptManager:dismissSubtitle();
	ScriptManager:wait( 1 );
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE7_1_MISSION_PRIMARY_02_FLAVOR_02 );
	ScriptManager:wait( 5 );
	ScriptManager:dismissSubtitle();
	ScriptManager:wait( 1 );
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE7_1_MISSION_PRIMARY_02_FLAVOR_03 );
	ScriptManager:wait( 4 );
	ScriptManager:dismissSubtitle();
	
	EndCutscene();
	DisablePlayerInteraction();
	
	ScriptManager:setMissionStatus( missionIds.destroyAttackers, MissionStatus.IN_PROGRESS );
	ScriptManager:wait( 2 );
	
	EnablePlayerInteraction();
end


-- END Player Defense
-- BEGIN Enemy Waves


function crossroadAttack()
	ScriptManager:removeTrigger( "crossroadAttack" );
	Unit:changeSquadState( 1, enemySquadIds.northCrossroadNS, AiState.ATTACK_AGGRESSIVE );
	--ScriptManager:setTargetTile( 1, enemySquadIds.northCrossroadNS, 3, 25 );
end

function onEnemyTurnStart( playerId, turnId, roundId )
	-- spawn Enemies every third round up until a certain number waves has spawned
	if noOfWavesSpawned <= maxWaves and roundId % 3 == 2 then
		local rng = math.random( 1, 3 );
		local spawnTable = _G["enemyAttackWave" .. rng];
		
		for i = 1, #spawnTable do
			local spawnUnit = spawnTable[i];
			ScriptManager:spawnUnit( 1, spawnUnit[1], spawnUnit[2], spawnUnit[3], enemySquadIds.attackWave, AiState.ATTACK_AGGRESSIVE );
		end
		
		noOfWavesSpawned = noOfWavesSpawned + 1;
		ScriptManager:storeValue( "noOfWavesSpawned", noOfWavesSpawned );
		
		ScriptManager:setTargetTile( 1, enemySquadIds.attackWave, 3, 25 );
		
		ScriptManager:setMissionProgress( missionIds.destroyAttackers,
										  ScriptManager:getMissionProgress( missionIds.destroyAttackers ) + #spawnTable );
	end
end

function enemyReinforcementTrigger()
	ScriptManager:removeTrigger( "enemyReinforcementTrigger" );
	ScriptManager:removeTrigger( "onEnemyTurnStart" ); -- stop normal wave spawn, cause now spawns the big last wave
	
	-- if player was not there, this squad will attack from behind the player
	Unit:changeSquadState( 1, enemySquadIds.southVillage, AiState.ATTACK_AGGRESSIVE );
	Unit:changeSquadState( 1, enemySquadIds.northVillage, AiState.ATTACK_AGGRESSIVE );
	
	ScriptManager:runForPlayer( 1, "spawnEnemyReinforcements" );
end

function spawnEnemyReinforcements()
	local unit;
	for i = 1, #enemyReinforcements do
		unit = ScriptManager:spawnUnit( 1, enemyReinforcements[i][1], enemyReinforcements[i][2], enemyReinforcements[i][3],
								 enemySquadIds.reinforcements, AiState.ATTACK_AGGRESSIVE );
	end
	
	-- the kv1 is the last unit in the table
	ScriptManager:storeValue( "kv1X", unit:getX() );
	ScriptManager:storeValue( "kv1Y", unit:getY() );
	
	ScriptManager:setMissionProgress( missionIds.destroyAttackers,
									  ScriptManager:getMissionProgress( missionIds.destroyAttackers ) + #enemyReinforcements );
	
	ScriptManager:runForPlayer( 0, "prepareEnemyReinforcements" );
end

function prepareEnemyReinforcements()
	table.insert( cutsceneQueue, showEnemyReinforcements );
	SubtitleQueue();
end

function showEnemyReinforcements()
	StartCutscene();
	
	ScriptManager:displaySubtitle( Text.T_MAP_3_2_ENEMY_REINFORCEMENTS );
	ScriptManager:moveCameraEx( ScriptManager:getValue( "kv1X"), ScriptManager:getValue( "kv1Y"), 0.7, 4, true );
	ScriptManager:dismissSubtitle();
	
	EndCutscene();
end


-- END Enemy Waves
-- BEGIN Enemy Unit Destruction


function onEnemyUnitDestroyed( unit )
	ScriptManager:setMissionProgress( missionIds.destroyAttackers, ScriptManager:getMissionProgress( missionIds.destroyAttackers ) - 1 );
	
	print( "enemies left " .. ScriptManager:getMissionProgress( missionIds.destroyAttackers ) );
	
	ScriptManager:runForPlayer( 0, "prepareAttackerMissionProgress" );
	
end

function prepareAttackerMissionProgress()
	table.insert( cutsceneQueue, showAttackerMissionProgress );
	SubtitleQueue();
end

function showAttackerMissionProgress()
	local progress = ScriptManager:getMissionProgress( missionIds.destroyAttackers );
	if progress <= 0 then
		DisablePlayerInteraction()
		
		ScriptManager:setMissionStatus( missionIds.keepUnits,		 MissionStatus.ACCOMPLISHED );
		ScriptManager:setMissionStatus( missionIds.destroyAttackers, MissionStatus.ACCOMPLISHED );
		ScriptManager:wait( 2 );
		
		EnablePlayerInteraction();
		startVictoryCutscene();
	
	elseif ( progress % 5 == 0 or progress <= 10 ) and -- only show progress every 5th enemy / when 5 or less enemies remain
		ScriptManager:getMissionStatus( missionIds.destroyAttackers ) == MissionStatus.IN_PROGRESS  then
		ShowMissionProgressPopup( ScriptManager:getMissionProgress( missionIds.destroyAttackers ), Text.T_MAP_DE7_1_MISSION_PRIMARY_02_COUNTER );
	end
	
	-- if only a few enemies remain, set them all to aggro to avoid the player looking all over the map for them
	if progress <= 7 then
		for i = 1, enemySquadIds.maxId do
			Unit:changeSquadState( 1, i, AiState.ATTACK_AGGRESSIVE );
			ScriptManager:setTargetTile( 1, i, 3, 25 );
		end
		
		-- iterates over each entry in the table only save way to do it 
		--[[for e, i in pairs(enemySquadIds) do
			if i > 0 then
				print( e .. " " .. i .. " " .. " blubb" );
				Unit:changeSquadState( 1, i, AiState.ATTACK_AGGRESSIVE );
				ScriptManager:setTargetTile( 1, i, 3, 25 );
			end
		end]]
	end
end


-- END Enemy Unit Destruction
-- BEGIN Victory


function startVictoryCutscene()
	StartCutscene();
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE7_1_MISSION_VICTORY_FLAVOR_01 );
	ScriptManager:wait( 5 );
	HideDialogue();
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE7_1_MISSION_VICTORY_FLAVOR_02 );
	ScriptManager:wait( 5 );
	HideDialogue();
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE7_1_MISSION_VICTORY_FLAVOR_03 );
	ScriptManager:wait( 5 );
	HideDialogue();
	
	EndCutscene();
	victoryEndGame();
end

function victoryEndGame()
	DisablePlayerInteraction();
	HideDialogue(); -- remove possible texts on the screens
	
	if	ScriptManager:getMissionStatus( missionIds.dontLoseUnits ) ~= MissionStatus.FAILED then
		ScriptManager:setMissionStatus( missionIds.dontLoseUnits, MissionStatus.ACCOMPLISHED );
	end
	
	ScriptManager:displaySubtitle( Text.T_MISSION_VICTORY );
	ScriptManager:wait( 4 );
	HideDialogue();
	endGame();
end


-- END Victory
-- BEGIN Defeated


function missionFailed()
	DisablePlayerInteraction();
	HideDialogue(); -- remove possible texts on the screens
	
	ScriptManager:displaySubtitle( Text.T_MISSION_FAILED );
	ScriptManager:wait( 4 );
	HideDialogue();
	endGame();
end


-- END Defeated


function endGame()
	ScriptManager:endGame()
end