require "lua/Constants"
require "lua/Functions"
require "lua/Functions_2"
require "lua/Text"
require "lua/Units"
require "math"


-- mission stuff
missionIds = {
	capNorthCity	= 1, -- airfield + depot
	capMidAirfield	= 2,
	capFactory		= 3,
	defBuildings	= 4,
	capOutpost		= 5,
	destroyKV1		= 6,
	destroyBomber	= 7,
};

-- missionId, primary( boolean ), textkey, xpReward
missions = {
	{ missionIds.capNorthCity,	 true,	Text.T_MAP_DE7_2_MISSION_PRIMARY_01,	175 },
	{ missionIds.capMidAirfield, true,	Text.T_MAP_DE7_2_MISSION_PRIMARY_02,	100 },
	{ missionIds.capFactory,	 true,	Text.T_MAP_DE7_2_MISSION_PRIMARY_03,	100 },
	{ missionIds.defBuildings,	 true,	Text.T_MAP_DE7_2_MISSION_PRIMARY_04,	125 },
	{ missionIds.capOutpost,	 false,	Text.T_MAP_DE7_2_MISSION_SECONDARY_01,	 50 },
	{ missionIds.destroyKV1,	 false,	Text.T_MAP_DE7_2_MISSION_SECONDARY_02,	 75 },
	{ missionIds.destroyBomber,	 false,	Text.T_MAP_DE7_2_MISSION_SECONDARY_03,	 75 },
};

enemySquadIds = {
	outpost				=  1;
	midAirfield			=  2,
	cityAirfield		=  3,
	cityDepot			=  4,
	factory				=  5,
	midVillage			=  6,-- guard area
	eastIce				=  7,
	midCauseway			=  8,
	westForest			=  9,-- guard area
	bomberAttack		= 10,
	planeAttack			= 11,
	cityAttackWave		= 12,
	cityPlaneWave		= 13,
	midPlaneWave		= 14,
	factoryAttackWave	= 15,
};

cityAirfieldId	= 3;
cityDepotId		= 1;
midAirfieldId	= 2;
factoryId		= 4;
outpostId		= 0;

kvsToDestroy		= 4;
bombersToDestroy	= 3;

cityPos			= {  4, 32 }; -- pos for camera in cutscenes
cityDepotPos	= {  4, 30 }; -- pos für city attack wave spawn
airfieldPos		= {  9, 21 };
factoryPos		= { 14, 31 };

midAirfieldTiles	= { { 8, 21 }, { 9, 22 }, };
cityAirfieldTiles	= { { 5, 34 }, { 5, 33 }, { 5, 32 }, };

spawnAttackWaves		= 0;
cityAttackWaveRespawn	= 6;
cityPlaneWaveRespawn	= 5;
factoryAttackRespawn	= 6;
midPlaneWaveRespawn		= 5;

-- every third wave will be a bomber spawn. i use % 3 == bomberIndicator to check when it spawns to avoid global bomber spawn
midPlaneWaveBomberIndicator		= 0;
midPlaneWavesSpawned			= 0;
cityPlaneWaveBomberIndicator	= 0;
cityPlaneWavesSpawned			= 0;

cityWave1 = {
	Units.UNIT_SU_RIFLE,
	Units.UNIT_SU_RIFLE,
	Units.UNIT_SU_MORTAR,
};

cityWave2 = {
	Units.UNIT_SU_ANTI_INFANTRY,
	Units.UNIT_SU_TANK_DESTROYER,
};

cityWave3 = {
	Units.UNIT_SU_TANK_KV1,
	Units.UNIT_SU_RIFLE,
};

factoryWave1 = {
	Units.UNIT_SU_RIFLE,
	Units.UNIT_SU_KATYUSHA,
}

factoryWave2 = {
	Units.UNIT_SU_TANK_DESTROYER,
	Units.UNIT_SU_TANK_T34,
}

factoryWave3 = {
	Units.UNIT_SU_MORTAR,
	Units.UNIT_SU_ANTI_INFANTRY,
	Units.UNIT_SU_MOBILE_AA,
}


-- BEGIN Init


function onGameStart()
	CreateMissions( missions );
	setTriggers();
	storeValues();
	
	ScriptManager:setGuardArea( 1, enemySquadIds.midVillage, 9, 11, 3 );
	ScriptManager:setGuardArea( 1, enemySquadIds.westForest, 2, 18, 2 );
	
	ScriptManager:gainGold( 0, 1500 );
	
	-- start intro cutscene
	ScriptManager:sleep( 0, "startIntroCutscene" ); -- without sleep player turn popup shows
end

function storeValues()
	ScriptManager:storeValue( "midPlaneWavesSpawned",	midPlaneWavesSpawned );
	ScriptManager:storeValue( "cityPlaneWavesSpawned",	cityPlaneWavesSpawned );
	ScriptManager:storeValue( "spawnAttackWaves",		spawnAttackWaves );
end

function setTriggers()
	-- load all needed values after loading
	ScriptManager:addOnGameResumeTrigger( "loadValues", "loadValues", false );
	
	-- check if the player has units left
	ScriptManager:addOnUnitDestroyedTrigger( 0, -1, -1, "checkForEnd", "checkForEnd" , false );
	
	-- trigger for all the buildings
	ScriptManager:addOnUnitCapturedTrigger(		 0, -1, cityAirfieldId,	"onCityBuildingCaptured",		"onCityBuildingCaptured",		false );
	ScriptManager:addOnUnitCapturedTrigger(		 0, -1, cityDepotId,	"onCityBuildingCaptured",		"onCityBuildingCaptured",		false );
	ScriptManager:addOnUnitHasAttackedTrigger(	-1, -1, cityAirfieldId,	"ProtectTheBuildingHintSingle",	"ProtectTheBuildingHintSingle",	false );
	ScriptManager:addOnUnitHasAttackedTrigger(	-1, -1, cityDepotId,	"ProtectTheBuildingHintSingle",	"ProtectTheBuildingHintSingle",	false );
	ScriptManager:addOnUnitDestroyedTrigger(	-1, -1, cityAirfieldId,	"transitionToFail",				"transitionToFail",				false );
	ScriptManager:addOnUnitDestroyedTrigger(	-1, -1, cityDepotId,	"transitionToFail",				"transitionToFail",				false );
	
	ScriptManager:addOnUnitCapturedTrigger(		 0, -1, midAirfieldId, "onMidAirfieldCaptured",			"onMidAirfieldCaptured",		false );
	ScriptManager:addOnUnitHasAttackedTrigger(	-1, -1, midAirfieldId, "ProtectTheBuildingHintSingle",	"ProtectTheBuildingHintSingle",	false );
	ScriptManager:addOnUnitDestroyedTrigger(	-1, -1, midAirfieldId, "transitionToFail",				"transitionToFail",				false );
	
	ScriptManager:addOnUnitCapturedTrigger(		 0, -1, factoryId, "onFactoryCaptured",				"onFactoryCaptured",			false );
	ScriptManager:addOnUnitHasAttackedTrigger(	-1, -1, factoryId, "ProtectTheBuildingHintSingle",	"ProtectTheBuildingHintSingle",	false );
	ScriptManager:addOnUnitDestroyedTrigger(	-1, -1, factoryId, "transitionToFail",				"transitionToFail",				false );
	
	ScriptManager:addOnUnitCapturedTrigger(  0, -1, outpostId, "onOutpostCaptured",		"onOupostCaptured",		false );
	ScriptManager:addOnUnitDestroyedTrigger( 1, -1, outpostId, "onOutpostDestroyed",	"onOutpostDestroyed",	false );
	
	-- trigger for north city capture
	ScriptManager:addOnEnterTileTrigger( 6, 28, 0, -1, -1, true, "startNorthCityCutscene", "startNorthCityCutscene", false );
	ScriptManager:addOnEnterTileTrigger( 6, 26, 0, -1, -1, true, "startNorthCityCutscene", "startNorthCityCutscene", false );
	for x = 1, 5 do
		ScriptManager:addOnEnterTileTrigger( x, 21, 0, -1, -1, true, "startNorthCityCutscene", "startNorthCityCutscene", false );
	end
	
	-- trigger for factory capture
	ScriptManager:addOnEnterTileTrigger( 11, 30, 0, -1, -1, true, "startFactoryCutscene", "startFactoryCutscene", false );
	ScriptManager:addOnEnterTileTrigger( 12, 22, 0, -1, -1, true, "startFactoryCutscene", "startFactoryCutscene", false );
	ScriptManager:addOnEnterTileTrigger( 15, 20, 0, -1, -1, true, "startFactoryCutscene", "startFactoryCutscene", false );
	
	-- trigger for secondary destroy missions
	ScriptManager:addOnUnitHasAttackedTrigger( -1, Units.UNIT_SU_TANK_KV1, -1, "onKV1HasAttacked",	"onKV1HasAttacked",	false );
	ScriptManager:addOnUnitDestroyedTrigger(	1, Units.UNIT_SU_TANK_KV1, -1, "onKV1Destroyed",	"onKV1Destroyed",	false );
	
	ScriptManager:addOnUnitHasAttackedTrigger( -1, Units.UNIT_SU_BOMBER, -1, "onBomberHasAttacked",	"onBomberHasAttacked",	false );
	ScriptManager:addOnUnitDestroyedTrigger( 	1, Units.UNIT_SU_BOMBER, -1, "onBomberDestroyed",	"onBomberDestroyed",	false );
	
	-- trigger for waves
	ScriptManager:addOnTurnStartTrigger(  1, -1, -1,				"onEnemyTurnStart",		"onEnemyTurnStart",		false );
	ScriptManager:addOnEnterTileTrigger( -1, 19,  0, -1, -1, false,	"triggerAttackWaves",	"triggerAttackWaves",	false );
end

function loadValues()
	midPlaneWavesSpawned	= ScriptManager:getValue( "midPlaneWavesSpawned" );
	cityPlaneWavesSpawned	= ScriptManager:getValue( "cityPlaneWavesSpawned" );
	spawnAttackWaves		= ScriptManager:getValue( "spawnAttackWaves" );
end


-- END Init
-- BEGIN Intro Cutscene


function startIntroCutscene()
	StartCutscene();
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE7_2_MISSION_INTRO_FLAVOR_01 );
	ScriptManager:moveCameraEx( factoryPos[1], factoryPos[2], 0.7, 6, true );
	ScriptManager:wait( 1 );
	HideDialogue();
	ScriptManager:wait( 1 );
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE7_2_MISSION_INTRO_FLAVOR_02 );
	ScriptManager:moveCameraEx( cityPos[1], cityPos[2], 0.7, 5, true );
	ScriptManager:wait( 1 );
	HideDialogue();
	ScriptManager:wait( 1 );
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE7_2_MISSION_INTRO_FLAVOR_03 );
	ScriptManager:moveCameraEx( airfieldPos[1], airfieldPos[2], 0.7, 5, true );
	HideDialogue();
	ScriptManager:wait( 1 );
	
	ScriptManager:revealFogOfWar(	airfieldPos[1], airfieldPos[2], 1, true );
	ScriptManager:showMarkerAt(		airfieldPos[1], airfieldPos[2] );
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE7_2_MISSION_INTRO_FLAVOR_04 );
	ScriptManager:wait( 6 );
	HideDialogue();
	
	ScriptManager:removeAllMarkers()
	ScriptManager:revealFogOfWar( airfieldPos[1], airfieldPos[2], 1, false );
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE7_2_MISSION_INTRO_FLAVOR_05 );
	ScriptManager:moveCameraEx( 2, 9, 0.7, 5, true );
	
	ScriptManager:revealFogOfWar(	2, 9, 1, true );
	ScriptManager:showMarkerAt(		2, 9 );
	ScriptManager:wait( 3 );
	ScriptManager:removeAllMarkers()
	ScriptManager:revealFogOfWar( 2, 9, 1, false );
	
	HideDialogue();
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE7_2_MISSION_INTRO_FLAVOR_06 );
	ScriptManager:moveCameraEx( 11, 3, 0.7, 4, true );
	ScriptManager:wait( 1 );
	HideDialogue();
	ScriptManager:wait( 1 );
	
	EndCutscene();
	
	startTheMission();
end

function startTheMission()
	ScriptManager:setMissionStatus( missionIds.capMidAirfield,	MissionStatus.IN_PROGRESS );
	ScriptManager:setMissionStatus( missionIds.capOutpost,	 	MissionStatus.IN_PROGRESS );
	
	ScriptManager:setMissionProgress( missionIds.capNorthCity,	0 );
	ScriptManager:setMissionProgress( missionIds.destroyBomber,	0 );
	ScriptManager:setMissionProgress( missionIds.destroyKV1,	0 );
	
	--TODO: remove this function!
	--testing();
	--removeSquads();
	--ScriptManager:addOnUnitSelectedTrigger( 1, -1, -1, "showParameters", "showParameters",false)
end

function testing()
	--ScriptManager:spawnUnit( 0, Units.GOTT, 9, 20, 42, AiState.WAIT );
	--ScriptManager:spawnUnit( 0, Units.GOTT, 4, 31, 42, AiState.WAIT );
	--ScriptManager:spawnUnit( 0, Units.GOTT, 4, 32, 42, AiState.WAIT );
	--ScriptManager:spawnUnit( 0, Units.GOTT, 15, 30, 42, AiState.WAIT );
	--ScriptManager:spawnUnit( 0, Units.GOTT, 2, 9, 42, AiState.WAIT );
	ScriptManager:spawnUnit( 0, Units.GOTT, 4, 1, 42, AiState.WAIT );
end

function removeSquads()
	--for i = 1, # do
	--	ScriptManager:removeUnitEx( [i], false );
	--end
end

function showParameters( selectedUnit )
	--print( string.format( "[] %s", selectedUnit:get() ) );
end


-- END Intro Cutscene
-- BEGIN Enemy Waves


function triggerAttackWaves()
	ScriptManager:removeTrigger( "triggerAttackWaves" );
	
	spawnAttackWaves = 1;
	ScriptManager:storeValue( "spawnAttackWaves", spawnAttackWaves );
end

--function spawnPlaneWave( roundId, respawn, spawnTurn, spawnCounter, bomberIndicator, airfieldTiles, spquadId )
function spawnPlaneWave( roundId, spawnTurn, name, squadId )
	local wavesSpawned = _G[name .. "PlaneWavesSpawned"];
	
	if roundId % _G[name .. "PlaneWaveRespawn"] == spawnTurn then -- i dont want them to spawn at the very start / at the same time
		local airfield = _G[name .. "AirfieldTiles"];
		
		if wavesSpawned % 2 == _G[name .. "PlaneWaveBomberIndicator"] then
			ScriptManager:spawnUnit( 1, Units.UNIT_SU_BOMBER, airfield[1][1], airfield[1][2], squadId, AiState.AIRPLANE );
		
		else
			for i = 1, 2 do
				ScriptManager:spawnUnit( 1, Units.UNIT_SU_FIGHTER, airfield[i][1], airfield[i][2], squadId, AiState.AIRPLANE );
			end
		end
		
		wavesSpawned = wavesSpawned + 1;
	end
	
	return wavesSpawned;
end

function onEnemyTurnStart( playerId, turnId, roundId )
	-- guard areas -- with updated guard are no longer needed
	--ScriptManager:setGuardArea( 1, enemySquadIds.midVillage, 9, 11, 3 );
	--ScriptManager:setGuardArea( 1, enemySquadIds.westForest, 2, 18, 2 );
	
	-- mid plane wave
	if ScriptManager:getMissionStatus( missionIds.capMidAirfield ) ~= MissionStatus.ACCOMPLISHED then
		midPlaneWavesSpawned = spawnPlaneWave( roundId, 2, "mid", enemySquadIds.midPlaneWave );
		ScriptManager:storeValue( "midPlaneWavesSpawned", midPlaneWavesSpawned );
	end
	
	-- city plane wave
	local airfield = Unit:getUnitAt( 4, 33 );
	if airfield:getPlayerId() == 1 then -- no need to check for nil cause then its game over
		cityPlaneWavesSpawned = spawnPlaneWave( roundId, 4, "city", enemySquadIds.cityPlaneWave );
		ScriptManager:storeValue( "cityPlaneWavesSpawned", cityPlaneWavesSpawned );
	end
	
	if spawnAttackWaves == 1 then
		local rng, spawnTable;
		
		-- city attack wave
		local depot = Unit:getUnitAt( 4, 30 );
		if depot:getPlayerId() == 1 then -- no need to check for nil cause then its game over
			if roundId % cityAttackWaveRespawn == 0 then
				rng = math.random( 1, 3 );
				spawnTable = _G["cityWave" .. rng];
				
				for i = 1, #spawnTable do
					ScriptManager:spawnUnit( 1, spawnTable[i], cityDepotPos[1], cityDepotPos[2],
											enemySquadIds.cityAttackWave, AiState.ATTACK_AGGRESSIVE );
				end
			end
		end
		
		-- factory attack wave
		if ScriptManager:getMissionStatus( missionIds.capFactory ) ~= MissionStatus.ACCOMPLISHED then
			if roundId % factoryAttackRespawn == 2 then
				rng = math.random( 1, 3 );
				spawnTable = _G["factoryWave" .. rng];
				
				for i = 1, #spawnTable do
					ScriptManager:spawnUnit( 1, spawnTable[i], factoryPos[1], factoryPos[2],
											enemySquadIds.factoryAttackWave, AiState.ATTACK_AGGRESSIVE );
				end
			end
		end
	end
end


-- END Enemy Waves
-- BEGIN Secondary Kill Objectives


function genericDestroyCutscene( saveName, text, missionId )
	print( saveName .. "Y" );
	StartCutscene()
	
	ScriptManager:displaySubtitle( text );
	
	local unit = Unit:getUnitById( ScriptManager:getValue( saveName .. "Id" ) );
	if unit ~= nil then
		local x = unit:getX();
		local y = unit:getY();
		ScriptManager:moveCameraEx( x, y, 0.7,  3,  true );
		
		ScriptManager:revealFogOfWar(	x, y, 1, true );
		ScriptManager:showMarkerAt(		x, y );
		ScriptManager:wait( 3 );
		ScriptManager:removeAllMarkers();
		ScriptManager:revealFogOfWar( x, y, 1, false );
	else
		ScriptManager:wait( 3 );
	end
	
	HideDialogue();
	ScriptManager:moveCameraEx( ScriptManager:getValue( "playerX" ), ScriptManager:getValue( "playerY" ), 0.7, 3,  true );
	
	EndCutscene()
	SetMissionStatus( missionId, MissionStatus.IN_PROGRESS );
	ScriptManager:deleteValue( saveName .. "X" );
	ScriptManager:deleteValue( saveName .. "Y" );
end

function onKV1HasAttacked( attacker, defender )
	ScriptManager:removeTrigger( "onKV1HasAttacked" );
	
	if attacker:getTemplateId() == Units.UNIT_SU_TANK_KV1 then
		ScriptManager:storeValue( "KV1Id",	 attacker:getId() );
		ScriptManager:storeValue( "playerX", defender:getX() );
		ScriptManager:storeValue( "playerY", defender:getY() );
	
	else
		ScriptManager:storeValue( "KV1Id",	 defender:getId() );
		ScriptManager:storeValue( "playerX", attacker:getX() );
		ScriptManager:storeValue( "playerY", attacker:getY() );
	end
	
	ScriptManager:runForPlayer( 0, "prepareKV1Cutscene" );
end

function prepareKV1Cutscene()
	table.insert( cutsceneQueue, startKV1Cutscene );
	CutsceneQueue();
end

function startKV1Cutscene()
	genericDestroyCutscene( "KV1", Text.T_MAP_DE7_2_MISSION_SECONDARY_02_FLAVOR_01, missionIds.destroyKV1 )
end

function onKV1Destroyed()
	ScriptManager:setMissionProgress( missionIds.destroyKV1, ScriptManager:getMissionProgress( missionIds.destroyKV1 ) + 1 );
	
	ScriptManager:runForPlayer( 0, "showKV1MissionProgress" );
end

function showKV1MissionProgress()
	local progress = ScriptManager:getMissionProgress( missionIds.destroyKV1 );
	if progress >= kvsToDestroy then
		ScriptManager:removeTrigger( "onKV1Destroyed")
		SetMissionStatus( missionIds.destroyKV1, MissionStatus.ACCOMPLISHED );
		
	else
		ShowMissionProgressPopup( progress, Text.T_MAP_DE7_2_MISSION_SECONDARY_02_COUNTER );
	end
end

function onBomberHasAttacked( attacker, defender )
	ScriptManager:removeTrigger( "onBomberHasAttacked" );
	
	if attacker:getTemplateId() == Units.UNIT_SU_BOMBER then
		ScriptManager:storeValue( "bomberId", attacker:getId() );
		ScriptManager:storeValue( "playerX", defender:getX() );
		ScriptManager:storeValue( "playerY", defender:getY() );
	
	else
		ScriptManager:storeValue( "bomberId", defender:getId() );
		ScriptManager:storeValue( "playerX", attacker:getX() );
		ScriptManager:storeValue( "playerY", attacker:getY() );
	end
	
	ScriptManager:runForPlayer( 0, "prepareBomberCutscene" );
end

function prepareBomberCutscene()
	table.insert( cutsceneQueue, startBomberCutscene );
	CutsceneQueue()
end

function startBomberCutscene()
	genericDestroyCutscene( "bomber", Text.T_MAP_DE7_2_MISSION_SECONDARY_03_FLAVOR_01, missionIds.destroyBomber );
end

function onBomberDestroyed()
	ScriptManager:setMissionProgress( missionIds.destroyBomber, ScriptManager:getMissionProgress( missionIds.destroyBomber ) + 1 );
	
	ScriptManager:runForPlayer( 0, "showBomberMissionProgress" );
end

function showBomberMissionProgress()
	local progress = ScriptManager:getMissionProgress( missionIds.destroyBomber );
	
	if progress >= bombersToDestroy then
		ScriptManager:removeTrigger( "onBomberDestroyed")
		SetMissionStatus( missionIds.destroyBomber, MissionStatus.ACCOMPLISHED );
	
	else
		ShowMissionProgressPopup( progress, Text.T_MAP_DE7_2_MISSION_SECONDARY_03_COUNTER );
	end
end


-- END Secondary Kill Objectives
-- BEGIN Building Mission Handling & Cutscenes


function genericAlmostThereCutscene( tileX, tileY, target )
	local movePosTable, missionId, textKey, contentFunction
	if target == "NorthCity" then
		movePosTable	= cityPos;
		missionId		= missionIds.capNorthCity;
		textKey			= Text.T_MAP_DE7_2_MISSION_PRIMARY_01_FLAVOR_01;
		contentFunction	= northCityCutsceneContent;
		
	else -- "Factory"
		movePosTable	= factoryPos;
		missionId		= missionIds.capFactory;
		textKey			= Text.T_MAP_DE7_2_MISSION_PRIMARY_03_FLAVOR_01
		contentFunction	= factoryCutsceneContent;
		
	end
	
	ScriptManager:removeTrigger( "start" .. target .. "Cutscene" );
	ScriptManager:wait( 0.25 );
	StartCutscene();
	
	ScriptManager:displaySubtitle( textKey );
	ScriptManager:moveCameraEx( movePosTable[1], movePosTable[2], 0.6, 4, true );
	HideDialogue();
	ScriptManager:wait( 1 );
	
	contentFunction( tileX, tileY );
	
	EndCutscene();
	SetMissionStatus( missionId, MissionStatus.IN_PROGRESS );
end

function startNorthCityCutscene( tileX, tileY )
	genericAlmostThereCutscene( tileX, tileY, "NorthCity" );
end

function startFactoryCutscene( tileX, tileY )
	genericAlmostThereCutscene( tileX, tileY, "Factory" );
end

function northCityCutsceneContent( tileX, tileY )
	ScriptManager:revealFogOfWar( 4, 33, 1, true );
	ScriptManager:revealFogOfWar( 4, 30, 1, true );
	
	ScriptManager:showMarkerAt( 4, 33 );
	ScriptManager:showMarkerAt( 4, 30 );
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE7_2_MISSION_PRIMARY_01_FLAVOR_02 );
	ScriptManager:wait( 4 );
	HideDialogue();
	
	ScriptManager:removeAllMarkers();
	
	ScriptManager:revealFogOfWar( 4, 33, 1, false );
	ScriptManager:revealFogOfWar( 4, 30, 1, false );
	ScriptManager:wait( 1 );
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE7_2_MISSION_PRIMARY_01_FLAVOR_03 );
	ScriptManager:moveCameraEx( tileX, tileY, 0.6, 4, true );
	HideDialogue();
	ScriptManager:wait( 1 );
end

function factoryCutsceneContent( tileX, tileY )
	local movePosX = factoryPos[1];
	local movePosY = factoryPos[2];
	
	ScriptManager:revealFogOfWar(	movePosX, movePosY, 1, true );
	ScriptManager:showMarkerAt(		movePosX, movePosY );
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE7_2_MISSION_PRIMARY_03_FLAVOR_02 );
	ScriptManager:wait( 4 );
	HideDialogue();
	
	ScriptManager:removeAllMarkers();
	ScriptManager:revealFogOfWar( movePosX, movePosY, 1, false );
	ScriptManager:wait( 1 );
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE7_2_MISSION_PRIMARY_03_FLAVOR_03 );
	ScriptManager:moveCameraEx( tileX, tileY, 0.6, 4, true );
	HideDialogue();
	ScriptManager:wait( 1 );
end


-- END Building Mission Handling & Cutscenes
-- BEGIN Building Capture


function onMidAirfieldCaptured()
	SetMissionStatus( missionIds.capMidAirfield, MissionStatus.ACCOMPLISHED );
	checkForVictory(  missionIds.capMidAirfield );
end

function onOutpostCaptured()
	ScriptManager:removeTrigger( "onOutpostCaptured" );
	ScriptManager:removeTrigger( "onOutpostDestroyed" );
	
	ScriptManager:displaySubtitleAutoEx( Text.T_SUBTITLE_REINFORCMENT_PRAISE_1, false );
	SetMissionStatus( missionIds.capOutpost, MissionStatus.ACCOMPLISHED );
end

function onOutpostDestroyed()
	ScriptManager:removeTrigger( "onOutpostCaptured" );
	ScriptManager:removeTrigger( "onOutpostDestroyed" );
	
	SetMissionStatus( missionIds.capOutpost, MissionStatus.FAILED );
end

function onCityBuildingCaptured()
	ScriptManager:setMissionProgress( missionIds.capNorthCity, ScriptManager:getMissionProgress( missionIds.capNorthCity ) + 1 );
	ScriptManager:wait( 0.5 ); -- just to be safe to get the updated progress
	
	local progress = ScriptManager:getMissionProgress( missionIds.capNorthCity )
	if progress >= 2 then
		SetMissionStatus( missionIds.capNorthCity, MissionStatus.ACCOMPLISHED );
		checkForVictory(  missionIds.capNorthCity );
	
	else
		ShowMissionProgressPopup( progress, Text.T_MAP_DE7_2_MISSION_PRIMARY_01_COUNTER );
	end
end

function onFactoryCaptured()
	SetMissionStatus( missionIds.capFactory, MissionStatus.ACCOMPLISHED );
	checkForVictory(  missionIds.capFactory );
end

function checkForVictory( lastAccomplished )
	local cityStatus		= ScriptManager:getMissionStatus( missionIds.capNorthCity );
	local airfieldStatus	= ScriptManager:getMissionStatus( missionIds.capMidAirfield );
	local factoryStatus		= ScriptManager:getMissionStatus( missionIds.capFactory );
	
	-- if all done -> victory
	if	cityStatus		== MissionStatus.ACCOMPLISHED and
		airfieldStatus	== MissionStatus.ACCOMPLISHED and
		factoryStatus	== MissionStatus.ACCOMPLISHED then
		SetMissionStatus( missionIds.defBuildings, MissionStatus.ACCOMPLISHED );
		startVictoryCutscene();
	
	-- if nothing someting has not started yet -> start it
	elseif	cityStatus		== MissionStatus.NOT_STARTED or
			airfieldStatus	== MissionStatus.NOT_STARTED or
			factoryStatus	== MissionStatus.NOT_STARTED then
			
		local returnX, returnY;
		if lastAccomplished == missionIds.capMidAirfield then
			returnX = airfieldPos[1];
			returnY = airfieldPos[2];
		
		elseif lastAccomplished == missionIds.capNorthCity then
			returnX = cityPos[1];
			returnY = cityPos[2];
		
		elseif lastAccomplished == missionIds.capFactory then
			returnX = factoryPos[1];
			returnY = factoryPos[2];
		end
		StartCutscene();
		
		ScriptManager:displaySubtitle( Text.T_MAP_DE7_2_MISSION_PRIMARY_FLAVOR_01 );
		
		-- show cutscene for the city if not yet triggered
		if cityStatus == MissionStatus.NOT_STARTED then
			ScriptManager:moveCameraEx( cityPos[1], cityPos[2], 0.6, 4, true );
			HideDialogue();
			ScriptManager:wait( 1 )
			
			-- add factory highlight into cutscene if it is still needed
			if factoryStatus == MissionStatus.NOT_STARTED then
				northCityCutsceneContent( factoryPos[1], factoryPos[2] );
				ScriptManager:wait( 1 );
				
				factoryCutsceneContent( returnX, returnY );
			
			else
				northCityCutsceneContent( returnX, returnY );
			end
		
		elseif factoryStatus == MissionStatus.NOT_STARTED then
			ScriptManager:moveCameraEx( factoryPos[1], factoryPos[2], 0.6, 4, true );
			HideDialogue();
			ScriptManager:wait( 1 )
			
			factoryCutsceneContent( returnX, returnY );
		
		else
			ScriptManager:wait( 3 );
			HideDialogue();
		end
		
		EndCutscene();
		DisablePlayerInteraction()
		
		if cityStatus == MissionStatus.NOT_STARTED then
			ScriptManager:removeTrigger( "startNorthCityCutscene")
			ScriptManager:setMissionStatus( missionIds.capNorthCity, MissionStatus.IN_PROGRESS );
		end
		
		if factoryStatus == MissionStatus.NOT_STARTED then
			ScriptManager:removeTrigger( "startFactoryCutscene")
			ScriptManager:setMissionStatus( missionIds.capFactory, MissionStatus.IN_PROGRESS );
		end
		if ScriptManager:getMissionStatus( missionIds.defBuildings ) == MissionStatus.NOT_STARTED then
			ScriptManager:setMissionStatus( missionIds.defBuildings, MissionStatus.IN_PROGRESS );
		end
		
		ScriptManager:wait( 2 );
		
		EnablePlayerInteraction();
	
	-- if all started, -> give def mission too
	else
		if ScriptManager:getMissionStatus( missionIds.defBuildings ) == MissionStatus.NOT_STARTED then
			DisablePlayerInteraction()
			
			ScriptManager:setMissionStatus( missionIds.defBuildings, MissionStatus.IN_PROGRESS );
			ScriptManager:wait( 2 );
			
			EnablePlayerInteraction();
		end
	end
end


-- END Building Capture
-- BEGIN Victory


function startVictoryCutscene()
	StartCutscene();
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE7_2_MISSION_VICTORY_FLAVOR_01 );
	ScriptManager:wait( 3 );
	HideDialogue();
	ScriptManager:wait( 1 );
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE7_2_MISSION_VICTORY_FLAVOR_02 );
	ScriptManager:wait( 3 );
	HideDialogue();
	
	EndCutscene();
	victoryEndGame();
end

function victoryEndGame()
	DisablePlayerInteraction();
	HideDialogue(); -- remove possible texts on the screens

	-- accomplish secondary missions that are accomplished when the game is finished with certain condition
	
	
	ScriptManager:displaySubtitle( Text.T_MISSION_VICTORY );
	ScriptManager:wait( 4 );
	HideDialogue();
	endGame();
end


-- END Victory
-- BEGIN Defeated


function transitionToFail( unit )
	local unitId = unit:getId();
	local playerId = unit:getPlayerId();
	
	if playerId == 1 then -- player destroyed it
		if unitId == cityAirfieldId or unitId == cityDepotId then
			SetMissionStatus( missionIds.capNorthCity, MissionStatus.FAILED );
		
		elseif unitId == midAirfieldId then
			SetMissionStatus( missionIds.capMidAirfield, MissionStatus.FAILED );
		
		else -- == factoryId
			SetMissionStatus( missionIds.capFactory, MissionStatus.FAILED );
		end
	
	else -- ai destroyed it
		SetMissionStatus( missionIds.defBuildings, MissionStatus.FAILED );
	end
	
	ScriptManager:runForPlayer( 0, "missionFailed" );
end

function checkForEnd()
	if ScriptManager:noUnitsLeft( 0 ) then
		ScriptManager:runForPlayer( 0, "missionFailed" )
	end
end

function missionFailed()
	DisablePlayerInteraction();
	HideDialogue(); -- remove possible texts on the screens
	
	ScriptManager:displaySubtitle( Text.T_MISSION_FAILED );
	ScriptManager:wait( 4 );
	HideDialogue();
	endGame();
end


-- END Defeated


function endGame()
	ScriptManager:endGame()
end