require "lua/Constants"
require "lua/Functions"
require "lua/Functions_2"
require "lua/Text"
require "lua/Units"
require "math"


-- mission stuff
missionIds = {
	capHq			= 1,
	capFactory		= 2,
	destroyKarl		= 3,
	capAirfield		= 4,
	breakDefLine	= 5,
};

-- missionId, primary( boolean ), textkey, xpReward
missions = {
	{ missionIds.capHq, 		true,	Text.T_MAP_DE7_3_MISSION_PRIMARY_01,	100 },
	{ missionIds.capFactory,	true,	Text.T_MAP_DE7_3_MISSION_PRIMARY_02,	100 },
	{ missionIds.destroyKarl,	true,	Text.T_MAP_DE7_3_MISSION_PRIMARY_03,	150 },
	{ missionIds.capAirfield,	false,	Text.T_MAP_DE7_3_MISSION_SECONDARY_01,	 75 },
	{ missionIds.breakDefLine,	false,	Text.T_MAP_DE7_3_MISSION_SECONDARY_02,	 75 },
};

enemySquadIds = {
	firstAttackerW	=  1,
	firstAttackerE	=  2,
	depot			=  3,
	factory			=  4,
	airfield		=  5,
	cityAaN			=  6,
	cityAaS			=  7,
	leftBarricade	=  8, -- guard area 5 | 17
	rightBarricade	=  9, -- guard area 13 | 9
	centralForest	= 10,
	karlGuards		= 11, -- guard area 15 | 20
	planeWave		= 12,
	attackWave		= 13, -- atm no attack waves, needed?
	bunkerS			= 14,
	bunkerW			= 15,
	BunkerN			= 16,
};

mineShaft		= { {  7,  8 }, {  8, 11 } };
airfieldTiles	= { { 18, 20 }, { 18, 18 }, { 18, 19 }, };
hqId	= 2;

planeWaveRespawn = 1; -- changes depending on spawned units

 -- number of units within the barricade; 2 barricades total to get secondary mission
leftBarricadeToDestroy	= 7;
rightBarricadeToDestroy	= 6;
leftBarricadeDestroyed	= 0;
rightBarricadeDestroyed	= 0;


-- BEGIN Init


function onGameStart()
	CreateMissions( missions );
	setTriggers();
	storeValues();
	
	ScriptManager:setGuardArea( 1, enemySquadIds.leftBarricade,		 4, 17, 2 );
	ScriptManager:setGuardArea( 1, enemySquadIds.rightBarricade,	13,  8, 2 );
	ScriptManager:setGuardArea( 1, enemySquadIds.karlGuards,		15, 20, 1 );
	
	ScriptManager:gainGold( 0, 1000 );
	
	-- start intro cutscene
	ScriptManager:sleep( 0, "startIntroCutscene" ); -- without sleep player turn popup shows
end

function storeValues()
	ScriptManager:storeValue( "leftBarricadeToDestroy",		leftBarricadeToDestroy );
	ScriptManager:storeValue( "rightBarricadeToDestroy",	rightBarricadeToDestroy );
	ScriptManager:storeValue( "leftBarricadeDestroyed",		leftBarricadeDestroyed );
	ScriptManager:storeValue( "rightBarricadeDestroyed",	rightBarricadeDestroyed );
	ScriptManager:storeValue( "planeWaveRespawn",			planeWaveRespawn );
end

function setTriggers()
	-- load all needed values after loading
	ScriptManager:addOnGameResumeTrigger( "loadValues", "loadValues", false );
	
	-- trigger for buildings; attacked warnings for all buildings
	ScriptManager:addOnUnitCapturedTrigger(		 0, -1,					hqId, "onHqCaptured",	"onHqCaptured",	 false );
	ScriptManager:addOnUnitDestroyedTrigger(	-1, -1,					hqId, "missionFailed",	"missionFailed", false );
	ScriptManager:addOnUnitHasAttackedTrigger(	-1, Units.UNIT_DE_HQ,	-1,
												"ProtectTheBuildingHintSingle",	"ProtectTheBuildingHintSingle", false );
	
	ScriptManager:addOnUnitCapturedTrigger(		 0, Units.UNIT_TANKFACTORY_STANDARD, -1, "onFactoryCaptured", "onFactoryCaptured", false );
	ScriptManager:addOnUnitDestroyedTrigger(	-1, Units.UNIT_TANKFACTORY_STANDARD, -1, "missionFailed",		"missionFailed",	 false );
	ScriptManager:addOnUnitHasAttackedTrigger(	-1, Units.UNIT_TANKFACTORY_STANDARD, -1,
												"ProtectTheBuildingHintSingle",	"ProtectTheBuildingHintSingle", false );
	
	ScriptManager:addOnUnitCapturedTrigger(		 0, Units.UNIT_DE_AIRFIELD, -1, "onAirfieldCaptured",	"onAirfieldCaptured",	false );
	ScriptManager:addOnUnitDestroyedTrigger(	-1, Units.UNIT_DE_AIRFIELD, -1, "onAirfieldDestroyed",	"onAirfieldDestroyed",	false );
	ScriptManager:addOnUnitHasAttackedTrigger(	-1, Units.UNIT_DE_AIRFIELD, -1,
												"ProtectTheBuildingHintSingle",	"ProtectTheBuildingHintSingle", false );
	
	-- trigger for karl
	ScriptManager:addOnUnitHasAttackedTrigger(	-1, Units.UNIT_DE_MORTAR_KARL, -1, "onKarlAttacked",	"onKarlAttacked",	false );
	ScriptManager:addOnUnitDestroyedTrigger(	 1, Units.UNIT_DE_MORTAR_KARL, -1, "onKarlDestroyed",	"onKarlDestroyed",	false );
	
	-- check if the player has units left
	ScriptManager:addOnUnitDestroyedTrigger( 0, -1, -1, "checkForEnd", "checkForEnd" , false );
	
	-- trigger for mine shaft
	ScriptManager:addOnEnterTileTrigger( mineShaft[1][1], mineShaft[1][2], -1, -1, -1, true, "onEnterMineShaft", "onEnterMineShaft", false );
	ScriptManager:addOnEnterTileTrigger( mineShaft[2][1], mineShaft[2][2], -1, -1, -1, true, "onEnterMineShaft", "onEnterMineShaft", false );
	
	-- trigger for def line
	ScriptManager:addOnUnitSightedTrigger(	 1, Units.UNIT_SU_BARRICADE, -1, true,	"onBarricadeSighted",	"onBarricadeSighted", false );
	ScriptManager:addOnUnitDestroyedTrigger( 1, -1,						 -1,		"onEnemyDestroyed",		"onEnemyDestroyed",	 false );
	
	-- trigger for plane waves
	ScriptManager:addOnTurnStartTrigger( 1, -1, -1, "onEnemyTurnStart", "onEnemyTurnStart", false );
end

function loadValues()
	leftBarricadeToDestroy	= ScriptManager:getValue( "leftBarricadeToDestroy" );
	rightBarricadeToDestroy	= ScriptManager:getValue( "rightBarricadeToDestroy" );
	leftBarricadeDestroyed	= ScriptManager:getValue( "leftBarricadeDestroyed" );
	rightBarricadeDestroyed	= ScriptManager:getValue( "rightBarricadeDestroyed" );
	planeWaveRespawn		= ScriptManager:getValue( "planeWaveRespawn" );
end

function onEnterMineShaft( tileX, tileY, playerId, unitTemplateId, unitId )
	OnShaftEntered( tileY, unitId, mineShaft );
end


-- END Init
-- BEGIN Intro Cutscene


function startIntroCutscene()
	StartCutscene();
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE7_3_MISSION_INTRO_FLAVOR_01 );
	ScriptManager:moveCameraEx( 10, 22, 0.7, 6, true );
	HideDialogue();
	ScriptManager:wait( 1 );
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE7_3_MISSION_INTRO_FLAVOR_02 );
	ScriptManager:revealFogOfWar(	10, 22, 1, true );
	ScriptManager:showMarkerAt(		10, 22 );
	ScriptManager:wait( 2 );
	ScriptManager:removeAllMarkers()
	ScriptManager:revealFogOfWar( 10, 22, 1, false );
	ScriptManager:wait( 1 );
	
	ScriptManager:moveCameraEx( 17, 18, 0.7, 4, true );
	HideDialogue();
	ScriptManager:wait( 1 );
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE7_3_MISSION_INTRO_FLAVOR_03 );
	ScriptManager:revealFogOfWar(	16, 16, 1, true );
	ScriptManager:revealFogOfWar(	17, 19, 1, true );
	ScriptManager:showMarkerAt(		16, 16 );
	ScriptManager:showMarkerAt(		17, 19 );
	ScriptManager:wait( 3 );
	ScriptManager:removeAllMarkers()
	ScriptManager:revealFogOfWar( 16, 16, 1, false );
	ScriptManager:revealFogOfWar( 17, 19, 1, false );
	ScriptManager:wait( 1 );
	HideDialogue();
	ScriptManager:wait( 1 );
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE7_3_MISSION_INTRO_FLAVOR_04 );
	ScriptManager:moveCameraEx( 8, 3, 0.7, 7, true );
	HideDialogue();
	ScriptManager:wait( 1 );
	
	EndCutscene();
	
	startTheMission();
end

function startTheMission()
	ScriptManager:setMissionStatus( missionIds.capHq,		MissionStatus.IN_PROGRESS );
	ScriptManager:setMissionStatus( missionIds.capFactory,	MissionStatus.IN_PROGRESS );
	ScriptManager:setMissionStatus( missionIds.capAirfield, MissionStatus.IN_PROGRESS );
	
	ScriptManager:setMissionProgress( missionIds.breakDefLine, 0 );
	
	--TODO: remove this function!
	--testing();
	--removeSquads();
	--ScriptManager:addOnUnitSelectedTrigger( 1, -1, -1, "showParameters", "showParameters",false)
end

function testing()
	ScriptManager:spawnUnit( 0, Units.GOTT, 1, 2, 42, AiState.WAIT );
	ScriptManager:spawnUnit( 0, Units.GOTT, 2, 2, 42, AiState.WAIT );
end

function removeSquads()
	--for i = 1, # do
	--	ScriptManager:removeUnitEx( [i], false );
	--end
end

function showParameters( selectedUnit )
	--print( string.format( "[] %s", selectedUnit:get() ) );
end


-- END Intro Cutscene
-- BGEIN Enemy Waves


function buildPlaneWave( wavePoints, planeTable )
	--[[ wave Points is a random number between 1 - 4. 
		instead of using if 1 == 1 fighter; 2 = 1 bomber; 3 = 1 fighter, 1 bomber; 4 = 2 fighters
		i use this recursive function to fill my table and send it back to the spawn function ]]
	if wavePoints >= 3 then
		table.insert( planeTable, Units.UNIT_SU_BOMBER );
		return buildPlaneWave( wavePoints - 3, planeTable );
	
	elseif wavePoints >= 1 then
		table.insert( planeTable, Units.UNIT_SU_FIGHTER );
		return buildPlaneWave( wavePoints - 1, planeTable );
	
	else -- wavePoints == 0 then
		return planeTable;
	end
end

function onEnemyTurnStart( playerId, turnId, roundId )
	if roundId % planeWaveRespawn == 0 then
		local rng = math.random( 1, 4 );
		local planes = {};
		print( "rng = " .. rng );
		planes = buildPlaneWave( rng, planes );
		
		for i = 1, #planes do
			ScriptManager:spawnUnit( 1, planes[i], airfieldTiles[i][1], airfieldTiles[i][2], enemySquadIds.planeWave, AiState.AIRPLANE );
		end
		
		planeWaveRespawn = rng + #planes;-- range: 2 ( 1 fighter ) - 6 ( 1 bomber, 1 fighter )
		ScriptManager:storeValue( "planeWaveRespawn", planeWaveRespawn );
	end
end


-- END Enemy Waves
-- BEGIN Secondary Missionst


function onAirfieldCaptured()
	ScriptManager:removeTrigger( "onAirfieldDestroyed" );
	ScriptManager:removeTrigger( "onAirfieldCaptured" );
	
	ScriptManager:displaySubtitle( Text.T_SUBTITLE_REINFORCMENT_PRAISE_1 );
	SetMissionStatus( missionIds.capAirfield, MissionStatus.ACCOMPLISHED );
	HideDialogue();
end

function onAirfieldDestroyed()
	ScriptManager:removeTrigger( "onAirfieldDestroyed" );
	ScriptManager:removeTrigger( "onAirfieldCaptured" );
	
	SetMissionStatus( missionIds.capAirfield, MissionStatus.FAILED );
end

function onBarricadeSighted( seeingUnit, seenUnit )
	ScriptManager:removeTrigger( "onBarricadeSighted" );
	StartCutscene();
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE7_3_MISSION_SECONDARY_02_FLAVOR_01 );
	ScriptManager:moveCameraEx( seenUnit:getX(), seenUnit:getY(), 0.7, 2, true );
	HideDialogue();
	ScriptManager:wait( 1 );
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE7_3_MISSION_SECONDARY_02_FLAVOR_02 );
	ScriptManager:wait( 5 );
	HideDialogue();
	ScriptManager:wait( 1 );
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE7_3_MISSION_SECONDARY_02_FLAVOR_03 );
	ScriptManager:moveCameraEx( seeingUnit:getX(), seeingUnit:getY(), 0.7, 2, true );
	HideDialogue();
	ScriptManager:wait( 1 );
	
	EndCutscene();
	SetMissionStatus( missionIds.breakDefLine, MissionStatus.IN_PROGRESS );
end

function onEnemyDestroyed( unit )
	local squadId = unit:getSquadId();
	
	if squadId == enemySquadIds.leftBarricade then
		leftBarricadeDestroyed = leftBarricadeDestroyed + 1;
		ScriptManager:storeValue( "leftBarricadeDestroyed", leftBarricadeDestroyed );
		
		ScriptManager:runForPlayer( 0, "prepareLeftBarricadeProgress" );
	
	elseif squadId == enemySquadIds.rightBarricade then
		rightBarricadeDestroyed = rightBarricadeDestroyed + 1;
		ScriptManager:storeValue( "rightBarricadeDestroyed", rightBarricadeDestroyed );
		
		ScriptManager:runForPlayer( 0, "prepareRightBarricadeProgress" );
	end
end

function showBarricadeProgress( barricade, counterText )
	local barricadeCounter	= _G[barricade .. "Destroyed"];
	local barricadeMax		= _G[barricade .. "ToDestroy"];
	
	if barricadeCounter >= barricadeMax then
		ScriptManager:setMissionProgress( missionIds.breakDefLine, ScriptManager:getMissionProgress( missionIds.breakDefLine ) + 1 );
		checkForBarricadeMissionEnd();
		
	else
		ShowMissionProgressPopup( barricadeCounter, counterText );
	end
end

-- left barricade update
function prepareLeftBarricadeProgress()
	table.insert( cutsceneQueue, showLeftBarricadeProgress );
	CutsceneQueue();
end

function showLeftBarricadeProgress()
	showBarricadeProgress( "leftBarricade", Text.T_MAP_DE7_3_MISSION_SECONDARY_02_COUNTER_L );
end

-- right barricade update
function prepareRightBarricadeProgress()
	table.insert( cutsceneQueue, showRightBarricadeProgress );
	CutsceneQueue();
end

function showRightBarricadeProgress()
	showBarricadeProgress( "rightBarricade", Text.T_MAP_DE7_3_MISSION_SECONDARY_02_COUNTER_R );
end

-- check for both barricades
function checkForBarricadeMissionEnd()
	local progress = ScriptManager:getMissionProgress( missionIds.breakDefLine );
	if progress >= 2 then
		ScriptManager:removeTrigger( "onEnemyDestroyed" );
		SetMissionStatus ( missionIds.breakDefLine, MissionStatus.ACCOMPLISHED );
		
	else
		ShowMissionProgressPopup( progress, Text.T_MAP_DE7_3_MISSION_SECONDARY_02_COUNTER );
	end
end


-- END Secondary Missions
-- BEGIN Karl Gerät


function onKarlAttacked()
	ScriptManager:removeTrigger( "onKarlAttacked" );
	
	ScriptManager:runForPlayer( 0, "prepareKarlAttackedCutscene" );
end

function prepareKarlAttackedCutscene()
	table.insert( cutsceneQueue, startKarlAttackedCutscene );
	CutsceneQueue();
end

function startKarlAttackedCutscene()
	StartCutscene();
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE7_3_MISSION_PRIMARY_03_FLAVOR_01 );
	ScriptManager:wait( 5 );
	HideDialogue();
	ScriptManager:wait( 2 );
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE7_3_MISSION_PRIMARY_03_FLAVOR_02 );
	ScriptManager:wait( 7 );
	HideDialogue();
	ScriptManager:wait( 1 );
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE7_3_MISSION_PRIMARY_03_FLAVOR_03 );
	ScriptManager:wait( 7 );
	HideDialogue();
	ScriptManager:wait( 1 );
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE7_3_MISSION_PRIMARY_03_FLAVOR_04 );
	ScriptManager:wait( 7 );
	HideDialogue();
	ScriptManager:wait( 1 );
	
	EndCutscene();
	SetMissionStatus( missionIds.destroyKarl, MissionStatus.IN_PROGRESS );
end

function onKarlDestroyed()
	ScriptManager:removeTrigger( "onKarlDestroyed" );
	
	ScriptManager:runForPlayer( 0, "prepareKarlVictoryCheck" );
end

function prepareKarlVictoryCheck()
	checkForVictory( missionIds.destroyKarl );
end


-- END Karl Gerät
-- BEGIN Main Building Stuff


function onHqCaptured()
	ScriptManager:removeTrigger( "onHqCaptured" );
	
	checkForVictory( missionIds.capHq );
end

function onFactoryCaptured()
	ScriptManager:removeTrigger( "onFactoryCaptured" );
	
	checkForVictory( missionIds.capFactory );
end


-- END Main Building Stuff
-- BEGIN Victory


function checkForVictory( lastAccomplished )
	SetMissionStatus( lastAccomplished, MissionStatus.ACCOMPLISHED );
	ScriptManager:wait( 0.25 ) -- just to be sure
	
	local hqStatus		= ScriptManager:getMissionStatus( missionIds.capHq );
	local factoryStatus	= ScriptManager:getMissionStatus( missionIds.capFactory );
	local karlStatus	= ScriptManager:getMissionStatus( missionIds.destroyKarl );
	
	if 	hqStatus 		== MissionStatus.ACCOMPLISHED and
		factoryStatus	== MissionStatus.ACCOMPLISHED and
		karlStatus		== MissionStatus.ACCOMPLISHED then
		startVictoryCutscene();
		
	else
		if lastAccomplished == missionIds.destroyKarl then
			StartCutscene();
			
			ScriptManager:displaySubtitle( Text.T_MAP_DE7_3_MISSION_PRIMARY_03_FLAVOR_05 );
			ScriptManager:wait( 7 );
			HideDialogue();
			ScriptManager:wait( 1 );
			
			EndCutscene();
			
		end
	end
end

function startVictoryCutscene()
	StartCutscene();
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE7_3_MISSION_VICTORY_FLAVOR_01 );
	ScriptManager:wait( 4 );
	HideDialogue();
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE7_3_MISSION_VICTORY_FLAVOR_02 );
	ScriptManager:wait( 4 );
	HideDialogue();
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE7_3_MISSION_VICTORY_FLAVOR_03 );
	ScriptManager:wait( 4 );
	HideDialogue();
	
	EndCutscene();
	
	victoryEndGame();
end

function victoryEndGame()
	DisablePlayerInteraction();
	HideDialogue(); -- remove possible texts on the screens
	
	ScriptManager:displaySubtitle( Text.T_MISSION_VICTORY );
	ScriptManager:wait( 4 );
	HideDialogue();
	endGame();
end


-- END Victory
-- BEGIN Defeated


function checkForEnd()
	if ScriptManager:noUnitsLeft( 0 ) then
		ScriptManager:runForPlayer( 0, "missionFailed" )
	end
end

function missionFailed()
	DisablePlayerInteraction();
	HideDialogue(); -- remove possible texts on the screens
	
	ScriptManager:displaySubtitle( Text.T_MISSION_FAILED );
	ScriptManager:wait( 4 );
	HideDialogue();
	endGame();
end


-- END Defeated


function endGame()
	ScriptManager:endGame()
end