require "lua/Constants"
require "lua/Functions"
require "lua/Functions_2"
require "lua/Text"
require "lua/Units"
require "math"

-- mission stuff
missionIds = {
	ReachGermans	= 1,
	breakMoral		= 2,
	killAllScouts	= 3,
	noAlarm 		= 4
};

-- missionId, primary( boolean ), textkey, xpReward
missions = {
	{ missionIds.ReachGermans,	 true, Text.T_MAP_SU_3_1_MISSION_PRIMARY_01, 	100 },
	{ missionIds.breakMoral,	 true, Text.T_MAP_SU_3_1_MISSION_PRIMARY_02, 	150 }, 
	{ missionIds.killAllScouts,	false, Text.T_MAP_SU_3_1_MISSION_SECONDARY_01,	 75 },
	{ missionIds.noAlarm,		false, Text.T_MAP_SU_3_1_MISSION_SECONDARY_02,	 50 } 
};

-- random pos for scouts. a scout will move to one of the two pos in each table line
-- tileX, tileY 			scoutID		has random pos?
scoutPos = {
	-- pos1			pos2
	{ 10, 25 }, { 11, 21 }, -- 15		y
	{  8, 25 }, {  8, 24 }, -- 16		y
	{  9, 16 }, {  9, 16 }, -- 17		n
	{  6, 21 }, {  6, 21 }, -- 18		n
	{  3, 15 }, {  5, 14 }, -- 19		y
	{ 17, 13 }, { 19, 12 }, -- 20		y
	{ 20,  4 }, { 20,  4 }, -- 21		n
	{ 15,  5 }, { 15,  3 }, -- 22		y
	{ 15,  9 }, { 15,  9 }, -- 23		n
	{ 10,  7 }, { 11,  4 }, -- 24		n
};

-- some scouts alarm reinforcemts near them
-- unit1ID, unit2ID, squadID
scoutReinforcement = { 
	{ 43,	44,		89 }, -- northern scout @front
	{ 45,	45,		90 }  -- southern scout @front
};

-- table for every scout in order to check if he has already been seen or marked
-- 0 = not seen / marked, 1 = seen / marked;
sightedScouts		= { 0, 0, 0, 0, 0, 0, 0, 0, 0 };
markedScouts		= { 0, 0, 0, 0, 0, 0, 0, 0, 0 };
moveReductionScouts = {}; -- table for the scouts to get their ap reduced when they see the player ( not vice versa! )

-- ids of all enemy units that get camouflaged @ beginning
camouflageUnitIds = { 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48 };

-- needed in order to spawn enemies in enemy turn
globalScoutIdForReinforcementCheck = -1;

-- will spawn when a scout reaches his army
-- playerId, templateId, tileX, tileY, squadId, AiState
germanReinforcements = { 
	{ 1, Units.UNIT_DE_TIGER, 		2, 3, 0, AiState.DEFEND },
	{ 1, Units.UNIT_DE_PANZERFAUST,	3, 4, 0, AiState.DEFEND },
	{ 1, Units.UNIT_DE_TANK_3,		3, 2, 0, AiState.DEFEND }, 
	{ 1, Units.UNIT_DE_MP40,		1, 3, 0, AiState.DEFEND }
};

-- Ids and retreat pos of the german hq troops ( without reinforcements )
-- id, tileX, tileY
germanHqTroops = { 
	{  8, 0, 0 }, -- mg nest, just remove, pos is irrelevant
	{ 12, 1, 2 }, -- tank
	{ 13, 1, 2 }, -- tank
	{ 14, 1, 3 }, -- tank
	{  9, 1, 4 }, -- infantry
	{ 10, 3, 1 }, -- tank
	{ 11, 3, 1 }, -- infantry
};

playerSeen			=  0; -- indicates if the player has been seen by a scout
playerSeenScoutId	=  0; -- id of the scout that saw the player 
scoutIdsBegin		= 15; -- id of the first scout in the editor
scoutIdsEnd			= 24; -- id of the last scout in the editor
scoutsAlarmed 		=  0; -- number of enemy scouts that saw you and are alive
scoutsAlive 		= #scoutPos / 2; -- number of souts on the map;
germanMoral 		= 90; -- if a german hq unit gets destroyed this value goes down by x ( depending on unit )
MoralSubIndicator	=  3; -- handle to show subs for certain moral levels only once ( see checkMoral() for details )


-- BEGIN Init


function onGameStart()
	CreateMissions( missions );	
	setTriggers();	
	storeValues();
	camouflageEnemy();
	
	ScriptManager:gainGold( 0, 1000 );
	
	-- start intro cutscene
	ScriptManager:sleep( 0, "startIntroCutscene" ); -- without sleep player turn popup shows
end

function camouflageEnemy()
	for i = 1, #camouflageUnitIds do
		CamouflageUnit( camouflageUnitIds[i] );
	end
end

function storeValues()
	-- no need to load sightedScouts cause its empty @beginning 
	ScriptManager:storeValue( "scoutsAlarmed",		scoutsAlarmed );
	ScriptManager:storeValue( "scoutsAlive",		scoutsAlive );
	ScriptManager:storeValue( "germanMoral",		germanMoral );
	ScriptManager:storeValue( "MoralSubIndicator",	MoralSubIndicator );
	ScriptManager:storeValue( "playerSeen", 		playerSeen );
end

function setTriggers()
	-- load all needed values after loading
	ScriptManager:addOnGameResumeTrigger( "loadValues", "loadValues", false );
	
	-- + check if scouts are on the run and inform the player about them	
	ScriptManager:addOnTurnStartTrigger( 0, -1, -1, "onPlayerTurnStart", "onPlayerTurnStart", false );
	
	-- trigger for enemy units killed
	ScriptManager:addOnUnitDestroyedTrigger( 1, -1, -1, "onEnemyDestroyed", "onenemyDestroyed", false );
	
	-- trigger to check if the player has units left
	ScriptManager:addOnUnitDestroyedTrigger( 0, -1, -1, "checkForEnd", "checkForEnd", false );
	
	-- trigger to change enemy state from passive to attack & for the scouts to run away
	ScriptManager:addOnUnitSightedTrigger( 0, -1, -1, false, "onBeingSeen", "onBeingSeen", false );
	
	-- trigger for scouts that getting seen by the player despite camouflage ( => standing next to him )
	ScriptManager:addOnUnitSightedTrigger( 1, Units.UNIT_DE_SCOUT, -1, true, "scoutSeen", "scoutSeen", false );
	
	-- triggers around german base for the start of the second main mission, 
	local y = 11
	for x = 1, 9 do
		-- start the line horizontal
		if  x < 4 or
			x == 6 then
			ScriptManager:addOnEnterTileTrigger( x, y, 0, -1, -1, true, "nearGermanHq", "nearGermanHq" );
			
		-- now start moving down right until the map border is hit 
		else
			for i = 1, 2 do
				ScriptManager:addOnEnterTileTrigger( x, y, 0, -1, -1, true, "nearGermanHq", "nearGermanHq" );
				y = y - 1;
			end
		end
	end
	
	-- triggers around german base for the scouts to alarm the hq ( this one is smaller than the one above )
	y = 8;
	for x = 1, 7 do
		-- start the line horizontal
		if x < 4 then
			ScriptManager:addOnEnterTileTrigger( x, y, 1, Units.UNIT_DE_SCOUT, -1, false, "scoutInHq", "scoutInHq" );
			
		-- when x > 3 start moving down right
		else
			for i = 1, 2 do
				ScriptManager:addOnEnterTileTrigger( x, y, 1, Units.UNIT_DE_SCOUT, -1, false, "scoutInHq", "scoutInHq" );
				y = y - 1;
			end
		end
	end
end

function loadValues()
	local value = 0;
	for i = 1, #sightedScouts do
		value = ScriptManager:getValue( string.format( "sighted %i", i ) );
		if value == 1 then
			sightedScouts[i] = value;
		end 
		
		value = ScriptManager:getValue( string.format( "marked %s", i ) );
		if value == 1 then
			markedScouts[i] = value;
		end
	end
	
	scoutsAlarmed		= ScriptManager:getValue( "scoutsAlarmed" );	
	scoutsAlive			= ScriptManager:getValue( "scoutsAlive" );
	germanMoral			= ScriptManager:getValue( "germanMoral" );
	MoralSubIndicator	= ScriptManager:getValue( "MoralSubIndicator" );
	playerSeen			= ScriptManager:getValue( "playerSeen" );
end

function onPlayerTurnStart()
	if ScriptManager:getMissionStatus( missionIds.noAlarm ) ~= MissionStatus.FAILED then
		-- show message that a scout saw u last round
		if playerSeen == 1 then
			-- disable player stuff, move camera to scout, show marker, display sub and move camera back
			-- at the moment if I see a scout it is handled as if the scout saw me cause otherwise its not possible ( yet )
			DisablePlayerInteraction();
			
			for i = 1, #sightedScouts do
				-- if a scout is sighted, but NOT marked, mark him now
				local messageShown = false; -- show the popup message only once
				if sightedScouts[i] == 1 and markedScouts[i] == 0 then
					local unit = Unit:getUnitById( i + ( scoutIdsBegin - 1 ) );
					if unit ~= nil then					
						ScriptManager:revealFogOfWar(	unit:getX(), unit:getY(), 1, true );
						ScriptManager:moveCameraEx(		unit:getX(), unit:getY(), 0.7, 1, true );
						ScriptManager:showMarkerAt(		unit:getX(), unit:getY() );
						
						if messageShown == false then
							ScriptManager:displaySubtitle( Text.T_MAP_SU_3_1_MISSION_SECONDARY_02_FLAVOR_01 );
							messageShown = true;
						end
						ScriptManager:wait( 4 );
						
						HideAllMarkers();
						HideDialogue();
						ScriptManager:revealFogOfWar(unit:getX(), unit:getY(), 1, false );
						
						markedScouts[i] = 1;
						ScriptManager:storeValue( string.format( "marked %s", i ), markedScouts[i] );
					end
				end
			end
			
			playerSeen = 0;
			ScriptManager:storeValue( "playerSeen", playerSeen );
			
			EnablePlayerInteraction();
		
		-- show scout warning if needed
		elseif scoutsAlarmed > 0 then
			ScriptManager:displaySubtitle( Text.T_MAP_SU_3_1_MISSION_SECONDARY_01_FLAVOR_01 );
			ScriptManager:sleep( 3, "HideDialogue" );
		end
	end
	
	
end


-- END Init
-- BEGIN Intro Cutscene


function startIntroCutscene()
	StartCutscene();

	ScriptManager:displaySubtitle( Text.T_MAP_SU_3_1_MISSION_PRIMARY_01_FLAVOR_01 );
	ScriptManager:wait( 5 );
	HideDialogue();
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_3_1_MISSION_PRIMARY_01_FLAVOR_02 );
	ScriptManager:moveCameraEx( 2, 3, 0.7, 5, true );
	ScriptManager:wait( 2 ); -- antirush
	HideDialogue();
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_3_1_MISSION_PRIMARY_01_FLAVOR_03 )
	ScriptManager:moveCameraEx( 8, 21, 0.3, 5, true );
	HideDialogue();
	
	ScriptManager:wait( 1 ); -- antirush
	ScriptManager:displaySubtitle( Text.T_MAP_SU_3_1_MISSION_PRIMARY_01_FLAVOR_04 )
	ScriptManager:moveCameraEx( 17, 8, 0.3, 5, true );
	HideDialogue();
	
	ScriptManager:wait( 1 ); -- antirush
	ScriptManager:displaySubtitle( Text.T_MAP_SU_3_1_MISSION_PRIMARY_01_FLAVOR_05 )
	ScriptManager:wait( 4 );
	HideDialogue();
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_3_1_MISSION_PRIMARY_01_FLAVOR_06 )
	ScriptManager:moveCameraEx( 21, 25, 0.7, 5, true );
	HideDialogue();
	
	if ScriptManager:isControllerConnected() then
		ScriptManager:setControllerCursorPosition( 19, 24 );
	end

	EndCutscene();
	
	DisablePlayerInteraction(); -- stop player from messing around an possibly stopping the missions from starting
	startTheMission();
	EnablePlayerInteraction();
	
	moveScouts();
end

function startTheMission()
	-- TODO: remove this function
	--testing();
	
	ScriptManager:setMissionStatus( missionIds.ReachGermans, MissionStatus.IN_PROGRESS ); -- reach german base
	ScriptManager:setMissionStatus( missionIds.noAlarm, MissionStatus.IN_PROGRESS ); -- raise no alarm
end

function testing()
	local unit = ScriptManager:spawnUnit( 0, Units.GOTT, 2, 19, 0, AiState.WAIT );
	ScriptManager:moveUnitEx( unit:getId(), 2, 18, true );
	--ScriptManager:spawnUnit( 1, Units.UNIT_DE_SCOUT, 14, 6, 77, AiState.RETREAT );
	--ScriptManager:setTargetTile( 1, 77, 2, 2 );
end

function moveScouts()
	-- the scouts are already in the editor and have to be moved to their final position
	local randPos;
	local scoutId = scoutIdsBegin;
	for i = 1, #scoutPos, 2 do
		randPos = scoutPos[ math.random( i, i + 1 ) ]; -- define a random pos out of 2 possible
		ScriptManager:moveUnitEx( scoutId, randPos[1], randPos[2], false );
		
		scoutId = scoutId + 1; -- move on to next scout
	end
	
	for i = scoutIdsBegin, scoutIdsEnd do
		CamouflageUnit( i );
	end
end


-- END Intro Cutscene
-- BEGIN Scouted


function onBeingSeen( seeingUnit, seenUnit )
	-- change non-scout enemy behaviour when they attacked once
	if seeingUnit:getTemplateId() ~= Units.UNIT_DE_SCOUT then
		if seeingUnit:getAiState() == AiState.PASSIVE then
			Unit:changeSquadState( 1, seeingUnit:getSquadId(), AiState.ATTACK_AGGRESSIVE );
		end
	
	-- scouts saw u	
	else
		-- for the table where i save if i have seen the scout already
		-- first scout will get first place in the table ( = 1 )
		playerSeenScoutId = seeingUnit:getId();
		index = playerSeenScoutId - ( scoutIdsBegin - 1 );
		
		ScriptManager:storeValue( "playerSeenScoutId", playerSeenScoutId );
		
		if sightedScouts[index] == 0 then
			-- let scout retreat to german base
			local scoutSquadId = seeingUnit:getSquadId();			
			Unit:changeSquadState( 1, scoutSquadId, AiState.RETREAT )
			ScriptManager:setTargetTile( 1, scoutSquadId, 2, 5 );
			
			-- spawn additional troops for some of the scouts
			if playerSeenScoutId == 15 then
				callScoutReinforcement( scoutReinforcement[1][3] );
				ScriptManager:runForPlayer( 1, "removeAP" ); -- prevents the reinforcement from moving for 1 round 
				
			elseif playerSeenScoutId == 20 then
				callScoutReinforcement( scoutReinforcement[2][3] );
				ScriptManager:runForPlayer( 1, "removeAP" ); -- prevents the reinforcement from moving for 1 round
			end
			
			table.insert( moveReductionScouts, #moveReductionScouts+1, playerSeenScoutId );
			ScriptManager:runForPlayer( 1, "reduceScoutMovement" );
			
			sightedScouts[index] = 1; -- change number in the table, showing that this scout has been seen
			scoutsAlarmed = scoutsAlarmed + 1; -- a new scout saw the player
			playerSeen = 1; -- triggers the marker @ next player turn
			
			ScriptManager:storeValue( string.format( "sighted %s", index ), sightedScouts[index] );
			ScriptManager:storeValue( "scoutsAlarmed", scoutsAlarmed );
			ScriptManager:storeValue( "playerSeen", playerSeen );
		end
	end
end

function scoutSeen( seeingUnit, seenUnit )
	playerSeenScoutId = seenUnit:getId();
	index = playerSeenScoutId - ( scoutIdsBegin - 1 );
	
	-- if player reaches scout and reveal him, only show the message to kill him and mark him
	-- the rest of the logic ( running away, reinforcement check happened already when he saw you )
	if markedScouts[index] == 0 then
		DisablePlayerInteraction();
		ScriptManager:moveCameraEx( seenUnit:getX(), seenUnit:getY(), 0.7, 1, true );
		ScriptManager:showMarkerAt( seenUnit:getX(), seenUnit:getY() );
		
		ScriptManager:displaySubtitle( Text.T_MAP_SU_3_1_MISSION_SECONDARY_02_FLAVOR_01 );
		ScriptManager:wait( 4 );
		
		HideAllMarkers();
		HideDialogue();
		
		markedScouts[index] = 1;
		
		ScriptManager:storeValue( string.format( "marked %s", index ), markedScouts[index] );
		EnablePlayerInteraction();
	end
end

function callScoutReinforcement( squadId )
	Unit:changeSquadState( 1, squadId, AiState.ATTACK_AGGRESSIVE );
end

function removeAP()
 -- prevents the reinforcement from moving for 1 round
	for i = 1, #scoutReinforcement do
		for j = 1, 2 do
			local unit = Unit:getUnitById( scoutReinforcement[i][j] );
			
			if unit:getAiState() == AiState.ATTACK_AGGRESSIVE then
				unit:setActionPoints( 0 );
			end
		end
	end
end

function reduceScoutMovement()
-- reduce movement to 1 for all scouts that have been seen in the last turn
	local unit = Unit:getUnitById( moveReductionScouts[1] );
	local index = unit:getId() - ( scoutIdsBegin - 1 );
	
	-- only remove the ap for the scouts that have seen you, not the one that you saw
	-- cause then you are already point blank and the scout doesn't have to be slowed
	if markedScouts[index] == 0 then
		unit:setActionPoints( 1 );
	end
	
	table.remove( moveReductionScouts, 1 );
	print( string.format( "scout %i movement reduced", unit:getId() ) );
	
	if #moveReductionScouts > 0 then
		reduceScoutMovement();	
	end
end


-- END Scouted
-- BEGIN on Enemy attacked / killed


function onEnemyDestroyed( unit )
	if unit:getTemplateId() == Units.UNIT_DE_SCOUT then
		-- start secondary mission 1 ( kill all scouts ) when first scout got killed
		if ScriptManager:getMissionStatus( missionIds.killAllScouts ) == MissionStatus.NOT_STARTED then
			ScriptManager:setMissionStatus( missionIds.killAllScouts, MissionStatus.IN_PROGRESS );
		end
		
		-- decrease scout counts cause one was killed and save the values
		scoutsAlive		= scoutsAlive - 1;
		scoutsAlarmed	= scoutsAlarmed - 1;
		ScriptManager:storeValue( "scoutsAlarmed", scoutsAlarmed );
		ScriptManager:storeValue( "scoutsAlive", scoutsAlive );
		
		-- if all scouts are dead, secondary mission 1 ( kill all scouts ) is completed;
		-- also secondary mission 2 ( no alarm raised ) is now accomplished
		if scoutsAlive == 0 then
			ScriptManager:setMissionStatus( missionIds.killAllScouts, MissionStatus.ACCOMPLISHED );
			ScriptManager:setMissionStatus( missionIds.noAlarm, MissionStatus.ACCOMPLISHED );
		end
		
		-- if the number of scouts alarmed & alive is still > 0 after you killed one, a message to kill the rest pops up
		-- if there are none you get the message to move on
		if ScriptManager:getMissionStatus( missionIds.noAlarm ) ~= MissionStatus.FAILED then
			if scoutsAlarmed > 0 then
				ScriptManager:displaySubtitle( Text.T_MAP_SU_3_1_MISSION_SECONDARY_01_FLAVOR_01 );
			else
				ScriptManager:displaySubtitle( Text.T_MAP_SU_3_1_MISSION_SECONDARY_01_FLAVOR_02 );
			end
		end

		ScriptManager:wait( 4 );
		HideDialogue();
			
	-- if a german hq unit is destroyed ( relevant for main mission 2 )
	elseif unit:getSquadId() == 0 then
		-- morals without reinforcements:		with reinforcements
		--	infantry: 	2 x  7 = 14				4 x  7 = 28
		--	tanks:		4 x 17 = 58				5 x 17 = 75
		--	mg-nest:	1 x 19 = 19				1 x 19 = 19
		--  tiger:		0 x 40 =  0				1 x 40 = 40
		--  depot:		1 x 45 = 45				1 x 45 = 45
		--  whole moral			136						207
		--  moral in script: 	 90						160
		--	( normally a depot doesn't get focused so I reduce the moral for ~ that amount )
		if unit:getTemplateId() == Units.UNIT_DE_MP40 or unit:getTemplateId() == Units.UNIT_DE_PANZERFAUST then
			decreaseMoral( 7 );
		
		elseif unit:getTemplateId() == Units.UNIT_DE_TANK_3 or unit:getTemplateId() == Units.UNIT_DE_TANK_4 then
			decreaseMoral( 19 );
		
		elseif unit:getTemplateId() == Units.UNIT_DE_MG_NEST then
			decreaseMoral( 17 );
			
		elseif unit:getTemplateId() == Units.UNIT_DE_TIGER then
			decreaseMoral( 40 );
		
		elseif unit:getTemplateId() == Units.UNIT_DE_HQ then
			decreaseMoral( 45 );
		end
	end	
end

function decreaseMoral( value )
	germanMoral = germanMoral - value;
	checkMoral();
	ScriptManager:storeValue( "germanMoral", germanMoral );
end


-- END on Enemy attacked / killed
-- BEGIN Scoutalarm


function scoutInHq( tileX, tileY, playerId, unitTemplateId, unitId )
	if ScriptManager:getMissionStatus( missionIds.noAlarm ) ~= MissionStatus.FAILED then
		ScriptManager:storeValue( "triggerScout", unitId ); -- store ID of the scout that alarmed in order to remove it manually
		ScriptManager:runForPlayer( 1, "reinforcementSpawn" ); -- spawn reinforcements in enemy turn
		ScriptManager:runForPlayer( 0, "startAlarmCutscene" ); -- start cutscene in player turn
		
		Unit:changeSquadState( 1, 0, AiState.DEFEND ); -- set state of troops in base to defense ( instead of passive )
	else
		-- I remove only passive scouts in the if above, but there can be multiple on the run so if another one
		-- passes the trigger line, just remove him
		ScriptManager:removeUnit( Unit:getUnitById( unitId ), "" );
	end
end

function startAlarmCutscene()
	StartCutscene();
	
	ScriptManager:playSound( "sfx_siren_1" );
	ScriptManager:wait( 1 );
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_3_1_MISSION_SECONDARY_02_FLAVOR_02 );
	ScriptManager:wait( 4 );
	HideDialogue();
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_3_1_MISSION_SECONDARY_02_FLAVOR_03 );
	ScriptManager:wait( 4 );
	HideDialogue();
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_3_1_MISSION_SECONDARY_02_FLAVOR_04 );
	ScriptManager:wait( 4 );
	HideDialogue();
	
	EndCutscene();
	
	-- alarm was raised so both secondary missions fail
	ScriptManager:setMissionStatus( missionIds.noAlarm, MissionStatus.FAILED );
	ScriptManager:setMissionStatus( missionIds.killAllScouts, MissionStatus.FAILED );
	
	removeRemainingScouts();
	
	-- change state of the hq troops to defend until player arrives
	Unit:changeSquadState( 1, 0, AiState.DEFEND );
end

function reinforcementSpawn()
	-- step through the reinforcement table and spawn its content
	for i = 1, #germanReinforcements do
		local data = germanReinforcements[i]
		local unit = ScriptManager:spawnUnit(data[1], data[2], data[3], data[4], data[5], data[6] );
		
		-- store Ids in order to remove them later in victory cutscene
		ScriptManager:storeValue( string.format( "Reinforcement %s", i ), unit:getId() );
	end
	
	germanMoral = germanMoral + 70; -- increase moral due to reinforcements
	ScriptManager:storeValue( "germanMoral", germanMoral );
end

function removeRemainingScouts()
	-- check for remaining scouts on the map and remove them
	for i = scoutIdsBegin, scoutIdsEnd do
		local unit = Unit:getUnitById( i );
		if unit ~= nil then
			-- remove only the scouts that are still in hiding
			 if unit:getAiState() == AiState.PASSIVE then
				ScriptManager:removeUnit( i, "" );
			end
		end
	end
	
	-- remove the scout that triggered the line if there is one ( he wont get deleted cause he is on retreat-state )
	if ScriptManager:getMissionStatus( missionIds.noAlarm ) == MissionStatus.FAILED then
		ScriptManager:removeUnit( ScriptManager:getValue( "triggerScout" ), "" );
		ScriptManager:deleteValue( "triggerScout" );
	end
end


-- END Scoutalarm
-- BEGIN German base reached ( end of main mission 1 / start of main 2 )


function nearGermanHq()
	DisablePlayerInteraction();
	-- complete first main mission ( move to german base )
	ScriptManager:setMissionStatus( missionIds.ReachGermans, MissionStatus.ACCOMPLISHED );
	ScriptManager:wait( 3 );
	EnablePlayerInteraction();
	
	-- start almostThereCutscene
	StartCutscene();
	
	-- display message depending on having raised alarm or not
	if ScriptManager:getMissionStatus( missionIds.noAlarm ) ~= MissionStatus.FAILED then
		ScriptManager:displaySubtitle( Text.T_MAP_SU_3_1_MISSION_SECONDARY_01_FLAVOR_03 );
	else
		ScriptManager:displaySubtitle( Text.T_MAP_SU_3_1_MISSION_SECONDARY_01_FLAVOR_04 );
	end
	
	ScriptManager:wait( 4 );
	HideDialogue();
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_3_1_MISSION_SECONDARY_01_FLAVOR_05 );
	ScriptManager:wait(	4 );
	HideDialogue();
	
	EndCutscene();
	
	-- start the next main mission( break german moral )
	ScriptManager:setMissionStatus( missionIds.breakMoral, MissionStatus.IN_PROGRESS );
	
	RemoveTrigger( "nearGermanHq" );
end


-- END German base reached ( end of main mission 1 / start of main 2 )
-- BEGIN Check for victory


function checkMoral()
	-- MoralSubIndicator stopps messages from displaying more than once 
	if germanMoral < 20 and ScriptManager:getValue( "MoralSubIndicator" ) ~= 0 then -- = victory
		ScriptManager:runForPlayer( 0, "startVictoryCutscene" );		
		ScriptManager:storeValue( "MoralSubIndicator", 0 );
		
	elseif germanMoral < 55 and ScriptManager:getValue( "MoralSubIndicator" ) ~= 1 then
		ScriptManager:displaySubtitle( Text.T_MAP_SU_3_1_MISSION_PRIMARY_02_FLAVOR_02 );
		ScriptManager:sleep( 3, "HideDialogue" );		
		ScriptManager:storeValue( "MoralSubIndicator", 1 );
		
	elseif germanMoral < 90 and ScriptManager:getValue( "MoralSubIndicator" ) ~= 2 and
		   ScriptManager:getValue( "MoralSubIndicator" ) ~= 1 then
		ScriptManager:displaySubtitle( Text.T_MAP_SU_3_1_MISSION_PRIMARY_02_FLAVOR_01 );
		ScriptManager:sleep( 3, "HideDialogue" );
		ScriptManager:storeValue( "MoralSubIndicator", 2 );
	end
end


-- END Check for victory
-- BEGIN Victory Cutscene


function startVictoryCutscene()
	StartCutscene();

	ScriptManager:displaySubtitle( Text.T_MAP_SU_3_1_MISSION_PRIMARY_02_FLAVOR_03 );
	ScriptManager:moveCameraEx( 2, 3, 0.7, 3, true );
	ScriptManager:revealFogOfWar( 2, 2, 2 , true );
	HideDialogue();
	
	-- all units left alive will "flee" ( moved to map border & removeUnit )
	-- hq troops
	for i = 1, #germanHqTroops do
		local unit = germanHqTroops[ i ];
	
		if Unit:getUnitById( unit[1] ) ~= nil then -- if unit not dead yet
			
			if unit[1] ~= Units.UNIT_DE_MG_NEST then -- if not mg-nest, move it to map border
				-- sometimes the moveUnit wouldn't move a unit so I stopped using moveUnitEx
				-- and started removing them after a certain amount of time
				ScriptManager:moveUnit( unit[1], unit[2], unit[3], "");
				ScriptManager:wait( 2 );
			end
			
			ScriptManager:removeUnit( unit[1], "" );
		end
	end
	
	-- reinforcement troops ( if there )
	if ScriptManager:getMissionStatus( missionIds.noAlarm ) == MissionStatus.FAILED then
		for i = 1, 4 do
			-- if a value was saved and the unit is not dead yet
			local rUnit = Unit:getUnitById( ScriptManager:getValue( string.format( "Reinforcement %s", i ) ) );
			if rUnit ~= nil then
				-- unit i = 1 and i = 3 will flee to the south, i = 2 and i = 4 will flee to the west
				if i % 2 == 1 then -- 1 & 3
					ScriptManager:moveUnit( rUnit:getId(), 2, 1, "" );
				else -- 2 & 4
					ScriptManager:moveUnit( rUnit:getId(), 1, 2, "" );
				end			
				ScriptManager:wait( 2 );
				
				ScriptManager:removeUnit( rUnit:getId(), "" );
				rUnit:getId()
			end
		end
	end
	
	ScriptManager:wait( 1 );
	
	EndCutscene();
	DisablePlayerInteraction();
	
	ScriptManager:revealFogOfWar( 2, 2, 2, false );
	victoryEndGame()
end

function victoryEndGame()
	HideDialogue(); -- remove possible texts on the screens
	
	-- main mission 2 ( break german moral ) completed
	if ScriptManager:getMissionStatus( missionIds.breakMoral ) == MissionStatus.IN_PROGRESS then
			ScriptManager:setMissionStatus( missionIds.breakMoral, MissionStatus.ACCOMPLISHED );
	end
	
	-- secondary mission 2 ( no alarm raised ) accomplished if not failed
	if ScriptManager:getMissionStatus( missionIds.noAlarm ) == MissionStatus.IN_PROGRESS then
			ScriptManager:setMissionStatus( missionIds.noAlarm, MissionStatus.ACCOMPLISHED );
	end

	ScriptManager:displaySubtitle( Text.T_MISSION_VICTORY );
	ScriptManager:wait( 4 );
	HideDialogue();
	endGame();
end


-- END Victory Cutscene
-- BEGIN Defeated


function checkForEnd()
	if ScriptManager:noUnitsLeft( 0 ) then
		ScriptManager:runForPlayer( 0, "missionFailed" )
	end
end

function missionFailed()
	DisablePlayerInteraction();
	HideDialogue(); -- remove possible texts on the screens
		
	ScriptManager:displaySubtitle( Text.T_MISSION_FAILED );
	ScriptManager:wait( 4 );
	HideDialogue();
	endGame();
end


-- END Defeated


function endGame()
	ScriptManager:endGame()
end