require "lua/Constants"
require "lua/Functions"
require "lua/Functions_2"
require "lua/Text"
require "lua/Units"
require "math"

-- mission stuff
missionIds = {
	escortArty		= 1,
	dontLoseArty	= 2,
	killTiger 		= 3,
};

-- missionId, primary( boolean ), textkey, xpReward
missions = {
	{ missionIds.escortArty,	 true, Text.T_MAP_SU_3_2_MISSION_PRIMARY, 		175 },
	{ missionIds.dontLoseArty,	false, Text.T_MAP_SU_3_2_MISSION_SECONDARY_01,	 75 },
	{ missionIds.killTiger,		false, Text.T_MAP_SU_3_2_MISSION_SECONDARY_02,	 75 }
};

artilleriesInGoal		=  0;
artilleriesInGoalID1	= -1;
artilleriesInGoalID2	= -1;

-- indicators if named artillery got already attacked / destroyed this turn
artillery1Attacked		= false;
artillery2Attacked		= false;
artillery3Attacked		= false;
artilleryDestroyed		= false;

-- the su convoy that spawns in introCutscene 
-- some pos are the same cause I will move them in the cutscene
-- playerId, templateID, tileX, tileY, squadId, AiState
suConvoy = {
	{ 0, Units.UNIT_SU_RIFLE,				7, 10, 0, AiState.DEFEND },
	{ 0, Units.UNIT_SU_RIFLE,				5, 11, 0, AiState.DEFEND },
	{ 0, Units.UNIT_SU_RIFLE,				6, 10, 0, AiState.DEFEND },
	{ 0, Units.UNIT_SU_RIFLE,				6,  9, 0, AiState.DEFEND },
	{ 0, Units.UNIT_SU_MORTAR,				5, 10, 0, AiState.DEFEND },
	{ 0, Units.UNIT_SU_HOWITZER_TANKY,		4, 10, 0, AiState.DEFEND },
	{ 0, Units.UNIT_SU_HOWITZER_TANKY,		3,  9, 0, AiState.DEFEND },
	{ 0, Units.UNIT_SU_HOWITZER_TANKY,		3,  8, 0, AiState.DEFEND },
	{ 0, Units.UNIT_SU_TANK_T34,			5,  9, 0, AiState.DEFEND },
	{ 0, Units.UNIT_SU_TANK_KV1,			4,  9, 0, AiState.DEFEND },
	{ 0, Units.UNIT_SU_TANK_T34,			4,  8, 0, AiState.DEFEND },
	{ 0, Units.UNIT_SU_TANK_T34,			4, 11, 0, AiState.DEFEND },
	{ 0, Units.UNIT_SU_TANK_KV1,			3, 11, 0, AiState.DEFEND },
	{ 0, Units.UNIT_SU_TANK_T34,			3, 10, 0, AiState.DEFEND },
	{ 0, Units.UNIT_SU_SUPPLY_TRUCK,		2,  9, 0, AiState.DEFEND },
	{ 0, Units.UNIT_SU_SUPPLY_TRUCK,		2,  8, 0, AiState.DEFEND },
	{ 0, Units.UNIT_SU_SUPPLY_TRUCK,		1,  7, 0, AiState.DEFEND },
	{ 0, Units.UNIT_SU_SUPPLY_TRUCK,		2,  6, 0, AiState.DEFEND },
	{ 0, Units.UNIT_SU_RIFLE,				1,  9, 0, AiState.DEFEND },
	{ 0, Units.UNIT_SU_RIFLE,				3,  6, 0, AiState.DEFEND }
};

-- the names of the stored convoy ids
-- so i can acces them via ScriptManager:getvalue
suConvoyIds = {
	"suFrontInfantryRifle1",
	"suFrontInfantryRifle2",
	"suFrontInfantryRifle3",
	"suFrontInfantryRifle4",
	"suFrontInfantryMortar",
	"suArtillery1",
	"suArtillery2",
	"suArtillery3",
	"suBotTankMedium1",
	"suBotTankHeavy",
	"suBotTankMedium2",
	"suTopTankMedium1",
	"suTopTankHeavy",
	"suTopTankMedium2",
	"suBackSupplyTruck1",
	"suBackSupplyTruck2",
	"suBackSupplyTruck3",
	"suBackSupplyTruck4",
	"suBackInfantryRifle1",
	"suBackInfantryRifle2"
	
};

-- the units that are ambushing in the intro cutscene
gerAssaultUnits = {
	-- top
	{ 1, Units.UNIT_DE_TANK_4,		1, 14, 21, AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_DE_TANK_4,		2, 14, 21, AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_DE_TIGER,		3, 14, 21, AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_DE_MP40,		4, 14, 21, AiState.ATTACK_AGGRESSIVE },
	-- bot
	{ 1, Units.UNIT_DE_TANK_4,		7,  3, 21, AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_DE_TIGER,		6,  3, 21, AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_DE_MP40,		8,  3, 21, AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_DE_MP40,		2,  3, 21, AiState.ATTACK_AGGRESSIVE },
	-- top again, but he spawns extra cause he will attack the rear supply trucks and should not be visible from the very beginning
	{ 1, Units.UNIT_DE_PANZERFAUST,	1, 12, 21, AiState.ATTACK_AGGRESSIVE }
}

-- the names of the german assault units
-- so I can acces them via ScriptManager:getvalue
gerAssaultUnitIds = { 
	"gerTopTank1",
	"gerTopTank2",
	"gerTopTiger",
	"gerTopMp",
	"gerBotTank",
	"gerBotTiger",
	"gerBotMp1",
	"gerBotMp2",
	"gerTopPanzerfaust"
 };

instantAmbushTopSquad	= 7;
instantAmbushBotSquad	= 6;
turnAmbushSquad			= 9;
barricadeAmbushSquad	= 2;
botAmbushSquad			= 4;
endAmbushSquad			= 3;
 
 -- get spawned when near the tank barricades
ambushUnitsBarricades = {
 	{ 1, Units.UNIT_DE_TANK_4,		13, 19, 37, AiState.ATTACK_AGGRESSIVE },
 	{ 1, Units.UNIT_DE_MP40,		14, 19, 37, AiState.ATTACK_AGGRESSIVE },
 	{ 1, Units.UNIT_DE_MP40,		15, 18, 37, AiState.ATTACK_AGGRESSIVE }
 };

-- Ids of units on the map that gets stealthed @beginning
--[[	units @top begining:	 6-tank4,		30-mp40,	32-panzerfaust
		units @bot beginning:	 8-tank4,		 9-mp40,	31-panzerfaust
		units @top turn:		 3-panzerfaust,	 7-tank3,	28-panzerfaust,	29-tank3
		units @first barrier:	 0-panzerfaust,	 1-tiger,	10-mg-nest,		14-mp40
		units @bot turn:		24-tank4,		25-mp40
		units @end:				 2-tiger,		 4-mp40,	12-tank3,		22-mp40,	23-tank4	]]
camouflageUnitIds = { 0, 1, 2, 3, 4, 6, 7, 8, 9, 10, 12, 14, 22, 23, 24, 25, 28, 29, 30, 31, 32 };
tigerId = 1; -- the tiger that has to be destroyed for the secondary mission 2


-- BEGIN Init


function onGameStart()
	CreateMissions( missions );
	setTriggers();
	storeValues();
	
	ScriptManager:gainGold( 0, 2000 );
	
	-- hide enemies on the map
	camouflageEnemy();
	
	-- start intro cutscene
	ScriptManager:sleep( 0, "startIntroCutscene" ); -- without sleep player turn popup shows
end

function camouflageEnemy()
	for i = 1, #camouflageUnitIds do
		CamouflageUnit( camouflageUnitIds[i] );
	end
end

function storeValues()
	ScriptManager:storeValue( "artilleriesInGoal", 		artilleriesInGoal );
	ScriptManager:storeValue( "artilleriesInGoalID1",	artilleriesInGoalID1);
	ScriptManager:storeValue( "artilleriesInGoalID2",	artilleriesInGoalID2);
end

function setTriggers()
	-- load all needed values after loading
	ScriptManager:addOnGameResumeTrigger( "loadValues", "loadValues", false );
	
	-- reset artilleries
	ScriptManager:addOnTurnStartTrigger( 0, -1, -1, "resetArtillery", "resetArtillery", false );
	
	-- trigger to start German ambush
	ScriptManager:addOnUnitSelectedTrigger( 0, -1, -1, "startGermanAmbush", "startGermanAmbush", false );
	
	-- trigger for destroying artillery
	ScriptManager:addOnUnitDestroyedTrigger( 0, Units.UNIT_SU_HOWITZER_TANKY, -1, "onArtilleryDestroyed", "onArtilleryDestroyed", false );
	
	-- check for attacked artilleries
	ScriptManager:addOnUnitHasAttackedTrigger( 1, -1, -1, "onEnemyAttack", "onEnemyAttack", false );
	
	-- trigger for secondary mission 2
	ScriptManager:addOnUnitHasAttackedTrigger(	1, Units.UNIT_DE_TIGER, tigerId, "tigerHasAttacked", "tigerHasAttacked", false );
	ScriptManager:addOnUnitDestroyedTrigger(	1, Units.UNIT_DE_TIGER, tigerId, "tigerDestroyed",	 "tigerDestroyed", false );
	
	-- create some ambushes	
	ScriptManager:addOnUnitSightedTrigger( 0, -1, -1, false, "startAmbushes", "startAmbushes", false );
	
	-- goal trigger circles ( gets triggered when arti moves over )
	-- for artillery moving into the endzone
	local y;
	local x;
	for x = 18, 21 do
		if x == 18 then
			y = 11
		elseif x == 19 then
			for y = 12, 13 do
				ScriptManager:addOnEnterTileTrigger( x, y, 0, Units.UNIT_SU_HOWITZER_TANKY, -1, false, "artilleryInGoal", "artilleryInGoal", false );
			end
			for y = 9, 10 do
				ScriptManager:addOnEnterTileTrigger( x, y, 0, Units.UNIT_SU_HOWITZER_TANKY, -1, false, "artilleryInGoal", "artilleryInGoal", false );
			end			
		else
			y = 13;
			ScriptManager:addOnEnterTileTrigger( x, y, 0, Units.UNIT_SU_HOWITZER_TANKY, -1, false, "artilleryInGoal", "artilleryInGoal", false );
			y = 9;
		end
		ScriptManager:addOnEnterTileTrigger( x, y, 0, Units.UNIT_SU_HOWITZER_TANKY, -1, false, "artilleryInGoal", "artilleryInGoal", false );
	end
	
	-- for artillery moving out of the endzone
	for x = 18, 21 do
		y = 14;
		ScriptManager:addOnEnterTileTrigger( x, y, 0, Units.UNIT_SU_HOWITZER_TANKY, -1, false, "artilleryOutOfGoal", "artilleryOutOfGoal", false );
		y =  8;
		ScriptManager:addOnEnterTileTrigger( x, y, 0, Units.UNIT_SU_HOWITZER_TANKY, -1, false, "artilleryOutOfGoal", "artilleryOutOfGoal", false );
	end
	
end

function loadValues()
	artilleriesInGoal		= ScriptManager:getValue( "artilleriesInGoal" );
	artilleriesInGoalID1	= ScriptManager:getValue( "artilleriesInGoalID1" );
	artilleriesInGoalID2	= ScriptManager:getValue( "artilleriesInGoalID2" );
end

function resetArtillery()
	-- reset the values for next turn
	artillery1Attacked = false;
	artillery2Attacked = false;
	artillery3Attacked = false;
	artilleryDestroyed = false;
end


-- END Init
-- BEGIN Intro Cutscene


function startIntroCutscene()
	StartCutscene();
	
	-- convoy movement start
	ScriptManager:displaySubtitle( Text.T_MAP_SU_3_2_MISSION_PRIMARY_FLAVOR_01 );
	ScriptManager:moveCameraEx( 20, 11, 0.7, 6, true );
	spawnConvoy();
	ScriptManager:wait( 1 ); -- antirush
	HideDialogue();
	ScriptManager:wait( 1 ); -- antirush
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_3_2_MISSION_PRIMARY_FLAVOR_02 );
	ScriptManager:moveCameraEx( 3,  9, 0.6, 6, true );
	ScriptManager:wait( 1 ); -- antirush
	HideDialogue();
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_3_2_MISSION_PRIMARY_FLAVOR_03 )
	ScriptManager:wait( 5 );
	HideDialogue()
	
	if ScriptManager:isControllerConnected() then
		ScriptManager:setControllerCursorPosition( 4, 10 );
	end
	
	EndCutscene();
	
	startTheMission();	
end
-- convoy movement complete
-- german assault start, starts when player selects a unit
function startGermanAmbush()
	spawnGerAssaultUnits();
	
	StartCutscene();
	
	ScriptManager:moveCameraEx( 3, 9, 0.6, 1, true );
	
	-- move top tanks for attack
	ScriptManager:moveUnitEx( ScriptManager:getValue( "gerTopTank1" ),				2, 13, false );
	ScriptManager:moveUnitEx( ScriptManager:getValue( "gerTopTank2" ),				3, 13, false );
	ScriptManager:moveUnitEx( ScriptManager:getValue( "gerTopMp" ),					5, 12, false );
	ScriptManager:moveUnitEx( ScriptManager:getValue( "gerTopTiger" ),				4, 13,  true );
	
	killUnit( ScriptManager:getValue( "gerTopTiger" ),				ScriptManager:getValue( "suTopTankHeavy" ) );
	
	-- i don't want to kill this unit so I just attack it and it survives with low hp
	ScriptManager:attackUnitEx( ScriptManager:getValue( "gerTopTank1" ), ScriptManager:getValue( "suTopTankMedium1" ),	true );
	ScriptManager:attackUnitEx( ScriptManager:getValue( "gerTopTank2" ), ScriptManager:getValue( "suTopTankMedium1" ),	true );
	
	killUnit( ScriptManager:getValue( "gerTopMp" ),					ScriptManager:getValue( "suFrontInfantryRifle2" ) );

	ScriptManager:displaySubtitle( Text.T_MAP_SU_3_2_MISSION_PRIMARY_FLAVOR_04 );
	ScriptManager:wait( 3 )
	HideDialogue();
	
	ScriptManager:moveUnitEx( ScriptManager:getValue( "gerBotTank" ),				5, 7,  true );
	ScriptManager:moveUnitEx( ScriptManager:getValue( "gerBotMp1" ),				6, 8, false );
	ScriptManager:moveUnitEx( ScriptManager:getValue( "gerBotTiger" ),				5, 6,  true );
	
	killUnit( ScriptManager:getValue( "gerBotTiger" ),				ScriptManager:getValue( "suBotTankMedium2" ) );
	
	-- i dont want to lose this unit, so i just attack it and it survives with low hp
	ScriptManager:attackUnitEx( ScriptManager:getValue( "gerBotMp1" ),	 ScriptManager:getValue( "suFrontInfantryRifle4" ), true );

	ScriptManager:displaySubtitle( Text.T_MAP_SU_3_2_MISSION_PRIMARY_FLAVOR_05 );
	ScriptManager:wait( 3 )
	HideDialogue();
	
	killUnit( ScriptManager:getValue( "suArtillery2" ),				ScriptManager:getValue( "gerTopTiger" ) );
	killUnit( ScriptManager:getValue( "suTopTankMedium2" ),			ScriptManager:getValue( "gerTopTank1" ) );
	killUnit( ScriptManager:getValue( "suArtillery1" ),				ScriptManager:getValue( "gerTopTank2" ) );
	killUnit( ScriptManager:getValue( "suArtillery3" ),				ScriptManager:getValue( "gerBotTiger" ) );
	killUnit( ScriptManager:getValue( "suFrontInfantryRifle3" ),	ScriptManager:getValue( "gerTopMp" ) );
	killUnit( ScriptManager:getValue( "suFrontInfantryRifle1" ),	ScriptManager:getValue( "gerBotMp1" ) );

	ScriptManager:displaySubtitle( Text.T_MAP_SU_3_2_MISSION_PRIMARY_FLAVOR_06 );
	ScriptManager:sleep( 4, "HideDialogue" );
	
	ScriptManager:moveUnitEx( ScriptManager:getValue( "gerBotMp2" ),				4, 6, true );
	
	killUnit( ScriptManager:getValue( "suBackInfantryRifle2" ),		ScriptManager:getValue( "gerBotMp2" ) );
	killUnit( ScriptManager:getValue( "suBotTankHeavy" ),			ScriptManager:getValue( "gerBotTank" ) );

	spawnAssaultUnit( #gerAssaultUnitIds ); -- spawn last unit here, cause it would've been visible too early if I had spawned it with the loop
	
	killUnit( ScriptManager:getValue( "gerTopPanzerfaust" ),		ScriptManager:getValue( "suBackSupplyTruck1" ) );
	
	ScriptManager:moveUnitEx( ScriptManager:getValue( "suBackInfantryRifle1" ),		1, 10, true );
	
	killUnit( ScriptManager:getValue( "suBackInfantryRifle1" ),		ScriptManager:getValue( "gerTopPanzerfaust" ) );
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_3_2_MISSION_PRIMARY_FLAVOR_07 );
	ScriptManager:wait( 4 );
	HideDialogue();
	
	-- german assault complete

	EndCutscene();
	DisablePlayerInteraction(); -- stop player from messing around an possibly stopping the missions from starting
	
	-- resupply all convoyUnits
	for i = 1, #suConvoyIds do
		local unit = Unit:getUnitById( ScriptManager:getValue( suConvoyIds[i] ) );
		
		-- no dead units or supply trucks ( cause it throws an error 'invalid call Unit.setAmmo() on non-combat unit'
		if  unit ~= nil
			and unit:getTemplateId() ~= Units.UNIT_SU_SUPPLY_TRUCK then
			-- random value, i can't "overload" them, so i just took a high number just to be sure
			unit:setFuel( 100 ); 
			unit:setAmmo( 100 );
		end
	end
	
	-- let the ambush units @ top and bot get aggressive
	Unit:changeSquadState( 1, instantAmbushTopSquad, AiState.ATTACK_AGGRESSIVE );
	Unit:changeSquadState( 1, instantAmbushBotSquad, AiState.ATTACK_AGGRESSIVE );
	
	RemoveTrigger( "startGermanAmbush" );
	EnablePlayerInteraction();
end

function spawnConvoy()
	-- only spawn the first six units, the others follow in moveAndCompleteConvoy
	for i = 1, #suConvoy do
		spawnConvoyUnit( i );
	end
	
end

function spawnConvoyUnit( unitIndex )
	-- spawns a convoyUnit and saves the id. it can be called with the string in the suConvoyIds
	local spawnData = suConvoy[unitIndex];
	local unit = ScriptManager:spawnUnit( spawnData[1], spawnData[2], spawnData[3], spawnData[4], spawnData[5], spawnData[6] );
	
	-- save the id
	ScriptManager:storeValue( suConvoyIds[unitIndex], unit:getId() );
end

function spawnAssaultUnit( unitIndex )
	-- spawns a assaultUnit and saves the id. it can be called with the string in the gerAssaultUnitIds
	local spawnData = gerAssaultUnits[unitIndex];
	local unit = ScriptManager:spawnUnit( spawnData[1], spawnData[2], spawnData[3], spawnData[4], spawnData[5], spawnData[6] );
	ScriptManager:camouflageUnit( unit:getId() );
	
	-- save the id
	ScriptManager:storeValue( gerAssaultUnitIds[unitIndex], unit:getId() );
end

function spawnGerAssaultUnits()
	-- dont spawn last unit, that one spawns manually in assaultActions()
	for i = 1, #gerAssaultUnits - 1 do
		spawnAssaultUnit( i );
	end
end

function killUnit( attacker, killedUnit )
	-- set the health of the killedUnit to 1 and then get it killed by the attacker
	healthRemover( Unit:getUnitById( killedUnit ) );
	ScriptManager:attackUnitEx( attacker, killedUnit,	true );
end

function healthRemover( unit )
	unit:setHealth( 1 );
end

function startTheMission()
	ScriptManager:setMissionProgress(	missionIds.escortArty, 3 ); -- the player has 3 artis, so progress is 3 
	ScriptManager:setMissionStatus(		missionIds.escortArty,		MissionStatus.IN_PROGRESS ); -- go through mountain
	ScriptManager:setMissionStatus( 	missionIds.dontLoseArty,	MissionStatus.IN_PROGRESS ); -- lose no arti
	
	--TODO: remove this function & trigger!
	--testing();
	--ScriptManager:addOnUnitSelectedTrigger( 1, -1, -1, "showParameters", "showParameters",false);
end

function testing()
	ScriptManager:spawnUnit( 0, Units.GOTT, 7, 11, 0, AiState.WAIT );
--	ScriptManager:spawnUnit( 0, Units.UNIT_SU_HOWITZER_TANKY, 14, 13, 0, AiState.WAIT );
--	ScriptManager:spawnUnit( 0, Units.UNIT_SU_HOWITZER_TANKY, 20, 7, 0, AiState.WAIT );
--	ScriptManager:spawnUnit( 0, Units.UNIT_SU_HOWITZER_TANKY, 21, 7, 0, AiState.WAIT );
--	ScriptManager:removeUnit( 22, "" );
--	ScriptManager:removeUnit( 23, "" );
--	ScriptManager:removeUnit( 20, "" );
--	ScriptManager:removeUnit( 19, "" );
--	ScriptManager:removeUnit( 4, "" );
--	onArtilleryDestroyed();
end

function showParameters( selectedUnit )
	print( string.format( "[AiState] %s", selectedUnit:getAiState() ) );
end


-- END Intro Cutscene
-- BEGIN Tiger Mission


function tigerHasAttacked( attacker, defender )
	ScriptManager:storeValue( "tigerId", attacker:getId() );
	ScriptManager:runForPlayer( 0, "startKillTigerMission" );
end

function startKillTigerMission()
	table.insert(cutsceneQueue, startKillTigerCutscene)
	SubtitleQueue()
	
	RemoveTrigger( "tigerHasAttacked" );
end

function startKillTigerCutscene()
	DisablePlayerInteraction();
	if ScriptManager:getMissionStatus( missionIds.killTiger ) == MissionStatus.NOT_STARTED then
		ScriptManager:setMissionStatus( missionIds.killTiger, MissionStatus.IN_PROGRESS );
	end

	local tiger  = Unit:getUnitById( ScriptManager:getValue( "tigerId" ) );
	local tigerX = tiger:getX();
	local tigerY = tiger:getY();
	
	ScriptManager:revealFogOfWar(	tigerX, tigerY, 1, true );
	ScriptManager:moveCameraEx(		tigerX, tigerY, 0.7, 2, true );
	ScriptManager:showMarkerAt(		tigerX, tigerY );
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_3_2_MISSION_SECONDARY_02_FLAVOR );
	ScriptManager:wait( 4 );
	HideDialogue();
	HideAllMarkers();	
	ScriptManager:revealFogOfWar( tigerX, tigerY, 1, false );
	EnablePlayerInteraction();
end

function tigerDestroyed()
	ScriptManager:setMissionStatus( missionIds.killTiger, MissionStatus.ACCOMPLISHED );
end


-- END Tiger Mission
-- BEGIN Ambush


function startAmbushes( seeingUnit, seenUnit )
	local seeingSquad = seeingUnit:getSquadId();
	
	if seeingUnit:getAiState() == AiState.PASSIVE then
		if seeingSquad == turnAmbushSquad then
			makeSquadAggressive( turnAmbushSquad );
			
		elseif seeingSquad == barricadeAmbushSquad then
			makeSquadAggressive( barricadeAmbushSquad );
			ScriptManager:runForPlayer( 1, "spawnAmbushBarricades" );
			
		elseif seeingSquad == botAmbushSquad then
			makeSquadAggressive( botAmbushSquad );
			
		elseif seeingSquad == endAmbushSquad then
			makeSquadAggressive( endAmbushSquad );
		end
	end
end

function spawnAmbushBarricades()
	-- spawn some additional troops that come from behind
	local spawnData;
	for i = 1, #ambushUnitsBarricades do
		spawnData = ambushUnitsBarricades[i];
		spawnAmbushUnit( spawnData );
	end
end

function makeSquadAggressive( squadId )
	Unit:changeSquadState( 1, squadId, AiState.ATTACK_AGGRESSIVE );
end

function spawnAmbushUnit( spawnData )
	ScriptManager:spawnUnit( spawnData[1], spawnData[2], spawnData[3], spawnData[4], spawnData[5], spawnData[6] );
end


-- END Ambush
-- BEGIN Artillery attacked / destroyed


function onEnemyAttack( attacker, defender )
	if defender:getTemplateId() == Units.UNIT_SU_HOWITZER_TANKY then
		artilleryId = defender:getId();
		ScriptManager:runForPlayer( 0, "artilleryGotAttackedWarning");
	end
end

function artilleryGotAttackedWarning()
	-- if arti is attacked and not destroyed move camera to attacked arti, show a marker and display a warning
	local unit = Unit:getUnitById( artilleryId );
	
	if unit ~= nil then
		if unit:getId() == ScriptManager:getValue( "suArtillery1" ) and artillery1Attacked == false then
			artillery1Attacked = true; -- do only display the sub once
			table.insert( cutsceneQueue, artillery1GotAttackedWarningSub )
			print( "start1" );
			SubtitleQueue()
		elseif unit:getId() == ScriptManager:getValue( "suArtillery2" ) and artillery2Attacked == false then
			artillery2Attacked = true; -- do only display the sub once
			table.insert( cutsceneQueue, artillery2GotAttackedWarningSub )
			print( "start2" );
			SubtitleQueue()
		elseif unit:getId() == ScriptManager:getValue( "suArtillery3" ) and artillery3Attacked == false then
			artillery3Attacked = true; -- do only display the sub once
			table.insert( cutsceneQueue, artillery3GotAttackedWarningSub )
			print( "start3" );
			SubtitleQueue()
		end
	end
end

-- as i have to queue the subtitle scenes below i cant just make one of it
-- or at least i dont know a better way with lua / this engine
function artillery1GotAttackedWarningSub()
	DisablePlayerInteraction();
	local artillery1 = Unit:getUnitById( ScriptManager:getValue( "suArtillery1") );
	print( "arty1" );
	ScriptManager:displaySubtitle( Text.T_MAP_SU_3_2_MISSION_SECONDARY_01_FLAVOR_01 );
	ScriptManager:showMarkerAt( artillery1:getX(), artillery1:getY() );
	ScriptManager:moveCameraEx(	artillery1:getX(), artillery1:getY(), 1.0, 1.5, true );
	ScriptManager:wait( 1.5)
	HideAllMarkers();
	HideDialogue();
	EnablePlayerInteraction();	
end

function artillery2GotAttackedWarningSub()
	DisablePlayerInteraction();
	local artillery2 = Unit:getUnitById( ScriptManager:getValue( "suArtillery2") );
print( "arty2" );
	ScriptManager:displaySubtitle( Text.T_MAP_SU_3_2_MISSION_SECONDARY_01_FLAVOR_01 );
	ScriptManager:showMarkerAt( artillery2:getX(), artillery2:getY() );
	ScriptManager:moveCameraEx(	artillery2:getX(), artillery2:getY(), 1.0, 1.5, true );
	ScriptManager:wait( 1.5)
	HideAllMarkers();
	HideDialogue();
	EnablePlayerInteraction();	
end

function artillery3GotAttackedWarningSub()
	DisablePlayerInteraction();
	local artillery3 = Unit:getUnitById( ScriptManager:getValue( "suArtillery3" ) );
	print( "arty3" );
	ScriptManager:displaySubtitle( Text.T_MAP_SU_3_2_MISSION_SECONDARY_01_FLAVOR_01 );
	ScriptManager:showMarkerAt( artillery3:getX(), artillery3:getY() );
	ScriptManager:moveCameraEx(	artillery3:getX(), artillery3:getY(), 1.0, 1.5, true );
	ScriptManager:wait( 1.5)
	HideAllMarkers();
	HideDialogue();
	EnablePlayerInteraction();	
end

function onArtilleryDestroyed()
	-- reduce number of artis in main mission by 1
	ScriptManager:setMissionProgress( missionIds.escortArty, ScriptManager:getMissionProgress( 1 ) - 1 );
	
	-- check if there are still artis alive
	if ScriptManager:getMissionProgress( missionIds.escortArty ) > 0 then
		-- check if all living artis are in the goal
		if ScriptManager:getMissionProgress( missionIds.escortArty ) == artilleriesInGoal then
			ScriptManager:runForPlayer( 0, "startVictoryCutscene");
		else
			ScriptManager:runForPlayer( 0, "showMissionProgress" );
		end
	else
		ScriptManager:runForPlayer( 0, "missionFailed" );
	end
end

function showMissionProgress()	
	-- if all three artis will be destroyed in one turn he shouldn't show the progress popup
	if ScriptManager:getMissionProgress( missionIds.escortArty ) == 0 then
		ScriptManager:displaySubtitle( Text.T_MAP_SU_3_2_MISSION_PRIMARY_FLAVOR_08 );
		ScriptManager:wait( 3 );
		HideDialogue();
		missionFailed();
	else
		if artilleryDestroyed == false then
			artilleryDestroyed = true;
			table.insert( cutsceneQueue, artilleryGotDestroyedMissionProgressSub )
			SubtitleQueue()
		end
	end
end

function artilleryGotDestroyedMissionProgressSub()
	-- fail secondary mission ( dont lose any arti ) if one is destroyed
	if ScriptManager:getMissionStatus( missionIds.dontLoseArty ) == MissionStatus.IN_PROGRESS then
		ScriptManager:setMissionStatus( missionIds.dontLoseArty, MissionStatus.FAILED );
	end
	
	ScriptManager:preparePopupText();
	ScriptManager:addPopupTextParameter( "x",ScriptManager:getMissionProgress( missionIds.escortArty ) );
	ScriptManager:displaySubtitle( Text.T_MAP_SU_3_2_MISSION_SECONDARY_01_COUNTER );
	ScriptManager:sleep( 3, "HideDialogue" );
end


-- END Artillery attacked / destroyed
-- BEGIN Artillery in goal


function artilleryInGoal( tileX, tileY, playerId, unitTemplateId, unitId )
	-- artillery moves into goal zone
	--if no register spot is taken by the arti, add the arti
	if artilleriesInGoalID1 ~= unitId and artilleriesInGoalID2 ~= unitId then
		if artilleriesInGoalID1 == -1 then
			artilleriesInGoalID1 = unitId;
			artilleriesInGoal = artilleriesInGoal + 1;
			ScriptManager:storeValue( "artilleriesInGoal", artilleriesInGoal );
			ScriptManager:storeValue( "artilleriesInGoalID1", artilleriesInGoalID1 );
			
		elseif artilleriesInGoalID2 == -1 then
			artilleriesInGoalID2 = unitId;
			artilleriesInGoal = artilleriesInGoal + 1;
			ScriptManager:storeValue( "artilleriesInGoal", artilleriesInGoal );
			ScriptManager:storeValue( "artilleriesInGoalID1", artilleriesInGoalID1 );
		else	
			-- if both spots are taken and the ids dont match, it means all artilleries made it to the goal -> victory
			startVictoryCutscene();
		end
	end
	
	-- check if all living artis are in the goal ( cause the if above only detects victory if all artis are alive)
	if ScriptManager:getMissionProgress( missionIds.escortArty ) == artilleriesInGoal then
		startVictoryCutscene();
	end
end

function artilleryOutOfGoal( tileX, tileY, playerId, unitTemplateId, unitId )
	-- artillery moved out of goal zone
	if artilleriesInGoalID1 == unitId then
		artilleriesInGoalID1 = -1;
		artilleriesInGoal = artilleriesInGoal - 1;
		ScriptManager:storeValue( "artilleriesInGoal", artilleriesInGoal );
		ScriptManager:storeValue( "artilleriesInGoalID1", artilleriesInGoalID1 );
		
	elseif artilleriesInGoalID2 == unitId then
		artilleriesInGoalID2 = -1;
		artilleriesInGoal = artilleriesInGoal - 1;
		ScriptManager:storeValue( "artilleriesInGoal", artilleriesInGoal );
		ScriptManager:storeValue( "artilleriesInGoalID1", artilleriesInGoalID1 );
	end
end


-- END Artillery in goal
-- BEGIN Victory


function startVictoryCutscene()
	StartCutscene();
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_3_1_MISSION_PRIMARY_FLAVOR_10 );
	ScriptManager:wait( 4 );
	HideDialogue();
	
	EndCutscene();
	DisablePlayerInteraction();
	
	victoryEndGame();
end

function victoryEndGame()
	HideDialogue(); -- remove possible texts on the screens

	-- accomplish main missions
	ScriptManager:setMissionStatus( missionIds.escortArty, MissionStatus.ACCOMPLISHED );
	
	-- accomplish sec mission if all artis are still alive
	if ScriptManager:getMissionProgress( missionIds.escortArty ) == 3 then
		ScriptManager:setMissionStatus( missionIds.dontLoseArty, MissionStatus.ACCOMPLISHED );
	end
	
	ScriptManager:displaySubtitle( Text.T_MISSION_VICTORY );
	ScriptManager:wait( 4 );
	HideDialogue();
	endGame();
end


-- END Victory
-- BEGIN Defeated


function missionFailed()
	DisablePlayerInteraction();
	HideDialogue(); -- remove possible texts on the screens
		
	ScriptManager:displaySubtitle( Text.T_MISSION_FAILED );
	ScriptManager:wait( 4 );
	HideDialogue();
	endGame();
end


-- END Defeated


function endGame()
	ScriptManager:endGame()
end