require "lua/Constants"
require "lua/Functions"
require "lua/Functions_2"
require "lua/Text"
require "lua/Units"
require "math"

-- mission stuff
missionIds = {
	defeatGermans 		= 1,
	captureSupplyDepot	= 2,
	breakThrough		= 3,
	killArtillery  		= 4,
	destroyAirfield  	= 5
};

-- missionId, primary( boolean ), textkey, xpReward
missions = {
	{ missionIds.defeatGermans, 	 true, Text.T_MAP_SU_3_3_MISSION_PRIMARY_01,	175 },
	{ missionIds.captureSupplyDepot, true, Text.T_MAP_SU_3_3_MISSION_PRIMARY_02,	125 },
	{ missionIds.breakThrough,		 true, Text.T_MAP_SU_3_3_MISSION_PRIMARY_03,	100 },
	{ missionIds.killArtillery,		false, Text.T_MAP_SU_3_3_MISSION_SECONDARY_01,	 50 },
	{ missionIds.destroyAirfield,	false, Text.T_MAP_SU_3_3_MISSION_SECONDARY_02,	 75 }
};

-- enemy ids / units
-- table of all squadIds 
enemySquadIds = {
	firstBarricadeSquad			= 1,
	secondBarricadeSquad		= 2,
	thirdBarricadeSquad			= 3,
	headquarterDefenseSquad		= 4,
	icePathDefenseSquad 		= 6,
	thirdBarricadeAttackWave	= 7,
	planeAttackWave				= 8,
};

firstBarricadeUnits = {
--	unitId,	templateId
	{  6, Units.UNIT_DE_MG_NEST },
	{  9, Units.UNIT_DE_ARTILLERY },
	{ 24, Units.UNIT_DE_TANK_4 },
	{ 25, Units.UNIT_DE_MP40 },
	{ 26, Units.UNIT_DE_MP40 },
	{ 27, Units.UNIT_DE_PANZERFAUST },
	{ 28, Units.UNIT_DE_TANK_4 }
};
secondBarricadeUnits = {
--	unitId,	templateId
	{  5, Units.UNIT_DE_MG_NEST },
	{ 29, Units.UNIT_DE_MG_NEST },
	{ 30, Units.UNIT_DE_TANK_DESTROYER },
	{ 31, Units.UNIT_DE_TANK_3 },
	{ 32, Units.UNIT_DE_MP40 },
	{ 33, Units.UNIT_DE_PANZERFAUST }
};
thirdBarricadeUnits = {
--	unitId,	templateId
	{  0, Units.UNIT_SU_BUNKER },
	{  1, Units.UNIT_SU_BUNKER },
	{  8, Units.UNIT_DE_MG_NEST },
	{ 10, Units.UNIT_DE_MG_NEST },
	{ 34, Units.UNIT_DE_MP40 },
	{ 35, Units.UNIT_DE_MP40 },
	{ 36, Units.UNIT_DE_TANK_4 },
	{ 37, Units.UNIT_DE_TANK_4 }
};
supplyDepotDefenseUnits = {
--	unitId,	templateId
	{ 38, Units.UNIT_DE_TANK_4 },
	{ 39, Units.UNIT_DE_TANK_4 },
	{ 40, Units.UNIT_DE_TIGER },
	{ 41, Units.UNIT_DE_TANK_3 },
	{ 42, Units.UNIT_DE_MP40 },
	{ 43, Units.UNIT_DE_MP40 },
	{ 44, Units.UNIT_DE_MP40 },
	{ 45, Units.UNIT_DE_PANZERFAUST },
	{ 46, Units.UNIT_DE_PANZERFAUST },
	{ 47, Units.UNIT_DE_PANZERFAUST }
};
icePathDefenseUnits = {
--	unitId,	templateId
	{ 49, Units.UNIT_DE_MP40 },
	{ 50, Units.UNIT_DE_MP40 }
};

-- additional enemies that spawn from the supply depot to defend the third barricade
-- they respawn in a certain interval ( every x turns ) until the barricade is gone
spawnAttackWaves				=  0; -- 0 == false; 1 == true; cant save booleans so I have to use numbers
attackWaveUnitsAlive			=  0; -- indicates how much units are alive ( in order to cap them )
maxAttackUnits					=  4; -- indicates how much units can be alive at the same time
attackWaveRespawnInterval		=  4; -- must be an even number! ( see onEnemyTurnStart for Detail )
lastAttackWaveSpawnedInRound	= -1; -- indicates the round in which the last wave spawned
thirdBarricadeAttackWaveUnits = {
--	playerId, templateID, 	tileX, tileY, 		squadId,						AiState
	{ 1, Units.UNIT_DE_MP40,	2, 43, enemySquadIds.thirdBarricadeAttackWave, AiState.ATTACK },
	{ 1, Units.UNIT_DE_TANK_4,	3, 44, enemySquadIds.thirdBarricadeAttackWave, AiState.ATTACK }
};

-- I spawn either two Fighters or one Bomber depending on the number of spawns and the round count
-- ( every x turns I spawn an plane wave and every y spawns the bomber comes instead of fighters )
spawnPlaneWaves				=  0; -- 0 == false; 1 == true; cant save booleans so I have to use numbers
planeWaveUnitsAlive			=  0; -- indicates how many plane units are alive ( in order to cap them )
maxPlaneUnits				=  5; -- indicates how many plane units can be alive at the same time
planeWaveRespawnInterval	=  4; -- must be an even number! ( see onEnemyTurnStart for Detail )
bomberAppearance			=  3; -- every bomberAppearance-spawn a bomber appears instead of two fighters
numberOfPlaneWavesSpawned	=  0; -- the number of plane waves that have already spawend in this game ( needed to check for bomber )
lastPlaneWaveSpawnedInRound	= -1; -- indicates the round in which the last wave spawned
planeAttackWaveUnits = {
--	playerId, templateID, 	tileX, tileY,		 squadId,				AiState
	{ 1, Units.UNIT_DE_FIGHTER,	7, 43, enemySquadIds.planeAttackWave, AiState.AIRPLANE },
	{ 1, Units.UNIT_DE_FIGHTER,	8, 43, enemySquadIds.planeAttackWave, AiState.AIRPLANE },
	{ 1, Units.UNIT_DE_BOMBER,	8, 43, enemySquadIds.planeAttackWave, AiState.AIRPLANE }
};

-- number of living units of the last german defense line ( for third main mission )
lastDefenseLineUnitsLeft = #thirdBarricadeUnits;
hqDefenseUnitsLeft = #supplyDepotDefenseUnits;
hqAlive = 1;


-- BEGIN Init


function onGameStart()
	CreateMissions( missions );
	setTriggers();
	storeValues();	
	camouflageEnemies();
	
	ScriptManager:gainGold( 0, 1000 ); 
	
	-- start intro cutscene
	ScriptManager:sleep( 0, "startIntroCutscene" ); -- without sleep player turn popup shows
end

function camouflageEnemies()
	camouflageEnemySquad( firstBarricadeUnits );
	camouflageEnemySquad( secondBarricadeUnits );
	camouflageEnemySquad( thirdBarricadeUnits );
	camouflageEnemySquad( supplyDepotDefenseUnits );
	camouflageEnemySquad( icePathDefenseUnits );
end

function camouflageEnemySquad( unitTable )
	for i = 1, #unitTable do
		CamouflageUnit( unitTable[i][1] );
	end	
end

function storeValues()
	ScriptManager:storeValue( "spawnAttackWaves",				spawnAttackWaves );
	ScriptManager:storeValue( "atackWaveUnitsAlive", 			attackWaveUnitsAlive );
	ScriptManager:storeValue( "lastAttackWaveSpawnedInRound",	lastAttackWaveSpawnedInRound );
	ScriptManager:storeValue( "spawnPlaneWaves",				spawnPlaneWaves );
	ScriptManager:storeValue( "planeWaveUnitsAlive", 			planeWaveUnitsAlive );
	ScriptManager:storeValue( "numberOfPlaneWavesSpawned",		numberOfPlaneWavesSpawned );
	ScriptManager:storeValue( "lastPlaneWaveSpawnedInRound",	lastPlaneWaveSpawnedInRound );
	ScriptManager:storeValue( "lastDefenseLineUnitsLeft",		lastDefenseLineUnitsLeft );
	ScriptManager:storeValue( "hqDefenseUnitsLeft", 			hqDefenseUnitsLeft );
	ScriptManager:storeValue( "hqAlive", 						hqAlive );
end

function setTriggers()
	local x, y;
	-- load all needed values after loading
	ScriptManager:addOnGameResumeTrigger( "loadValues", "loadValues", false );
	
	-- trigger for killing an enemy
	ScriptManager:addOnUnitDestroyedTrigger( 1, -1, -1, "onEnemyDestroyed", "onEnemyDestroyed", false );
	
	-- trigger to change enemy state from passive to attack
	ScriptManager:addOnUnitSightedTrigger( 0, -1, -1, false, "onBeingSeen","onBeingSeen", false );
	
	-- trigger for kill artillery secondary mission start
	ScriptManager:addOnUnitHasAttackedTrigger( 1, Units.UNIT_DE_ARTILLERY, firstBarricadeUnits[2][1], "onArtilleryHasAttacked", "onArtilleryHasAttacked", false );

	-- trigger for planes to start the airfield destroy mission if depot doesn't get captured
	ScriptManager:addOnUnitHasAttackedTrigger( 1, Units.UNIT_DE_BOMBER,		-1, "onPlaneHasAttacked", "onPlaneHasAttacked", false );
	ScriptManager:addOnUnitHasAttackedTrigger( 1, Units.UNIT_DE_FIGHTER,	-1, "onPlaneHasAttacked", "onPlaneHasAttacked", false );
	
	-- triggers to start cutscene + secondary main mission to cap enemy supply depot
	-- trigger line: 6/14, 6/15, 7/16, 7/17, 8/18
	x = 5;
	for y = 14, 18 do
		-- for every two y-steps increase x by 1
		if y % 2 == 0 then
			x = x + 1;
		end
		
		ScriptManager:addOnEnterTileTrigger( x, y, 0, -1, -1, true, "nearSupplyDepot", "nearSupplyDepot", false );
	end
	
	-- trigger to not destroy the supply depot
	ScriptManager:addOnUnitHasAttackedTrigger( -1, Units.UNIT_DE_HQ, 4, "onSupplyDepotAttacked", "onSupplyDepotAttacked", false );
	
	-- triggers for the last german defense line ( triggers attack wave spawn + a subtitle display )
	-- trigger line: 1/30, 2/30, 3/30, 4/30, 5/30, 6/30
	y = 30;
	for x = 1, 6 do
		ScriptManager:addOnEnterTileTrigger( x, y, 0, -1, -1, false, "nearLastDefense", "nearLastDefense", false );
	end
	
	-- trigger to start airplane waves if moving on without capturing it
	for x = 2, 9 do
		ScriptManager:addOnEnterTileTrigger( x, 22, 0, -1, -1, false ,"triggerAerialAttacks","triggerAerialAttacks", false );
	end
	
	-- trigger for the headquarter to spawn some additional troops when low hp
	ScriptManager:addOnUnitHasAttackedTrigger( 0, Units.UNIT_DE_SUPER_HQ, -1, "onHqAttacked", "onHqAttacked", false );
	
	-- trigger to show depot warnings / game over
	ScriptManager:addOnUnitHasAttackedTrigger(	1, Units.UNIT_SU_HQ, -1, "ProtectTheSupplyDepotHintSingle",	"ProtectTheSupplyDepotHintSingle",	false );
	ScriptManager:addOnUnitHasAttackedTrigger(	1, Units.UNIT_DE_HQ, -1, "ProtectTheSupplyDepotHintSingleNoDeath",	"ProtectTheSupplyDepotHintSingleNoDeath",	false );
	ScriptManager:addOnUnitDestroyedTrigger(	0, Units.UNIT_SU_HQ, -1, "missionFailedTransition",			"missionFailedTransition",			false );	
end

function loadValues()
		spawnAttackWaves				= ScriptManager:getValue( "spawnAttackWaves" );
		attackWaveUnitsAlive			= ScriptManager:getValue( "attackWaveUnitsAlive" );
		lastAttackWaveSpawnedInRound	= ScriptManager:getValue( "lastAttackWaveSpawnedInRound" );
		
		spawnPlaneWaves 				= ScriptManager:getValue( "spawnPlaneWaves" );
		planeWaveUnitsAlive				= ScriptManager:getValue( "planeWaveUnitsAlive" );
		numberOfPlaneWavesSpawned 		= ScriptManager:getValue( "numberOfPlaneWavesSpawned" ); 
		lastPlaneWaveSpawnedInRound 	= ScriptManager:getValue( "lastPlaneWaveSpawnedInRound" );
		
		lastDefenseLineUnitsLeft		= ScriptManager:getValue( "lastDefenseLineUnitsLeft" );
		hqDefenseUnitsLeft				= ScriptManager:getValue( "hqDefenseUnitsLeft" );
		hqAlive							= ScriptManager:getValue( "hqAlive" );
end


-- END Init
-- BEGIN Intro Cutscene


function startIntroCutscene()
	StartCutscene();
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_3_3_MISSION_PRIMARY_01_FLAVOR_01 );
	ScriptManager:wait( 4 );
	HideDialogue();
	ScriptManager:wait( 1 ); -- anti rush
	
	ScriptManager:moveCameraEx( 3, 43, 0.7, 6, false );
	ScriptManager:wait( 3 );
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_3_3_MISSION_PRIMARY_01_FLAVOR_02 );
	ScriptManager:wait( 2 );
	
	ScriptManager:revealFogOfWar( 3, 43, 1, true );
	ScriptManager:showMarkerAt( 3, 43 );
	ScriptManager:wait( 2 );
	
	HideDialogue();
	ScriptManager:wait( 3 ); -- anti rush
	
	HideAllMarkers();
	ScriptManager:revealFogOfWar( 3, 43, 1, false );
	ScriptManager:moveCameraEx( 4, 33, 0.7, 2.5, false );
	
	ScriptManager:wait( 2 ); -- anti rush
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_3_3_MISSION_PRIMARY_01_FLAVOR_03 );
	ScriptManager:wait( 2 );
	
	ScriptManager:moveCameraEx( 3, 19, 0.7, 2.5, true );
	ScriptManager:wait( 1.5 );
	
	ScriptManager:moveCameraEx( 8, 12, 0.7, 2.5, true );
	ScriptManager:wait( 1.5 );
	HideDialogue();
	
	ScriptManager:moveCameraEx( 5, 4, 0.7, 2, false );
	ScriptManager:wait( 1 );
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_3_3_MISSION_PRIMARY_01_FLAVOR_04 );
	ScriptManager:wait( 3 );
	HideDialogue();
	
	if ScriptManager:isControllerConnected() then
		ScriptManager:setControllerCursorPosition( 6, 4 );
	end
	
	EndCutscene();
	
	startTheMission();
end

function startTheMission()
	ScriptManager:setMissionStatus( missionIds.defeatGermans, MissionStatus.IN_PROGRESS ); -- destroy German base
	
	--TODO: remove this function!
	--testing();
	--ScriptManager:addOnUnitSelectedTrigger( 1, -1, -1, "showMeUnitParameters", "showMeUnitParameters",false)
end

function testing()
	ScriptManager:spawnUnit( 0, Units.GOTT,				9, 42, 0, AiState.WAIT );
	ScriptManager:spawnUnit( 0, Units.GOTT,				8, 42, 0, AiState.WAIT );
	ScriptManager:spawnUnit( 0, Units.UNIT_SU_KATYUSHA,	5, 20, 0, AiState.WAIT );
	ScriptManager:spawnUnit( 1, Units.UNIT_SU_KATYUSHA,	6, 20, 0, AiState.WAIT );
end

function showMeUnitParameters( selectedUnit )
	print( string.format( "[squad] %i", selectedUnit:getSquadId() ) );
end


-- END Intro Cutscene
-- BEGIN onEnemyDestroyed / Attacked


function onEnemyDestroyed( unit )
	if  unit:getTemplateId() == Units.UNIT_DE_ARTILLERY and
		unit:getId()		 == firstBarricadeUnits[2][1] then
		onMountainArtilleryDestroyed();
	
	elseif unit:getTemplateId() == Units.UNIT_DE_HQ then
		onSupplyDepotDestroyed();
		
	elseif unit:getTemplateId() == Units.UNIT_DE_SUPER_HQ then
		hqAlive = 0;
		ScriptManager:storeValue( "hqAlive", hqAlive );
		
		-- if no defenders left alive = victory
		if hqDefenseUnitsLeft == 0 then
			startVictoryCutscene();
		end
		
	elseif unit:getTemplateId() == Units.UNIT_DE_AIRFIELD_UNCAPTUREABLE then
		onAirfieldDestroyed();
	
	elseif  unit:getTemplateId() == Units.UNIT_DE_FIGHTER or
			unit:getTemplateId() == Units.UNIT_DE_BOMBER then
		planeWaveUnitsAlive = planeWaveUnitsAlive - 1;
		ScriptManager:storeValue( "planeWaveUnitsAlive", planeWaveUnitsAlive );
		
	else		
		-- if unit is from the third barricade attack wave, decrease number of alive attackers	
		if unit:getSquadId() == enemySquadIds.thirdBarricadeAttackWave then
			attackWaveUnitsAlive = attackWaveUnitsAlive - 1;
			ScriptManager:storeValue( "attackWaveUnitsAlive", attackWaveUnitsAlive );
		
		elseif unit:getSquadId() == enemySquadIds.thirdBarricadeSquad then
			-- if unit is from the last defense line decrease the nr of defenders alive, if 0, trigger func
			for i = 1, #thirdBarricadeUnits do
				if unit:getId() == thirdBarricadeUnits[i][1] then
					lastDefenseLineUnitsLeft = lastDefenseLineUnitsLeft - 1;
					ScriptManager:storeValue( "lastDefenseLineUnitsLeft", lastDefenseLineUnitsLeft );
					
					if lastDefenseLineUnitsLeft == 0 then
						ScriptManager:runForPlayer( 0, "lastDefenseLineBroken" );
					end
				end
			end

		elseif unit:getSquadId() == enemySquadIds.headquarterDefenseSquad then			
			-- if unit is from the hq defense decrease the nr of defenders alive, if 0, trigger func
			for i = 1, #supplyDepotDefenseUnits do
			
				-- additional trigger for making defense squad aggressive
				if unit:getAiState() == AiState.PASSIVE then
					Unit:changeSquadState( 1, unit:getSquadId(), AiState.ATTACK_AGGRESSIVE );
				end
			
				if unit:getId() == supplyDepotDefenseUnits[i][1] then
					hqDefenseUnitsLeft = hqDefenseUnitsLeft - 1;
					ScriptManager:storeValue( "hqDefenseUnitsLeft", hqDefenseUnitsLeft );
					print( hqDefenseUnitsLeft );
					
					-- if all units dead + headquarter destroyed = victory
					if hqDefenseUnitsLeft == 0 and hqAlive == 0 then
						ScriptManager:runForPlayer( 0, "startVictoryCutscene" );
					end
					
					-- stop looking further into the tables if unit has been found
					break;
				end
			end
		end
	end
end

function onBeingSeen( attacker, defender )
	-- change enemy behaviour when they attacked once
	if  attacker:getTemplateId() ~= Units.UNIT_DE_FIGHTER or
		attacker:getTemplateId() ~= Units.UNIT_DE_BOMBER then
		if attacker:getAiState() == AiState.PASSIVE then
			Unit:changeSquadState( 1, attacker:getSquadId(), AiState.ATTACK_AGGRESSIVE );
		end
	end
end

function onHqAttacked()
	-- additional trigger for making defense squad aggressive
	Unit:changeSquadState( 1, enemySquadIds.headquarterDefenseSquad, AiState.ATTACK_AGGRESSIVE );
	
	RemoveTrigger( "onHqAttacked" );
end


-- END onEnemyDestroyed / Attacked
-- BEGIN Artillery Attack / Destroy ( secondary mission )


function onArtilleryHasAttacked( attacker, defender )
	ScriptManager:runForPlayer( 0, "startKillArtilleryMission" );
end

function startKillArtilleryMission()
	ScriptManager:displaySubtitle( Text.T_MAP_SU_3_3_MISSION_SECONDARY_01_FLAVOR );
	ScriptManager:wait( 4 );
	HideDialogue();

	ScriptManager:setMissionStatus( missionIds.killArtillery, MissionStatus.IN_PROGRESS );
	RemoveTrigger( "onArtilleryHasAttacked" );
end

function onMountainArtilleryDestroyed()
	ScriptManager:runForPlayer( 0, "accomplishKillArtilleryMission" );
end

function accomplishKillArtilleryMission()
	if ScriptManager:getMissionStatus ( missionIds.killArtillery ) ~= MissionStatus.ACCOMPLISHED then
		ScriptManager:setMissionStatus( missionIds.killArtillery, MissionStatus.ACCOMPLISHED );
	end
end


-- END Artillery Attack / Destroy ( secondary mission )
-- BEGIN Cap Supply Depot ( 2. main mission ) / airfield destroy mission


function nearSupplyDepot(tileX, tileY )
	StartCutscene();
	
	ScriptManager:moveCameraEx( 5, 22, 0.7, 2, true );
	ScriptManager:revealFogOfWar( 5, 22, 1, true );
	ScriptManager:showMarkerAt( 5, 22 );
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_3_3_MISSION_PRIMARY_02_FLAVOR_01 );
	ScriptManager:wait( 4 );
	HideDialogue();
	
	HideAllMarkers();
	ScriptManager:revealFogOfWar( 5, 22, 1, false );
	ScriptManager:moveCameraEx( tileX, tileY, 0.7, 2, true );
	
	EndCutscene();
	
	if ScriptManager:getMissionStatus( missionIds.captureSupplyDepot ) == MissionStatus.NOT_STARTED then
		ScriptManager:setMissionStatus( missionIds.captureSupplyDepot, MissionStatus.IN_PROGRESS );
	end
	
	RemoveTrigger( "nearSupplyDepot" );
end

function onSupplyDepotCaptured()
	DisablePlayerInteraction();
	ScriptManager:setMissionStatus( missionIds.captureSupplyDepot,	MissionStatus.ACCOMPLISHED );
	ScriptManager:wait( 2 );
	EnablePlayerInteraction();
	
	StartCutscene();
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_3_3_MISSION_PRIMARY_02_FLAVOR_02 );
	ScriptManager:wait( 4 );
	HideDialogue();
	ScriptManager:wait( 1 ); -- anti rush
	
	-- spawn reinforcemen units for the player around the just capped depot
	ScriptManager:spawnUnits( 0, Units.UNIT_SU_AA, 5, 22, 0, AiState.WAIT, 1 );
	ScriptManager:spawnUnits( 0, Units.UNIT_SU_RIFLE, 5, 22, 0, AiState.WAIT, 2 );
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_3_3_MISSION_PRIMARY_02_FLAVOR_03 );
	ScriptManager:wait( 4 );
	HideDialogue();
	ScriptManager:wait( 1 ); -- anti rush
									     
	ScriptManager:displaySubtitle( Text.T_MAP_SU_3_3_MISSION_PRIMARY_02_FLAVOR_04 ); 
	ScriptManager:moveCameraEx( 9, 34, 1, 4, true );
	ScriptManager:wait( 1 );
	HideDialogue();
	ScriptManager:wait( 1 ); -- anti rush
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_3_3_MISSION_PRIMARY_02_FLAVOR_05 ); 
	ScriptManager:moveCameraEx( 5, 34, 0.7, 2, true );
	ScriptManager:wait( 2 );
	HideDialogue();
	ScriptManager:wait( 1 ); -- anti rush
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_3_3_MISSION_PRIMARY_02_FLAVOR_06 );
	ScriptManager:wait( 4 );
	HideDialogue();
	ScriptManager:wait( 1 ); -- anti rush
	
	ScriptManager:moveCameraEx( 5, 22, 0.7, 3, true );
	
	EndCutscene();
	
	ScriptManager:gainGold( 0, 2500 ); 
	
	ScriptManager:setMissionStatus( missionIds.breakThrough, MissionStatus.IN_PROGRESS );
	if ScriptManager:getMissionStatus( missionIds.destroyAirfield ) == MissionStatus.NOT_STARTED then
		ScriptManager:setMissionStatus( missionIds.destroyAirfield,	MissionStatus.IN_PROGRESS );
	end
	
	RemoveTrigger( "onSupplyDepotAttacked" );
	RemoveTrigger( "onSupplyDepotCaptured" );
	
	-- check needed since triggerAerialAttacks() was added ( => you can trigger waves without capping depot )
	if spawnPlaneWaves ~= 1 then
		-- start enemy airstrikes
		spawnPlaneWaves = 1;
		-- next wave should spawn immediatelly
		lastPlaneWaveSpawnedInRound	= ScriptManager:getCurrentRound() - planeWaveRespawnInterval;
		numberOfPlaneWavesSpawned 	= numberOfPlaneWavesSpawned + 1;
		ScriptManager:storeValue( "spawnPlaneWaves",				spawnPlaneWaves );
		ScriptManager:storeValue( "lastPlaneWaveSpawnedInRound",	lastPlaneWaveSpawnedInRound );
		ScriptManager:storeValue( "numberOfPlaneWavesSpawned",		numberOfPlaneWavesSpawned );
	end
end

function triggerAerialAttacks()
	-- start enemy airstrikes
	spawnPlaneWaves = 1;
	-- next wave should spawn immediatelly
	lastPlaneWaveSpawnedInRound	= ScriptManager:getCurrentRound() - planeWaveRespawnInterval;
	numberOfPlaneWavesSpawned 	= numberOfPlaneWavesSpawned + 1;
	ScriptManager:storeValue( "spawnPlaneWaves",				spawnPlaneWaves );
	ScriptManager:storeValue( "lastPlaneWaveSpawnedInRound",	lastPlaneWaveSpawnedInRound );
	ScriptManager:storeValue( "numberOfPlaneWavesSpawned",		numberOfPlaneWavesSpawned );
	
	RemoveTrigger( "triggerAerialAttacks" );
end

function onPlaneHasAttacked()
	if ScriptManager:getMissionStatus( missionIds.destroyAirfield ) == MissionStatus.NOT_STARTED then
		ScriptManager:setMissionStatus( missionIds.destroyAirfield,	MissionStatus.IN_PROGRESS );
	end
	
	RemoveTrigger( "onPlaneHasAttacked" );
end

function onSupplyDepotAttacked( attacker, defender )
	-- if the player attacks the depot ( instead of capturing it ) he gets a warning to not destroy it
	if defender ~= nil then
		ScriptManager:showMarkerAt( defender:getX(), defender:getY() );
		ScriptManager:displaySubtitle( Text.T_MAP_SU_3_3_MISSION_PRIMARY_02_FLAVOR_07 );
		ScriptManager:wait( 4 );
		ScriptManager:removeAllMarkers();
		HideDialogue();
	end
end

function onSupplyDepotDestroyed()
	missionFailed(); -- mission only fails if the player destroys the depot while its still in enemy control
end


-- END Cap Supply Depot ( 2. main mission ) / airfield destroy mission
-- BEGIN Wave Spawns


function onEnemyTurnStart()
	-- top artillery from moving around the mountain
	if ScriptManager:getMissionStatus( missionIds.killArtillery ) ~= MissionStatus.ACCOMPLISHED then
		local unit = Unit:getUnitById( firstBarricadeUnits[2][1] );	
		unit:setActionPoints( 0 );
	end

	-- spawn logic for plane waves
	if spawnPlaneWaves == 1 then -- should waves spawn? 1 == yes, 0 == no
		if planeWaveUnitsAlive < maxPlaneUnits then -- cap max number of wave units
		
			-- check if a wave spawns this round
			if lastPlaneWaveSpawnedInRound + planeWaveRespawnInterval <= ScriptManager:getCurrentRound() then
				numberOfPlaneWavesSpawned = numberOfPlaneWavesSpawned + 1;
				ScriptManager:storeValue( "numberOfPlaneWavesSpawned", numberOfPlaneWavesSpawned );
			
				-- check if current spawn is a bomber spawn
				if numberOfPlaneWavesSpawned % bomberAppearance == 0 then -- spawn Bomber
					spawnWaveUnit( planeAttackWaveUnits[3] );
					planeWaveUnitsAlive = planeWaveUnitsAlive + 1;
					
				else -- spawn two fighters
					for i = 1, #planeAttackWaveUnits - 1 do
						if planeWaveUnitsAlive < maxPlaneUnits then
							spawnWaveUnit( planeAttackWaveUnits[i] )
							planeWaveUnitsAlive = planeWaveUnitsAlive + 1;
							
						else
							break; -- stop the loop if max number of alive units is reached while running the loop
						end
					end
				end
				
				lastPlaneWaveSpawnedInRound = ScriptManager:getCurrentRound(); -- reset round count
				ScriptManager:storeValue( "lastPlaneWaveSpawnedInRound",	lastPlaneWaveSpawnedInRound );
				ScriptManager:storeValue( "planeWaveUnitsAlive",			planeWaveUnitsAlive );
			end
				
		-- if max units are alive, change the last spawned round to half of the interval
		-- so the player has some more time to kill remaining units before getting more enemies against him
		-- because I divide by 2 here, the number has to be even ( as mentioned in the beginning )
		else
			lastPlaneWaveSpawnedInRound = ScriptManager:getCurrentRound() - ( planeWaveRespawnInterval / 2 );
		end
	end
	
	-- spawn logic for waves @ last defense line ( third barricade )
	if spawnAttackWaves == 1 then -- should waves spawn? 1 == yes, 0 == no
		if attackWaveUnitsAlive < maxAttackUnits then -- cap max number of wave units
		
			-- check if a wave spawns this round
			if lastAttackWaveSpawnedInRound + attackWaveRespawnInterval <= ScriptManager:getCurrentRound() then
				
				-- spawn the wave
				for i = 1, #thirdBarricadeAttackWaveUnits do
					if attackWaveUnitsAlive < maxAttackUnits then
						spawnWaveUnit( thirdBarricadeAttackWaveUnits[i] );
						attackWaveUnitsAlive = attackWaveUnitsAlive + 1;
						
					else
						break; -- stop the loop if max number of alive units is reached while running the loop
					end
				end
				
				lastAttackWaveSpawnedInRound = ScriptManager:getCurrentRound(); -- reset round count
				ScriptManager:storeValue( "lastAttackWaveSpawnedInRound",	lastAttackWaveSpawnedInRound );
				ScriptManager:storeValue( "attackWaveUnitsAlive",			attackWaveUnitsAlive );
			end
		
		-- if max units are alive, change the last spawned round to half of the interval
		-- so the player has some more time to kill the remaining units before getting more enemies against him
		-- because I divide by 2 here, the number has to be even ( as mentioned in the beginning )
		else
			lastAttackWaveSpawnedInRound = ScriptManager:getCurrentRound() - ( attackWaveRespawnInterval / 2 );
		end	
	end
end

function spawnWaveUnit( spawnData )
	ScriptManager:spawnUnit( spawnData[1], spawnData[2], spawnData[3], spawnData[4], spawnData[5], spawnData[6] );
end


-- END Wave Spawns
-- BEGIN Airfield Cap ( 2. secondary mission )


function onAirfieldDestroyed()
	ScriptManager:setMissionStatus( missionIds.destroyAirfield, MissionStatus.ACCOMPLISHED );
	
	DisablePlayerInteraction()
	ScriptManager:displaySubtitle( Text.T_MAP_SU_3_3_MISSION_SECONDARY_02_FLAVOR );
	ScriptManager:wait( 3 );
	HideDialogue();
	EnablePlayerInteraction();
	
	-- stop enemy airstrikes
	spawnPlaneWaves = 0;
	ScriptManager:storeValue( "spawnPlaneWaves", spawnPlaneWaves );
end


-- END Arifield Cap ( 2. secondary mission )
-- BEGIN Last German Defense Line


function nearLastDefense()
	-- change enemy behaviour
	Unit:changeSquadState( 1, enemySquadIds.thirdBarricadeSquad, AiState.ATTACK_AGGRESSIVE );

	-- let additional troops spawn behind last german defense line to make it harder
	spawnAttackWaves = 1;
	-- needed to spawn wave immediately
	lastAttackWaveSpawnedInRound = ScriptManager:getCurrentRound() - attackWaveRespawnInterval;
	ScriptManager:storeValue( "spawnAttackWaves", spawnAttackWaves );
	
	RemoveTrigger( "nearLastDefense" );
end

function lastDefenseLineBroken()
	ScriptManager:setMissionStatus( missionIds.breakThrough, MissionStatus.ACCOMPLISHED );
	
	DisablePlayerInteraction();
	ScriptManager:displaySubtitle( Text.T_MAP_SU_3_3_MISSION_PRIMARY_03_FLAVOR );
	ScriptManager:wait( 4 );
	HideDialogue();
	EnablePlayerInteraction();
	
	-- stop wave spawn
	spawnAttackWaves = 0;
	ScriptManager:storeValue( "spawnAttackWaves", spawnAttackWaves );
end


-- END Last German Defense Line
-- BEGIN Victory


function startVictoryCutscene()
	StartCutscene();
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_3_3_MISSION_PRIMARY_01_FLAVOR_05 );
	ScriptManager:wait( 5 );
	HideDialogue();
	ScriptManager:wait( 1 ); -- anti-rush
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_3_3_MISSION_PRIMARY_01_FLAVOR_06 );
	ScriptManager:wait( 5 );
	HideDialogue();
	
	EndCutscene();
	DisablePlayerInteraction();
	ScriptManager:wait( 1 );
	
	victoryEndGame();
end

function victoryEndGame()
	-- main missions
	if  ScriptManager:getMissionStatus( missionIds.breakThrough ) ~= MissionStatus.ACCOMPLISHED then
		ScriptManager:setMissionXp( missionIds.breakThrough, 50 ); -- reduce xp reward cause the player didn't do it properly
		ScriptManager:setMissionStatus( missionIds.breakThrough, MissionStatus.ACCOMPLISHED );
	end
	ScriptManager:setMissionStatus( missionIds.defeatGermans, MissionStatus.ACCOMPLISHED );
	
	ScriptManager:displaySubtitle( Text.T_MISSION_VICTORY );
	ScriptManager:wait( 4 );
	HideDialogue();
	endGame();
end


-- END Victory
-- BEGIN Defeated


function missionFailedTransition()
	ScriptManager:runForPlayer( 0, "missionFailed" );
end

function missionFailed()
	DisablePlayerInteraction();
	HideDialogue(); -- remove possible texts on the screens
	
	ScriptManager:displaySubtitle( Text.T_MISSION_FAILED );
	ScriptManager:wait( 4 );
	HideDialogue();
	endGame();
end


-- END Defeated


function endGame()
	ScriptManager:endGame()
end