require "lua/Constants"
require "lua/Functions"
require "lua/Functions_2"
require "lua/Text"
require "lua/Units"
require "math"


-- mission stuff
missionIds = {
	searchForSurvivors	= 1,
	captureDepots 		= 2,
	bringSurvivorsHome	= 3,
	annihilateAmbushes	= 4
};

-- missionId, primary( boolean ), textkey, xpReward
missions = {
	{ missionIds.searchForSurvivors, true,	Text.T_MAP_SU_4_1_MISSION_PRIMARY_01,		 75 },
	{ missionIds.captureDepots,		 true,	Text.T_MAP_SU_4_1_MISSION_PRIMARY_02,		175 },
	{ missionIds.annihilateAmbushes, false, Text.T_MAP_SU_4_1_MISSION_SECONDARY_01,		125 },
	{ missionIds.bringSurvivorsHome, false, Text.T_MAP_SU_4_1_MISSION_SECONDARY_02,		 75 }	
};

survivorIds = {}; -- when soldiers rescued, add Id here
capturedDepots = 0;
numberOfAmbushes = 4;
maxNumberOfSurvivors = 3;
firstSurvivorCutscenePlayed = 0;

enemySquadIds = {
	northAmbush			= 1,
	westDepotDefense	= 2,
	soutwestAmbush		= 3,
	centreDepotDefense	= 4,
	southAmbush			= 5,
	southDepotDefense	= 6,
	eastAmbush			= 7,
	yoloGuys			= 8
};

-- all enemy Ids in their squads
northAmbushIds			= {  4,  5,  6,  7,  8,  9 };
westDepotDefenseIds		= { 10, 11, 12, 14, 15 }; -- depot NOT included
southwestAmbushIds		= { 13, 16, 17, 18 ,19 };
centreDepotDefenseIds	= { 20, 21, 22, 23, 24 }; -- depot NOT included
southAmbushIds			= { 25, 26, 27, 28, 29 };
southDepotDefenseIds	= { 30, 31, 32, 33, 34, 35, 36, 37 }; -- depot NOT included
eastAmbushIds			= { 38, 39, 40, 41, 42, 43 };
yoloGuysIds				= { 44, 45, 46 };

-- number of units left alive in the squads
northAmbushUnitsLeft		= #northAmbushIds;
southwestAmbushUnitsLeft	= #southwestAmbushIds;
southAmbushUnitsLeft		= #southAmbushIds;
eastAmbushUnitsLeft			= #eastAmbushIds;


-- BEGIN Init


function onGameStart()
	CreateMissions( missions );
	setTriggers();
	storeValues();
	camouflageAmbushSquads();
	
	ScriptManager:gainGold( 0, 1500 );
	
	-- start intro cutscene
	ScriptManager:sleep( 0, "startIntroCutscene" ); -- without sleep player turn popup shows
end

function camouflageAmbushSquads()
	CamouflageSquad( northAmbushIds );
	CamouflageSquad( southwestAmbushIds );
	CamouflageSquad( southAmbushIds );
	CamouflageSquad( eastAmbushIds );
	CamouflageSquad( yoloGuysIds );
	
	-- dont want to show the defenders in the intro cutscene when i am marking the depots
	CamouflageSquad( westDepotDefenseIds );
	CamouflageSquad( centreDepotDefenseIds );
	CamouflageSquad( southDepotDefenseIds );
end

function storeValues()
	ScriptManager:storeValue( "northAmbushUnitsLeft",		northAmbushUnitsLeft );
	ScriptManager:storeValue( "southwestAmbushUnitsLeft",	southwestAmbushUnitsLeft );
	ScriptManager:storeValue( "southAmbushUnitsLeft",		southAmbushUnitsLeft );
	ScriptManager:storeValue( "eastAmbushUnitsLeft",		eastAmbushUnitsLeft );
	
	ScriptManager:storeValue( "capturedDepots",				capturedDepots );
	ScriptManager:storeValue( "firstSurvivorCutscenePlayed",firstSurvivorCutscenePlayed );
end

function setTriggers()
	-- load all needed values after loading
	ScriptManager:addOnGameResumeTrigger( "loadValues", "loadValues", false );	
	
	-- capturing the three depots
	ScriptManager:addOnUnitCapturedTrigger( 0, Units.UNIT_SU_HQ, -1, "onDepotCaptured", "onDepotCaptured", false );
	
	-- losing an infantry unit ( in order to check for survivor sec objective )
	ScriptManager:addOnUnitDestroyedTrigger( 0, Units.UNIT_SU_PRISONER_RIFLE,	-1, "onPlayerInfantryKilled", "onPlayerInfantryKilled", false );
	ScriptManager:addOnUnitDestroyedTrigger( 0, Units.UNIT_SU_PRISONER_MORTAR,	-1, "onPlayerInfantryKilled", "onPlayerInfantryKilled", false );
	
	-- killing an enemy unit
	ScriptManager:addOnUnitDestroyedTrigger( 1, -1, -1, "onEnemyKilled", "onEnemyKilled", false );
	
	-- player seen by enemy
	ScriptManager:addOnUnitSightedTrigger( 0, -1, -1, false, "onPlayerSeen", "onPlayerSeen", false );
	
	-- show secondary intro cutscene when encoutering the infamous yolo guys
	ScriptManager:addOnUnitSightedTrigger( 1, -1, -1, true, "onYoloGuysSeen",	"onYoloGuys", false );
	ScriptManager:addOnUnitHasAttackedTrigger( 1, -1, -1, "onYoloGuysAttacked",	"onYoloGuys", false );
	
	-- show warning when attacking enemy depots // own depot getting attacked
	ScriptManager:addOnUnitHasAttackedTrigger( 1, Units.UNIT_SU_HQ, -1, "ProtectTheSupplyDepotHintSingle", "ProtectTheSupplyDepotHintSingle", false );
	ScriptManager:addOnUnitHasAttackedTrigger( 0, Units.UNIT_SU_HQ, -1, "ProtectTheSupplyDepotHintSingle", "ProtectTheSupplyDepotHintSingle", false );
	
	-- mission failed if depots go go down
	ScriptManager:addOnUnitDestroyedTrigger( -1, Units.UNIT_SU_HQ, -1, "missionFailedTransition", "missionFailedTransition", false );
end

function loadValues()
	firstSurvivorCutscenePlayed = ScriptManager:getValue( "firstSurvivorCutscenePlayed" );
	
	capturedDepots = ScriptManager:getValue( "capturedDepots" );
	
	northAmbushUnitsLeft		= ScriptManager:getValue( "northAmbushUnitsLeft" );
	southwestAmbushUnitsLeft	= ScriptManager:getValue( "southwestAmbushUnitsLeft" );
	southAmbushUnitsLeft		= ScriptManager:getValue( "southAmbushUnitsLeft" );
	eastAmbushUnitsLeft			= ScriptManager:getValue( "eastAmbushUnitsLeft" );
	
	-- load the survivors back into the table if it is empty ( = game was closed and is now continued )
	if #survivorIds == 0 then		
		for i = 1, maxNumberOfSurvivors do
			local value = ScriptManager:getValue( string.format( "survivorId %i", i ) );
			
			if value ~= -1 then -- if value exists
				table.insert( survivorIds, value );
			else
				break; -- no more survivors
			end
		end
	end
	for i = 1, #survivorIds do
		print( string.format( "[survivorId] %i", survivorIds[i] ) );
	end
end


-- END Init
-- BEGIN Intro Cutscene


function startIntroCutscene()
	StartCutscene();
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_4_1_MISSION_INTRO_FLAVOR_01 );
	ScriptManager:wait( 5 );
	HideDialogue();
	ScriptManager:wait( 0.5 ) -- anti-rush
	
	ScriptManager:moveCameraEx( 6, 13, 0.6, 3, false );
	ScriptManager:wait( 1 );
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_4_1_MISSION_INTRO_FLAVOR_02 );
	ScriptManager:wait( 2 );
	
	-- mark depot north
	ScriptManager:revealFogOfWar(	5, 16, 1, true );
	ScriptManager:showMarkerAt(		5, 16 );
	
	-- mark depot centre
	ScriptManager:revealFogOfWar(	8, 10, 1, true );
	ScriptManager:showMarkerAt(		8, 10 );
	ScriptManager:wait( 3 );
	
	HideAllMarkers();
	ScriptManager:revealFogOfWar(	8, 10, 1, false ); -- centre
	ScriptManager:revealFogOfWar(	5, 16, 1, false ); -- north
	ScriptManager:wait( 1 );
	HideDialogue();
	ScriptManager:wait( 0.5 ) -- anti-rush
	
	ScriptManager:moveCameraEx( 15, 2, 0.6, 3, false );
	ScriptManager:wait( 1 );
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_4_1_MISSION_INTRO_FLAVOR_03 );
	ScriptManager:wait( 2 );
	
	-- mark depot south
	ScriptManager:revealFogOfWar(	15, 2, 1, true );
	ScriptManager:showMarkerAt(		15, 2 );
	ScriptManager:wait( 3 );
	
	HideAllMarkers();
	ScriptManager:revealFogOfWar(	15, 2, 1, false );
	ScriptManager:wait( 1 );	
	HideDialogue();
	ScriptManager:wait( 0.5 ) -- anti-rush
	
	ScriptManager:moveCameraEx( 15, 23, 0.7, 3, false );
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_4_1_MISSION_INTRO_FLAVOR_04 );
	ScriptManager:wait( 5 );
	HideDialogue();
	
	if ScriptManager:isControllerConnected() then
		ScriptManager:setControllerCursorPosition( 14, 23 );
	end
	
	EndCutscene();
	
	startTheMission();
end

function startTheMission()
	ScriptManager:setMissionStatus( missionIds.searchForSurvivors, MissionStatus.IN_PROGRESS );
	
	--TODO: remove this function!
	--testing();
	--removeSquads();
	--ScriptManager:addOnUnitSelectedTrigger( 1, -1, -1, "showParameters", "showParameters",false)
end

function testing()
	ScriptManager:spawnUnit( 0, Units.GOTT, 17, 25, 0, AiState.WAIT );
	ScriptManager:spawnUnit( 0, Units.GOTT, 16, 25, 0, AiState.WAIT );
	ScriptManager:spawnUnit( 0, Units.GOTT, 15, 25, 0, AiState.WAIT );
end

function removeSquads()
	for i = 1, #northAmbushIds do
		ScriptManager:removeUnitEx( northAmbushIds[i], false );
	end
	
	for i = 1, #westDepotDefenseIds do
		ScriptManager:removeUnitEx( westDepotDefenseIds[i], false );
	end
	
	for i = 1, #southwestAmbushIds do
		ScriptManager:removeUnitEx( southwestAmbushIds[i], false );
	end
	
	for i = 1, #centreDepotDefenseIds do
		ScriptManager:removeUnitEx( centreDepotDefenseIds[i], false );
	end
	
	for i = 1, #southAmbushIds do
		ScriptManager:removeUnitEx( southAmbushIds[i], false );
	end
	
	for i = 1, #southDepotDefenseIds do
		ScriptManager:removeUnitEx( southDepotDefenseIds[i], false );
	end
	
	for i = 1, #eastAmbushIds do
		ScriptManager:removeUnitEx( eastAmbushIds[i], false );
	end
	
	for i = 1, #yoloGuysIds do
		ScriptManager:removeUnitEx( yoloGuysIds[i], false );
	end
end

function showParameters( selectedUnit )
	print( string.format( "[SquadId] %s", selectedUnit:getSquadId() ) );
end


-- END Intro Cutscene
-- BEGIN onPlayerX


function onPlayerInfantryKilled( killedUnit )
	-- check if the killed infantry was one of the survivors
	if #survivorIds ~= 0 then -- if table not empty
		local killedUnitId = killedUnit:getId();
		
		for i = 1, #survivorIds do
			if killedUnitId == survivorIds[i] then
				ScriptManager:setMissionStatus( missionIds.bringSurvivorsHome, MissionStatus.FAILED );
				RemoveTrigger( "onPlayerInfantryKilled" );
			end 
		end
	end
end

function onYoloGuysSeen( seeingUnit, seenUnit )
	checkForSecondaryCutscene( seenUnit );
end

function onYoloGuysAttacked( activeUnit, passiveUnit )
	checkForSecondaryCutscene( activeUnit );
end

function checkForSecondaryCutscene( unit )
	if unit:getSquadId() == enemySquadIds.yoloGuys then
		ScriptManager:runForPlayer( 0, "showSecondaryIntroCutscene" );
		RemoveTrigger( "onYoloGuys")
	end
end

function showSecondaryIntroCutscene()
	StartCutscene()
	
	ScriptManager:moveCamera( 13, 20, 0.7, 1.5, "" );
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_4_1_MISSION_SECONDARY_01_FLAVOR_01 );
	ScriptManager:wait( 3 );
	HideDialogue();
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_4_1_MISSION_SECONDARY_01_FLAVOR_02 );
	ScriptManager:wait( 5 );
	HideDialogue();
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_4_1_MISSION_SECONDARY_01_FLAVOR_03 );
	ScriptManager:wait( 5 );
	HideDialogue();
	
	EndCutscene();
	
	ScriptManager:setMissionStatus( missionIds.annihilateAmbushes, MissionStatus.IN_PROGRESS );
end


-- END onPlayerX
-- BEGIN onEnemyX


function onPlayerSeen( seeingUnit, seenUnit )
	-- check if enemy is passive ( = ambush unit ) and change its squad state to attacking if so
	if seeingUnit:getAiState() == AiState.PASSIVE then
		local squadId = seeingUnit:getSquadId();
		
		Unit:changeSquadState( 1,squadId, AiState.ATTACK_AGGRESSIVE );
	end
end

function onEnemyKilled( killedUnit )
	-- countdown function for all ambush squads ( if a ambush squad member is killed, reduce number by 1 );
	local killedUnitSquadId = killedUnit:getSquadId();
	
	if killedUnitSquadId == enemySquadIds.northAmbush then
		northAmbushUnitsLeft = reduceUnitsLeftInSquad( northAmbushUnitsLeft );
		ScriptManager:storeValue( "northAmbushUnitsLeft", northAmbushUnitsLeft );
		
	elseif killedUnitSquadId == enemySquadIds.soutwestAmbush then
		southwestAmbushUnitsLeft = reduceUnitsLeftInSquad( southwestAmbushUnitsLeft );
		ScriptManager:storeValue( "southwestAmbushUnitsLeft", southwestAmbushUnitsLeft );
		
	elseif killedUnitSquadId == enemySquadIds.southAmbush then
		southAmbushUnitsLeft = reduceUnitsLeftInSquad( southAmbushUnitsLeft );
		ScriptManager:storeValue( "southAmbushUnitsLeft", southAmbushUnitsLeft );
	
	elseif killedUnitSquadId == enemySquadIds.eastAmbush then
		eastAmbushUnitsLeft = reduceUnitsLeftInSquad( eastAmbushUnitsLeft );
		ScriptManager:storeValue( "eastAmbushUnitsLeft", eastAmbushUnitsLeft );
	end
end

function reduceUnitsLeftInSquad( unitsLeftAlive, bAmbushUnit )
	unitsLeftAlive = unitsLeftAlive - 1;
	print( unitsLeftAlive );
	
	-- if squad is annihilate, update the mission
	if unitsLeftAlive <= 0 then
		print( "squadDown" );
		ScriptManager:setMissionProgress(
			missionIds.annihilateAmbushes, ScriptManager:getMissionProgress( missionIds.annihilateAmbushes ) + 1 );
			
		ScriptManager:runForPlayer( 0, "showProgressInPlayerTurn" );
	end
	
	return unitsLeftAlive;
end

function showProgressInPlayerTurn()		
	-- if there are still ambushes left, show progress, otherwise accomplish mission
	if ScriptManager:getMissionProgress( missionIds.annihilateAmbushes ) < numberOfAmbushes then
		ShowMissionProgressPopup( 
			ScriptManager:getMissionProgress( missionIds.annihilateAmbushes ), Text.T_MAP_SU_4_1_MISSION_SECONDARY_01_COUNTER );
			
	else
		ScriptManager:setMissionStatus( missionIds.annihilateAmbushes, MissionStatus.ACCOMPLISHED );
	end
end


-- END Enemy killed
-- BEGIN Depots


function onDepotCaptured( activeUnit, capturedDepot )
	capturedDepots = capturedDepots + 1;
	print( string.format( "captured depots: %i", capturedDepots ) );
	ScriptManager:storeValue( "capturedDepots", capturedDepots );
	
	-- player captured all depots
	if capturedDepots == 3 then
		DisablePlayerInteraction();
		ScriptManager:setMissionStatus( missionIds.captureDepots, MissionStatus.ACCOMPLISHED );
		ScriptManager:wait( 2 );
		
		startVictoryCutscene();
		
	else	
		if capturedDepot:getId() ~= 1 then -- if southern or western depot, spawn survivors and show their cutscene
			spawnSurvivors( capturedDepot );
			
			-- if survivors spawned once
			if firstSurvivorCutscenePlayed == 0 then
				DisablePlayerInteraction();
				
				if  ScriptManager:getMissionStatus( missionIds.searchForSurvivors ) ~= MissionStatus.ACCOMPLISHED then
					ScriptManager:setMissionStatus( missionIds.searchForSurvivors, MissionStatus.ACCOMPLISHED )
				end
				ScriptManager:wait( 1 );
				
				showFirstSurvivorCutscene();
				
				ScriptManager:wait( 1 );
				
				if ScriptManager:getMissionStatus( missionIds.captureDepots ) == MissionStatus.NOT_STARTED then
					ScriptManager:setMissionStatus( missionIds.captureDepots, MissionStatus.IN_PROGRESS );
				end
				
				ScriptManager:setMissionStatus( missionIds.bringSurvivorsHome, MissionStatus.IN_PROGRESS );
				EnablePlayerInteraction();
				
				firstSurvivorCutscenePlayed = 1;
				ScriptManager:storeValue( "firstSurvivorCutscenePlayed", firstSurvivorCutscenePlayed );
			else
				showSecondSurvivorCutscene();
			end
		else -- if centre depot, show another cutscene
			-- lower stats of the ruined depot
			capturedDepot:setSupplyLoad( capturedDepot:getSupplyLoad() * 0.3 );
			capturedDepot:setHealth( capturedDepot:getHealth() * 0.6 );
			
			DisablePlayerInteraction();
			if  ScriptManager:getMissionStatus( missionIds.searchForSurvivors ) ~= MissionStatus.ACCOMPLISHED then
				ScriptManager:setMissionStatus( missionIds.searchForSurvivors, MissionStatus.ACCOMPLISHED )
			end
			ScriptManager:wait( 1 );
			
			StartCutscene();
			
			ScriptManager:displaySubtitle( Text.T_MAP_SU_4_1_MISSION_PRIMARY_01_FLAVOR_01 );
			ScriptManager:wait( 5 );
			HideDialogue();
			
			-- if first main mission still in progress, this is the first outpost captured by the player
			if ScriptManager:getMissionStatus( missionIds.captureDepots ) == MissionStatus.IN_PROGRESS then
				ScriptManager:displaySubtitle( Text.T_MAP_SU_4_1_MISSION_PRIMARY_02_FLAVOR_01 );
				ScriptManager:wait( 5 );
				HideDialogue();
			
				EndCutscene();				
			else
				ScriptManager:displaySubtitle( Text.T_MAP_SU_4_1_MISSION_PRIMARY_01_FLAVOR_02 );
				ScriptManager:wait( 5 );
				HideDialogue();
			
				EndCutscene();
				
				ScriptManager:setMissionStatus( missionIds.captureDepots, MissionStatus.IN_PROGRESS );
				ScriptManager:wait( 1 );
			end
			
			
			EnablePlayerInteraction();
		end
	end
end

function spawnSurvivors( depot )
	depotId = depot:getId();
	
	if depotId == 0 then -- westernDepot
		for i = 1, 2 do
			local unit = ScriptManager:spawnUnit( 0, Units.UNIT_SU_PRISONER_RIFLE, depot:getX(), depot:getY(), 1, AiState.WAIT ); -- spawn
			table.insert( survivorIds, unit:getId() ); -- add the survivor ID into the table to check if for its death
			print( string.format( "survivor added: %i", unit:getId() ) );
		end
	else -- southernDepot
		local unit = ScriptManager:spawnUnit( 0, Units.UNIT_SU_PRISONER_RIFLE, depot:getX(), depot:getY(), 1, AiState.WAIT ); -- spawn
		table.insert( survivorIds, unit:getId() ); -- add the survivor ID into the table to check if for its death
		print( string.format( "survivor added: %i", unit:getId() ) );
	end
	
	-- save the ids
	for i = 1, #survivorIds do
		ScriptManager:storeValue( string.format( "survivorId %i", i ), survivorIds[i] );
		print( string.format( "survivor in table: %i", survivorIds[i] ) );
	end
end

function showFirstSurvivorCutscene()
	StartCutscene();
		
	ScriptManager:displaySubtitle( Text.T_MAP_SU_4_1_MISSION_PRIMARY_01_FLAVOR_03 );
	ScriptManager:wait( 4 );
	HideDialogue();
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_4_1_MISSION_PRIMARY_01_FLAVOR_04 );
	ScriptManager:wait( 5 );
	HideDialogue();
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_4_1_MISSION_PRIMARY_01_FLAVOR_05 );
	ScriptManager:wait( 5 );
	HideDialogue();
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_4_1_MISSION_PRIMARY_01_FLAVOR_06 );
	ScriptManager:wait( 5 );
	HideDialogue();
		
	if ScriptManager:getMissionStatus( missionIds.captureDepots ) ~= MissionStatus.IN_PROGRESS then
		ScriptManager:displaySubtitle( Text.T_MAP_SU_4_1_MISSION_PRIMARY_01_FLAVOR_07 );
		ScriptManager:wait( 5 );
		HideDialogue();
	end
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_4_1_MISSION_PRIMARY_01_FLAVOR_08 );
	ScriptManager:wait( 5 );
	HideDialogue();
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_4_1_MISSION_PRIMARY_01_FLAVOR_09 );
	ScriptManager:wait( 8 );
	HideDialogue();
		
	EndCutscene();
end

function showSecondSurvivorCutscene()
	StartCutscene();
		
		ScriptManager:displaySubtitle( Text.T_MAP_SU_4_1_MISSION_PRIMARY_02_FLAVOR_02 );
		ScriptManager:wait( 4 );
		HideDialogue();
		
	EndCutscene();
end


-- END Depots
-- BEGIN Victory


function startVictoryCutscene()
	StartCutscene();
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_4_1_MISSION_VICTORY_FLAVOR_01 );
	ScriptManager:wait( 5 );
	HideDialogue();
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_4_1_MISSION_VICTORY_FLAVOR_02 );
	ScriptManager:wait( 5 );
	HideDialogue();
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_4_1_MISSION_VICTORY_FLAVOR_03 );
	ScriptManager:wait( 5 );
	HideDialogue();
	
	EndCutscene();
	
	victoryEndGame();
end

function victoryEndGame()
	DisablePlayerInteraction();
	HideDialogue(); -- remove possible texts on the screens
	
	-- set sec mission
	if  ScriptManager:getMissionStatus( missionIds.bringSurvivorsHome ) == MissionStatus.IN_PROGRESS then
		ScriptManager:setMissionStatus( missionIds.bringSurvivorsHome, MissionStatus.ACCOMPLISHED );
	end
	
	ScriptManager:displaySubtitle( Text.T_MISSION_VICTORY );
	ScriptManager:wait( 5 );
	HideDialogue();
	endGame();
end


-- END Victory
-- BEGIN Defeated


function missionFailedTransition()
	ScriptManager:runForPlayer( 0, "missionFailed" );
end

function missionFailed()
	DisablePlayerInteraction();
	HideDialogue(); -- remove possible texts on the screens
	
	ScriptManager:displaySubtitle( Text.T_MISSION_FAILED );
	ScriptManager:wait( 5 );
	HideDialogue();
	endGame();
end


-- END Defeated


function endGame()
	ScriptManager:endGame()
end