require "lua/Constants"
require "lua/Functions"
require "lua/Functions_2"
require "lua/Text"
require "lua/Units"
require "math"


-- mission stuff
missionIds = {
	destroyPrison 	= 1,
	getOutposts		= 2
};

-- missionId, primary( boolean ), textkey, xpReward
missions = {
	{ missionIds.destroyPrison, true,  Text.T_MAP_SU_4_2_MISSION_PRIMARY,	225 },
	{ missionIds.getOutposts,	false, Text.T_MAP_SU_4_2_MISSION_SECONDARY,	175 }
};

enemySquadIds = {
	northOutpost		=  1,
	southOutpost		=  2,
	thePrison			=  3,
	loneMgNest			=  4,
	mountainAA			=  6,
	lowerWaveSquad		=  7,
--	upperWaveSquad		=  8++;
};

-- Ids 0, 9, 10, 11 are depots
-- all enemy Ids in their squads
northOutpostIds 		= {  6, 12, 13, 23, 24, 25, 26, 27 };
southOutpostIds			= {  2,  8, 14, 15, 16, 17, 18, 19, 20, 21, 22  };
thePrisonIds			= { 30, 31, 32, 33, 34, 35, 36, 37 };
loneMgNest				= 38;
mountainAA				=  3;
northOutpostDepot		=  9;
southOutpostDepot		= 10;
thePrisonDepot			= 11;

-- intro cutscene battle
attacker				= { 47, 48 }
defender				= { 41, 42, 43 };
sacrifice				= 49;

prisonUnitsLeft			= #thePrisonIds;

lastWaveSpawnedInRound	= -5;
spawnWaveInterval		=  5;
spawnWavesEnabled		=  1;
upperSquadCountStart	=  8;

-- all the waves that can spawn
wave1 = {
--	playerId, templateID, 	tileX, tileY,	AiState
	{ 1, Units.UNIT_DE_MP40,		4, 5, AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_DE_TANK_4,		4, 5, AiState.ATTACK_AGGRESSIVE }
};

wave2 = {
--	playerId, templateID, 	tileX, tileY,	AiState
	{ 1, Units.UNIT_DE_MP40,		4, 5, AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_DE_PANZERFAUST,	4, 5, AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_DE_TANK_3,		4, 5, AiState.ATTACK_AGGRESSIVE }
};

wave3 = {
--	playerId, templateID, 	tileX, tileY, 	AiState
	{ 1, Units.UNIT_DE_MP40,		4, 5,  AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_DE_PANZERFAUST,	4, 5,  AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_DE_PANZERFAUST,	4, 5,  AiState.ATTACK_AGGRESSIVE }
};

wave4 = {
--	playerId, templateID, 	tileX, tileY, AiState
	{ 1, Units.UNIT_DE_TANK_4,		4, 5, AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_DE_TANK_3,		4, 5, AiState.ATTACK_AGGRESSIVE }
};


-- BEGIN Init


function onGameStart()
	CreateMissions( missions );
	setTriggers();
	storeValues();
	
	ScriptManager:moveCameraEx( 9, 16, 0.7, 0, false );
	ScriptManager:gainGold( 0, 1500 );
	
	-- start intro cutscene
	ScriptManager:sleep( 0, "startIntroCutscene" ); -- without sleep player turn popup shows
end

function storeValues()
	ScriptManager:storeValue( "prisonUnitsLeft",		prisonUnitsLeft );
	ScriptManager:storeValue( "lastWaveSpawnedInRound", lastWaveSpawnedInRound );
	ScriptManager:storeValue( "spawnWaveInterval",		spawnWaveInterval );
	ScriptManager:storeValue( "spawnWavesEnabled",		spawnWavesEnabled );
	ScriptManager:storeValue( "upperSquadCountStart",	upperSquadCountStart );
end

function setTriggers()
	-- load all needed values after loading
	ScriptManager:addOnGameResumeTrigger( "loadValues", "loadValues", false );	
	
	-- player captured a depot
	ScriptManager:addOnUnitCapturedTrigger( 0, -1, -1, "onDepotCaptured", "onDepotCaptured", false );
	
	-- player killed enemy
	ScriptManager:addOnUnitDestroyedTrigger( 1, -1, -1, "onEnemyDestroyed", "onEnemyDestroyed", false );
	
	-- player attacking prison / enemy attacking depot
	ScriptManager:addOnUnitHasAttackedTrigger( 0, Units.UNIT_DE_PRISON, 11, "ProtectTheBuildingHintSingle",		"ProtectTheSupplyDepotHintSingle", false );
	ScriptManager:addOnUnitHasAttackedTrigger( 1, Units.UNIT_SU_HQ, 	 0, "ProtectTheSupplyDepotHintSingle",  "ProtectTheSupplyDepotHintSingle", false );
	ScriptManager:addOnUnitHasAttackedTrigger( 1, Units.UNIT_DE_PRISON, 11, "ProtectTheBuildingHintSingle",		"ProtectTheSupplyDepotHintSingle", false );
	
	ScriptManager:addOnUnitDestroyedTrigger(   0, Units.UNIT_DE_PRISON, 11, "missionFailedTransition", "missionFailedTransition" );
	ScriptManager:addOnUnitDestroyedTrigger(   0, Units.UNIT_SU_HQ, 	 0, "missionFailedTransition", "missionFailedTransition" );
	
	-- enemy wave spawn
	ScriptManager:addOnTurnStartTrigger( 1, -1, -1, "spawnEnemyWave", "spawnEnemyWave", false );
	
	-- additional trigger for waves that take the upper way to move on to the player base
	ScriptManager:addOnEnterTileTrigger( 5, 17, 1, -1, -1, false, "moveOnToHq", "moveOnToHq", false );
	ScriptManager:addOnEnterTileTrigger( 6, 16, 1, -1, -1, false, "moveOnToHq", "moveOnToHq", false );
	ScriptManager:addOnEnterTileTrigger( 6, 15, 1, -1, -1, false, "moveOnToHq", "moveOnToHq", false );
	ScriptManager:addOnEnterTileTrigger( 7, 14, 1, -1, -1, false, "moveOnToHq", "moveOnToHq", false );
end

function loadValues()
	prisonUnitsLeft			= ScriptManager:getValue( "prisonUnitsLeft" );
	lastWaveSpawnedInRound	= ScriptManager:getValue( "lastWaveSpawnedInRound" );
	spawnWaveInterval		= ScriptManager:getValue( "spawnWaveInterval" );
	spawnWavesEnabled		= ScriptManager:getValue( "spawnWavesEnabled" );
	upperSquadCountStart	= ScriptManager:getValue( "upperSquadCountStart" );
end


-- END Init
-- BEGIN Intro Cutscene


function startIntroCutscene()
	StartCutscene();
	ScriptManager:wait( 1 );
	
	ScriptManager:moveUnitEx(	attacker[1], 10, 16,		true );	-- move attacker 1 in range
	ScriptManager:attackUnitEx( attacker[1], sacrifice,		true );	-- attack the guy that will die with attacker 1
	
	local unit  = Unit:getUnitById( attacker[1] );
	unit:setHealth( 1 );
	
	ScriptManager:attackUnitEx( defender[3], attacker[1],	true );	-- kill attacker 1 with defender 3
	
	ScriptManager:moveUnitEx(	attacker[2], 10, 16,		true );	-- move attacker 2 in range
	ScriptManager:attackUnitEx( attacker[2], sacrifice,		true );	-- kill the guy that will die with attacker 2
	ScriptManager:attackUnitEx( defender[1], attacker[2],	true );	-- attack attacker 2 with defender 1
	ScriptManager:attackUnitEx( defender[2], attacker[2],	true );	-- kill attacker 2 with defender 2;
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_4_2_MISSION_INTRO_FLAVOR_01 );
	ScriptManager:wait( 5 );
	HideDialogue();
	ScriptManager:wait( 0.5 ); -- antirush
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_4_2_MISSION_INTRO_FLAVOR_02 );
	ScriptManager:wait( 5 );
	HideDialogue();
	ScriptManager:wait( 0.5 ); -- antirush
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_4_2_MISSION_INTRO_FLAVOR_03 );
	ScriptManager:moveCameraEx(  5, 15, 0.6, 2, true );
	ScriptManager:wait( 0.5 ); -- antirush
	
	--marker for north outpost
	ScriptManager:revealFogOfWar(	4, 15, 2, true );
	ScriptManager:showMarkerAt(		4, 16 );
	ScriptManager:wait( 3 );
	
	HideAllMarkers();
	ScriptManager:revealFogOfWar(	4, 15, 2, false );
	ScriptManager:wait( 0.5 ); -- antirush
	
	ScriptManager:moveCameraEx( 10,  6, 0.6, 2, true );
	ScriptManager:wait( 0.5 ); -- antirush	
	
	-- marker for south outpost
	ScriptManager:revealFogOfWar(	10, 5, 2, true );
	ScriptManager:showMarkerAt(		10, 4 );
	ScriptManager:wait( 3 );
	
	HideAllMarkers();
	ScriptManager:revealFogOfWar(	10, 5, 2, false );
	ScriptManager:wait( 0.5 ); -- antirush
	
	HideDialogue();
	ScriptManager:wait( 0.5 ); -- antirush
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_4_2_MISSION_INTRO_FLAVOR_04 );
	ScriptManager:moveCameraEx( 4, 5, 0.6, 2.5, true );
	ScriptManager:wait( 0.5 ); -- antirush
	
	-- marker for prison
	ScriptManager:revealFogOfWar( 4, 5, 2, true );
	ScriptManager:showMarkerAt( 4, 5 );
	ScriptManager:wait( 5 );	
		
	HideAllMarkers();
	ScriptManager:revealFogOfWar( 4, 5, 2, false );
	HideDialogue();
	ScriptManager:wait( 0.5 ); -- antirush
	
	ScriptManager:moveCameraEx( 13, 16, 0.6, 3, false )
	ScriptManager:displaySubtitle( Text.T_MAP_SU_4_2_MISSION_INTRO_FLAVOR_05 );
	ScriptManager:wait( 5 );
	HideDialogue();
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_4_2_MISSION_INTRO_FLAVOR_06 );
	ScriptManager:wait( 4 );
	HideDialogue();
	
	if ScriptManager:isControllerConnected() then
		ScriptManager:setControllerCursorPosition( 13, 16 );
	end
	
	EndCutscene();
	
	startTheMission();
end

function startTheMission()
	ScriptManager:setMissionStatus( missionIds.destroyPrison,	MissionStatus.IN_PROGRESS );
	ScriptManager:setMissionStatus( missionIds.getOutposts,		MissionStatus.IN_PROGRESS );
	
	--TODO: remove this function!
	--testing();
	--removeSquads();
	--ScriptManager:addOnUnitSelectedTrigger( -1, -1, -1, "showParameters", "showParameters",false)
end

function testing()
	ScriptManager:spawnUnit( 0, Units.GOTT, 1, 1, 0, AiState.WAIT );
	ScriptManager:spawnUnit( 0, Units.GOTT, 1, 2, 0, AiState.WAIT );
end

function removeSquads()
	for i = 1, #southOutpostIds do
		ScriptManager:removeUnitEx( southOutpostIds[i], false );
	end
	
	for i = 1, #northOutpostIds do
		ScriptManager:removeUnitEx( northOutpostIds[i], false );
	end
	
	--for i = 1, #thePrisonIds do
	--	ScriptManager:removeUnitEx( thePrisonIds[i], false );
	--end
end

function showParameters( selectedUnit )
	print( string.format( "[squadId] %s", selectedUnit:getSquadId() ) );
end


-- END Intro Cutscene
-- BEGIN onEnemyX


function onDepotCaptured( activeUnit, passiveUnit )
	local depotId = passiveUnit:getId();
	
	-- if it is an outpost set progress for sec mission
	if depotId == northOutpostDepot or depotId == southOutpostDepot then
		setOutpostMissionProgress();
	
	-- if it is the prison the player won the game
	elseif depotId == thePrisonDepot then
		-- if there are any defenders left make them aggressive so the player does not have to look hours for them
		local depotSquad = passiveUnit:getSquadId();
		Unit:changeSquadState( 		 1, depotSquad, AiState.ATTACK_AGGRESSIVE );		
		ScriptManager:setTargetTile( 1, depotSquad, 4, 5 );
		
		--passiveUnit:setIsAttackable( false ); -- make prison unattackable to stop the enemies from ignoring the player and destroying it
		
		spawnWavesEnabled = 0;
		ScriptManager:storeValue( "spawnWavesEnabled", spawnWavesEnabled );
		RemoveTrigger( "spawnEnemyWave" );
		
		if prisonUnitsLeft > 0 then
			StartCutscene();
			
			ScriptManager:displaySubtitle( Text.T_MAP_SU_4_2_MISSION_PRIMARY_FLAVOR );
			ScriptManager:wait( 5 );
			HideDialogue();
			
			EndCutscene();			
		end
		
		checkForVictory();
	end
end

function onEnemyDestroyed( unit )
	local squadId		= unit:getSquadId();
	local unitId		= unit:getId();
	
	-- player destroyed the prison = mission failed
	if squadId == enemySquadIds.thePrison then
		if unitId == thePrisonDepot then
			missionFailed();
		
		-- player needs to cap the prison AND kill all the defenders around it, the latter happens here
		else
			prisonUnitsLeft = prisonUnitsLeft -1;
			print( prisonUnitsLeft );			
			ScriptManager:storeValue( "prisonUnitsLeft", prisonUnitsLeft );
			
			if prisonUnitsLeft <= 0 then
				print( "defendersDown" );
				
				ScriptManager:runForPlayer( 0, "checkForVictory" );
				
			else
				ShowMissionProgressPopup( prisonUnitsLeft, Text.T_MAP_SU_4_2_MISSION_PRIMARY_COUNTER );
			end
		end
	
	-- sec mission progress
	elseif unitId == northOutpostDepot or unitId == southOutpostDepot then
		setOutpostMissionProgress();
	end
end

function setOutpostMissionProgress()
	-- use progress here, so I don't have to save it in a variable
	ScriptManager:setMissionProgress( missionIds.getOutposts, ScriptManager:getMissionProgress( missionIds.getOutposts ) + 1 );
	
	if  ScriptManager:getMissionProgress( missionIds.getOutposts ) == 2 then
		ScriptManager:setMissionStatus( missionIds.getOutposts, MissionStatus.ACCOMPLISHED );
		
		ScriptManager:displaySubtitle( Text.T_MAP_SU_4_2_MISSION_SECONDARY_FLAVOR );
		ScriptManager:wait( 5 );
		HideDialogue();
	end
end


-- END onEnemyX
-- BEGIN Enemy Waves


function spawnEnemyWave()
	if spawnWavesEnabled == 1 then
	
		-- check if a wave spawns this round
		if lastWaveSpawnedInRound + spawnWaveInterval <= ScriptManager:getCurrentRound() then
			-- define by which wave will spawn
			local waveNumber = math.random( 0, 3 );
			print( waveNumber );
			
			if waveNumber == 0 then
				spawnWave( wave1 );
			
			elseif waveNumber == 1 then
				spawnWave( wave2 );
			
			elseif waveNumber == 2 then
				spawnWave( wave3 );
			
			elseif waveNumber == 3 then
				spawnWave( wave4 );
			end
			
			-- set counter for nexts spawn
			lastWaveSpawnedInRound = ScriptManager:getCurrentRound();
			ScriptManager:storeValue( "lastWaveSpawnedInRound", lastWaveSpawnedInRound );
		end
	end
end

function spawnWave( waveData )
	for i = 1, #waveData do
		spawnWaveUnit( waveData[i] );		
	end
end

function spawnWaveUnit( spawnData )
	local squadId = 0;
	local tileX = 0;
	local tileY = 0;
	
	-- define by random which path the squad will take ( through upper or lower outpost )
	if math.random( 0, 1 ) == 0 then
		squadId = enemySquadIds.lowerWaveSquad;
		tileX = 13;
		tileY = 17;		
		print( "lower path" );
	
	else
		-- as the upper path is too long to reach the player I need to set the target tile again ( see moveOnToHq )
		-- in order to be able to, I need a unique squadId for each squad otherwise I may fuck up the path of them
		squadId = upperSquadCountStart;
		tileX =  9;
		tileY = 17
		print ( "upper path" );
		
		upperSquadCountStart = upperSquadCountStart + 1;
		ScriptManager:storeValue("upperSquadCountStart", upperSquadCountStart );
	end
	
	ScriptManager:spawnUnit( spawnData[1], spawnData[2], spawnData[3], spawnData[4], squadId, spawnData[5] );
	ScriptManager:setTargetTile( 1, squadId, tileX, tileY ); -- send squad 8+ through upper exit
end

function moveOnToHq( tileX, tileY, playerId, templateId, unitId )
	-- if wave unit moves upper path it does not reach the depot ( cause the path is too long )
	-- so I manually push the unit here to the player depot
	local unit = Unit:getUnitById( unitId );
	local squadId = unit:getSquadId();
	
	if squadId >= 8 then
		ScriptManager:setTargetTile( 1, squadId, 13, 17 );
	end
end


-- END Enemy Waves
-- BEGIN Victory


function checkForVictory()
	-- check if both victory conditions ( prison captured, defenders killed ) are accomplished
	local prison = Unit:getUnitById( thePrisonDepot );
	
	-- if the prison is captured and all defenders are dead, start victory cutscene
	if prisonUnitsLeft <= 0 and prison:getPlayerId() == 0 then
		ScriptManager:runForPlayer( 0 , "victoryInPlayerTurn" );
	end
end

function victoryInPlayerTurn()
		DisablePlayerInteraction();
		ScriptManager:setMissionStatus( missionIds.destroyPrison, MissionStatus.ACCOMPLISHED );
		ScriptManager:wait( 2 );
		startVictoryCutscene();
	end

function startVictoryCutscene()
	StartCutscene();
	
	ScriptManager:moveCameraEx( 4, 5, 0.7, 2, true );
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_4_2_MISSION_VICTORY_FLAVOR_01 );
	ScriptManager:wait( 4 );
	HideDialogue();
	ScriptManager:wait( 0.5 ); -- antirush
	
	ScriptManager:spawnUnits( 0, Units.UNIT_SU_PRISONER_RIFLE,  4, 5, 1, AiState.PASSIVE, 3 );
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_4_2_MISSION_VICTORY_FLAVOR_02 );
	ScriptManager:wait( 4 );
	HideDialogue();
	
	EndCutscene();
	
	victoryEndGame();
end

function victoryEndGame()
	DisablePlayerInteraction();
	HideDialogue(); -- remove possible texts on the screens
	
	ScriptManager:displaySubtitle( Text.T_MISSION_VICTORY );
	ScriptManager:wait( 4 );
	HideDialogue();
	endGame();
end


-- END Victory
-- BEGIN Defeated


function missionFailedTransition()
	ScriptManager:runForPlayer( 0, "missionFailed" );
end

function missionFailed()
	DisablePlayerInteraction();
	HideDialogue(); -- remove possible texts on the screens
	
	ScriptManager:displaySubtitle( Text.T_MISSION_FAILED );
	ScriptManager:wait( 4 );
	HideDialogue();
	endGame();
end


-- END Defeated


function endGame()
	ScriptManager:endGame()
end