require "lua/Constants"
require "lua/Functions"
require "lua/Functions_2"
require "lua/Text"
require "lua/Units"
require "math"


-- mission stuff
missionIds = {
	getDepots		= 1,
	defeatDefenders = 2,
	repelAttackers	= 3,
	getAirfield		= 4,	
	getFactory		= 5
};

-- missionId, primary( boolean ), textkey, xpReward
missions = {
	{ missionIds.getDepots,			true,  Text.T_MAP_SU_4_3_MISSION_PRIMARY_01,	200 },
	{ missionIds.defeatDefenders,	true,  Text.T_MAP_SU_4_3_MISSION_PRIMARY_03,	150 },
	{ missionIds.repelAttackers,	true,  Text.T_MAP_SU_4_3_MISSION_PRIMARY_02,	100 },
	{ missionIds.getAirfield,		false, Text.T_MAP_SU_4_3_MISSION_SECONDARY_01,	100 },
	{ missionIds.getFactory,		false, Text.T_MAP_SU_4_3_MISSION_SECONDARY_02,	 75 }
};

-- all enemy squad / unit Ids
buildingIds = {
	west		= 0,
	north		= 1,
	south		= 2,
	airfield	= 5
};

enemySquadIds = {
	factory			= 1,
	northOutpost	= 2,
	southOutpost	= 3,
	airfield		= 4,
	lake			= 5,
	planeAttackWave	= 6,
	attackWave		= 7,
	attackers		= 8
};

factoryIds			= { 15, 17, 18, 19, 20, 21, 22, 34, };
northOutpostIds		= { 23, 24, 25, 26, 27, 28, 29, 30, 32 };
southOutpostIds		= { 16, 40, 41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54 };
airfieldIds			= { 36, 37, 38, 39 };
lakeIds				= { 55, 56, 57, 58, 59 };

capturedDepots		= 0;
remainingDefenders	= {}; -- for last main objective
maxNumberOfDef		= #northOutpostIds + #southOutpostIds + #lakeIds;

-- spawn for the plane waves
airfieldSpawnTiles	= {
	{ 22, 13 },
	{ 23, 14 }
};
--spawn for the attack waves
depotSpawnTiles = {
	{ 21,  2 },
	{ 21,  3 },
	{ 21,  4 }
};
-- spawn for the reinforcements
streetSpawnTiles = {
	{ 20,  1 },
	{ 21,  1 },
	{ 22,  1 },
	{ 23,  1 }
};

-- I spawn either two Fighters or one Bomber depending on the number of spawns and the round count
-- ( every x turns I spawn an plane wave and every y spawns the bomber comes instead of fighters )
spawnPlaneWaves				=  1; -- 0 == false; 1 == true; cant save booleans so I have to use numbers
planeWaveRespawnInterval	=  6; -- defines how often waves will spawn
bomberAppearance			=  3; -- every bomberAppearance-spawn a bomber appears instead of two fighters
numberOfPlaneWavesSpawned	=  0; -- the number of plane waves that have already spawend in this game
lastPlaneWaveSpawnedInRound	= -4; -- indicates the round in which the last wave spawned ( -3 in order to spawn the first instant )
planeAttackWaveUnits = {
--	playerId, templateID, 				tileX,					 tileY,							 squadId,				AiState
	{ 1, Units.UNIT_DE_FIGHTER,	airfieldSpawnTiles[1][1], airfieldSpawnTiles[1][2], enemySquadIds.planeAttackWave, AiState.AIRPLANE },
	{ 1, Units.UNIT_DE_FIGHTER,	airfieldSpawnTiles[2][1], airfieldSpawnTiles[2][2], enemySquadIds.planeAttackWave, AiState.AIRPLANE },
	{ 1, Units.UNIT_DE_BOMBER,	airfieldSpawnTiles[1][1], airfieldSpawnTiles[1][2], enemySquadIds.planeAttackWave, AiState.AIRPLANE }
};

-- additional enemies that spawn from the southern depot to defend
-- they respawn in a certain interval ( every x turns ) until the barricade is gone
spawnAttackWaves				=  1; -- 0 == false; 1 == true; cant save booleans so I have to use numbers
attackWaveRespawnInterval		=  6; -- defines how often waves will spawn
lastAttackWaveSpawnedInRound	= -2; -- indicates the round in which the last wave spawned
attackWaveUnits1 = {
--	playerId, templateID, 				tileX,					 tileY,							 squadId,				AiState
	{ 1, Units.UNIT_DE_MP40,			depotSpawnTiles[1][1], depotSpawnTiles[1][2], enemySquadIds.attackWave, AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_DE_TANK_4,			depotSpawnTiles[2][1], depotSpawnTiles[2][2], enemySquadIds.attackWave, AiState.ATTACK_AGGRESSIVE }
};

attackWaveUnits2 = {
--	playerId, templateID, 				tileX,					 tileY,							 squadId,				AiState
	{ 1, Units.UNIT_DE_MP40,			depotSpawnTiles[1][1], depotSpawnTiles[1][2], enemySquadIds.attackWave, AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_DE_PANZERFAUST,		depotSpawnTiles[2][1], depotSpawnTiles[2][2], enemySquadIds.attackWave, AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_DE_TANK_3,			depotSpawnTiles[3][1], depotSpawnTiles[3][2], enemySquadIds.attackWave, AiState.ATTACK_AGGRESSIVE }
};

attackWaveUnits3 = {
--	playerId, templateID, 				tileX,					 tileY,							 squadId,				AiState
	{ 1, Units.UNIT_DE_MP40,			depotSpawnTiles[1][1], depotSpawnTiles[1][2], enemySquadIds.attackWave, AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_DE_PANZERFAUST,		depotSpawnTiles[2][1], depotSpawnTiles[2][2], enemySquadIds.attackWave, AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_DE_PANZERFAUST,		depotSpawnTiles[3][1], depotSpawnTiles[3][2], enemySquadIds.attackWave, AiState.ATTACK_AGGRESSIVE }
};

attackWaveUnits4 = {
--	playerId, templateID, 				tileX,					 tileY,							 squadId,				AiState
	{ 1, Units.UNIT_DE_TANK_4,			depotSpawnTiles[1][1], depotSpawnTiles[1][2], enemySquadIds.attackWave, AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_DE_TANK_3,			depotSpawnTiles[2][1], depotSpawnTiles[2][2], enemySquadIds.attackWave, AiState.ATTACK_AGGRESSIVE }
};

attackWaveUnits5 = {
--	playerId, templateID, 				tileX,					 tileY,							 squadId,				AiState
	{ 1, Units.UNIT_DE_MP40,			depotSpawnTiles[1][1], depotSpawnTiles[1][2], enemySquadIds.attackWave, AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_DE_TANK_DESTROYER,	depotSpawnTiles[2][1], depotSpawnTiles[2][2], enemySquadIds.attackWave, AiState.ATTACK_AGGRESSIVE },
};

-- German reinforcements
-- they will spawn over three turns and the player has to kill them all to win
reinforcementSpawnInTurn = 0;
reinforcementCounter = 1;
reinforcementIds = {}; -- all the ids in order to check if they are still alive
maxNumberOfReinforcements = 9; -- maximum possible number of reinforcements alive
reinforcements1 = {
--	playerId, templateID, 				tileX,					 tileY,							 squadId,				AiState
	{ 1, Units.UNIT_DE_MP40,			streetSpawnTiles[1][1], streetSpawnTiles[1][2], enemySquadIds.attackers, AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_DE_PANZERFAUST,		streetSpawnTiles[2][1], streetSpawnTiles[2][2], enemySquadIds.attackers, AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_DE_MP40,			streetSpawnTiles[3][1], streetSpawnTiles[3][2], enemySquadIds.attackers, AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_DE_PANZERFAUST,		streetSpawnTiles[4][1], streetSpawnTiles[4][2], enemySquadIds.attackers, AiState.ATTACK_AGGRESSIVE },
};

reinforcements2 = {
--	playerId, templateID, 				tileX,					 tileY,							 squadId,				AiState
	{ 1, Units.UNIT_DE_TANK_3,			streetSpawnTiles[2][1], streetSpawnTiles[1][2], enemySquadIds.attackers, AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_DE_TANK_4,			streetSpawnTiles[3][1], streetSpawnTiles[2][2], enemySquadIds.attackers, AiState.ATTACK_AGGRESSIVE },
};

reinforcements3 = {
--	playerId, templateID, 				tileX,					 tileY,							 squadId,				AiState
	{ 1, Units.UNIT_DE_MP40,			streetSpawnTiles[1][1], streetSpawnTiles[1][2], enemySquadIds.attackers, AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_DE_TIGER,			streetSpawnTiles[2][1], streetSpawnTiles[2][2], enemySquadIds.attackers, AiState.ATTACK_AGGRESSIVE },
	{ 1, Units.UNIT_DE_MP40,			streetSpawnTiles[3][1], streetSpawnTiles[3][2], enemySquadIds.attackers, AiState.ATTACK_AGGRESSIVE },
};


-- BEGIN Init


function onGameStart()
	CreateMissions( missions );
	setTriggers();
	storeValues();
	
	ScriptManager:gainGold( 0, 2500 );
	
	-- start intro cutscene
	ScriptManager:sleep( 0, "startIntroCutscene" ); -- without sleep player turn popup shows
end

function storeValues()
	ScriptManager:storeValue( "capturedDepots", 				capturedDepots );
	
	ScriptManager:storeValue( "spawnPlaneWaves",				spawnPlaneWaves );
	ScriptManager:storeValue( "numberOfPlaneWavesSpawned",		numberOfPlaneWavesSpawned );
	ScriptManager:storeValue( "lastPlaneWaveSpawnedInRound",	lastPlaneWaveSpawnedInRound );
	
	ScriptManager:storeValue( "spawnAttackWaves",				spawnAttackWaves );
	ScriptManager:storeValue( "attackWaveRespawnInterval",		attackWaveRespawnInterval );
	ScriptManager:storeValue( "lastAttackWaveSpawnedInRound",	lastAttackWaveSpawnedInRound );
	
	ScriptManager:storeValue( "reinforcementSpawnInTurn",		reinforcementSpawnInTurn );
	ScriptManager:storeValue( "reinforcementCounter", 			reinforcementCounter );
end

function setTriggers()
	-- load all needed values after loading
	ScriptManager:addOnGameResumeTrigger( "loadValues", "loadValues", false );
		
	-- player depot getting attacked warning
	ScriptManager:addOnUnitHasAttackedTrigger( 1, Units.UNIT_SU_HQ, -1, "ProtectTheSupplyDepotHintSingle", "ProtectTheSupplyDepotHintSingle", false );
	ScriptManager:addOnUnitHasAttackedTrigger( 1, Units.UNIT_DE_HQ, -1, "ProtectTheSupplyDepotHintSingle", "ProtectTheSupplyDepotHintSingle", false );
	
	-- player depot got destroyed
	ScriptManager:addOnUnitDestroyedTrigger( 0, Units.UNIT_SU_HQ, -1, "missionFailedTransition", "missionFailedTransition", false );
	ScriptManager:addOnUnitDestroyedTrigger( 0, Units.UNIT_DE_HQ, -1, "missionFailedTransition", "missionFailedTransition", false );
	
	-- capturing an enemy depot / factory / airfield
	ScriptManager:addOnUnitCapturedTrigger( 0, Units.UNIT_DE_HQ, 		-1, "onDepotCaptured",		"onDepotCaptured", 	 false );
	ScriptManager:addOnUnitCapturedTrigger( 0, 49,						-1, "onFactoryCaptured",	"onFactoryCaptured", false );
	ScriptManager:addOnUnitCapturedTrigger( 0, Units.UNIT_DE_AIRFIELD,	-1, "onAirfieldCaptured",	"onAifieldCaptured", false );
	
	-- killing an enemy
	ScriptManager:addOnUnitDestroyedTrigger( 1, -1, -1, "onEnemyDestroyed", "onEnemyDestroyed", false );
	
	-- trigger  destroy airfield mission
	ScriptManager:addOnUnitHasAttackedTrigger( 1, Units.UNIT_DE_BOMBER,		-1, "onPlaneHasAttacked", "onPlaneHasAttacked", false );
	ScriptManager:addOnUnitHasAttackedTrigger( 1, Units.UNIT_DE_FIGHTER,	-1, "onPlaneHasAttacked", "onPlaneHasAttacked", false );
	
	
	-- trigger for wave spawns
	ScriptManager:addOnTurnStartTrigger( 1, -1, -1, "onEnemyTurnStart", "onEnemyTurnStart", false );
end

function loadValues()
	capturedDepots 					= ScriptManager:getValue( "capturedDepots" );
	
	spawnPlaneWaves					= ScriptManager:getValue( "spawnPlaneWaves" );
	numberOfPlaneWavesSpawned		= ScriptManager:getValue( "numberOfPlaneWavesSpawned" );
	lastPlaneWaveSpawnedInRound		= ScriptManager:getValue( "lastPlaneWaveSpawnedInRound" );
	
	spawnAttackWaves				= ScriptManager:getValue( "spawnAttackWaves" );
	attackWaveRespawnInterval 		= ScriptManager:getValue( "attackWaveRespawnInterval" );
	lastAttackWaveSpawnedInRound	= ScriptManager:getValue( "lastAttackWaveSpawnedInRound" );
	
	reinforcementSpawnInTurn		= ScriptManager:getValue( "reinforcementSpawnInTurn" );
	reinforcementCounter			= ScriptManager:getValue( "reinforcementCounter" );
	
	-- load the reinforcements back into the table if it is empty and the reinforcement mission is in progress
	if #reinforcementIds == 0 and ScriptManager:getMissionStatus( missionIds.repelAttackers ) == MissionStatus.IN_PROGRESS then		
		for i = 1, maxNumberOfReinforcements do
			local value = ScriptManager:getValue( string.format( "reinforcementIds %i", i ) );
			
			if value ~= -1 then -- if value exists
				table.insert( reinforcementIds, value );
			end
		end
	end

	-- load remaining defenders back into the table if it is empty
	if #remainingDefenders == 0 and ScriptManager:getMissionStatus( missionIds.defeatDefenders ) == MissionStatus.IN_PROGRESS then	
		for i = 1, maxNumberOfDef do
			local value = ScriptManager:getValue( string.format( "remainingDefenders %i", i ) );
			
			if value ~= -1 then -- if value exists
				print( string.format( "def %i found in slot %i", value, i )  );
				table.insert( remainingDefenders, value );
			end
		end
	end
end


-- END Init
-- BEGIN Intro Cutscene


function startIntroCutscene()
	StartCutscene();
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_4_3_MISSION_INTRO_FLAVOR_01 );
	ScriptManager:wait( 5 );
	HideDialogue();
	ScriptManager:wait( 0.5 ); -- antirush
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_4_3_MISSION_INTRO_FLAVOR_02 );
	ScriptManager:wait( 5 );
	HideDialogue();
	ScriptManager:wait( 0.5 ); -- antirush
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_4_3_MISSION_INTRO_FLAVOR_03 );
	ScriptManager:moveCameraEx( 16, 14, 0.7, 2, true );
	ScriptManager:wait( 0.5 ); -- antirush
	
	-- mark north depot
	ScriptManager:revealFogOfWar( 16, 14, 1, true );
	ScriptManager:showMarkerAt( 16, 14 );
	ScriptManager:wait( 3 );
	HideAllMarkers();
	ScriptManager:revealFogOfWar( 16, 14, 1, false );
	HideDialogue();
	ScriptManager:wait( 0.5 ); -- antirush
	
	ScriptManager:moveCameraEx( 20, 3, 0.7, 2, false );
	ScriptManager:wait( 1 );
	ScriptManager:displaySubtitle( Text.T_MAP_SU_4_3_MISSION_INTRO_FLAVOR_04 );
	ScriptManager:wait( 2 );
	
	-- mark south depot
	ScriptManager:revealFogOfWar( 20, 3, 1, true );
	ScriptManager:showMarkerAt( 20, 3 );
	ScriptManager:wait( 3.5 );
	HideAllMarkers();
	ScriptManager:revealFogOfWar( 20, 3, 1, false );
	HideDialogue();
	ScriptManager:wait( 0.5 ); -- antirush
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_4_3_MISSION_INTRO_FLAVOR_05 );
	ScriptManager:moveCameraEx( 3, 3, 0.7, 5, true );
	HideDialogue();
	ScriptManager:wait( 0.5 ); -- antirush
	
	-- mark factory
	ScriptManager:displaySubtitle( Text.T_MAP_SU_4_3_MISSION_INTRO_FLAVOR_06 );
	ScriptManager:revealFogOfWar( 3, 3, 1, true );
	ScriptManager:showMarkerAt( 3, 3 );
	ScriptManager:wait( 4 );
	HideAllMarkers();
	ScriptManager:revealFogOfWar( 3, 3, 1, false );
	HideDialogue();
	ScriptManager:wait( 0.5 ); -- antirush
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_4_3_MISSION_INTRO_FLAVOR_07 );
	ScriptManager:moveCameraEx( 8, 11, 0.7, 3, true );
	HideDialogue();
	
	if ScriptManager:isControllerConnected() then
		ScriptManager:setControllerCursorPosition( 8, 11 );
	end
	
	EndCutscene();
	
	startTheMission();
end

function startTheMission()
	ScriptManager:setMissionStatus( missionIds.getDepots,  MissionStatus.IN_PROGRESS );
	ScriptManager:setMissionStatus( missionIds.getFactory, MissionStatus.IN_PROGRESS );
		
	--TODO: remove this function!
	--testing();
	--removeSquads();
	--ScriptManager:addOnUnitSelectedTrigger( 1, -1, -1, "showParameters", "showParameters",false)
end

function testing()
	ScriptManager:spawnUnit( 0, Units.GOTT, 11, 15, 0, AiState.WAIT );
	ScriptManager:spawnUnit( 0, Units.GOTT, 12, 15, 0, AiState.WAIT );
end

function removeSquads()
	for i = 1, #factoryIds do
		ScriptManager:removeUnitEx( factoryIds[i], false );
	end
	
	for i = 1, #northOutpostIds do
		ScriptManager:removeUnitEx( northOutpostIds[i], false );
	end
	function planeDestroyed()
	ScriptManager:setMissionProgress(missionIds.destroyPlanes, ScriptManager:getMissionProgress( missionIds.destroyPlanes ) + 1 );
	
	ScriptManager:runForPlayer( 0, "showPlaneDestroyedProgress" );
end

function showPlaneDestroyedProgress()
	if ScriptManager:getMissionProgress(missionIds.destroyPlanes) >= planesToDestroy then
		DisablePlayerInteraction();
		
		ScriptManager:setMissionStatus(missionIds.destroyPlanes, MissionStatus.ACCOMPLISHED );
		ScriptManager:wait( 2 );
		
		EnablePlayerInteraction();
	
	else
		ShowMissionProgressPopup( ScriptManager:getMissionProgress( missionIds.destroyPlanes ), Text.T_MAP_SU_4_2_MISSION_PRIMARY_COUNTER );
	end
end
	for i = 1, #southOutpostIds do
		ScriptManager:removeUnitEx( southOutpostIds[i], false );
	end
	
	for i = 1, #airfieldIds do
		ScriptManager:removeUnitEx( airfieldIds[i], false );
	end
	
	for i = 1, #lakeIds do
		ScriptManager:removeUnitEx( lakeIds[i], false );
	end
end

function showParameters( selectedUnit )
	print( string.format( "[SquadId] %s", selectedUnit:getSquadId() ) );
	print( string.format( "[AiState] %s", selectedUnit:getAiState() ) );
end


-- END Intro Cutscene
-- BEGIN onEnemyX


function onDepotCaptured( activeUnit, passiveUnit )
	local depotId = passiveUnit:getId();
	capturedDepots = capturedDepots + 1;
	ScriptManager:storeValue( "capturedDepots", capturedDepots );
	
	if capturedDepots == 2 then
		DisablePlayerInteraction();
		ScriptManager:setMissionStatus( missionIds.getDepots, MissionStatus.ACCOMPLISHED );
		ScriptManager:wait( 1 );
		
		startGermanCounterattackCutscene();
		
	else
		ScriptManager:displaySubtitle( Text.T_MAP_SU_4_3_MISSION_PRIMARY_01_FLAVOR_01 );
		ScriptManager:wait( 5 );
		HideDialogue();
	end
	
	-- make skipped / not killed units angry
	if depotId == buildingIds.north then
		Unit:changeSquadState( 1, enemySquadIds.northOutpost, AiState.ATTACK );
	
	else
		Unit:changeSquadState( 1, enemySquadIds.southOutpost, AiState.ATTACK )
		
		-- stop the spawning
		spawnAttackWaves = 0;
		ScriptManager:storeValue( "spawnAttackWaves", spawnAttackWaves );
	end
	
	Unit:changeSquadState( 1, enemySquadIds.lake, AiState.ATTACK_AGGRESSIVE );
end

function onFactoryCaptured()
	DisablePlayerInteraction();
	ScriptManager:setMissionStatus( missionIds.getFactory, MissionStatus.ACCOMPLISHED );
	ScriptManager:wait( 1 );
	
	startFactoryCutscene();
	EnablePlayerInteraction();
	
	-- make remaining defenders angry
	Unit:changeSquadState( 1, enemySquadIds.factory, AiState.ATTACK_AGGRESSIVE );
	
	-- remove all units except tanks from the tankfactory
	ScriptManager:removeReinforcementsFromBase( buildingIds.west, Units.UNIT_SU_RIFLE );
	ScriptManager:removeReinforcementsFromBase( buildingIds.west, Units.UNIT_SU_MORTAR );
	ScriptManager:removeReinforcementsFromBase( buildingIds.west, Units.UNIT_SU_AA );
	ScriptManager:removeReinforcementsFromBase( buildingIds.west, Units.UNIT_SU_SUPPLY_TRUCK );
	ScriptManager:removeReinforcementsFromBase( buildingIds.west, Units.UNIT_SU_TRANSPORT_TRUCK );
	ScriptManager:removeReinforcementsFromBase( buildingIds.west, Units.UNIT_SU_KATYUSHA );
	ScriptManager:removeReinforcementsFromBase( buildingIds.west, Units.UNIT_SU_HOWITZER );
end

function startFactoryCutscene()
	StartCutscene();
	
	ScriptManager:moveCameraEx( 3, 3, 0.7, 2, false );
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_4_3_MISSION_SECONDARY_02_FLAVOR_01 );
	ScriptManager:wait( 5 );
	HideDialogue();
	ScriptManager:wait( 0.5 ); -- antirush
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_4_3_MISSION_SECONDARY_02_FLAVOR_02 );
	ScriptManager:wait( 5 );
	HideDialogue();
	ScriptManager:wait( 0.5 ); -- antirush
	
	local unit = 0;
	for i = 1, 2 do
		unit = ScriptManager:spawnUnit( 0, Units.UNIT_SU_TANK_KV1, 3, 3, 0, AiState.WAIT );
		unit:setHealth( unit:getHealth() * 0.55 );
	end
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_4_3_MISSION_SECONDARY_02_FLAVOR_03 );
	ScriptManager:wait( 5 );
	HideDialogue();
	
	EndCutscene();
end

function onAirfieldCaptured()
	finishAirfieldMission();
end

function onEnemyDestroyed( unit )
	local unitId = unit:getId();
	
	-- player destroyed the main objective depots = lose
	if unitId == buildingIds.north or unitId == buildingIds.south then
		missionFailed();
	
	-- player will get secondary objective but not the bonus kv-1 tanks
	elseif unitId == buildingIds.west then
		ScriptManager:setMissionStatus( missionIds.getFactory, MissionStatus.ACCOMPLISHED );
	
	elseif unitId == buildingIds.airfield then
		finishAirfieldMission();
	end
end

function finishAirfieldMission()
	ScriptManager:setMissionStatus( missionIds.getAirfield, MissionStatus.ACCOMPLISHED );
	
	-- stop plane waves
	spawnPlaneWaves = 0;
	ScriptManager:storeValue( "spawnPlaneWaves", spawnPlaneWaves );
	
	RemoveTrigger( "onAirfieldCaptured" );
end

function onPlaneHasAttacked( activeUnit, passiveUnit )
	-- save position to move camera back after the plane mission cutscene
	ScriptManager:storeValue( "cameraMoveBackX", passiveUnit:getX() );
	ScriptManager:storeValue( "cameraMoveBackY", passiveUnit:getY() );
	
	ScriptManager:runForPlayer( 0, "startPlaneMissionCutscene" );
	
	RemoveTrigger( "onPlaneHasAttacked" );
end

function startPlaneMissionCutscene(  )
	StartCutscene();
	
	ScriptManager:moveCameraEx( ScriptManager:getValue( "cameraMoveBackX" ), ScriptManager:getValue( "cameraMoveBackY" ), 0.7, 2, true );
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_4_3_MISSION_SECONDARY_01_FLAVOR_01 );
	ScriptManager:wait( 2 );
	ScriptManager:moveCameraEx( 22, 14, 0.7, 3, true );
	HideDialogue();
	ScriptManager:wait( 0.5 ); -- antirush
	
	-- mark arifield
	ScriptManager:revealFogOfWar( 22, 14, 1, true );
	ScriptManager:showMarkerAt( 22, 14 );	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_4_3_MISSION_SECONDARY_01_FLAVOR_02 );
	ScriptManager:wait( 3 );
	
	HideAllMarkers();
	ScriptManager:revealFogOfWar( 22, 14, 1, false );
	ScriptManager:moveCameraEx( ScriptManager:getValue( "cameraMoveBackX" ), ScriptManager:getValue( "cameraMoveBackY" ), 0.7, 3, true );
	HideDialogue();
	
	EndCutscene();
	DisablePlayerInteraction();
	ScriptManager:wait( 2 );
	
	if  ScriptManager:getMissionStatus( missionIds.getAirfield ) == MissionStatus.NOT_STARTED then
		ScriptManager:setMissionStatus( missionIds.getAirfield,	MissionStatus.IN_PROGRESS );
	end
		
	EnablePlayerInteraction();
end


-- END onEnemyX
-- BEGIN Wave Logic


function onEnemyTurnStart()
	-- spawn logic for plane waves
	if spawnPlaneWaves == 1 then -- should waves spawn? 1 == yes, 0 == no		
		-- check if a wave spawns this round
		if lastPlaneWaveSpawnedInRound + planeWaveRespawnInterval <= ScriptManager:getCurrentRound() then
			numberOfPlaneWavesSpawned = numberOfPlaneWavesSpawned + 1;
			ScriptManager:storeValue( "numberOfPlaneWavesSpawned", numberOfPlaneWavesSpawned );
			
			-- check if current spawn is a bomber spawn
			if numberOfPlaneWavesSpawned % bomberAppearance == 0 then -- spawn Bomber
				spawnWaveUnit( planeAttackWaveUnits[3] );
				
			else -- spawn two fighters
				for i = 1, #planeAttackWaveUnits - 1 do
					spawnWaveUnit( planeAttackWaveUnits[i] );
				end
			end
			
			lastPlaneWaveSpawnedInRound = ScriptManager:getCurrentRound(); -- reset round count
			ScriptManager:storeValue( "lastPlaneWaveSpawnedInRound",	lastPlaneWaveSpawnedInRound );
		end
	end
	
	-- spawn logic for attack waves
	if spawnAttackWaves == 1 then -- should waves spawn? 1 == yes, 0 == no
		-- check if a wave spawns this round
		if lastAttackWaveSpawnedInRound + attackWaveRespawnInterval <= ScriptManager:getCurrentRound() then
			-- find a wave to spawn by random number
			local rnd = math.random( 0, 4 );
			local spawnTable = 0;
			
			if rnd == 0 then
				spawnTable = attackWaveUnits1
			
			elseif rnd == 1 then
				spawnTable = attackWaveUnits2
			
			elseif rnd == 2 then
				spawnTable = attackWaveUnits3
			
			elseif rnd == 3 then
				spawnTable = attackWaveUnits4
			
			else
				spawnTable = attackWaveUnits5
			end
			
			-- spawn the wave
			for i = 1, #spawnTable do
				spawnWaveUnit( spawnTable[i] );
			end
			
			lastAttackWaveSpawnedInRound = ScriptManager:getCurrentRound(); -- reset round count
			ScriptManager:storeValue( "lastAttackWaveSpawnedInRound",	lastAttackWaveSpawnedInRound );
		end
	end
end

function spawnWaveUnit( spawnData )
	local unit = ScriptManager:spawnUnit( spawnData[1], spawnData[2], spawnData[3], spawnData[4], spawnData[5], spawnData[6] );
	
	return unit; -- i want to save the id for the reinforcements so i have to return the unit ( spawnGermanAttackers for detail )
end


-- END Wave Logic
-- BEGIN German Counterattack


function startGermanCounterattackCutscene()
	StartCutscene();
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_4_3_MISSION_PRIMARY_01_FLAVOR_02 );
	ScriptManager:wait( 3 );
	HideDialogue();
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_4_3_MISSION_PRIMARY_01_FLAVOR_03 );
	ScriptManager:wait( 5 );
	HideDialogue();
	
	ScriptManager:moveCameraEx( 20, 2, 0.7, 2, true );
		
	ScriptManager:displaySubtitle( Text.T_MAP_SU_4_3_MISSION_PRIMARY_01_FLAVOR_04 );
	ScriptManager:wait( 5 );
	HideDialogue();
	
	EndCutscene();
	
	ScriptManager:setMissionStatus( missionIds.defeatDefenders,	MissionStatus.IN_PROGRESS );
	ScriptManager:setMissionStatus( missionIds.repelAttackers,	MissionStatus.IN_PROGRESS );
	
	EnablePlayerInteraction();
	
	reinforcementSpawnInTurn = ScriptManager:getCurrentRound() + 3;
	ScriptManager:storeValue( "reinforcementSpawnInTurn", reinforcementSpawnInTurn );
	angerTheHorde();
	ScriptManager:addOnTurnStartTrigger(	 1, -1, -1, "spawnGermanAttackers",		"spawnGermanAttackers",		false );
	ScriptManager:addOnUnitDestroyedTrigger( 1, -1, -1, "onReinforcementDestroyed", "onReinforcementDestroyed", false );
end

function spawnGermanAttackers()
	local spawnTable = 0;
	
	if ScriptManager:getCurrentRound() == reinforcementSpawnInTurn then
		spawnTable = reinforcements1;
		print( "reinforcements1" );
	
	elseif ScriptManager:getCurrentRound() == ( reinforcementSpawnInTurn + 1 ) then
		spawnTable = reinforcements2;
		print( "reinforcements2" );
	
	elseif ScriptManager:getCurrentRound() == ( reinforcementSpawnInTurn + 2 ) then
		spawnTable = reinforcements3;
		print( "reinforcements3" );
		RemoveTrigger( "spawnGermanAttackers" );
		
	else
		ScriptManager:runForPlayer( 0, "getReady" );
	end
	
	-- spawn the wave
	if spawnTable ~= 0 then
		local unit = 0;
		for i = 1, #spawnTable do
			local unit = spawnWaveUnit( spawnTable[i] );
			local unitId = unit:getId();
			table.insert( reinforcementIds, unitId );
			ScriptManager:storeValue( string.format( "reinforcementIds %i", reinforcementCounter ), unitId );
			
			reinforcementCounter = reinforcementCounter + 1;
			ScriptManager:storeValue( "reinforcementCounter", reinforcementCounter );
		end
		
		ScriptManager:displaySubtitle( Text.T_MAP_3_2_ENEMY_REINFORCEMENTS );
		ScriptManager:wait( 4 );
		HideDialogue();
	end
end

function getReady()
	print( reinforcementSpawnInTurn );
	print( "getReady" );
	ScriptManager:displaySubtitle( Text.T_MAP_SU_4_3_MISSION_PRIMARY_01_FLAVOR_05 );
	ScriptManager:wait( 4 );
	HideDialogue();
end

function angerTheHorde()
	-- add all remaining defenders to the reinforcements and make them aggessive
	defCounter = 1;
	addUnitToRemainingDefenders( northOutpostIds );
	addUnitToRemainingDefenders( southOutpostIds );
	addUnitToRemainingDefenders( lakeIds );
	
	-- immediatelly accomplish mission if all defenders are already dead
	if #remainingDefenders == 0 then
		ScriptManager:setMissionStatus( missionIds.defeatDefenders, MissionStatus.ACCOMPLISHED );
	end
	
	print( #remainingDefenders );
	
	Unit:changeSquadState( 1, enemySquadIds.northOutpost,	AiState.ATTACK_AGGRESSIVE );
	Unit:changeSquadState( 1, enemySquadIds.southOutpost,	AiState.ATTACK_AGGRESSIVE );
	Unit:changeSquadState( 1, enemySquadIds.lake, 			AiState.ATTACK_AGGRESSIVE );
end

function addUnitToRemainingDefenders( squadTable )
	local unit;
	for i = 1, #squadTable do
		unit = Unit:getUnitById( squadTable[i] );
		
		-- add it to reinforcement if it exists
		if unit ~= nil then
			-- do not add mg nests
			if unit:getTemplateId() ~= Units.UNIT_DE_MG_NEST then
				table.insert( remainingDefenders, squadTable[i] );
				ScriptManager:storeValue( string.format( "remainingDefenders %i", defCounter ), squadTable[i] );
							
				print( string.format( "unit %i saved in slot %i", squadTable[i], defCounter ) );
				defCounter = defCounter + 1;
			end
		end
	end
end

function onReinforcementDestroyed( unit )
	local unitId = unit:getId();
	local unitFound = 0; -- 0 = not found, 1 == found in first ( reinforcement ) table, 2 = found in second ( defenders ) table
	
	
	-- check if a reinforcement unit was killed and remove it from the table
	for i = 1, #reinforcementIds do
		if unitId == reinforcementIds[i] then
			table.remove( reinforcementIds, i );
			ScriptManager:deleteValue( string.format( "reinforcementIds %i", i ) );
			print( string.format( "unit %i found and removed from reinforcements", unitId ) );
			unitFound = 1;
			break; -- stop searching if unit was found
		end
	end
	-- search remaining defenders if unit has not been found yet
	if unitFound == 0 then
		for i = 1, #remainingDefenders do
			if unitId == remainingDefenders[i] then
				table.remove( remainingDefenders, i );
				print( string.format( "unit %i found and removed from defenders", unitId ) );
				unitFound = 2;
				break;
			
			else
				print( string.format( "unit %i not found", unitId ) );
			end
		end
	end
	
	-- delete the killed unit from the savefile to not load it again
	if unitFound == 1 then
		for i = 1, maxNumberOfReinforcements do
			if ScriptManager:getValue( string.format( "reinforcementIds %i", i ) ) == unitId then
				ScriptManager:deleteValue( string.format( "reinforcementIds %i", i ) );
				print( "unit deleted" );
				break;
			end
		end
	
	-- delete the killed unit from the savefile to not load it again
	elseif unitFound == 2 then
		for i = 1, maxNumberOfDef do
			if ScriptManager:getValue( string.format( "remainingDefenders %i", i ) ) == unitId then
				ScriptManager:deleteValue( string.format( "remainingDefenders %i", i ) );
				print( "unit deleted" );
				break;
			end
		end
	end
	
	print( #remainingDefenders );
	
	-- check if defender mission is done
	if  next( remainingDefenders ) == nil and ScriptManager:getMissionStatus( missionIds.defeatDefenders ) == MissionStatus.IN_PROGRESS then
		ScriptManager:setMissionStatus( missionIds.defeatDefenders, MissionStatus.ACCOMPLISHED );
	end
	
	-- reinforcements spawn for three turns. so the earliest time of victory would be the fourth round
	if ScriptManager:getCurrentRound() >= ( reinforcementSpawnInTurn + 3 ) then
		-- check if reinforcement mission is done
		if next( reinforcementIds ) == nil then
			if ScriptManager:getMissionStatus( missionIds.repelAttackers ) ~= MissionStatus.ACCOMPLISHED then
				ScriptManager:setMissionStatus( missionIds.repelAttackers, MissionStatus.ACCOMPLISHED );
			end
		end
		-- check if both are done = vicory
		if next( reinforcementIds ) == nil and next( remainingDefenders ) == nil then
			DisablePlayerInteraction();
			ScriptManager:runForPlayer( 0, "startVictoryCutscene" );
		end
	end
end


-- END German Counterattack
-- BEGIN Victory


function startVictoryCutscene()
	StartCutscene();
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_4_3_MISSION_VICTORY_FLAVOR_01 );
	ScriptManager:wait( 5 );
	HideDialogue();
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_4_3_MISSION_VICTORY_FLAVOR_02 );
	ScriptManager:wait( 5 );
	HideDialogue();
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_4_3_MISSION_VICTORY_FLAVOR_03 );
	ScriptManager:wait( 5 );
	HideDialogue();
	
	EndCutscene();
	
	victoryEndGame();
end

function victoryEndGame()
	DisablePlayerInteraction();
	HideDialogue(); -- remove possible texts on the screens
	
	ScriptManager:displaySubtitle( Text.T_MISSION_VICTORY );
	ScriptManager:wait( 4 );
	HideDialogue();
	endGame();
end


-- END Victory
-- BEGIN Defeated


function missionFailedTransition()
	-- fail mission in player turn
	ScriptManager:runForPlayer( 0, "missionFailed" );
end

function missionFailed()
	DisablePlayerInteraction();
	HideDialogue(); -- remove possible texts on the screens
	
	ScriptManager:displaySubtitle( Text.T_MISSION_FAILED );
	ScriptManager:wait( 4 );
	HideDialogue();
	endGame();
end


-- END Defeated


function endGame()
	ScriptManager:endGame()
end