require "lua/Constants"
require "lua/Functions"
require "lua/Functions_2"
require "lua/Text"
require "lua/Units"
require "math"


-- mission stuff
missionIds = {
	captureDepot		= 1,	-- capture the depot to get new units
	captureTankfactory	= 2,	-- capture the tank factory to gain access to heavy units
	bringTanks			= 3,	-- bring tanks to the north-eastern junction
	defeatAttackers		= 4,	-- defeat the attack squad from the mountains
};


attackersToDefeat = 6;


-- missionId, 						primary( boolean ), textkey, 							xpReward
missions = {
	{ missionIds.captureDepot,			true,	Text.T_MAP_SU_5_2_MISSION_SECONDARY_01,		100	},
	{ missionIds.captureTankfactory, 	true, 	Text.T_MAP_SU_5_2_MISSION_PRIMARY_01,		175 },
	{ missionIds.bringTanks, 			true, 	Text.T_MAP_SU_5_2_MISSION_PRIMARY_02,		200 },
	{ missionIds.defeatAttackers, 		false, 	Text.T_MAP_SU_5_2_MISSION_SECONDARY_02,		 75 },	
};


buildingIds = {
	depot 		= 0,
	HQ 			= 1,
	airfield	= 3,
	tankFactory	= 4,	
};


enemySquadIds = {

	firstBridge 	=  1,
	depotNorth		=  2,
	airfield		=  3,
	mountains		=  4,
	tankFactory		=  5,
	HQ				=  6,
	planeWaveUnits	=  7,
	HQwaveUnits		=  8,
	ambushUnits		=  9,
	southBridge		= 10,
	
};


firstBridgeIds		= { 20, 22 };
depotNorthIds 		= { 21, 23, 24, 25 };
airfieldIds			= { 27, 28, 29, 30, 50 };
mountainsIds		= { 32, 33, 34 };
tankFactoryIds		= { 31, 35, 36, 37, 51, 52 };
HQIds				= { 38, 39, 40, 42, 43, 45, 47, 48, 49 };
ambushIds			= { }

-- Squad from the mountains when the factory is almost reached
ambushTriggertiles = {

	{ 4, 6 },
	{ 4, 3 },
	{ 5, 2 },
	{ 4, 1 },
};

-- When a tank reaches the HQ
tankHQTriggertiles = {

	{ 23, 11 },
	{ 24, 11 },
	{ 25, 11 },
	{ 26, 11 },
};

-- starts the enemy wave from the HQ

waveHQTriggertiles = {
	{ 17, 5 },
	{ 18, 5 },
	{ 19, 5 },
	
	{ 14, 1 },
	{ 15, 2 },
	{ 15, 3 },
	
	{ 21, 7 },
	{ 22, 7 },
	{ 22, 8 },
	{ 23, 8 },
	
	{ 23, 13 },
	{ 23 ,14 },
	{ 23, 15 },
};





-- waves that spawn

-- I spawn either one destroyer or one Bomber depending on the number of spawns and the round count
-- ( every x turns I spawn a plane wave (Destroyer) and every y turns the bomber spawns instead of fighters )
spawnPlaneWaves				=  1; -- 0 == false; 1 == true; can't save booleans so I have to use numbers
planeWaveRespawnInterval	=  4; -- defines how often waves will spawn
bomberAppearance			=  2; -- every bomberAppearance-spawn a bomber appears instead of a Destroyer
numberOfPlaneWavesSpawned	=  0; -- the number of plane waves that have already spawned in this game
lastPlaneWaveSpawnedInRound	= -3; -- indicates the round in which the last wave spawned 
planeWaveUnits = {
--	playerId,		 templateID, 			tileX, 		tileY,		squadID			AiState
	{ 1,			Units.UNIT_DE_DESTROYER,	18,			2,		  	7,			AiState.AIRPLANE },
	{ 1,			Units.UNIT_DE_BOMBER,		18,			3,			7,			AiState.AIRPLANE },		
}



-- I spawn either one MP40 or one Anti_Infantry depending on the number of spawns and the round count
-- ( every x turns an Infantry wave spawns, every y turns an Anti-Infantry wave spawns )
spawnAttackWaves					=  0; -- 0 == false; 1 == true; can't save booleans so I have to use numbers
AttackWaveRespawnInterval			=  3; -- defines how often waves will spawn
AntiInfantryAppearance				=  2; -- every AntiInfantryAppearance-spawn an AntiInfantry appears instead of a MP40
numberOfAttackWavesSpawned			=  0; -- the number of attack waves that have already spawned in this game
lastAttackWaveSpawnedInRound		= -1; -- indicates the round in which the last wave spawned 
HQwaveUnits = {
--	playerId,		 templateID, 				tileX, 		tileY,	squadID  		AiState
	{ 1,			Units.UNIT_DE_MP40,				23,			11,		8,		    AiState.ATTACK_AGGRESSIVE },
	{ 1,			Units.UNIT_DE_ANTI_INFANTRY,	24,			11,		8,   		AiState.ATTACK_AGGRESSIVE },
}	



-- BEGIN Init


function onGameStart()
	CreateMissions( missions );
	setTriggers();
	storeValues();
	
	ScriptManager:gainGold( 0, 1500 );
	
	-- start intro cutscene
	ScriptManager:sleep( 0, "startIntroCutscene" ); -- without sleep player turn pop up shows
end



function storeValues()
	
	ScriptManager:storeValue( "attackersToDefeat",				attackersToDefeat);
	
	ScriptManager:storeValue( "spawnPlaneWaves",				spawnPlaneWaves );
	ScriptManager:storeValue( "numberOfPlaneWavesSpawned",		numberOfPlaneWavesSpawned );
	ScriptManager:storeValue( "lastPlaneWaveSpawnedInRound",	lastPlaneWaveSpawnedInRound );
	
	ScriptManager:storeValue( "spawnAttackWaves", 				spawnAttackWaves);
	ScriptManager:storeValue( "numberOfAttackWavesSpawned",		numberOfAttackWavesSpawned);
	ScriptManager:storeValue( "lastAttackWaveSpawnedInRound",	lastAttackWaveSpawnedInRound);
	
	
end

function setTriggers()
	
	-- load all needed values after loading
	ScriptManager:addOnGameResumeTrigger( "loadValues", "loadValues", false );
	
	
	-- check if the player has units left
	ScriptManager:addOnUnitDestroyedTrigger( 0, -1, -1, "checkForEnd", "checkForEnd" , false );


	-- when a German Anti-Infantry has been spotted for the first time
	ScriptManager:addOnUnitSightedTrigger( 1, Units.UNIT_DE_ANTI_INFANTRY, -1, true, "onAntiInfantrySighted","onAntiInfantrySighted", false );

	-- player captured the depot
	ScriptManager:addOnUnitCapturedTrigger( 0, Units.UNIT_DE_HQ, 0, "onDepotCaptured", "onDepotCaptured", false );
	
	-- player destroyed the depot
	ScriptManager:addOnUnitDestroyedTrigger( 1, Units.UNIT_DE_HQ, 0, "onDepotdestroyed", "onDepotdestroyed", false );
	
	
	-- player captured the tank-factory
	ScriptManager:addOnUnitCapturedTrigger( 0, Units.UNIT_TANKFACTORY_STANDARD, -1, "onTankfactoryCaptured", "onTankfactoryCaptured", false );
	
	-- player destroyed the tank factory
	ScriptManager:addOnUnitDestroyedTrigger( 1, Units.UNIT_TANKFACTORY_STANDARD, -1, "onTankfactoryDestroyed","onTankfactoryDestroyed", false );
	
	-- trigger for attacking tank factory
	ScriptManager:addOnUnitHasAttackedTrigger( 0, Units.UNIT_TANKFACTORY_STANDARD, -1, "ProtectTheBuildingHintSingle", "ProtectTheBuildingHintSingle", false );
	
	-- when the airfield is captured/destroyed, the spawning of planes stops
	ScriptManager:addOnUnitCapturedTrigger(  0, Units.UNIT_DE_AIRFIELD, -1, "onAirfieldCaptured" , "onAirfieldCaptured", false );
	ScriptManager:addOnUnitDestroyedTrigger( 1, Units.UNIT_DE_AIRFIELD, -1, "onAirfieldDestroyed" , "onAirfieldDestroyed", false );
	
	
	-- when the HQ is captured, the spawning of units stops
	ScriptManager:addOnUnitCapturedTrigger(  0, Units.UNIT_DE_HQ, -1, "onHQcaptured" , "onHQcaptured", false );
	ScriptManager:addOnUnitDestroyedTrigger( 1, Units.UNIT_DE_HQ, -1, "onHQDestroyed" , "onHQDestroyed", false );	
	
	
	-- when a tank enters the tile, 2 Infantry Panzerfaust spawn
	ScriptManager:addOnEnterTileTrigger( 14, 2, 0, Units.UNIT_SU_TANK_DESTROYER, -1, true, "onTankonBridge", "onTankonBridge", false );
	ScriptManager:addOnEnterTileTrigger( 14, 2, 0, Units.UNIT_SU_TANK_KV1,		 -1, true, "onTankonBridge", "onTankonBridge", false );
	ScriptManager:addOnEnterTileTrigger( 14, 2, 0, Units.UNIT_SU_TANK_T34,		 -1, true, "onTankonBridge", "onTankonBridge", false );
		
		
	-- when one of the three types of tanks reaches the triggerline in the northeast	
	ScriptManager:addOnEnterTileTrigger( 23, 11, 0, Units.UNIT_SU_TANK_T34,			-1, true, "onTankArrived", "onTankArrived", false );
	ScriptManager:addOnEnterTileTrigger( 24, 11, 0, Units.UNIT_SU_TANK_T34,			-1, true, "onTankArrived", "onTankArrived", false );
	ScriptManager:addOnEnterTileTrigger( 25, 11, 0, Units.UNIT_SU_TANK_T34,			-1, true, "onTankArrived", "onTankArrived", false );
	ScriptManager:addOnEnterTileTrigger( 26, 11, 0, Units.UNIT_SU_TANK_T34,			-1, true, "onTankArrived", "onTankArrived", false );	
	
	ScriptManager:addOnEnterTileTrigger( 23, 11, 0, Units.UNIT_SU_TANK_KV1,			-1, true, "onTankArrived", "onTankArrived", false );
	ScriptManager:addOnEnterTileTrigger( 24, 11, 0, Units.UNIT_SU_TANK_KV1,			-1, true, "onTankArrived", "onTankArrived", false );
	ScriptManager:addOnEnterTileTrigger( 25, 11, 0, Units.UNIT_SU_TANK_KV1,			-1, true, "onTankArrived", "onTankArrived", false );
	ScriptManager:addOnEnterTileTrigger( 26, 11, 0, Units.UNIT_SU_TANK_KV1,			-1, true, "onTankArrived", "onTankArrived", false );	
	
	ScriptManager:addOnEnterTileTrigger( 23, 11, 0, Units.UNIT_SU_TANK_DESTROYER,	-1, true, "onTankArrived", "onTankArrived", false );
	ScriptManager:addOnEnterTileTrigger( 24, 11, 0, Units.UNIT_SU_TANK_DESTROYER,	-1, true, "onTankArrived", "onTankArrived", false );
	ScriptManager:addOnEnterTileTrigger( 25, 11, 0, Units.UNIT_SU_TANK_DESTROYER,	-1, true, "onTankArrived", "onTankArrived", false );
	ScriptManager:addOnEnterTileTrigger( 26, 11, 0, Units.UNIT_SU_TANK_DESTROYER,	-1, true, "onTankArrived", "onTankArrived", false );	
	
	-- starts the wavespawn from the german HQ
	ScriptManager:addOnEnterTileTrigger( 18, 6, 0, -1, -1, false, "onWaveHQspawned", "onWaveHQspawned", false );
	ScriptManager:addOnEnterTileTrigger( 14, 2, 0, -1, -1, false, "onWaveHQspawned", "onWaveHQspawned", false );
	ScriptManager:addOnEnterTileTrigger( 22, 8, 0, -1, -1, false, "onWaveHQspawned", "onWaveHQspawned", false );
	ScriptManager:addOnEnterTileTrigger( 23, 14, 0, -1, -1, false, "onWaveHQspawned", "onWaveHQspawned", false );
	
	-- starts the ambush from the southern road in the mountains
	ScriptManager:addOnEnterTileTrigger( 5, 6, 0, -1, -1, true, "onAmbushSpawned", "onAmbushSpawned", false );
	ScriptManager:addOnEnterTileTrigger( 4, 5, 0, -1, -1, true, "onAmbushSpawned", "onAmbushSpawned", false );
	ScriptManager:addOnEnterTileTrigger( 5, 4, 0, -1, -1, true, "onAmbushSpawned", "onAmbushSpawned", false );
	ScriptManager:addOnEnterTileTrigger( 4, 3, 0, -1, -1, true, "onAmbushSpawned", "onAmbushSpawned", false );
	ScriptManager:addOnEnterTileTrigger( 5, 2, 0, -1, -1, true, "onAmbushSpawned", "onAmbushSpawned", false );
	ScriptManager:addOnEnterTileTrigger( 4, 1, 0, -1, -1, true, "onAmbushSpawned", "onAmbushSpawned", false );
	for i = 1, 4 do
		ScriptManager:addOnEnterTileTrigger( i, 7, 0, -1, -1, true, "onAmbushSpawned", "onAmbushSpawned", false );
	end
	
	-- enemy attack-squad has been defeated
	ScriptManager:addOnUnitDestroyedTrigger( 1, Units.UNIT_DE_MP40, 			-1, 	"onAttackerDefeated", "onAttackerDefeated", false );
	ScriptManager:addOnUnitDestroyedTrigger( 1, Units.UNIT_DE_PANZERFAUST, 		-1,		"onAttackerDefeated", "onAttackerDefeated", false );
	ScriptManager:addOnUnitDestroyedTrigger( 1, Units.UNIT_DE_ANTI_INFANTRY,	-1, 	"onAttackerDefeated", "onAttackerDefeated", false );

	-- trigger for wave spawns
	ScriptManager:addOnTurnStartTrigger( 1, -1, -1, "onEnemyTurnStart", "onEnemyTurnStart", false );
	
		
end


function loadValues()

	attackersToDefeat				= ScriptManager:getValue( "attackersToDefeat" );
	
	spawnPlaneWaves					= ScriptManager:getValue( "spawnPlaneWaves" );
	numberOfPlaneWavesSpawned		= ScriptManager:getValue( "numberOfPlaneWavesSpawned" );
	lastPlaneWaveSpawnedInRound		= ScriptManager:getValue( "lastPlaneWaveSpawnedInRound" );
	
	spawnAttackWaves				= ScriptManager:getValue( "spawnAttackWaves" );
	numberOfAttackWavesSpawned		= ScriptManager:getValue( "numberOfAttackWavesSpawned" );
	lastAttackWaveSpawnedInRound	= ScriptManager:getValue( "lastAttackWaveSpawnedInRound" );
	
end


-- END Init
-- BEGIN Intro Cutscene



function startIntroCutscene()
		
	StartCutscene();
	
	-- a scout (MP40) leaves the picture into the FoW (returns when the depot is captured)
	

	ScriptManager:displaySubtitle( Text.T_MAP_SU_5_2_MISSION_INTRO_FLAVOR_01 );
	ScriptManager:wait( 8 )
	ScriptManager:displaySubtitle( Text.T_MAP_SU_5_2_MISSION_INTRO_FLAVOR_02 );
	
	ScriptManager:moveCameraEx( 14, 13, 1, 3, true );
	
	ScriptManager:revealFogOfWar( 14, 13, 1, true );
	ScriptManager:showMarkerAt( 14, 13 );
	ScriptManager:wait( 4 );
		
	ScriptManager:moveCameraEx( 2, 14, 0.7, 3, true );
	
	ScriptManager:revealFogOfWar( 14, 13, 1, false );
	ScriptManager:removeAllMarkers();
	ScriptManager:dismissSubtitle();
	ScriptManager:wait( 1 );
	ScriptManager:displaySubtitle( Text.T_MAP_SU_5_2_MISSION_INTRO_FLAVOR_03 );
	ScriptManager:wait( 4 );
	ScriptManager:dismissSubtitle();
	
	ScriptManager:moveUnitEx( 19, 11, 15, true );
	
	ScriptManager:removeUnitEx( 19, true );
	
	ScriptManager:wait( 2 );
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_5_2_MISSION_INTRO_FLAVOR_04 );
	ScriptManager:showMarkerAt( 2, 14 );
	ScriptManager:showMarkerAt( 2, 13 ); 
	ScriptManager:wait( 6 );
	ScriptManager:dismissSubtitle();
	
	ScriptManager:removeAllMarkers();
	
	ScriptManager:moveUnitEx( 24 , 14, 12, false );
	
	EndCutscene();
	
	removeFromHQ()
	
	ScriptManager:fortifyUnit( 27 );
	ScriptManager:fortifyUnit( 55 );
	ScriptManager:fortifyUnit( 56 );
	ScriptManager:fortifyUnit( 44 );
	ScriptManager:camouflageUnit( 44 );
	ScriptManager:fortifyUnit( 57 );
	
	
	
	startTheMission();
end



function removeFromHQ() -- remove all tanks from the northeastern HQ 
	ScriptManager:removeReinforcementsFromBase( 1, Units.UNIT_SU_TANK_DESTROYER );
	ScriptManager:removeReinforcementsFromBase( 1, Units.UNIT_SU_TANK_KV1 		);
	ScriptManager:removeReinforcementsFromBase( 1, Units.UNIT_SU_TANK_T34 		);
		
end


function startTheMission()
	ScriptManager:setMissionStatus( missionIds.captureDepot , MissionStatus.IN_PROGRESS );
	
	
	--TODO: remove this function!
	--testing();
	--removeSquads();
	--ScriptManager:addOnUnitSelectedTrigger( 1, -1, -1, "showParameters", "showParameters",false)
end

function testing()
	ScriptManager:spawnUnit( 0, Units.GOTT, 27, 5, 0, AiState.WAIT );
	ScriptManager:spawnUnit( 0, Units.GOTT, 27, 7, 0, AiState.WAIT );
	ScriptManager:spawnUnit( 0, Units.UNIT_DE_BOMBER, 5, 4, 0, AiState.WAIT );
end

function removeSquads()
	
	for i = 1, #firstBridgeIds do
	ScriptManager:removeUnitEx( firstBridgeIds[i], false );
	end

	for i = 1, #depotNorthIds do
	ScriptManager:removeUnitEx( depotNorthIds[i], false);
	end
	
	for i = 1, #airfieldIds do
	ScriptManager:removeUnitEx( airfieldIds[i], false );
	end
	
	for i = 1, #mountainsIds do
	ScriptManager:removeUnitEx( mountainsIds[i], false );
	end
	
	for i = 1, #tankFactoryIds do
	ScriptManager:removeUnitEx( tankFactoryIds[i], false );
	end
	
	for i = 1, #HQIds do
	ScriptManager:removeUnitEx( HQIds[i], false );
	end	
		
end



function showParameters( selectedUnit )
	--print( string.format( "[] %s", selectedUnit:get() ) );
end


-- END Intro Cutscene
-- BEGIN onEnemyX



function onAirfieldCaptured()
	spawnPlaneWaves = 0;
	ScriptManager:storeValue( "spawnPlaneWaves", spawnPlaneWaves );
end


function onAirfieldDestroyed()
	spawnPlaneWaves = 0;
	ScriptManager:storeValue( "spawnPlaneWaves", spawnPlaneWaves );
end


function onHQcaptured()
	spawnAttackWaves = 0;
	ScriptManager:storeValue( "spawnAttackWaves", spawnAttackWaves );
end

function onHQDestroyed()
	spawnAttackWaves = 0;	
	ScriptManager:storeValue( "spawnAttackWaves", spawnAttackWaves );
end



function onAntiInfantrySighted()
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_5_2_MISSION_INTRO_FLAVOR_05 );
	ScriptManager:wait(6);
	ScriptManager:dismissSubtitle();
	ScriptManager:removeTrigger( "onAntiInfantrySighted" );
end


function onDepotdestroyed()
	
	ScriptManager:setMissionStatus( missionIds.captureDepot, MissionStatus.FAILED );
	missionFailed();

end



function onDepotCaptured()

	ScriptManager:setMissionStatus( missionIds.captureDepot, MissionStatus.ACCOMPLISHED );
	
	ScriptManager:spawnUnit( 1, Units.UNIT_DE_PANZERFAUST, 	28, 12, -1, AiState.DEFEND );
	ScriptManager:spawnUnit( 1, Units.UNIT_DE_MP40, 		27, 12, -1, AiState.DEFEND );
	
	ScriptManager:spawnUnit( 1, Units.UNIT_DE_MP40, 26, 12, 60, AiState.MOVE_AGGRESSIVE );
	ScriptManager:setTargetTile( 1, 60, 17, 13 );
	
	
	
	if ScriptManager:getMissionStatus(missionIds.captureTankfactory) == MissionStatus.NOT_STARTED then
	
		StartCutscene(); -- the scout returns, mission: capture the tank factory, bring tanks to the HQ in the Northeast
	
		ScriptManager:displaySubtitle( Text.T_MAP_SU_5_2_MISSION_PRIMARY_01_FLAVOR_01 );
		ScriptManager:wait( 5 );
	
		local unit = ScriptManager:spawnUnit( 0, Units.UNIT_SU_RIFLE, 13, 3, -1, AiState.WAIT );
		ScriptManager:moveUnitEx( unit:getId(), 15, 14, true );
		ScriptManager:wait( 1 );
	
		ScriptManager:displaySubtitle( Text.T_MAP_SU_5_2_MISSION_PRIMARY_01_FLAVOR_02 );
		ScriptManager:wait( 6 );
	
		ScriptManager:moveCameraEx( 3, 2, 0.7, 2, true );
	
		ScriptManager:revealFogOfWar( 3, 2, 1, true);
		ScriptManager:showMarkerAt( 3, 2 );
	
		ScriptManager:wait( 3 );
	
		ScriptManager:moveCameraEx( 14, 13, 0.7, 2, true );
	
		ScriptManager:revealFogOfWar( 3, 2, 1,false );
		ScriptManager:removeAllMarkers();
	
		ScriptManager:wait( 2 );
	
		ScriptManager:dismissSubtitle();
	
		ScriptManager:displaySubtitle( Text.T_MAP_SU_5_2_MISSION_PRIMARY_01_FLAVOR_03 );
		ScriptManager:wait( 5 );
		ScriptManager:dismissSubtitle();
			
		EndCutscene();

		ScriptManager:setMissionStatus( missionIds.captureTankfactory, MissionStatus.IN_PROGRESS);
	

	--else 
		--ScriptManager:setMissionStatus( missionIds.captureTankfactory, MissionStatus.ACCOMPLISHED);
	
	
	end	
	removeFromDepot();
	
	checkForVictory();

end


function removeFromDepot() -- remove all tanks from the newly spawned depot -->
	ScriptManager:removeReinforcementsFromBase( 0, Units.UNIT_SU_TANK_DESTROYER );
	ScriptManager:removeReinforcementsFromBase( 0, Units.UNIT_SU_TANK_KV1 		);
	ScriptManager:removeReinforcementsFromBase( 0, Units.UNIT_SU_TANK_T34 		);
		
end


function onAmbushSpawned(tileX, tileY, playerId, unitTemplateId, unitId)

	StartCutscene(); -- an attack squad emerges from the southern road in the mountains, mission: defeat the attackers
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_5_2_MISSION_SECONDARY_02_FLAVOR_01 );
	
	ScriptManager:moveCameraEx( 11, 2, 0.7, 2, true );
	
	ScriptManager:spawnUnits( 1, Units.UNIT_DE_MP40, 			11, 1, 9, AiState.ATTACK_AGGRESSIVE, 3 );	
	ScriptManager:spawnUnits( 1, Units.UNIT_DE_ANTI_INFANTRY, 	11, 2, 9, AiState.ATTACK_AGGRESSIVE, 2 );
	ScriptManager:spawnUnit(  1, Units.UNIT_DE_PANZERFAUST, 	10, 3, 9, AiState.ATTACK_AGGRESSIVE );
		
	ScriptManager:revealFogOfWar( 11, 2, 3,true );
	ScriptManager:wait( 4 );
	ScriptManager:revealFogOfWar( 11, 2, 3, false );
	ScriptManager:dismissSubtitle();
	
	ScriptManager:moveCameraEx( 4, 6, 0.7, 2, true );
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_5_2_MISSION_SECONDARY_02_FLAVOR_02 );
	ScriptManager:wait( 4 );
	ScriptManager:dismissSubtitle();
	
	EndCutscene();
	
	ScriptManager:removeTrigger( "onAmbushSpawned" );

	ScriptManager:setMissionStatus( missionIds.defeatAttackers, MissionStatus.IN_PROGRESS );	

end


function onAttackerDefeated( unit )
	local squadId = unit:getSquadId();

	
	if squadId == enemySquadIds.ambushUnits then
		ScriptManager:setMissionProgress(missionIds.defeatAttackers, ScriptManager:getMissionProgress( missionIds.defeatAttackers ) + 1 );
		ScriptManager:runForPlayer( 0, "attackersDefeatedProgress" );

	end
end

function attackersDefeatedProgress()
	if  ScriptManager:getMissionProgress( missionIds.defeatAttackers ) >= attackersToDefeat then
		DisablePlayerInteraction();
	
		ScriptManager:setMissionStatus( missionIds.defeatAttackers, MissionStatus.ACCOMPLISHED );
		ScriptManager:removeTrigger( "onAttackerDefeated" );
		
		ScriptManager:displaySubtitle( Text.T_MAP_SU_5_2_MISSION_SECONDARY_02_FLAVOR_03 );
		ScriptManager:wait( 4 );
		ScriptManager:dismissSubtitle();
		
		EnablePlayerInteraction();
	
	
	else
		ShowMissionProgressPopup( ScriptManager:getMissionProgress( missionIds.defeatAttackers ), Text.T_MAP_SU_5_2_MISSION_SECONDARY_02_COUNTER );
	end
end


function onTankfactoryCaptured()

	ScriptManager:setMissionStatus( missionIds.captureTankfactory, MissionStatus.ACCOMPLISHED );
	tankFactoryCutscene()

end	
	
	
function onTankfactoryDestroyed()

	ScriptManager:setMissionStatus( missionIds.captureTankfactory, MissionStatus.FAILED );
	missionFailed();

end

function tankFactoryCutscene()	
	StartCutscene(); -- mission: bring (x) tanks to the northeastern junction
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_5_2_MISSION_PRIMARY_02_FLAVOR_01 );
	
	ScriptManager:wait( 1 );
	
	ScriptManager:spawnUnit( 0, Units.UNIT_SU_TANK_KV1, 2, 3, -1, AiState.WAIT );
	ScriptManager:spawnUnit( 0, Units.UNIT_SU_TANK_T34, 2, 2, -1, AiState.WAIT );
	
	ScriptManager:wait( 1 );
	
	ScriptManager:wait( 4 );
	ScriptManager:dismissSubtitle();
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_5_2_MISSION_PRIMARY_02_FLAVOR_02 );
	ScriptManager:wait( 6 );
	ScriptManager:displaySubtitle( Text.T_MAP_SU_5_2_MISSION_PRIMARY_02_FLAVOR_03 );
	
	ScriptManager:moveCameraEx( 24, 10, 0.7, 4, true );
	
	ScriptManager:revealFogOfWar( 24, 10, 1, true );
	ScriptManager:wait(4)
	
	ScriptManager:moveCameraEx( 3, 2, 0.7, 3, true);
	
	ScriptManager:revealFogOfWar( 24, 10, 1, false );
	ScriptManager:wait( 1 )
	
	ScriptManager:dismissSubtitle();
	
	
	EndCutscene();
	
	ScriptManager:wait( 1 );
	
	ScriptManager:gainGold( 0, 600 );


	checkForVictory();
	
	ScriptManager:setMissionStatus( missionIds.bringTanks, MissionStatus.IN_PROGRESS );

end


function onTankonBridge()
	
	StartCutscene()		
	
	ScriptManager:moveCameraEx( 21, 1, 1, 2, true );
	ScriptManager:revealFogOfWar( 21, 1, 2, true );
	ScriptManager:wait( 1 );
	
	ScriptManager:spawnUnit( 1, Units.UNIT_DE_PANZERFAUST, 21, 1, 66, AiState.MOVE_AGGRESSIVE );
	ScriptManager:spawnUnit( 1, Units.UNIT_DE_PANZERFAUST, 21, 2, 66, AiState.MOVE_AGGRESSIVE );
	ScriptManager:wait( 2 ); 
	ScriptManager:setTargetTile( 1, 66, 14, 2 );
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_5_2_MISSION_PRIMARY_02_FLAVOR_04 );
	ScriptManager:wait( 2 );
	ScriptManager:moveCameraEx( 14, 2, 0.7, 2, true );
	ScriptManager:revealFogOfWar( 21, 1, 2, false );
	ScriptManager:wait( 3 );
	
	ScriptManager:dismissSubtitle();
	
	EndCutscene();
	
	ScriptManager:removeTrigger( "onTankonBridge" );
end


function onTankArrived()

	DisablePlayerInteraction();
	
	ScriptManager:setMissionStatus( missionIds.bringTanks, MissionStatus.ACCOMPLISHED );
	
	ScriptManager:wait(1);
	
	EnablePlayerInteraction();
	
	checkForVictory();
	
	
	--startVictoryCutscene();
	
end


-- END onEnemyX



--BEGIN WaveLogic


function onEnemyTurnStart()

	-- spawn logic for plane waves
	if spawnPlaneWaves == 1 then -- should waves spawn? 1 == yes, 0 == no		
		-- check if a wave spawns this round
		if lastPlaneWaveSpawnedInRound + planeWaveRespawnInterval <= ScriptManager:getCurrentRound() then
			numberOfPlaneWavesSpawned = numberOfPlaneWavesSpawned + 1;
			ScriptManager:storeValue( "numberOfPlaneWavesSpawned", numberOfPlaneWavesSpawned );
			
			-- check if current spawn is a bomber spawn
			if numberOfPlaneWavesSpawned % bomberAppearance == 0 then -- spawn Bomber
				spawnWaveUnit( planeWaveUnits[2] );
				
			else -- spawn a destroyer
				for i = 1, #planeWaveUnits -1 do --	-1 zieht in der spawning-Tabelle die letzte Einheit ab (d.h., der Bomber wird nicht mitgespawnt)
					spawnWaveUnit( planeWaveUnits[1] );
				end
			end
			
			lastPlaneWaveSpawnedInRound = ScriptManager:getCurrentRound(); -- reset round count
			ScriptManager:storeValue( "lastPlaneWaveSpawnedInRound",	lastPlaneWaveSpawnedInRound );
		end
	end
	
	-- spawn logic for HQ attack waves 
	if spawnAttackWaves == 1 then -- should waves spawn? 1 == yes, 0 == no		
		-- check if a wave spawns this round
		if lastAttackWaveSpawnedInRound + AttackWaveRespawnInterval <= ScriptManager:getCurrentRound() then
			numberOfAttackWavesSpawned = numberOfAttackWavesSpawned + 1;
			ScriptManager:storeValue( "numberOfAttackWavesSpawned", numberOfAttackWavesSpawned );
			
			-- check if current spawn is an AntiInfantry spawn
			if numberOfAttackWavesSpawned % AntiInfantryAppearance == 0 then -- spawn AntiInfantry
				spawnWaveUnit( HQwaveUnits[2] );
				
			else -- spawn a MP40
				for i = 1, #HQwaveUnits - 1 do
					spawnWaveUnit( HQwaveUnits[1] );
				end
			end
			
			lastAttackWaveSpawnedInRound = ScriptManager:getCurrentRound(); -- reset round count
			ScriptManager:storeValue( "lastAttackWaveSpawnedInRound",	lastAttackWaveSpawnedInRound );
		end
	end
	
end



function onWaveHQspawned()
	spawnAttackWaves = 1;
	ScriptManager:storeValue( "spawnAttackWaves", 				spawnAttackWaves);
	ScriptManager:removeTrigger( "onWaveHQspawned" );
end



function spawnWaveUnit( spawnData )
	ScriptManager:spawnUnit( spawnData[1], spawnData[2], spawnData[3], spawnData[4], spawnData[5], spawnData[6] );
end


--END WaveLogic



function checkForVictory()

	if 	ScriptManager:getMissionStatus( missionIds.captureDepot )			== MissionStatus.ACCOMPLISHED and
		ScriptManager:getMissionStatus( missionIds.captureTankfactory )		== MissionStatus.ACCOMPLISHED and
		ScriptManager:getMissionStatus( missionIds.bringTanks ) 			== MissionStatus.ACCOMPLISHED then
		
				
		startVictoryCutscene();
		return true;
	end
	
	return false;
	
end


-- BEGIN Victory


function startVictoryCutscene()
	
	
	StartCutscene(); 	-- Russian units come in from the northern road
	
	ScriptManager:moveCameraEx( 24, 14, 1, 2, true );
	
	ScriptManager:spawnUnit( 0, Units.UNIT_SU_TRANSPORT_TRUCK, 23, 15, -1, AiState.WAIT );
	ScriptManager:wait( 0.5 );
	
	ScriptManager:spawnUnit( 0, Units.UNIT_SU_RIFLE, 	24, 15, -1, AiState.WAIT );
	ScriptManager:wait( 0.5 );
	
	ScriptManager:spawnUnit( 0, Units.UNIT_SU_MORTAR, 	24, 14, -1, AiState.WAIT );
	ScriptManager:wait( 0.5 );
	
	ScriptManager:spawnUnit( 0, Units.UNIT_SU_RIFLE, 	25, 15, -1, AiState.WAIT );
	ScriptManager:wait( 0.5 );
	
	ScriptManager:spawnUnit( 0, Units.UNIT_SU_ANTI_INFANTRY, 24, 13, -1, AiState.WAIT );
		
	
	HideDialogue();
	
	EndCutscene();
	
	victoryEndGame();
end

function victoryEndGame()
	DisablePlayerInteraction();
	HideDialogue(); -- remove possible texts on the screens

	-- accomplish secondary missions that are accomplished when the game is finished with certain condition
	
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_5_2_MISSION_OUTRO_FLAVOR_01 );
	ScriptManager:wait( 4 );
	HideDialogue();
	endGame();
end



-- END Victory
-- BEGIN Defeated


function checkForEnd()
	if ScriptManager:noUnitsLeft( 0 ) then
		ScriptManager:runForPlayer( 0, "missionFailed" );
	end
end

function missionFailed()
	DisablePlayerInteraction();
	HideDialogue(); -- remove possible texts on the screens
	
	ScriptManager:displaySubtitle( Text.T_MISSION_FAILED );
	ScriptManager:wait( 4 );
	HideDialogue();
	endGame();
end


-- END Defeated


function endGame()
	ScriptManager:endGame()
end