require "lua/Constants"
require "lua/Functions"
require "lua/Functions_2"
require "lua/Text"
require "lua/Units"
require "math"


-- mission stuff
missionIds = {
	captureDepot		= 1,	-- capture or destroy the Headquarters located on the island
	captureHQ			= 2,	-- capture or destroy the German HQ
	freeNorthernPrison	= 3,	-- free the northern prison
	freeSouthernPrison	= 4		-- free the southern prison
};

-- missionId, primary( boolean ), textkey, xpReward
missions = {
	{ missionIds.captureDepot,	 		true, 	Text.T_MAP_SU_5_3_MISSION_PRIMARY_01,	200 },
	{ missionIds.captureHQ, 			true, 	Text.T_MAP_SU_5_3_MISSION_PRIMARY_02,	150 },
	{ missionIds.freeNorthernPrison,	false,	Text.T_MAP_SU_5_3_MISSION_SECONDARY_01,	100 },
	{ missionIds.freeSouthernPrison, 	false, 	Text.T_MAP_SU_5_3_MISSION_SECONDARY_02,	100 },
};


enemySquadIds = {

depot			= 1,
northernPrison 	= 2,
southernPrison 	= 3,
airfield 		= 4,
HQ 				= 5,
firstEnemies	= 6,
woodsNorth		= 7,

};


depotIds			= { 12, 15, 16, 17, 18, 19, 20, 21, 29, 41, 59, 60   };
northernPrisonIds	= { 33, 34, 35 };
southernPrisonIds	= { 30, 31, 32 };
airfieldIds			= { 36, 37, 38, 39, 40, 55, 56 };
HQIds				= { 22, 23, 24, 25, 26, 27, 28, 42, 57, 58, 61, 62 };
woodsNorthIds		= { 66, 67 };


FirstEnemiesTriggerTiles = {
	{ 1, 12 },
	{ 2, 12 },
	{ 3, 12 },
	{ 4, 12 },
};


depotSpawnTriggerTiles = {
	{ 5, 11 },
	{ 6, 11 },
	{ 7, 10 },
	{ 7, 9  },
	{ 8, 8  },
	{ 8, 7  },
	{ 9, 6  },
	{ 9, 5  },
	{ 10, 4 },
	{ 10, 3 },
};

HQspawnTriggerTiles = {
	{ 18, 3 },
	{ 19, 2 },
	{ 19, 1 },
	{ 19, 15 },
	{ 19, 14 },
	{ 19, 13 },
	{ 20, 12 },
	{ 20, 11 },
	{ 21, 10 },
};


airfieldSpawnTriggerTiles = {
	{ 18, 3 },
	{ 19, 2 },
	{ 19, 1 },
	{ 19, 15 },
	{ 19, 14 },
	{ 19, 13 },
	{ 20, 12 },
	{ 20, 11 },
	{ 21, 10 },
	{ 18, 15 },
	{ 18, 16 },
	{ 17, 17 },
	{ 17, 19 },
	{ 17, 18 },
	{ 17, 20 },
	{ 17, 21 },
	{ 18, 22 },
	{ 18, 23 },
	{ 18, 24 },
	{ 17, 25 },
	{ 18, 26 },
};


prisonCutsceneTriggerTiles = {


	{ 8, 15 },
	{ 11, 19 },
	{ 12, 19 },
	{ 8, 7 },
	{ 9, 6 },
	{ 9, 5 },
	{ 10, 4 },	
	{ 12, 21 },
	{ 12, 22 },
	{ 11, 23 },
	{ 11, 24 },
	{ 10, 25 },

	
};


spawnReinforcementsTriggerTiles = {
	
	{ 4, 10 },
	{ 3, 9 },
	{ 3, 8 },
	{ 2, 8 },
	{ 1, 8 },	
	{ 6, 15 },
	{ 7, 16 },
	{ 7, 17 },
	

};


-- BEGIN Init

-- waves that spawn from the depot on the island:


spawnDepotWave				=  0; -- 0 == false; 1 == true; can't save booleans so I have to use numbers
depotWaveRespawnInterval	=  3; -- defines how often waves will spawn
numberOfdepotWavesSpawned	=  0; -- the number of attack waves that have already spawned in this game
lastDepotWaveSpawnedInRound	= -1; -- indicates the round in which the last wave spawned 


-- waves that can spawn from depot on the island
depotWave1 = {
--	playerId, 	templateID, 				tileX, tileY, squadID				AiState
	{ 1, 		Units.UNIT_DE_MP40, 			13, 14, 0,		AiState.ATTACK_AGGRESSIVE },
	{ 1, 		Units.UNIT_DE_PANZERFAUST,		13, 14, 0, 		AiState.ATTACK_AGGRESSIVE },
};

--	playerId, 	templateID, 				tileX, tileY, squadID				AiState
depotWave2 = {	
	{ 1, 		Units.UNIT_DE_TANK_3, 			13, 14, 0, 		AiState.DEFEND },
	{ 1, 		Units.UNIT_DE_ANTI_INFANTRY, 	13, 14, 0,		AiState.ATTACK_AGGRESSIVE },
};

--	playerId, 	templateID, 				tileX, tileY, squadID				AiState
depotWave3 = {
	{ 1, 		Units.UNIT_DE_TANK_4, 			13, 14, 0,		AiState.DEFEND },
	{ 1, 		Units.UNIT_DE_MP40, 			13, 14, 0,		AiState.ATTACK_AGGRESSIVE },
};

--	playerId, 	templateID, 				tileX, tileY, squadID				AiState
depotWave4 = {
	{ 1, 		Units.UNIT_DE_TIGER, 			13, 14, 0,		AiState.DEFEND },
};


spawnHQwave					=  0; -- 0 == false; 1 == true; can't save booleans so I have to use numbers
HQwaveRespawnInterval		=  3; -- defines how often waves will spawn
numberOfHQwavesSpawned		=  0; -- the number of attack waves that have already spawned in this game
lastHQwaveSpawnedInRound	= -1; -- indicates the round in which the last wave spawned 

-- waves that can spawn from HQ southeast

HQwave1 = {
--	playerId, 	templateID, 				tileX, tileY, squadID				AiState	
	{ 1, 		Units.UNIT_DE_TANK_3,			22, 7, 0,		AiState.ATTACK_AGGRESSIVE },
	{ 1,		Units.UNIT_DE_MP40,				22, 7, 0, 		AiState.ATTACK_AGGRESSIVE },
};

HQwave2 = {
--	playerId, 	templateID, 				tileX, tileY, squadID				AiState	
	{ 1,		Units.UNIT_DE_PANZERFAUST,		22, 7, 0,		AiState.ATTACK_AGGRESSIVE },
	{ 1,		Units.UNIT_DE_ANTI_INFANTRY,	22, 7, 0,		AiState.ATTACK_AGGRESSIVE },
};

HQwave3 = {
	{ 1,		Units.UNIT_DE_MP40,				22, 7, 0, 		AiState.ATTACK_AGGRESSIVE },
	{ 1,		Units.UNIT_DE_TANK_4,			22,	7, 0,		AiState.DEFEND },
};



-- plane wave spawn from airfield on the island
-- I spawn either one Fighter or one Destroyer depending on the number of spawns and the round count
-- ( every x turns I spawn a plane wave (Fighter) and every y turns the Destroyer spawns instead of fighters )
spawnPlaneWaveIsland		=  0; -- 0 == false; 1 == true; can't save booleans so I have to use numbers
planeWaveRespawnInterval	=  5; -- defines how often waves will spawn
DestroyerAppearance			=  3; -- every DestroyerAppearance-spawn a Destroyer appears instead of a Fighter
numberOfPlaneWavesSpawned	=  0; -- the number of plane waves that have already spawned in this game
lastPlaneWaveSpawnedInRound	= -1; -- indicates the round in which the last wave spawned 
planeWaveUnitsIsland = {
--	playerId,		 templateID, 				tileX, 		tileY,		squadID			AiState
	{ 1,			Units.UNIT_DE_FIGHTER,		11,			14,		  	0,			AiState.ATTACK },
	{ 1,			Units.UNIT_DE_DESTROYER,	11,			14,			0,			AiState.ATTACK },		
}


-- plane wave spawn from airfield northeast
-- I spawn either one Fighter or one Bomber depending on the number of spawns and the round count
-- ( every x turns I spawn a plane wave (Fighter) and every y turns the Destroyer spawns instead of fighters )
spawnPlaneWaveAirfield				=  0; -- 0 == false; 1 == true; can't save booleans so I have to use numbers
planeWaveRespawnInterval			=  5; -- defines how often waves will spawn
BomberAppearance					=  3; -- every bomberAppearance-spawn a bomber appears instead of a Fighter
numberOfPlaneWavesSpawned			=  0; -- the number of plane waves that have already spawned in this game
lastPlaneWaveSpawnedInRound			= -1; -- indicates the round in which the last wave spawned 
planeWaveUnitsAirfield = {
--	playerId,		 templateID, 				tileX, 		tileY,		squadID			AiState
	{ 1,			Units.UNIT_DE_FIGHTER,		23,			21,		  	0,			AiState.ATTACK },
	{ 1,			Units.UNIT_DE_BOMBER,		23,			21,			0,			AiState.ATTACK },		
}





function onGameStart()
	CreateMissions( missions );
	setTriggers();
	setGuardAreas()
	storeValues();
	
	ScriptManager:gainGold( 0, 2600 );
	
	-- start intro cutscene
	ScriptManager:sleep( 0, "startIntroCutscene" ); -- without sleep player turn popup shows
end

function storeValues()
	
	ScriptManager:storeValue( "spawnDepotWave", 				spawnDepotWave );
	ScriptManager:storeValue( "numberOfdepotWavesSpawned", 		numberOfdepotWavesSpawned );
	ScriptManager:storeValue( "lastDepotWaveSpawnedInRound", 	lastDepotWaveSpawnedInRound );
	
	ScriptManager:storeValue( "spawnHQwave",					spawnHQwave );
	ScriptManager:storeValue( "numberOfHQwavesSpawned",			numberOfHQwavesSpawned );
	ScriptManager:storeValue( "lastHQwaveSpawnedInRound",		lastHQwaveSpawnedInRound );
	
	ScriptManager:storeValue( "spawnPlaneWaveIsland", 			spawnPlaneWaveIsland );
	ScriptManager:storeValue( "numberOfPlaneWavesSpawned", 		numberOfPlaneWavesSpawned);
	ScriptManager:storeValue( "lastPlaneWaveSpawnedInRound",	lastPlaneWaveSpawnedInRound);
	
	ScriptManager:storeValue( "spawnPlaneWaveAirfield",			spawnPlaneWaveAirfield);
	ScriptManager:storeValue( "numberOfPlaneWavesSpawned", 		numberOfPlaneWavesSpawned);
	ScriptManager:storeValue( "lastPlaneWaveSpawnedInRound", 	lastPlaneWaveSpawnedInRound );
	
end

function setTriggers()

	-- load all needed values after loading
	ScriptManager:addOnGameResumeTrigger( "loadValues", "loadValues", false );
	
	-- check if the player has units left
	ScriptManager:addOnUnitDestroyedTrigger( 0, -1, -1, "checkForEnd", "checkForEnd" , false );

	-- trigger for wave spawns
	ScriptManager:addOnTurnStartTrigger( 1, -1, -1, "onEnemyTurnStart", "onEnemyTurnStart", false );

	-- Triggerline: spawning of the first enemies
	for i = 1, 	#FirstEnemiesTriggerTiles  do
	ScriptManager:addOnEnterTileTrigger( FirstEnemiesTriggerTiles[i][1], FirstEnemiesTriggerTiles[i][2], 0, -1, -1, false, "onFirstEnemiesStart", "onFirstEnemiesStart", false );
	end

	-- Triggerline: start prisonCutscene
	for i = 1, #prisonCutsceneTriggerTiles do
	ScriptManager:addOnEnterTileTrigger( prisonCutsceneTriggerTiles[i][1], prisonCutsceneTriggerTiles[i][2], 0, -1, -1, true, "onPrisonCutscenestart", "onPrisonCutscenestart", false );
	end
	
	-- Triggerline: start spawning from depot
	for i = 1, #depotSpawnTriggerTiles do
	ScriptManager:addOnEnterTileTrigger( depotSpawnTriggerTiles[i][1], depotSpawnTriggerTiles[i][2], 0, -1, -1, false, "onDepotWaveSpawned", "onDepotWaveSpawned",false );
	end

	-- Triggerline: start spawning from HQ
	for i = 1, #HQspawnTriggerTiles do
	ScriptManager:addOnEnterTileTrigger( HQspawnTriggerTiles[i][1], HQspawnTriggerTiles[i][2], 0, -1, -1, false, "onHQwaveSpawned", "onHQwaveSpawned",false );
	end

	-- Triggerline: start spawning from airfield northeast
	for i = 1, #airfieldSpawnTriggerTiles do
	ScriptManager:addOnEnterTileTrigger( airfieldSpawnTriggerTiles[i][1], airfieldSpawnTriggerTiles[i][2], 0, -1, -1, false, "onAirfieldWaveSpawned", "onAirfieldWaveSpawned", false );
	end

	-- Triggerline: spawn sovjet reinforcements in the south
	for i = 1, #spawnReinforcementsTriggerTiles do
	ScriptManager:addOnEnterTileTrigger( spawnReinforcementsTriggerTiles[i][1], spawnReinforcementsTriggerTiles[i][2], 0, -1, -1, true, "onSpawnReinforcements", "onSpawnReinforcements",false );
	end
	
	-- when the depot is captured/destroyed
	ScriptManager:addOnUnitCapturedTrigger( 0, Units.UNIT_DE_HQ, 1,		"onDepotCaptured", "onDepotCapturedDestroyed", false );
	ScriptManager:addOnUnitDestroyedTrigger( 1, Units.UNIT_DE_HQ, 1, 	"onDepotDestroyed", "onDepotCapturedDestroyed", false );
	
	-- when the HQ is captured/destroy
	ScriptManager:addOnUnitCapturedTrigger(  0, Units.UNIT_DE_HQ, 0, "onHQcaptured",  "onHQcapturedDestroyed", false );
	ScriptManager:addOnUnitDestroyedTrigger( 1, Units.UNIT_DE_HQ, 0, "onHQdestroyed", "onHQcapturedDestroyed", false );
	
	-- when the airfield on the island is captured/destroyed
	ScriptManager:addOnUnitCapturedTrigger( 0, Units.UNIT_DE_AIRFIELD, 5, "onAirfieldIslandcapturedDestroyed", "onAirfieldIslandcapturedDestroyed", false );
	ScriptManager:addOnUnitDestroyedTrigger( 0, Units.UNIT_DE_AIRFIELD, 5, "onAirfieldIslandcapturedDestroyed", "onAirfieldIslandcapturedDestroyed", false );
	
	-- when the airfield northeast is captured/destroyed
	ScriptManager:addOnUnitCapturedTrigger( 0, Units.UNIT_DE_AIRFIELD, 6, "onAirfieldcapturedDestroyed", "onAirfieldcapturedDestroyed", false );
	ScriptManager:addOnUnitDestroyedTrigger( 0,Units.UNIT_DE_AIRFIELD, 6, "onAirfieldcapturedDestroyed", "onAirfieldcapturedDestroyed", false );
	
	-- when the northern prison is freed, Prisoner Rifles spawn
	ScriptManager:addOnUnitCapturedTrigger( 0, Units.UNIT_DE_PRISON, 3, "onNorthernPrisonFreed","onNorthernPrisonFreed", false );
	
	-- when the northern prison is destroyed, the mission "free northern prison" is removed 
	ScriptManager:addOnUnitDestroyedTrigger( 0, Units.UNIT_DE_PRISON, 3, "onNorthernPrisonDestroyed", "onNorthernPrisonDestroyed", false );
	
	-- when the southern prison is freed, Prisoner Mortars spawn
	ScriptManager:addOnUnitCapturedTrigger( 0, Units.UNIT_DE_PRISON, 4, "onSouthernPrisonFreed","onNorthernPrisonFreed", false );

	-- when the southern prison is destroyed, the mission "free southern prison" is removed 
	ScriptManager:addOnUnitDestroyedTrigger(0, Units.UNIT_DE_PRISON, 4, "onSouthernPrisonDestroyed", "onSouthernPrisonDestroyed", false);
	
	-- prison attacked by player
	ScriptManager:addOnUnitHasAttackedTrigger( 0, Units.UNIT_DE_PRISON, -1, "ProtectTheBuildingHintSingle",		"ProtectTheSupplyDepotHintSingle", false );
	
end

function setGuardAreas()
	ScriptManager:setGuardArea( 1, 2, 16, 21, 2 );
	ScriptManager:setGuardArea( 1, 3, 15, 2, 2 );
end



function loadValues()
	spawnDepotWave 				= ScriptManager:getValue( "spawnDepotWave" );
	numberOfdepotWavesSpawned 	= ScriptManager:getValue( "numberOfdepotWavesSpawned" );
	lastDepotWaveSpawnedInRound	= ScriptManager:getValue( "lastDepotWaveSpawnedInRound" );
	
	spawnHQwave					= ScriptManager:getValue( "spawnHQwave" );
	numberOfHQwavesSpawned		= ScriptManager:getValue( "numberOfHQwavesSpawned" );
	lastHQwaveSpawnedInRound	= ScriptManager:getValue( "lastHQwaveSpawnedInRound" );

	spawnPlaneWaveIsland		= ScriptManager:getValue( "spawnPlaneWaveIsland" );
	numberOfPlaneWavesSpawned	= ScriptManager:getValue( "numberOfPlaneWavesSpawned" );
	lastPlaneWaveSpawnedInRound	= ScriptManager:getValue( "lastPlaneWaveSpawnedInRound" );
	
	spawnPlaneWaveAirfield		= ScriptManager:getValue( "spawnPlaneWaveAirfield" );	
	numberOfPlaneWavesSpawned	= ScriptManager:getValue( "numberOfPlaneWavesSpawned" );
	lastPlaneWaveSpawnedInRound	= ScriptManager:getValue( "lastPlaneWaveSpawnedInRound" );
end


-- END Init
-- BEGIN Intro Cutscene


function startIntroCutscene()
	StartCutscene();
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_5_3_MISSION_INTRO_FLAVOR_01 );
	ScriptManager:wait( 6 );
	ScriptManager:dismissSubtitle();
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_5_3_MISSION_INTRO_FLAVOR_02 );
	ScriptManager:moveCameraEx( 12, 15, 0.7, 3, true );
	ScriptManager:revealFogOfWar( 12, 15, 1, true );
	ScriptManager:showMarkerAt( 12, 15 );
	ScriptManager:wait( 3 );
	
	ScriptManager:moveCameraEx( 24, 5, 0.7, 3, true );
	ScriptManager:revealFogOfWar( 24, 5, 1, true );
	ScriptManager:showMarkerAt( 24, 5 );
	ScriptManager:wait( 3 );
	
	ScriptManager:dismissSubtitle();
	
	ScriptManager:moveCameraEx( 2, 22, 0.7, 3, true );
	ScriptManager:wait(2)
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_5_3_MISSION_INTRO_FLAVOR_03 );
	ScriptManager:wait( 5 );
	ScriptManager:dismissSubtitle();
	
	ScriptManager:revealFogOfWar( 12, 15, 1, false );
	ScriptManager:revealFogOfWar( 24, 5, 1, false );
	
	ScriptManager:removeAllMarkers();
	
	ScriptManager:setTargetTile( 1, 50, 13, 15 ); 
	ScriptManager:setTargetTile( 1, 60, 11, 12 ); 

	EndCutscene();


	ScriptManager:fortifyUnit( 16 );
	ScriptManager:fortifyUnit( 12 );
	ScriptManager:fortifyUnit( 29 );
	ScriptManager:camouflageUnit( 29 );
	ScriptManager:fortifyUnit( 43 );
	ScriptManager:fortifyUnit( 71 );
	ScriptManager:fortifyUnit( 72 );
	ScriptManager:fortifyUnit( 12 );
	ScriptManager:fortifyUnit( 74 );
	ScriptManager:fortifyUnit( 75 );
	ScriptManager:fortifyUnit( 77 );


	
	startTheMission();
end


--TODO remove all units from northern Prison / remove all units from southern Prison 


function startTheMission()
	ScriptManager:setMissionStatus( missionIds.captureDepot, MissionStatus.IN_PROGRESS );
	ScriptManager:setMissionStatus( missionIds.captureHQ, MissionStatus.IN_PROGRESS );
	
	--TODO: remove this function!
	--testing();
	--removeSquads();
	--ScriptManager:addOnUnitSelectedTrigger( 1, -1, -1, "showParameters", "showParameters",false)
end

function testing()
	ScriptManager:spawnUnit( 0, Units.GOTT, 1, 1, 0, AiState.WAIT );
	ScriptManager:spawnUnit( 0, Units.GOTT, 1, 2, 0, AiState.WAIT );
	ScriptManager:spawnUnit( 0, Units.UNIT_DE_ARTILLERY, 14, 1, 0, AiState.WAIT );
	ScriptManager:spawnUnit( 0, Units.UNIT_DE_ARTILLERY, 17, 20, 0, AiState.WAIT );
end

function removeSquads()
	for i = 1, #depotIds do
	ScriptManager:removeUnitEx( depotIds[i], false );
	end
	
	for i = 1, #northernPrisonIds do
	ScriptManager:removeUnitEx( northernPrisonIds[i], false ); 
	end
	
	for i = 1, #southernPrisonIds do
	ScriptManager:removeUnitEx( southernPrisonIds[i], false );
	end
	
	for i = 1, #airfieldIds do
	ScriptManager:removeUnitEx( airfieldIds[i], false );
	end
	
	for i = 1, #HQIds do
	ScriptManager:removeUnitEx( HQIds[i], false );
	end

	for i = 1, #woodsNorthIds do
	ScriptManager:removeUnitEx( woodsNorthIds, false );
 	end
end


function showParameters( selectedUnit )
	--print( string.format( "[] %s", selectedUnit:get() ) );
end

-- END Intro Cutscene
-- BEGIN onEnemyX


function onFirstEnemiesStart()

		

	ScriptManager:spawnUnits( 1, Units.UNIT_DE_MP40, 			8, 8, 6, AiState.MOVE_AGGRESSIVE, 2 );
	ScriptManager:spawnUnit(  1, Units.UNIT_DE_ANTI_INFANTRY,	8, 8, 6, AiState.MOVE_AGGRESSIVE );
	ScriptManager:spawnUnit(  1, Units.UNIT_DE_PANZERFAUST,		8, 8, 6, AiState.ATTACK_AGGRESSIVE	);
	
	ScriptManager:setTargetTile( 1, 6, 3, 8 );
	
	
	ScriptManager:removeTrigger( "onFirstEnemiesStart" );
end


function onPrisonCutscenestart()
	StartCutscene()
	ScriptManager:displaySubtitle( Text.T_MAP_SU_5_3_MISSION_SECONDARY_01_FLAVOR_01 );
	ScriptManager:wait( 5 );
	ScriptManager:dismissSubtitle();
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_5_3_MISSION_SECONDARY_01_FLAVOR_02 );
	ScriptManager:moveCameraEx( 16, 21, 1, 3, true );
	ScriptManager:revealFogOfWar( 16, 21, 1, true );
	ScriptManager:showMarkerAt( 16,21 );
	ScriptManager:wait( 4 );
	
	ScriptManager:moveCameraEx( 15, 2, 1, 3, true );
	ScriptManager:revealFogOfWar( 15, 2, 1, true );
	ScriptManager:showMarkerAt( 15, 2);
	ScriptManager:wait( 4 );
	
	ScriptManager:moveCameraEx( 8, 10, 0.7, 3, true );
	ScriptManager:displaySubtitle( Text.T_MAP_SU_5_3_MISSION_SECONDARY_01_FLAVOR_03 );
	ScriptManager:wait( 5 );
	
	ScriptManager:revealFogOfWar( 16, 21, 1, false );
	ScriptManager:revealFogOfWar( 15, 2, 1, false );
	ScriptManager:removeAllMarkers();
	
	ScriptManager:dismissSubtitle();
	
	EndCutscene()
	
	ScriptManager:setMissionStatus( missionIds.freeNorthernPrison, MissionStatus.IN_PROGRESS );
	ScriptManager:setMissionStatus( missionIds.freeSouthernPrison, MissionStatus.IN_PROGRESS );
	ScriptManager:removeTrigger( "onPrisonCutscenestart" );

end



function onSpawnReinforcements()

	StartCutscene()

	ScriptManager:moveCameraEx( 2, 2, 0.7, 2, true );
	
	ScriptManager:revealFogOfWar( 2, 2, 1, true );
	
	ScriptManager:wait( 1 );

	ScriptManager:displaySubtitle( Text.T_MAP_SU_5_3_MISSION_PRIMARY_01_FLAVOR_01 );
	ScriptManager:wait( 2 );

	ScriptManager:spawnUnit( 0, Units.UNIT_SU_ANTI_INFANTRY, 1, 1, 0, AiState.WAIT );
	ScriptManager:wait( 1 );

	ScriptManager:spawnUnit( 0, Units.UNIT_SU_MORTAR, 2, 1, 0, AiState.WAIT );
	ScriptManager:wait( 1 );

	ScriptManager:spawnUnit( 0, Units.UNIT_SU_TANK_T34, 2, 2, 0, AiState.WAIT );
	ScriptManager:wait( 1 );
	
	ScriptManager:spawnUnit( 0, Units.UNIT_SU_SUPPLY_TRUCK, 2, 3, 0, AiState.WAIT );
	
	ScriptManager:dismissSubtitle();

	ScriptManager:revealFogOfWar( 2, 2, 1, false );

	EndCutscene()
	
	ScriptManager:removeTrigger( "onSpawnReinforcements" );
end

function onDepotWaveSpawned()
	
	spawnDepotWave 			= 1;
	ScriptManager:storeValue( "spawnDepotWave", spawnDepotWave );
	spawnPlaneWaveIsland 	= 1;
	ScriptManager:storeValue( "spawnPlaneWaveIsland", spawnPlaneWaveIsland );
	
	ScriptManager:removeTrigger( "onDepotWaveSpawned" );	
end




function onDepotCaptured()
	
	spawnDepotWave = 0;
	ScriptManager:storeValue( "spawnDepotWave", spawnDepotWave );
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_5_3_MISSION_PRIMARY_01_FLAVOR_02 );
	ScriptManager:setMissionStatus( missionIds.captureDepot, MissionStatus.ACCOMPLISHED );
	ScriptManager:wait( 2 );
	ScriptManager:dismissSubtitle();
		
	ScriptManager:removeTrigger( "onDepotCapturedDestroyed" );
	
	checkForVictory();
end


function onDepotDestroyed()
	
	spawnDepotWave = 0;
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_5_3_MISSION_PRIMARY_01_FLAVOR_02 );
	ScriptManager:setMissionStatus( missionIds.captureDepot, MissionStatus.ACCOMPLISHED );
	ScriptManager:wait( 3 );
	ScriptManager:dismissSubtitle();
		
	ScriptManager:removeTrigger( "onDepotCapturedDestroyed" );
	
	checkForVictory();
end



function onHQcaptured()

	spawnHQwave = 0;
	ScriptManager:storeValue( "spawnHQwave", spawnHQwave );
		
	ScriptManager:setMissionStatus( missionIds.captureHQ, MissionStatus.ACCOMPLISHED );
	ScriptManager:removeTrigger( "onHQcapturedDestroyed" );
		
	checkForVictory();
end


function onHQdestroyed()

	spawnHQwave = 0;
	ScriptManager:storeValue( "spawnHQwave", spawnHQwave );
		
	ScriptManager:setMissionStatus( missionIds.captureHQ, MissionStatus.ACCOMPLISHED );
	ScriptManager:removeTrigger( "onHQcapturedDestroyed" );
		
	checkForVictory();
end



function onAirfieldIslandcapturedDestroyed()
 	spawnPlaneWaveIsland = 0;
 	ScriptManager:storeValue( "spawnPlaneWaveIsland", spawnPlaneWaveIsland );
end


function onAirfieldcapturedDestroyed()
	spawnPlaneWaveAirfield = 0;
	ScriptManager:storeValue( "spawnPlaneWaveAirfield", spawnPlaneWaveAirfield );
end


function onHQwaveSpawned()
	spawnHQwave = 1;
	ScriptManager:storeValue( "spawnHQwave", spawnHQwave );
	ScriptManager:removeTrigger( "onHQwaveSpawned" );
end


function onAirfieldWaveSpawned()

	spawnPlaneWaveAirfield = 1;
	ScriptManager:storeValue( "spawnPlaneWaveAirfield", spawnPlaneWaveAirfield );
	ScriptManager:removeTrigger( "onAirfieldWaveSpawned" );
	
	ScriptManager:spawnUnit( 1, Units.UNIT_DE_DESTROYER, 24, 22, -1, AiState.AIRPLANE );
end


function onNorthernPrisonFreed()

	StartCutscene();
	
	ScriptManager:moveCameraEx( 16, 21, 0.7, 2, true );

	ScriptManager:displaySubtitle( Text.T_MAP_SU_5_3_MISSION_SECONDARY_01_FLAVOR_04 );
	ScriptManager:wait( 4 );
	ScriptManager:dismissSubtitle();
	
	local unit = ScriptManager:spawnUnit( 0, Units.UNIT_SU_RIFLE,	17, 20, 0, AiState.WAIT );
	unit:setHealth( unit:getHealth() * 0.5 );
	unit:setAmmo( 0 );
	
	ScriptManager:wait( 1 );
	
	local unit = ScriptManager:spawnUnit( 0, Units.UNIT_SU_RIFLE,	16, 20, 0, AiState.WAIT ); 
	unit:setHealth( unit:getHealth() * 0.5 );
	unit:setAmmo( 0 );
	
	ScriptManager:wait( 1 );
	
	local unit = ScriptManager:spawnUnit( 0, Units.UNIT_SU_MORTAR,	17, 21, 0, AiState.WAIT );
	unit:setHealth( unit:getHealth() * 0.5 );
	unit:setAmmo( 0 );
	
	ScriptManager:wait( 1 );
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_5_3_MISSION_SECONDARY_01_FLAVOR_05 );
	ScriptManager:wait( 4 );
	ScriptManager:dismissSubtitle();
	EndCutscene();
	
	ScriptManager:setMissionStatus( missionIds.freeNorthernPrison, MissionStatus.ACCOMPLISHED );
	
	ScriptManager:spawnUnits( 1, Units.UNIT_DE_MP40, 13, 1, -1, AiState.ATTACK_AGGRESSIVE, 2 );
		
	ScriptManager:removeTrigger( "onNorthernPrisonDestroyed" );
		
		
end

function onNorthernPrisonDestroyed()
	ScriptManager:setMissionStatus( missionIds.freeNorthernPrison, MissionStatus.FAILED );
end


function onSouthernPrisonFreed()

	StartCutscene();

	ScriptManager:moveCameraEx( 15, 2, 0.7, 2, true );

	ScriptManager:displaySubtitle( Text.T_MAP_SU_5_3_MISSION_SECONDARY_02_FLAVOR_01 );
	ScriptManager:wait( 4 );
	ScriptManager:dismissSubtitle();
	
	local unit = ScriptManager:spawnUnit( 0, Units.UNIT_SU_MORTAR, 	14, 3, 0, AiState.WAIT );
	unit:setHealth( unit:getHealth() * 0.5 );
	unit:setAmmo( 0 );
	
	ScriptManager:wait( 1 );
	
	local unit = ScriptManager:spawnUnit( 0, Units.UNIT_SU_MORTAR, 	14, 2, 0, AiState.WAIT );
	unit:setHealth( unit:getHealth() * 0.5 );
	unit:setAmmo( 0 );
	
	ScriptManager:wait( 1 );
	
	local unit = ScriptManager:spawnUnit( 0, Units.UNIT_SU_RIFLE, 	14, 1, 0, AiState.WAIT );
	unit:setHealth( unit:getHealth() * 0.5 );
	unit:setAmmo( 0 );
	
	ScriptManager:wait( 1 );
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_5_3_MISSION_SECONDARY_02_FLAVOR_02 );
	ScriptManager:wait( 4 );
	ScriptManager:dismissSubtitle();
	
	EndCutscene();

	ScriptManager:setMissionStatus( missionIds.freeSouthernPrison, MissionStatus.ACCOMPLISHED );

	ScriptManager:removeTrigger( "onSouthernPrisonDestroyed" );

end

function onSouthernPrisonDestroyed()
	ScriptManager:setMissionStatus( missionIds.freeSouthernPrison, MissionStatus.FAILED );
end



function onEnemyTurnStart()
	
		
	local unit = getUnitById( 16 );
	unit:setActionPoints ( unit:getActionPoints() * 0 );

	local unit = getUnitById ( 12 );
	unit:setActionPoints (unit:getActionPoints() * 0 );

	local unit = getUnitById ( 75 );
	unit:setActionPoints (unit:getActionPoints() * 0 );

	local unit = getUnitById ( 77 );
	unit:setActionPoints (unit:getActionPoints() * 0 );

	
	
	if spawnDepotWave == 1 then
 		
		-- check if a wave spawns this round
		if lastDepotWaveSpawnedInRound + depotWaveRespawnInterval <= ScriptManager:getCurrentRound() then
		-- define by random which wave will spawn
			local rnd = math.random( 0, 3 );
			local spawnTable = 0;
			--print (waveNumber);
		
			if rnd == 0 then
				spawnTable = depotWave1;
		
			elseif rnd == 1 then
				spawnTable = depotWave2;
		
			elseif rnd == 2 then
				spawnTable = depotWave3;
		
			elseif rnd == 3 then
				spawnTable = depotWave4;
			
			end
		
			-- spawn the wave
			for i = 1, #spawnTable do
			spawnWave( spawnTable[i] );
			end
			
			-- set counter for nexts spawn
			lastDepotWaveSpawnedInRound = ScriptManager:getCurrentRound();
			ScriptManager:storeValue( "lastDepotWaveSpawnedInRound", lastDepotWaveSpawnedInRound );
		end
	end
	-- spawn logic for plane waves from airfield near depot

	
	if spawnPlaneWaveIsland == 1 then -- should waves spawn? 1 == yes, 0 == no		
				
		-- check if a wave spawns this round
		if lastPlaneWaveSpawnedInRound + planeWaveRespawnInterval <= ScriptManager:getCurrentRound() then
			numberOfPlaneWavesSpawned = numberOfPlaneWavesSpawned + 1;
			ScriptManager:storeValue( "numberOfPlaneWavesSpawned", numberOfPlaneWavesSpawned );
		
			-- check if current spawn is a Destroyer spawn
			if numberOfPlaneWavesSpawned % DestroyerAppearance == 0 then -- spawn Destroyer
			spawnPlaneWaveUnit( planeWaveUnitsIsland[2] );
			
			else -- spawn a Fighter
				for i = 1, #planeWaveUnitsIsland -1 do --	-1 zieht in der spawning-Tabelle die letzte Einheit ab (d.h., der Bomber wird nicht mitgespawnt)
				spawnPlaneWaveUnit( planeWaveUnitsIsland[1] );
				end
			end
				
			lastPlaneWaveSpawnedInRound = ScriptManager:getCurrentRound(); -- reset round count
			ScriptManager:storeValue( "lastPlaneWaveSpawnedInRound",	lastPlaneWaveSpawnedInRound );
		end
	end
	

	
	-- spawn logic for plane waves from airfield northeast
	if spawnPlaneWaveAirfield == 1 then -- should waves spawn? 1 == yes, 0 == no	
		-- check if a wave spawns this round
		
		if lastPlaneWaveSpawnedInRound + planeWaveRespawnInterval <= ScriptManager:getCurrentRound() then
			numberOfPlaneWavesSpawned = numberOfPlaneWavesSpawned + 1;
			ScriptManager:storeValue( "numberOfPlaneWavesSpawned", numberOfPlaneWavesSpawned );
			
			-- check if current spawn is a Bomber spawn
			if numberOfPlaneWavesSpawned % BomberAppearance == 0 then -- spawn Bomber
				spawnPlaneWaveUnit( planeWaveUnitsAirfield[2] );
				
			else -- spawn a Fighter
				for i = 1, #planeWaveUnitsAirfield -1 do --	-1 zieht in der spawning-Tabelle die letzte Einheit ab (d.h., der Bomber wird nicht mitgespawnt)
					spawnPlaneWaveUnit( planeWaveUnitsAirfield[1] );
				end
			end
			
			lastPlaneWaveSpawnedInRound = ScriptManager:getCurrentRound(); -- reset round count
			ScriptManager:storeValue( "lastPlaneWaveSpawnedInRound",	lastPlaneWaveSpawnedInRound );
		end
	end
	


	if spawnHQwave == 1 then
	
	-- check if a wave spawns this round
		if lastHQwaveSpawnedInRound + HQwaveRespawnInterval <= ScriptManager:getCurrentRound() then
			-- define by random which wave will spawn
			local rnd = math.random( 0, 2 );
			local spawnTable = 0;
			--print (waveNumber);
		
			if rnd == 0 then
				spawnTable = HQwave1;
		
			elseif rnd == 1 then
				spawnTable = HQwave2;
		
			elseif rnd == 2 then
				spawnTable = HQwave3;
		
			end
		
			-- spawn the wave
			for i = 1, #spawnTable do
			spawnWave( spawnTable[i] );
			end
		
			-- set counter for nexts spawn
			lastHQwaveSpawnedInRound = ScriptManager:getCurrentRound();
			ScriptManager:storeValue( "lastHQwaveSpawnedInRound", lastHQwaveSpawnedInRound );
		end
	end
end



function  spawnWave( spawnData )
		ScriptManager:spawnUnit( spawnData[1], spawnData[2], spawnData[3], spawnData[4], spawnData[5], spawnData[6] );
end


function spawnPlaneWaveUnit(spawnData)
	ScriptManager:spawnUnit( spawnData[1], spawnData[2], spawnData[3], spawnData[4], spawnData[5], spawnData[6] );
end




function checkForVictory()

	if 	ScriptManager:getMissionStatus( missionIds.captureDepot )	== MissionStatus.ACCOMPLISHED and
		ScriptManager:getMissionStatus( missionIds.captureHQ )		== MissionStatus.ACCOMPLISHED then
		
		startVictoryCutscene();
		return true;
	end
	
	return false;
	
end
-- END
-- BEGIN Victory


function startVictoryCutscene()

	StartCutscene();
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_5_3_MISSION_OUTRO_FLAVOR_01 );
	
	-- clear the tiles at the island depot
	for y = 14, 11, -1 do
		for x = 14, 11, -1 do
			local unit = Unit:getUnitAt( x, y );
			
			if unit ~= nil then
				ScriptManager:removeUnitEx( unit:getId(), true ); 
			end
		end
	end
	
	ScriptManager:wait( 1 );
	
	
	ScriptManager:moveCameraEx( 12, 15, 1.0, 3, true );	
	
	ScriptManager:revealFogOfWar( 12, 15, 3, true );
	
	ScriptManager:wait( 2 );
	
	-- clear the tiles at the HQ		
	for y = 9, 6, -1 do
		for x = 25, 21, -1 do
			local unit = Unit:getUnitAt( x, y );
	
			if unit ~= nil then
				ScriptManager:removeUnitEx( unit:getId(), true );
			end
		end
	end
	
	
	local unitRifleOne = ScriptManager:spawnUnit( 0, Units.UNIT_SU_RIFLE, 12, 12 , -1, AiState.WAIT );
	local RifleOneId = unitRifleOne:getId();
	
	local unitRifleTwo = ScriptManager:spawnUnit( 0, Units.UNIT_SU_MORTAR, 13, 12 , -1, AiState.WAIT );
	local RifleTwoId = unitRifleTwo:getId();
	
	local unitRifleThree = ScriptManager:spawnUnit( 0, Units.UNIT_SU_RIFLE, 14, 12 , -1, AiState.WAIT );
	local RifleThreeId = unitRifleThree:getId();
	
	ScriptManager:moveUnitEx( RifleOneId, 12, 14, true );
	ScriptManager:moveUnitEx( RifleTwoId, 13, 14, true );
	ScriptManager:moveUnitEx(RifleThreeId, 14, 14, true );
	
	ScriptManager:wait( 2 );
	

	ScriptManager:moveCameraEx( 23, 5, 1.0, 3, true );
	
	ScriptManager:revealFogOfWar( 23, 5, 3, true );
	
	ScriptManager:wait( 2 );


	local unitTankOne = ScriptManager:spawnUnit( 0, Units.UNIT_SU_TANK_KV1, 22, 6, -1, AiState.WAIT );
	local TankOneId = unitTankOne:getId();
	
	local unitTankTwo = ScriptManager:spawnUnit( 0, Units.UNIT_SU_TANK_T34, 22, 7, -1, AiState.WAIT );
	local TankTwoId = unitTankTwo:getId();
	
	local unitTankThree = ScriptManager:spawnUnit( 0, Units.UNIT_SU_TANK_KV1, 23, 7, -1, AiState.WAIT );
	local TankThreeId = unitTankThree:getId();
	
	ScriptManager:moveUnitEx( TankOneId, 23, 5, true );
	
	ScriptManager:moveUnitEx(TankTwoId,24, 6, true );
	
	ScriptManager:moveUnitEx(TankThreeId, 25, 6, true );

	
	ScriptManager:wait( 4 );
	
	HideDialogue();
	
	EndCutscene();
	
	victoryEndGame();
end

function victoryEndGame()
	DisablePlayerInteraction();
	HideDialogue(); -- remove possible texts on the screens

	-- accomplish secondary missions that are accomplished when the game is finished with certain condition
	
	
	ScriptManager:displaySubtitle( Text.T_MISSION_VICTORY );
	ScriptManager:wait( 4 );
	HideDialogue();
	endGame();
end


-- END Victory
-- BEGIN Defeated


function checkForEnd()
	if ScriptManager:noUnitsLeft( 0 ) then
		ScriptManager:runForPlayer( 0, "missionFailed" )
	end
end

function missionFailed()
	DisablePlayerInteraction();
	HideDialogue(); -- remove possible texts on the screens
	
	ScriptManager:displaySubtitle( Text.T_MISSION_FAILED );
	ScriptManager:wait( 4 );
	HideDialogue();
	endGame();
end


-- END Defeated


function endGame()
	ScriptManager:endGame()
end