require "lua/Constants"
require "lua/Functions"
require "lua/Functions_2"
require "lua/Text"
require "lua/Units"
require "math"


-- mission stuff
missionIds = {
		defeat25Attackers 				= 1,
		doNotLoseMoreThanEightUnits		= 2,
		killTheGermanOfficers			= 3,
		CaptureOrDestroyBothOutposts	= 4, 
};

-- missionId, primary( boolean ), textkey, xpReward
missions = {
	{ missionIds.defeat25Attackers,				 true, Text.T_MAP_SU_6_1_MISSION_PRIMARY_01,	175 },
	{ missionIds.doNotLoseMoreThanEightUnits, 	 true, Text.T_MAP_SU_6_1_MISSION_PRIMARY_02, 	 50 },
	{ missionIds.killTheGermanOfficers, 	 	 true, Text.T_MAP_SU_6_1_MISSION_PRIMARY_03, 	 75 },
	{ missionIds.CaptureOrDestroyBothOutposts, 	 false, Text.T_MAP_SU_6_1_MISSION_SECONDARY_01,  50 },	
};

enemySquadIds = {

attackWave1 = 1,
attackWave2	= 2,
attackWave3	= 3,
attackWave4	= 4,
attackWave5	= 5,
attackWave6	= 6,

tigerTank1	= 7, -- north
tigerTank2	= 8, -- south

headquarterNorth = 9,
headquarterSouth = 10,

defendersTankNorth = 11,
defendersDepotNorth = 12,

defendersTankSouth = 13,
defendersDepotSouth = 14,

artilleryNorth = 15,
artillerySouth = 16,

mgNestNorthEast = 17,
mgNestSouthWest = 18,

troopsNorth = 19,
troopsSouth = 20,

mgNestNorthWest = 21,
mgNestSouthEast = 22,

attackersAfterIntroCutscene = 23,

squadOneAfterFirstTigerDead = 24,
squadTwoAfterFirstTigerDead = 25,
};


depotDefendersNorthIds = { 13, 19 };
depotDefendersSouthIds = { 20, 36  };

tigerNorthId = 2;
tigerSouthId = 1;
 
attackWave1 = {
--	 playerId, templateID, tileX, tileY, squadId, AiState
	{ 1, Units.UNIT_DE_MP40, 			14, 42, enemySquadIds.attackWave1, AiState.ATTACK_AGGRESSIVE };
	{ 1, Units.UNIT_DE_PANZERFAUST, 	13, 42, enemySquadIds.attackWave1, AiState.ATTACK_AGGRESSIVE };
	{ 1, Units.UNIT_DE_TANK_3, 			13, 41, enemySquadIds.attackWave1, AiState.ATTACK_AGGRESSIVE };
	{ 1, Units.UNIT_DE_ANTI_INFANTRY, 	12, 41, enemySquadIds.attackWave1, AiState.ATTACK_AGGRESSIVE };
};

attackWave2 = {
--	 playerId, templateID, tileX, tileY, squadId, AiState
	{ 1, Units.UNIT_DE_TANK_4, 			3, 1, enemySquadIds.attackWave2, AiState.ATTACK_AGGRESSIVE };
	{ 1, Units.UNIT_DE_MP40,			4, 2, enemySquadIds.attackWave2, AiState.ATTACK_AGGRESSIVE };
	{ 1, Units.UNIT_DE_PANZERFAUST, 	3, 2, enemySquadIds.attackWave2, AiState.ATTACK_AGGRESSIVE };
	{ 1, Units.UNIT_DE_ANTI_INFANTRY, 	2, 1, enemySquadIds.attackWave2, AiState.ATTACK_AGGRESSIVE };
};

attackWave3 = {
--	 playerId, templateID, tileX, tileY, squadId, AiState
	{ 1, Units.UNIT_DE_PANZERFAUST, 	4, 42, enemySquadIds.attackWave3, AiState.ATTACK_AGGRESSIVE };
	{ 1, Units.UNIT_DE_MP40,			3, 41, enemySquadIds.attackWave3, AiState.ATTACK_AGGRESSIVE };
	{ 1, Units.UNIT_DE_ANTI_INFANTRY, 	2, 41, enemySquadIds.attackWave3, AiState.ATTACK_AGGRESSIVE };
	{ 1, Units.UNIT_DE_TANK_DESTROYER, 	3, 42, enemySquadIds.attackWave3, AiState.ATTACK_AGGRESSIVE };
};

attackWave4 = {
--	 playerId, templateID, tileX, tileY, squadId, AiState
	{ 1, Units.UNIT_DE_TANK_4, 		11, 1, enemySquadIds.attackWave4, AiState.ATTACK_AGGRESSIVE };
	{ 1, Units.UNIT_DE_TANK_3,		12, 2, enemySquadIds.attackWave4, AiState.ATTACK_AGGRESSIVE };
	{ 1, Units.UNIT_DE_MP40, 		10, 1, enemySquadIds.attackWave4, AiState.ATTACK_AGGRESSIVE };
	{ 1, Units.UNIT_DE_PANZERFAUST, 11, 2, enemySquadIds.attackWave4, AiState.ATTACK_AGGRESSIVE };
};

attackWave5 = {
--	 playerId, templateID, tileX, tileY, squadId, AiState
	{ 1, Units.UNIT_DE_MP40, 		12, 41, enemySquadIds.attackWave5, AiState.ATTACK_AGGRESSIVE };
	{ 1, Units.UNIT_DE_PANZERFAUST,	13, 41, enemySquadIds.attackWave5, AiState.ATTACK_AGGRESSIVE };
	{ 1, Units.UNIT_DE_TANK_4, 		14, 42, enemySquadIds.attackWave5, AiState.ATTACK_AGGRESSIVE };
	{ 1, Units.UNIT_DE_MP40,	 	13, 42, enemySquadIds.attackWave5, AiState.ATTACK_AGGRESSIVE };
};

attackWave6 = {
--	 playerId, templateID, tileX, tileY, squadId, AiState
	{ 1, Units.UNIT_DE_TANK_3, 			3, 1, enemySquadIds.attackWave6, AiState.ATTACK_AGGRESSIVE };
	{ 1, Units.UNIT_DE_MP40,			4, 1, enemySquadIds.attackWave6, AiState.ATTACK_AGGRESSIVE };
	{ 1, Units.UNIT_DE_TANK_4, 			4, 2, enemySquadIds.attackWave6, AiState.ATTACK_AGGRESSIVE };
	{ 1, Units.UNIT_DE_TANK_DESTROYER,  3, 2, enemySquadIds.attackWave6, AiState.ATTACK_AGGRESSIVE };
	{ 1, Units.UNIT_DE_PANZERFAUST,     2, 1, enemySquadIds.attackWave6, AiState.ATTACK_AGGRESSIVE };
};


outpost1 = 3;

outpost2 = 0;

unitsAreAlive = 1;

attackersToKill = 25;

middleOfAttack = 17;

fiveAttackersDead = 5;

nineAttackersDead = 7;

officersToKill = 2;

unitsLost = 0;


unitsToLose = 8;

outpostsToCapture = 2;

-- BEGIN Init


function onGameStart()
	CreateMissions( missions );
	setTriggers();
	storeValues();
	
	ScriptManager:gainGold( 0, 800 );
	
	ScriptManager:fortifyUnit( 1 );
	ScriptManager:fortifyUnit( 2 ); 
	ScriptManager:fortifyUnit( 13 ); 
	ScriptManager:fortifyUnit( 14 ); 	
	ScriptManager:fortifyUnit( 19 );	
	ScriptManager:fortifyUnit( 20 );
	ScriptManager:fortifyUnit( 29 );
	ScriptManager:fortifyUnit( 30 );
	
	
	-- start intro cutscene
	ScriptManager:sleep( 0, "startIntroCutscene" ); -- without sleep player turn popup shows
end

function storeValues()
	ScriptManager:storeValue( "unitsLost", unitsLost );
end

function setTriggers()
	-- load all needed values after loading
	ScriptManager:addOnGameResumeTrigger( "loadValues", "loadValues", false );
	
	-- check if the player has units left
	ScriptManager:addOnUnitDestroyedTrigger( 0, -1, -1, "checkForEnd", "checkForEnd" , false );
	
	-- When one of the players units gets destroyed
	ScriptManager:addOnUnitDestroyedTrigger( 0, -1, -1, "onUnitLost", "onUnitLost", false );
	
	-- when the German Tiger Tanks/officers get Destroyed
	ScriptManager:addOnUnitDestroyedTrigger( 1, Units.UNIT_DE_TIGER, -1, "onTigerDestroyed", "onTigerDestroyed", false );
	
	-- when a Tiger gets attacked, remove the Action point block
	ScriptManager:addOnUnitHasAttackedTrigger( 0, -1, -1, "onTigerAttacked", "onTigerAttacked", false );
	
	-- when units of the attacking waves get destroyed
	ScriptManager:addOnUnitDestroyedTrigger( 1, Units.UNIT_DE_ARTILLERY, -1, 		"onAttackerDestroyed", "onAttackerDestroyed", false );
	ScriptManager:addOnUnitDestroyedTrigger( 1, Units.UNIT_DE_ANTI_INFANTRY, -1, 	"onAttackerDestroyed", "onAttackerDestroyed", false );
	ScriptManager:addOnUnitDestroyedTrigger( 1, Units.UNIT_DE_MP40, -1, 			"onAttackerDestroyed", "onAttackerDestroyed", false );
	ScriptManager:addOnUnitDestroyedTrigger( 1, Units.UNIT_DE_PANZERFAUST, -1, 		"onAttackerDestroyed", "onAttackerDestroyed", false );
	ScriptManager:addOnUnitDestroyedTrigger( 1, Units.UNIT_DE_TANK_3, -1, 			"onAttackerDestroyed", "onAttackerDestroyed", false );
	ScriptManager:addOnUnitDestroyedTrigger( 1, Units.UNIT_DE_TANK_4, -1, 			"onAttackerDestroyed", "onAttackerDestroyed", false );
	ScriptManager:addOnUnitDestroyedTrigger( 1, Units.UNIT_DE_TANK_DESTROYER, -1, 	"onAttackerDestroyed", "onAttackerDestroyed", false );
	

	-- when the outposts get captured or destroyed
	ScriptManager:addOnUnitCapturedTrigger( 0, Units.UNIT_DE_HQ, -1, "onOutpostCaptured", "onOutpostCaptured", false );
	ScriptManager:addOnUnitDestroyedTrigger( 1, Units.UNIT_DE_HQ, -1, "onOutpostCaptured", "onOutpostCaptured", false );
	
	-- spawning of the enemy waves
	
	ScriptManager:addOnTurnStartTrigger( 1, -1, -1, "onEnemyTurnStart", "onEnemyTurnStart", false );
	
	-- reduce the movement of the german officers
	ScriptManager:addOnTurnStartTrigger( 1 , -1, -1, "reduceTigerNorthAction", "reduceTigerNorthAction", false );
	ScriptManager:addOnTurnStartTrigger( 1 , -1, -1, "reduceTigerSouthAction", "reduceTigerSouthAction", false );
	
	
	ScriptManager:addOnTurnStartTrigger( -1, -1, -1, "reduceDepotDefendersNorthActionPoint", "reduceDepotDefendersNorthActionPoint", false );

	ScriptManager:addOnTurnStartTrigger( -1, -1, -1, "reduceDepotDefendersSouthActionPoint", "reduceDepotDefendersSouthActionPoint", false );
end


function loadValues()
	ScriptManager:getValue( "unitsLost" );
end


-- END Init
-- BEGIN Intro Cutscene


function startIntroCutscene()
	StartCutscene();
	
	ScriptManager:moveCameraEx( 7, 24, 0.7, 2, true );
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_6_1_MISSION_INTRO_FLAVOR_01 );
	ScriptManager:wait( 4 );
	ScriptManager:dismissSubtitle();
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_6_1_MISSION_INTRO_FLAVOR_02 );
	
	ScriptManager:moveCameraEx( 8, 34, 0.7, 3, true );
	ScriptManager:revealFogOfWar( 8, 34, 1, true );
	ScriptManager:wait( 1.5 );
	ScriptManager:revealFogOfWar( 8, 34, 1, false );
	
	ScriptManager:moveCameraEx( 9, 12, 0.7, 3, true );
	ScriptManager:revealFogOfWar( 9, 12, 1, true );
	ScriptManager:wait( 1.5 );
	ScriptManager:revealFogOfWar( 9, 12, 1, false );

	ScriptManager:dismissSubtitle();
	
	ScriptManager:moveCameraEx( 13, 19, 0.7, 2, true );
	ScriptManager:wait( 1 );
	ScriptManager:displaySubtitle( Text.T_MAP_SU_6_1_MISSION_INTRO_FLAVOR_03 );
	ScriptManager:wait( 5 );
	ScriptManager:dismissSubtitle();
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_6_1_MISSION_INTRO_FLAVOR_04 );
	
	ScriptManager:wait( 4 );
	
	ScriptManager:moveCameraEx( 7, 24, 0.7, 2, true );
	ScriptManager:displaySubtitle( Text.T_MAP_SU_6_1_MISSION_INTRO_FLAVOR_05 );
	ScriptManager:wait( 5 );
	ScriptManager:dismissSubtitle();	
		
	ScriptManager:wait( 1 );	
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_6_1_MISSION_INTRO_FLAVOR_06 );
	ScriptManager:wait( 4 );
	ScriptManager:dismissSubtitle();	
	
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_6_1_MISSION_INTRO_FLAVOR_07 );	
	ScriptManager:wait( 7 );
	ScriptManager:dismissSubtitle();
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_6_1_MISSION_INTRO_FLAVOR_08 );	
	ScriptManager:wait( 7 );
	ScriptManager:dismissSubtitle();
		
	EndCutscene();
	
	startTheMission();
end

function startTheMission()
	ScriptManager:setMissionStatus( missionIds.defeat25Attackers , MissionStatus.IN_PROGRESS );
	ScriptManager:setMissionStatus(missionIds.doNotLoseMoreThanEightUnits,MissionStatus.IN_PROGRESS );
	
	ScriptManager:fortifyUnit( 29 );
	ScriptManager:fortifyUnit( 30 );
	
	ScriptManager:setTargetTile( 1, enemySquadIds.attackersAfterIntroCutscene, 6, 24 );
	
	ScriptManager:setGuardArea( 1, enemySquadIds.defendersDepotNorth, 9, 39, 1 );
	
	--ScriptManager:addOnUnitSelectedTrigger( 1, -1, -1, "showParameters", "showParameters",false)
end


function removeSquads()
	--for i = 1, # do
	--	ScriptManager:removeUnitEx( [i], false );
	--end
end

function showParameters( selectedUnit )
	--print( string.format( "[] %s", selectedUnit:get() ) );
end


-- END Intro Cutscene
-- BEGIN


-- insert missionstuff here


function onEnemyTurnStart()

	local currentRound = ScriptManager:getCurrentRound();
		
	if currentRound == 1 then
		for i = 1, #attackWave1 do
		spawnWave( attackWave1[i] );
		ScriptManager:setTargetTile( 1, enemySquadIds.attackWave1, 6, 26 );
		end
	

	elseif currentRound == 2 then
		for i = 1, #attackWave2 do
		spawnWave( attackWave2[i] );
		ScriptManager:setTargetTile( 1, enemySquadIds.attackWave2, 6, 21 );
		end
	

	elseif currentRound == 4 then
		for i = 1, #attackWave3 do
		spawnWave( attackWave3[i] );
		ScriptManager:setTargetTile( 1, enemySquadIds.attackWave3, 7, 25 );	
		end
	

	elseif currentRound == 6 then
		for i = 1, #attackWave4 do
		spawnWave( attackWave4[i] );
		ScriptManager:setTargetTile( 1, enemySquadIds.attackWave4, 9, 22 );	
		end
	
	
	elseif currentRound == 8 then
		for i = 1, #attackWave5 do
		spawnWave( attackWave5[i] );
		ScriptManager:setTargetTile( 1, enemySquadIds.attackWave5, 12, 20 );
		end
	
	
	elseif currentRound == 10 then
		for i = 1, #attackWave6 do
		spawnWave( attackWave6[i] );
		ScriptManager:setTargetTile( 1, enemySquadIds.attackWave6, 12, 20 );
		end	
	end	
end


function spawnWave( spawnData )
	ScriptManager:spawnUnit( spawnData[1], spawnData[2], spawnData[3], spawnData[4],spawnData[5], spawnData[6] )
end


function reduceTigerNorthAction()

	local unit
	for i = 1, tigerNorthId do
	unit = Unit:getUnitById( tigerNorthId );
	
		if unit ~= nil then
		unit:setActionPoints( 0 );
		end
	end
end


function reduceTigerSouthAction()
	local unit
	for i = 1, tigerSouthId do
	unit = Unit:getUnitById( tigerSouthId );
	
		if unit ~= nil then
		unit:setActionPoints( 0 );
		end
	end
end


function reduceDepotDefendersNorthActionPoint()
	
	-- set the maximal action points of the depot north defenders to 0
	local unit
	for i = 1, #depotDefendersNorthIds do
		unit = Unit:getUnitById( depotDefendersNorthIds[i] );
		if unit ~= nil then
			unit:setMaxActionPoints( 0 );
		end	
	end
end


function reduceDepotDefendersSouthActionPoint()

	-- set the maximal action points of the depot south defenders to 0
	local unit
	for i = 1, #depotDefendersSouthIds do
		unit = Unit:getUnitById( depotDefendersSouthIds[i] );	
		if unit ~= nil then
			unit:setMaxActionPoints( 0 );
		end	
	end
end



function onTigerAttacked( activeUnit, passiveUnit )
	if passiveUnit == tigerNorthId then
	ScriptManager:removeTrigger( "reduceTigerNorthAction" );
	Unit:changeSquadState( 1, enemySquadIds.tigerTank1,AiState.ATTACK_AGGRESSIVE );
	end

	if passiveUnit == tigerSouthId then
	ScriptManager:removeTrigger( "reduceTigerSouthAction" );
	Unit:changeSquadState( 1, enemySquadIds.tigerTank2, AiState.ATTACK_AGGRESSIVE );
	end
end


function onAttackerDestroyed( unit )
		
--	print (unit:getTemplateId())
--	
--		unit:getTemplateId()
--	if 	unit ~= Units.UNIT_DE_TIGER or
--		unit ~= Units.UNIT_DE_HQ then	
	
	ScriptManager:setMissionProgress( missionIds.defeat25Attackers, ScriptManager:getMissionProgress(missionIds.defeat25Attackers ) +1 );
	ShowMissionProgressPopup(ScriptManager:getMissionProgress( missionIds.defeat25Attackers ), Text.T_MAP_SU_6_1_MISSION_PRIMARY_01_COUNTER ); 
	
		if ScriptManager:getMissionProgress( missionIds.defeat25Attackers ) == fiveAttackersDead then
		ScriptManager:runForPlayer( 0, "playTigerCutscene" );
		end
		
		if ScriptManager:getMissionProgress( missionIds.defeat25Attackers ) == nineAttackersDead then
		ScriptManager:runForPlayer( 0, "playOutpostCutscene" );
		end
		
		if ScriptManager:getMissionProgress( missionIds.defeat25Attackers ) == middleOfAttack then
		ScriptManager:runForPlayer( 0, "playSupplytruckCutscene" );
		end
	
		if ScriptManager:getMissionProgress( missionIds.defeat25Attackers ) >= attackersToKill then
	
		ScriptManager:setMissionStatus(missionIds.defeat25Attackers, MissionStatus.ACCOMPLISHED );
		ScriptManager:removeTrigger( "onAttackerDestroyed" );
	
		ScriptManager:displaySubtitle( Text.T_MAP_SU_6_1_MISSION_PRIMARY_01_FLAVOR_03 );
		ScriptManager:wait( 5 );
		ScriptManager:dismissSubtitle();
	
		checkForVictory()
	
		EnablePlayerInteraction();
		end	
--	end
end


function playTigerCutscene()

	ScriptManager:startCutscene();
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_6_1_MISSION_PRIMARY_03_FLAVOR_01 );
	ScriptManager:wait( 6 );
	ScriptManager:dismissSubtitle();
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_6_1_MISSION_PRIMARY_03_FLAVOR_02 );
	
	ScriptManager:moveCameraEx( 2, 40, 1, 3, true );
	ScriptManager:revealFogOfWar( 2, 40, 1, true );
	ScriptManager:wait( 1 );
	ScriptManager:showMarkerAt( 2, 40 );
	ScriptManager:wait( 1 );
	
	
	ScriptManager:moveCameraEx( 9, 3, 1, 4, true );
	ScriptManager:revealFogOfWar( 9, 3, 1, true );
	ScriptManager:wait( 1 );
	ScriptManager:showMarkerAt( 9, 3 );
	ScriptManager:wait( 1 );
	
	ScriptManager:moveCameraEx( 7, 24, 0.7, 3, true );
	ScriptManager:dismissSubtitle();
	
	ScriptManager:hideMarkerAt( 2, 40 );
	ScriptManager:revealFogOfWar( 2, 40, 1, false );
	
	ScriptManager:hideMarkerAt( 9, 3 );
	ScriptManager:revealFogOfWar( 9, 3, 1, false );
	
	ScriptManager:wait( 2 );
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_6_1_MISSION_PRIMARY_03_FLAVOR_03 ); 
	ScriptManager:wait( 6 );
	ScriptManager:dismissSubtitle();
	
	ScriptManager:stopCutscene();
	
	ScriptManager:setMissionStatus( missionIds.killTheGermanOfficers, MissionStatus.IN_PROGRESS );

end


function playSupplytruckCutscene()
				
	ScriptManager:startCutscene();
	
	ScriptManager:moveCameraEx( 13, 19, 0.7, 3, true );
	ScriptManager:revealFogOfWar( 13, 19, 2, true );
	
	local supplyTruck = ScriptManager:spawnUnit( 0, Units.UNIT_SU_SUPPLY_TRUCK, 14, 19, -1, AiState.WAIT ):getId();

			
	ScriptManager:wait( 1 );
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_6_1_MISSION_PRIMARY_01_FLAVOR_01 );
	
	ScriptManager:wait( 4 );
	
	ScriptManager:dismissSubtitle();
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_6_1_MISSION_PRIMARY_01_FLAVOR_02 );
	
	ScriptManager:wait( 4 );
	
	ScriptManager:dismissSubtitle();
	

	ScriptManager:moveUnitEx( supplyTruck, 12, 19, true );
	
	ScriptManager:wait( 2 );
	
	ScriptManager:spawnUnit( 0, Units.UNIT_SU_RIFLE, 13, 20, -1, AiState.WAIT );
	ScriptManager:wait( 1 );
	
	ScriptManager:spawnUnit( 0, Units.UNIT_SU_MORTAR, 12, 20, -1, AiState.WAIT );
	ScriptManager:wait( 1 );
	
	ScriptManager:spawnUnit( 0, Units.UNIT_SU_ANTI_INFANTRY, 11, 19, -1, AiState.WAIT );
	ScriptManager:wait( 1 );
	
	ScriptManager:revealFogOfWar( 13, 19, 2, false );
	
	ScriptManager:stopCutscene();
end



function onTigerDestroyed( unit )

	if unit:getId() == tigerNorthId then		
		ScriptManager:removeTrigger( "reduceTigerSouthAction" );
		Unit:changeSquadState( 1, enemySquadIds.tigerTank2, AiState.ATTACK_AGGRESSIVE );
		ScriptManager:setTargetTile( 1, enemySquadIds.tigerTank2, 4, 39 );
	end
	
	if unit:getId() == tigerSouthId then
		ScriptManager:removeTrigger( "reduceTigerNorthAction" );
		Unit:changeSquadState( 1, enemySquadIds.tigerTank1, AiState.ATTACK_AGGRESSIVE );
		ScriptManager:setTargetTile( 1, enemySquadIds.tigerTank1, 9, 5 );
	end

	ScriptManager:setMissionProgress( missionIds.killTheGermanOfficers, ScriptManager:getMissionProgress( missionIds.killTheGermanOfficers) + 1 );
	
	ShowMissionProgressPopup( ScriptManager:getMissionProgress( missionIds.killTheGermanOfficers ), Text.T_MAP_SU_6_1_MISSION_PRIMARY_03_COUNTER ); 
		
	ScriptManager:wait( 4 );
	
	ScriptManager:dismissSubtitle();
	
	-- squad one goes middle left
	ScriptManager:spawnUnits( 1, Units.UNIT_DE_MP40, 1, 23, enemySquadIds.squadOneAfterFirstTigerDead, AiState.ATTACK_AGGRESSIVE, 1 );
	ScriptManager:spawnUnits( 1, Units.UNIT_DE_PANZERFAUST, 1, 22, enemySquadIds.squadOneAfterFirstTigerDead, AiState.ATTACK_AGGRESSIVE, 1 );
	ScriptManager:setTargetTile( 1, enemySquadIds.squadOneAfterFirstTigerDead, 5, 24 );
	
	-- squad two goes middle right
	ScriptManager:spawnUnits( 1, Units.UNIT_DE_MP40, 1, 21, enemySquadIds.squadTwoAfterFirstTigerDead, AiState.ATTACK_AGGRESSIVE, 1 );
	ScriptManager:spawnUnits( 1, Units.UNIT_DE_PANZERFAUST, 1, 20, enemySquadIds.squadTwoAfterFirstTigerDead, AiState.ATTACK_AGGRESSIVE, 1 );
	ScriptManager:setTargetTile( 1, enemySquadIds.squadTwoAfterFirstTigerDead, 9, 24 );
	
	if ScriptManager:getMissionProgress( missionIds.killTheGermanOfficers ) >= officersToKill then
	
	DisablePlayerInteraction();
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_6_1_MISSION_PRIMARY_03_FLAVOR_04 );
	ScriptManager:wait( 5 );
	ScriptManager:dismissSubtitle();
	
	ScriptManager:setMissionStatus( missionIds.killTheGermanOfficers, MissionStatus.ACCOMPLISHED );
	ScriptManager:removeTrigger( "onTigerDestroyed" );
		
	
	checkForVictory()
	
	EnablePlayerInteraction();
	end
end


function onUnitLost( unit )

	if unit:getTemplateId() ~= Units.UNIT_SU_BUNKER then
	   
	ScriptManager:setMissionProgress( missionIds.doNotLoseMoreThanEightUnits, ScriptManager:getMissionProgress( missionIds.doNotLoseMoreThanEightUnits) + 1 );
	ScriptManager:runForPlayer( 0, "showUnitLostProgress" );
	
	ScriptManager:storeValue( "unitsLost", unitsLost );
	
		if ScriptManager:getMissionProgress( missionIds.doNotLoseMoreThanEightUnits ) >= unitsToLose then	
		ScriptManager:runForPlayer( 0, "onEightUnitsLost" );
		end
	end
end


function showUnitLostProgress()	
	ShowMissionProgressPopup( ScriptManager:getMissionProgress( missionIds.doNotLoseMoreThanEightUnits ), Text.T_MAP_SU_6_1_MISSION_PRIMARY_02_COUNTER );
end


function onEightUnitsLost()

	DisablePlayerInteraction();
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_6_1_MISSION_PRIMARY_02_FLAVOR_01 );
	ScriptManager:wait( 5 );
	ScriptManager:dismissSubtitle();
	
	ScriptManager:setMissionStatus(missionIds.doNotLoseMoreThanEightUnits, MissionStatus.FAILED );
	ScriptManager:removeTrigger( "onUnitLost" );
	
	unitsAreAlive = 0;
	
	missionFailed()
end


function playOutpostCutscene()

	ScriptManager:startCutscene();
	
	ScriptManager:moveCameraEx( 3, 6, 0.7, 2, true );
	ScriptManager:wait( 1 );
	ScriptManager:revealFogOfWar( 3, 6, 1, true );
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_6_1_MISSION_SECONDARY_01_FLAVOR_01 );
	
	ScriptManager:wait( 3 );
	
	ScriptManager:moveCameraEx( 9, 39, 0.7, 2, true );
	ScriptManager:wait( 1 );
	ScriptManager:revealFogOfWar( 9, 39, 1, true );
	ScriptManager:wait( 3 );
	
	ScriptManager:dismissSubtitle();
	
	ScriptManager:revealFogOfWar( 3, 6, 1, false );
	ScriptManager:revealFogOfWar( 9, 39, 1, false );
	
	ScriptManager:moveCameraEx( 6, 25, 0.7, 2, true );
	
	ScriptManager:wait( 1 );
	
	ScriptManager:stopCutscene();
	
	ScriptManager:setMissionStatus( missionIds.CaptureOrDestroyBothOutposts, MissionStatus.IN_PROGRESS );

end


function onOutpostCaptured()
	
	ScriptManager:setMissionProgress(missionIds.CaptureOrDestroyBothOutposts, ScriptManager:getMissionProgress(missionIds.CaptureOrDestroyBothOutposts) +1 );
	ShowMissionProgressPopup( ScriptManager:getMissionProgress( missionIds.CaptureOrDestroyBothOutposts ), Text.T_MAP_SU_6_1_MISSION_SECONDARY_01_COUNTER );
	
	if ScriptManager:getMissionProgress(missionIds.CaptureOrDestroyBothOutposts ) >= outpostsToCapture then
		
	DisablePlayerInteraction();
	
	ScriptManager:setMissionStatus(missionIds.CaptureOrDestroyBothOutposts, MissionStatus.ACCOMPLISHED );	
	ScriptManager:removeTrigger( "onOutpostCaptured" );
	
	EnablePlayerInteraction()
	
	end	
end


-- END
-- BEGIN Victory


function checkForVictory()
	
	if  ScriptManager:getMissionStatus( missionIds.defeat25Attackers ) 		== MissionStatus.ACCOMPLISHED and

		ScriptManager:getMissionStatus( missionIds.killTheGermanOfficers ) 	== MissionStatus.ACCOMPLISHED then
		
		checkTheSurvivors()
	end
end


function checkTheSurvivors()	
	if  ScriptManager:getMissionStatus( missionIds.doNotLoseMoreThanEightUnits ) ==	MissionStatus.IN_PROGRESS  then
		ScriptManager:setMissionStatus( missionIds.doNotLoseMoreThanEightUnits, MissionStatus.ACCOMPLISHED );
		ScriptManager:wait( 2 );
	end	
		startVictoryCutscene()
end


function startVictoryCutscene()
	StartCutscene();
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE_6_1_MISSION_OUTRO_FLAVOR_01 );
	ScriptManager:wait( 4 );
	ScriptManager:dismissSubtitle();
	
	ScriptManager:displaySubtitle( Text.T_MAP_DE_6_1_MISSION_OUTRO_FLAVOR_02 );
	ScriptManager:wait( 3 );
	ScriptManager:dismissSubtitle();
	
	HideDialogue();
	
	EndCutscene();
	
	victoryEndGame();
end

function victoryEndGame()
	DisablePlayerInteraction();
	HideDialogue(); -- remove possible texts on the screens

	-- accomplish secondary missions that are accomplished when the game is finished with certain condition
	
	
	ScriptManager:displaySubtitle( Text.T_MISSION_VICTORY );
	ScriptManager:wait( 4 );
	HideDialogue();
	endGame();
end


-- END Victory
-- BEGIN Defeated


function checkForEnd()
	if ScriptManager:noUnitsLeft( 0 ) then
		ScriptManager:runForPlayer( 0, "missionFailed" )
	end
end

function missionFailed()
	DisablePlayerInteraction();
	HideDialogue(); -- remove possible texts on the screens
	
	ScriptManager:displaySubtitle( Text.T_MISSION_FAILED );
	ScriptManager:wait( 4 );
	HideDialogue();
	endGame();
end


-- END Defeated


function endGame()
	ScriptManager:endGame()
end