require "lua/Constants"
require "lua/Functions"
require "lua/Functions_2"
require "lua/Text"
require "lua/Units"
require "math"


-- mission stuff
missionIds = {
		meetYourComrades 	= 1, 
		sabotageTankfactory	= 2,
		regainSupplyTrucks	= 3, -- when regained, you have to bring them back to the meeting point, BUT: some units spawn (south) that pursue you and the trucks back to the meeting point
		doNotLetThemSeeYou	= 4,
		destroyTankFactory	= 5,
		killAllDepotGuards	= 6,
};

-- missionId, primary( boolean ), textkey, xpReward
missions = {
	{ missionIds.meetYourComrades, 	 true, Text.T_MAP_SU_6_2_MISSION_PRIMARY_01,	175 },
	{ missionIds.sabotageTankfactory,true, Text.T_MAP_SU_6_2_MISSION_PRIMARY_02,	50 },
	{ missionIds.regainSupplyTrucks, true, Text.T_MAP_SU_6_2_MISSION_PRIMARY_03,	 50 },
	
	{ missionIds.doNotLetThemSeeYou, false, Text.T_MAP_SU_6_2_MISSION_SECONDARY_01,	 50 },
	{ missionIds.destroyTankFactory, false, Text.T_MAP_SU_6_2_MISSION_SECONDARY_02,	 50 },
	{ missionIds.killAllDepotGuards, false, Text.T_MAP_SU_6_2_MISSION_SECONDARY_03,	 50 },
	
};

enemySquadIds = {

-- right side of the river

depotGuards = 1,
tankFactoryGuards = 2,
reinforcementsMiddle = 3,
unitsMiddle = 4,
tankFactoryLeft = 5,
unitsMiddleRight = 6, -- between depot and TF

reinforcementsWoods = 10,
reinforcementsWoods2 = 11,
reinforcementsWoods3 = 11,

-- squads left of the river:
upLeft = 12,
upRight = 13,
middleLeft = 14,
middleRight = 15,

mgNestSouth = 16,
mgNestNorth = 17,

south = 18,

};


depotGuardsIds = { 15, 16, 17, 31 };


reinforcementsMiddle = {
-- playerId, templateID, tileX, tileY, squadId, AiState
	{ 1, Units.UNIT_DE_MP40, 			28, 12, enemySquadIds.reinforcementsMiddle, AiState.ATTACK_AGGRESSIVE };
	{ 1, Units.UNIT_DE_PANZERFAUST, 	28, 12, enemySquadIds.reinforcementsMiddle, AiState.ATTACK_AGGRESSIVE };
	{ 1, Units.UNIT_DE_ANTI_INFANTRY,	28, 12, enemySquadIds.reinforcementsMiddle, AiState.ATTACK_AGGRESSIVE };
	{ 1, Units.UNIT_DE_TANK_3,			28, 12, enemySquadIds.reinforcementsMiddle, AiState.ATTACK_AGGRESSIVE };
};


depotGuardsToKill = 4;

trucksLost = 2;


-- BEGIN Init


function onGameStart()
	CreateMissions( missions );
	setTriggers();
	storeValues();
	
	ScriptManager:gainGold( 0, 1200 );
	
	-- start intro cutscene
	ScriptManager:sleep( 0, "startIntroCutscene" ); -- without sleep player turn popup shows
end

function storeValues()
	
end

function setTriggers()
		
	-- load all needed values after loading
	ScriptManager:addOnGameResumeTrigger( "loadValues", "loadValues", false );
	
	-- check if the player has units left
	ScriptManager:addOnUnitDestroyedTrigger( 0, -1, -1, "checkForEnd", "checkForEnd" , false );
	
	-- meeting points where your units unite
	ScriptManager:addOnEnterTileTrigger( 15, 19, 0, -1, -1, true, "onMeetingPointReached", "onMeetingPointReached", false );
	ScriptManager:addOnEnterTileTrigger( 15, 18, 0, -1, -1, true, "onMeetingPointReached", "onMeetingPointReached", false );
	ScriptManager:addOnEnterTileTrigger( 16, 18, 0, -1, -1, true, "onMeetingPointReached", "onMeetingPointReached", false )	;
	ScriptManager:addOnEnterTileTrigger( 15, 17, 0, -1, -1, true, "onMeetingPointReached", "onMeetingPointReached", false )	;
	ScriptManager:addOnEnterTileTrigger( 16, 17, 0, -1, -1, true, "onMeetingPointReached", "onMeetingPointReached", false )	;
	ScriptManager:addOnEnterTileTrigger( 15, 20, 0, -1, -1, true, "onMeetingPointReached", "onMeetingPointReached", false )	;
	ScriptManager:addOnEnterTileTrigger( 14, 22, 0, -1, -1, true, "onMeetingPointReached", "onMeetingPointReached", false )	;
	
	
	-- when the enemy sees you
--	ScriptManager:addOnUnitSightedTrigger( 0, -1, -1, false, "onUnitSpotted", "onUnitSpotted", false );
	
	ScriptManager:addOnUnitHasAttackedTrigger( 1, -1, -1, "onUnitSpotted", "onUnitSpotted", false );
	ScriptManager:addOnUnitHasAttackedTrigger( 0, -1, -1, "onUnitSpotted", "onUnitSpotted", false );
	
	-- steal back the trucks
	ScriptManager:addOnUnitCapturedTrigger( 0, Units.UNIT_DE_HQ, -1, "onDepotCaptured", "onDepotCaptured", false );
	ScriptManager:addOnUnitDestroyedTrigger( 1, Units.UNIT_DE_HQ, -1, "onDepotCaptured", "onDepotCaptured", false );
	
	
	-- When a depot guard dies
	ScriptManager:addOnUnitDestroyedTrigger( 1, -1, -1, "onDepotGuardDestroyed", "onDepotGuardDestroyed", false );
	
	
	-- on Unit Destroyed 
	-- --> When the tank factory is destroyed
	
	-- When the tank factory is at half health
	ScriptManager:addOnUnitHasAttackedTrigger( -1, Units.UNIT_DE_TANKFACTORY_UNCAPTURABLE, -1, "onTankfactoryAttacked", "onTankfactoryAttacked", false );
	
	-- when the tank factory is destroyed
	ScriptManager:addOnUnitDestroyedTrigger( 1, Units.UNIT_DE_TANKFACTORY_UNCAPTURABLE, -1, "onTankfactoryDestroyed", "onTankfactoryDestroyed", false );
	 
	 
	-- set the action points for depot guards to 0
	ScriptManager:addOnTurnStartTrigger( -1, -1, -1, "reduceDepotGuardsActionPoints", "reduceDepotGuardsActionPoints", false );
	 
	 
	-- when the stolen supplyTrucks reach the road in the north
	ScriptManager:addOnEnterTileTrigger( 17, 22, 0, Units.UNIT_SU_SUPPLY_TRUCK, -1, true, "onSupplyTruckReachedRoad", "onSupplyTruckReachedRoad", false );
	
	
	
	-- cutscene to sec. obj.: kill all the guards
--	local x

	
--	for x = 19, 28 do
--	ScriptManager:addOnEnterTileTrigger( x, 11, 0, -1, -1, true, "checkForMissions","checkForMissions",false );
--	end
	local y
	for y = 1, 16 do
	ScriptManager:addOnEnterTileTrigger( 16, y, 0, -1, -1, true, "checkForMissions", "checkForMissions", false );
	end	

	-- entertiletrigger:
	-- "depotGuardCutscene"
	
	local x
	for x = 20, 28 do
	ScriptManager:addOnEnterTileTrigger( x, 12, 0, -1, -1, true, "depotGuardCutscene", "depotGuardCutscene", false );
	end
	
	local y
	for y = 1, 12 do
	ScriptManager:addOnEnterTileTrigger( 20, y, 0, -1, -1, true, "depotGuardCutscene", "depotGuardCutscene", false );
	end	
end


function loadValues()
	
end


-- END Init
-- BEGIN Intro Cutscene


function startIntroCutscene()
	StartCutscene();
	
	
	ScriptManager:startCutscene();
	
	ScriptManager:moveCameraEx( 2, 2, 1, 1, true );
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_6_2_MISSION_INTRO_FLAVOR_01 );
	ScriptManager:wait( 6 );
	ScriptManager:dismissSubtitle();
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_6_2_MISSION_INTRO_FLAVOR_02 ); 
	ScriptManager:wait( 6 );
	ScriptManager:dismissSubtitle();
	
	ScriptManager:spawnUnit( 0, Units.UNIT_SU_ANTI_INFANTRY_ELITE, 2, 1, -1, AiState.WAIT );
	ScriptManager:spawnUnit( 0, Units.UNIT_SU_RIFLE_ELITE, 2, 2, -1, AiState.WAIT );
	ScriptManager:spawnUnit( 0, Units.UNIT_SU_MORTAR_ELITE, 1, 1, -1, AiState.WAIT );
	ScriptManager:spawnUnit( 0, Units.UNIT_SU_RIFLE_ELITE, 1, 2, -1, AiState.WAIT );
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_6_2_MISSION_INTRO_FLAVOR_03 );
	ScriptManager:moveCameraEx( 2, 1, 1, 1, true );

	
	ScriptManager:wait( 5 );
	ScriptManager:dismissSubtitle();
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_6_2_MISSION_INTRO_FLAVOR_04 );
	ScriptManager:wait( 6 );
	ScriptManager:dismissSubtitle();


	ScriptManager:displaySubtitle( Text.T_MAP_SU_6_2_MISSION_INTRO_FLAVOR_05 );	
 
	ScriptManager:moveCameraEx( 15, 19, 1, 3, true );
	ScriptManager:revealFogOfWar( 15, 19, 1, true );
	ScriptManager:wait( 5 );
	ScriptManager:dismissSubtitle();
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_6_2_MISSION_INTRO_FLAVOR_06 );
	ScriptManager:wait( 5 );
	ScriptManager:dismissSubtitle();
	
	ScriptManager:revealFogOfWar( 15, 19, 1, false );
	
	ScriptManager:moveCameraEx( 7, 18, 0.7, 2, true );
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_6_2_MISSION_INTRO_FLAVOR_07 );

	ScriptManager:wait( 1.5 );

	ScriptManager:moveCameraEx( 3, 20, 0.7, 2, true );
	
	ScriptManager:wait( 1.5 );
	
	ScriptManager:moveCameraEx( 6, 12, 0.7, 2, true );
	
	ScriptManager:wait( 1.5 );
	
	ScriptManager:moveCameraEx( 10, 10, 0.7, 2, true );
	
	ScriptManager:wait( 1.5 );
	ScriptManager:dismissSubtitle();
	
	ScriptManager:moveCameraEx( 2, 2, 0.7, 2, true );
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_6_2_MISSION_INTRO_FLAVOR_08 );
	ScriptManager:wait( 4 );
	ScriptManager:dismissSubtitle();
	
	ScriptManager:displaySubtitle(Text. T_MAP_SU_6_2_MISSION_INTRO_FLAVOR_09 );

	ScriptManager:wait( 3 );
	ScriptManager:dismissSubtitle();

	ScriptManager:stopCutscene();	
		
	startTheMission();
end

function startTheMission()
	ScriptManager:setMissionStatus( missionIds.meetYourComrades, MissionStatus.IN_PROGRESS );
	ScriptManager:setMissionStatus( missionIds.doNotLetThemSeeYou, MissionStatus.IN_PROGRESS );
	
	ScriptManager:fortifyUnit( 15 );
	ScriptManager:fortifyUnit( 16 );
	ScriptManager:fortifyUnit( 17 );
	ScriptManager:fortifyUnit( 31 );
	

	--ScriptManager:addOnUnitSelectedTrigger( 1, -1, -1, "showParameters", "showParameters",false)
end



function removeSquads()
	--for i = 1, # do
	--	ScriptManager:removeUnitEx( [i], false );
	--end
end



function showParameters( selectedUnit )
	--print( string.format( "[] %s", selectedUnit:get() ) );
end



-- END Intro Cutscene
-- BEGIN


-- insert missionstuff here


function onMeetingPointReached()

	ScriptManager:removeTrigger( "onMeetingPointReached" );
	
	
	ScriptManager:startCutscene();
	
	ScriptManager:moveCameraEx( 16, 19, 0.7, 2, true );
	ScriptManager:wait( 1 );
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_6_2_MISSION_PRIMARY_01_FLAVOR_01 );
	ScriptManager:wait( 3 )
	ScriptManager:dismissSubtitle();
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_6_2_MISSION_PRIMARY_01_FLAVOR_02 );
	ScriptManager:wait( 4 );
	ScriptManager:dismissSubtitle();
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_6_2_MISSION_PRIMARY_01_FLAVOR_03 );
	ScriptManager:wait( 2 );
	
	ScriptManager:dismissSubtitle();
	
	ScriptManager:wait( 1 );
	
	
	local AntiInfantry = ScriptManager:spawnUnit( 0, Units.UNIT_SU_ANTI_INFANTRY_ELITE, 17, 22, -1, AiState.WAIT ):getId();
	ScriptManager:wait( 0.5 );
	ScriptManager:moveUnitEx( AntiInfantry, 16, 19, true );
	
	local AntiInfantry2 = ScriptManager:spawnUnit( 0, Units.UNIT_SU_ANTI_INFANTRY_ELITE, 17, 22, -1, AiState.WAIT ):getId();
	ScriptManager:wait( 0.5 );
	ScriptManager:moveUnitEx( AntiInfantry2, 16, 21, true );
		
	local Rifle = ScriptManager:spawnUnit( 0, Units.UNIT_SU_RIFLE_ELITE, 17, 22, -1, AiState.WAIT ):getId();
	ScriptManager:wait( 0.5 );
	ScriptManager:moveUnitEx( Rifle, 15, 20, true );
	
	local Mortar = ScriptManager:spawnUnit( 0, Units.UNIT_SU_MORTAR_ELITE, 17, 22, -1, AiState.WAIT ):getId();
	ScriptManager:wait( 0.5 );
	ScriptManager:moveUnitEx( Mortar, 16, 20, true );
	
	local Mortar2 = ScriptManager:spawnUnit( 0, Units.UNIT_SU_MORTAR_ELITE, 17, 22, -1, AiState.WAIT ):getId();
	ScriptManager:wait( 0.5 )
	ScriptManager:moveUnitEx( Mortar2, 17, 20, true );
	
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_6_2_MISSION_PRIMARY_02_FLAVOR_01 ); 
	
	ScriptManager:wait( 4 );
	
	ScriptManager:dismissSubtitle();
	
	ScriptManager:stopCutscene();
	
	DisablePlayerInteraction();
		
	ScriptManager:setMissionStatus( missionIds.meetYourComrades, MissionStatus.ACCOMPLISHED );
	
	checkForVictory()
	
	ScriptManager:wait( 3 );
	
	EnablePlayerInteraction();
		
	if 
	ScriptManager:getMissionStatus( missionIds.doNotLetThemSeeYou ) == MissionStatus.IN_PROGRESS then
	ScriptManager:removeTrigger( "onUnitSpotted" );
	ScriptManager:setMissionStatus( missionIds.doNotLetThemSeeYou, MissionStatus.ACCOMPLISHED );
	end
	ScriptManager:wait( 3 );
	

	if  ScriptManager:getMissionStatus( missionIds.sabotageTankfactory ) ~= MissionStatus.IN_PROGRESS and
		ScriptManager:getMissionStatus( missionIds.sabotageTankfactory ) ~= MissionStatus.ACCOMPLISHED and
	
		ScriptManager:getMissionStatus( missionIds.regainSupplyTrucks) ~= MissionStatus.IN_PROGRESS and
		ScriptManager:getMissionStatus( missionIds.regainSupplyTrucks )~= MissionStatus.ACCOMPLISHED then

		startTankFactoryAndDepotCutscene()
	end
end


function checkForMissions()
	
	ScriptManager:removeTrigger( "checkForMissions" );

	if ScriptManager:getMissionStatus( missionIds.doNotLetThemSeeYou ) ~= MissionStatus.ACCOMPLISHED and
	   ScriptManager:getMissionStatus( missionIds.doNotLetThemSeeYou ) ~= MissionStatus.FAILED then
	   
		ScriptManager:setMissionStatus( missionIds.doNotLetThemSeeYou, MissionStatus.ACCOMPLISHED );
		ScriptManager:removeTrigger( "onUnitSpotted" );
		ScriptManager:wait( 3 );
	end	

	if ScriptManager:getMissionStatus( missionIds.meetYourComrades ) ~= MissionStatus.ACCOMPLISHED then
	startTankFactoryAndDepotCutscene()
	end
end



function startTankFactoryAndDepotCutscene()

	-- move camera to the tank factory (you have to sabotage it...) and the depot (you have to regain two of out trucks which got stolen)
	ScriptManager:startCutscene();
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_6_2_MISSION_PRIMARY_02_FLAVOR_02 );
	
	ScriptManager:wait( 5 );
	
	ScriptManager:moveCameraEx( 25, 20, 0.7, 2, true );
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_6_2_MISSION_PRIMARY_02_FLAVOR_03 ); 
		
	ScriptManager:wait( 5 );
	
	ScriptManager:dismissSubtitle();
	
	ScriptManager:displaySubtitle(Text.T_MAP_SU_6_2_MISSION_PRIMARY_02_FLAVOR_04 );
	
	ScriptManager:wait( 6 );
	
	ScriptManager:dismissSubtitle();
	
	ScriptManager:moveCameraEx( 25, 5, 0.7, 3, true );
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_6_2_MISSION_PRIMARY_03_FLAVOR_01 );
	
	ScriptManager:wait( 4 );
	
	ScriptManager:dismissSubtitle();
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_6_2_MISSION_PRIMARY_03_FLAVOR_02 );
	
	ScriptManager:wait( 5 );
	
	ScriptManager:dismissSubtitle();
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_6_2_MISSION_PRIMARY_03_FLAVOR_03 );
		
	ScriptManager:moveCameraEx( 17, 22, 0.7, 2, true );
	
	ScriptManager:revealFogOfWar( 17, 22, 1, true );
	ScriptManager:showMarkerAt( 17, 22 );
	
	ScriptManager:wait( 3 );

	ScriptManager:hideMarkerAt( 17, 22 ); 	

	ScriptManager:revealFogOfWar( 17, 22, 1, false );

	ScriptManager:dismissSubtitle();
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_6_2_MISSION_PRIMARY_03_FLAVOR_04 );
	
	ScriptManager:wait( 4 );
	
	ScriptManager:dismissSubtitle();
		
	ScriptManager:stopCutscene();

	ScriptManager:setMissionStatus( missionIds.sabotageTankfactory, MissionStatus.IN_PROGRESS );
	
	ScriptManager:setMissionStatus( missionIds.regainSupplyTrucks, MissionStatus.IN_PROGRESS );
	
	EnablePlayerInteraction();
end



function onUnitSpotted( seeingUnit, sightedUnit )

	ScriptManager:removeTrigger( "onUnitSpotted" );

	ScriptManager:runForPlayer( 0, "playAttackedCutscene" );

	seeingUnit:getId()
	seeingUnit:getPlayerId()
--	seeingUnit:getX() 
--	seeingUnit:getY()
	seeingUnit:getSquadId()
		
--	local sightedUnit = sightedUnit:getId()	
--	local sightedUnitPlayer = sightedUnit:getPlayerId()	
--	local sightedUnitX = sightedUnit:getX()
--	local sightedUnitY = sightedUnit:getY()

end


function playAttackedCutscene()

	DisablePlayerInteraction();	
	
	ScriptManager:startCutscene();
	
--	ScriptManager:moveCameraEx( seeingUnit:getX(), seeingUnit:getY(), 0.7, 2, true );
 
 	ScriptManager:displaySubtitle( Text.T_MAP_SU_6_2_MISSION_PRIMARY_01_FLAVOR_04 );
	ScriptManager:wait( 6 );
	
	ScriptManager:dismissSubtitle();
	
	ScriptManager:stopCutscene();

--	Unit:changeSquadState( 1, seeingUnit:getSquadId(), AiState.ATTACK_AGGRESSIVE );
	
	ScriptManager:setMissionStatus( missionIds.doNotLetThemSeeYou, MissionStatus.FAILED );
	
	ScriptManager:wait( 2 );
	
	EnablePlayerInteraction();
	

end



function onTankfactoryAttacked( activeUnit, passiveUnit )

-- when tank factory is at 90%
	if passiveUnit:getHealth() < 2700 and passiveUnit:getHealth() > 2400 then
		ScriptManager:displaySubtitle(Text.T_MAP_SU_6_2_MISSION_PRIMARY_02_FLAVOR_05 );
		ScriptManager:wait( 2 );
		ScriptManager:dismissSubtitle();
	end


-- when tank factory is at 80%
	if	
	passiveUnit:getHealth() < 2400 and passiveUnit:getHealth() > 2100 then
		ScriptManager:displaySubtitle(Text.T_MAP_SU_6_2_MISSION_PRIMARY_02_FLAVOR_06 );
		ScriptManager:wait( 2 );
		ScriptManager:dismissSubtitle();
	end

-- when tank factory is at 70%
	if	
	passiveUnit:getHealth() < 2100  and passiveUnit:getHealth() > 1800 then
		ScriptManager:displaySubtitle(Text.T_MAP_SU_6_2_MISSION_PRIMARY_02_FLAVOR_07 );
		ScriptManager:wait( 2 );
		ScriptManager:dismissSubtitle();
	end

-- when tank factory is at 60%
	if	
	passiveUnit:getHealth() < 1800 and passiveUnit:getHealth() > 1500 then
		ScriptManager:displaySubtitle(Text.T_MAP_SU_6_2_MISSION_PRIMARY_02_FLAVOR_08 );
		ScriptManager:wait( 2 );
		ScriptManager:dismissSubtitle();
	end
	
	
	if passiveUnit:getHealth() < 1500 then

	DisablePlayerInteraction();

	ScriptManager:removeTrigger( "onTankfactoryAttacked" );
		
	ScriptManager:setMissionStatus( missionIds.sabotageTankfactory, MissionStatus.ACCOMPLISHED );	
	
	--TODO
	
	if ScriptManager:getMissionStatus( missionIds.meetYourComrades ) 	== MissionStatus.ACCOMPLISHED and
	   ScriptManager:getMissionStatus( missionIds.regainSupplyTrucks )  == MissionStatus.ACCOMPLISHED then
	
		ScriptManager:removeMission( missionIds.destroyTankFactory );
	end

	--TODO
	
	checkForVictory()

	ScriptManager:wait( 3 );
		
	ScriptManager:startCutscene();
	
	ScriptManager:moveCameraEx( 25, 20, 0.7, 2, true );
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_6_2_MISSION_PRIMARY_02_FLAVOR_09 );
	ScriptManager:wait( 6 );
	ScriptManager:dismissSubtitle()
		
--	ScriptManager:stopCutscene();
--				
--	playDestroyTankFactoryCutscene()
--	
--	end
--end 
--
--	 
--function playDestroyTankFactoryCutscene()

--	ScriptManager:startCutscene();
	
	ScriptManager:moveCameraEx( 25, 20, 0.7, 2, true );
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_6_2_MISSION_SECONDARY_02_FLAVOR_01 );
	
	ScriptManager:wait( 6 );
	
	ScriptManager:dismissSubtitle();
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_6_2_MISSION_SECONDARY_02_FLAVOR_02 );
	
	ScriptManager:wait( 5 );
	
	ScriptManager:dismissSubtitle();
		
	ScriptManager:displaySubtitle( Text.T_MAP_SU_6_2_MISSION_SECONDARY_02_FLAVOR_03 );
	
	ScriptManager:wait( 3 );
	
	ScriptManager:dismissSubtitle();
	
	ScriptManager:moveCameraEx( 28, 12, 0.7, 2, true );
	
	ScriptManager:revealFogOfWar( 28, 12, 1, true );

	for i = 1, #reinforcementsMiddle do
	local data = reinforcementsMiddle[i]
	local unit = ScriptManager:spawnUnit( data[1], data[2], data[3], data[4], data[5], data[6] );
	end
	
	ScriptManager:setTargetTile( 1, enemySquadIds.reinforcementsMiddle, 24, 19 );

	ScriptManager:displaySubtitle( Text.T_MAP_SU_6_2_MISSION_SECONDARY_02_FLAVOR_04 );
	
	ScriptManager:wait( 5 );
	
	ScriptManager:revealFogOfWar( 28, 12, 1,false );
	
	ScriptManager:dismissSubtitle();

	ScriptManager:stopCutscene()
	
	ScriptManager:setMissionStatus( missionIds.destroyTankFactory, MissionStatus.IN_PROGRESS );

	EnablePlayerInteraction()
	end
end




function onTankfactoryDestroyed()

	ScriptManager:displaySubtitle( Text.T_MAP_SU_6_2_MISSION_SECONDARY_02_FLAVOR_05 );
	
	ScriptManager:wait( 3 );
	
	ScriptManager:dismissSubtitle();
	
	ScriptManager:setMissionStatus( missionIds.destroyTankFactory, MissionStatus.ACCOMPLISHED );
	
end



function depotGuardCutscene()

	ScriptManager:removeTrigger( "depotGuardCutscene" );

	ScriptManager:startCutscene();
	
	ScriptManager:moveCameraEx( 25, 5, 0.7, 2, true );
	
	ScriptManager:revealFogOfWar( 25, 5, 1, true );
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_6_2_MISSION_SECONDARY_03_FLAVOR_01 );
	
	ScriptManager:wait( 4 );
	
	ScriptManager:dismissSubtitle();
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_6_2_MISSION_SECONDARY_03_FLAVOR_02 );
	
	ScriptManager:wait( 5 );
	
	ScriptManager:dismissSubtitle();
	
	ScriptManager:revealFogOfWar( 25, 5, 1, false );
	
	ScriptManager:wait( 1 );
	
	ScriptManager:stopCutscene();
	
	ScriptManager:setMissionStatus( missionIds.killAllDepotGuards, MissionStatus.IN_PROGRESS );
	
end



function reduceDepotGuardsActionPoints()

	-- set the maximal action points of the depot guards 0
	local unit
	for i = 1, #depotGuardsIds do
		unit = Unit:getUnitById( depotGuardsIds[i] );		
		
		if unit ~= nil then
			unit:setMaxActionPoints ( 0 );
		end
	end	
end



function onDepotGuardDestroyed( unit )
		
	if unit:getSquadId() == enemySquadIds.depotGuards then
	ScriptManager:setMissionProgress(missionIds.killAllDepotGuards, ScriptManager:getMissionProgress(missionIds.killAllDepotGuards) +1 );
--	ScriptManager:storeValue( "depotGuardsKilled", depotGuardsKilled );
	ShowMissionProgressPopup( ScriptManager:getMissionProgress( missionIds.killAllDepotGuards ), Text.T_MAP_SU_6_2_MISSION_SECONDARY_03_COUNTER ); 
	
		if ScriptManager:getMissionProgress( missionIds.killAllDepotGuards ) >= depotGuardsToKill then
		ScriptManager:setMissionStatus( missionIds.killAllDepotGuards, MissionStatus.ACCOMPLISHED );
		ScriptManager:removeTrigger( "onDepotGuardDestroyed" );
		
		end		
	end	
end



function onDepotCaptured()
	
	ScriptManager:removeTrigger( "onDepotCaptured" );
	
	ScriptManager:wait( 2 );

	ScriptManager:spawnUnits( 0, Units.UNIT_SU_SUPPLY_TRUCK, 26, 5, -1, AiState.WAIT, 2 );
	
	ScriptManager:wait( 2 );
	
	onSupplyTruckSaved()

end



function onSupplyTruckSaved()
	
	ScriptManager:startCutscene();
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_6_2_MISSION_PRIMARY_03_FLAVOR_05 );
	
	ScriptManager:wait( 4 );
	
	ScriptManager:dismissSubtitle();
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_6_2_MISSION_PRIMARY_03_FLAVOR_06 );
	
	ScriptManager:wait( 5 );
	
	ScriptManager:dismissSubtitle();
	
	ScriptManager:stopCutscene();
	
	-- spawn German units in the northern woods, move the close to the road
	local unit1 = ScriptManager:spawnUnit( 1, Units.UNIT_DE_MP40, 10, 22, enemySquadIds.reinforcementsWoods, AiState.DEFEND );
	local unit2 = ScriptManager:spawnUnit( 1, Units. UNIT_DE_MP40, 9, 22, enemySquadIds.reinforcementsWoods, AiState.DEFEND );
	local unit3 = ScriptManager:spawnUnit( 1, Units.UNIT_DE_PANZERFAUST, 11, 22, enemySquadIds.reinforcementsWoods, AiState.DEFEND);
--	local unit4 = ScriptManager:spawnUnit( 1, Units.UNIT_DE_PANZERFAUST, 8, 22, enemySquadIds.reinforcementsWoods, AiState.DEFEND);
--	local unit5 = ScriptManager:spawnUnit( 1, Units. UNIT_DE_MP40, 12, 22, enemySquadIds.reinforcementsWoods3, AiState.DEFEND );
--	local unit6 = ScriptManager:spawnUnit( 1, Units.UNIT_DE_MP40, 1, 19, enemySquadIds.reinforcementsWoods3, AiState.DEFEND );
	
--	ScriptManager:setTargetTile( 1, enemySquadIds.reinforcementsWoods , 15, 21 );
--	ScriptManager:setTargetTile( 1, enemySquadIds.reinforcementsWoods2, 17, 20 );
--	ScriptManager:setTargetTile( 1, enemySquadIds.reinforcementsWoods3, 16, 15 );
	
	
	ScriptManager:addOnTurnStartTrigger( 1, -1, -1, "setGuardAreaForLastUnits", "setGuardAreaForLastUnits", false );
	
	ScriptManager:addOnUnitDestroyedTrigger( 0, Units.UNIT_SU_SUPPLY_TRUCK, -1, "onSupplytruckDestroyed", "onSupplytruckDestroyed", false );
	
end


function setGuardAreaForLastUnits()
	ScriptManager:setGuardArea( 1, enemySquadIds.reinforcementsWoods, 15, 19, 2 );
end



function onSupplytruckDestroyed( unit )
	
	if unit:getTemplateId() == Units.UNIT_SU_SUPPLY_TRUCK then
	ScriptManager:setMissionProgress( missionIds.regainSupplyTrucks, ScriptManager:getMissionProgress( missionIds.regainSupplyTrucks ) +1 );
	ScriptManager:storeValue( "trucksLost", trucksLost );
		if ScriptManager:getMissionProgress( missionIds.regainSupplyTrucks ) >= trucksLost then
		ScriptManager:setMissionStatus( missionIds.regainSupplyTrucks, MissionStatus.FAILED );
		end
	else
	ShowMissionProgressPopup( ScriptManager:getMissionProgress( missionIds.regainSupplyTrucks ), Text.T_MAP_SU_6_2_MISSION_PRIMARY_03_COUNTER ); 
	end

end


function onSupplyTruckReachedRoad()
	
	ScriptManager:removeTrigger( "onSupplytruckDestroyed" );
	ScriptManager:removeTrigger( "onSupplyTruckReachedRoad" );

	DisablePlayerInteraction()

	ScriptManager:wait( 1 );
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_6_2_MISSION_SECONDARY_02_FLAVOR_05 );
	
	ScriptManager:wait( 3 );
	
	ScriptManager:dismissSubtitle();
	
	ScriptManager:setMissionStatus( missionIds.regainSupplyTrucks, MissionStatus.ACCOMPLISHED );
	
	ScriptManager:wait( 3 );
	
	local truck = Unit:getUnitAt( 17, 22 ):getId(); 
	ScriptManager:removeUnitEx( truck, true );
	
	EnablePlayerInteraction();
	
	checkForVictory()
end


function checkForVictory()
	
	if  ScriptManager:getMissionStatus( missionIds.meetYourComrades ) 	 == MissionStatus.ACCOMPLISHED and

		ScriptManager:getMissionStatus( missionIds.sabotageTankfactory ) == MissionStatus.ACCOMPLISHED and
		
		ScriptManager:getMissionStatus( missionIds.regainSupplyTrucks )  == MissionStatus.ACCOMPLISHED then
		
		startVictoryCutscene()
	end
end


-- END
-- BEGIN Victory


function startVictoryCutscene()
	StartCutscene();
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_6_2_MISSION_OUTRO_FLAVOR_01 );
	ScriptManager:wait( 5 );
	ScriptManager:dismissSubtitle();
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU_6_2_MISSION_OUTRO_FLAVOR_02 );
	ScriptManager:wait( 5 );
	ScriptManager:dismissSubtitle();
	
	HideDialogue();
	
	EndCutscene();
	
	victoryEndGame();
end

function victoryEndGame()
	DisablePlayerInteraction();
	HideDialogue(); -- remove possible texts on the screens

	-- accomplish secondary missions that are accomplished when the game is finished with certain condition
	
	
	ScriptManager:displaySubtitle( Text.T_MISSION_VICTORY );
	ScriptManager:wait( 4 );
	HideDialogue();
	endGame();
end


-- END Victory
-- BEGIN Defeated


function checkForEnd()
	if ScriptManager:noUnitsLeft( 0 ) then
		ScriptManager:runForPlayer( 0, "missionFailed" )
	end
end

function missionFailed()
	DisablePlayerInteraction();
	HideDialogue(); -- remove possible texts on the screens
	
	ScriptManager:displaySubtitle( Text.T_MISSION_FAILED );
	ScriptManager:wait( 4 );
	HideDialogue();
	endGame();
end


-- END Defeated


function endGame()
	ScriptManager:endGame()
end