require "lua/Constants"
require "lua/Functions"
require "lua/Functions_2"
require "lua/Text"
require "lua/Units"
require "math"


-- mission stuff
missionIds = {
	destroyBunkers	= 1,
	destroyEnemies	= 2,
	destroyMgNests	= 3,
	capOutpost		= 4,
};

-- missionId, primary( boolean ), textkey, xpReward
missions = {
	{ missionIds.destroyBunkers, true,	Text.T_MAP_SU7_1_MISSION_PRIMARY_01,	150 },
	{ missionIds.destroyEnemies, true,	Text.T_MAP_SU7_1_MISSION_PRIMARY_02,	175 },
	{ missionIds.destroyMgNests, false,	Text.T_MAP_SU7_1_MISSION_SECONDARY_01,	 75 },
	{ missionIds.capOutpost,	 false,	Text.T_MAP_DE7_2_MISSION_SECONDARY_01,	 75 }, -- mission was added later, has the same text
};

enemySquadIds = {
	firstCheckpoint		=  1, -- GUARD AREA
	secondCheckpoint	=  2, -- GUARD AREA
	southStreet			=  3, -- WAIT, may need guard are if not behaving
	firstAttackers		=  4, -- ATTACK
	southBridge			=  5, -- WAIT, may need guard are if not behaving
	southBridge2		=  6, -- WAIT, may need guard are if not behaving
	depot				=  7, -- DEFEND
	northStreet			=  8, -- WAIT, may need guard are if not behaving
	northStreet2		=  9, -- WAIT, may need guard are if not behaving
	firstBunkers		= 10, -- ATTACK
	secondBunkers		= 11, -- ATTACK
};

firstPointToDestroy		= 8; -- max
firstPointDestroyed		= 0; -- counter
secondPointToDestroy	= 9; -- max
secondPointDestroyed	= 0; -- counter
bunkersToDestroy		= 6;
mgNestsToDestroy		= 4;
checkpointsToDestroy	= 2;


-- BEGIN Init


function onGameStart()
	CreateMissions( missions );
	setTriggers();
	storeValues();
	setGuardArea();
	
	ScriptManager:gainGold( 0, 1000 );
	
	-- start intro cutscene
	ScriptManager:sleep( 0, "startIntroCutscene" ); -- without sleep player turn popup shows
end

function storeValues()
	ScriptManager:storeValue( "firstPointDestroyed",	firstPointDestroyed );
	ScriptManager:storeValue( "secondPointDestroyed",	secondPointDestroyed );
end

function setTriggers()
	-- load all needed values after loading
	ScriptManager:addOnGameResumeTrigger( "loadValues", "loadValues", false );
	
	-- check if the player has units left
	ScriptManager:addOnUnitDestroyedTrigger(	 0, Units.UNIT_SU_HQ, -1, "transitionToFail",				"transitionToFail",					false );
	ScriptManager:addOnUnitHasAttackedTrigger(	-1, Units.UNIT_SU_HQ, -1, "ProtectTheSupplyDepotHintSingle",	"ProtectTheSupplyDepotHintSingle",	false );
		
	-- trigger for destroying stuff
	ScriptManager:addOnUnitDestroyedTrigger( 1, -1, -1, "onEnemyDestroyed", "onEnemyDestroyed", false );
	
	-- trigger for starting mgNest mission
	ScriptManager:addOnUnitHasAttackedTrigger( -1, Units.UNIT_DE_MG_NEST, -1, "onMgNestAttacked", "onMgNestAttacked", false );
	
	-- trigger for outpost
	ScriptManager:addOnUnitCapturedTrigger(	 0, Units.UNIT_DE_HQ, -1, "onOutpostCaptured",	"onOutpostCaptured",	false );
	ScriptManager:addOnUnitDestroyedTrigger( 1, Units.UNIT_DE_HQ, -1, "onOutpostDestroyed",	"onOutpostDestroyed",	false );
end

function setGuardArea()
	ScriptManager:setGuardArea( 1, enemySquadIds.firstCheckpoint,	 4, 14, 3 );
	ScriptManager:setGuardArea( 1, enemySquadIds.secondCheckpoint,	17, 17, 3 );
end

function loadValues()
	firstPointDestroyed		= ScriptManager:getValue( "firstPointDestroyed" );
	secondPointDestroyed	= ScriptManager:getValue( "secondPointDestroyed" );
	
	setGuardArea();
	
	ScriptManager:disableInteractionButton( InteractionButton.TRANSPORT );
	ScriptManager:disableInteractionButton( InteractionButton.TRANSPORT_UNLOAD_1 );
	ScriptManager:disableInteractionButton( InteractionButton.TRANSPORT_UNLOAD_2 );
	ScriptManager:disableInteractionButton( InteractionButton.TRANSPORT_UNLOAD_3 );
end


-- END Init
-- BEGIN Intro Cutscene


function startIntroCutscene()
	StartCutscene();
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU7_1_MISSION_INTRO_FLAVOR_01 );
	ScriptManager:wait( 4 );
	HideDialogue();
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU7_1_MISSION_INTRO_FLAVOR_02 );
	ScriptManager:moveCameraEx( 3, 13, 0.7, 6, true );
	ScriptManager:wait( 1 );
	HideDialogue();
	ScriptManager:wait( 1 );
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU7_1_MISSION_INTRO_FLAVOR_03 );
	ScriptManager:moveCameraEx( 17, 17, 0.7, 6, true );
	ScriptManager:wait( 1 );
	HideDialogue();
	ScriptManager:wait( 1 );
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU7_1_MISSION_INTRO_FLAVOR_04 );
	ScriptManager:moveCameraEx( 19, 4, 0.7, 6, true );
	ScriptManager:wait( 1 );
	HideDialogue();
	
	EndCutscene();
	
	startTheMission();
end

function startTheMission()
	ScriptManager:setMissionStatus( missionIds.destroyBunkers,	MissionStatus.IN_PROGRESS );
	ScriptManager:setMissionStatus( missionIds.destroyEnemies,	MissionStatus.IN_PROGRESS );
	ScriptManager:setMissionStatus( missionIds.capOutpost,		MissionStatus.IN_PROGRESS );
	
	--TODO: remove this function!
	--testing();
	--removeSquads();
	--ScriptManager:addOnUnitSelectedTrigger( 1, -1, -1, "showParameters", "showParameters",false)
end

function testing()
	ScriptManager:spawnUnit( 0, Units.GOTT, 1, 2, 42, AiState.WAIT );
	ScriptManager:spawnUnit( 0, Units.GOTT, 2, 2, 42, AiState.WAIT );
end

function removeSquads()
	--for i = 1, # do
	--	ScriptManager:removeUnitEx( [i], false );
	--end
end

function showParameters( selectedUnit )
	--print( string.format( "[] %s", selectedUnit:get() ) );
end


-- END Intro Cutscene
-- BEGIN Capture Outpost


function updateOutpostMission( missionState )
	ScriptManager:removeTrigger( "onOutpostCaptured" );
	ScriptManager:removeTrigger( "onOutpostDestroyed" );
	
	DisablePlayerInteraction();
	
	ScriptManager:setMissionStatus( missionIds.capOutpost, missionState );
	ScriptManager:wait( 2 );
	
	EnablePlayerInteraction();
end

function onOutpostCaptured()
	updateOutpostMission( MissionStatus.ACCOMPLISHED );
end

function onOutpostDestroyed()
	updateOutpostMission( MissionStatus.FAILED );
end


-- END Capture Outpost
-- BEGIN Destroy Stuff


function onMgNestAttacked( attacker, defender)
	ScriptManager:removeTrigger( "onMgNestAttacked" );
	
	local mgNest, playerUnit;
	if attacker:getPlayerId() == 1 then
		mgNest		= attacker;
		playerUnit	= defender;
	
	else
		mgNest		= defender;
		playerUnit	= attacker;
	end
	
	-- save pos for cutscene below
	ScriptManager:storeValue( "mgNestX", mgNest:getX() );
	ScriptManager:storeValue( "mgNestY", mgNest:getY() );
	
	ScriptManager:storeValue( "playerX", playerUnit:getX() );
	ScriptManager:storeValue( "playerY", playerUnit:getY() );
	
	ScriptManager:runForPlayer( 0, "prepareMgNestCutscene" );
end

function prepareMgNestCutscene()
	table.insert( cutsceneQueue, startMgNestCutscene );
	CutsceneQueue();
end

function startMgNestCutscene()
	StartCutscene();
	
	local mgNestX = ScriptManager:getValue( "mgNestX" );
	local mgNestY = ScriptManager:getValue( "mgNestY" );
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU7_1_MISSION_SECONDARY_01_FLAVOR_01 );
	ScriptManager:moveCameraEx( mgNestX, mgNestY, 0.7, 3, true );
	MarkUnit(					mgNestX, mgNestY, 1, 3 )
	HideDialogue();
	
	local playerX = ScriptManager:getValue( "playerX" );
	local playerY = ScriptManager:getValue( "playerY" );
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU7_1_MISSION_SECONDARY_01_FLAVOR_02 );
	ScriptManager:moveCameraEx( playerX, playerY, 0.7, 3, true );
	HideDialogue();
	
	EndCutscene();
	DisablePlayerInteraction()
	
	ScriptManager:setMissionStatus( missionIds.destroyMgNests, MissionStatus.IN_PROGRESS );
	ScriptManager:wait( 2 );
	
	EnablePlayerInteraction();
end

function onEnemyDestroyed( unit )
	local templateId = unit:getTemplateId();
	
	if templateId == Units.UNIT_SU_BUNKER then
		print( "bunker" );
		onBunkerDestroyed();
	
	elseif templateId == Units.UNIT_DE_MG_NEST then
		print( "mgNest" );
		onMgNestDestroyed();
	
	else
		local squadId = unit:getSquadId();
		if squadId == enemySquadIds.firstCheckpoint or squadId == enemySquadIds.secondCheckpoint then
			print( "enemy" );
			onCheckpointUnitDestroyed( squadId );
		end
	end
end

function onBunkerDestroyed()
	ScriptManager:setMissionProgress( missionIds.destroyBunkers, ScriptManager:getMissionProgress( missionIds.destroyBunkers ) + 1 );
	
	print( "bunkers destroyed " .. ScriptManager:getMissionProgress( missionIds.destroyBunkers ) );
	
	ScriptManager:runForPlayer( 0, "prepareBunkerMissionProgress" );
end

function prepareBunkerMissionProgress()
	table.insert( cutsceneQueue, showBunkerMissionProgress );
	SubtitleQueue();
end

function showBunkerMissionProgress()
	local progress = ScriptManager:getMissionProgress( missionIds.destroyBunkers );
	if progress >= bunkersToDestroy then
		DisablePlayerInteraction()
		
		ScriptManager:setMissionStatus( missionIds.destroyBunkers, MissionStatus.ACCOMPLISHED );
		ScriptManager:wait( 2 );
		
		EnablePlayerInteraction();
		checkForVictory();
	
	else
		ShowMissionProgressPopup( ScriptManager:getMissionProgress( missionIds.destroyBunkers ), Text.T_MAP_SU7_1_MISSION_PRIMARY_01_COUNTER );
	end
end

function onMgNestDestroyed()
	ScriptManager:setMissionProgress( missionIds.destroyMgNests, ScriptManager:getMissionProgress( missionIds.destroyMgNests ) + 1 );
	
	print( "mgNests destroyed " .. ScriptManager:getMissionProgress( missionIds.destroyMgNests ) );
	
	ScriptManager:runForPlayer( 0, "prepareMgNestMissionProgress" );
	
end

function prepareMgNestMissionProgress()
	table.insert( cutsceneQueue, showMgNestMissionProgress );
	SubtitleQueue();
end

function showMgNestMissionProgress()
	local progress = ScriptManager:getMissionProgress( missionIds.destroyMgNests );
	if progress >= mgNestsToDestroy then
		DisablePlayerInteraction()
		
		ScriptManager:setMissionStatus( missionIds.destroyMgNests, MissionStatus.ACCOMPLISHED );
		ScriptManager:wait( 2 );
		
		EnablePlayerInteraction();
	
	else
		ShowMissionProgressPopup( ScriptManager:getMissionProgress( missionIds.destroyMgNests ), Text.T_MAP_SU7_1_MISSION_SECONDARY_01_COUNTER );
	end
end

function onCheckpointUnitDestroyed( squadId )
	local counter, maxDestroyed;
	-- check which checkpoint the unit belonged to
	if squadId == enemySquadIds.firstCheckpoint then
		firstPointDestroyed = firstPointDestroyed + 1;
		ScriptManager:storeValue( "firstPointDestroyed", firstPointDestroyed );
		
		counter		 = firstPointDestroyed;
		maxDestroyed = firstPointToDestroy;
		print( "first" );
	
	else
		secondPointDestroyed = secondPointDestroyed + 1;
		ScriptManager:storeValue( "secondPointDestroyed", secondPointDestroyed );
		
		counter		 = secondPointDestroyed;
		maxDestroyed = secondPointToDestroy;
		print( "second" );
	end
	
	-- check if checkpoint was annihilated
	print( counter )
	print( maxDestroyed );
	if counter >= maxDestroyed then
		ScriptManager:setMissionProgress( missionIds.destroyEnemies, ScriptManager:getMissionProgress( missionIds.destroyEnemies ) + 1 );
		
		ScriptManager:runForPlayer( 0, "prepareCheckpointMissionProgress" );
	end
end

function prepareCheckpointMissionProgress()
	table.insert( cutsceneQueue, showCheckpointMissionProgress );
	SubtitleQueue();
end

function showCheckpointMissionProgress()
	local progress = ScriptManager:getMissionProgress( missionIds.destroyEnemies );
	if progress >= checkpointsToDestroy then
		DisablePlayerInteraction()
		
		ScriptManager:setMissionStatus( missionIds.destroyEnemies, MissionStatus.ACCOMPLISHED );
		ScriptManager:wait( 2 );
		
		EnablePlayerInteraction();
		checkForVictory();
	
	else
		ShowMissionProgressPopup( ScriptManager:getMissionProgress( missionIds.destroyEnemies ), Text.T_MAP_SU7_1_MISSION_PRIMARY_02_COUNTER);
	end
end


-- END Destroy Stuff
-- BEGIN Victory


function checkForVictory()
	if ScriptManager:getMissionStatus( missionIds.destroyBunkers ) == MissionStatus.ACCOMPLISHED and
	   ScriptManager:getMissionStatus( missionIds.destroyEnemies ) == MissionStatus.ACCOMPLISHED then
		startVictoryCutscene();
	end
end

function startVictoryCutscene()
	StartCutscene();
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU7_1_MISSION_VICTORY_FLAVOR_01 );
	ScriptManager:wait( 4 );
	HideDialogue();
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU7_1_MISSION_VICTORY_FLAVOR_02 );
	ScriptManager:wait( 4 );
	HideDialogue();
	
	EndCutscene();
	
	victoryEndGame();
end

function victoryEndGame()
	DisablePlayerInteraction();
	HideDialogue(); -- remove possible texts on the screens

	-- accomplish secondary missions that are accomplished when the game is finished with certain condition
	
	
	ScriptManager:displaySubtitle( Text.T_MISSION_VICTORY );
	ScriptManager:wait( 4 );
	HideDialogue();
	endGame();
end


-- END Victory
-- BEGIN Defeated


function transitionToFail()
	ScriptManager:runForPlayer( 0, "missionFailed" )
end

function missionFailed()
	DisablePlayerInteraction();
	HideDialogue(); -- remove possible texts on the screens
	
	ScriptManager:displaySubtitle( Text.T_MISSION_FAILED );
	ScriptManager:wait( 4 );
	HideDialogue();
	endGame();
end


-- END Defeated


function endGame()
	ScriptManager:endGame()
end