require "lua/Constants"
require "lua/Functions"
require "lua/Functions_2"
require "lua/Text"
require "lua/Units"
require "math"


-- mission stuff
missionIds = {
	raidWeapons		= 1,
	keepTrucks		= 2,
	keepAllTrucks	= 3,
	destroyTanks	= 4,
};

-- missionId, primary( boolean ), textkey, xpReward
missions = {
	{ missionIds.raidWeapons,	true,	Text.T_MAP_SU7_2_MISSION_PRIMARY_01,	155 },
	{ missionIds.keepTrucks,	true,	Text.T_MAP_SU7_2_MISSION_PRIMARY_02,	200 },
	{ missionIds.keepAllTrucks, false,	Text.T_MAP_SU7_2_MISSION_SECONDARY_01,	 75 },
	{ missionIds.destroyTanks,	false,	Text.T_MAP_SU7_2_MISSION_SECONDARY_02,	100 },
};

enemySquadIds = {
	factory1			=  1,
	factory2			=  2,
	factory3			=  3,
	factory4			=  4,
	firstAttackersN		=  5,
	firstAttackersE		=  6,
	northStreet			=  7, -- WAIT
	loneMgNestN			=  8,
	reinforcementsW		=  9,
	reinforcementsNE	= 10,
	reinforcementsE		= 11,
};

enemyReinforcementsW	= {
	{ Units.UNIT_DE_MP40,	0, 20 },
	{ Units.UNIT_DE_MP40,	0, 19 },
	{ Units.UNIT_DE_TANK_4,	0, 18 },
	{ Units.UNIT_DE_TANK_3,	1, 18 },
};
enemyReinforcementsNE	= {
	{ Units.UNIT_DE_PANZERFAUST,	16, 25 },
	{ Units.UNIT_DE_MP40,			17, 25 },
	{ Units.UNIT_DE_TANK_4,			16, 24 },
	{ Units.UNIT_DE_ANTI_INFANTRY,	17, 24 },
};
enemyReinforcementsE	= {
	{ Units.UNIT_DE_MP40,			19, 12},
	{ Units.UNIT_DE_PANZERFAUST,	19, 13 },
	{ Units.UNIT_DE_TANK_3,			10, 14 },
	{ Units.UNIT_DE_TIGER,			18, 13 },
};

tanksToDestroy	= 5;
trucksToKeep	= 3;

factoryIds		= { 1, 2, 3, 4 }; -- TODO: ids need to stay 1-4!
factoryPos		= { {  4, 16 }, { 10, 20 }, { 13, 14 }, { 14,  5 } };
transporterPos	= { {  3,  4 }, {  2,  3 }, {  3,  3 } };

-- BEGIN Init


function onGameStart()
	CreateMissions( missions );
	setTriggers();
	storeValues();
	
	ScriptManager:gainGold( 0, 1000 );
	
	for i = 1, #factoryIds do
		for j = 1, Units.UNIT_SU_ANTI_INFANTRY do
			ScriptManager:removeReinforcementsFromBase( factoryIds[i], j );
		end
	end
	
	-- start intro cutscene
	ScriptManager:sleep( 0, "startIntroCutscene" ); -- without sleep player turn popup shows
end

function storeValues()
	
end

function setTriggers()
	-- load all needed values after loading
	ScriptManager:addOnGameResumeTrigger( "loadValues", "loadValues", false );
	
	-- check if the player has units left
	ScriptManager:addOnUnitDestroyedTrigger( 0, Units.UNIT_SU_RAID_TRUCK, -1, "checkForEnd", "checkForEnd" , false );
	
	-- trigger for weapon raids
	local x, y;
	for i = 1, #factoryPos do
		x = factoryPos[i][1];
		y = factoryPos[i][2];
		
		-- each factory gets a function to easily find out which factory was raided
		addOnEnterAreaTrigger2( x, y, 0, Units.UNIT_SU_RAID_TRUCK, -1, true, "onTruckAtFactory" .. i, "onTruckAtFactory" .. i, false );
	end
	ScriptManager:addOnUnitCapturedTrigger(		0, Units.UNIT_DE_WEAPON_FACOTRY, -1, "onFactoryCaptured",	"onFactoryCaptured", false );
	ScriptManager:addOnUnitHasAttackedTrigger(	0, Units.UNIT_DE_WEAPON_FACOTRY, -1, "onFactoryAttacked",	"onFactoryAttacked", false );
	ScriptManager:addOnUnitDestroyedTrigger(	1, Units.UNIT_DE_WEAPON_FACOTRY, -1, "missionFailed",		"missionFailed",	 false );
	
	-- trigger for tank mission
	ScriptManager:addOnUnitHasAttackedTrigger( -1, Units.UNIT_DE_TANK_3,		 -1, "onTankAttacked", "onTankAttacked", false );
	ScriptManager:addOnUnitHasAttackedTrigger( -1, Units.UNIT_DE_TANK_4,		 -1, "onTankAttacked", "onTankAttacked", false );
	ScriptManager:addOnUnitHasAttackedTrigger( -1, Units.UNIT_DE_TANK_DESTROYER, -1, "onTankAttacked", "onTankAttacked", false );
	ScriptManager:addOnUnitHasAttackedTrigger( -1, Units.UNIT_DE_TIGER,			 -1, "onTankAttacked", "onTankAttacked", false );
	
	ScriptManager:addOnUnitDestroyedTrigger( 1, Units.UNIT_DE_TANK_3,			-1, "onTankDestroyed", "onTankDestroyed", false );
	ScriptManager:addOnUnitDestroyedTrigger( 1, Units.UNIT_DE_TANK_4,			-1, "onTankDestroyed", "onTankDestroyed", false );
	ScriptManager:addOnUnitDestroyedTrigger( 1, Units.UNIT_DE_TANK_DESTROYER,	-1, "onTankDestroyed", "onTankDestroyed", false );
	ScriptManager:addOnUnitDestroyedTrigger( 1, Units.UNIT_DE_TIGER,			-1, "onTankDestroyed", "onTankDestroyed", false );
end

function loadValues()
	
end


-- END Init
-- BEGIN Intro Cutscene


function startIntroCutscene()
	StartCutscene();
	
	local factoryX = factoryPos[1][1];
	local factoryY = factoryPos[1][2];
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU7_2_MISSION_INTRO_FLAVOR_01 );
	ScriptManager:wait( 1 );
	ScriptManager:moveCameraEx(	factoryX, factoryY, 0.7, 4, true );
	MarkUnit(					factoryX, factoryY, 1, 3 )
	HideDialogue();
	ScriptManager:wait( 1 );
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU7_2_MISSION_INTRO_FLAVOR_02 );
	ScriptManager:moveCameraEx( 11, 18, 0.5, 5, true );
	
	for i = 2, 3 do
		local posX, posY = factoryPos[i][1], factoryPos[i][2];
		ScriptManager:revealFogOfWar(	posX, posY, 1, true );
		ScriptManager:showMarkerAt(		posX, posY );
	end
	ScriptManager:wait( 3 );
	ScriptManager:removeAllMarkers();
	ScriptManager:revealFogOfWar( factoryPos[2][1], factoryPos[2][2], 1, false );
	ScriptManager:revealFogOfWar( factoryPos[3][1], factoryPos[3][2], 1, false );
	HideDialogue();
	ScriptManager:wait( 1 );
	
	local factoryX = factoryPos[4][1];
	local factoryY = factoryPos[4][2];
	ScriptManager:displaySubtitle( Text.T_MAP_SU7_2_MISSION_INTRO_FLAVOR_03 );
	ScriptManager:moveCameraEx( factoryX, factoryY, 0.7, 5, true );
	MarkUnit(					factoryX, factoryY, 1, 3 )
	HideDialogue();
	ScriptManager:wait( 2 );
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU7_2_MISSION_INTRO_FLAVOR_04 );
	ScriptManager:moveCameraEx( 2, 3, 0.7, 4, true );
	local pos;
	for i = 1, #transporterPos do
		pos = transporterPos[i];
		ScriptManager:showMarkerAt( pos[1], pos[2] );
	end
	ScriptManager:wait( 3 );
	ScriptManager:removeAllMarkers();
	HideDialogue();
	
	EndCutscene();
	
	startTheMission();
end

function startTheMission()
	ScriptManager:setMissionStatus( missionIds.raidWeapons,		MissionStatus.IN_PROGRESS );
	ScriptManager:setMissionStatus( missionIds.keepTrucks,		MissionStatus.IN_PROGRESS );
	ScriptManager:setMissionStatus( missionIds.keepAllTrucks,	MissionStatus.IN_PROGRESS );
	
	ScriptManager:setMissionProgress( missionIds.keepTrucks, trucksToKeep );
	
	--TODO: remove this function!
	--testing();
	--removeSquads();
	--ScriptManager:addOnUnitSelectedTrigger( 1, -1, -1, "showParameters", "showParameters",false)
end

function testing()
	ScriptManager:spawnUnit( 0, Units.GOTT, 1, 4, 42, AiState.WAIT );
	ScriptManager:spawnUnit( 0, Units.GOTT, 2, 5, 42, AiState.WAIT );
	ScriptManager:spawnUnit( 0, Units.UNIT_SU_RAID_TRUCK, 2, 15, 42, AiState.WAIT );
end

function removeSquads()
	--for i = 1, # do
	--	ScriptManager:removeUnitEx( [i], false );
	--end
end

function showParameters( selectedUnit )
	--print( string.format( "[] %s", selectedUnit:get() ) );
end


-- END Intro Cutscene
-- BEGIN Destroy Tanks


function onTankAttacked( attacker, defender )
	ScriptManager:removeTrigger( "onTankAttacked" );
	
	-- find enemyTank
	local tank, playerUnit;
	if attacker:getPlayerId() == 1 then
		tank		= attacker;
		playerUnit	= defender;
	
	else
		tank		= defender;
		playerUnit	= attacker;
	end
	
	-- save pos for cutscene below
	ScriptManager:storeValue( "tankId", tank:getId() );
	
	ScriptManager:storeValue( "playerX", playerUnit:getX() );
	ScriptManager:storeValue( "playerY", playerUnit:getY() );
	
	ScriptManager:runForPlayer( 0, "prepareTankCutscene" );
end

function prepareTankCutscene()
	table.insert( cutsceneQueue, startTankCutscene );
	CutsceneQueue();
end

function startTankCutscene()
	StartCutscene();
	
	local tankId = ScriptManager:getValue( "tankId" );
	local tank = Unit:getUnitById( tankId );
	local tankX = tank:getX();
	local tankY = tank:getY();
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU7_2_MISSION_SECONDARY_02_FLAVOR_01 );
	ScriptManager:moveCameraEx( tankX, tankY, 0.7, 3, true );
	MarkUnit(					tankX, tankY, 1, 3 )
	HideDialogue();
	
	local playerX = ScriptManager:getValue( "playerX" );
	local playerY = ScriptManager:getValue( "playerY" );
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU7_2_MISSION_SECONDARY_02_FLAVOR_02 );
	ScriptManager:moveCameraEx( playerX, playerY, 0.7, 3, true );
	HideDialogue();
	
	EndCutscene();
	DisablePlayerInteraction();
	
	ScriptManager:setMissionStatus( missionIds.destroyTanks, MissionStatus.IN_PROGRESS );
	ScriptManager:wait( 2 );
	
	EnablePlayerInteraction();
end

function onTankDestroyed()
	ScriptManager:setMissionProgress( missionIds.destroyTanks, ScriptManager:getMissionProgress( missionIds.destroyTanks ) + 1 );
	
	ScriptManager:runForPlayer( 0, "prepareTankMissionProgress" );
end

function prepareTankMissionProgress()
	table.insert( cutsceneQueue, showTankMissionProgress );
	SubtitleQueue();
end

function showTankMissionProgress()
	if ScriptManager:getMissionProgress( missionIds.destroyTanks ) >= tanksToDestroy then
		DisablePlayerInteraction();
		
		ScriptManager:removeTrigger( "onTankDestroyed" );
		
		ScriptManager:setMissionStatus( missionIds.destroyTanks, MissionStatus.ACCOMPLISHED );
		ScriptManager:wait( 2 );
		
		EnablePlayerInteraction();
	
	else
		ShowMissionProgressPopup( ScriptManager:getMissionProgress( missionIds.destroyTanks ), Text.T_MAP_SU7_2_MISSION_SECONDARY_02_COUNTER );
	end
end


-- END Destroy Tanks
-- BEGIN Raid


function onFactoryAttacked( activeUnit, passiveUnit )
	local factoryHp = passiveUnit:getHealth();
	if factoryHp > 0 then
		DisablePlayerInteraction();
		
		ScriptManager:displaySubtitle( Text.T_MISSION_SUBTITLE_BUILDING_ATTACKED_WARNING );
		ScriptManager:moveCameraEx( passiveUnit:getX(), passiveUnit:getY(), 1.0, 1, true );
		ScriptManager:showMarkerAt( passiveUnit:getX(), passiveUnit:getY() );
		ScriptManager:wait( 2 );
		ScriptManager:removeAllMarkers();
		HideDialogue();
		
		EnablePlayerInteraction();
	end
end

function onFactoryCaptured( capturer, factory )
	local factoryId = factory:getId();
	factory:setIsAttackableByAI( false );
	
	-- get surrounding tiles
	local factoryX			= factory:getX();
	local factoryY			= factory:getY();
	local surroundingTiles	= {};
	-- center stripe
	for i = factoryX -1, factoryX + 1 do
		table.insert( surroundingTiles, { i, factoryY } );
	end
	
	local xx = factoryY % 2 -1
	-- top & bottom tiles
	for i = factoryY - 1, factoryY + 1, 2 do
		table.insert( surroundingTiles, { factoryX + xx,		i } );
		table.insert( surroundingTiles, { factoryX + xx + 1,	i } );
	end
	
	-- check surrounding tiles for transporter
	local pos, unit;
	for i = 1, #surroundingTiles do
		pos		= surroundingTiles[i];
		unit	= Unit:getUnitAt( pos[1], pos[2] );
		
		if unit ~= nil then
			if unit:getTemplateId() == Units.UNIT_SU_RAID_TRUCK then
				factoryRaided( factoryId );
				break;
			end
		end
	end
	
	if factoryId == factoryIds[1] then
		ScriptManager:runForPlayer( 1, "spawnReinforcementsW" );
	
	elseif factoryId == factoryIds[2] then
		ScriptManager:runForPlayer( 1, "spawnReinforcementsNE" );
	
	elseif factoryId == factoryIds[3] then
		ScriptManager:runForPlayer( 1, "spawnReinforcementsE" );
	end
end

function spawnReinforcementsW()
	spawnReinforcements( enemyReinforcementsW, enemySquadIds.reinforcementsW );
end

function spawnReinforcementsNE()
	spawnReinforcements( enemyReinforcementsNE, enemySquadIds.reinforcementsNE );
end

function spawnReinforcementsE()
	spawnReinforcements( enemyReinforcementsE, enemySquadIds.reinforcementsE );
end

function spawnReinforcements( reinforcementTable, squadId )
	for i = 1, #reinforcementTable do
		local enemyUnit = reinforcementTable[i];
		ScriptManager:spawnUnit( 1, enemyUnit[1], enemyUnit[2], enemyUnit[3], squadId, AiState.ATTACK_AGGRESSIVE );
	end
end

function onTruckAtFactory1( tileX, tileY, playerId, unitTemplateId, unitId )
	factoryRaided( 1 );
end

function onTruckAtFactory2( tileX, tileY, playerId, unitTemplateId, unitId )
	factoryRaided( 2 );
end

function onTruckAtFactory3( tileX, tileY, playerId, unitTemplateId, unitId )
	factoryRaided( 3 );
end

function onTruckAtFactory4( tileX, tileY, playerId, unitTemplateId, unitId )
	factoryRaided( 4 );
end

function factoryRaided( factoryId )
	print( "piep" )
	local factory = Unit:getUnitById( factoryId );
	if factory:getPlayerId() == 0 then
		ScriptManager:removeTrigger( "onTruckAtFactory" .. factoryId );
		
		ScriptManager:setMissionProgress( missionIds.raidWeapons, ScriptManager:getMissionProgress( missionIds.raidWeapons ) + 1 );
		
		if ScriptManager:getMissionProgress( missionIds.raidWeapons ) >= #factoryIds then
			DisablePlayerInteraction()
			
			ScriptManager:setMissionStatus( missionIds.raidWeapons, MissionStatus.ACCOMPLISHED );
			ScriptManager:wait( 2 );
			
			startVictoryCutscene();
		
		else
			ShowMissionProgressPopup( ScriptManager:getMissionProgress( missionIds.raidWeapons ), Text.T_MAP_SU7_2_MISSION_PRIMARY_01_COUNTER );
		end
	end
end


-- END Raid
-- BEGIN Victory


function startVictoryCutscene()
	StartCutscene();
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU7_2_MISSION_VICTORY_FLAVOR_01 );
	ScriptManager:wait( 4 );
	HideDialogue();
	
	EndCutscene();
	
	victoryEndGame();
end

function victoryEndGame()
	DisablePlayerInteraction();
	HideDialogue(); -- remove possible texts on the screens

	-- accomplish missions that are accomplished when the game is finished with certain condition
	if ScriptManager:getMissionStatus(	missionIds.keepTrucks ) == MissionStatus.IN_PROGRESS then
		ScriptManager:setMissionStatus( missionIds.keepTrucks, MissionStatus.ACCOMPLISHED );
	end
	
	if ScriptManager:getMissionStatus(	missionIds.keepAllTrucks ) == MissionStatus.IN_PROGRESS then
		ScriptManager:setMissionStatus( missionIds.keepAllTrucks, MissionStatus.ACCOMPLISHED );
	end
	
	ScriptManager:displaySubtitle( Text.T_MISSION_VICTORY );
	ScriptManager:wait( 4 );
	HideDialogue();
	endGame();
end


-- END Victory
-- BEGIN Defeated


function checkForEnd()
	ScriptManager:setMissionProgress( missionIds.keepTrucks, ScriptManager:getMissionProgress( missionIds.keepTrucks ) - 1 );
	
	ScriptManager:runForPlayer( 0, "truckDestroyed" )
end

function truckDestroyed()
	if ScriptManager:getMissionStatus(	missionIds.keepAllTrucks ) == MissionStatus.IN_PROGRESS then
		ScriptManager:setMissionStatus(	missionIds.keepAllTrucks, MissionStatus.FAILED );
	end
	
	if ScriptManager:getMissionProgress( missionIds.keepTrucks ) <= 0 then
		missionFailed();
	
	else
		ShowMissionProgressPopup( ScriptManager:getMissionProgress( missionIds.keepTrucks ), Text.T_MAP_SU7_2_MISSION_PRIMARY_02_COUNTER );
	end
end

function missionFailed()
	DisablePlayerInteraction();
	HideDialogue(); -- remove possible texts on the screens
	
	ScriptManager:displaySubtitle( Text.T_MISSION_FAILED );
	ScriptManager:wait( 4 );
	HideDialogue();
	endGame();
end


-- END Defeated


function endGame()
	ScriptManager:endGame()
end