require "lua/Constants"
require "lua/Functions"
require "lua/Functions_2"
require "lua/Text"
require "lua/Units"
require "math"


-- mission stuff
missionIds = {
	keepTrucks		= 1, -- = reach the city
	keepDepots		= 2,
	keepAirfield	= 3,
};

-- missionId, primary( boolean ), textkey, xpReward
missions = {
	{ missionIds.keepTrucks,	true,	Text.T_MAP_SU7_3_MISSION_PRIMARY_01,	150 },
	{ missionIds.keepDepots,	true,	Text.T_MAP_SU7_3_MISSION_PRIMARY_02,	200 },
	{ missionIds.keepAirfield,	false,	Text.T_MAP_SU7_3_MISSION_SECONDARY_01,	 75 },
};

enemySquadIds = {
	bridge		= 1;
	crossroads1	= 2,
	crossroads2	= 3,
	pursuers	= 4,
	attackWaveN	= 5,
	attackWaveS	= 6,
	planeWaveN	= 7,
	planeWaveC	= 8,
	planeWaveS	= 9,
};

transporterIds		= { 21, 22, 23, }; -- will be removed when reaching city
trucksToKeep		=  3;
attackStartRound	= -1;

-- to remove them for easier testing
startingPlayerIds	= { 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28 };
startingEnemyIds	= {  7,  8,  9, 10, 11, 12, 13, 14, 15, 16, 17, 29 };

-- needed to remove neutral & spawn player units there when near
bunkerPos	= { { 23,  4 }, { 22,  8 }, { 20, 12 }, { 22, 16 } };
depotPos	= { { 27, 13 }, { 27,  3 }, };
airfieldPos = { 28, 6 };

cityUnits	= {
	{ Units.UNIT_SU_RIFLE,			24,  8 },
	{ Units.UNIT_SU_RIFLE,			26,  5 },
	{ Units.UNIT_SU_RIFLE,			28, 14 },
	{ Units.UNIT_SU_RIFLE,			23, 15 },
	{ Units.UNIT_SU_MORTAR,			25, 15 },
	{ Units.UNIT_SU_MORTAR,			23, 10 },
	{ Units.UNIT_SU_MORTAR,			26,  6 },
	{ Units.UNIT_SU_TANK_T34,		25,  8 },
	{ Units.UNIT_SU_TANK_T34,		23,  5 },
	{ Units.UNIT_SU_ANTI_INFANTRY,	28,  4 },
	{ Units.UNIT_SU_TANK_KV1,		26, 13 },
	{ Units.UNIT_SU_TANK_DESTROYER,	27,  4 },
	{ Units.UNIT_SU_AA,				24, 15 },
	{ Units.UNIT_SU_AA,				21, 13 },
	{ Units.UNIT_SU_AA,				23,  9 },
	{ Units.UNIT_SU_AA,				24,  6 },
	{ Units.UNIT_SU_KATYUSHA,		26, 11 },
	{ Units.UNIT_SU_FIGHTER,		27,  7 },
	{ Units.UNIT_SU_SUPPLY_TRUCK,	27, 14 },
};

enemyPursuers = {
	{ Units.UNIT_DE_MP40,			1, 18 },
	{ Units.UNIT_DE_MP40,			1, 17 },
	{ Units.UNIT_DE_TANK_4,			1, 16 },
	{ Units.UNIT_DE_TANK_DESTROYER,	1, 15 },
	{ Units.UNIT_DE_TANK_4,			1, 14 },
	{ Units.UNIT_DE_MP40,			2, 16 },
};

endSceneUnits = {
	{ Units.UNIT_SU_TANK_KV1,		29, 17 },
	{ Units.UNIT_SU_TANK_DESTROYER,	30, 18 },
	{ Units.UNIT_SU_BOMBER,			30, 17 },
	{ Units.UNIT_SU_BOMBER,			29, 18 },
	{ Units.UNIT_SU_BOMBER,			30, 16 },
};

attackSpawnN = { { 10, 20 }, { 11, 20 }, { 12, 20 }, { 10, 19 }, {  1, 19 }, };
attackSpawnS = { { 12,  1 }, { 13,  1 }, { 14,  1 }, { 13,  2 }, { 14,  2 }, };

maxAttackWaves = 5;
attackWave1 = {
	Units.UNIT_DE_MP40,
	Units.UNIT_DE_MP40,
	Units.UNIT_DE_PANZERFAUST,
	Units.UNIT_DE_TANK_4,
	Units.UNIT_DE_TANK_4,
};

attackWave2 = {
	Units.UNIT_DE_TIGER,
	Units.UNIT_DE_TANK_DESTROYER,
	Units.UNIT_DE_MP40,
	Units.UNIT_DE_MP40,
};

attackWave3 = {
	Units.UNIT_DE_MP40,
	Units.UNIT_DE_PANZERFAUST,
	Units.UNIT_DE_PANZERFAUST,
	Units.UNIT_DE_ANTI_INFANTRY,
	Units.UNIT_DE_TANK_3,
};

attackWave4 = {
	Units.UNIT_DE_MP40,
	Units.UNIT_DE_PANZERFAUST,
	Units.UNIT_DE_PANZERFAUST,
	Units.UNIT_DE_ANTI_INFANTRY,
	Units.UNIT_DE_TANK_3,
};

attackWave5 = {
	Units.UNIT_DE_MP40,
	Units.UNIT_DE_PANZERFAUST,
	Units.UNIT_DE_TANK_4,
	Units.UNIT_DE_TANK_3,
	Units.UNIT_DE_TANK_3,
};

planeSpawnN = { { 11, 21 }, { 12, 20 }, };
planeSpawnS = { {  6,  1 }, {  9,  1 }, };
planeSpawnC = { {  1, 11 }, {  1, 10 }, };

maxPlaneWaves = 3;
planeWave1 = {
	Units.UNIT_DE_FIGHTER,
	Units.UNIT_DE_FIGHTER,
};

planeWave2 = {
	Units.UNIT_DE_BOMBER,
	Units.UNIT_DE_FIGHTER,
};

planeWave3 = {
	Units.UNIT_DE_DESTROYER,
	Units.UNIT_DE_FIGHTER,
};


-- BEGIN Init


function onGameStart()
	CreateMissions( missions );
	setTriggers();
	storeValues();
	
	ScriptManager:gainGold( 0, 500 );
	
	-- start intro cutscene
	ScriptManager:sleep( 0, "startIntroCutscene" ); -- without sleep player turn popup shows
end

function storeValues()
	ScriptManager:storeValue( "attackStartRound", attackStartRound );
end

function setTriggers()
	-- load all needed values after loading
	ScriptManager:addOnGameResumeTrigger( "loadValues", "loadValues", false );
	
	-- check if the player has units left
	ScriptManager:addOnUnitDestroyedTrigger( 0, Units.UNIT_SU_RAID_TRUCK, -1, "onTruckDestroyed", "onTruckDestroyed" , false );
	
	-- trigger for reaching city and starting the second part of the mission
	for y = 1, 7 do
		ScriptManager:addOnEnterTileTrigger( 19, y, 0, -1, -1, true, "nearCity", "nearCity", false );
	end
	
	for y = 13, 15 do
		ScriptManager:addOnEnterTileTrigger( 16, y, 0, -1, -1, true, "nearCity", "nearCity", false );
	end
	ScriptManager:addOnEnterTileTrigger( 21, 20, 0, -1, -1, true, "nearCity", "nearCity", false );
	
	-- trigger for keeping buildings
	ScriptManager:addOnUnitDestroyedTrigger(	 0, Units.UNIT_SU_HQ, -1, "onHqDestroyed",					"onHqDestroyed",						false );
	ScriptManager:addOnUnitHasAttackedTrigger(	-1, Units.UNIT_SU_HQ, -1, "ProtectTheSupplyDepotHintSingle",	"ProtectTheSupplyDepotHintSingle",	false );
	
	ScriptManager:addOnUnitDestroyedTrigger(	 0, Units.UNIT_SU_AIRFIELD, -1, "onAirfieldDestroyed",			"onAirfieldDestroyed",			false );
	ScriptManager:addOnUnitHasAttackedTrigger(	-1, Units.UNIT_SU_AIRFIELD, -1, "ProtectTheBuildingHintSingle",	"ProtectTheBuildingHintSingle",	false );
	
	-- trigger to spawn enemy pursuers
	ScriptManager:addOnTurnStartTrigger( 1, -1, 4, "spawnEnemyPursuers", "spawnEnemyPursuers", false );
end

function loadValues()
	attackStartRound = ScriptManager:getValue( "attackStartRound" );
end


-- END Init
-- BEGIN Intro Cutscene


function startIntroCutscene()
	StartCutscene();
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU7_3_MISSION_INTRO_FLAVOR_01 );
	ScriptManager:moveCameraEx( 15, 8, 0.7, 6, true );
	ScriptManager:wait( 1 );
	HideDialogue();
	ScriptManager:wait( 1 );
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU7_3_MISSION_INTRO_FLAVOR_02 );
	ScriptManager:moveCameraEx( 23, 11, 0.5, 6, true );
	ScriptManager:wait( 1 );
	HideDialogue();
	ScriptManager:wait( 1 );
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU7_3_MISSION_INTRO_FLAVOR_03 );
	ScriptManager:moveCameraEx( 4, 14, 0.7, 6, true );
	ScriptManager:wait( 1 );
	HideDialogue();
	
	EndCutscene();
	
	startTheMission();
end

function startTheMission()
	ScriptManager:setMissionStatus( missionIds.keepTrucks, MissionStatus.IN_PROGRESS );
	
	ScriptManager:setMissionProgress( missionIds.keepTrucks, trucksToKeep );
	
	--TODO: remove this function!
	--testing();
	--removeSquads();
	--ScriptManager:addOnUnitSelectedTrigger( 1, -1, -1, "showParameters", "showParameters",false)
end

function testing()
	nearCity( 19, 13 );
	
	ScriptManager:removeTrigger( "spawnEnemyPursuers" );
	
	ScriptManager:spawnUnit( 0, Units.UNIT_SU_RIFLE,			19, 15, 42, AiState.WAIT );
	ScriptManager:spawnUnit( 0, Units.UNIT_SU_RIFLE,			19, 14, 42, AiState.WAIT );
	ScriptManager:spawnUnit( 0, Units.UNIT_SU_MORTAR,			19, 13, 42, AiState.WAIT );
	ScriptManager:spawnUnit( 0, Units.UNIT_SU_TANK_T34,			19, 12, 42, AiState.WAIT );
	ScriptManager:spawnUnit( 0, Units.UNIT_SU_ANTI_INFANTRY,	19, 11, 42, AiState.WAIT );
	ScriptManager:spawnUnit( 0, Units.UNIT_SU_SUPPLY_TRUCK,		19, 10, 42, AiState.WAIT );

	ScriptManager:spawnUnit( 0, Units.GOTT, 25, 12, 42, AiState.WAIT );
	
	for i = 1, #startingPlayerIds do
		ScriptManager:removeUnitEx( startingPlayerIds[i], true );
	end
	for i = 1, #startingEnemyIds do
		ScriptManager:removeUnitEx( startingEnemyIds[i], true );
	end
end

function removeSquads()
	--for i = 1, # do
	--	ScriptManager:removeUnitEx( [i], false );
	--end
end

function showParameters( selectedUnit )
	--print( string.format( "[] %s", selectedUnit:get() ) );
end


-- END Intro Cutscene
-- BEGIN Truck Escort


function onTruckDestroyed()
	ScriptManager:setMissionProgress( missionIds.keepTrucks, ScriptManager:getMissionProgress( missionIds.keepTrucks ) - 1 );
	
	ScriptManager:runForPlayer( 0, "truckDestroyed" )
end

function truckDestroyed()
	if ScriptManager:getMissionProgress( missionIds.keepTrucks ) <= 0 then
		missionFailed();
	
	else
		ShowMissionProgressPopup( ScriptManager:getMissionProgress( missionIds.keepTrucks ), Text.T_MAP_SU7_2_MISSION_PRIMARY_02_COUNTER );
	end
end

function spawnEnemyPursuers()
	ScriptManager:removeTrigger( "spawnEnemyPursuers" );
	ScriptManager:wait( 0.2 ); -- fix for some enemies not moving after spawn
	
	local enemy;
	for i = 1, #enemyPursuers do
		enemy = enemyPursuers[i];
		ScriptManager:spawnUnit( 1, enemy[1], enemy[2], enemy[3], enemySquadIds.pursuers, AiState.ATTACK_AGGRESSIVE );
	end
	
	ScriptManager:runForPlayer( 0, "prepareEnemyPursuersCutscene" );
end

function prepareEnemyPursuersCutscene()
	table.insert( cutsceneQueue, startEnemyPursuersCutscene );
	CutsceneQueue();
end

function startEnemyPursuersCutscene()
	StartCutscene();
	
	ScriptManager:displaySubtitle( Text.T_MISSION_REINFORCEMENTS_ENEMY );
	ScriptManager:moveCameraEx( 2, 16, 0.7, 4, true );
	ScriptManager:revealFogOfWar( 2, 16, 2, true );
	ScriptManager:wait( 2 );
	ScriptManager:revealFogOfWar( 2, 16, 2, false );
	HideDialogue();
	
	local unit, playerX, playerY;
	for i = 1, #startingPlayerIds do
		unit = Unit:getUnitById( startingPlayerIds[i] );
		if unit ~= nil then
			playerX = unit:getX();
			playerY = unit:getY();
		end
	end
	
	EndCutscene();
end

function replaceWithPlayerUnit( templateId, pos )
	local posX, posY	= pos[1], pos[2];
	local unit			= Unit:getUnitAt( posX, posY );
	local unitId		= unit:getId();
	
	ScriptManager:removeUnitEx( unitId, true );
	
	ScriptManager:spawnUnit( 0, templateId, posX, posY, 0, AiState.WAIT );
end

function nearCity( tileX, tileY )
	ScriptManager:removeTrigger( "onTruckDestroyed" );
	ScriptManager:removeTrigger( "nearCity" );
	
	DisablePlayerInteraction();
	
	ScriptManager:setMissionStatus( missionIds.keepTrucks, MissionStatus.ACCOMPLISHED );
	
	--trigger for enemy waves
	ScriptManager:addOnTurnStartTrigger( 1, -1, -1, "onEnemyTurnStart", "onEnemyTurnStart", false );
	
	attackStartRound = ScriptManager:getCurrentRound(); -- needed for incoming attack waves
	ScriptManager:storeValue( "attackStartRound", attackStartRound );
	ScriptManager:wait( 2 );
	
	EnablePlayerInteraction();
	StartCutscene()
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU7_3_MISSION_PRIMARY_02_FLAVOR_01 );
	ScriptManager:moveCameraEx( tileX, tileY, 0.8, 1, false );
	
	local unit;
	--respawn Bunkers
	for i = 1, #bunkerPos do
		unit = bunkerPos[i];
		
		replaceWithPlayerUnit(		Units.UNIT_SU_BUNKER,	unit );
		ScriptManager:spawnUnit( 0,	Units.UNIT_SU_RIFLE,	unit[1], unit[2], 0, AiState.WAIT );
	end
	
	--respawn buildings
	for i = 1, #depotPos do
		replaceWithPlayerUnit( Units.UNIT_SU_HQ, depotPos[i] );
	end
	
	-- respawn airfield
	replaceWithPlayerUnit( Units.UNIT_SU_AIRFIELD, airfieldPos );
	
	-- spawn the other units in town
	for i = 1, #cityUnits do
		unit = cityUnits[i];
		ScriptManager:spawnUnit( 0, unit[1], unit[2], unit[3], 0, AiState.WAIT );
	end
	ScriptManager:wait( 3 );
	HideDialogue();
	ScriptManager:wait( 1 );
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU7_3_MISSION_PRIMARY_02_FLAVOR_02 );
	ScriptManager:moveCameraEx( 24, 11, 0.6, 4, true );
	ScriptManager:wait( 1 );
	HideDialogue();
	
	-- remove remaining transporter, they are not the focus of the mission anymore
	local transporter, transporterId;
	for i = 1, #transporterIds do
		transporterId	= transporterIds[i]
		transporter		= Unit:getUnitById( transporterId );
		
		if transporter ~= nil then
			ScriptManager:removeUnitEx( transporterId, false );
		end
	end
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU7_3_MISSION_PRIMARY_02_FLAVOR_03 );
	ScriptManager:wait( 6 );
	HideDialogue();
	ScriptManager:wait( 1 );
	
	ScriptManager:displaySubtitle( Text.T_MAP_SU7_3_MISSION_PRIMARY_02_FLAVOR_04 );
	ScriptManager:moveCameraEx( tileX, tileY, 0.7, 5, true );
	ScriptManager:wait( 1 );
	HideDialogue();
	ScriptManager:wait( 1 );
	
	ScriptManager:enableInteractionButton( InteractionButton.TRANSPORT );
	ScriptManager:enableInteractionButton( InteractionButton.TRANSPORT_UNLOAD_1 );
	ScriptManager:enableInteractionButton( InteractionButton.TRANSPORT_UNLOAD_2 );
	ScriptManager:enableInteractionButton( InteractionButton.TRANSPORT_UNLOAD_3 );
	
	EndCutscene()
	DisablePlayerInteraction();
	
	ScriptManager:setMissionStatus( missionIds.keepDepots,		MissionStatus.IN_PROGRESS );
	ScriptManager:setMissionStatus( missionIds.keepAirfield,	MissionStatus.IN_PROGRESS );
	ScriptManager:wait( 2 );
	
	EnablePlayerInteraction();
end


-- END Truck Escort
-- BEGIN Enemy Waves


function onEnemyTurnStart( playerId, turnId, roundId )
	local roundsSinceAttackStart = roundId - attackStartRound;
	
	local attackChecker = roundsSinceAttackStart % 3;
	if attackChecker == 0 then
		-- attack wave north
		spawnWave( attackSpawnN, enemySquadIds.attackWaveN, AiState.ATTACK_AGGRESSIVE )
	
	elseif attackChecker == 2 then
		-- attack wave south
		spawnWave( attackSpawnS, enemySquadIds.attackWaveS, AiState.ATTACK_AGGRESSIVE )
	end
	
	local planeChecker = roundsSinceAttackStart % 5;
	if planeChecker == 0 then
		-- plane wave north
		spawnWave( planeSpawnN, enemySquadIds.planeWaveN, AiState.ATTACK_AGGRESSIVE )
	
	elseif attackChecker == 2 then
		-- plane wave south
		spawnWave( planeSpawnS, enemySquadIds.planeWaveS, AiState.ATTACK_AGGRESSIVE )
	end
	
	if roundsSinceAttackStart % 4 == 0 then
		-- plane wave center
		spawnWave( planeSpawnC, enemySquadIds.planeWaveC, AiState.ATTACK_AGGRESSIVE )
	end
	
	if roundsSinceAttackStart >= 15 then
		ScriptManager:runForPlayer( 0, "startVictoryCutscene" );
	
	else
		
		ScriptManager:runForPlayer( 0, "showRemainingTurns" );
	end
end

function showRemainingTurns()
	local roundsSinceAttackStart = ScriptManager:getCurrentRound() - attackStartRound;
	local roundsRemaining = 16 - roundsSinceAttackStart;
	
	if roundsRemaining > 1 then	
		ShowMissionProgressPopup( 16 - roundsSinceAttackStart, Text.T_MAP_SU7_3_MISSION_PRIMARY_02_COUNTER );
	end
end

function getRandomWave( waveName, maxWaveNumber )
	local rng = math.random( 1, maxWaveNumber );
	
	print( waveName .. rng .. " will spawn" );
	return _G[ waveName .. rng ];
end

function spawnUnits( waveTable, spawnPosTable, squadId, aiState )
	ScriptManager:wait( 0.2 ); -- fix for some enemies not moving after spawn
	
	local spawnPos;
	for i = 1, #waveTable do
		spawnPos = spawnPosTable[i];
		ScriptManager:spawnUnit( 1, waveTable[i], spawnPos[1], spawnPos[2], squadId, aiState );
	end
end

function spawnWave( spawnPosTable, squadId, aiState )
	-- attack waves always have length 5, so i can use it to identify the wave
	local wave;
	if #spawnPosTable == 5 then
		wave = getRandomWave( "attackWave", maxAttackWaves );
	
	else
		wave = getRandomWave( "planeWave", maxPlaneWaves );
	end
	
	spawnUnits( wave, spawnPosTable, squadId, aiState );
end


-- END Enemy Waves
-- BEGIN Building Stuff


function onHqDestroyed()
	ScriptManager:removeTrigger( "onHqDestroyed" );
	
	ScriptManager:runForPlayer( 0, "missionFailed" )
end

function onAirfieldDestroyed()
	ScriptManager:removeTrigger( "onAirfieldDestroyed" );
	
	DisablePlayerInteraction()
	
	ScriptManager:setMissionStatus( missionIds.keepAirfield, MissionStatus.FAILED );
	ScriptManager:wait( 2 );
	
	EnablePlayerInteraction();
end


--END Building Stuff
-- BEGIN Victory


function startVictoryCutscene()
	local gameWon = true;
	for i = 1, #depotPos do
		local position = depotPos[i];
		local depot = Unit:getUnitAt( position[1], position[2] );
		
		if depot == nil then
			gameWon = false;
			break;
		end
	end
	
	if gameWon then
		DisablePlayerInteraction()
		
		ScriptManager:setMissionStatus( missionIds.keepDepots,		MissionStatus.ACCOMPLISHED );
		ScriptManager:setMissionStatus( missionIds.keepAirfield,	MissionStatus.ACCOMPLISHED );
		
		local unit;
		for i = 1, #endSceneUnits do
			unit = endSceneUnits[i];
			ScriptManager:spawnUnit( 0, unit[1], unit[2], unit[3], 0, AiState.WAIT );
		end
		
		ScriptManager:wait( 2 );
		
		EnablePlayerInteraction();
		StartCutscene();
		
		ScriptManager:displaySubtitle( Text.T_MAP_SU7_3_MISSION_VICTORY_FLAVOR_01 );
		ScriptManager:moveCameraEx( 29, 17, 0.6, 4, true );
		ScriptManager:wait( 1 );
		HideDialogue();
		ScriptManager:wait( 1 );
		
		ScriptManager:displaySubtitle( Text.T_MAP_SU7_3_MISSION_VICTORY_FLAVOR_02 );
		ScriptManager:wait( 6 );
		HideDialogue();
		
		EndCutscene();
		
		victoryEndGame();
	
	else
		missionFailed();
	end
end

function victoryEndGame()
	DisablePlayerInteraction();
	HideDialogue(); -- remove possible texts on the screens
	ScriptManager:wait( 1 );
	
	ScriptManager:displaySubtitle( Text.T_MISSION_VICTORY );
	ScriptManager:wait( 4 );
	HideDialogue();
	endGame();
end


-- END Victory
-- BEGIN Defeated


function missionFailed()
	DisablePlayerInteraction();
	HideDialogue(); -- remove possible texts on the screens
	
	ScriptManager:displaySubtitle( Text.T_MISSION_FAILED );
	ScriptManager:wait( 4 );
	HideDialogue();
	endGame();
end


-- END Defeated


function endGame()
	ScriptManager:endGame()
end