require "lua/Constants"
require "lua/Functions"
require "lua/Text"
require "lua/Units"
require "math"


spawnPositions = {
	{21, 1,	
			17, 9},
	{7, 25,
			9, 21},
	{1, 20,
			3, 13},
	{22, 15,
			17, 9},
	{9, 1,
			6, 7},
	{1, 1,
			6, 7}
}

attackerWaves = {
	{Units.UNIT_DE_MP40, Units.UNIT_DE_PANZERFAUST},
	{Units.UNIT_DE_TANK_3, Units.UNIT_DE_TANK_3},
	{Units.UNIT_DE_TANK_4, Units.UNIT_DE_TANK_4},
	{Units.UNIT_DE_PANZERFAUST,Units.UNIT_DE_PANZERFAUST},
	{Units.UNIT_DE_FIGHTER},
	{Units.UNIT_DE_TIGER},
	{Units.UNIT_DE_TANK_DESTROYER},
	{Units.UNIT_DE_BOMBER},
	{Units.UNIT_DE_TANK_3},
	{Units.UNIT_DE_TANK_4},
	{Units.UNIT_DE_PANZERFAUST, Units.UNIT_DE_MP40},
	{Units.UNIT_DE_MP40},
	{Units.UNIT_DE_MP40},
	{Units.UNIT_DE_PANZERFAUST},
	{Units.UNIT_DE_FIGHTER},
	{Units.UNIT_DE_MP40, Units.UNIT_DE_MP40},
	{Units.UNIT_DE_TANK_3},
	{Units.UNIT_DE_TANK_4}
}

function onGameStart()

	--starting gold
	ScriptManager:gainGold(0,1500)
		
	ScriptManager:preparePopupText()	
		
	-- Mission objectives--				
	
	ScriptManager:createMission(1,true,Text.T_MAP_3_2_MISSION_MAIN) 							-- first goal at start	
	ScriptManager:setMissionXp(1,25)
	ScriptManager:createMission(2,true,Text.T_MAP_3_2_MISSION_MAIN_2)							-- Collect all hidden troops	
	ScriptManager:setMissionXp(2,300)
	ScriptManager:createMission(3,false,Text.T_MAP_3_2_MISSION_SECONDARY)						-- capture airfield
	ScriptManager:setMissionXp(3,150)
	ScriptManager:createMission(4,true,Text.T_MAP_3_2_MISSION_PROTECTOUTPOST) 					-- protect the outpost	
	ScriptManager:setMissionXp(4,100)
	ScriptManager:createMission(5,false,Text.T_MAP_3_2_MISSION_SECONDARY_2)						-- destroy artillery guns
	ScriptManager:setMissionXp(5,150)
	
	--Stuka attacks then airfield mission starts
	
	ScriptManager:addOnUnitHasAttackedTrigger(-1,Units.UNIT_DE_BOMBER,-1,"Cutscene_MoveCam_Airbase","Cutscene_MoveCam_Airbase")
	ScriptManager:addOnUnitDestroyedTrigger(-1,Units.UNIT_DE_BOMBER,-1,"Cutscene_MoveCam_Airbase","Cutscene_MoveCam_Airbase")
	
	--Airfield captured then airfield mission accomplished
	ScriptManager:addOnUnitCapturedTrigger(-1, -1, 11, "Missionupdate_02", "Missionupdate_02")
	ScriptManager:addOnUnitDestroyedTrigger(-1, -1, 11, "Airbase_failed", "Airbase_failed")	
	
	--needed for the artillery mission:
	ScriptManager:addOnUnitDestroyedTrigger(1,Units.UNIT_DE_SUPER_ARTILLERY,-1,"OnenemyArtillerydestroyed","OnenemyArtillerydestroyed")
	ScriptManager:addOnUnitHasAttackedTrigger(-1,Units.UNIT_DE_SUPER_ARTILLERY,-1,"OnenemyArtilleryspotted","OnenemyArtilleryspotted")
	
	ScriptManager:camouflageUnit(5)
	ScriptManager:camouflageUnit(3)
	ScriptManager:camouflageUnit(2)
	
	--start first cutscene w/o having the player, "your turn" stuff showing!
	ScriptManager:sleep(0,"FirstCutscene")
	
	--Check if player units or base is dead
	ScriptManager:addOnUnitDestroyedTrigger(0,-1,-1,"checkForEnd","checkForEnd")
	
	--check if anyone has attacked the base
	ScriptManager:addOnUnitHasAttackedTrigger(-1,Units.UNIT_SU_HQ,-1,"ProtectTheSupplyDepotHint","ProtectTheSupplyDepotHint")
	ScriptManager:addOnUnitDestroyedTrigger(-1,Units.UNIT_SU_HQ,-1,"Depot_destroyed","Depot_destroyed")
		
	
	
	ScriptManager:addOnTurnStartTrigger(0,-1,-1,"CheckForGameEnd")
	
end
		--TODO: NO UNIT LEFT ISSUE=> has to be in all mission to end game if the player cannot do anything
		
function checkForEnd()
	if ScriptManager:noUnitsLeft(0) or Unit:getUnitById(0) == nil then
		ScriptManager:runForPlayer(0, "endGameWait")
	end
end

function endGameWait()

	ScriptManager:runForPlayer(0,"Missionfailed")
	
end


--Game End Check

function CheckForGameEnd()
	MainGoalAccomplished(2,Mission_completed,Missionfailed)		
	--Main Mission ID, Script to start if Accomplished, Script to start if Failed
end

function FirstCutscene()

	ScriptManager:startCutscene()
	ScriptManager:sleep(2,"SpawnUnit1")
	
end

function KillDialogue()				-- Function to disable the dialog for texts
	ScriptManager:dismissSubtitle()
end

function yourTurn()
	if ScriptManager:getCurrentTurn() % 2 == 1 then
		return false
	else
		return true
	end
end

--END OF GAMESTART


function show_warning()
	ScriptManager:runForPlayer(0, "show_warningRunAfterCutscene")
end

function show_warningRunAfterCutscene()
	ScriptManager:disableEndTurnButton()
	ScriptManager:blockUI(true)
	ScriptManager:displaySubtitle(Text.T_MAP_3_2_WARNING)	
	ScriptManager:moveCamera(6,7,1.0,3,"end_warning")
	ScriptManager:showMarkerAt(6,7)
end

function end_warning()
	ScriptManager:enableEndTurnButton()
	ScriptManager:blockUI(false)
	ScriptManager:sleep(3,"KillDialogue")
	ScriptManager:hideMarkerAt(6,7)
end
	--areas for hidden troops
	
	function depotcaptured()
		ScriptManager:dismissSubtitle()
		
		addOnEnterAreaTrigger(8,14,0,-1,-1,"revealTroops_1","revealTroops_1")
		addOnEnterAreaTrigger(2,22,0,-1,-1,"revealTroops_2","revealTroops_2")
		addOnEnterAreaTrigger(16,4,0,-1,-1,"revealTroops_3","revealTroops_3")
		addOnEnterAreaTrigger(6,20,0,-1,-1,"revealTroops_4","revealTroops_4")
		addOnEnterAreaTrigger(16,16,0,-1,-1,"revealTroops_5","revealTroops_5")
		addOnEnterAreaTrigger(2,7,0,-1,-1,"revealTroops_6","revealTroops_6")
		addOnEnterAreaTrigger(17,9,0,-1,-1,"revealTroops_7","revealTroops_7")
		addOnEnterAreaTrigger(9,21,0,-1,-1,"revealTroops_8","revealTroops_8")
		ScriptManager:displaySubtitle(Text.T_MAP_3_2_MISSION_MAIN_2_SUB)	
	end

--TODO:			BEGINNING CUTSCENE UNITS SPAWN		--


function SpawnUnit1()
	
	
	local Soldiers1 = ScriptManager:spawnUnit(0,18,2,2,-1,-1)				--spawn hvy tank
	ScriptManager:moveUnit(Soldiers1:getId(),3,5,"SpawnUnit2")
	
end

function SpawnUnit2()
	
	
	local Soldiers2 = ScriptManager:spawnUnit(0,15,2,2,-1,-1)				--spawn inf
	ScriptManager:moveUnit(Soldiers2:getId(),4,4,"SpawnUnit2_1")
	
	
end


function SpawnUnit2_1()
	
	
	local Soldiers2_1 = ScriptManager:spawnUnit(0,16,1,5,-1,-1)				--spawn inf
	ScriptManager:moveUnit(Soldiers2_1:getId(),4,6,"SpawnUnit3")
		
end

function SpawnUnit3()
	
	
	local Soldiers3 = ScriptManager:spawnUnit(0,15,2,3,-1,-1)				--spawn inf
	ScriptManager:moveUnit(Soldiers3:getId(),3,6,"SpawnUnit3_1")
		
	
end

function SpawnUnit3_1()
	
	
	local Soldiers3_1 = ScriptManager:spawnUnit(0,20,2,3,-1,-1)				--spawn katusha
	ScriptManager:moveUnit(Soldiers3_1:getId(),5,4,"SpawnUnit4")
	
end

function SpawnUnit4()
	
	
	local Soldiers4 = ScriptManager:spawnUnit(0,18,2,2,-1,-1)				--spawn hvytank
	ScriptManager:moveUnit(Soldiers4:getId(),4,5,"SpawnUnit4_1")
	ScriptManager:displaySubtitle(Text.T_MAP_3_2_MISSION_MAIN_SUB)
end

function SpawnUnit4_1()
	
	
	local Soldiers4_1 = ScriptManager:spawnUnit(0,17,3,2,-1,-1)				--spawn tank
	ScriptManager:moveUnit(Soldiers4_1:getId(),3,4,"SpawnUnit5")
	
end

function SpawnUnit5()
	
	
	local Soldiers5 = ScriptManager:spawnUnit(0,Units.UNIT_SU_SUPPLY_TRUCK,3,2,-1,-1)				--spawn tank
	ScriptManager:moveUnit(Soldiers5:getId(),3,3,"Basecamera")
	ScriptManager:dismissSubtitle()
end


--Moves units and camera to the base

function Basecamera()

	ScriptManager:moveCamera(6,7,0.6,6,"Cutscene_end_01")
	ScriptManager:showMarkerAt(6,7)
end

function Cutscene_end_01()
	
	ScriptManager:stopCutscene()
	if ScriptManager:isControllerConnected() then
		ScriptManager:setControllerCursorPosition( 5, 6 );
	end
	ScriptManager:sleep(2,"KillDialogue")
	ScriptManager:hideMarkerAt(6,7)
	ScriptManager:continueAfterCutscene("Mission_update")
end

function Mission_update()

	ScriptManager:hideMarkerAt(6,7)
	ScriptManager:sleep(2,"onHQcaptured")
	ScriptManager:setMissionStatus(1,MissionStatus.IN_PROGRESS)
	
	------------------------------------------------------------------------
	--TODO:REMOVE GOTT!
	--ScriptManager:spawnUnit(0,Units.GOTT,1,1,0,0)
	--ScriptManager:spawnUnit(0,Units.GOTT,10,10,0,0)
	--ScriptManager:moveCamera(2,2,0.77,0,"")	--TODO: SCREENSHOTS
	------------------------------------------------------------------------
end

function startplanes()

	Unit:changeSquadState(1,69,AiState.AIRPLANE)
	ScriptManager:removeTrigger("startplanes")		
	
end

function Cutscene_MoveCam_Airbase()			-- only happens in the player turn
	ScriptManager:removeTrigger( "Cutscene_MoveCam_Airbase" );
	ScriptManager:runForPlayer(0, "waitaftercutscene")
	
end

function waitaftercutscene()

	ScriptManager:continueAfterCutscene("Cutscene_MoveCam_Airbase_2")

end

function Cutscene_MoveCam_Airbase_2()			--TODO: CAMERA MOVEMENT TOWARDS THE ENEMY Airbase	--

	ScriptManager:dismissSubtitle()
	ScriptManager:revealFogOfWar(21,23,4,true)
	ScriptManager:showMarkerAt(21,23)
	ScriptManager:startCutscene()
	ScriptManager:sleep(3,"waitatbase")
end

function waitatbase()
	ScriptManager:spawnUnit(1,Units.UNIT_DE_FIGHTER,21,22,69,AiState.WAIT) --TODO: changed squadId from 101 to 69 by pdo (plane wont start at airfield)
	ScriptManager:moveCamera(20,21,1.0,5,"cutscene_end_2")
	ScriptManager:displaySubtitle(Text.T_MAP_3_2_MISSION_SECONDARY_SUB)
	
end

function cutscene_end_2()		
	ScriptManager:hideMarkerAt(21,23)
	ScriptManager:stopCutscene()
	ScriptManager:sleep(2,"KillDialogue")
	ScriptManager:revealFogOfWar(21,23,4,false)
	ScriptManager:continueAfterCutscene("backtoBaseandstartmission")

end

function Missionupdate_02()
	ScriptManager:removeTrigger("Airbase_failed")
	ScriptManager:setMissionStatus(3,MissionStatus.ACCOMPLISHED)

end

function Airbase_failed()
	ScriptManager:removeTrigger("Cutscene_MoveCam_Airbase")
	ScriptManager:setMissionStatus(3,MissionStatus.FAILED)
end

function onHQcaptured()			--capture and destroy mechanic of outpost

	ScriptManager:dismissSubtitle()
	ScriptManager:addOnUnitCapturedTrigger(-1, -1, 0, "Missionupdate_01", "Missionupdate_01")
	ScriptManager:addOnTurnStartTrigger(0,-1,ScriptManager:getCurrentRound()+1,"startplanes", "startplanes")
	
end

--	Mission 2 scattered units behavior

function Missionupdate_01()					--When outpost is captured, complete first objective

	ScriptManager:dismissSubtitle()
	ScriptManager:startCutscene()
	ScriptManager:setMissionStatus(1,MissionStatus.ACCOMPLISHED)
	ScriptManager:moveCamera(7,8,1.1,5,"nextmission_1")		
		--Start spawning reinforcements
	ScriptManager:addOnTurnStartTrigger(0,-1,ScriptManager:getCurrentRound()+3,"spawnEnemyWave", "spawnEnemyWave")		
	depotcaptured()
	ScriptManager:sleep(10,"startprotectmission")		
		
end

function startprotectmission()

	ScriptManager:dismissSubtitle()
	ScriptManager:setMissionStatus(4,MissionStatus.IN_PROGRESS)

end

function nextmission_1()			--find first troops and start search for troops mission
	
	ScriptManager:spawnUnit(0,Units.UNIT_SU_RIFLE,6,9,-1,-1)
	ScriptManager:spawnUnit(0,Units.UNIT_SU_AA,7,8,-1,-1)
	ScriptManager:setMissionStatus(2,MissionStatus.IN_PROGRESS)
	ScriptManager:stopCutscene()
	ScriptManager:dismissSubtitle()
	ScriptManager:continueAfterCutscene("Ontroopsfound")
	
end

function showtroops_progresstext()			--Show the x/3 artillery counter
	
	ScriptManager:dismissSubtitle()
	ScriptManager:preparePopupText()
	ScriptManager:addPopupTextParameter("x",ScriptManager:getMissionProgress(2))		
	ScriptManager:displaySubtitle(Text.T_MAP_3_2_MISSION_MAIN_2_COUNTER)							
	ScriptManager:sleep(3,"KillDialogue")		
end

function Ontroopsfound()		--Count troops and when all are found, set mission to complete
	ScriptManager:sleep(3,"KillDialogue")
	ScriptManager:stopCutscene()
	ScriptManager:setMissionProgress(2,ScriptManager:getMissionProgress(2)+1)				
		ScriptManager:continueAfterCutscene("showtroops_progresstext")
		
	if ScriptManager:getMissionProgress(2) == 9 then
		ScriptManager:setMissionStatus(2,MissionStatus.ACCOMPLISHED)
		ScriptManager:setMissionStatus(4,MissionStatus.ACCOMPLISHED)
		ScriptManager:continueAfterCutscene("SleepNextMissionTeaser")
	end
end	

function SleepNextMissionTeaser()
		ScriptManager:sleep(3,"Next_mission_teaser")
end

-- Area trigger around the tiles where the troops hide

function addOnEnterAreaTrigger(x,y,playerId,unitTemplateId,unitInstanceId,callbackName,triggerId)
	for i = x-1, x+1 do
		ScriptManager:addOnEnterTileTrigger(i,y,playerId,unitTemplateId,unitInstanceId,true,callbackName,triggerId)
	end
	local xx = y % 2 -1
	ScriptManager:addOnEnterTileTrigger(x+xx,y+1,playerId,unitTemplateId,unitInstanceId,true,callbackName,triggerId)
	ScriptManager:addOnEnterTileTrigger(x+xx+1,y+1,playerId,unitTemplateId,unitInstanceId,true,callbackName,triggerId)
	ScriptManager:addOnEnterTileTrigger(x+xx,y-1,playerId,unitTemplateId,unitInstanceId,true,callbackName,triggerId)
	ScriptManager:addOnEnterTileTrigger(x+xx+1,y-1,playerId,unitTemplateId,unitInstanceId,true,callbackName,triggerId)
end

function revealTroops_1()
	ScriptManager:blockUI(false)
	for i = 1, 7 do
		ScriptManager:removeTrigger("revealTroops_1")
	end
	
	ScriptManager:spawnUnit(0,Units.UNIT_SU_AA,7,13,0,0)
	ScriptManager:spawnUnit(0,Units.UNIT_SU_MORTAR,8,13,0,0)
	ScriptManager:dismissSubtitle()
	ScriptManager:startCutscene()
	ScriptManager:displaySubtitle(Text.T_MAP_3_2_MISSION_TROOPSFOUND)
	ScriptManager:moveCamera(7,13,1.0,5,"Ontroopsfound")		
	
end

function revealTroops_2()
	ScriptManager:blockUI(false)
	for i = 1, 7 do
		ScriptManager:removeTrigger("revealTroops_2")
	end
	
	ScriptManager:spawnUnit(0,Units.UNIT_SU_AA,1,23,0,0)
	ScriptManager:spawnUnit(0,Units.UNIT_SU_MORTAR,2,22,0,0)
	ScriptManager:dismissSubtitle()
	ScriptManager:startCutscene()
	ScriptManager:displaySubtitle(Text.T_MAP_3_2_MISSION_TROOPSFOUND)
	ScriptManager:moveCamera(1,23,1.0,5,"Ontroopsfound")		
end

function revealTroops_3()

	ScriptManager:blockUI(false)
	for i = 1, 7 do
		ScriptManager:removeTrigger("revealTroops_3")
	end
	ScriptManager:spawnUnit(0,Units.UNIT_SU_TANK_KV1,16,4,0,0)
	ScriptManager:spawnUnit(0,Units.UNIT_SU_MORTAR,17,4,0,0)
	ScriptManager:dismissSubtitle()
	ScriptManager:startCutscene()
	ScriptManager:displaySubtitle(Text.T_MAP_3_2_MISSION_TROOPSFOUND)
	ScriptManager:moveCamera(16,4,1.0,5,"Ontroopsfound")		
	
end

function revealTroops_4()

	ScriptManager:blockUI(false)
	for i = 1, 7 do
		ScriptManager:removeTrigger("revealTroops_4")
	end
	ScriptManager:spawnUnit(0,Units.UNIT_SU_KATYUSHA,6,20,0,0)
	ScriptManager:spawnUnit(0,Units.UNIT_SU_TANK_T34,6,21,0,0)
	ScriptManager:dismissSubtitle()
	ScriptManager:startCutscene()
	ScriptManager:displaySubtitle(Text.T_MAP_3_2_MISSION_TROOPSFOUND)
	ScriptManager:moveCamera(6,20,1.0,3,"Ontroopsfound")		
end

function revealTroops_5()

	ScriptManager:blockUI(false)
	for i = 1, 7 do
		ScriptManager:removeTrigger("revealTroops_5")
	end
	ScriptManager:spawnUnit(0,Units.UNIT_SU_HOWITZER,16,16,0,0)
	ScriptManager:spawnUnit(0,Units.UNIT_SU_MORTAR,15,17,0,0)
	ScriptManager:dismissSubtitle()
	ScriptManager:startCutscene()
	ScriptManager:displaySubtitle(Text.T_MAP_3_2_MISSION_TROOPSFOUND)
	ScriptManager:moveCamera(16,16,1.0,3,"Ontroopsfound")		
end

function revealTroops_6()

	ScriptManager:blockUI(false)
	for i = 1, 7 do
		ScriptManager:removeTrigger("revealTroops_6")
	end
	ScriptManager:spawnUnit(0,Units.UNIT_SU_RIFLE,2,8,0,0)
	ScriptManager:spawnUnit(0,Units.UNIT_SU_MORTAR,2,7,0,0)
	ScriptManager:dismissSubtitle()
	ScriptManager:startCutscene()
	ScriptManager:displaySubtitle(Text.T_MAP_3_2_MISSION_TROOPSFOUND)
	ScriptManager:moveCamera(2,8,1.0,3,"Ontroopsfound")		
end

function revealTroops_7()

	ScriptManager:blockUI(false)
	for i = 1, 7 do
		ScriptManager:removeTrigger("revealTroops_7")
	end
	ScriptManager:spawnUnit(0,Units.UNIT_SU_TANK_DESTROYER,18,10,0,0)
	ScriptManager:spawnUnit(0,Units.UNIT_SU_RIFLE,16,8,0,0)
	ScriptManager:dismissSubtitle()
	ScriptManager:startCutscene()
	ScriptManager:displaySubtitle(Text.T_MAP_3_2_MISSION_TROOPSFOUND)
	ScriptManager:moveCamera(18,10,1.0,3,"Ontroopsfound")		
end

function revealTroops_8()
	ScriptManager:blockUI(false)
	for i = 1, 7 do
		ScriptManager:removeTrigger("revealTroops_8")
	end
	ScriptManager:spawnUnit(0,Units.UNIT_SU_SUPPLY_TRUCK,10,22,0,0)
	ScriptManager:spawnUnit(0,Units.UNIT_SU_TANK_T34,9,20,0,0)
	ScriptManager:dismissSubtitle()
	ScriptManager:startCutscene()
	ScriptManager:displaySubtitle(Text.T_MAP_3_2_MISSION_TROOPSFOUND)
	ScriptManager:moveCamera(10,22,1.0,3,"Ontroopsfound")		
end

--ENEMY SPAWNING BEHAVIOUR-

function spawnEnemyWave()

if ScriptManager:getCurrentRound() < 15 then
		
	ScriptManager:startCutscene()
	ScriptManager:removeTrigger("spawnEnemyWave")
	local position = spawnPositions[math.random(1,#spawnPositions)]
	ScriptManager:addOnTurnStartTrigger(0,-1,ScriptManager:getCurrentRound()+3,"spawnEnemyWave", "spawnEnemyWave")		
		-- Every X rounds reinforcements arrive
	ScriptManager:moveCamera(position[1],position[2],1,5,"waitaftercutscene_2")
	ScriptManager:storeValue("waveX",position[1])
	ScriptManager:storeValue("waveY",position[2])
	ScriptManager:storeValue("targetX",position[3])
	ScriptManager:storeValue("targetY",position[4])
	end
end

function waitaftercutscene_2()

	ScriptManager:continueAfterCutscene("showNewWave")

end

function showNewWave()

	ScriptManager:dismissSubtitle()
	local x = ScriptManager:getValue("waveX")
	local y = ScriptManager:getValue("waveY")
	local targetX = ScriptManager:getValue("targetX")
	local targetY = ScriptManager:getValue("targetY")
	
	ScriptManager:revealFogOfWar(x,y,4,true)
	ScriptManager:displaySubtitle(Text.T_MAP_3_2_ENEMY_REINFORCEMENTS)			
	
	local attackers  = attackerWaves[math.random(1,#attackerWaves)]
	
	for i = 1, #attackers do
		if attackers[i] == Units.UNIT_DE_BOMBER or attackers[i] == Units.UNIT_DE_FIGHTER then
			ScriptManager:spawnUnit(1,attackers[i],x,y,100+ScriptManager:getCurrentRound(),AiState.AIRPLANE)
		else
			ScriptManager:spawnUnit(1,attackers[i],x,y,100+ScriptManager:getCurrentRound(),AiState.ATTACK)
		end
	end
	ScriptManager:setTargetTile(1,100+ScriptManager:getCurrentRound(),targetX,targetY)
	ScriptManager:sleep(3,"hideNewWave")
end

function hideNewWave()
	local x = ScriptManager:getValue("waveX")
	local y = ScriptManager:getValue("waveY")
	
	ScriptManager:revealFogOfWar(x,y,4,false)
	
	ScriptManager:dismissSubtitle()
	ScriptManager:moveCamera(x, y,0.7,2,"")
	ScriptManager:stopCutscene()
	ScriptManager:continueAfterCutscene("backtoBase")
end

function backtoBase()

	ScriptManager:moveCamera(6,7,0.7,2,"")

end
--	Start airfield mission

function backtoBaseandstartmission()
	ScriptManager:moveCamera(6,7,0.7,2,"")
	ScriptManager:setMissionStatus(3,MissionStatus.IN_PROGRESS)
end

--- Artillery mission

function OnenemyArtilleryspotted(attacker, defender)
KillDialogue()
	targetID = -1
	if attacker:getTemplateId() == Units.UNIT_DE_SUPER_ARTILLERY then				
		targetID = attacker:getId()
	end
	if defender:getTemplateId() == Units.UNIT_DE_SUPER_ARTILLERY then
		targetID = defender:getId()
	end
	ScriptManager:removeTrigger("OnenemyArtilleryspotted")
	
	
	if targetID == nil then
		ScriptManager:runForPlayer(0,"startArtilleryMission")
		else
		ScriptManager:runForPlayer(0, "MoveCamToTarget")
	end
end

function MoveCamToTarget()
	ScriptManager:startCutscene()
	ScriptManager:moveCamera(Unit:getUnitById(targetID):getX(),Unit:getUnitById(targetID):getY(),0.9,2,"CamOnTarget")
end

function CamOnTarget()
	ScriptManager:revealFogOfWar(Unit:getUnitById(targetID):getX(),Unit:getUnitById(targetID):getY(),1,true)
	ScriptManager:showMarkerAt(Unit:getUnitById(targetID):getX(),Unit:getUnitById(targetID):getY())
	ScriptManager:displaySubtitle(Text.T_MAP_3_2_ARTILLERY_SUB)
	ScriptManager:sleep(5,"KillDialogue")
	ScriptManager:moveCamera(Unit:getUnitById(targetID):getX(),Unit:getUnitById(targetID):getY(),0.9,3,"ZoomCamOutAndEndScript")
end

function ZoomCamOutAndEndScript()
	ScriptManager:moveCamera(Unit:getUnitById(targetID):getX(),Unit:getUnitById(targetID):getY(),0.8,2,"EndPseudoCutScene")
	ScriptManager:hideMarkerAt(Unit:getUnitById(targetID):getX(),Unit:getUnitById(targetID):getY())
	ScriptManager:revealFogOfWar(Unit:getUnitById(targetID):getX(),Unit:getUnitById(targetID):getY(),1,false)
end

function EndPseudoCutScene()
	ScriptManager:stopCutscene()
	ScriptManager:moveCamera(6,7,0.7,2,"")
	ScriptManager:continueAfterCutscene("startArtilleryMission")
end

function startArtilleryMission()
	ScriptManager:setMissionStatus(5,MissionStatus.IN_PROGRESS)
		
end

function OnenemyArtillerydestroyed()
	ScriptManager:runForPlayer(0,"_OnenemyArtillerydestroyed")
end

function _OnenemyArtillerydestroyed()
	
	ScriptManager:removeTrigger("OnenemyArtilleryspotted")
	ScriptManager:sleep(3,"KillDialogue")
	if ScriptManager:getMissionStatus(5)== MissionStatus.NOT_STARTED then
		ScriptManager:setMissionStatus(5,MissionStatus.IN_PROGRESS)
	end		
	ScriptManager:setMissionProgress(5,1)						
	--Add progress +1 because you just killed one Artillery		
	ScriptManager:removeTrigger("OnenemyArtillerydestroyed")	
	-- Destroy trigger so the progress doesn't start all over again each time you kill an Artillery
	ScriptManager:addOnUnitDestroyedTrigger(1,Units.UNIT_DE_SUPER_ARTILLERY,-1,"OnenemyArtillerydestroyed_1","OnenemyArtillerydestroyed_1")
	showprogresstext()
	
end

function showprogresstext()

	ScriptManager:dismissSubtitle()
	ScriptManager:removeTrigger("OnenemyArtilleryspotted")
	ScriptManager:preparePopupText()
	ScriptManager:addPopupTextParameter("x",ScriptManager:getMissionProgress(5))		
	ScriptManager:displaySubtitle(Text.T_MAP_3_2_MISSION_SECONDARY_2_COUNTER)					--"Artillery destroyed:\n{x}/3"			
	ScriptManager:sleep(3,"KillDialogue")		

end

function OnenemyArtillerydestroyed_1()
	
	ScriptManager:setMissionProgress(5,ScriptManager:getMissionProgress(5)+1)				--Update on mission progress when one Artillery is destroyed
	showprogresstext()
	if ScriptManager:getMissionProgress(5) == 3 then
		ScriptManager:setMissionStatus(5,MissionStatus.ACCOMPLISHED)
		
	end
end	

--MISSION ENDINGS--


function Next_mission_teaser()	
	ScriptManager:disableEndTurnButton()
	ScriptManager:blockUI(true)
	ScriptManager:displaySubtitleAutoWithCallback(Text.T_MAP_3_2_NEXT_MISSION_TEASER,"Mission_completed")			
end

function Mission_completed()
	ScriptManager:blockUI(true)														-- Mission END!--
	ScriptManager:displaySubtitleAutoWithCallback(Text.T_MISSION_VICTORY,"")	
	ScriptManager:continueAfterCutscene("Showmessage")
end

function Depot_destroyed_wait()
	--ScriptManager:blockUI(true)
	ScriptManager:disableEndTurnButton()
	ScriptManager:runForPlayer(0, "Depot_destroyed")
	
end

function Depot_destroyed()
	
	--ScriptManager:blockUI(true)
	ScriptManager:moveCamera(6,7,1.1,2,"")
	ScriptManager:blockUI(true)
	ScriptManager:setMissionStatus(1,MissionStatus.FAILED)
	ScriptManager:displaySubtitleAutoWithCallback(Text.T_MAP_3_2_MISSION_FAILED,"Missionfailed")
end

function Missionfailed()	
	ScriptManager:disableEndTurnButton()
	ScriptManager:blockUI(true)
	ScriptManager:displaySubtitleAutoWithCallback(Text.T_MISSION_FAILED,"")
	ScriptManager:sleep(5,"Showmessage")											
end

function Showmessage()
	ScriptManager:disableEndTurnButton()
	ScriptManager:blockUI(true)
	ScriptManager:sleep(3,"endgame_sleep")
	
end

function endgame_sleep()
	ScriptManager:endGame()
end

