require "lua/Constants"
require "lua/Functions"
require "lua/Functions_2"
require "lua/Text"
require "lua/Units"
require "math"

Missions = {
	MISSION_MAIN = 1,
}

function onGameStart()
	--print( string.format( "[SCRIPT] onGameStart()..." ) );
	ScriptManager:preparePopupText();
	ScriptManager:createMission( Missions.MISSION_MAIN, true, Text.T_MISSION_TUTORIAL_MAIN_1 );
	ScriptManager:setMissionStatus( Missions.MISSION_MAIN, MissionStatus.IN_PROGRESS );
	ScriptManager:setMissionXp( Missions.MISSION_MAIN, 25 );
	ScriptManager:setMedalReward( 200 );
	-- TODO: ENABLE AFTER TESTING!
	ScriptManager:disableSkipCutsceneButton();
	ScriptManager:disableSkipReplayButton();
	Unit:getUnitById( 13 ):setHealth( 80 );
	Unit:getUnitById( 14 ):setHealth( 60 );
	Unit:getUnitById( 12 ):setHealth( 50 );
	ScriptManager:sleep( 0, "StartIntro" );
	ScriptManager:disableAICalculation();
	ScriptManager:moveCamera( 7, 2, 0.85, 0, "" );
	ScriptManager:addOnControllerConnectedTrigger( "onControllerConnect", "onControllerConnect", false );
	ScriptManager:addOnControllerDisconnectedTrigger( "onControllerDisconnect", "onControllerDisconnect", false );
	return 1;
end

function HideDialogue()
	ScriptManager:dismissSubtitle();
end

function StartIntro()
	ScriptManager:startCutscene();
	ScriptManager:sleep( 1, "IntroUnitBattle" );
	ScriptManager:sleep( 0.5, "MoveCamera" );
	ScriptManager:sleep( 3, "IntroDialogues" );
end

function IntroDialogues()
	ScriptManager:displaySubtitle( Text.T_MISSION_TUTORIAL_2_1 );
	ScriptManager:sleep( 4, "HideDialogue" );
	ScriptManager:sleep( 8, "Dialogue2" );
end

function Dialogue2()
	ScriptManager:displaySubtitle( Text.T_MISSION_TUTORIAL_2_2 );
	ScriptManager:sleep( 4, "HideDialogue" );
	ScriptManager:sleep( 9, "Dialogue3" );
end

function Dialogue3()
	ScriptManager:displaySubtitle( Text.T_MISSION_TUTORIAL_2_3 );
	ScriptManager:sleep( 3, "HideDialogue" );
end

function DialogueReinforcements()
	ScriptManager:displaySubtitle( Text.T_MISSION_REINFORCEMENTS );
end

function MoveCamera()
	ScriptManager:moveCamera( 
	7, 4, 		--X, Y
	0.8, 10,	--ZOOM, TIME
	""	--CallBack
	 );
end

function CutsceneEnd()
	ScriptManager:moveUnit( 11, 7, 4, "" );
	ScriptManager:moveUnit( 10, 7, 3, "" );
	ScriptManager:fortifyUnit( 10 );
	ScriptManager:stopCutscene();
	--	local TigerID = ScriptManager:getValue( "unitTankTiger" );
	--	local tiger = Unit:getUnitById( TigerID );
	--	tiger:
		ScriptManager:continueAfterCutscene( "SelectUnit" ); 
	end

function IntroUnitBattle()
	ScriptManager:moveUnit( 9, 7, 5, "Attack9" );
end

function Attack9()
	Unit:getUnitById( 2 ):setHealth( 500 )
	ScriptManager:attackUnit( 
	9,	--ATTACKER
	2,	--DEFENDER
	"AttackTiger" );
end

function AttackTiger()
	local defender = Unit:getUnitById( 5 );
	defender:setHealth( 50 );
	ScriptManager:attackUnit( 
	7,	--ATTACKER
	5, 	--DEFEND
	"Attack6" );
end

function Attack6()
	local unit = Unit:getUnitById( 7 );
	unit:setHealth( 20 );
	ScriptManager:attackUnit( 
	3,	--ATTACKER
	7,	--DEFENDER
	"Move11" );
end

function Move11()
	local defender = Unit:getUnitById( 6 );
	defender:setHealth( 100 );
	ScriptManager:moveUnit( 11, 5, 3, "Attack11" );
end

function Attack11()
	ScriptManager:attackUnit( 
	11,	--ATTACKER
	3,	--DEFENDER
	"Move4" );
end

function Move4()
	ScriptManager:moveUnit( 4, 9, 4, "Attack4" );
end

function Attack4()
	ScriptManager:attackUnit( 
	4,	--ATTACKER
	2,	--DEFENDER
	"Move10" );
end

function Move10()
	ScriptManager:moveUnit( 10, 8, 3, "Attack10" );
end

function Attack10()
	local defender = Unit:getUnitById( 6 );
	defender:setHealth( 100 );
	local attacker = Unit:getUnitById( 10 );
	attacker:setHealth( 400 );
	ScriptManager:attackUnit( 
	10,	--ATTACKER
	6, 	--DEFEND
	"EnemyCaptureBase" );
end

function EnemyCaptureBase()
	ScriptManager:captureUnit( 9, 0, "SpawnReinforcements_wait" );
end

function SpawnReinforcements_wait()
	Unit:getUnitById( 11 ):setHealth( 450 );
	ScriptManager:sleep( 2, "SpawnReinforcements" );
end

function SpawnReinforcements()
	ScriptManager:moveCamera( 
	3, 4, 		--X, Y
	0.8, 4,	--ZOOM, TIME
	"CutsceneEnd"	--CallBack
	 );
	
	local unit = ScriptManager:spawnUnit( 
	0, 4, 	--PLAYER, UNITID
	1, 2,	--X, Y
	30, 5 );	--AI
	ScriptManager:storeValue( "unitTankTiger",unit:getId() );
	ScriptManager:moveUnit( unit:getId(),
	2, 4,	--X, Y
	"" );
end

function SelectUnit()
	ScriptManager:showMarkerAt( 2, 4 )
	
	cleanupController();
	if ScriptManager:isControllerConnected() then
		ScriptManager:addOnControllerSelectedTileTrigger( 2, 4, "cursorOverUnitSinglePress", "cursorOverUnit", false );
		ScriptManager:setControllerCursorPosition( 4, 4 );
		ScriptManager:setControllerCursorEnabled( true );
		
	else
		ScriptManager:showFingerAtTile( 2, 4, 1 );
	end
	
	ScriptManager:disableEndTurnButton();
	ScriptManager:displaySubtitle( Text.T_MISSION_TUTORIAL_2_5 );
	ScriptManager:disableAllInteractionButtons();
	ScriptManager:disableAllTilesExcept( 2, 4 );
	local Tiger = ScriptManager:getValue( "unitTankTiger" );
	ScriptManager:addOnUnitSelectedTrigger( 0, -1, Tiger, "TigerSelected" );
end

function TigerSelected()
	ScriptManager:dismissSubtitle();
	ScriptManager:displaySubtitle( Text.T_MISSION_TUTORIAL_2_6 );
	
	
	cleanupController();
	if ScriptManager:isControllerConnected() then
		ScriptManager:addOnControllerSelectedTileTrigger( 4, 3, "cursorOverUnit", "cursorOverUnit", false );
		ScriptManager:setControllerCursorEnabled( true );
		
	else
		ScriptManager:moveFingerToTile( 4, 3 );
	end
	
	ScriptManager:hideMarkerAt( 2, 4 );
	ScriptManager:showMarkerAt( 4, 3 );
	ScriptManager:disableAllTilesExcept( 4, 3 );
	ScriptManager:addOnTileDeselectedTrigger( 4, 3, 0, "TigerMove" );
end

function TigerMove()
	ScriptManager:disableAllTilesExcept( 0, 0 );
	ScriptManager:dismissSubtitle();
	ScriptManager:hideFinger();
	ScriptManager:hideMarkerAt( 4, 3 );
	
	local Tiger = ScriptManager:getValue( "unitTankTiger" );
	ScriptManager:addOnEnterTileTrigger( 4, 3, 0, -1, Tiger, true, "TigerInPosition" );
end

function TigerInPosition()
	checkControllerAndHideButton();
	
	ScriptManager:moveCamera( 6, 3, 0.8, 1, "" );
	ScriptManager:removeAllTriggers();
	ScriptManager:displaySubtitle( Text.T_MISSION_TUTORIAL_2_6_1 );
	ScriptManager:sleep( 3, "HideDialogue" );
	ScriptManager:sleep( 4, "TigerFortify" );
	Unit:getUnitById(  0 ):setIsAttackable( false );
	Unit:getUnitById(  9 ):setIsAttackable( false );
	Unit:getUnitById( 10 ):setIsAttackable( false );
	Unit:getUnitById( 11 ):setIsAttackable( false );
end

function TigerFortify()
		ScriptManager:moveCamera( 4, 3, 0.9, 2, "TigerFortify2" );
	end 

function TigerFortify2()
	RemoveTrigger( "TigerFortify2" );
	
	cleanupController();
	if ScriptManager:isControllerConnected() then
		ScriptManager:displaySubtitle( Text.T_TUTORIAL_CONTROLLER_PRESS_BUTTON );
		-- show it with other function cause otherwise next button display won't disappear
		ScriptManager:showControllerButtonAtTile( ControllerButton.FACE_LEFT, 4, 3, 0.2 );
		ScriptManager:addOnControllerButtonReleasedTrigger( ControllerButton.FACE_LEFT, "TigerFortify2Controller", "TigerFortify2Controller" );
	
	else
		ScriptManager:displaySubtitle( Text.T_MISSION_TUTORIAL_2_7 );
	end
	
	ScriptManager:enableInteractionButton( InteractionButton.FORTIFY );
	ScriptManager:startHighlightingInteractionButton( InteractionButton.FORTIFY );
	local Tiger = ScriptManager:getValue( "unitTankTiger" );
	ScriptManager:addOnInteractionButtonSelectedTrigger( 0, -1, Tiger, InteractionButton.FORTIFY, "TigerFortified", "TigerFortified" )
end

function TigerFortify2Controller()
	RemoveTrigger( "TigerFortify2Controller" );
	ScriptManager:dismissSubtitle();
	ScriptManager:displaySubtitle( Text.T_MISSION_TUTORIAL_2_7 );
	ScriptManager:hideControllerButton();
	showControllerButtonGfx( ControllerButton.FACE_DOWN, 1, 4, 3 );
end

function TigerFortified()
	checkControllerAndHideButton();
	
	ScriptManager:dismissSubtitle()
	ScriptManager:removeAllTriggers()
	ScriptManager:stopHighlightingInteractionButton( InteractionButton.FORTIFY )
	ScriptManager:disableAllInteractionButtons();
	ScriptManager:displaySubtitle( Text.T_MISSION_TUTORIAL_2_8 );
	ScriptManager:sleep( 2.5, "AttackEnemies" );
	ScriptManager:sleep( 3.5, "HideDialogue" );
end

function AttackEnemies()
	ScriptManager:moveCamera( 6, 3, 0.8, 1, "MarkEnemies" );
end

function MarkEnemies()
		--TODO:SCREENSHOT
--	ScriptManager:spawnUnit( 0, 30, 10, 2, 0, 0 );
--	ScriptManager:moveCamera( 10, 2, 0.77, 0, "" );
	Unit:getUnitById( 9 ):setIsAttackable( true );
	Unit:getUnitById( 10 ):setIsAttackable( true );
	Unit:getUnitById( 11 ):setIsAttackable( true );
	ScriptManager:displaySubtitle( Text.T_MISSION_TUTORIAL_2_9 );
	ScriptManager:removeAllMarkers();
	ScriptManager:disableAllTilesExcept( 0, 0 );
	
	cleanupController();
	if ScriptManager:isControllerConnected() then
		ScriptManager:setControllerCursorEnabled( true );
	end
	
	local tank1 = Unit:getUnitById( 11 );
	local tank2 = Unit:getUnitById(  9 );
	local tank3 = Unit:getUnitById( 10 );
	
	if tank1 ~= nil then
		ScriptManager:showMarkerAt( tank1:getX(), tank1:getY() );
		ScriptManager:enableAdditionalTile( tank1:getX(), tank1:getY() );
		
		if ScriptManager:isControllerConnected() then
			ScriptManager:addOnControllerSelectedTileTrigger( tank1:getX(), tank1:getY(), "cursorOverUnit", "cursorOverUnit", false );
		end
	end
	
	if tank2 ~= nil then
		ScriptManager:showMarkerAt( tank2:getX(), tank2:getY() );
		ScriptManager:enableAdditionalTile( tank2:getX(), tank2:getY() );
		
		if ScriptManager:isControllerConnected() then
			ScriptManager:addOnControllerSelectedTileTrigger( tank2:getX(), tank2:getY(), "cursorOverUnit", "cursorOverUnit", false );
		end
	end
	
	if tank3 ~= nil then
		ScriptManager:showMarkerAt( tank3:getX(), tank3:getY() );
		ScriptManager:enableAdditionalTile( tank3:getX(), tank3:getY() );
		
		if ScriptManager:isControllerConnected() then
			ScriptManager:addOnControllerSelectedTileTrigger( tank3:getX(), tank3:getY(), "cursorOverUnit", "cursorOverUnit", false );
		end
	end
	
	ScriptManager:addOnUnitSelectedTrigger( -1, -1, -1, "HideDialogue", "HideDialogue" );
	ScriptManager:addOnUnitHasAttackedTrigger( -1, -1, -1, "EngageUnit", "EngageUnit" );
	ScriptManager:addOnUnitDeselectedTrigger( -1, -1, 0, "HideDialogue", "HideDialogue" );
end

function HideDialogue()
	ScriptManager:dismissSubtitle();
end

function EngageUnit()
	ScriptManager:disableAllTilesExcept( 0, 0 );
	ScriptManager:dismissSubtitle();
	ScriptManager:removeAllMarkers();
	ScriptManager:moveCamera( 6, 3, 0.75, 1, "EndTurn" );
end

--end first turn 
function EndTurn()
	ScriptManager:displaySubtitle( Text.T_MISSION_TUTORIAL_2_11 );
	ScriptManager:enableEndTurnButton();
	
	cleanupController();
	if ScriptManager:isControllerConnected() then
		showControllerButtonGfx( ControllerButton.FACE_UP, 1, 6, 3 );
	end
		
		ScriptManager:startHighlightingEndTurnButton();
	
	ScriptManager:addOnTurnStartTrigger( 1, -1, 0, "endEnemyTurnImmediatly" );
	ScriptManager:addOnTurnStartTrigger( 0, -1, 1, "StartOwnTurn" );
end

-------------------- Enemy attack tiger cutscene --------------------

function endEnemyTurnImmediatly()
	ScriptManager:hideControllerButton();
	ScriptManager:endTurn();
end

function StartOwnTurn()
	ScriptManager:removeAllTriggers();
	ScriptManager:moveCamera( 8, 4, 0.7, 0, "" );
	ScriptManager:startCutscene();
	ScriptManager:dismissSubtitle();
	ScriptManager:sleep( 1.5, "EnemyAttack" );
end

function EnemyAttack()
	ScriptManager:startCutscene();
	ScriptManager:dismissSubtitle();
	ScriptManager:sleep( 1, "showDialogue12" );
	local Tiger = ScriptManager:getValue( "unitTankTiger" );
	if Unit:getUnitById( 9 ) ~= nil then
		ScriptManager:attackUnit( 
		9,	--ATTACKER
		Tiger,	--DEFENDER
		"Tank1InPosition" );
	else
		Tank1InPosition();
	end
end

function showDialogue12()
	ScriptManager:displaySubtitle( Text.T_MISSION_TUTORIAL_2_12 );
	ScriptManager:sleep( 5, "HideDialogue" );
end

function Tank1InPosition()
	local Tiger = ScriptManager:getValue( "unitTankTiger" );
	if Unit:getUnitById( 10 ) ~= nil then
		ScriptManager:attackUnit( 
		10,	--ATTACKER
		Tiger,	--DEFENDER
		"Tank2Attacked" );
	else
		Tank2Attacked();
	end
end

function Tank2Attacked()
	local Tiger = ScriptManager:getValue( "unitTankTiger" );
	if Unit:getUnitById( 11 ) ~= nil then
		ScriptManager:attackUnit( 
		11,	--ATTACKER
		Tiger,	--DEFENDER
		"Tank3Attacked" );
	else
		Tank3Attacked();
	end
end

function Tank3Attacked() --move mortars
	ScriptManager:moveUnit( 4, 6, 5, "MortarInPosition" );
end

function MortarInPosition()
	local Tiger = ScriptManager:getValue( "unitTankTiger" );
	ScriptManager:attackUnit( 
	4,	--ATTACKER
	Tiger,	--DEFENDER
	"MortarAttacked" );
end

function MortarAttacked()
	local defender = Unit:getUnitById( 4 );
	defender:setHealth( 80 );
	ScriptManager:fortifyUnit( 10 );
	ScriptManager:sleep( 0, "MoveCam_1" );
end

function MoveCam_1()
	ScriptManager:moveCamera( 5, 2, 0.8, 3, "PlayerReinforcements" );
	ScriptManager:sleep( 2, "ShowDialogue13" );
	ScriptManager:sleep( 6.5, "HideDialogue" );
end

function ShowDialogue13()
	ScriptManager:displaySubtitle( Text.T_MISSION_TUTORIAL_2_13 );
end

function PlayerReinforcements()
	local unit = ScriptManager:spawnUnit( 
	0, 0, 	--PLAYER, UNITID
	4, 1,	--X, Y
	30, 5 );	--AI
	ScriptManager:storeValue( "Infantry_1", unit:getId() );
	ScriptManager:moveUnit( unit:getId(),
	5, 4,	--X, Y
	"PlayerReinforcements_1" );
end

function PlayerReinforcements_1()
	local unit = ScriptManager:spawnUnit( 
	0, 0, 	--PLAYER, UNITID
	4, 1,	--X, Y
	30, 5 );	--AI
	ScriptManager:storeValue( "Infantry_2", unit:getId() );
	ScriptManager:moveUnit( unit:getId(),
	5, 3,	--X, Y
	"PlayerReinforcements_2" );
end

function PlayerReinforcements_2()
	local unit = ScriptManager:spawnUnit( 
	0, 1, 	--PLAYER, UNITID
	4, 1,	--X, Y
	30, 5 );	--AI
	ScriptManager:storeValue( "Infantry_3", unit:getId() );
	ScriptManager:moveUnit( unit:getId(),
	5, 2,	--X, Y
	"PlayerReinforcements_3" );
end

function PlayerReinforcements_3()
	local unit = ScriptManager:spawnUnit( 
	0, 1, 	--PLAYER, UNITID
	4, 1,	--X, Y
	30, 5 );	--AI
	ScriptManager:storeValue( "Infantry_4", unit:getId() );
	ScriptManager:moveUnit( unit:getId(),
	5, 1,	--X, Y
	"endFirstEnemyTurnWait" );
end

function endFirstEnemyTurnWait()
	ScriptManager:moveCamera( 5, 3, 0.75, 2, "endFirstEnemyTurn" );
end

function endFirstEnemyTurn()
	ScriptManager:continueAfterCutscene( "endFirstEnemyTurn_1" );
end

function endFirstEnemyTurn_1()
	Unit:getUnitById( ScriptManager:getValue( "unitTankTiger" ) ):setAttacksPerTurn( 1 );
	ScriptManager:stopCutscene();
	ScriptManager:disableEndTurnButton();
	ScriptManager:stopHighlightingEndTurnButton();
	cleanUp( true, true, true, true );
	ScriptManager:displaySubtitle( Text.T_MISSION_TUTORIAL_2_14 );
	ScriptManager:addOnUnitSelectedTrigger( 0, -1, ScriptManager:getValue( "Infantry_1" ), "infantrySelected", "infantrySelected" );
	ScriptManager:addOnUnitSelectedTrigger( 0, -1, ScriptManager:getValue( "Infantry_2" ), "infantrySelected", "infantrySelected" );
	ScriptManager:addOnUnitSelectedTrigger( 0, -1, ScriptManager:getValue( "Infantry_3" ), "antitankSelected", "antitankSelected" );
	ScriptManager:addOnUnitSelectedTrigger( 0, -1, ScriptManager:getValue( "Infantry_4" ), "antitankSelected", "antitankSelected" );
	ScriptManager:addOnUnitSelectedTrigger( 0, -1, ScriptManager:getValue( "unitTankTiger" ), "tigerSelected", "tigerSelected" );
	ScriptManager:addOnUnitHasAttackedTrigger( 0, -1, -1, "enemyWasAttacked", "enemyWasAttacked" );
	showMarkerAtUnits();
end

function cleanUp( markers, triggers, tiles, subtitles )
	if markers then
		ScriptManager:removeAllMarkers();
	end
	
	if triggers then
		ScriptManager:removeAllTriggers();
	end
	
	if tiles then
		ScriptManager:disableAllTilesExcept( 0,0 );
	end
	
	if subtitles then
		ScriptManager:dismissSubtitle();
	end
end

function areEnemysKilled()
	return areInfantrysKilled() and areTanksKilled();
end

function areInfantrysKilled()
	local morter = Unit:getUnitById( 4 );
	if morter == nil then
		return true;
	end

	return false;
end

function areTanksKilled()
	local tank1 = Unit:getUnitById(  9 );
	local tank2 = Unit:getUnitById( 10 );
	local tank3 = Unit:getUnitById( 11 );

	if tank1 == nil and tank2 == nil and tank3 == nil then
		return true;
	end

	return false;
end

function showMarkersAtEnemyInfantry()
	local infantry = Unit:getUnitById( 4 );
	
	if infantry ~= nil then
		ScriptManager:showMarkerAt( infantry:getX(), infantry:getY() );
		ScriptManager:enableAdditionalTile( infantry:getX(), infantry:getY() );
		
	cleanupController();
		if ScriptManager:isControllerConnected() then
			ScriptManager:addOnControllerSelectedTileTrigger( infantry:getX(), infantry:getY(), "cursorOverUnit", "cursorOverUnit",  false );
		end
	end
end

function showMarkersAtEnemyTanks()
	for i = 9, 11 do
		local tank = Unit:getUnitById( i );
		
		if tank ~= nil then
			ScriptManager:showMarkerAt( tank:getX(), tank:getY() );
			ScriptManager:enableAdditionalTile( tank:getX(), tank:getY() );
			
			ScriptManager:hideControllerButton();
			if ScriptManager:isControllerConnected() then
				ScriptManager:addOnControllerSelectedTileTrigger( tank:getX(), tank:getY(), "cursorOverUnit", "cursorOverUnit",  false );
			end
		end
	end
end

function showMarkerAtUnits()
	cleanUp( true, false, true, true )
	local rifle1 = Unit:getUnitById( ScriptManager:getValue( "Infantry_1" ) );
	local rifle2 = Unit:getUnitById( ScriptManager:getValue( "Infantry_2" ) );
	local mortar1 = Unit:getUnitById( ScriptManager:getValue( "Infantry_3" ) );
	local mortar2 = Unit:getUnitById( ScriptManager:getValue( "Infantry_4" ) );
	local tiger = Unit:getUnitById( ScriptManager:getValue( "unitTankTiger" ) );

	cleanupController();
	if ScriptManager:isControllerConnected() then
		ScriptManager:setControllerCursorPosition( 6, 3 );
		ScriptManager:setControllerCursorEnabled( true );
	end

	if rifle1:hasAttacksLeft() and areInfantrysKilled() == false then
		ScriptManager:showMarkerAt( rifle1:getX(), rifle1:getY() );
		ScriptManager:enableAdditionalTile( rifle1:getX(), rifle1:getY() );
		
		if ScriptManager:isControllerConnected() then
			ScriptManager:addOnControllerSelectedTileTrigger( rifle1:getX(), rifle1:getY(), "cursorOverUnitSinglePress", "cursorOverUnit", false );
		end
	end
	if rifle2:hasAttacksLeft() and areInfantrysKilled() == false then
		ScriptManager:showMarkerAt( rifle2:getX(), rifle2:getY() );
		ScriptManager:enableAdditionalTile( rifle2:getX(), rifle2:getY() );
		
		if ScriptManager:isControllerConnected() then
			ScriptManager:addOnControllerSelectedTileTrigger( rifle2:getX(), rifle2:getY(), "cursorOverUnitSinglePress", "cursorOverUnit", false );
		end
	end
	if mortar1:hasAttacksLeft() and areTanksKilled() == false then
		ScriptManager:showMarkerAt( mortar1:getX(), mortar1:getY() );
		ScriptManager:enableAdditionalTile( mortar1:getX(), mortar1:getY() );
		
		if ScriptManager:isControllerConnected() then
			ScriptManager:addOnControllerSelectedTileTrigger( mortar1:getX(), mortar1:getY(), "cursorOverUnitSinglePress", "cursorOverUnit", false );
		end
	end
	if mortar2:hasAttacksLeft() and areTanksKilled() == false then
		ScriptManager:showMarkerAt( mortar2:getX(), mortar2:getY() );
		ScriptManager:enableAdditionalTile( mortar2:getX(), mortar2:getY() );
		
		if ScriptManager:isControllerConnected() then
			ScriptManager:addOnControllerSelectedTileTrigger( mortar2:getX(), mortar2:getY(), "cursorOverUnitSinglePress", "cursorOverUnit", false );
		end
	end
	if tiger:hasAttacksLeft() and areTanksKilled() == false then
		ScriptManager:showMarkerAt( tiger:getX(), tiger:getY() );
		ScriptManager:enableAdditionalTile( tiger:getX(), tiger:getY() );
		
		if ScriptManager:isControllerConnected() then
			ScriptManager:addOnControllerSelectedTileTrigger( tiger:getX(), tiger:getY(), "cursorOverUnitSinglePress", "cursorOverUnit", false );
		end
	end
end

function infantrySelected()
	cleanUp( true, false, true, true );
	ScriptManager:displaySubtitle( Text.T_MISSION_TUTORIAL_2_15 );
	
	cleanupController();
	if ScriptManager:isControllerConnected() then
		ScriptManager:setControllerCursorEnabled( true );
	end
	
	showMarkersAtEnemyInfantry()
end

function antitankSelected()
	cleanUp( true, false, true, true );
	ScriptManager:displaySubtitle( Text.T_MISSION_TUTORIAL_2_16 );
	
	cleanupController();
	if ScriptManager:isControllerConnected() then
		--cleanupController();
		ScriptManager:setControllerCursorEnabled( true );
	end
	
	showMarkersAtEnemyTanks();
end

function tigerSelected()
	cleanUp( true, false, true, true );
	
	cleanupController();
	if ScriptManager:isControllerConnected() then
		ScriptManager:setControllerCursorEnabled( true );
	end
	
	showMarkersAtEnemyTanks();
end

function enemyWasAttacked( attacker, defender )
	if areEnemysKilled() then
		local wasTiger = 0;
		if attacker == Unit:getUnitById( ScriptManager:getValue( "unitTankTiger" ) ) then 
			wasTiger = 1
		end
		
		ScriptManager:storeValue( "tigerStillSelected", wasTiger );
		captureBase();
		
	else
		showMarkerAtUnits();
	end
end

function captureBase()
	cleanUp( true, false, true, true );
	ScriptManager:displaySubtitle( Text.T_MISSION_TUTORIAL_2_17 );
	ScriptManager:moveCamera( 7, 2, 0.9, 2, "baseUnprotected" );
end

function baseUnprotected()
	if( ScriptManager:getValue( "tigerStillSelected" ) == 0 ) then
		ScriptManager:sleep( 2, "CaptureOrder" );
		
	else
		ScriptManager:sleep( 1.5, "CaptureOrderSelect" );
	end
end

function CaptureOrder()
	checkControllerAndHideButton();
	
	ScriptManager:dismissSubtitle();
	ScriptManager:displaySubtitle( Text.T_MISSION_TUTORIAL_2_18 );
	local tiger = Unit:getUnitById( ScriptManager:getValue( "unitTankTiger" ) );
	ScriptManager:moveCamera( tiger:getX(), tiger:getY(), 0.8, 1.5, "" );
	ScriptManager:disableAllTilesExcept( tiger:getX(), tiger:getY() );
	ScriptManager:showMarkerAt( tiger:getX(), tiger:getY() );
	
	cleanupController();
	if ScriptManager:isControllerConnected() then
		ScriptManager:setControllerCursorEnabled( true );
		ScriptManager:addOnControllerSelectedTileTrigger( tiger:getX(), tiger:getY(), "cursorOverUnitSinglePress", "cursorOverUnit", false );
				
	else
		ScriptManager:showFingerAtTile( tiger:getX(), tiger:getY(), 1 );
	end
	
	ScriptManager:addOnUnitSelectedTrigger( 0, -1, tiger:getId(), "CaptureOrderSelect" );
end

function CaptureOrderSelect()
	cleanUp( true, true, true, true )
	ScriptManager:displaySubtitle( Text.T_MISSION_TUTORIAL_2_19 )
	Unit:getUnitById( 0 ):setIsAttackable( true );
	ScriptManager:disableAllTilesExcept( 7, 2 );
	
	ScriptManager:hideControllerButton();
	if ScriptManager:isControllerConnected() then
		ScriptManager:setControllerCursorEnabled( true );
		ScriptManager:addOnControllerSelectedTileTrigger( 7, 2, "cursorOverUnitSinglePress", "cursorOverUnit", false );
		
	else
		ScriptManager:moveFingerToTile( 7, 2 );
	end

	ScriptManager:showMarkerAt( 7, 2 );
	ScriptManager:moveCamera( 7, 2, 0.8, 1.5, "" );
	ScriptManager:addOnUnitSelectedTrigger( -1, -1, 0, "CaptureOrderConfirm" );
end

function CaptureOrderConfirm()
	ScriptManager:hideControllerButton();
	
	cleanUp( true, true, false, true )
	ScriptManager:hideFinger();
	ScriptManager:addOnUnitCapturedTrigger( 0, -1, 0, "CaptureComplete", "CaptureComplete" );
end

function CaptureComplete()
	ScriptManager:disableAllTilesExcept( 0, 0 );
	cleanUp( true, true, true, true );
	ScriptManager:sleep( 1, "showDialogue21" );
end

function showDialogue21()
	ScriptManager:displaySubtitle( Text.T_MISSION_TUTORIAL_2_20 );
	ScriptManager:sleep( 2, "HideDialogue" );
	ScriptManager:sleep( 3, "DemandReinforcements" );
end

function DemandReinforcements()
	cleanUp( true, true, true, true );
	ScriptManager:displaySubtitle( Text.T_MISSION_TUTORIAL_2_21 );
	ScriptManager:disableAllTilesExcept( 7, 2 );
	
	cleanupController();
	if ScriptManager:isControllerConnected() then
		ScriptManager:setControllerCursorEnabled( true );-- TODO: #31382
		cursorOverUnitSinglePress( 7, 2 );
	
	else
		ScriptManager:showMarkerAt( 7, 2 )
		ScriptManager:showFingerAtTile( 7, 2, 1 );
	end
	
	ScriptManager:addOnUnitSelectedTrigger( -1, -1, 0, "baseSelected", "baseSelected" );
end

function baseSelected()
	cleanUp( true, true, true, true );
	ScriptManager:hideFinger();
	
	cleanupController();
	if ScriptManager:isControllerConnected() then
		showControllerButtonGfx( ControllerButton.FACE_DOWN, 1, 7, 2 );
	end
	ScriptManager:displaySubtitle( Text.T_MISSION_TUTORIAL_2_22 );
		
	ScriptManager:enableInteractionButton( InteractionButton.REINFORCEMENT_1 );
	ScriptManager:startHighlightingInteractionButton( InteractionButton.REINFORCEMENT_1 );
	ScriptManager:addOnInteractionButtonSelectedTrigger( 0, -1, -1, InteractionButton.REINFORCEMENT_1, "unitPurchaseSelected", "unitPurchaseSelected" )
end

function unitPurchaseSelected()
	ScriptManager:disableAllInteractionButtons();
	ScriptManager:disableAllTilesExcept( 7, 3 );
	ScriptManager:showMarkerAt( 7, 3 );
	
	cleanupController();
	if ScriptManager:isControllerConnected() then
		ScriptManager:addOnControllerSelectedTileTrigger( 7, 3, "cursorOverUnit", "cursorOverUnit", false );
		ScriptManager:setControllerCursorEnabled( true );
	
	else
		ScriptManager:showFingerAtTile( 7, 3, 1 );
	end

	ScriptManager:addOnUnitSelectedTrigger( -1, -1, -1, "infantryPurchased", "infantryPurchased" );
end

function infantryPurchased( unit )
	ScriptManager:storeValue( "newInfantry", unit:getId() );
	cleanUp( true, true, true, true );
	ScriptManager:displaySubtitle( Text.T_MISSION_TUTORIAL_2_23 );
		ScriptManager:sleep( 1, "infantryShouldMove" );
	end

function infantryShouldMove()
	cleanupController();
	if ScriptManager:isControllerConnected() then
		ScriptManager:setControllerCursorEnabled( true );
		ScriptManager:addOnControllerSelectedTileTrigger( 8, 5, "cursorOverUnit", "cursorOverUnit", false );
	
	else
	ScriptManager:showFingerAtTile( 8, 2, 1 );
	ScriptManager:moveFingerToTile( 8, 5 );
	ScriptManager:loopFingerClicks( 1.5 );
	end
	
	ScriptManager:showMarkerAt( 8, 5 );
	ScriptManager:disableAllTilesExcept( 8, 5 );
	ScriptManager:moveCamera( 7, 4, 0.8, 2, "" );
	ScriptManager:addOnTileDeselectedTrigger( 8, 5, -1, "infantryMovmentStarted", "infantryMovmentStarted" );
end

function infantryMovmentStarted()
	ScriptManager:hideFinger();
	cleanUp( true, true, true, true );
	ScriptManager:addOnEnterTileTrigger( 8, 5, -1, -1, ScriptManager:getValue( "newInfantry" ), true, "infantryNextToBunker", "infantryNextToBunker" )
end

function infantryNextToBunker()
	checkControllerAndHideButton();
	
	cleanUp( true, true, true, true );
	ScriptManager:moveCamera( 9, 5, 0.8, 3, "explainOccupying" );
	ScriptManager:displaySubtitle( Text.T_MISSION_TUTORIAL_2_24 );
	Unit:getUnitById( 14 ):setIsAttackable( false );
end

function explainOccupying()
	ScriptManager:dismissSubtitle();
	ScriptManager:displaySubtitle( Text.T_MISSION_TUTORIAL_2_25 );
	ScriptManager:moveCamera( 9, 5, 1.2, 3, "" );
		ScriptManager:sleep( 3.5, "occupyBunkerOrder" );
	end

function occupyBunkerOrder()
	cleanUp( true, true, true, true );
	ScriptManager:moveCamera( 8, 5, 0.8, 1, "" );
	ScriptManager:displaySubtitle( Text.T_MISSION_TUTORIAL_2_26 );
	ScriptManager:disableAllTilesExcept( 9, 5 );
	ScriptManager:showMarkerAt( 9, 5 );
	
	cleanupController();
	if ScriptManager:isControllerConnected() then
		ScriptManager:setControllerCursorEnabled( true );
		ScriptManager:addOnControllerSelectedTileTrigger( 9, 5, "cursorOverUnit", "cursorOverUnit", false );
	
	else
	ScriptManager:showFingerAtTile( 9, 5, 0 );
	ScriptManager:loopFingerClicks( 1 );
end

	ScriptManager:addOnUnitOccupiedTrigger( -1, -1, -1, "bunkerOccupied", "bunkerOccupied" );
end

function bunkerOccupied()
	Unit:getUnitById( 14 ):setIsAttackable( true );
	Unit:getUnitById( 15 ):setIsAttackable( false );
	cleanUp( true, true, true, true );
	
	cleanupController();
	ScriptManager:hideFinger();
	if ScriptManager:isControllerConnected() then
		ScriptManager:setControllerCursorEnabled( true );
		ScriptManager:addOnControllerSelectedTileTrigger( 10, 7, "cursorOverUnit", "cursorOverUnit", false );
		ScriptManager:addOnControllerSelectedTileTrigger( 11, 6, "cursorOverUnit", "cursorOverUnit", false ); 
		ScriptManager:addOnControllerSelectedTileTrigger( 11, 5, "cursorOverUnit", "cursorOverUnit", false ); 
	end
	
	ScriptManager:displaySubtitle( Text.T_MISSION_TUTORIAL_2_27 );
	enableAndMarkEnemyInfantries( 12, 14 );
	ScriptManager:addOnUnitHasAttackedTrigger( -1, -1, -1, "bunkerAttacked", "bunkerAttacked" );
	ScriptManager:addOnUnitSelectedTrigger( -1, -1, -1, "bunkerAttackStarted", "bunkerAttackStarted" );
end

function enableAndMarkEnemyInfantries( startId, endId )
	for i = startId, endId do
		local unit = Unit:getUnitById( i );
		
		if unit ~= nil then
			ScriptManager:showMarkerAt( unit:getX(), unit:getY() );
			ScriptManager:enableAdditionalTile( unit:getX(), unit:getY() );
			
			ScriptManager:hideControllerButton();
			if ScriptManager:isControllerConnected() then
				ScriptManager:addOnControllerSelectedTileTrigger( unit:getX(), unit:getY(), "cursorOverUnit", "cursorOverUnit", false );
			end
		end
	end
end

function showMarkerAtInfantrys()
	cleanUp( true, false, true, true );
	ScriptManager:displaySubtitle( Text.T_MISSION_TUTORIAL_2_29 );
	ScriptManager:moveCamera( 6, 3, 0.8, 1, "" );
	local rifle1 = Unit:getUnitById( ScriptManager:getValue( "Infantry_1" ) );
	local rifle2 = Unit:getUnitById( ScriptManager:getValue( "Infantry_2" ) );
	local mortar1 = Unit:getUnitById( ScriptManager:getValue( "Infantry_3" ) );
	local mortar2 = Unit:getUnitById( ScriptManager:getValue( "Infantry_4" ) );
	local allUnitsDone = true;
	
	cleanupController();
	if ScriptManager:isControllerConnected() then
		ScriptManager:setControllerCursorEnabled( true );
	end
	
	if rifle1:getActionPoints() > 0 then
		ScriptManager:showMarkerAt( rifle1:getX(), rifle1:getY() );
		ScriptManager:enableAdditionalTile( rifle1:getX(), rifle1:getY() );
		allUnitsDone = false;
		
		if ScriptManager:isControllerConnected() then
			ScriptManager:addOnControllerSelectedTileTrigger( rifle1:getX(), rifle1:getY(), "cursorOverUnitSinglePress", "cursorOverUnit", false );
		end
	end
	if rifle2:getActionPoints() > 0 then
		ScriptManager:showMarkerAt( rifle2:getX(), rifle2:getY() );
		ScriptManager:enableAdditionalTile( rifle2:getX(), rifle2:getY() );
		allUnitsDone = false;
		
		if ScriptManager:isControllerConnected() then
			ScriptManager:addOnControllerSelectedTileTrigger( rifle2:getX(), rifle2:getY(), "cursorOverUnitSinglePress", "cursorOverUnit", false );
		end
	end
	if mortar1:getActionPoints() > 0 then
		ScriptManager:showMarkerAt( mortar1:getX(), mortar1:getY() );
		ScriptManager:enableAdditionalTile( mortar1:getX(), mortar1:getY() );
		allUnitsDone = false;
		
		if ScriptManager:isControllerConnected() then
			ScriptManager:addOnControllerSelectedTileTrigger( mortar1:getX(), mortar1:getY(), "cursorOverUnitSinglePress", "cursorOverUnit", false );
		end
	end
	if mortar2:getActionPoints() > 0 then
		ScriptManager:showMarkerAt( mortar2:getX(), mortar2:getY() );
		ScriptManager:enableAdditionalTile( mortar2:getX(), mortar2:getY() );
		allUnitsDone = false;
		
		if ScriptManager:isControllerConnected() then
			ScriptManager:addOnControllerSelectedTileTrigger( mortar2:getX(), mortar2:getY(), "cursorOverUnitSinglePress", "cursorOverUnit", false );
		end
	end
	
	if allUnitsDone then
		infantriesInPosition();

		if ScriptManager:isControllerConnected() then
			ScriptManager:setControllerCursorEnabled( false );
		end		
	end
end

function bunkerAttackStarted()
	ScriptManager:dismissSubtitle();
end

function bunkerAttacked()
	checkControllerAndHideButton();
	
	cleanUp( true, true, true, true );
	ScriptManager:displaySubtitle( Text.T_MISSION_TUTORIAL_2_28 );
	ScriptManager:sleep( 4.5, "HideDialogue" );
	ScriptManager:sleep( 4, "moveBackToBase" );
end

function moveBackToBase()
	ScriptManager:moveCamera( 6, 3, 0.8, 1, "cameraAtBase" );
end

function cameraAtBase()
	cleanUp( true, true, true, true );
	
	ScriptManager:addOnUnitSelectedTrigger( -1, -1, -1, "baseInfantrySelected", "baseInfantrySelected" );
	ScriptManager:addOnTileDeselectedTrigger( -1, -1, -1, "removeMarkers", "removeMarkers" );
	showMarkerAtInfantrys();
end

function baseInfantrySelected( unit )
	cleanUp( true, false, true, true );
	local targetX = unit:getX()+unit:getActionPoints()-1+math.fmod( unit:getY(), 2 );
	local targetY = unit:getY()+1;
	ScriptManager:storeValue( "targetX", targetX );
	ScriptManager:storeValue( "targetY", targetY );
	
	cleanupController();
	if ScriptManager:isControllerConnected() then
		ScriptManager:addOnControllerSelectedTileTrigger( targetX, targetY, "cursorOverUnit", "cursorOverUnit", false );
		ScriptManager:setControllerCursorEnabled( true );
	end
	
	ScriptManager:addOnEnterTileTrigger( targetX, targetY, -1, -1, -1, true, "targetTileEntered", "targetTileEntered" );
	ScriptManager:showMarkerAt( targetX, targetY );
	ScriptManager:enableAdditionalTile( targetX, targetY );
end

function removeMarkers()
	ScriptManager:removeAllMarkers();
end

function targetTileEntered()
	cleanUp( true, false, true, true );
	ScriptManager:removeTrigger( "targetTileEntered" );
	
	cleanupController();
	if ScriptManager:isControllerConnected() then
		ScriptManager:displaySubtitle( Text.T_TUTORIAL_CONTROLLER_PRESS_BUTTON );
		
		local x = ScriptManager:getValue( "targetX" );
		local y = ScriptManager:getValue( "targetY" );
		-- show it with other function cause otherwise next button display won't disappear
		ScriptManager:showControllerButtonAtTile( ControllerButton.FACE_LEFT, x, y, 0.2 );
		ScriptManager:addOnControllerButtonReleasedTrigger( ControllerButton.FACE_LEFT, "targetTileEnteredController", "targetTileEnteredController" );
	
	else
		ScriptManager:displaySubtitle( Text.T_MISSION_TUTORIAL_2_30 );
	end
	
	ScriptManager:enableInteractionButton( InteractionButton.FORTIFY );
	ScriptManager:startHighlightingInteractionButton( InteractionButton.FORTIFY );
	ScriptManager:addOnInteractionButtonSelectedTrigger( -1, -1, -1, InteractionButton.FORTIFY, "unitFortified", "unitFortified" );
end

function targetTileEnteredController()
	RemoveTrigger( "targetTileEnteredController" );
	ScriptManager:dismissSubtitle();
	ScriptManager:displaySubtitle( Text.T_MISSION_TUTORIAL_2_30 );	
	ScriptManager:hideControllerButton();
	
	local x = ScriptManager:getValue( "targetX" );
	local y = ScriptManager:getValue( "targetY" );
	showControllerButtonGfx( ControllerButton.FACE_DOWN, 1, x, y );
end

function unitFortified()
	cleanUp( true, false, true, true )
	ScriptManager:stopHighlightingInteractionButton( InteractionButton.FORTIFY );
	ScriptManager:disableInteractionButton( InteractionButton.FORTIFY );
	ScriptManager:removeTrigger( "unitFortified" );
	showMarkerAtInfantrys();
end

function infantriesInPosition()
	cleanUp( true, true, true, true );
	ScriptManager:displaySubtitle( Text.T_MISSION_TUTORIAL_2_31 );
	ScriptManager:sleep( 3, "tigerStillDamaged" );
end

function tigerStillDamaged()
	cleanUp( true, true, true, true );
	ScriptManager:displaySubtitle( Text.T_MISSION_TUTORIAL_2_32 );
	ScriptManager:moveCamera( 6, 3, 1.2, 3, "explainSupply" );
end

function explainSupply()
	cleanUp( true, true, true, true );
	ScriptManager:displaySubtitle( Text.T_MISSION_TUTORIAL_2_33 );
	ScriptManager:moveCamera( 7, 2, 0.8, 2, "" );
		ScriptManager:sleep( 4.5, "askToSelectBase" );
	end

function askToSelectBase()
	cleanUp( true, true, true, true );
	ScriptManager:displaySubtitle( Text.T_MISSION_TUTORIAL_2_34 );
	ScriptManager:disableAllTilesExcept( 7, 2 );
	ScriptManager:showMarkerAt( 7, 2 );
	
	cleanupController();
	if ScriptManager:isControllerConnected() then
		ScriptManager:addOnControllerSelectedTileTrigger( 7, 2, "cursorOverUnitSinglePress", "cursorOverUnit", false );
		ScriptManager:setControllerCursorEnabled( true );
		
	else
	ScriptManager:showFingerAtTile( 7, 2, 0 );
	ScriptManager:loopFingerClicks( 1 );
end

	ScriptManager:addOnUnitSelectedTrigger( -1, -1, -1, "supplyBaseSelected", "supplyBaseSelected" );
end

function supplyBaseSelected()
	Unit:getUnitById( ScriptManager:getValue( "Infantry_4" ) ):setHealth( 100 );
	Unit:getUnitById( ScriptManager:getValue( "Infantry_3" ) ):setHealth( 100 );
	Unit:getUnitById( ScriptManager:getValue( "Infantry_4" ) ):setAmmo( 100 );
	Unit:getUnitById( ScriptManager:getValue( "Infantry_3" ) ):setAmmo( 100 );
	cleanUp( true, true, true, true )
	ScriptManager:hideFinger()
	
	cleanupController();
	ScriptManager:displaySubtitle( Text.T_MISSION_TUTORIAL_2_35 );-- TODO
	if ScriptManager:isControllerConnected() then
		showControllerButtonGfx( ControllerButton.FACE_DOWN, 1, 7, 2 );
	end
	
	ScriptManager:disableAllInteractionButtons();
	ScriptManager:enableInteractionButton( InteractionButton.SUPPLY );
	ScriptManager:startHighlightingInteractionButton( InteractionButton.SUPPLY );
	ScriptManager:addOnInteractionButtonSelectedTrigger( -1, -1, -1, InteractionButton.SUPPLY, "supplyModeEnabled", "supplyModeEnabled" );
	ScriptManager:addOnUnitSuppliedTrigger( -1, -1, -1, "firstTimeSupplied", "firstTimeSupplied" );
end

function supplyModeEnabled()
	ScriptManager:disableAllInteractionButtons();
	ScriptManager:removeTrigger( "supplyModeEnabled" );
	ScriptManager:addOnInteractionButtonDeselectedTrigger( -1, -1, -1, InteractionButton.SUPPLY, "supplyModeDisabled", "supplyModeDisabled" );
	ScriptManager:disableAllTilesExcept( 6, 3 );
	ScriptManager:showMarkerAt( 6, 3 );
	
	cleanupController();
	if ScriptManager:isControllerConnected() then
		ScriptManager:setControllerCursorEnabled( true );
		ScriptManager:addOnControllerSelectedTileTrigger( 6, 3, "cursorOverTiger", "cursorOverUnit", false );
		
	else
		ScriptManager:showFingerAtTile( 6, 3, 0 );
		ScriptManager:loopFingerClicks( 1 );
	end
end

function cursorOverTiger( tileX, tileY )
	cursorOver();
	ScriptManager:showControllerButtonAtTile( ControllerButton.FACE_DOWN, 6, 3, 0.2 );
end

function supplyModeDisabled()
	cleanUp( true, false, true, false );
	ScriptManager:removeTrigger( "supplyModeDisabled" );
		ScriptManager:addOnInteractionButtonSelectedTrigger( -1, -1, -1, InteractionButton.SUPPLY, "supplyModeEnabled", "supplyModeEnabled" );
	end

function firstTimeSupplied()
	ScriptManager:removeTrigger( "firstTimeSupplied" );
	ScriptManager:hideFinger();
	ScriptManager:dismissSubtitle();
	ScriptManager:displaySubtitle( Text.T_MISSION_TUTORIAL_2_36 );
	ScriptManager:addOnUnitSuppliedTrigger( -1, -1, -1, "checkForFullHealth", "checkForFullHealth" );
end

function checkForFullHealth( base, tiger )
	if tiger:getHealth() >= 1200 then
		cleanUp( true, true, true, true );
		ScriptManager:hideFinger();
		ScriptManager:displaySubtitle( Text.T_MISSION_TUTORIAL_2_37 );
		ScriptManager:stopHighlightingInteractionButton( InteractionButton.SUPPLY );
		ScriptManager:disableAllInteractionButtons();
		ScriptManager:enableEndTurnButton();
		
		ScriptManager:hideControllerButton();
		if ScriptManager:isControllerConnected() then
			showControllerButtonGfx( ControllerButton.FACE_UP, 1, 6, 3 );
			
		end

		ScriptManager:startHighlightingEndTurnButton();
		
		ScriptManager:moveCamera( 6, 3, 0.7, 2, "" )
		ScriptManager:addOnTurnStartTrigger( 1, -1, 1, "endEnemyTurnImmediatly" );
		ScriptManager:addOnTurnStartTrigger( 0, -1, 2, "StartOwnTurn2" );
	end
end

function StartOwnTurn2()
	cleanUp( true, true, true, true );
	ScriptManager:startCutscene();
	ScriptManager:moveCamera( 9, 7, 0.8, 0, "" );
	ScriptManager:sleep( 1, "moveMortar" );
end

function moveMortar()
	ScriptManager:displaySubtitle( Text.T_MISSION_TUTORIAL_2_38 );
	ScriptManager:moveUnit( 15, 9, 7, "" );
	ScriptManager:sleep( 2, "finishedMoving" );
end

function finishedMoving()
	ScriptManager:dismissSubtitle()
	ScriptManager:displaySubtitle( Text.T_MISSION_TUTORIAL_2_39 );
	ScriptManager:moveCamera( 9, 6, 1.0, 7, "mortarAttack" );
end

function mortarAttack()
	ScriptManager:dismissSubtitle();
	ScriptManager:attackUnit( 15, ScriptManager:getValue( "Infantry_1" ), "mortarAttackFinished" );
end

function mortarAttackFinished()
	--Unit:getUnitById( ScriptManager:getValue( "Infantry_1" ) ):setHealth( 50 );
	ScriptManager:sleep( 1, "startBunkerAttack" );
end

function startBunkerAttack()
	ScriptManager:attackUnit( 8, 15, "delayBeforeNextUnit" );
	ScriptManager:dismissSubtitle();
	ScriptManager:displaySubtitle( Text.T_MISSION_TUTORIAL_2_41 );
	ScriptManager:storeValue( "nextUnit", 14 );
end

function moveEnemyInfantry()
	local enemyID = ScriptManager:getValue( "nextUnit" );
	local dy = 0;
	
	if enemyID == 14 then
		dy = 1
	end
	
	if enemyID == 11 then
		replayFinished();
		return;
	end
	
	local enemy = Unit:getUnitById( enemyID );
	
	if enemy ~= nil then
		ScriptManager:moveUnit( enemyID, enemy:getX()-1, enemy:getY()-dy, "attackWithEnemyInfantry" );
		
	else
		enemyID = enemyID - 1;
		ScriptManager:storeValue( "nextUnit", enemyID );
		moveEnemyInfantry();
	end
end

function attackWithEnemyInfantry()
	local enemyID = ScriptManager:getValue( "nextUnit" );
	local enemy = Unit:getUnitById( enemyID );
	
	if enemy ~= nil and enemy:hasAttacksLeft() then
		ScriptManager:storeValue( "nextUnit", enemyID-1 );
		local Infantry = Unit:getUnitById( ScriptManager:getValue( "Infantry_1" ) );
		local health = Infantry:getHealth();
		
		if health <= 50 then
			Infantry = Unit:getUnitById( ScriptManager:getValue( "Infantry_2" ) );
		end
		
		ScriptManager:attackUnit( enemyID, Infantry:getId(), "delayBeforeNextUnit" );
	end
	
end

function delayBeforeNextUnit()
	ScriptManager:sleep( 0.5, "moveEnemyInfantry" );
end

function replayFinished()
	cleanUp( true, true, true, true );
	ScriptManager:stopCutscene();
	ScriptManager:continueAfterCutscene( "startThirdPlayerTurn" );
end

function startThirdPlayerTurn()
	ScriptManager:stopHighlightingEndTurnButton();
	ScriptManager:disableEndTurnButton();
	cleanUp( true, true, true, true );
	ScriptManager:displaySubtitle( Text.T_MISSION_TUTORIAL_2_42 );
	ScriptManager:sleep( 4.5, "markBunker" );
	fillAttacksForAllUnits();
end

function markBunker()
	ScriptManager:dismissSubtitle();
	ScriptManager:displaySubtitle( Text.T_MISSION_TUTORIAL_2_43 );
	ScriptManager:sleep( 4, "HideDialogue" );
	local unit = Unit:getUnitById( 8 );
	ScriptManager:showMarkerAt( unit:getX(), unit:getY() );
	ScriptManager:disableAllTilesExcept( unit:getX(), unit:getY() );
	ScriptManager:addOnUnitSelectedTrigger( 0, -1, -1, "unitSelected", "unitSelected" );
	
	if ScriptManager:isControllerConnected() then
		ScriptManager:setControllerCursorEnabled( true );
		ScriptManager:addOnControllerSelectedTileTrigger( unit:getX(), unit:getY(), "cursorOverUnitSinglePress", "cursorOverUnit", false );
	end
end

function unitSelected( unit )
	cleanUp( true, true, true, false );
	if unitAtPosition( unit:getId() ) == false then
		markTargetPosition( unit:getId() );
		
		if ScriptManager:isControllerConnected() then
			ScriptManager:setControllerCursorEnabled( true );
		end
		
	else
		ScriptManager:moveCamera( 9, 5, 0.8, 1, "" );
		enableAndMarkEnemyInfantries( 12, 14 );
		ScriptManager:addOnUnitHasAttackedTrigger( -1, -1, -1, "attackHasFinished", "attackHasFinished" );
		
		if ScriptManager:isControllerConnected() then
			ScriptManager:setControllerCursorEnabled( true );
		
			for i = 1, 3 do
				local enemy = Unit:getUnitById( 11 + i ); -- = minimum 12
				
				if enemy ~= nil then
					ScriptManager:addOnControllerSelectedTileTrigger( enemy:getX(), enemy:getY(), "cursorOverUnit", "cursorOverUnit", false );
				end				
			end
		end
	end
end

function unitAtTargetPosition()
	ScriptManager:moveCamera( 9, 5, 0.8, 1, "" );
	enableAndMarkEnemyInfantries( 12, 14 );
	ScriptManager:addOnUnitHasAttackedTrigger( -1, -1, -1, "attackHasFinished", "attackHasFinished" );
	
	if ScriptManager:isControllerConnected() then
		ScriptManager:setControllerCursorEnabled( true );
	end
end

function attackHasFinished()
	cleanUp( true, true, true, false );
	if allUnitsDead() then
			purchaseSupplyTruck();
			return;
	end
	
	markAllYourUnitsExceptBase();
	ScriptManager:addOnUnitSelectedTrigger( 0, -1, -1, "unitSelected", "unitSelected" );
end

function unitAtPosition( unitId )
	if unitId == 8 then
		return true;
	end
	if unitId == ScriptManager:getValue( "Infantry_1" ) then
		return true;
	end
	if unitId == ScriptManager:getValue( "Infantry_2" ) then
		return true;
	end
	
	if unitId == ScriptManager:getValue( "Infantry_3" ) then
		if Unit:getUnitById( unitId ):getX() == 8 and Unit:getUnitById( unitId ):getX() == 4 then
			return true;
		end
		
		return false;
	end
	if unitId == ScriptManager:getValue( "Infantry_4" ) then
		if Unit:getUnitById( unitId ):getX() == 8 and Unit:getUnitById( unitId ):getX() == 3 then
			return true;
		end
		
		return false;
	end
	if unitId == ScriptManager:getValue( "unitTankTiger" ) then
		if Unit:getUnitById( unitId ):getX() == 8 and Unit:getUnitById( unitId ):getX() == 5 then
			return true;
		end
		
		return false;
	end
	
	return false;
end

function markTargetPosition( unitId )
	local bla = Unit:getUnitAt( 9, 6 );
	local x = -1;
	local y = -1;
	if unitId == ScriptManager:getValue( "Infantry_3" ) then
		x = 8;
		y = 4;
	end
	
	if unitId == ScriptManager:getValue( "Infantry_4" ) then
		x = 8;
		y = 3;
	end

	if unitId == ScriptManager:getValue( "unitTankTiger" ) and 
	bla == nil then
		x = 9;
		y = 6;
		
		ScriptManager:hideControllerButton();
		if ScriptManager:isControllerConnected() then
			ScriptManager:addOnControllerSelectedTileTrigger( x, y, "cursorOverUnit", "cursorOverUnit", false );
		end
		
	elseif unitId == ScriptManager:getValue( "unitTankTiger" ) then
		x = 8
		y = 6
		
		ScriptManager:hideControllerButton();
		if ScriptManager:isControllerConnected() then			
			ScriptManager:addOnControllerSelectedTileTrigger( x, y, "cursorOverUnit", "cursorOverUnit", false );
		end
	end
	
	if x > -1 and y > -1 then
		ScriptManager:showMarkerAt( x, y );
		ScriptManager:disableAllTilesExcept( x, y );
		ScriptManager:addOnEnterTileTrigger( x, y, -1, -1, -1, true, "unitAtTargetPosition", "unitAtTargetPosition" );
		ScriptManager:addOnTileDeselectedTrigger( x, y, -1, "unmarkPathTarget", "unmarkPathTarget" );
		ScriptManager:moveCamera( x, y, 0.8, 1, "" );
	end
end

function unmarkPathTarget( x, y, playerId )
	ScriptManager:hideMarkerAt( x,y );
	ScriptManager:removeTrigger( "unmarkPathTarget" );
end

function fillAttacksForAllUnits()
	ScriptManager:moveCamera( 8, 4, 0.75, 1, "" );
	ScriptManager:disableAllTilesExcept( 0,0 );
	
	for i=1, 20 do
		local unit = Unit:getUnitById( i );
		
		if unit ~= nil and unit:getPlayerId() == 0 and unit:hasAttacksLeft() == false then
			unit:setAttacksPerTurn( 1 );
		end
	end
end

function markAllYourUnitsExceptBase()
	ScriptManager:moveCamera( 8, 4, 0.75, 1, "" );
	ScriptManager:disableAllTilesExcept( 0, 0 );
	
	for i=1, 20 do
		local unit = Unit:getUnitById( i )
		
		if unit ~= nil and unit:getPlayerId() == 0 and unit:hasAttacksLeft() and unit:getId() ~= ScriptManager:getValue( "Infantry_3" ) and unit:getId() ~= ScriptManager:getValue( "Infantry_4" ) then
			ScriptManager:showMarkerAt( unit:getX(), unit:getY() );
			ScriptManager:enableAdditionalTile( unit:getX(), unit:getY() );
			
			ScriptManager:hideControllerButton();
			if ScriptManager:isControllerConnected() then				
				ScriptManager:setControllerCursorEnabled( true );
				ScriptManager:addOnControllerSelectedTileTrigger( unit:getX(), unit:getY(), "cursorOverUnitSinglePress", "cursorOverUnit", false );
			end
		end
	end
end

function allUnitsDead()
	for i=12, 14 do
		local unit = Unit:getUnitById( i );
		
		if unit ~= nil then
			return false;
		end
	end
	
	return true;
end

function finishedTutorial()
	cleanUp( true, true, true, true );
	ScriptManager:displaySubtitle( Text.T_MISSION_TUTORIAL_2_44 );
	ScriptManager:sleep( 5, "endGame" );
	ScriptManager:sleep( 4, "HideDialogue" );
	ScriptManager:setMissionStatus( Missions.MISSION_MAIN, MissionStatus.ACCOMPLISHED );
end

function endGame()
	ScriptManager:endGame();
end

-- Tutorial for supply truck --

function purchaseSupplyTruck()																				--Clean up and timer to start supplytruck tutorial.
	checkControllerAndHideButton();
	
	cleanUp( true, true, true, true );
	ScriptManager:sleep( 2, "startPurchaseFunction" );
end

function startPurchaseFunction()																			--Creat marker, finger and text macro to purchase a Supplytruck
	ScriptManager:displaySubtitle( Text.T_MISSION_TUTORIAL_2_21 );
	ScriptManager:moveCamera( 7, 2, 0.8, 1, "" );
	ScriptManager:showMarkerAt( 7, 2 );
	
	cleanupController();
	if ScriptManager:isControllerConnected() then
		ScriptManager:setControllerCursorEnabled( true );
		ScriptManager:addOnControllerSelectedTileTrigger( 7, 2, "cursorOverUnit", "cursorOverUnit", false );
	
	else
		ScriptManager:showFingerAtTile( 7, 2, 1 );
	end
	
	ScriptManager:disableAllTilesExcept( 7, 2 );
	ScriptManager:addOnUnitSelectedTrigger( -1, -1, -1, "purchaseTruckBaseSelected", "purchaseTruckBaseSelected" );
end

function purchaseTruckBaseSelected()																		--Base selected & highlight supplytruck button
	cleanUp( true, true, true, true )
	ScriptManager:hideFinger();
	ScriptManager:dismissSubtitle();
	
	cleanupController();
	if ScriptManager:isControllerConnected() then
		showControllerButtonGfx( ControllerButton.FACE_DOWN, 1, 7, 2 );
	end
	ScriptManager:displaySubtitle( Text.T_MISSION_TUTORIAL_2_45 );
												
	ScriptManager:enableInteractionButton( InteractionButton.REINFORCEMENT_2 )
	ScriptManager:startHighlightingInteractionButton( InteractionButton.REINFORCEMENT_2 )
	ScriptManager:addOnInteractionButtonSelectedTrigger( 0, -1, -1, InteractionButton.REINFORCEMENT_2, "unitPurchaseSelected_2", "unitPurchaseSelected_2" )
end

function unitPurchaseSelected_2()
	RemoveTrigger( "purchaseTruckBaseSelected" );
	ScriptManager:disableAllInteractionButtons();
	ScriptManager:disableAllTilesExcept( 7, 1 );
	ScriptManager:showMarkerAt( 7, 1 );
	
	ScriptManager:hideControllerButton();
	if ScriptManager:isControllerConnected() then
		ScriptManager:setControllerCursorEnabled( true );
		ScriptManager:addOnControllerSelectedTileTrigger( 7, 1, "cursorOverUnitSinglePress", "cursorOverUnit", false );
	
	else
		ScriptManager:showFingerAtTile( 7, 1, 1 );
	end
	
	ScriptManager:addOnUnitSelectedTrigger( -1, -1, -1, "supplyTruckPurchased", "supplyTruckPurchased" );
end

function supplyTruckPurchased( unit )																				--Supplytruck purchased and selected
	checkControllerAndHideButton();
	
	cleanUp( true, true, true, true );
	ScriptManager:hideFinger();
	ScriptManager:dismissSubtitle();
	ScriptManager:moveCamera( unit:getX(), unit:getY(), 0.8, 0.75, "" );
	ScriptManager:sleep( 2, "startResupplyTutorial" );
end

function startResupplyTutorial()
	RemoveTrigger( "startResupplyTutorial" );

	cleanupController();
	if ScriptManager:isControllerConnected() then
		ScriptManager:displaySubtitle( Text.T_TUTORIAL_CONTROLLER_PRESS_BUTTON );
		ScriptManager:showControllerButtonAtTile( ControllerButton.FACE_LEFT, 7, 1, 0.2 );
		ScriptManager:addOnControllerButtonReleasedTrigger( ControllerButton.FACE_LEFT, "purchaseTruckBaseSelectedController", "purchaseTruckBaseSelectedController" );
	
	else																			--Start resupply tutorial, Select supply
		ScriptManager:displaySubtitle( Text.T_MISSION_TUTORIAL_2_35 );
	end	
				
	ScriptManager:enableInteractionButton( InteractionButton.SUPPLY );
	ScriptManager:startHighlightingInteractionButton( InteractionButton.SUPPLY );
	ScriptManager:addOnInteractionButtonSelectedTrigger( -1, -1, -1, InteractionButton.SUPPLY, "supplyModeEnabled2Once", "supplyModeEnabled2Once" );
	ScriptManager:addOnUnitSuppliedTrigger( -1, -1, -1, "unitSupplied", "unitSupplied" );
	ScriptManager:addOnUnitDeselectedTrigger( -1, -1, -1, "removeMarkers", "removeMarkers" );
end

function supplyModeEnabled2Once()
	ScriptManager:disableAllInteractionButtons();
	ScriptManager:removeTrigger( "supplyModeEnabled2Once" );
	ScriptManager:addOnInteractionButtonDeselectedTrigger( -1, -1, -1, InteractionButton.SUPPLY, "supplyModeDisabled2", "supplyModeDisabled2" );
	ScriptManager:disableAllTilesExcept( 0, 0 );
	markFirstLowHealthUnit();
end

function supplyModeEnabled2()
	ScriptManager:disableAllInteractionButtons();
	ScriptManager:removeTrigger( "supplyModeEnabled2" );
	ScriptManager:addOnInteractionButtonDeselectedTrigger( -1, -1, -1, InteractionButton.SUPPLY, "supplyModeDisabled2", "supplyModeDisabled2" );
	ScriptManager:disableAllTilesExcept( 0, 0 );
	markLowHealthUnits();
end

function HideFinger()
	--print( string.format( "hidefunction" ) );
	ScriptManager:addOnUnitDeselectedTrigger( -1, -1, -1, "HideFingerTrue", "HideFingerTrue" );
end

function HideFingerTrue()
	ScriptManager:hideFinger();
	ScriptManager:removeTrigger( "HideFingerTrue" );
end

function supplyModeDisabled2()
	cleanUp( true, false, true, false );
	ScriptManager:removeTrigger( "supplyModeDisabled2" );
	ScriptManager:addOnInteractionButtonSelectedTrigger( -1, -1, -1, InteractionButton.SUPPLY, "supplyModeEnabled2", "supplyModeEnabled2" );
end

function markFirstLowHealthUnit()
	checkControllerAndHideButton();
	
	local cameraX;
	local cameraY;
	
	for i=1, 20 do
		local unit = Unit:getUnitById( i );
	
		if unit ~= nil and unit:getPlayerId() == 0 and unit:needSupplys() then
			ScriptManager:showMarkerAt( unit:getX(), unit:getY() );
			ScriptManager:enableAdditionalTile( unit:getX(), unit:getY() );
			cameraX = unit:getX();
			cameraY = unit:getY();
			
			if ScriptManager:isControllerConnected() then
				ScriptManager:addOnControllerSelectedTileTrigger( unit:getX(), unit:getY(), "cursorOverUnit", "cursorOverUnit", false );
				ScriptManager:setControllerCursorEnabled( true );
			
			else
				ScriptManager:showFingerAtTile( unit:getX(), unit:getY(),1 );
			end
		
			break;
		end
	end
	
	ScriptManager:moveCamera( cameraX, cameraY, 0.8, 1, "" );
	HideFinger();
end

function markLowHealthUnits()
	checkControllerAndHideButton();
	
	local cameraX;
	local cameraY;
	for i=1, 20 do
		local unit = Unit:getUnitById( i );
		
		if unit ~= nil and unit:getPlayerId() == 0 and unit:needSupplys() then
			ScriptManager:showMarkerAt( unit:getX(), unit:getY() );
			ScriptManager:enableAdditionalTile( unit:getX(), unit:getY() );
			cameraX = unit:getX();
			cameraY = unit:getY();
			
			if ScriptManager:isControllerConnected() then
				ScriptManager:addOnControllerSelectedTileTrigger( unit:getX(), unit:getY(), "cursorOverUnit", "cursorOverUnit", false );
				ScriptManager:setControllerCursorEnabled( true );
			end
		end
	end
	ScriptManager:moveCamera( cameraX, cameraY, 0.8, 1, "" )
end

function unitSupplied( truck, suppliedUnit )
	if allUnitsSupplied() then
		cleanUp( true, true, true, true );
		ScriptManager:hideControllerButton();		
		finishedTutorial();
		
	else
		ScriptManager:disableAllTilesExcept( 0, 0 );
		ScriptManager:sleep( 0.5, "markLowHealthUnits" );
	end
end

function allUnitsSupplied()
	for i=1, 20 do
		local unit = Unit:getUnitById( i );
	
		if unit ~= nil and unit:getPlayerId() == 0 and unit:needSupplys() then
			return false;
		end
	end
	
	return true;
end