require "lua/Constants"
require "lua/Functions"
require "lua/Functions_2"
require "lua/Text"
require "lua/Units"
require "math"

function onGameStart()
	--print( string.format( "[SCRIPT] onGameStart()..." ) );
	ScriptManager:createMission( 0, true, Text.T_MISSION_TUTORIAL_MAIN_1 );
	ScriptManager:setMissionStatus( 0, MissionStatus.IN_PROGRESS );
	ScriptManager:setMissionXp( 0, 25 );
	ScriptManager:setMedalReward( 250 );
		
	ScriptManager:camouflageUnit( 4 );
	ScriptManager:disableAllTilesExcept( 0, 0 );
	ScriptManager:disableEndTurnButton();
	ScriptManager:disableAllInteractionButtons();
	-- TODO: ENABLE AFTER TESTING!
	ScriptManager:disableSkipCutsceneButton();
	ScriptManager:disableSkipReplayButton();
	ScriptManager:preparePopupText();
	ScriptManager:gainGold( 0, 2000 );
	ScriptManager:addOnControllerConnectedTrigger( "onControllerConnect", "onControllerConnect", false );
	ScriptManager:addOnControllerDisconnectedTrigger( "onControllerDisconnect", "onControllerDisconnect", false );
	
	if ScriptManager:isControllerConnected() then
		ScriptManager:setControllerCursorEnabled( false );
		ScriptManager:setControllerCursorPosition( 4, 4 );
	end
	
	ScriptManager:sleep( 0.5, "intro" );
	--ScriptManager:sleep( 0.5, "selectAirplane" );
end

function HideDialogue()
	ScriptManager:dismissSubtitle();
end

function intro()
	ScriptManager:displaySubtitle( Text.T_MISSION_TUTORIAL_3_1 );
	ScriptManager:sleep( 2, "explainAirplanes" );
end

function explainAirplanes()
	ScriptManager:sleep( 2.5, "HideDialogue" );
	ScriptManager:sleep( 4, "ShowPlaneExplanation" );
	ScriptManager:sleep( 9, "HideDialogue" );
	ScriptManager:moveCamera( 2, 2, 1, 8, "selectAirplaneWait" );
end

function ShowPlaneExplanation()
	ScriptManager:displaySubtitle( Text.T_MISSION_TUTORIAL_3_2 );
end

function selectAirplaneWait()
	ScriptManager:sleep( 2, "selectAirplane" );
end

function selectAirplane()
	ScriptManager:moveCamera( 2, 3, 0.8, 1, "" );
	ScriptManager:dismissSubtitle();
	ScriptManager:displaySubtitle( Text.T_MISSION_TUTORIAL_3_3 );
	ScriptManager:showMarkerAt( 2, 3 );
	ScriptManager:disableAllTilesExcept( 2, 3 );
	
	if ScriptManager:isControllerConnected() then
		ScriptManager:addOnControllerSelectedTileTrigger( 2, 3, "cursorOverUnitSinglePress", "cursorOverUnit", false );
		ScriptManager:setControllerCursorEnabled( true );
	
	else
		ScriptManager:showFingerAtTile( 2, 3, 0 );
		ScriptManager:loopFingerClicks( 1 );
	end
	
	ScriptManager:addOnUnitSelectedTrigger( -1, -1, -1, "airplaneSelected", "airplaneSelected" );
end

function cleanUp( markers, triggers, tiles, subtitles, finger )
	if markers then
		ScriptManager:removeAllMarkers();
	end
	
	if triggers then
		ScriptManager:removeAllTriggers();
	end
	
	if tiles then
		ScriptManager:disableAllTilesExcept( 0, 0 );
	end
	
	if subtitles then
		ScriptManager:dismissSubtitle();
	end
	
	if finger then
		ScriptManager:hideFinger();
	end
end

function airplaneSelected()
	cleanUp( true, true, true, true, false );
	ScriptManager:disableAllTilesExcept( 5, 6 );
	ScriptManager:showMarkerAt( 5, 6 );
	
	cleanupController();
	if ScriptManager:isControllerConnected() then
		ScriptManager:addOnControllerSelectedTileTrigger( 5, 6, "cursorOverUnitQuadraPress", "cursorOverUnit", false );
		ScriptManager:setControllerCursorEnabled( true );
	
	else
		ScriptManager:moveFingerToTile( 5, 6 );
	end
	
	ScriptManager:displaySubtitle( Text.T_MISSION_TUTORIAL_3_4 );
	ScriptManager:addOnEnterTileTrigger( 5, 6, -1, -1, -1, true, "enemySpotted", "enemySpotted" );
	ScriptManager:addOnTileDeselectedTrigger( 5, 6, -1, "unmarkTile", "unmarkTile" );
	ScriptManager:moveCamera( 4, 5, 0.8, 1.5, "" );
end

function unmarkTile()
	--ScriptManager:removeTrigger( "unmarkTile" );
	--ScriptManager:removeAllMarkers();
	--ScriptManager:showFingerAtTile( 5, 6, 0 );
	if ScriptManager:isControllerConnected() == false then
		ScriptManager:loopFingerClicks( 1 );
	end	
end

function enemySpotted()
	checkControllerAndHideButton();
	
	cleanUp( true, true, true, true, true );
	ScriptManager:displaySubtitleAutoWithCallback( Text.T_MISSION_TUTORIAL_3_5, "HideDialogue" );
	ScriptManager:moveCamera( 5, 8, 0.8, 1, "" );
	ScriptManager:removeTrigger( "unmarkTile" );
	ScriptManager:hideFinger();
	ScriptManager:removeAllMarkers();
	ScriptManager:sleep( 2, "attackAirplane" );
end

function attackAirplane()
	cleanUp( true, true, true, true, true );
	ScriptManager:displaySubtitle( Text.T_MISSION_TUTORIAL_3_6 );
	ScriptManager:disableAllTilesExcept( 5, 8 );
	ScriptManager:showMarkerAt( 5, 8 );
	
	cleanupController();
	if ScriptManager:isControllerConnected() then
		ScriptManager:setControllerCursorEnabled( true );
		ScriptManager:addOnControllerSelectedTileTrigger( 5, 8, "cursorOverUnit", "cursorOverUnit", false );	
	end
	
	ScriptManager:addOnUnitDeselectedTrigger( -1, -1, -1, "unmarkTile", "unmarkTile" );
	ScriptManager:addOnUnitHasAttackedTrigger( -1, -1, 4, "antiAirFired", "antiAirFired" );
	ScriptManager:addOnUnitHasAttackedTrigger( -1, -1, 5, "playerPlaneHasAttacked", "playerPlaneHasAttacked" );
end

function playerPlaneHasAttacked()
	checkControllerAndHideButton();
	
	ScriptManager:removeTrigger( "playerPlaneHasAttacked" );
	Unit:getUnitById( 5 ):setHealth( 70 );
end

function antiAirFired()
	cleanUp( true, true, true, true, true );
	ScriptManager:displaySubtitle( Text.T_MISSION_TUTORIAL_3_7 );
	ScriptManager:sleep( 4.5, "explainAA" );
end

function explainAA()
	cleanUp( true, true, true, true, true );
	ScriptManager:displaySubtitle( Text.T_MISSION_TUTORIAL_3_8 );
	ScriptManager:moveCamera( 4, 5, 1, 6.5, "selectTiger" );
end

function selectTiger()
	cleanUp( true, true, true, true, true );
	ScriptManager:displaySubtitle( Text.T_MISSION_TUTORIAL_3_9 );
	ScriptManager:showMarkerAt( 4, 6 );
	ScriptManager:disableAllTilesExcept( 4, 6 );
	
	cleanupController();
	if ScriptManager:isControllerConnected() then
		ScriptManager:setControllerCursorEnabled( true );
		ScriptManager:addOnControllerSelectedTileTrigger( 4, 6, "cursorOverUnitSinglePress", "cursorOverUnit", false );
	
	else
		ScriptManager:showFingerAtTile( 4, 6, 0 );
		ScriptManager:loopFingerClicks( 1 );
	end
	
	ScriptManager:addOnUnitSelectedTrigger( -1, -1, -1, "tigerSelected", "tigerSelected" )	;
end

function tigerSelected()
	cleanUp( true, true, true, true, false );
	ScriptManager:disableAllTilesExcept( 5, 5 );
	ScriptManager:showMarkerAt( 5, 5 );
	
	cleanupController();
	if ScriptManager:isControllerConnected() then
		ScriptManager:setControllerCursorEnabled( true );
		ScriptManager:addOnControllerSelectedTileTrigger( 5, 5, "cursorOverUnit", "cursorOverUnit", false );
	
	else
		ScriptManager:moveFingerToTile( 5, 5 );
	end
	
	ScriptManager:addOnEnterTileTrigger( 5, 5, -1, -1, -1, true, "aaRevealed", "aaRevealed" );
	ScriptManager:addOnTileDeselectedTrigger( 5, 5, -1, "unmarkTile", "unmarkTile" );
end

function aaRevealed()
	cleanUp( true, true, true, true, true );
	ScriptManager:disableAllTilesExcept( 6, 4 );
	ScriptManager:showMarkerAt( 6, 4 );
	
	cleanupController();
	if ScriptManager:isControllerConnected() then
		ScriptManager:setControllerCursorEnabled( true );
		ScriptManager:addOnControllerSelectedTileTrigger( 6, 4, "cursorOverUnit", "cursorOverUnit", false );
	end
	
	ScriptManager:addOnUnitDeselectedTrigger( -1, -1, -1, "unmarkTile", "unmarkTile" );
	ScriptManager:addOnUnitHasAttackedTrigger( -1, -1, -1, "aaDestroyed", "aaDestroyed" );
end

function aaDestroyed()
	checkControllerAndHideButton();
	
	cleanUp( true, true, true, true, true );
	ScriptManager:displaySubtitle( Text.T_MISSION_TUTORIAL_3_10 );
	ScriptManager:sleep( 2, "selectSecondAirplane" );
end

function selectSecondAirplane()
	ScriptManager:moveCamera( 3, 2, 0.8, 1, "" );
	ScriptManager:dismissSubtitle();
	ScriptManager:displaySubtitle( Text.T_MISSION_TUTORIAL_3_11 );
	ScriptManager:showMarkerAt( 3, 2 );
	ScriptManager:disableAllTilesExcept( 3, 2 );
	
	cleanupController();
	if ScriptManager:isControllerConnected() then
		ScriptManager:setControllerCursorEnabled( true );
		ScriptManager:addOnControllerSelectedTileTrigger( 3, 2, "cursorOverUnitSinglePress", "cursorOverUnit", false );
	
	else
		ScriptManager:showFingerAtTile( 3, 2, 0 );
		ScriptManager:loopFingerClicks( 1 );
	end
	
	ScriptManager:addOnUnitSelectedTrigger( -1, -1, -1, "secondAirplaneSelected", "secondAirplaneSelected" );
end

function secondAirplaneSelected()
	cleanUp( true, true, true, false, false );
	ScriptManager:disableAllTilesExcept( 5, 6 );
	ScriptManager:showMarkerAt( 5, 6 );
	
	cleanupController();
	if ScriptManager:isControllerConnected() then
		ScriptManager:setControllerCursorEnabled( true );
		ScriptManager:addOnControllerSelectedTileTrigger( 5, 6, "cursorOverUnit", "cursorOverUnit", false );
		
	else
		ScriptManager:moveFingerToTile( 5, 6 );
	end
	
	ScriptManager:addOnEnterTileTrigger( 5, 6, -1, -1, -1, true, "secondAirplaneAtPosition", "secondAirplaneAtPosition" );
	ScriptManager:addOnTileDeselectedTrigger( 5, 6, -1, "unmarkTile", "unmarkTile" );
	ScriptManager:moveCamera( 4, 5, 0.8, 1.5, "" );
end

function secondAirplaneAtPosition()
	cleanUp( true, true, true, true, true );
	ScriptManager:displaySubtitle( Text.T_MISSION_TUTORIAL_3_6 );
	ScriptManager:disableAllTilesExcept( 5, 8 );
	ScriptManager:showMarkerAt( 5, 8 );
	
	cleanupController();
	if ScriptManager:isControllerConnected() then
		ScriptManager:setControllerCursorEnabled( true );
		ScriptManager:addOnControllerSelectedTileTrigger( 5, 8, "cursorOverUnit", "cursorOverUnit", false );
	end
	
	ScriptManager:addOnUnitDeselectedTrigger( -1, -1, -1, "unmarkTile", "unmarkTile" );
	ScriptManager:addOnUnitHasAttackedTrigger( -1, -1, -1, "enemyAirplaneDead", "enemyAirplaneDead" );
	ScriptManager:moveCamera( 5, 8, 0.8, 1, "" );
end

function enemyAirplaneDead()
	checkControllerAndHideButton();
	
	cleanUp( true, true, true, true, true );
	ScriptManager:displaySubtitle( Text.T_MISSION_TUTORIAL_3_12 );
	ScriptManager:moveCamera( 1, 9, 0.9, 2.5, "cameraAtBase" );
	ScriptManager:createMission( 1, false, "" );
end

function cameraAtBase()
	cleanUp( true, true, true, false, true );
	ScriptManager:showMarkerAt( 1, 9 );
	ScriptManager:disableAllTilesExcept( 1, 9 );
	
	cleanupController();
	if ScriptManager:isControllerConnected() then
		ScriptManager:setControllerCursorEnabled( true );
		ScriptManager:addOnControllerSelectedTileTrigger( 1, 9, "cursorOverUnit", "cursorOverUnit", false );
	
	else
		ScriptManager:showFingerAtTile( 1, 9, 0 );
		ScriptManager:loopFingerClicks( 1 );
	end
	
	ScriptManager:addOnUnitSelectedTrigger( -1, -1, -1, "baseSelected", "baseSelected" );
	ScriptManager:setMissionProgress( 1, ScriptManager:getMissionProgress( 1 ) + 1 );
end

function baseSelected()
	cleanUp( true, true, true, false, true );
	ScriptManager:enableInteractionButton( InteractionButton.REINFORCEMENT_1 );
	ScriptManager:startHighlightingInteractionButton( InteractionButton.REINFORCEMENT_1 );
	
	cleanupController();
	if ScriptManager:isControllerConnected() then
		showControllerButtonGfx( ControllerButton.FACE_DOWN, 1, 1, 9 );
	end
	
	ScriptManager:addOnInteractionButtonSelectedTrigger( 0, -1, -1, InteractionButton.REINFORCEMENT_1, "unitPurchaseSelected", "unitPurchaseSelected" );
end

function unitPurchaseSelected()
	ScriptManager:disableAllInteractionButtons()
	if( ScriptManager:getMissionProgress( 1 ) == 1 ) then
		ScriptManager:disableAllTilesExcept( 2, 8 );
		ScriptManager:showMarkerAt( 2, 8 );
		
		cleanupController();
		if ScriptManager:isControllerConnected() then
			ScriptManager:setControllerCursorEnabled( true );
			ScriptManager:addOnControllerSelectedTileTrigger( 2, 8, "cursorOverUnitSinglePress", "cursorOverUnit", false );
		
		else
			ScriptManager:showFingerAtTile( 2, 8, 1 );
		end
	else
		ScriptManager:disableAllTilesExcept( 2, 9 );
		ScriptManager:showMarkerAt( 2, 9 );
		
		cleanupController();
		if ScriptManager:isControllerConnected() then
			ScriptManager:setControllerCursorEnabled( true );
			ScriptManager:addOnControllerSelectedTileTrigger( 2, 9, "cursorOverUnitSinglePress", "cursorOverUnit", false );
		
		else
			ScriptManager:showFingerAtTile( 2, 9, 1 );
		end
	end
	ScriptManager:addOnUnitSelectedTrigger( -1, -1, -1, "antAirPurchased", "antAirPurchased" );
end

function antAirPurchased( antiAir )
	if( ScriptManager:getMissionProgress( 1 ) == 2 ) then
		ScriptManager:storeValue( "aa2", antiAir:getId() );
		antiAirsBought();
	else
		if( ScriptManager:getMissionProgress( 1 ) == 1 ) then
			ScriptManager:storeValue( "aa1", antiAir:getId() );
			cameraAtBase();
		end
	end
end

function antiAirsBought()
	checkControllerAndHideButton();

	cleanUp( true, true, true, true, true );
	ScriptManager:displaySubtitle( Text.T_MISSION_TUTORIAL_3_13 );
	ScriptManager:moveCamera( 1, 8, 1.2, 5, "transporterExplained" );
end

function transporterExplained()
	cleanUp( true, true, true, true, true );
	ScriptManager:moveCamera( 1, 8, 0.9, 1, "" );
	ScriptManager:showMarkerAt( 1, 8 );
	ScriptManager:disableAllTilesExcept( 1, 8 );
	
	cleanupController();
	if ScriptManager:isControllerConnected() then
		ScriptManager:setControllerCursorEnabled( true );
		ScriptManager:addOnControllerSelectedTileTrigger( 1, 8, "cursorOverUnitSinglePress", "cursorOverUnit", false );
	
	else	
		ScriptManager:showFingerAtTile( 1, 8, 0 );
		ScriptManager:loopFingerClicks( 1 );
	end
	
	ScriptManager:addOnUnitSelectedTrigger( -1, -1, -1, "transporterSelected", "transporterSelected" );
end

function transporterSelected()
	cleanUp( true, true, true, true, true )
	ScriptManager:displaySubtitle( Text.T_MISSION_TUTORIAL_3_14 );
	ScriptManager:disableAllInteractionButtons();
	ScriptManager:enableInteractionButton( InteractionButton.TRANSPORT );
	ScriptManager:startHighlightingInteractionButton( InteractionButton.TRANSPORT );
	
	cleanupController();
	if ScriptManager:isControllerConnected() then
		showControllerButtonGfx( ControllerButton.FACE_LEFT, 1, 1, 8 );
	end
	
	ScriptManager:addOnInteractionButtonSelectedTrigger( -1, -1, -1, InteractionButton.TRANSPORT, "transportModeEnabled", "transportModeEnabled" );
	ScriptManager:addOnUnitOccupiedTrigger( -1, -1, -1, "firstAntiAirLoaded", "firstAntiAirLoaded" );
end

function transportModeEnabled()
	ScriptManager:removeTrigger( "transportModeEnabled" );
	ScriptManager:addOnInteractionButtonDeselectedTrigger( -1, -1, -1, InteractionButton.TRANSPORT, "transportModeDisabled", "transportModeDisabled" );
	ScriptManager:disableAllTilesExcept( 2, 8 );
	ScriptManager:disableAllInteractionButtons();
	ScriptManager:showMarkerAt( 2, 8 );
	
	cleanupController();
	if ScriptManager:isControllerConnected() then
		ScriptManager:setControllerCursorEnabled( true );
		ScriptManager:addOnControllerSelectedTileTrigger( 2, 8, "cursorOverUnit", "cursorOverUnit", false );
	
	else
		ScriptManager:moveFingerToTile( 2, 8 );
		ScriptManager:loopFingerClicks( 1 );
	end
	
	ScriptManager:sleep( 0, "stopHighlightTransportIcons" );
end

function transportModeDisabled()
	cleanUp( true, false, true, false, true );
	ScriptManager:removeTrigger( "transportModeDisabled" );
	ScriptManager:addOnInteractionButtonSelectedTrigger( -1, -1, -1, InteractionButton.TRANSPORT, "transportModeEnabled", "transportModeEnabled" );
	ScriptManager:sleep( 0, "highlightTransportIcons" );
end

function highlightTransportIcons()
	ScriptManager:startHighlightingInteractionButton( InteractionButton.TRANSPORT );
end

function stopHighlightTransportIcons()
	ScriptManager:stopHighlightingInteractionButton( InteractionButton.TRANSPORT );
end

function highlightUnloadIcons()
	ScriptManager:startHighlightingInteractionButton( InteractionButton.TRANSPORT_UNLOAD_1 );
	ScriptManager:startHighlightingInteractionButton( InteractionButton.TRANSPORT_UNLOAD_2 );
end

function stopHighlightUnloadIcons()
	ScriptManager:stopHighlightingInteractionButton( InteractionButton.TRANSPORT_UNLOAD_1 );
	ScriptManager:stopHighlightingInteractionButton( InteractionButton.TRANSPORT_UNLOAD_2 );
end

function firstAntiAirLoaded()
	cleanUp( true, true, true, false, true );
	ScriptManager:addOnInteractionButtonDeselectedTrigger( -1,-1,-1,InteractionButton.TRANSPORT,"transportModeDisabled2","transportModeDisabled2" );
	ScriptManager:disableAllTilesExcept( 2,9 );
	ScriptManager:showMarkerAt( 2,9 );
	
	ScriptManager:hideControllerButton();
	if ScriptManager:isControllerConnected() then
		ScriptManager:setControllerCursorEnabled( true );
		ScriptManager:addOnControllerSelectedTileTrigger( 2, 9, "cursorOverUnit", "cursorOverUnit", false );
		
	else
		ScriptManager:moveFingerToTile( 2, 9 );
		ScriptManager:loopFingerClicks( 1 );
	end
	
	ScriptManager:addOnUnitOccupiedTrigger( -1, -1, -1, "secondAntiAirLoaded", "secondAntiAirLoaded" );
end

function transportModeEnabled2()
	ScriptManager:removeTrigger( "transportModeEnabled2" );
	ScriptManager:addOnInteractionButtonDeselectedTrigger( -1, -1, -1, InteractionButton.TRANSPORT, "transportModeDisabled2", "transportModeDisabled2" );
	ScriptManager:disableAllInteractionButtons();
	ScriptManager:disableAllTilesExcept( 2, 9 );
	ScriptManager:showMarkerAt( 2, 9 );
	
	ScriptManager:hideControllerButton();
	if ScriptManager:isControllerConnected() then
		ScriptManager:setControllerCursorEnabled( true );
		ScriptManager:addOnControllerSelectedTileTrigger( 2, 9, "cursorOverUnit", "cursorOverUnit", false );
	
	else
		ScriptManager:moveFingerToTile( 2, 9 );
		ScriptManager:loopFingerClicks( 1 );
	end
	
	ScriptManager:sleep( 0, "stopHighlightTransportIcons" );
end

function transportModeDisabled2()
	cleanUp( true, false, true, false, true );
	ScriptManager:removeTrigger( "transportModeDisabled2" );
	ScriptManager:addOnInteractionButtonSelectedTrigger( -1, -1, -1, InteractionButton.TRANSPORT, "transportModeEnabled2", "transportModeEnabled2" );
	ScriptManager:sleep( 0, "highlightTransportIcons" );
end

function secondAntiAirLoaded()
	cleanUp( true, true, true, true, true );
	ScriptManager:stopHighlightingInteractionButton( InteractionButton.TRANSPORT );
	ScriptManager:disableAllInteractionButtons();
	ScriptManager:moveCamera( 3, 6, 0.8, 2, "startUnloadingSequence" );
end

function startUnloadingSequence()
	cleanUp( true, true, true, true, true );
	ScriptManager:displaySubtitle( Text.T_MISSION_TUTORIAL_3_15 );
	ScriptManager:disableAllTilesExcept( 3, 5 );
	ScriptManager:showMarkerAt( 3, 5 );
	
	cleanupController();
	if ScriptManager:isControllerConnected() then
		ScriptManager:setControllerCursorEnabled( true );
		ScriptManager:addOnControllerSelectedTileTrigger( 3, 5, "cursorOverUnit", "cursorOverUnit", false );
	
	else
		ScriptManager:moveFingerToTile( 3, 5 );
		ScriptManager:loopFingerClicks( 1 );
	end
	
	ScriptManager:addOnEnterTileTrigger( 3, 5, -1, -1, -1, true, "transporterAtFirstPosition", "transporterAtFirstPosition" );
	ScriptManager:addOnTileDeselectedTrigger( 3, 5, -1, "unmarkTile", "unmarkTile" );
end

function transporterAtFirstPosition()
	cleanUp( true, true, true, false, true );
	ScriptManager:enableInteractionButton( InteractionButton.TRANSPORT_UNLOAD_2 );
	ScriptManager:startHighlightingInteractionButton( InteractionButton.TRANSPORT_UNLOAD_2 );
	
	cleanupController();
	if ScriptManager:isControllerConnected() then
		showControllerButtonGfx( ControllerButton.FACE_LEFT, 1, 3, 5 );
	end
	
	ScriptManager:addOnInteractionButtonSelectedTrigger( -1, -1, -1, -1, "unloadModeEnabled", "unloadModeEnabled" );
	ScriptManager:addOnUnitUnloadedTrigger( -1, -1, -1, "firstAAUnloaded", "firstAAUnloaded" );
end

function unloadModeEnabled()
	ScriptManager:removeTrigger( "unloadModeEnabled" );
	ScriptManager:addOnInteractionButtonDeselectedTrigger( -1, -1, -1, -1, "unloadModeDisabled", "unloadModeDisabled" );
	ScriptManager:disableAllInteractionButtons();
	ScriptManager:disableAllTilesExcept( 4, 5 );
	ScriptManager:showMarkerAt( 4, 5 );
	
	cleanupController();
	if ScriptManager:isControllerConnected() then
		ScriptManager:setControllerCursorEnabled( true );
		ScriptManager:addOnControllerSelectedTileTrigger( 4, 5, "cursorOverUnit", "cursorOverUnit", false );
	
	else
		ScriptManager:moveFingerToTile( 4, 5 );
		ScriptManager:loopFingerClicks( 1 );
	end
	
	ScriptManager:sleep( 0, "stopHighlightUnloadIcons" );
end

function unloadModeDisabled()
	cleanUp( true, false, true, false, true );
	ScriptManager:removeTrigger( "unloadModeDisabled" );
	ScriptManager:addOnInteractionButtonSelectedTrigger( -1, -1, -1, -1, "unloadModeEnabled", "unloadModeEnabled" );
	ScriptManager:sleep( 0, "highlightUnloadIcons" );
end

function firstAAUnloaded()
	cleanUp( true, true, true, false, true );
	ScriptManager:disableAllInteractionButtons();
	ScriptManager:disableAllTilesExcept( 5, 3 );
	ScriptManager:showMarkerAt( 5, 3 );
	
	cleanupController();
	if ScriptManager:isControllerConnected() then
		ScriptManager:setControllerCursorEnabled( true );
		ScriptManager:addOnControllerSelectedTileTrigger( 5, 3, "cursorOverUnit", "cursorOverUnit", false );
	
	else
		ScriptManager:moveFingerToTile( 5, 3 );
		ScriptManager:loopFingerClicks( 1 );
	end
	
	ScriptManager:addOnEnterTileTrigger( 5, 3, -1, -1, -1, true, "transporterAtSecondPosition", "transporterAtSecondPosition" );
	ScriptManager:addOnTileDeselectedTrigger( 5, 3, -1, "unmarkTile", "unmarkTile" );
end

function transporterAtSecondPosition()
	cleanUp( true, true, true, false, true );
	ScriptManager:enableInteractionButton( InteractionButton.TRANSPORT_UNLOAD_1 );
	ScriptManager:startHighlightingInteractionButton( InteractionButton.TRANSPORT_UNLOAD_1 );
	
	cleanupController();
	if ScriptManager:isControllerConnected() then
		showControllerButtonGfx( ControllerButton.FACE_LEFT, 1, 5, 3 );
	end
	
	ScriptManager:addOnInteractionButtonSelectedTrigger( -1, -1, -1, -1, "unloadModeEnabled2", "unloadModeEnabled2" );
	ScriptManager:addOnUnitUnloadedTrigger( -1, -1, -1, "secondAAUnloaded", "secondAAUnloaded" );
end

function unloadModeEnabled2()
	ScriptManager:removeTrigger( "unloadModeEnabled2" );
	ScriptManager:addOnInteractionButtonDeselectedTrigger( -1, -1, -1, -1, "unloadModeDisabled2", "unloadModeDisabled2" );
	ScriptManager:disableAllInteractionButtons();
	ScriptManager:disableAllTilesExcept( 6, 4 );
	ScriptManager:showMarkerAt( 6, 4 );
	
	ScriptManager:hideControllerButton();
	if ScriptManager:isControllerConnected() then
		ScriptManager:setControllerCursorEnabled( true );
		ScriptManager:addOnControllerSelectedTileTrigger( 6, 4, "cursorOverUnit", "cursorOverUnit", false );
	
	else
		ScriptManager:moveFingerToTile( 6, 4 );
		ScriptManager:loopFingerClicks( 1 );
	end
	
	ScriptManager:sleep( 0, "stopHighlightUnloadIcons" );
end

function unloadModeDisabled2()
	cleanUp( true, false, true, false, true );
	ScriptManager:removeTrigger( "unloadModeDisabled2" );
	ScriptManager:addOnInteractionButtonSelectedTrigger( -1, -1, -1, -1, "unloadModeEnabled2", "unloadModeEnabled2" );
	ScriptManager:sleep( 0, "highlightUnloadIcons" );
end

function secondAAUnloaded()
	checkControllerAndHideButton();
	
	cleanUp( true, true, true, true, true );
	ScriptManager:sleep( 0.5, "ShowSubtitle16" );
	ScriptManager:sleep( 5, "StealthAASequence" );
	ScriptManager:disableAllInteractionButtons();
end

function ShowSubtitle16()
	ScriptManager:displaySubtitle( Text.T_MISSION_TUTORIAL_3_16 );
end

--Stealth AA's

function StealthAASequence()
	ScriptManager:dismissSubtitle();
	ScriptManager:sleep( 1.5, "ShowCamoDialogue" );
	ScriptManager:moveCamera( 4, 5, 0.8, 3, "" );
	ScriptManager:sleep( 4, "ShowMarkerOnAA1" );
end

function ShowCamoDialogue()
	ScriptManager:displaySubtitle( Text.T_MISSION_TUTORIAL_3_CAMOUFLAGE );
end

function ShowMarkerOnAA1()
	ScriptManager:moveCamera( 4, 5, 0.9, 1, "" );
	ScriptManager:showMarkerAt( 4, 5 );
	ScriptManager:disableAllTilesExcept( 4, 5 );
	
	cleanupController();
	if ScriptManager:isControllerConnected() then
		ScriptManager:setControllerCursorEnabled( true );
		ScriptManager:addOnControllerSelectedTileTrigger( 4, 5, "cursorOverUnitSinglePress", "cursorOverUnit", false );
		
	else
		ScriptManager:showFingerAtTile( 4, 5, 1 );
	end
	
	ScriptManager:addOnUnitSelectedTrigger( -1, -1, -1, "AASelected_1", "AASelected_1" );
end

function AASelected_1()
	ScriptManager:dismissSubtitle()
	cleanUp( true, true, true, true, true );
	ScriptManager:enableInteractionButton( InteractionButton.CAMOUFLAGE );
	ScriptManager:startHighlightingInteractionButton( InteractionButton.CAMOUFLAGE );
	
	cleanupController();
	if ScriptManager:isControllerConnected() then
		showControllerButtonGfx( ControllerButton.FACE_LEFT, 1, 4, 5 );
	end
	
	ScriptManager:addOnInteractionButtonSelectedTrigger( -1, -1, -1, InteractionButton.CAMOUFLAGE, "ShowMarkerOnAA2", "ShowMarkerOnAA2" );
end

function ShowMarkerOnAA2()
	ScriptManager:moveCamera( 6, 4, 0.9, 1, "" );
	ScriptManager:showMarkerAt( 6, 4 );
	ScriptManager:disableAllTilesExcept( 6, 4 );
	
	cleanupController();
	if ScriptManager:isControllerConnected() then
		ScriptManager:setControllerCursorEnabled( true );
		ScriptManager:addOnControllerSelectedTileTrigger( 6, 4, "cursorOverUnitSinglePress", "cursorOverUnit", false );
		
	else
		ScriptManager:showFingerAtTile( 6, 4, 1 );
	end
	
	ScriptManager:addOnUnitSelectedTrigger( -1, -1, -1, "AASelected_2", "AASelected_2" )	;
end

function AASelected_2()
	cleanUp( true, true, true, true, true );
	ScriptManager:enableInteractionButton( InteractionButton.CAMOUFLAGE );
	
	cleanupController();
	if ScriptManager:isControllerConnected() then
		showControllerButtonGfx( ControllerButton.FACE_LEFT, 1, 6, 4 );
	end
	
	ScriptManager:addOnInteractionButtonSelectedTrigger( -1, -1, -1, InteractionButton.CAMOUFLAGE, "WaitForEndTurn", "WaitForEndTurn" );
end

function WaitForEndTurn()
	checkControllerAndHideButton();
	
	ScriptManager:moveCamera( 5, 5, 0.8, 2, "endFirstTurn" );
end

function endFirstTurn()
	ScriptManager:dismissSubtitle();
	ScriptManager:displaySubtitle( Text.T_MISSION_TUTORIAL_3_17 );
	ScriptManager:enableEndTurnButton();
	
	cleanupController();
	if ScriptManager:isControllerConnected() then
		showControllerButtonGfx( ControllerButton.FACE_UP, 1, 5, 5 );
	end
		
	ScriptManager:startHighlightingEndTurnButton();
	
	ScriptManager:addOnTurnStartTrigger( 0, -1, -1, "startCutscene" );
end

function startCutscene()
	ScriptManager:hideControllerButton();

	cleanUp( true, true, true, true, true )
	ScriptManager:startCutscene();
	ScriptManager:moveCamera( 5, 6, 0.8, 0, "" );
	ScriptManager:displaySubtitle( Text.T_MISSION_TUTORIAL_3_18 );
	ScriptManager:stopHighlightingEndTurnButton();
	ScriptManager:disableEndTurnButton();
	ScriptManager:sleep( 3, "moveBomber" );
	ScriptManager:sleep( 5, "HideDialogue" );
end

function moveBomber()
	cleanUp( true, true, true, false, true );
	local bomber = ScriptManager:spawnUnit( 1, Units.UNIT_SU_BOMBER, 6, 9, 0, 0 );
	local bomberId = bomber:getId();
	ScriptManager:storeValue( "hostileBomberID", bomberId );
	ScriptManager:moveUnit( bomberId, 6, 6, "bomberAtPosition" );
end

function bomberAtPosition()
	cleanUp( true, true, true, true, true );
	ScriptManager:sleep( 2, "ShowAAProtectFunction" );
	ScriptManager:sleep( 3, "bomberStartAttack" );
end

function ShowAAProtectFunction()
	ScriptManager:displaySubtitle( Text.T_MISSION_TUTORIAL_3_19 );
end

function bomberStartAttack()
	ScriptManager:attackUnit( ScriptManager:getValue( "hostileBomberID" ), 3, "bomberAttackedTiger" );
end

function bomberAttackedTiger()
	cleanUp( true, true, true, false, true );
	ScriptManager:sleep( 1, "startAntiAir1Attack" );
end

function startAntiAir1Attack()
	ScriptManager:attackUnit( ScriptManager:getValue( "aa1" ), ScriptManager:getValue( "hostileBomberID" ),"firstAntiAirAttacked" );
end

function firstAntiAirAttacked()
	cleanUp( true, true, true, false, true );
	ScriptManager:sleep( 1, "startAntiAir2Attack" );
end

function startAntiAir2Attack()
	ScriptManager:attackUnit( ScriptManager:getValue( "aa2" ), ScriptManager:getValue( "hostileBomberID" ),"secondAntiAirAttacked" );
end

function secondAntiAirAttacked()
	cleanUp( true, true, true, true, true );
	ScriptManager:sleep( 2.5, "spawnArtillery" );
end

function spawnArtillery()
	cleanUp( true, true, true, true, true );
	ScriptManager:moveCamera( 6, 9, 1, 1, "" );
	local unit = ScriptManager:spawnUnit( 1, Units.UNIT_SU_HOWITZER, 6, 9, 0, 0 );
	local unitId = unit:getId();
	ScriptManager:storeValue( "hostileArtilleryID", unitId );
	ScriptManager:sleep( 1.5, "startArtilleryAttack" );
end

function startArtilleryAttack()
	ScriptManager:moveCamera( 5, 5, 0.8, 1.5, "" );
	ScriptManager:attackUnit( ScriptManager:getValue( "hostileArtilleryID" ), 3,"artilleryAttacked" );
end

function artilleryAttacked()
	cleanUp( true, true, true, true, true );
	ScriptManager:displaySubtitle( Text.T_MISSION_TUTORIAL_3_20 );
	ScriptManager:sleep( 1.5, "WaitSpawnBomber" );
end

function WaitSpawnBomber()
	ScriptManager:moveCamera( 6, 1, 0.9, 2, "spawnBomber" );
end

function spawnBomber()
	local unit = ScriptManager:spawnUnit( 0, Units.UNIT_DE_BOMBER, 6, 1, 0, 0 );
	local unitId = unit:getId();
	ScriptManager:storeValue( "bomberID", unitId );
	ScriptManager:sleep( 3.5, "endCutscene" );
	unit:setHealth( 200 );
	unit:setAmmo( 0 );
	unit:setPlaneTurns( 0 );
end

function endCutscene()
	cleanUp( true, true, true, true, true );
	ScriptManager:moveCamera( 6, 1, 0.8, 2, "" );
	ScriptManager:stopCutscene();
	ScriptManager:continueAfterCutscene( "startSecondPlayerTurn" );
end

function startSecondPlayerTurn()
	ScriptManager:sleep( 1, "selectBomber" );
end

function selectBomber()
	cleanUp( true, true, true, true, true );
	ScriptManager:displaySubtitle( Text.T_MISSION_TUTORIAL_3_21 );
	ScriptManager:showMarkerAt( 6, 1 );
	ScriptManager:disableAllTilesExcept( 6, 1 );
	
	cleanupController();
	if ScriptManager:isControllerConnected() then
		ScriptManager:setControllerCursorEnabled( true );
		ScriptManager:addOnControllerSelectedTileTrigger( 6, 1, "cursorOverUnitSinglePress", "cursorOverUnit", false );
	
	else
		ScriptManager:showFingerAtTile( 6, 1, 0 );
		ScriptManager:loopFingerClicks( 1 );
	end
	
	ScriptManager:addOnUnitSelectedTrigger( -1, -1, -1, "bomberSelected", "bomberSelected" );
end

function bomberSelected()
	cleanUp( true, true, true, false, true );
	ScriptManager:enableInteractionButton( InteractionButton.INSTANT_SUPPLY_SINGLE );
	ScriptManager:startHighlightingInteractionButton( InteractionButton.INSTANT_SUPPLY_SINGLE );
	ScriptManager:enableInteractionButton( InteractionButton.INSTANT_SUPPLY_FULL );
	ScriptManager:startHighlightingInteractionButton( InteractionButton.INSTANT_SUPPLY_FULL );
	
	cleanupController();
	if ScriptManager:isControllerConnected() then
		showControllerButtonGfx( ControllerButton.FACE_LEFT, 1, 6, 1 );
	end
	
	ScriptManager:addOnUnitInstantSuppliedTrigger( -1, -1, -1, "bomberSupplied", "bomberSupplied" );
end

function bomberSupplied()
	local bomber = Unit:getUnitById( ScriptManager:getValue( "bomberID" ) );
	if bomber:getHealth() >= 450 then
		moveBomber2();
	end
end

function moveBomber2()
	cleanUp( true, true, true, true, true )
	ScriptManager:displaySubtitle( Text.T_MISSION_TUTORIAL_3_26 );
	ScriptManager:stopHighlightingInteractionButton( InteractionButton.INSTANT_SUPPLY_SINGLE );
	ScriptManager:stopHighlightingInteractionButton( InteractionButton.INSTANT_SUPPLY_FULL );
	ScriptManager:disableAllInteractionButtons();
	ScriptManager:moveCamera( 5, 5, 0.7, 2, "" );
	
	ScriptManager:disableAllTilesExcept( 6, 8 );
	ScriptManager:showMarkerAt( 6, 8 );
	
	cleanupController();
	if ScriptManager:isControllerConnected() then
		ScriptManager:setControllerCursorEnabled( true );
		ScriptManager:addOnControllerSelectedTileTrigger( 6, 8, "cursorOverUnit", "cursorOverUnit", false );
	
	else
		ScriptManager:showFingerAtTile( 6, 1, 0 );
		ScriptManager:moveFingerToTile( 6, 8 );
	end
	
	ScriptManager:addOnEnterTileTrigger( 6, 8, -1, -1, -1, true, "bomberNextToAtillery", "bomberNextToAtillery" );
	ScriptManager:addOnTileDeselectedTrigger( 6, 8, -1, "unmarkTile", "unmarkTile" );
end

function bomberNextToAtillery()
	cleanUp( true, true, true, false, true );
	ScriptManager:displaySubtitle( Text.T_MISSION_TUTORIAL_3_26 );
	ScriptManager:disableAllTilesExcept( 6, 9 );
	ScriptManager:showMarkerAt( 6, 9 );
	
	cleanupController();
	if ScriptManager:isControllerConnected() then
		ScriptManager:setControllerCursorEnabled( true );
		ScriptManager:addOnControllerSelectedTileTrigger( 6, 9, "cursorOverUnit", "cursorOverUnit", false );
	end
	
	ScriptManager:addOnUnitDeselectedTrigger( -1, -1, -1, "unmarkTile", "unmarkTile" );
	ScriptManager:addOnUnitHasAttackedTrigger( -1, -1, -1, "artilleryDead", "artilleryDead" );
	ScriptManager:moveCamera( 6, 9, 0.8, 1, "" );
end

function artilleryDead()
	cleanUp( true, true, true, true, true );
	ScriptManager:displaySubtitle( Text.T_MISSION_TUTORIAL_3_22 );
	ScriptManager:moveCamera( 4, 7, 0.9, 2.5, "selectAirplaneToKillBomber" );
end

function selectAirplaneToKillBomber()
	cleanUp( true, true, true, true, true );
	ScriptManager:displaySubtitle( Text.T_MISSION_TUTORIAL_3_23 );
	ScriptManager:showMarkerAt( 5, 6 );
	ScriptManager:disableAllTilesExcept( 5, 6 );
	
	cleanupController();
	if ScriptManager:isControllerConnected() then
		ScriptManager:setControllerCursorEnabled( true );
		ScriptManager:addOnControllerSelectedTileTrigger( 5, 6, "cursorOverUnitSinglePress", "cursorOverUnit", false );
	
	else
		ScriptManager:showFingerAtTile( 5, 6, 0 );
		ScriptManager:loopFingerClicks( 1 );
	end
	
	ScriptManager:addOnUnitSelectedTrigger( -1, -1, -1, "airplaneToKillBomberSelected", "airplaneToKillBomberSelected" );
end

function airplaneToKillBomberSelected()
	cleanUp( true, true, true, false, true );
	ScriptManager:disableAllTilesExcept( 6, 6 );
	ScriptManager:showMarkerAt( 6, 6 );
	
	cleanupController();
	if ScriptManager:isControllerConnected() then
		ScriptManager:setControllerCursorEnabled( true );
		ScriptManager:addOnControllerSelectedTileTrigger( 6, 6, "cursorOverUnit", "cursorOverUnit", false );
	end
	
	ScriptManager:addOnUnitDeselectedTrigger( -1, -1, -1, "unmarkTile", "unmarkTile" );
	ScriptManager:addOnUnitHasAttackedTrigger( -1, -1, -1, "bomberKilled", "bomberKilled" );
end

function bomberKilled()
	checkControllerAndHideButton();

	cleanUp( true, true, true, true, true );
	ScriptManager:displaySubtitle( Text.T_MISSION_TUTORIAL_3_24 );
	ScriptManager:moveCamera( 3, 4, 0.8, 2, "markAirfields" );
end

function markAirfields()
	cleanUp( true, true, true, false, true );
	ScriptManager:enableAdditionalTile( 2, 2 );
	ScriptManager:showMarkerAt( 2, 2 );
	ScriptManager:enableAdditionalTile( 2, 3 );
	ScriptManager:showMarkerAt( 2, 3 );
	ScriptManager:enableAdditionalTile( 3, 2 );
	ScriptManager:showMarkerAt( 3, 2 );
	
	cleanupController();
	if ScriptManager:isControllerConnected() then
		ScriptManager:setControllerCursorEnabled( true );
		ScriptManager:addOnControllerSelectedTileTrigger( 2, 2, "cursorOverUnit", "cursorOverUnit", false );
		ScriptManager:addOnControllerSelectedTileTrigger( 2, 3, "cursorOverUnit", "cursorOverUnit", false );
		ScriptManager:addOnControllerSelectedTileTrigger( 3, 2, "cursorOverUnit", "cursorOverUnit", false );
	end
	
	ScriptManager:addOnEnterTileTrigger( 2, 2, -1, -1, -1, true, "planeLanded", "planeLanded1" );
	ScriptManager:addOnEnterTileTrigger( 2, 3, -1, -1, -1, true, "planeLanded", "planeLanded2" );
	ScriptManager:addOnEnterTileTrigger( 3, 2, -1, -1, -1, true, "planeLanded", "planeLanded3" );
	ScriptManager:addOnEnterTileTrigger( 4, 5, -1, -1, -1, true, "removeMarkers", "removeMarkers" );
end

function removeMarkers()
	ScriptManager:removeTrigger( "removeMarkers" );
	ScriptManager:removeAllMarkers();
end

function planeLanded()
	cleanUp( true, true, true, true, true )
	ScriptManager:displaySubtitle( Text.T_MISSION_TUTORIAL_3_27 );
	ScriptManager:disableAllTilesExcept( 1, 3 );
	ScriptManager:showMarkerAt( 1, 3 );
	
	cleanupController();
	if ScriptManager:isControllerConnected() then
		ScriptManager:setControllerCursorEnabled( true );
		ScriptManager:addOnControllerSelectedTileTrigger( 1, 3, "cursorOverUnitSinglePress", "cursorOverUnit", false );
	
	else
		ScriptManager:showFingerAtTile( 1, 3, 0 );
		ScriptManager:loopFingerClicks( 1 );
	end
	
	ScriptManager:addOnUnitSelectedTrigger( -1, -1, -1, "hangarSelected", "hangarSelected" );
end

function markAirplane()
	local airplane = Unit:getUnitById( 7 );
	ScriptManager:showMarkerAt( airplane:getX(), airplane:getY() );
	ScriptManager:disableAllTilesExcept( airplane:getX(), airplane:getY() );
	
	cleanupController();
	if ScriptManager:isControllerConnected() then
		ScriptManager:setControllerCursorEnabled( true );
		ScriptManager:addOnControllerSelectedTileTrigger( airplane:getX(), airplane:getY(), "cursorOverUnit", "cursorOverUnit", false );
	end
end

function hangarSelected()
	cleanUp( true, true, true, false, true );
	ScriptManager:disableAllInteractionButtons();
	ScriptManager:enableInteractionButton( InteractionButton.SUPPLY );
	ScriptManager:startHighlightingInteractionButton( InteractionButton.SUPPLY );
	
	cleanupController();
	if ScriptManager:isControllerConnected() then
		--cleanupController();
		showControllerButtonGfx( ControllerButton.FACE_DOWN, 1, 1, 3 );--TODO
	end
	
	ScriptManager:addOnInteractionButtonSelectedTrigger( -1, -1, -1, InteractionButton.SUPPLY, "supplyModeEnabled", "supplyModeEnabled" );
	ScriptManager:addOnUnitSuppliedTrigger( -1, -1, -1, "planeSupplied", "planeSupplied" );
end

function supplyModeEnabled()
	ScriptManager:removeTrigger( "supplyModeEnabled" );
	ScriptManager:disableAllInteractionButtons();
	ScriptManager:addOnInteractionButtonDeselectedTrigger( -1, -1, -1, InteractionButton.SUPPLY, "supplyModeDisabled", "supplyModeDisabled" );
	markAirplane();
end

function supplyModeDisabled()
	cleanUp( true, false, true, false, true );
	ScriptManager:removeTrigger( "supplyModeDisabled" );
	ScriptManager:addOnInteractionButtonSelectedTrigger( -1, -1, -1, InteractionButton.SUPPLY, "supplyModeEnabled", "supplyModeEnabled" );
end

function planeSupplied()
	cleanUp( true, false, true, true, true );
	ScriptManager:displaySubtitle( Text.T_MISSION_TUTORIAL_3_25 );
	ScriptManager:moveCamera( 3, 5, 0.75, 4, "endTutorialWait" );
	ScriptManager:setMissionStatus( 0, MissionStatus.ACCOMPLISHED );
end

function endTutorialWait()
	ScriptManager:sleep( 1, "HideDialogue" );
	ScriptManager:sleep( 2, "endTutorial" );
end

function endTutorial()
	ScriptManager:endGame();
end